using System;
using System.Drawing;
using EnPalermo.Lit;
using EnPalermo.Mem;
using System.Drawing.Imaging;

namespace EnPalermo.Img
{

    /// <summary>
    /// Opciones de bsqueda y comparacin sobre bitmaps.
    /// </summary>
    public enum FindMode
    {
        /// <summary>
        /// Busqueda Exacta (comparacin directa sin operador).
        /// </summary>
        Equal,
        /// <summary>
        /// Considera encontrados a los elementos que luego de AND coinciden con el criterio.
        /// </summary>
        And,
        /// <summary>
        /// Considera encontrados a los elementos que luego de OR coinciden con el criterio.
        /// </summary>
        Or,
        /// <summary>
        /// Considera encontrados a los elementos que luego de XOR coinciden con el criterio.
        /// </summary>
        Xor,
        /// <summary>
        /// Busqueda Exacta de elementos diferentes (comparacin directa sin operador).
        /// </summary>
        Not
    }

	/// <summary>
	/// Funciones de delimitacin, bsqueda y reemplazo sobre bitmaps.
	/// </summary>
    public unsafe static class ImgComp
    {

        #region Reemplazo de Colores

        /// <summary>
        /// Reemplaza colores en Bitmaps. El reemplazo es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="find">Color a buscar (Argb).</param>
        /// <param name="fMode">Modo de bsqueda.</param>
        /// <param name="op">Operador (ignorado en las bsquedas Equal y Not).</param>
        /// <param name="rep">Reemplazo (Argb).</param>
        public static void ReplaceArgb32(int find, FindMode fMode, int op, int rep, Bitmap bm)
        {
            ReplaceArgb32(find, fMode,op,rep,bm,new Rectangle(0,0,bm.Width,bm.Height));
        }

        /// <summary>
        /// Reemplaza colores en Bitmaps. El reemplazo es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="x">Inicio X.</param>
        /// <param name="y">Inicio Y.</param>
        /// <param name="width">Ancho.</param>
        /// <param name="height">Alto.</param>
        /// <param name="find">Color a buscar (Argb).</param>
        /// <param name="fMode">Modo de bsqueda.</param>
        /// <param name="op">Operador (ignorado en las bsquedas Equal y Not).</param>
        /// <param name="rep">Reemplazo (Argb).</param>
        public static void ReplaceArgb32(int find, FindMode fMode, int op, int rep, Bitmap bm, int x, int y, int width, int height)
        {
            ReplaceArgb32(find, fMode,op, rep,bm,new Rectangle(x,y,width,height));
        }

        /// <summary>
        /// Reemplaza colores en Bitmaps. El reemplazo es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="rect">Rectngulo.</param>
        /// <param name="find">Color a buscar (Argb).</param>
        /// <param name="fMode">Modo de bsqueda.</param>
        /// <param name="op">Operador (ignorado en las bsquedas Equal y Not).</param>
        /// <param name="rep">Reemplazo (Argb).</param>
        public static void ReplaceArgb32(int find, FindMode fMode, int op, int rep, Bitmap bm, Rectangle rect)
        {
            if (rect.X<0)
                throw new ArgumentOutOfRangeException("rect.X",rect.X,Errors.NEGATIVE);
            if (rect.Y<0)
                throw new ArgumentOutOfRangeException("rect.Y",rect.Y,Errors.NEGATIVE);
            if (rect.Width<0)
                throw new ArgumentOutOfRangeException("rect.Width",rect.Width,Errors.NEGATIVE);
            if (rect.Height<0)
                throw new ArgumentOutOfRangeException("rect.Height",rect.Height,Errors.NEGATIVE);
            BitmapData bd=bm.LockBits(rect,ImageLockMode.ReadWrite,PixelFormat.Format32bppArgb);
            ReplaceArgb32(find,fMode,op, rep,bd);
            bm.UnlockBits(bd);
        }

        /// <summary>
        /// Reemplaza colores en Bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bd">BitmapData con formato Argb 32.</param>
        /// <param name="find">Color a buscar (Argb).</param>
        /// <param name="fMode">Modo de bsqueda.</param>
        /// <param name="op">Operador (ignorado en las bsquedas Equal y Not).</param>
        /// <param name="rep">Reemplazo (Argb).</param>
        public static void ReplaceArgb32(int find, FindMode fMode, int op, int rep, BitmapData bd)
        {

            if(bd.PixelFormat!=PixelFormat.Format32bppArgb)
                throw new InvalidOperationException(ImgLiterals.PF_NO_ARGB32);
            ReplaceArgb32(find, fMode,op, rep,(int*)bd.Scan0,bd.Stride,bd.Width,bd.Height);
        }

        /// <summary>
        /// Reemplaza colores en Bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="pbm">Puntero con formato Argb 32.</param>
        /// <param name="find">Color a buscar (Argb).</param>
        /// <param name="fMode">Modo de bsqueda.</param>
        /// <param name="op">Operador (ignorado en las bsquedas Equal y Not).</param>
        /// <param name="bmStrBytes">Stride (Bytes).</param>
        /// <param name="width">Ancho (unidades de 32 bits)</param>
        /// <param name="height">Alto (cantidad de lneas).</param>
        /// <param name="rep">Reemplazo (Argb).</param>
        public static void ReplaceArgb32(int find, FindMode fMode, int op, int rep, int* pbm, int bmStrBytes, int width, int height)
        {
            if (pbm==null)
                throw new ArgumentNullException("pbm",Errors.NULL_POINTER);
            if (width<1)
            {
                if (width!=0)
                    throw new ArgumentOutOfRangeException("width",width,Errors.NEGATIVE);
                if (height!=0)
                    throw new ArgumentOutOfRangeException("height",height,ImgLiterals.HEIGHT_NO_CERO);
                return;
            }
            if (height<1)
                throw new ArgumentOutOfRangeException("width",width,ImgLiterals.HEIGHT_MINOR_ONE);
           
            bmStrBytes>>=2;
            if (bmStrBytes<width)
                throw new ArgumentOutOfRangeException("bmStrBytes",bmStrBytes<<2,ImgLiterals.STRIDE_MINOR_WIDTH);
            int a=0;
            int bmSum=0;
            int *p;
            int b;

            switch(fMode)
            {
                case FindMode.And:
                    do
                    {
                        b=0;
                        p=pbm+bmSum;
                        do
                        {
                            if ((p[b]&op)==find)
                                p[b]=rep;
                        }while(++b!=width);
                        bmSum+=bmStrBytes;
                    }while(++a!=height);
                    return;
                case FindMode.Or:
                    do
                    {
                        b=0;
                        p=pbm+bmSum;
                        do
                        {
                            if ((p[b]|op)==find)
                                p[b]=rep;
                        }while(++b!=width);
                        bmSum+=bmStrBytes;
                    }while(++a!=height);
                    return;
                case FindMode.Xor:
                    do
                    {
                        b=0;
                        p=pbm+bmSum;
                        do
                        {
                            if ((p[b]^op)==find)
                                p[b]=rep;
                        }while(++b!=width);
                        bmSum+=bmStrBytes;
                    }while(++a!=height);
                    return;
                case FindMode.Not:
                    do
                    {
                        b=0;
                        p=pbm+bmSum;
                        do
                        {
                            if (p[b]!=find)
                                p[b]=rep;
                        }while(++b!=width);
                        bmSum+=bmStrBytes;
                    }while(++a!=height);
                    return;
            }

            do
            {
                b=0;
                p=pbm+bmSum;
                do
                {
                    if (p[b]==find)
                        p[b]=rep;
                }while(++b!=width);
                bmSum+=bmStrBytes;
            }while(++a!=height);
        }

        #endregion

        #region Deteccin de limites

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap. La funcin esta optimizada para bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="empty">Color vacio (solo se toman en cuenta los valores RGB).</param>
        /// <returns>
        /// Rectngulo con el rea de pxeles no vacios.
        /// Si no se detectan pxeles llenos, se devuelve un rectngulo vacio donde X e Y siempre son cero.
        /// </returns>
        public static Rectangle AreaArgb32(Color empty, Bitmap bm)
        {
            return AreaArgb32(empty.ToArgb(),bm);
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap. La funcin esta optimizada para bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="empty">Color vacio Argb (solo se toman en cuenta los valores RGB).</param>
        /// <returns>
        /// Rectngulo con el rea de pxeles no vacios.
        /// Si no se detectan pxeles llenos, se devuelve un rectngulo vacio donde X e Y siempre son cero.
        /// </returns>
        public static Rectangle AreaArgb32(int empty, Bitmap bm)
        {
            return AreaArgb32(empty,bm,new Rectangle(0,0,bm.Width,bm.Height));
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap. La funcin esta optimizada para bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="x">Inicio X.</param>
        /// <param name="y">Inicio Y.</param>
        /// <param name="width">Ancho.</param>
        /// <param name="height">Alto.</param>
        /// <param name="empty">Color vacio (solo se toman en cuenta los valores RGB).</param>
        /// <returns>
        /// Rectngulo con el rea de pxeles no vacios.
        /// Si no se detectan pxeles llenos, se devuelve un rectngulo vacio donde X e Y siempre son cero.
        /// </returns>
        public static Rectangle AreaArgb32(Color empty, Bitmap bm, int x, int y, int width, int height)
        {
            return AreaArgb32(empty.ToArgb(),bm,x,y,width,height);

        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap. La funcin esta optimizada para bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="x">Inicio X.</param>
        /// <param name="y">Inicio Y.</param>
        /// <param name="width">Ancho.</param>
        /// <param name="height">Alto.</param>
        /// <param name="empty">Color vacio Argb (solo se toman en cuenta los valores RGB).</param>
        /// <returns>
        /// Rectngulo con el rea de pxeles no vacios.
        /// Si no se detectan pxeles llenos, se devuelve un rectngulo vacio donde X e Y siempre son cero.
        /// </returns>
        public static Rectangle AreaArgb32(int empty, Bitmap bm, int x, int y, int width, int height)
        {
            return AreaArgb32(empty,bm,new Rectangle(x,y,width,height));
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap. La funcin esta optimizada para bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="rect">Rectngulo.</param>
        /// <param name="empty">Color vacio (solo se toman en cuenta los valores RGB).</param>
        /// <returns>
        /// Rectngulo con el rea de pxeles no vacios.
        /// Si no se detectan pxeles llenos, se devuelve un rectngulo vacio donde X e Y siempre son cero.
        /// </returns>
        public static Rectangle AreaArgb32(Color empty, Bitmap bm, Rectangle rect)
        {
            return AreaArgb32(empty.ToArgb(),bm,rect);
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap. La funcin esta optimizada para bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="rect">Rectngulo.</param>
        /// <param name="empty">Color vacio Argb (solo se toman en cuenta los valores RGB).</param>
        /// <returns>
        /// Rectngulo con el rea de pxeles no vacios.
        /// Si no se detectan pxeles llenos, se devuelve un rectngulo vacio donde X e Y siempre son cero.
        /// </returns>
        public static Rectangle AreaArgb32(int empty, Bitmap bm, Rectangle rect)
        {
            if (rect.X<0)
                throw new ArgumentOutOfRangeException("rect.X",rect.X,Errors.NEGATIVE);
            if (rect.Y<0)
                throw new ArgumentOutOfRangeException("rect.Y",rect.Y,Errors.NEGATIVE);
            if (rect.Width<0)
                throw new ArgumentOutOfRangeException("rect.Width",rect.Width,Errors.NEGATIVE);
            if (rect.Height<0)
                throw new ArgumentOutOfRangeException("rect.Height",rect.Height,Errors.NEGATIVE);
            BitmapData bd=bm.LockBits(rect,ImageLockMode.ReadOnly,PixelFormat.Format32bppArgb);
            Rectangle ret = AreaArgb32(empty,bd);
            bm.UnlockBits(bd);
            if (!ret.IsEmpty)
            {
                ret.X+=rect.X;
                ret.Y+=rect.Y;
            }
            return ret;
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="bd">BitmapData Argb 32.</param>
        /// <param name="empty">Color vacio (solo se toman en cuenta los valores RGB).</param>
        /// <returns>Rectngulo con el rea de pxeles no vacios.</returns>
        public static Rectangle AreaArgb32(Color empty, BitmapData bd)
        {
            return AreaArgb32(empty.ToArgb(),bd);
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="bd">BitmapData Argb 32.</param>
        /// <param name="empty">Color vacio Argb (solo se toman en cuenta los valores RGB).</param>
        /// <returns>Rectngulo con el rea de pxeles no vacios.</returns>
        public static Rectangle AreaArgb32(int empty, BitmapData bd)
        {
            if(bd.PixelFormat!=PixelFormat.Format32bppArgb)
                throw new InvalidOperationException(ImgLiterals.PF_NO_ARGB32);
            return AreaArgb32(empty,(int*)bd.Scan0,bd.Stride,bd.Width,bd.Height);
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="pbm">Puntero Argb 32.</param>
        /// <param name="bmStrBytes">Stride (Bytes).</param>
        /// <param name="width">Ancho (unidades de 32 bits)</param>
        /// <param name="height">Alto (cantidad de lneas).</param>
        /// <param name="empty">Color vacio (solo se toman en cuenta los valores RGB).</param>
        /// <returns>Rectngulo con el rea de pxeles no vacios.</returns>
        public static Rectangle AreaArgb32(Color empty, int* pbm, int bmStrBytes, int width, int height)
        {
            return AreaArgb32(empty.ToArgb(),pbm,bmStrBytes,width,height);
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0 o
        /// que su color, independientemente de la transparencia, coincida con empty.
        /// </summary>
        /// <param name="pbm">Puntero Argb 32.</param>
        /// <param name="bmStrBytes">Stride (Bytes).</param>
        /// <param name="width">Ancho (unidades de 32 bits)</param>
        /// <param name="height">Alto (cantidad de lneas).</param>
        /// <param name="empty">Color vacio Argb (solo se toman en cuenta los valores RGB).</param>
        /// <returns>Rectngulo con el rea de pxeles no vacios.</returns>
        public static Rectangle AreaArgb32(int empty, int* pbm, int bmStrBytes, int width, int height)
        {
            if (pbm==null)
                throw new ArgumentNullException("pbm",Errors.NULL_POINTER);
            if (width<1)
            {
                if (width!=0)
                    throw new ArgumentOutOfRangeException("width",width,Errors.NEGATIVE);
                if (height!=0)
                    throw new ArgumentOutOfRangeException("height",height,ImgLiterals.HEIGHT_NO_CERO);
                return Rectangle.Empty;
            }
            if (height<1)
                throw new ArgumentOutOfRangeException("width",width,ImgLiterals.HEIGHT_MINOR_ONE);
           
            bmStrBytes>>=2;
            if (bmStrBytes<width)
                throw new ArgumentOutOfRangeException("bmStrBytes",bmStrBytes<<2,ImgLiterals.STRIDE_MINOR_WIDTH);

            int a=0;
            int bmSum=0;
            int xmin=width,xmax=-1,ymin=height,ymax=-1;
            uint wu=((uint)empty&0x00FFFFFF);
            uint*p;
            int b;
            uint t;

            do
            {
                p=(uint*)(pbm+bmSum);
                b=0;
                do
                {
                    t=p[b];
                    if ((t&0xFF000000)!=0U && (t&wu)!=wu)
                    {
                        if (b<xmin)
                            xmin=b;
                        else if(b>xmax)
                            xmax=b;
                        if(a<ymin)
                            ymin=a;
                        else if(a>ymax)
                            ymax=a;
                    }
                }while(++b!=width);
                bmSum+=bmStrBytes;
            }while(++a!=height);
            
            if (xmin==width)
                return Rectangle.Empty;
            if (xmax==-1)
                xmax=xmin;
            if (ymax==-1)
                ymax=ymin;
            return new Rectangle(xmin,ymin,(xmax-xmin)+1,(ymax-ymin)+1);
        }

        #endregion

        #region Deteccin de limites basado en transparencia solamente


        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap. La funcin esta optimizada para bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <returns>
        /// Rectngulo con el rea de pxeles no vacios.
        /// Si no se detectan pxeles llenos, se devuelve un rectngulo vacio donde X e Y siempre son cero.
        /// </returns>
        public static Rectangle AreaArgb32(Bitmap bm)
        {
            return AreaArgb32(bm,new Rectangle(0,0,bm.Width,bm.Height));
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap. La funcin esta optimizada para bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="x">Inicio X.</param>
        /// <param name="y">Inicio Y.</param>
        /// <param name="width">Ancho.</param>
        /// <param name="height">Alto.</param>
        /// <returns>
        /// Rectngulo con el rea de pxeles no vacios.
        /// Si no se detectan pxeles llenos, se devuelve un rectngulo vacio donde X e Y siempre son cero.
        /// </returns>
        public static Rectangle AreaArgb32(Bitmap bm, int x, int y, int width, int height)
        {
            return AreaArgb32(bm,new Rectangle(x,y,width,height));
        }

        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap. La funcin esta optimizada para bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="rect">Rectngulo.</param>
        /// <returns>
        /// Rectngulo con el rea de pxeles no vacios.
        /// Si no se detectan pxeles llenos, se devuelve un rectngulo vacio donde X e Y siempre son cero.
        /// </returns>
        public static Rectangle AreaArgb32(Bitmap bm, Rectangle rect)
        {
            if (rect.X<0)
                throw new ArgumentOutOfRangeException("rect.X",rect.X,Errors.NEGATIVE);
            if (rect.Y<0)
                throw new ArgumentOutOfRangeException("rect.Y",rect.Y,Errors.NEGATIVE);
            if (rect.Width<0)
                throw new ArgumentOutOfRangeException("rect.Width",rect.Width,Errors.NEGATIVE);
            if (rect.Height<0)
                throw new ArgumentOutOfRangeException("rect.Height",rect.Height,Errors.NEGATIVE);
            BitmapData bd=bm.LockBits(rect,ImageLockMode.ReadOnly,PixelFormat.Format32bppArgb);
            Rectangle ret = AreaArgb32(bd);
            bm.UnlockBits(bd);
            if (!ret.IsEmpty)
            {
                ret.X+=rect.X;
                ret.Y+=rect.Y;
            }
            return ret;
        }


        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0.
        /// </summary>
        /// <param name="bd">BitmapData Argb 32.</param>
        /// <returns>Rectngulo con el rea de pxeles no vacios.</returns>
        public static Rectangle AreaArgb32(BitmapData bd)
        {
            if(bd.PixelFormat!=PixelFormat.Format32bppArgb)
                throw new InvalidOperationException(ImgLiterals.PF_NO_ARGB32);
            return AreaArgb32((int*)bd.Scan0,bd.Stride,bd.Width,bd.Height);
        }


        /// <summary>
        /// Crea un rectngulo detectando lmites en un bitmap de 32 bits Argb.
        /// La deteccin se efectua considerando vacio a todo pixel con transparencia 0.
        /// </summary>
        /// <param name="pbm">Puntero Argb 32.</param>
        /// <param name="bmStrBytes">Stride (Bytes).</param>
        /// <param name="width">Ancho (unidades de 32 bits)</param>
        /// <param name="height">Alto (cantidad de lneas).</param>
        /// <returns>Rectngulo con el rea de pxeles no vacios.</returns>
        public static Rectangle AreaArgb32(int* pbm, int bmStrBytes, int width, int height)
        {
            if (pbm==null)
                throw new ArgumentNullException("pbm",Errors.NULL_POINTER);
            if (width<1)
            {
                if (width!=0)
                    throw new ArgumentOutOfRangeException("width",width,Errors.NEGATIVE);
                if (height!=0)
                    throw new ArgumentOutOfRangeException("height",height,ImgLiterals.HEIGHT_NO_CERO);
                return Rectangle.Empty;
            }
            if (height<1)
                throw new ArgumentOutOfRangeException("width",width,ImgLiterals.HEIGHT_MINOR_ONE);
           
            bmStrBytes>>=2;
            if (bmStrBytes<width)
                throw new ArgumentOutOfRangeException("bmStrBytes",bmStrBytes<<2,ImgLiterals.STRIDE_MINOR_WIDTH);

            int a=0;
            int bmSum=0;
            int xmin=width,xmax=-1,ymin=height,ymax=-1;
            uint*p;
            int b;
            do
            {
                p=(uint*)(pbm+bmSum);
                b=0;
                do
                {
                    if ((p[b]&0xFF000000)!=0U)
                    {
                        if (b<xmin)
                            xmin=b;
                        else if(b>xmax)
                            xmax=b;
                        if(a<ymin)
                            ymin=a;
                        else if(a>ymax)
                            ymax=a;
                    }
                }while(++b!=width);
                bmSum+=bmStrBytes;
            }while(++a!=height);
            
            if (xmin==width)
                return Rectangle.Empty;
            if (xmax==-1)
                xmax=xmin;
            if (ymax==-1)
                ymax=ymin;
            return new Rectangle(xmin,ymin,(xmax-xmin)+1,(ymax-ymin)+1);
        }

        #endregion




    }	
}
