using System;
using System.Drawing;
using EnPalermo.Lit;
using EnPalermo.Mem;
using System.Drawing.Imaging;

namespace EnPalermo.Img
{
	/// <summary>
	/// Funciones estticas bsicas para operacin con bitmaps.
	/// </summary>
    public unsafe static class ImgGeneral
    {


        #region Copia de bitmaps

        /// <summary>
        /// Copia BitMaps. La copia es optimizada para bitmaps de 32 bits.
        /// </summary>
        /// <param name="bmOrig">Bitmap origen.</param>
        /// <param name="bmDest">Bitmap destino.</param>
        public static void Bitmap32Copy(Bitmap bmOrig, Bitmap bmDest)
        {
            Bitmap32Copy(bmOrig,new Rectangle(0,0,bmOrig.Width,bmOrig.Height),bmDest,0,0);
        }

        /// <summary>
        /// Copia BitMaps. La copia es optimizada para bitmaps de 32 bits.
        /// </summary>
        /// <param name="bmOrig">Bitmap origen.</param>
        /// <param name="bmDest">Bitmap destino.</param>
        /// <param name="destX">X destino.</param>
        /// <param name="destY">Y destino.</param>
        public static void Bitmap32Copy(Bitmap bmOrig, Bitmap bmDest, int destX, int destY)
        {
            Bitmap32Copy(bmOrig,new Rectangle(0,0,bmOrig.Width,bmOrig.Height),bmDest,destX,destY);
        }

        /// <summary>
        /// Copia BitMaps. La copia es optimizada para bitmaps de 32 bits.
        /// </summary>
        /// <param name="bmOrig">Bitmap origen.</param>
        /// <param name="origX">X origen.</param>
        /// <param name="origY">Y origen.</param>
        /// <param name="height">Ancho.</param>
        /// <param name="width">Alto.</param>
        /// <param name="bmDest">Bitmap destino.</param>
        /// <param name="destX">X destino.</param>
        /// <param name="destY">Y destino.</param>
        public static void Bitmap32Copy(Bitmap bmOrig, int origX, int origY, int width, int height, Bitmap bmDest, int destX, int destY)
        {
            Bitmap32Copy(bmOrig,new Rectangle(origX,origY,width,height),bmDest,destX,destY);
        }

        /// <summary>
        /// Copia BitMaps. La copia es optimizada para bitmaps de 32 bits.
        /// </summary>
        /// <param name="bmOrig">Bitmap origen.</param>
        /// <param name="rOrig">Rectngulo origen.</param>
        /// <param name="bmDest">Bitmap destino.</param>
        /// <param name="destX">X destino.</param>
        /// <param name="destY">Y destino.</param>
        public static void Bitmap32Copy(Bitmap bmOrig, Rectangle rOrig, Bitmap bmDest, int destX, int destY)
        {
            if (destX<0)
                throw new ArgumentOutOfRangeException("destX",destX,Errors.NEGATIVE);
            if (destY<0)
                throw new ArgumentOutOfRangeException("destY",destY,Errors.NEGATIVE);
            if (rOrig.X<0)
                throw new ArgumentOutOfRangeException("rOrig.X",rOrig.X,Errors.NEGATIVE);
            if (rOrig.Y<0)
                throw new ArgumentOutOfRangeException("rOrig.Y",rOrig.Y,Errors.NEGATIVE);
            if (rOrig.Width<0)
                throw new ArgumentOutOfRangeException("rOrig.Width",rOrig.Width,Errors.NEGATIVE);
            if (rOrig.Height<0)
                throw new ArgumentOutOfRangeException("rOrig.Height",rOrig.Height,Errors.NEGATIVE);
            PixelFormat pix=bmOrig.PixelFormat;
            switch(pix)
            {
                case PixelFormat.Format32bppArgb:
                case PixelFormat.Format32bppPArgb:
                case PixelFormat.Format32bppRgb:
                    break;
                case PixelFormat.Format48bppRgb:
                case PixelFormat.Format64bppArgb:
                case PixelFormat.Format64bppPArgb:
                    throw new InvalidOperationException(ImgLiterals.PF_NO_32BITS);
                default:
                    pix=PixelFormat.Format32bppArgb;
                    break;
            }
            BitmapData bdOrig=bmOrig.LockBits(rOrig,ImageLockMode.ReadOnly,pix);
            rOrig.X=destX;
            rOrig.Y=destY;
            BitmapData bdDest=bmDest.LockBits(rOrig,ImageLockMode.WriteOnly,pix);
            Bitmap32Copy(bdOrig,bdDest);
            bmOrig.UnlockBits(bdOrig);
            bmDest.UnlockBits(bdDest);
        }


        /// <summary>
        /// Copia BitMaps de 32 bits.
        /// </summary>
        /// <param name="bdOrig">BitmapData origen.</param>
        /// <param name="bdDest">BitmapData destino.</param>
        public static void Bitmap32Copy(BitmapData bdOrig, BitmapData bdDest)
        {
            if (bdDest.Width<bdOrig.Width || bdDest.Height<bdOrig.Height)
                throw new ArgumentOutOfRangeException("bdDest",Errors.DEST_MINOR);
            switch(bdOrig.PixelFormat)
            {
                case PixelFormat.Format32bppArgb:
                case PixelFormat.Format32bppPArgb:
                case PixelFormat.Format32bppRgb:
                    if (bdOrig.PixelFormat!=bdDest.PixelFormat)
                        throw new InvalidOperationException(ImgLiterals.PF_DEST_NO_EQUAL);
                    break;
                default:
                    throw new InvalidOperationException(ImgLiterals.PF_NO_32BITS);
            }
            Bitmap32Copy((int*)bdOrig.Scan0,(int*)bdDest.Scan0,bdOrig.Stride,bdDest.Stride,bdOrig.Width,bdOrig.Height);
        }

        /// <summary>
        /// Copia BitMaps de 32 bits.
        /// </summary>
        /// <param name="pOrig">Puntero origen.</param>
        /// <param name="pDest">Puntero destino.</param>
        /// <param name="oStrBytes">Stride origen (Bytes).</param>
        /// <param name="dStrBytes">Stride destino (Bytes).</param>
        /// <param name="width">Ancho (unidades de 32 bits)</param>
        /// <param name="height">Alto (cantidad de lneas).</param>
        public static void Bitmap32Copy(int* pOrig, int* pDest, int oStrBytes, int dStrBytes, int width, int height)
        {
            if (pOrig==null)
                throw new ArgumentNullException("pOrig",Errors.NULL_POINTER);
            if (pDest==null)
                throw new ArgumentNullException("pDest",Errors.NULL_POINTER);
            if (width<0)
                throw new ArgumentOutOfRangeException("width",width,Errors.NEGATIVE);
            int w=width<<2;
            if (w==0)
            {
                if (height!=0)
                    throw new ArgumentOutOfRangeException("height",height,ImgLiterals.HEIGHT_NO_CERO);
                return;
            }
            if (height<1)
                throw new ArgumentOutOfRangeException("width",width,ImgLiterals.HEIGHT_MINOR_ONE);
            if (oStrBytes<w)
                throw new ArgumentOutOfRangeException("oStrBytes",oStrBytes,ImgLiterals.STRIDE_MINOR_WIDTH);
            if (dStrBytes<w)
                throw new ArgumentOutOfRangeException("dStrBytes",dStrBytes,ImgLiterals.STRIDE_MINOR_WIDTH);
            byte* po=(byte*)pOrig;
            byte* pd=(byte*)pDest;
            int a=0;
            int oSum=0;
            int dSum=0;
            do
            {
                General.Copy(po+oSum,pd+dSum,w);
                oSum+=oStrBytes;
                dSum+=dStrBytes;
            }while(++a!=height);
        }

        #endregion

        #region Relleno de bitmaps

        /// <summary>
        /// Rellena BitMaps. El relleno es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="fill">Relleno.</param>
        public static void FillArgb32(Color fill, Bitmap bm)
        {

            FillArgb32(fill.ToArgb(),bm);
        }

        /// <summary>
        /// Rellena BitMaps. El relleno es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="fill">Relleno (Argb).</param>
        public static void FillArgb32(int fill, Bitmap bm)
        {
            FillArgb32(fill,bm,new Rectangle(0,0,bm.Width,bm.Height));
        }

        /// <summary>
        /// Rellena BitMaps. El relleno es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="x">Inicio X.</param>
        /// <param name="y">Inicio Y.</param>
        /// <param name="width">Ancho.</param>
        /// <param name="height">Alto.</param>
        /// <param name="fill">Relleno.</param>
        public static void FillArgb32(Color fill, Bitmap bm, int x, int y, int width, int height)
        {
            FillArgb32(fill.ToArgb(),bm,x,y,width,height);
        }

        /// <summary>
        /// Rellena BitMaps. El relleno es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="x">Inicio X.</param>
        /// <param name="y">Inicio Y.</param>
        /// <param name="width">Ancho.</param>
        /// <param name="height">Alto.</param>
        /// <param name="fill">Relleno (Argb).</param>
        public static void FillArgb32(int fill, Bitmap bm, int x, int y, int width, int height)
        {
            FillArgb32(fill,bm,new Rectangle(x,y,width,height));
        }

        /// <summary>
        /// Rellena BitMaps. El relleno es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="rect">Rectngulo.</param>
        /// <param name="fill">Relleno.</param>
        public static void FillArgb32(Color fill, Bitmap bm, Rectangle rect)
        {
            FillArgb32(fill.ToArgb(),bm,rect);
        }

        /// <summary>
        /// Rellena BitMaps. El relleno es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="rect">Rectngulo.</param>
        /// <param name="fill">Relleno (Argb).</param>
        public static void FillArgb32(int fill, Bitmap bm, Rectangle rect)
        {
            if (rect.X<0)
                throw new ArgumentOutOfRangeException("rect.X",rect.X,Errors.NEGATIVE);
            if (rect.Y<0)
                throw new ArgumentOutOfRangeException("rect.Y",rect.Y,Errors.NEGATIVE);
            if (rect.Width<0)
                throw new ArgumentOutOfRangeException("rect.Width",rect.Width,Errors.NEGATIVE);
            if (rect.Height<0)
                throw new ArgumentOutOfRangeException("rect.Height",rect.Height,Errors.NEGATIVE);
            BitmapData bd=bm.LockBits(rect,ImageLockMode.WriteOnly,PixelFormat.Format32bppArgb);
            FillArgb32(fill,bd);
            bm.UnlockBits(bd);
        }

        /// <summary>
        /// Rellena BitMaps de 32 bits Argb.
        /// </summary>
        /// <param name="bd">BitmapData con formato Argb 32.</param>
        /// <param name="fill">Relleno.</param>
        public static void FillArgb32(Color fill, BitmapData bd)
        {
            FillArgb32(fill.ToArgb(),bd);
        }

        /// <summary>
        /// Rellena BitMaps de 32 bits Argb.
        /// </summary>
        /// <param name="bd">BitmapData con formato Argb 32.</param>
        /// <param name="fill">Relleno (Argb).</param>
        public static void FillArgb32(int fill, BitmapData bd)
        {

            if(bd.PixelFormat!=PixelFormat.Format32bppArgb)
                throw new InvalidOperationException(ImgLiterals.PF_NO_ARGB32);
            FillArgb32(fill,(int*)bd.Scan0,bd.Stride,bd.Width,bd.Height);
        }

        /// <summary>
        /// Rellena BitMaps de 32 bits Argb.
        /// </summary>
        /// <param name="pbm">Puntero con formato Argb 32.</param>
        /// <param name="bmStrBytes">Stride (Bytes).</param>
        /// <param name="width">Ancho (unidades de 32 bits)</param>
        /// <param name="height">Alto (cantidad de lneas).</param>
        /// <param name="fill">Relleno.</param>
        public static void FillArgb32(Color fill, int* pbm, int bmStrBytes, int width, int height)
        {
            FillArgb32(fill.ToArgb(),pbm,bmStrBytes,width,height);
        }

        /// <summary>
        /// Rellena BitMaps de 32 bits Argb.
        /// </summary>
        /// <param name="pbm">Puntero con formato Argb 32.</param>
        /// <param name="bmStrBytes">Stride (Bytes).</param>
        /// <param name="width">Ancho (unidades de 32 bits)</param>
        /// <param name="height">Alto (cantidad de lneas).</param>
        /// <param name="fill">Relleno (Argb).</param>
        public static void FillArgb32(int fill, int* pbm, int bmStrBytes, int width, int height)
        {
            if (pbm==null)
                throw new ArgumentNullException("pbm",Errors.NULL_POINTER);
            if (width<1)
            {
                if (width!=0)
                    throw new ArgumentOutOfRangeException("width",width,Errors.NEGATIVE);
                if (height!=0)
                    throw new ArgumentOutOfRangeException("height",height,ImgLiterals.HEIGHT_NO_CERO);
                return;
            }
            if (height<1)
                throw new ArgumentOutOfRangeException("width",width,ImgLiterals.HEIGHT_MINOR_ONE);
           
            bmStrBytes>>=2;
            if (bmStrBytes<width)
                throw new ArgumentOutOfRangeException("bmStrBytes",bmStrBytes<<2,ImgLiterals.STRIDE_MINOR_WIDTH);
            int a=0;
            int bmSum=0;
            do
            {
                General.Fill(pbm+bmSum,width,fill);
                bmSum+=bmStrBytes;
            }while(++a!=height);
        }

        #endregion

        #region Relleno de bitmaps (lineas)

        /// <summary>
        /// Rellena BitMaps (lneas horizontales). El relleno es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="fill">Relleno para cada lnea (Argb 32 bits).</param>
        public static void FillArgb32(int* fill, Bitmap bm)
        {
            FillArgb32(fill,bm,new Rectangle(0,0,bm.Width,bm.Height));
        }

        /// <summary>
        /// Rellena BitMaps (lneas horizontales). El relleno es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="x">Inicio X.</param>
        /// <param name="y">Inicio Y.</param>
        /// <param name="width">Ancho.</param>
        /// <param name="height">Alto.</param>
        /// <param name="fill">Relleno para cada lnea (Argb 32 bits).</param>
        public static void FillArgb32(int* fill, Bitmap bm, int x, int y, int width, int height)
        {
            FillArgb32(fill,bm,new Rectangle(x,y,width,height));
        }

        /// <summary>
        /// Rellena BitMaps (lneas horizontales). El relleno es optimizado para bitmaps de 32 bits Argb.
        /// </summary>
        /// <param name="bm">Bitmap. El formato de pxeles no debe ser mayor a 32 bits.</param>
        /// <param name="rect">Rectngulo.</param>
        /// <param name="fill">Relleno para cada lnea (Argb 32 bits).</param>
        public static void FillArgb32(int* fill, Bitmap bm, Rectangle rect)
        {
            if (rect.X<0)
                throw new ArgumentOutOfRangeException("rect.X",rect.X,Errors.NEGATIVE);
            if (rect.Y<0)
                throw new ArgumentOutOfRangeException("rect.Y",rect.Y,Errors.NEGATIVE);
            if (rect.Width<0)
                throw new ArgumentOutOfRangeException("rect.Width",rect.Width,Errors.NEGATIVE);
            if (rect.Height<0)
                throw new ArgumentOutOfRangeException("rect.Height",rect.Height,Errors.NEGATIVE);
            BitmapData bd=bm.LockBits(rect,ImageLockMode.WriteOnly,PixelFormat.Format32bppArgb);
            FillArgb32(fill,bd);
            bm.UnlockBits(bd);
        }

        /// <summary>
        /// Rellena BitMaps Argb 32 bits (lneas horizontales).
        /// </summary>
        /// <param name="bd">BitmapData (Argb 32 Bits).</param>
        /// <param name="fill">Relleno para cada lnea (Argb 32 bits).</param>
        public static void FillArgb32(int* fill, BitmapData bd)
        {
            if(bd.PixelFormat!=PixelFormat.Format32bppArgb)
                throw new InvalidOperationException(ImgLiterals.PF_NO_ARGB32);
            FillArgb32(fill,(int*)bd.Scan0,bd.Stride,bd.Width,bd.Height);
        }

        /// <summary>
        /// Rellena BitMaps Argb 32 bits (lneas horizontales).
        /// </summary>
        /// <param name="pbm">Puntero (Argb 32 Bits).</param>
        /// <param name="bmStrBytes">Stride (Bytes).</param>
        /// <param name="width">Ancho (unidades de 32 bits)</param>
        /// <param name="height">Alto (cantidad de lneas).</param>
        /// <param name="fill">Relleno para cada lnea (Argb 32 bits).</param>
        public static void FillArgb32(int* fill, int* pbm, int bmStrBytes, int width, int height)
        {
            if (pbm==null)
                throw new ArgumentNullException("pbm",Errors.NULL_POINTER);
            if (fill==null)
                throw new ArgumentNullException("fill",Errors.NULL_POINTER);
            if (width<1)
            {
                if (width!=0)
                    throw new ArgumentOutOfRangeException("width",width,Errors.NEGATIVE);
                if (height!=0)
                    throw new ArgumentOutOfRangeException("height",height,ImgLiterals.HEIGHT_NO_CERO);
                return;
            }
            if (height<1)
                throw new ArgumentOutOfRangeException("width",width,ImgLiterals.HEIGHT_MINOR_ONE);
            int w=width<<2;
            if (bmStrBytes<w)
                throw new ArgumentOutOfRangeException("bmStrBytes",bmStrBytes,ImgLiterals.STRIDE_MINOR_WIDTH);
            
            int a=0;
            int bmSum=0;

            byte* pf=(byte*)fill;
            byte*p=(byte*)pbm;
            
            do
            {
                General.Copy(pf,p+bmSum,w);
                bmSum+=bmStrBytes;
            }while(++a!=height);
        }

        #endregion

    }
}
