using System;
using System.Drawing;
using EnPalermo.Lit;
using EnPalermo.Mem;
using EnPalermo.Txt;
using System.Drawing.Imaging;


namespace EnPalermo.Img
{
    /// <summary>
    /// Funciones estticas de utilidad para operar con grficos y bitmaps.
    /// </summary>
    public static class ImgUtil
    {

        #region GetFont

        /// <summary>
        /// Genera una fuente.
        /// </summary>
        /// <param name="name">Nombre de la fuente.</param>
        /// <param name="alternateName">
        /// Nombre alternativo, para ser usado en caso de fallar la generacin con el primer nombre.
        /// Si tambin falla con este nombre (o si este es nulo o igual a name) se intentar tambin con "Microsoft Sans Serif".
        /// </param>
        /// <param name="style">Estilo de la fuente.</param>
        /// <param name="emSize">Tamao Em de la fuente.</param>
        /// <param name="unit">Unidad base para el tamao Em.</param>
        /// <returns>Fuente.</returns>
        public static Font GetFont(string name, string alternateName, float emSize, FontStyle style, GraphicsUnit unit)
        {
            int sans=0;
            if (name==null || name.Length==0)
            {
                if (alternateName==null || alternateName.Length==0 || TxtComp.EqualNCInvariant(alternateName,ImgLiterals.MICROSOFT_SANS_SERIF,false))
                {
                    name=ImgLiterals.MICROSOFT_SANS_SERIF;
                    sans=1;
                }
                else
                {
                    name=alternateName;
                    sans=-1;
                }
                alternateName=null;
            }


            try
            {
                return new Font(name,emSize,style,unit);
            }
            catch
            {
                if (emSize<=0.01f)
                    throw;
                switch(sans)
                {
                    case 0:
                        if (alternateName==null || alternateName.Length==0 || TxtComp.EqualNCInvariant(alternateName,ImgLiterals.MICROSOFT_SANS_SERIF,false))
                        {
                            name=ImgLiterals.MICROSOFT_SANS_SERIF;
                            sans=1;
                        }
                        else
                        {
                            name=alternateName;
                            sans=-1;
                        }
                        break;
                    case -1:
                        name=ImgLiterals.MICROSOFT_SANS_SERIF;
                        sans=1;
                        break;
                    default:
                        throw;
                }

                try
                {
                    return new Font(name,emSize,style,unit);
                }
                catch
                {
                    if (sans!=-1)
                        throw;
                    return new Font(ImgLiterals.MICROSOFT_SANS_SERIF,emSize,style,unit);
                }
            }
        }



        #endregion

        #region Colores

        /// <summary>
        /// Color negro.
        /// </summary>
        public readonly static Color Black=Color.Black;

        /// <summary>
        /// Color blanco.
        /// </summary>
        public readonly static Color White=Color.White;

        /// <summary>
        /// Color transparente.
        /// </summary>
        public readonly static Color Transparent=Color.Transparent;


        /// <summary>
        /// Color blanco.
        /// </summary>
        public const int cWhite=-1;
        
        /// <summary>
        /// Color negro.
        /// </summary>
        public const int cBlack=-16777216;
  
        /// <summary>
        /// Color transparente.
        /// </summary>
        public const int cTransparent=16777215;

        #endregion


        #region GetGraphics

        /// <summary>
        /// Genera grafico seteando opciones de alta calidad y unidades de pgina en pxeles.
        /// </summary>
        /// <param name="bm">Bitmap.</param>
        /// <param name="bmFill">Si true, rellenar al bitmap antes de crear el grafico.</param>
        /// <param name="fill">Color de relleno (ignorado si bmFill es false).</param>
        /// <returns></returns>
        public static Graphics GetGraphics(Bitmap bm, bool bmFill, Color fill)
        {
            if (bmFill)
                ImgGeneral.FillArgb32(fill,bm);
            Graphics g=Graphics.FromImage(bm);
            g.PageUnit=GraphicsUnit.Pixel;
            g.PixelOffsetMode = System.Drawing.Drawing2D.PixelOffsetMode.HighQuality;
            g.SmoothingMode=System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            g.TextRenderingHint=System.Drawing.Text.TextRenderingHint.AntiAliasGridFit;
            g.CompositingMode=System.Drawing.Drawing2D.CompositingMode.SourceOver;
            g.CompositingQuality=System.Drawing.Drawing2D.CompositingQuality.HighQuality;
            g.InterpolationMode=System.Drawing.Drawing2D.InterpolationMode.HighQualityBicubic;
            return g;
        }


        #endregion

        #region RenderLineText


        /// <summary>
        /// Genera un bitmap de una sola lnea de texto con precisin de pixel.
        /// </summary>
        /// <param name="text">Texto: No debe contener saltos de lnea intermedios (no verifica).</param>
        /// <param name="fName">Nombre de la fuente.</param>
        /// <param name="fAlternateName">Nombre alternativo de la fuente.
        /// Si no es null se usar este nombre en caso de fallar la generacin de fuente y si este tambin falla, se usar "Microsoft Sans Serif".
        /// Si es null, se genera excepcin al primer intento con fName.
        /// </param>
        /// <param name="style">Estilo de la fuente.</param>
        /// <param name="brush">Pincel.</param>
        /// <param name="width">Ancho del bitmap final. El alto se determina a partir del carcter ms alto.</param>
        /// <param name="tNormalize">Si true, al finalizar normalizar los pxeles transparentes reemplazndolos con el color de fondo.</param>
        /// <param name="bg">Color de fondo.</param>
        /// <returns>Bitmap de 32 bits Argb con la linea de texto ajustada al ancho.</returns>
        public static Bitmap RenderTextLine(string text, string fName, string fAlternateName, FontStyle style, Brush brush, int width, Color bg, bool tNormalize)
        {
            int l=text==null?0:text.Length;
            if (l==0)
                throw new ArgumentNullException("text",Errors.STRING_EMPTY);
            if (width<1)
                throw new ArgumentOutOfRangeException("width",width,Errors.MINOR_ONE);
            if (brush==null)
                throw new ArgumentNullException("brush",Errors.NULL_OBJECT);
            if (fName==null || fName.Length==0)
                throw new ArgumentNullException("fName",Errors.STRING_EMPTY);

            int maxPix=(((width<<1)/l)+1)*width;

            if (maxPix<0 || maxPix>1<<27)
                throw new ArgumentOutOfRangeException("width",width,Errors.BIG_VALUE);
            if (maxPix>1<<22)
                maxPix=width+(width>>3);
            else if (maxPix>1<<20)
                maxPix=width+(width>>2);
            else if (maxPix>1<<18)
                maxPix=width+(width>>1);
            else if (maxPix>1<<16)
                maxPix=width<<1;
            else 
                maxPix=width+width<<1;
            float em=((float)maxPix/(float)l);
            
            Font font;
            if (fAlternateName==null || fAlternateName.Length==0)
                font=new Font(fName,em,style,GraphicsUnit.Pixel);
            else
                font=GetFont(fName,fAlternateName,em,style,GraphicsUnit.Pixel);
            int maxH=font.Height;

            Bitmap bmFont=new Bitmap(maxPix,maxH,PixelFormat.Format32bppArgb);
            int fill=bg.ToArgb();
            Graphics gFont = GetGraphics(bmFont,fill!=0,bg);
            gFont.DrawString(text,font,brush,0f,0f,new StringFormat(0,0));
            gFont.Dispose();
            font.Dispose();
            Rectangle rect=ImgComp.AreaArgb32(fill,bmFont,0,0,maxPix,maxH);;
            if (rect.IsEmpty)
                throw new Exception(Errors.UNEXPECTED);
            maxH=Convert.ToInt32((double)rect.Height*((double)width/(double)rect.Width));
            if (maxH<1)
                maxH=1;
            Bitmap bmRet=new Bitmap(width,maxH,PixelFormat.Format32bppArgb);
            Graphics gRet=GetGraphics(bmRet,false,bg);
            Rectangle rect2=new Rectangle(0,0,width,maxH);
            gRet.DrawImage(bmFont,rect2,rect,GraphicsUnit.Pixel);
            bmFont.Dispose();
            gRet.Dispose();
            
            if (tNormalize)
            {
                BitmapData bd=bmRet.LockBits(rect2,ImageLockMode.ReadWrite,PixelFormat.Format32bppArgb);
                fill&=0x00FFFFFF;
                ImgComp.ReplaceArgb32(0,FindMode.And,-16777216,fill,bd);
                ImgComp.ReplaceArgb32(fill,FindMode.And,fill,fill,bd);
                bmRet.UnlockBits(bd);
            }
            return bmRet;
        }

        #endregion

        #region MeasureStringWidth

        /// <summary>
        /// Obitiene el acho preciso de una cadena.
        /// </summary>
        /// <param name="g">Graphics (font y g deben compartir la misma unidad de medida).</param>
        /// <param name="text">Cadena.</param>
        /// <param name="font">Fuente (font y g deben compartir la misma unidad de medida).</param>
        /// <param name="sf">StringFormat.</param>
        /// <returns>Ancho de la cadena en unidades de acuerdo a Font y Graphics.</returns>
        static public float MeasureStringWidth(Graphics g, string text, Font font, StringFormat sf)
        {
            if (g.PageUnit!=font.Unit)
                throw new ArgumentException(ImgLiterals.UNITS_NO_EQUAL,"g,font");
            int l=text.Length;
            sf=new StringFormat(sf);
            sf.SetMeasurableCharacterRanges(new CharacterRange[]{new CharacterRange(0, l)});
            float ret=g.MeasureCharacterRanges (text, font, new RectangleF(0f,0f,(float)l*font.Size,(float)font.Height), sf)[0].GetBounds(g).Right;
            sf.Dispose();
            return ret;
        }

        #endregion

    }	
}
