using System;
using EnPalermo.Mem;
using EnPalermo.Uti;
using EnPalermo.Lit;
using EnPalermo.Arr;

namespace EnPalermo.Col
{
    /// <summary>
    /// Coleccin de tipos de valor basada en array.
    /// A diferencia de las colecciones estndar, permite el acceso directo al array subyacente y es de mejor
    /// rendimiento dado que no usa boxing/unboxing ni es afectado por las limitaciones relativas al uso de
    /// genricos, como la imposibilidad de usar cdigo inseguro.
    /// Nota: Esta coleccin no es segura en tamaos equivalentes o mayores a (Int32.MaxValue/(2*sizeof(tipo)):
    /// Esto es: 1073741823 bytes, 536870911 shorts o chars, 268435455 ints o 134217727 longs.
    /// </summary>
    public unsafe sealed class PChars
    {
        #region Campos

#pragma warning disable 429
        /// <summary>
        /// Capacidad por defecto para esta coleccin.
        /// </summary>
        public const int DefaultCapacity=Longs.DefaultCapacity;

        /// <summary>
        /// Sumatoria por defecto para algorritmo de aumento de capacidad.
        /// </summary>
        private const int sumCap=DefaultCapacity>=8?4:2;
#pragma warning restore 429
        /// <summary>
        /// Largo actual del array.
        /// </summary>
        private int _size;

        /// <summary>
        /// Capacidad Actual. Si es cero el array subyacente podr ser null.
        /// </summary>
        private int _cap;


        /// <summary>
        /// Flags para almacenar informacin variada acerca del objeto.
        /// Definidas: 
        /// - Bytes  0 al 7: Tolerancia para Compact (SB). De 0 a 255.
        /// - Byte   8: AppendExact.
        /// - Byte   9: Unix definido (Bytes, PChars y SB).
        /// - Byte  10: Unix (Bytes, PChars y SB).
        /// - Bytes 11 y 12: Tabla MayMin (PChars y SB).
        /// - Bytes 12, 13, 14 y 15: Cantidad mxima de bytes por PChar segn cdigo de pgina (Bytes).
        /// - Bytes 16 al 31: Cdigo de pgina (Bytes).
        /// </summary>
        private uint _flags;
      
        /// <summary>
        /// Array subyacente.
        /// </summary>
        private PChar[] _items;

        #endregion

        #region Constructores

        /// <summary>
        /// Inicializa la coleccin con un array nulo.
        /// </summary>
        public PChars()
        {}

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada y largo 0.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        public PChars(int cap)
        {
            if (cap!=0)
            {
                this._items=new PChar[cap];
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin con un array nulo, permitiendo especificar la propiedad AppendExact.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        public PChars(bool appendExact)
        {
            if (appendExact)
                this._flags=1U<<8;
        }

        /// <summary>
        /// Inicializa la coleccin permitiendo especificar la propiedad AppendExact.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="cap">Capacidad inicial.</param>
        public PChars(bool appendExact, int cap):this(cap)
        {
            if (appendExact)
                this._flags=1U<<8;
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="count">Cantidad inicial de elementos.</param>
        public PChars(int cap, int count)
        {
            if (count<0 || count>cap)
                throw new ArgumentOutOfRangeException("cap,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (cap!=0)
            {
                this._items=new PChar[cap];
                this._size=count;
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="countCap">La cantidad inicial de elementos ser igual a la capacidad.</param>
        public PChars(int cap, bool countCap)
        {
            if (cap!=0)
            {
                this._items=new PChar[cap];
                this._cap=cap;
                if (countCap)
                    this._size=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece a la capacidad del array.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        public PChars(PChar[] arr, bool asRef):this(arr,asRef,arr==null?0:arr.Length){}

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece de acuerdo a iLength.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        /// <param name="iLength">
        /// Largo inicial (no puede ser negativo ni superar el largo del array).
        /// </param>
        public PChars(PChar[] arr, bool asRef, int iLength)
        {
            if (iLength<1)
            {
                if (iLength==0)
                {
                    if (asRef && arr!=null)
                    {
                        this._items=arr;
                        this._cap=arr.Length;
                    }
                    return;
                }
            }
            else if (arr!=null)
            {
                if (asRef)
                {
                    this._cap=arr.Length;
                    if (iLength<=this._cap)
                    {
                        this._items=arr;
                        this._size=iLength;
                        return;
                    }
                }
                else
                {
                    this._cap=Calc.MultMajor(iLength+sumCap,DefaultCapacity,false);
                    this._size=iLength;
                    this._items=ArrGeneral.Clone(arr,0,iLength,0,this._cap);
                    return;
                }
                
            }
            throw new ArgumentOutOfRangeException("iLength", iLength, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array (puede ser null).</param>
        public PChars(PChar[] arr):this(arr,0,arr==null?0:arr.Length,sumCap){}


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, count deber ser 0.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public PChars(PChar[] arr, int count):this(arr,0,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public PChars(PChar[] arr, int arrIndex, int count):this(arr,arrIndex,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="extraCap">Capacidad extra para agregar al array subyacente.</param>
        public PChars(PChar[] arr, int arrIndex, int count, int extraCap)
        {
            if (count<1)
            {
                if (count==0 && arrIndex==0)
                {
                    if (extraCap>0)
                    {
                        this._cap=Calc.AddIfOdd(extraCap);
                        this._items=new PChar[this._cap];
                    }
                    return;
                }
            }
            else if (arrIndex>-1 && arrIndex+count<=arr.Length)
            {
                this._items=new PChar[this._cap=Calc.AddIfOdd(extraCap+count)];
                fixed(PChar*po=arr,pd=this._items)
                    General.Copy(po+arrIndex,pd,count);
                this._size=count;
                return;
            }
            throw new ArgumentOutOfRangeException("arrIndex,count,extraCap", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }

        #endregion

        #region Asistentes

        /// <summary>
        /// Aplica suma sobre la mitad de DefaultCapacity y luego otra suma de redondeo a DefaultCapacity.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int round(int minItems)
        {
            return Calc.MultMajor(minItems+sumCap,DefaultCapacity,true);
        }

        /// <summary>
        /// Genera una nueva capacidad mnima aplicando algorritmo y redondeando.
        /// </summary>
        /// <param name="sum">Elementos a agregar.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int newMinCap(int sum)
        {
            if (this.AppendExact)
                return round(this._size+sum);

            if (sum==1)
            {
                if (this._size==0)
                    return sumCap;
                if (this._size<=(DefaultCapacity*(1<<10)))
                    return round(this._size<<1);
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>1));
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>2));
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>3));
                return round(this._size+(this._size>>4));
            }
            if (this._size==0)
            {
                if (sum<=DefaultCapacity)
                    return round(sum+DefaultCapacity);
                if (sum<=(DefaultCapacity*(1<<5)))
                    return round(sum<<1);
                if (sum<=(DefaultCapacity*(1<<10)))
                    return round(sum+(sum>>1));
                if (sum<=(DefaultCapacity*(1<<15)))
                    return round(sum+(sum>>2));
                if (sum<=(DefaultCapacity*(1<<18)))
                    return round(sum+(sum>>3));
                return round(sum+(sum>>4));
            }
            if (sum<=DefaultCapacity || this._size>sum)
            {
                if (this._size<=(DefaultCapacity*(1<<10)))
                {
                    if (this._size>sum)
                        return round(this._size<<1);
                    return round(this._size+(this._size>>1)+sum);
                }
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>2)+sum);
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>3)+sum);
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>4)+sum);
                return round(this._size+(this._size>>5)+sum);
            }
            if (this._size<=(DefaultCapacity*(1<<10)))
                return round(this._size+(this._size>>3)+sum);
            if (this._size<=(DefaultCapacity*(1<<15)))
                return round(this._size+(this._size>>4)+sum);
            if (this._size<=(DefaultCapacity*(1<<18)))
                return round(this._size+(this._size>>5)+sum);
            return round(this._size+sum);
        }

        /// <summary>
        /// Genera un nuevo array en base al tamao mnimo especificado,
        /// sumando la mitad de DefaultCapacity y redondeando.
        /// Tambin actualiza capacidad.
        /// A diferencia de newArraySumItems no compara en algorritmo 
        /// el crecimiento sobre el array actual.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Nuevo array.</returns>
        private PChar[] newArrayMin(int minItems)
        {
            return new PChar[this._cap=round(minItems)];
        }


        /// <summary>
        /// Genera un nuevo array en base al crecimiento requerido y aplicando algorritmo.
        /// Tambin actualiza capacidad.
        /// </summary>
        /// <param name="sumItems">Crecimiento requerido.</param>
        private PChar[] newArraySum(int sumItems)
        {
            return new PChar[this._cap=newMinCap(sumItems)];
        }

        /// <summary>
        /// Actualiza la capacidad para aumento.
        /// </summary>
        /// <param name="items">Nueva cantidad.</param>
        private void setCapacity(int items)
        {
            PChar[] array2=new PChar[items];
            if (this._size !=0)
            {
                fixed (PChar* po=this._items, pd=array2)
                    General.Copy(po, pd, this._size);
            }
            this._items=array2;
            this._cap=items;
        }

        #endregion

        #region Propiedades

        /// <summary>
        /// Si true, no usar algoritmo de aumento para la coleccin, aunque igual efectuar redondeo.
        /// Por lo tanto, si este parmetro esta activo se redimensionar
        /// la coleccion cada vez que se agreguen nuevos elementos.
        /// </summary>
        public bool AppendExact
        {
            get
            {
                return (this._flags&(1U<<8))!=0;
            }
            set
            {
                if (value)
                    this._flags|=1U<<8;
                else
                    this._flags&=0xFFFFFFFF^(1U<<8);
            }
        }

        /// <summary>
        /// Obtiene el array que actualmente esta siendo usado por la coleccin (puede ser null) y,
        /// por lo tanto, suceptible de ser modificado o reemplazado por la coleccin.
        /// Al establecer, cambia la instancia del array por el especificado en parametro y 
        /// setea la cantidad de elementos en el largo del nuevo array (si es nulo ser 0).
        /// </summary>
        public PChar[] Array
        {
            get
            {
                return this._items;
            }
            set
            {
                this._cap=value==null?0:value.Length;
                this._size=this._cap;
                this._items=value;
            }
        }

        /// <summary>
        /// Obtiene un array con los elementos de la coleccin.
        /// Esta copia puede ser el mismo objeto subyacente si la cantidad de elementos coincide con el largo de array
        /// o un array vacio si la capacidad es cero.
        /// </summary>
        public PChar[] ArrayCompact
        {
            get
            {
                if (this._cap==0)
                    return new PChar[0];
                if (this._size==this._cap)
                    return this._items;
                return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
            }
        }

        /// <summary>
        /// Obtiene una copia compacta al largo actual de la coleccin.
        /// Al establecer, copia los elementos de la instancia actual al array subyacente o, en caso que el
        /// array de entrada sea de capacidad mayor, genera una nueva instancia con la copia de los elementos.
        /// Setea la cantidad de elementos en el largo del nuevo array.
        /// </summary>
        public PChar[] ArrayCopy
        {
            get
            {
                if (this._size!=0)
                    return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
                return new PChar[0];
            }
            set
            {
                this._size=value==null?0:value.Length;
                if (this._size !=0)
                {
                    if (this._cap < this._size)
                        this._items=newArrayMin(this._size+sumCap);
                    fixed (PChar* po=value, pd=this._items)
                        General.Copy(po, pd, this._size);
                }
            }
        }



        /// <summary>
        /// Obtiene o establece, solo para aumentar, la capacidad del Array.
        /// En caso de establecer una capacidad menor, se ignora. Si se establece capacidad mayor, el valor ser
        /// redondeado.
        /// </summary>
        public int Capacity
        {
            get
            {
                return this._cap;
            }
            set
            {
                if (value > this._cap)
                    setCapacity(round(value));
            }
        }


        /// <summary>
        /// Obtiene o establece la cantidad de elementos en la coleccin.
        /// Se permite el establecimiento de un largo mayor al actual siempre y cuanto este no exceda la capacidad
        /// del array subyacente (el contenido de los elementos excedentes no ser modificado).
        /// Por lo tanto, si se especifica una cantidad negativa o mayor a la capacidad actual,
        /// se genera una excepcin.
        /// En todo caso, para modificar el largo con mayor flexibilidad, usar el mtodo SetLength.
        /// </summary>
        public int Length
        {
            get
            {
                return this._size;
            }
            set
            {
                if (value > this._cap || value < 0)
                    throw new ArgumentOutOfRangeException("value", value, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                this._size=value;
            }
        }


        /// <summary>
        /// Obtiene o establece el elemento especificado en index.
        /// Para acceder a los elementos frecuentemente, es conveniente acceder al array directamente.
        /// Si el index se encuentra fuera de rango, genera excepcin.
        /// </summary>
        public PChar this[int index]
        {
            get
            {
                if (this._size <=index)
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
                return this._items[index];
            }
            set
            {
                if (index < this._size)
                    this._items[index]=value;
                else 
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
            }
        }


        #endregion

        #region Metodos Administrativos

        /// <summary>
        /// Agrega un nuevo elemento al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        public void Append(PChar item)
        {
            this.EnsureAppend();
            this._items[this._size++]=item;
        }

        /// <summary>
        /// Agrega elementos al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        /// <param name="count">Cantidad de repeticiones del elemento.</param>
        public void Append(PChar item, int count)
        {
            if (item==PChar.Zero)
            {
                this.AppendBlank(count);
                return;
            }

            if (count < 2)
            {
                if (count==1)
                    this.Append(item);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }
            this.EnsureAppend(count);
            fixed (PChar* pd=this._items)
                General.Fill(pd + this._size, count, item);
            this._size +=count;
        }

        /// <summary>
        /// Agrega un cero al final de la coleccin.
        /// </summary>
        public void AppendBlank()
        {
            this.EnsureAppend();
            this._items[this._size++]=PChar.Zero;
        }

        /// <summary>
        /// Agrega ceros al final de la coleccin.
        /// </summary>
        /// <param name="count">Cantidad de ceros.</param>
        public void AppendBlank(int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.AppendBlank();
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }
            this.EnsureAppend(count);
            fixed (PChar* pd=this._items)
                General.Clean(pd + this._size, count);
            this._size +=count;
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        public void Append(PChar[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (PChar* po=arr)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void Append(PChar[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (PChar* po=arr)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Append(PChar[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (PChar* po=arr)
                    this.Append(po + arrIndex, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void Append(PChar* arr, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.Append(*arr);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }

            this.EnsureAppend(count);
            fixed (PChar* pd=this._items)
                General.Copy(arr, pd + this._size, count);
            this._size +=count;
        }


        /// <summary>
        /// Elimina todos los elementos de la coleccin.
        /// </summary>
        public void Clear()
        {
            this._size=0;
        }

        /// <summary>
        /// Elimina todos los elementos de la coleccin y establece el array en null.
        /// </summary>
        public void ClearCompact()
        {
            this._size=0;
            this._cap=0;
            this._items=null;
        }

        /// <summary>
        /// Compacta la coleccin a la cantidad de elementos actual.
        /// Si la cantidad actual de elementos es cero, el array subyacente ser null.
        /// </summary>
        public void Compact()
        {
            if (this._size !=this._cap)
            {
                if (this._size!=0)
                {
                    PChar[] array2=new PChar[this._size];
                    fixed (PChar* po=this._items, pd=array2)
                        General.Copy(po, pd, this._size);
                    this._items=array2;
                }
                else
                    this._items=null;
                this._cap=this._size;
            }
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        public void CopyTo(PChar* arr)
        {
            if (this._size !=0)
                fixed (PChar* po=this._items)
                    General.Move(po, arr, this._size);
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        public void CopyTo(PChar* arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (this._size !=startIndex)
                fixed (PChar* po=this._items)
                    General.Move(po + startIndex, arr, this._size - startIndex);
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        /// <param name="count">Elementos a copiar.</param>
        public void CopyTo(PChar* arr, int startIndex, int count)
        {

            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (this._size !=startIndex)
                fixed (PChar* po=this._items)
                    General.Move(po + startIndex, arr, count);
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        public void CopyTo(PChar[] arr)
        {
            if (this._size !=0)
            {
                if (arr.Length < this._size)
                    throw new ArgumentOutOfRangeException("arr", arr.Length, Errors.ARRAY_SMALL_DEST);
                fixed (PChar* po=this._items, pd=arr)
                    General.Copy(po, pd, this._size);
            }
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        public void CopyTo(PChar[] arr, int startIndex)
        {
            int count=this._size - startIndex;
            if (startIndex > this._size || startIndex < 0 || count > arr.Length)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                fixed (PChar* po=this._items, pd=arr)
                    General.Copy(po + startIndex, pd, count);
        }


        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        /// <param name="count">Elementos a copiar.</param>
        public void CopyTo(PChar[] arr, int startIndex, int count)
        {
            if (startIndex > this._size || startIndex < 0 || count < 0 || count > arr.Length)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                fixed (PChar* po=this._items, pd=arr)
                    General.Copy(po + startIndex, pd, count);
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de 1 elemento.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        public void EnsureAppend()
        {
            if (this._size == this._cap)
                setCapacity(newMinCap(1));
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de n elementos.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        /// <param name="items">Elementos a agregar.</param>
        public void EnsureAppend(int items)
        {
            if ((items + this._size) > this._cap)
                setCapacity(newMinCap(items));
        }

        /// <summary>
        /// Recupera una copia de los elementos.
        /// </summary>
        /// <returns>Array con los elementos copiados.</returns>
        public PChar[] GetCopy()
        {
            return this.ArrayCopy;
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public PChar[] GetCopy(int count)
        {
            if (this._size < count || count < 0)
                throw new System.ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            PChar[] ret=new PChar[count];
            if (count !=0)
            {
                fixed (PChar* po=this._items, pd=ret)
                    General.Copy(po, pd, count);
            }
            return ret;
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="startIndex">Index donde se inicia la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public PChar[] GetCopy(int startIndex, int count)
        {
            if (this._size - startIndex < count || count < 0 || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            PChar[] ret=new PChar[count];
            if (count !=0)
            {
                fixed (PChar* po=this._items, pd=ret)
                    General.Copy(po + startIndex, pd, count);
            }
            return ret;
        }

        /// <summary>
        /// Quita todos los elementos del rango especificado.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        /// <param name="count">Cantidad de elementos a eliminar.</param>
        public void RemoveRange(int startIndex, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.RemoveAt(startIndex);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                return;
            }
            int max=startIndex + count;

            if (startIndex < 0 || max > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (max==this._size)
            {
                this._size=startIndex;
                return;
            }
            fixed (PChar* p=this._items)
                General.Cut(p, this._size, startIndex, count, false);
            this._size -=count;
        }

        /// <summary>
        /// Inserta el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="index">Index en la coleccin donde se insertar el elemento.</param>
        public void Insert(int index, PChar item)
        {
            this.Reserve(index);
            this._items[index]=item;
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void Insert(int startIndex, PChar item, int count)
        {
            this.Reserve(startIndex,count);
            this.Set(startIndex,item,count);
        }

        /// <summary>
        /// Inserta un cero en la ubicacin especificada.
        /// </summary>
        /// <param name="index">Index en la coleccin donde insertar.</param>
        public void InsertBlank(int index)
        {
            this.Reserve(index);
            this._items[index]=PChar.Zero;
        }


        /// <summary>
        /// Inserta ceros en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void InsertBlank(int startIndex, int count)
        {
            this.Reserve(startIndex,count);
            this.SetBlank(startIndex,count);
        }


        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Insert(int startIndex, PChar[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (PChar* po=arr)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Insert(int startIndex, PChar[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (PChar* po=arr)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, PChar[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (PChar* po=arr)
                    this.Insert(startIndex,po + arrIndex, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, PChar* arr, int count)
        {
            this.Reserve(startIndex,count);
            this.Set(startIndex,arr,count);
        }

        /// <summary>
        /// Quita del array el elemento especificado.
        /// </summary>
        /// <param name="index">Index del elemento.</param>
        public void RemoveAt(int index)
        {
            if (index >=this._size || index < 0)
                throw new System.ArgumentOutOfRangeException("index", index, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (--this._size !=index)
            {
                fixed (PChar* p=this._items)
                    General.Copy(p + (index + 1), p + index, this._size - index);
            }

        }

        /// <summary>
        /// Quita todos los elementos desde la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        public void RemoveRange(int startIndex)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
        }

        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, PChar[] r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex);
            else
                fixed(PChar*pr=r)
                    ReplaceRange(startIndex,pr,lr);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, PChar[] r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex,count);
            else
                fixed(PChar*pr=r)
                    ReplaceRange(startIndex,count,pr,lr);
        }

        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, PChar* pr, int lr)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
            this.Append(pr,lr);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, PChar* pr, int lr)
        {
            if (count>=lr)
            {
                Set(startIndex,pr,lr);
                if (count>lr)
                    RemoveRange(startIndex+lr,count-lr);
                return;
            }
            if (count<1)
            {
                if (count==0)
                {
                    Insert(startIndex,pr,lr);
                    return;
                }
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            Reserve(startIndex+count,lr-count);
            Set(startIndex,pr,lr);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        public void Reverse()
        {
            if (this._size > 1)
                fixed (PChar* p=this._items)
                    Sort.Reverse(p, this._size);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void Reverse(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (PChar* p=this._items)
                    Sort.Reverse(p + startIndex, count);
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de un elemento. A diferencia de la funcin Insert,
        /// no efecta ninguna copia o limpieza.
        /// </summary>
        /// <param name="index">Index de la insercin.</param>
        public void Reserve(int index)
        {
            if (index<0 || index>this._size)
                throw new ArgumentOutOfRangeException("index",index,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (index==this._size)
                this.EnsureAppend();
            else
            {
                if (this._size==this._cap)
                {
                    PChar[] array2=newArraySum(1);
                    fixed (PChar* po=this._items, pd=array2)
                    {
                        if (index!=0)
                            General.Copy(po, pd, index);
                        General.Copy(po + index, pd + (index + 1), this._size - index);
                    }
                    this._items=array2;
                }
                else
                {
                    fixed (PChar* po=this._items)
                        General.Move(po + index, po + (index + 1), this._size - index);
                }
            }
            this._size++;
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de la cantidad de elementos especificada mediante las funciones Set
        /// u otras. A diferencia de la funcin Insert, no efecta ninguna copia o limpieza
        /// sobre el area de insercin, siendo el contenido de esta indeterminado.
        /// </summary>
        /// <param name="startIndex">Index de inicio de insercin.</param>
        /// <param name="count">Cantidad de elementos a reservar.</param>
        public void Reserve(int startIndex, int count)
        {
            if (count!=0)
            {
                if (count==1)
                    this.Reserve(startIndex);
                else if (startIndex<0 || count<0)
                    throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE);
                else if (startIndex==this._size)
                {
                    this.EnsureAppend(count);
                    this._size+=count;
                }
                else if (startIndex>this._size)
                    throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.SUPERIOR_ARRAY);
                else
                {
                    int nue=this._size + count;
                    if (nue > this._cap)
                    {
                        PChar[] array2=newArraySum(count);
                        fixed (PChar* po=this._items, pd=array2)
                        {
                            if (startIndex!=0)
                                General.Copy(po, pd, startIndex);
                            General.Copy(po + startIndex, pd + (startIndex + count), this._size - startIndex);
                        }
                        this._items=array2;
                    }
                    else
                    {
                        fixed (PChar* po=this._items)
                            General.Move(po + startIndex, po + (startIndex + count), this._size - startIndex);
                    }
                    this._size=nue;
                }
            }
        }

        /// <summary>
        /// Setea el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="index">Index en la coleccin donde se setear el elemento.</param>
        public void Set(int index, PChar item)
        {
            this[index]=item;
        }


        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a setear.</param>
        public void Set(int startIndex, PChar item, int count)
        {
            if (count==1)
                this[startIndex]=item;
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (PChar* po=this._items)
                    General.Fill(po + startIndex, count, item);
        }

        /// <summary>
        /// Setea un cero en la ubicacin especificada.
        /// </summary>
        /// <param name="index">Index en la coleccin donde setear.</param>
        public void SetBlank(int index)
        {
            this[index]=PChar.Zero;
        }


        /// <summary>
        /// Setea ceros en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a setear.</param>
        public void SetBlank(int startIndex, int count)
        {
            if (count==1)
                this[startIndex]=PChar.Zero;
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (PChar* po=this._items)
                    General.Clean(po + startIndex, count);
        }

        /// <summary>
        /// Modifica la cantidad de elementos en la coleccin.
        /// </summary>
        /// <param name="count">Nueva cantidad de elementos.</param>
        /// <param name="exact">En caso de que sea necesario aumentar la capacidad, no aplica algoritmo al aumento.</param>
        /// <param name="clean">En caso de aumentar la cantidad, rellena con ceros el nuevo espacio.</param>
        public void SetLength(int count, bool exact, bool clean)
        {
            if (count <=this._cap)
            {
                if (count <=this._size)
                {
                    if (count < 0)
                        throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                }
                else if (clean)
                    fixed (PChar* p=this._items)
                        General.Clean(p + this._size, count - this._size);
            }
            else
            {
                PChar[] array2;
                if (exact)
                {
                    array2=new PChar[count];
                    this._cap=count;
                }
                else
                    array2=newArraySum(count-this._size);

                if (this._size !=0)
                {
                    fixed (PChar* po=this._items, pd=array2)
                        General.Copy(po, pd, this._size);
                }
                this._items=array2;
            }
            this._size=count;
        }

        /// <summary>
        /// Modifica la cantidad de elementos para que coincida con la capacidad actual.
        /// </summary>
        /// <param name="clean">Rellena con ceros el rea excedente.</param>
        public void SetLengthToCapacity(bool clean)
        {
            if (this._size!=this._cap)
            {
                if (clean)
                    fixed(PChar*p=this._items)
                        General.Clean(p+this._size,this._cap-this._size);
                this._size=this._cap;
            }

        }


        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Set(int startIndex, PChar[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (PChar* po=arr)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Set(int startIndex, PChar[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (PChar* po=arr)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, PChar[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (PChar* po=arr)
                    this.Set(startIndex, po + arrIndex, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, PChar* arr, int count)
        {
            if (count==1)
                this[startIndex]=*arr;
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (PChar* p=this._items)
                    General.Copy(arr,p + startIndex, count);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortAscending()
        {
            if (this._size > 1)
                fixed (PChar* p=this._items)
                    Sort.Ascending(p, this._size);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortAscending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (PChar* p=this._items)
                    Sort.Ascending(p + startIndex, count);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortDescending()
        {
            if (this._size > 1)
                fixed (PChar* p=this._items)
                    Sort.Descending(p, this._size);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortDescending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (PChar* p=this._items)
                    Sort.Descending(p + startIndex, count);
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        public void SortRandom()
        {
            if (this._size > 1)
                fixed (PChar* p=this._items)
                    Sort.Randomize(p, this._size);
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortRandom(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (PChar* p=this._items)
                    Sort.Randomize(p + startIndex, count);
        }


        #endregion

        #region Metodos de Bsqueda, reemplazo y comparacin.

        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="first">Se asegura de que la ubicacin de retorno corresponda a la primer aparicin del elemento especificado.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// La coleccin debe estar ordenada en forma ascendente.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public int BinarySearch(PChar item, bool first)
        {
            if (this._size == 0)
                return -1;
            fixed (PChar* p = this._items)
            {
                int ret = Comp.BinarySearch(p, this._size, item);
                if (first && ret > 0)
                    while (p[ret - 1] == item && --ret != 0) ;
                return ret;
            }
        }
        #endregion
    }
}
