using System;
using EnPalermo.Uti;
using EnPalermo.Lit;
using EnPalermo.Arr;
using EnPalermo.Txt;

namespace EnPalermo.Col
{
    /// <summary>
    /// Coleccin de tipos de referencia basada en array.
    /// A diferencia de las colecciones estndar, permite el acceso directo al array subyacente.
    /// </summary>
    public sealed class Strings
    {

        #region Campos

#pragma warning disable 429
        /// <summary>
        /// Capacidad por defecto para esta coleccin.
        /// </summary>
        public const int DefaultCapacity=8;
#pragma warning restore 429

        /// <summary>
        /// Sumatoria por defecto para algorritmo de aumento de capacidad.
        /// </summary>
        private const int sumCap=4;
       
        /// <summary>
        /// Largo actual del array.
        /// </summary>
        private int _size;

        /// <summary>
        /// Capacidad Actual. Si es cero el array subyacente podr ser null.
        /// </summary>
        private int _cap;
        
        /// <summary>
        /// Array subyacente.
        /// </summary>
        private string[] _items;

        #endregion

        #region Constructores

        /// <summary>
        /// Inicializa la coleccin con un array nulo.
        /// </summary>
        public Strings()
        {}

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada y largo 0.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        public Strings(int cap)
        {
            if (cap!=0)
            {
                this._items=new string[cap];
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="count">Cantidad inicial de elementos.</param>
        public Strings(int cap, int count)
        {
            if (count<0 || count>cap)
                throw new ArgumentOutOfRangeException("cap,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (cap!=0)
            {
                this._items=new string[cap];
                this._size=count;
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="countCap">La cantidad inicial de elementos ser igual a la capacidad.</param>
        public Strings(int cap, bool countCap)
        {
            if (cap!=0)
            {
                this._items=new string[cap];
                this._cap=cap;
                if (countCap)
                    this._size=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece a la capacidad del array.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        public Strings(string[] arr, bool asRef):this(arr,asRef,arr==null?0:arr.Length){}

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece de acuerdo a iLength.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        /// <param name="iLength">
        /// Largo inicial (no puede ser negativo ni superar el largo del array).
        /// </param>
        public Strings(string[] arr, bool asRef, int iLength)
        {
            if (iLength<1)
            {
                if (iLength==0)
                {
                    if (asRef && arr!=null)
                    {
                        this._items=arr;
                        this._cap=arr.Length;
                    }
                    return;
                }
            }
            else if (arr!=null)
            {
                if (asRef)
                {
                    this._cap=arr.Length;
                    if (iLength<=this._cap)
                    {
                        this._items=arr;
                        this._size=iLength;
                        return;
                    }
                }
                else
                {
                    this._cap=Calc.MultMajor(iLength+sumCap,DefaultCapacity,false);
                    this._size=iLength;
                    this._items=ArrGeneral.Clone(arr,0,iLength,0,this._cap);
                    return;
                }
                
            }
            throw new ArgumentOutOfRangeException("iLength", iLength, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array (puede ser null).</param>
        public Strings(string[] arr):this(arr,0,arr==null?0:arr.Length,sumCap){}


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, count deber ser 0.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public Strings(string[] arr, int count):this(arr,0,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public Strings(string[] arr, int arrIndex, int count):this(arr,arrIndex,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="extraCap">Capacidad extra para agregar al array subyacente.</param>
        public Strings(string[] arr, int arrIndex, int count, int extraCap)
        {
            if (count<1)
            {
                if (count==0 && arrIndex==0)
                {
                    if (extraCap>0)
                    {
                        this._cap=Calc.AddIfOdd(extraCap);
                        this._items=new string[this._cap];
                    }
                    return;
                }
            }
            else if (arrIndex>-1 && arrIndex+count<=arr.Length)
            {
                this._items=new string[this._cap=Calc.AddIfOdd(extraCap+count)];
                System.Array.Copy(arr,arrIndex,this._items,0,count);
                this._size=count;
                return;
            }
            throw new ArgumentOutOfRangeException("arrIndex,count,extraCap", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }

        #endregion

        #region Asistentes

        /// <summary>
        /// Aplica suma sobre la mitad de DefaultCapacity y luego otra suma de redondeo a DefaultCapacity.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int round(int minItems)
        {
            return Calc.MultMajor(minItems+sumCap,DefaultCapacity,true);
        }

        /// <summary>
        /// Genera una nueva capacidad mnima aplicando algorritmo y redondeando.
        /// </summary>
        /// <param name="sum">Elementos a agregar.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int newMinCap(int sum)
        {
            if (sum==1)
            {
                if (this._size==0)
                    return sumCap;
                if (this._size<=(DefaultCapacity*(1<<10)))
                    return round(this._size<<1);
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>1));
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>2));
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>3));
                return round(this._size+(this._size>>4));
            }
            if (this._size==0)
            {
                if (sum<=DefaultCapacity)
                    return round(sum+DefaultCapacity);
                if (sum<=(DefaultCapacity*(1<<5)))
                    return round(sum<<1);
                if (sum<=(DefaultCapacity*(1<<10)))
                    return round(sum+(sum>>1));
                if (sum<=(DefaultCapacity*(1<<15)))
                    return round(sum+(sum>>2));
                if (sum<=(DefaultCapacity*(1<<18)))
                    return round(sum+(sum>>3));
                return round(sum+(sum>>4));
            }
            if (sum<=DefaultCapacity || this._size>sum)
            {
                if (this._size<=(DefaultCapacity*(1<<10)))
                {
                    if (this._size>sum)
                        return round(this._size<<1);
                    return round(this._size+(this._size>>1)+sum);
                }
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>2)+sum);
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>3)+sum);
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>4)+sum);
                return round(this._size+(this._size>>5)+sum);
            }
            if (this._size<=(DefaultCapacity*(1<<10)))
                return round(this._size+(this._size>>3)+sum);
            if (this._size<=(DefaultCapacity*(1<<15)))
                return round(this._size+(this._size>>4)+sum);
            if (this._size<=(DefaultCapacity*(1<<18)))
                return round(this._size+(this._size>>5)+sum);
            return round(this._size+sum);
        }

        /// <summary>
        /// Genera un nuevo array en base al tamao mnimo especificado,
        /// sumando la mitad de DefaultCapacity y redondeando.
        /// Tambin actualiza capacidad.
        /// A diferencia de newArraySumItems no compara en algorritmo 
        /// el crecimiento sobre el array actual.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Nuevo array.</returns>
        private string[] newArrayMin(int minItems)
        {
            return new string[this._cap=round(minItems)];
        }


        /// <summary>
        /// Genera un nuevo array en base al crecimiento requerido y aplicando algorritmo.
        /// Tambin actualiza capacidad.
        /// </summary>
        /// <param name="sumItems">Crecimiento requerido.</param>
        private string[] newArraySum(int sumItems)
        {
            return new string[this._cap=newMinCap(sumItems)];
        }

        /// <summary>
        /// Actualiza la capacidad para aumento.
        /// </summary>
        /// <param name="items">Nueva cantidad.</param>
        private void setCapacity(int items)
        {
            string[] array2=new string[items];
            if (this._size !=0)
                System.Array.Copy(this._items,0,array2,0,this._size);
            this._items=array2;
            this._cap=items;
        }

        #endregion

        #region Propiedades

        /// <summary>
        /// Obtiene el array que actualmente esta siendo usado por la coleccin (puede ser null) y,
        /// por lo tanto, suceptible de ser modificado o reemplazado por la coleccin.
        /// Al establecer, cambia la instancia del array por el especificado en parametro y 
        /// setea la cantidad de elementos en el largo del nuevo array (si es nulo ser 0).
        /// </summary>
        public string[] Array
        {
            get
            {
                return this._items;
            }
            set
            {
                this._cap=value==null?0:value.Length;
                this._size=this._cap;
                this._items=value;
            }
        }

        /// <summary>
        /// Obtiene un array con los elementos de la coleccin.
        /// Esta copia puede ser el mismo objeto subyacente si la cantidad de elementos coincide con el largo de array
        /// o un array vacio si la capacidad es cero.
        /// </summary>
        public string[] ArrayCompact
        {
            get
            {
                if (this._cap==0)
                    return ArrGeneral.ArrayEmptyString;
                if (this._size==this._cap)
                    return this._items;
                return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
            }
        }

        /// <summary>
        /// Obtiene una copia compacta al largo actual de la coleccin.
        /// Al establecer, copia los elementos de la instancia actual al array subyacente o, en caso que el
        /// array de entrada sea de capacidad mayor, genera una nueva instancia con la copia de los elementos.
        /// Setea la cantidad de elementos en el largo del nuevo array.
        /// </summary>
        public string[] ArrayCopy
        {
            get
            {
                if (this._size!=0)
                    return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
                return ArrGeneral.ArrayEmptyString;
            }
            set
            {
                this._size=value==null?0:value.Length;
                if (this._size !=0)
                {
                    if (this._cap < this._size)
                        this._items=newArrayMin(this._size+sumCap);
                    System.Array.Copy(value,0,this._items,0,this._size);
                }
            }
        }



        /// <summary>
        /// Obtiene o establece, solo para aumentar, la capacidad del Array.
        /// En caso de establecer una capacidad menor, se ignora. Si se establece capacidad mayor, el valor ser
        /// redondeado.
        /// </summary>
        public int Capacity
        {
            get
            {
                return this._cap;
            }
            set
            {
                if (value > this._cap)
                    setCapacity(round(value));
            }
        }


        /// <summary>
        /// Obtiene o establece la cantidad de elementos en la coleccin.
        /// Se permite el establecimiento de un largo mayor al actual siempre y cuanto este no exceda la capacidad
        /// del array subyacente (el contenido de los elementos excedentes no ser modificado).
        /// Por lo tanto, si se especifica una cantidad negativa o mayor a la capacidad actual,
        /// se genera una excepcin.
        /// En todo caso, para modificar el largo con mayor flexibilidad, usar el mtodo SetLength.
        /// </summary>
        public int Length
        {
            get
            {
                return this._size;
            }
            set
            {
                if (value > this._cap || value < 0)
                    throw new ArgumentOutOfRangeException("value", value, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                this._size=value;
            }
        }


        /// <summary>
        /// Obtiene o establece el elemento especificado en index.
        /// Para acceder a los elementos frecuentemente, es conveniente acceder al array directamente.
        /// Si el index se encuentra fuera de rango, genera excepcin.
        /// </summary>
        public string this[int index]
        {
            get
            {
                if (this._size <=index)
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
                return this._items[index];
            }
            set
            {
                if (index < this._size)
                    this._items[index]=value;
                else 
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
            }
        }


        #endregion

        #region Metodos Administrativos

        /// <summary>
        /// Agrega un nuevo elemento al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        public void Append(string item)
        {
            this.EnsureAppend();
            this._items[this._size++]=item;
        }

        /// <summary>
        /// Agrega elementos al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        /// <param name="count">Cantidad de repeticiones del elemento.</param>
        public void Append(string item, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.Append(item);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }
            this.EnsureAppend(count);
            ArrGeneral.Fill(this._items,this._size,count,item);
        }

  
        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        public void Append(string[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                this.EnsureAppend(count);
                System.Array.Copy(arr,0,this._items,this._size,count);
                this._size+=count;
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void Append(string[] arr, int count)
        {
            if (count !=0)
            {
                this.EnsureAppend(count);
                System.Array.Copy(arr,0,this._items,this._size,count);
                this._size+=count;
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Append(string[] arr, int arrIndex, int count)
        {
            if (count !=0)
            {
                this.EnsureAppend(count);
                System.Array.Copy(arr,arrIndex,this._items,this._size,count);
                this._size+=count;
            }
        }

        /// <summary>
        /// Elimina todos los elementos de la coleccin.
        /// </summary>
        public void Clear()
        {
            this._size=0;
        }

        /// <summary>
        /// Elimina todos los elementos de la coleccin y establece el array en null.
        /// </summary>
        public void ClearCompact()
        {
            this._size=0;
            this._cap=0;
            this._items=null;
        }

        /// <summary>
        /// Compacta la coleccin a la cantidad de elementos actual.
        /// Si la cantidad actual de elementos es cero, el array subyacente ser null.
        /// </summary>
        public void Compact()
        {
            if (this._size !=this._cap)
            {
                if (this._size!=0)
                    this._items=ArrGeneral.Clone(this._items,0,this._size,0,this._size);
                else
                    this._items=null;
                this._cap=this._size;
            }
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        public void CopyTo(string[] arr)
        {
            if (this._size !=0)
                System.Array.Copy(this._items,0,arr,0,this._size);
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        public void CopyTo(string[] arr, int startIndex)
        {
            int count=this._size - startIndex;
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                System.Array.Copy(this._items,startIndex,arr,0,count);
        }


        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        /// <param name="count">Elementos a copiar.</param>
        public void CopyTo(string[] arr, int startIndex, int count)
        {
            if (startIndex > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                System.Array.Copy(this._items,startIndex,arr,0,count);
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de 1 elemento.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        public void EnsureAppend()
        {
            if (this._size == this._cap)
                setCapacity(newMinCap(1));
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de n elementos.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        /// <param name="items">Elementos a agregar.</param>
        public void EnsureAppend(int items)
        {
            if ((items + this._size) > this._cap)
                setCapacity(newMinCap(items));
        }

        /// <summary>
        /// Recupera una copia de los elementos.
        /// </summary>
        /// <returns>Array con los elementos copiados.</returns>
        public string[] GetCopy()
        {
            return this.ArrayCopy;
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public string[] GetCopy(int count)
        {
            if (this._size < count || count < 0)
                throw new System.ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            return ArrGeneral.Clone(this._items,0,this._size,0,this._size);
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="startIndex">Index donde se inicia la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public string[] GetCopy(int startIndex, int count)
        {
            if (this._size - startIndex < count || count < 0 || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            return ArrGeneral.Clone(this._items,startIndex,count,0,count);
        }

        /// <summary>
        /// Quita todos los elementos del rango especificado.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        /// <param name="count">Cantidad de elementos a eliminar.</param>
        public void RemoveRange(int startIndex, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.RemoveAt(startIndex);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                return;
            }
            int max=startIndex + count;

            if (startIndex < 0 || max > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (max==this._size)
            {
                this._size=startIndex;
                return;
            }
            System.Array.Copy(this._items,max,this._items,startIndex,this._size-max);
            this._size -=count;
        }

        /// <summary>
        /// Inserta el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="index">Index en la coleccin donde se insertar el elemento.</param>
        public void Insert(int index, string item)
        {
            this.Reserve(index);
            this._items[index]=item;
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void Insert(int startIndex, string item, int count)
        {
            this.Reserve(startIndex,count);
            this.Set(startIndex,item,count);
        }


        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Insert(int startIndex, string[] arr)
        {
            int count=arr.Length;
            if (count!=0)
            {
                this.Reserve(startIndex,count);
                this.Set(startIndex,arr,count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Insert(int startIndex, string[] arr, int count)
        {
            if (count!=0)
            {
                if (arr.Length < count)
                    throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                this.Reserve(startIndex,count);
                this.Set(startIndex,arr,count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, string[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                this.Reserve(startIndex,count);
                this.Set(startIndex,arr,arrIndex,count);
            }
        }

        /// <summary>
        /// Quita del array el elemento especificado.
        /// </summary>
        /// <param name="index">Index del elemento.</param>
        public void RemoveAt(int index)
        {
            if (index >=this._size || index < 0)
                throw new System.ArgumentOutOfRangeException("index", index, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (--this._size !=index)
                System.Array.Copy(this._items,index+1,this._items,index,this._size-index);
        }

        /// <summary>
        /// Quita todos los elementos desde la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        public void RemoveRange(int startIndex)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
        }

        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, string[] r)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
            this.Append(r);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, string[] r)
        {
            if (count<1)
            {
                if (count==0)
                {
                    this.Insert(startIndex,r);
                    return;
                }
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            int max=count+startIndex;
            if (max>=this._size)
            {
                if (count==this._size)
                {
                    this.ReplaceRange(startIndex,r);
                    return;
                }
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            }
            int lr=r.Length;
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            if (count>lr)
                this.RemoveRange(startIndex+lr,count-lr);
            else if (count!=lr)
                this.Reserve(startIndex+count,lr-count);
            this.Set(startIndex,r);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        public void Reverse()
        {
             if (this._size > 1)
                System.Array.Reverse(this._items,0,this._size);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void Reverse(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                System.Array.Reverse(this._items,startIndex,count);
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de un elemento. A diferencia de la funcin Insert,
        /// no efecta ninguna copia o limpieza.
        /// </summary>
        /// <param name="index">Index de la insercin.</param>
        public void Reserve(int index)
        {
            if (index<0 || index>this._size)
                throw new ArgumentOutOfRangeException("index",index,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (index==this._size)
                this.EnsureAppend();
            else
            {
                if (this._size==this._cap)
                {
                    string[] array2=newArraySum(1);
                    if (index!=0)
                        System.Array.Copy(this._items,0,array2,0,index);
                    System.Array.Copy(this._items,index,array2,index+1,this._size-index);
                    this._items=array2;
                }
                else
                {
                    System.Array.Copy(this._items,index,this._items,index+1,this._size-index);
                }
            }
            this._size++;
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de la cantidad de elementos especificada mediante las funciones Set
        /// u otras. A diferencia de la funcin Insert, no efecta ninguna copia o limpieza
        /// sobre el area de insercin, siendo el contenido de esta indeterminado.
        /// </summary>
        /// <param name="startIndex">Index de inicio de insercin.</param>
        /// <param name="count">Cantidad de elementos a reservar.</param>
        public void Reserve(int startIndex, int count)
        {
            if (count!=0)
            {
                if (count==1)
                    this.Reserve(startIndex);
                else if (startIndex<0 || count<0)
                    throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE);
                else if (startIndex==this._size)
                {
                    this.EnsureAppend(count);
                    this._size+=count;
                }
                else if (startIndex>this._size)
                    throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.SUPERIOR_ARRAY);
                else
                {
                    int nue=this._size + count;
                    if (nue > this._cap)
                    {
                        string[] array2=newArraySum(count);
                        if (startIndex!=0)
                            System.Array.Copy(this._items,0,array2,0,startIndex);
                        System.Array.Copy(this._items,startIndex,array2,startIndex+count,this._size-startIndex);
                        this._items=array2;
                    }
                    else
                    {
                        System.Array.Copy(this._items,startIndex,this._items,startIndex+count,this._size-startIndex);
                    }
                    this._size=nue;
                }
            }
        }

        /// <summary>
        /// Setea el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="index">Index en la coleccin donde se setear el elemento.</param>
        public void Set(int index, string item)
        {
            this[index]=item;
        }


        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a setear.</param>
        public void Set(int startIndex, string item, int count)
        {
            if (count==1)
                this[startIndex]=item;
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                ArrGeneral.Fill(this._items,startIndex,count,item);
        }

        /// <summary>
        /// Modifica la cantidad de elementos en la coleccin.
        /// </summary>
        /// <param name="count">Nueva cantidad de elementos.</param>
        /// <param name="exact">En caso de que sea necesario aumentar la capacidad, no aplica algoritmo al aumento.</param>
        /// <param name="clean">En caso de aumentar la cantidad, rellena con ceros el nuevo espacio.</param>
        public void SetLength(int count, bool exact, bool clean)
        {
            if (count <=this._cap)
            {
                if (count <=this._size)
                {
                    if (count < 0)
                        throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                }
                else if (clean)
                    System.Array.Clear(this._items,this._size,count-this._size);
            }
            else
            {
                string[] array2;
                if (exact)
                {
                    array2=new string[count];
                    this._cap=count;
                }
                else
                    array2=newArraySum(count-this._size);

                if (this._size !=0)
                    System.Array.Copy(this._items,0,array2,0,this._size);
                this._items=array2;
            }
            this._size=count;
        }

        /// <summary>
        /// Modifica la cantidad de elementos para que coincida con la capacidad actual.
        /// </summary>
        /// <param name="clean">Rellena con ceros el rea excedente.</param>
        public void SetLengthToCapacity(bool clean)
        {
            if (this._size!=this._cap)
            {
                if (clean)
                    System.Array.Clear(this._items,this._size,this._cap-this._size);
                this._size=this._cap;
            }
        }


        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Set(int startIndex, string[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                if (startIndex+count>this._size)
                    throw new ArgumentOutOfRangeException("startIndex,count",Errors.SUPERIOR_ARRAY);
                System.Array.Copy(arr,0,this._items,startIndex,count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Set(int startIndex, string[] arr, int count)
        {
            if (count !=0)
            {
                if (startIndex+count>this._size)
                    throw new ArgumentOutOfRangeException("startIndex,count",Errors.SUPERIOR_ARRAY);
                System.Array.Copy(arr,0,this._items,startIndex,count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, string[] arr, int arrIndex, int count)
        {
            if (count !=0)
            {
                if (startIndex+count>this._size)
                    throw new ArgumentOutOfRangeException("startIndex,count",Errors.SUPERIOR_ARRAY);
                System.Array.Copy(arr,arrIndex,this._items,startIndex,count);
            }
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortAscending()
        {
            if (this._size > 1)
                System.Array.Sort(this._items,0,this._size);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortAscending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                System.Array.Sort(this._items,startIndex,count);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortDescending()
        {
            if (this._size > 1)
            {
                System.Array.Sort(this._items,0,this._size);
                System.Array.Reverse(this._items,0,this._size);
            }
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortDescending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
            {
                System.Array.Sort(this._items,startIndex,count);
                System.Array.Reverse(this._items,startIndex,count);
            }
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        public void SortRandom()
        {
            if (this._size > 1)
                ArrSort.Randomize(this._items,0,this._size);
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortRandom(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                ArrSort.Randomize(this._items,startIndex,count);
        }


        #endregion

        #region Set, Insert y Append para cantidades limitadas de elementos

        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, string item0, string item1)
        {
            if (startIndex<0 || startIndex+1>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._items[startIndex]=item0;
            this._items[startIndex+1]=item1;
        }

        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, string item0, string item1, string item2)
        {
            if (startIndex<0 || startIndex+2>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._items[startIndex]=item0;
            this._items[startIndex+1]=item1;
            this._items[startIndex+2]=item2;
        }


        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, string item0, string item1, string item2, string item3)
        {
            if (startIndex<0 || startIndex+3>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._items[startIndex]=item0;
            this._items[startIndex+1]=item1;
            this._items[startIndex+2]=item2;
            this._items[startIndex+3]=item3;
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, string item0, string item1)
        {
            this.Reserve(startIndex,2);
            this._items[startIndex]=item0;
            this._items[startIndex+1]=item1;
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, string item0, string item1, string item2)
        {
            this.Reserve(startIndex,3);
            this._items[startIndex]=item0;
            this._items[startIndex+1]=item1;
            this._items[startIndex+2]=item2;
        }


        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, string item0, string item1, string item2, string item3)
        {
            this.Reserve(startIndex,4);
            this._items[startIndex]=item0;
            this._items[startIndex+1]=item1;
            this._items[startIndex+2]=item2;
            this._items[startIndex+3]=item3;
        }

        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        public void AppendItems(string item0, string item1)
        {
            this.EnsureAppend(2);
            this._items[this._size]=item0;
            this._items[this._size+1]=item1;
            this._size+=2;
        }

        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        public void AppendItems(string item0, string item1, string item2)
        {
            this.EnsureAppend(3);
            this._items[this._size]=item0;
            this._items[this._size+1]=item1;
            this._items[this._size+2]=item2;
            this._size+=3;
        }


        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        public void AppendItems(string item0, string item1, string item2, string item3)
        {
            this.EnsureAppend(4);
            this._items[this._size]=item0;
            this._items[this._size+1]=item1;
            this._items[this._size+2]=item2;
            this._items[this._size+3]=item3;
            this._size+=4;
        }

        #endregion

        #region Metodos de Bsqueda, reemplazo y comparacin.
  
        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// La coleccin debe estar ordenada en forma ascendente.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public int BinarySearch(string item)
        {
            if (this._size==0)
                return -1;
            return System.Array.BinarySearch(this._items,0, this._size, item);
        }

        /// <summary>
        /// Retorna true si contiene el elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(string item,bool last)
        {
            if (this._size==0)
                return false;
            if (!last)
                return System.Array.IndexOf(this._items,item,0, this._size)!=-1;
            return System.Array.LastIndexOf(this._items, item, this._size-1, this._size) != -1;
        }



        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(string item)
        {
            return this.CountOf(item,0,this._size);
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(string item, int startIndex)
        {
            return this.CountOf(item,startIndex,this._size-startIndex);
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(string item, int startIndex, int count)
        {
            if ((count+=startIndex) > this._size || startIndex < 0 || count < startIndex)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int tot=0;
            if (count!=startIndex)
            {
                string[] arr=this._items;
                do
                {
                    if (arr[startIndex]==item)
                        tot++;
                }while(++startIndex!=count);
            }
            return tot;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(string item)
        {
            if (this._size !=0)
                return System.Array.IndexOf(this._items,item,0,this._size);
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(string item, int startIndex)
        {
            if (this._size==0)
            {
                if (startIndex!=0)
                    throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.SUPERIOR_ARRAY);
                return -1;
            }
            return System.Array.IndexOf(this._items,item,startIndex,this._size-startIndex);
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(string item, int startIndex, int count)
        {
            if (startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            return System.Array.IndexOf(this._items,item,startIndex,count);
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(string item)
        {
            if (this._size !=0)
                return System.Array.LastIndexOf(this._items, item, this._size - 1, this._size);
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(string item, int startIndex)
        {
            if (this._size==0)
            {
                if (startIndex!=0)
                    throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.SUPERIOR_ARRAY);
                return -1;
            }
            return System.Array.LastIndexOf(this._items,item,this._size-(1+startIndex),this._size-startIndex);
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(string item, int startIndex, int count)
        {
            if (startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            return System.Array.LastIndexOf(this._items, item, this._size - (1 + startIndex), count);

        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(string item)
        {
            return this.RemoveAll(item,0,this._size);
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(string item, int startIndex)
        {
            return this.RemoveAll(item,startIndex,this._size-startIndex);
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(string item, int startIndex, int count)
        {
            if ((count+=startIndex) > this._size || startIndex < 0 || count < startIndex)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=startIndex)
            {
                string[] arr=this._items;
                do
                {
                    if(arr[startIndex]==item)
                    {
                        int tot=1;
                        while(++startIndex!=count)
                        {
                            if(arr[startIndex]!=item)
                                arr[startIndex-tot]=item;
                            else
                                tot++;
                        }
                        this._size-=tot;
                        return tot;
                    }
                }while(++startIndex!=count);
            }
            return 0;
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(string item)
        {
            int ret=this.LastIndexOf(item);
            if (ret!=-1)
                this.RemoveAt(ret);
            return ret;
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(string item, int startIndex)
        {
            int ret=this.LastIndexOf(item,startIndex);
            if (ret!=-1)
                this.RemoveAt(ret);
            return ret;
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(string item, int startIndex, int count)
        {
            int ret=this.LastIndexOf(item,startIndex,count);
            if (ret!=-1)
                this.RemoveAt(ret);
            return ret;
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(string item)
        {
            int ret=this.IndexOf(item);
            if (ret!=-1)
                this.RemoveAt(ret);
            return ret;
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(string item, int startIndex)
        {
            int ret=this.IndexOf(item,startIndex);
            if (ret!=-1)
                this.RemoveAt(ret);
            return ret;
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(string item, int startIndex, int count)
        {
            int ret=this.IndexOf(item,startIndex,count);
            if (ret!=-1)
                this.RemoveAt(ret);
            return ret;
        }
        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(string item, string rep)
        {
            return ReplaceAll(item,rep,0,this._size);
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(string item, string rep, int startIndex)
        {
            return ReplaceAll(item,rep,startIndex,this._size-startIndex);
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(string item, string rep, int startIndex, int count)
        {
            if ((count+=startIndex) > this._size || startIndex < 0 || count < startIndex)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int tot=0;
            if (count!=startIndex)
            {
                string[] arr=this._items;
                do
                {
                    if (arr[startIndex]==item)
                    {
                        arr[startIndex]=rep;
                        tot++;
                    }
                }while(++startIndex!=count);
            }
            return tot;
        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(string item, string rep)
        {
            int ret=this.LastIndexOf(item);
            if (ret!=-1)
                this._items[ret]=rep;
            return ret;        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(string item, string rep, int startIndex)
        {
            int ret=this.LastIndexOf(item,startIndex);
            if (ret!=-1)
                this._items[ret]=rep;
            return ret;        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(string item, string rep, int startIndex, int count)
        {
            int ret=this.LastIndexOf(item,startIndex,count);
            if (ret!=-1)
                this._items[ret]=rep;
            return ret;
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOf(string item, string rep)
        {
            int ret=this.IndexOf(item);
            if (ret!=-1)
                this._items[ret]=rep;
            return ret;
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(string item, string rep, int startIndex)
        {
            int ret=this.IndexOf(item,startIndex);
            if (ret!=-1)
                this._items[ret]=rep;
            return ret;
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(string item, string rep, int startIndex, int count)
        {
            int ret=this.IndexOf(item,startIndex,count);
            if (ret!=-1)
                this._items[ret]=rep;
            return ret;
        }

        #endregion

        #region Metodos de Bsqueda, reemplazo y comparacin NC.

        /// <summary>
        /// Retorna true si contiene el elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(string item, bool last, Ordinal comp)
        {
            if (this._size == 0)
                return false;
            return TxtComp.IndexOf(this._items, 0, this._size, item, comp, last) != -1;
        }



        
        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(string item, bool last, Ordinal comp)
        {
            if (this._size != 0)
                return TxtComp.IndexOf(this._items, 0, this._size, item, comp, last);
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(string item, int startIndex, bool last, Ordinal comp)
        {
            if (this._size == 0)
            {
                if (startIndex != 0)
                    throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.SUPERIOR_ARRAY);
                return -1;
            }
            return TxtComp.IndexOf(this._items, startIndex, this._size - startIndex, item, comp, last);
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(string item, int startIndex, int count, bool last, Ordinal comp)
        {
            if (startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.SUPERIOR_ARRAY);
            if (count == 0)
                return -1;
            return TxtComp.IndexOf(this._items, startIndex, count, item, comp, last);
        }
        
       
        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(string item, bool last, Ordinal comp)
        {
            int ret = this.IndexOf(item, last, comp);
            if (ret != -1)
                this.RemoveAt(ret);
            return ret;
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(string item, int startIndex, bool last, Ordinal comp)
        {
            int ret = this.IndexOf(item, startIndex, last, comp);
            if (ret != -1)
                this.RemoveAt(ret);
            return ret;
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(string item, int startIndex, int count, bool last, Ordinal comp)
        {
            int ret = this.IndexOf(item, startIndex, count, last, comp);
            if (ret != -1)
                this.RemoveAt(ret);
            return ret;
        }
        
        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOf(string item, string rep, bool last, Ordinal comp)
        {
            int ret = this.IndexOf(item, last, comp);
            if (ret != -1)
                this._items[ret] = rep;
            return ret;
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(string item, string rep, int startIndex, bool last, Ordinal comp)
        {
            int ret = this.IndexOf(item, startIndex,last,comp);
            if (ret != -1)
                this._items[ret] = rep;
            return ret;
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// La bsqueda se realiza mediante comparacin ordinal.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(string item, string rep, int startIndex, int count, bool last, Ordinal comp)
        {
            int ret = this.IndexOf(item, startIndex, count,last,comp);
            if (ret != -1)
                this._items[ret] = rep;
            return ret;
        }

        #endregion

    }
}
