using System;
using EnPalermo.Mem;
using EnPalermo.Uti;
using EnPalermo.Lit;
using EnPalermo.Arr;

namespace EnPalermo.Col
{
    /// <summary>
    /// Coleccin de tipos de valor basada en array.
    /// A diferencia de las colecciones estndar, permite el acceso directo al array subyacente y es de mejor
    /// rendimiento dado que no usa boxing/unboxing ni es afectado por las limitaciones relativas al uso de
    /// genricos, como la imposibilidad de usar cdigo inseguro.
    /// Nota: Esta coleccin no es segura en tamaos equivalentes o mayores a (Int32.MaxValue/(2*sizeof(tipo)):
    /// Esto es: 1073741823 bytes, 536870911 shorts o chars, 268435455 ints o 134217727 longs.
    /// </summary>
    public unsafe sealed class UShorts
    {
        #region Campos

#pragma warning disable 429
        /// <summary>
        /// Capacidad por defecto para esta coleccin.
        /// </summary>
        public const int DefaultCapacity=((16 / sizeof(ushort)) !=8 && (16 / sizeof(ushort)) !=16) ? (4) : (16 / sizeof(ushort));

        /// <summary>
        /// Sumatoria por defecto para algorritmo de aumento de capacidad.
        /// </summary>
        private const int sumCap=DefaultCapacity>=8?4:2;
#pragma warning restore 429

        /// <summary>
        /// Largo actual del array.
        /// </summary>
        private int _size;

        /// <summary>
        /// Capacidad Actual. Si es cero el array subyacente podr ser null.
        /// </summary>
        private int _cap;


        /// <summary>
        /// Flags para almacenar informacin variada acerca del objeto.
        /// Definidas: 
        /// - Bytes  0 al 7: Tolerancia para Compact (SB). De 0 a 255.
        /// - Byte   8: AppendExact.
        /// - Byte   9: Unix definido (Bytes, UShorts y SB).
        /// - Byte  10: Unix (Bytes, UShorts y SB).
        /// - Bytes 11 y 12: Tabla MayMin (UShorts y SB).
        /// - Bytes 12, 13, 14 y 15: Cantidad mxima de bytes por ushort segn cdigo de pgina (Bytes).
        /// - Bytes 16 al 31: Cdigo de pgina (Bytes).
        /// </summary>
        private uint _flags;

        /// <summary>
        /// Array subyacente.
        /// </summary>
        private ushort[] _items;

        #endregion

        #region Constructores

        /// <summary>
        /// Inicializa la coleccin con un array nulo.
        /// </summary>
        public UShorts()
        {}

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada y largo 0.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        public UShorts(int cap)
        {
            if (cap!=0)
            {
                this._items=new ushort[cap];
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin con un array nulo, permitiendo especificar la propiedad AppendExact.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        public UShorts(bool appendExact)
        {
            if (appendExact)
                this._flags=1U<<8;
        }

        /// <summary>
        /// Inicializa la coleccin permitiendo especificar la propiedad AppendExact.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="cap">Capacidad inicial.</param>
        public UShorts(bool appendExact, int cap):this(cap)
        {
            if (appendExact)
                this._flags=1U<<8;
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="count">Cantidad inicial de elementos.</param>
        public UShorts(int cap, int count)
        {
            if (count<0 || count>cap)
                throw new ArgumentOutOfRangeException("cap,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (cap!=0)
            {
                this._items=new ushort[cap];
                this._size=count;
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="countCap">La cantidad inicial de elementos ser igual a la capacidad.</param>
        public UShorts(int cap, bool countCap)
        {
            if (cap!=0)
            {
                this._items=new ushort[cap];
                this._cap=cap;
                if (countCap)
                    this._size=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece a la capacidad del array.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        public UShorts(ushort[] arr, bool asRef):this(arr,asRef,arr==null?0:arr.Length){}

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece de acuerdo a iLength.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        /// <param name="iLength">
        /// Largo inicial (no puede ser negativo ni superar el largo del array).
        /// </param>
        public UShorts(ushort[] arr, bool asRef, int iLength)
        {
            if (iLength<1)
            {
                if (iLength==0)
                {
                    if (asRef && arr!=null)
                    {
                        this._items=arr;
                        this._cap=arr.Length;
                    }
                    return;
                }
            }
            else if (arr!=null)
            {
                if (asRef)
                {
                    this._cap=arr.Length;
                    if (iLength<=this._cap)
                    {
                        this._items=arr;
                        this._size=iLength;
                        return;
                    }
                }
                else
                {
                    this._cap=Calc.MultMajor(iLength+sumCap,DefaultCapacity,false);
                    this._size=iLength;
                    this._items=ArrGeneral.Clone(arr,0,iLength,0,this._cap);
                    return;
                }
                
            }
            throw new ArgumentOutOfRangeException("iLength", iLength, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array (puede ser null).</param>
        public UShorts(ushort[] arr):this(arr,0,arr==null?0:arr.Length,sumCap){}


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, count deber ser 0.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public UShorts(ushort[] arr, int count):this(arr,0,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public UShorts(ushort[] arr, int arrIndex, int count):this(arr,arrIndex,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="extraCap">Capacidad extra para agregar al array subyacente.</param>
        public UShorts(ushort[] arr, int arrIndex, int count, int extraCap)
        {
            if (count<1)
            {
                if (count==0 && arrIndex==0)
                {
                    if (extraCap>0)
                    {
                        this._cap=Calc.AddIfOdd(extraCap);
                        this._items=new ushort[this._cap];
                    }
                    return;
                }
            }
            else if (arrIndex>-1 && arrIndex+count<=arr.Length)
            {
                this._items=new ushort[this._cap=Calc.AddIfOdd(extraCap+count)];
                fixed(ushort*po=arr,pd=this._items)
                    General.Copy(po+arrIndex,pd,count);
                this._size=count;
                return;
            }
            throw new ArgumentOutOfRangeException("arrIndex,count,extraCap", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }

        #endregion

        #region Asistentes

        /// <summary>
        /// Aplica suma sobre la mitad de DefaultCapacity y luego otra suma de redondeo a DefaultCapacity.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int round(int minItems)
        {
            return Calc.MultMajor(minItems+sumCap,DefaultCapacity,true);
        }

        /// <summary>
        /// Genera una nueva capacidad mnima aplicando algorritmo y redondeando.
        /// </summary>
        /// <param name="sum">Elementos a agregar.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int newMinCap(int sum)
        {
            if (this.AppendExact)
                return round(this._size+sum);

            if (sum==1)
            {
                if (this._size==0)
                    return sumCap;
                if (this._size<=(DefaultCapacity*(1<<10)))
                    return round(this._size<<1);
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>1));
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>2));
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>3));
                return round(this._size+(this._size>>4));
            }
            if (this._size==0)
            {
                if (sum<=DefaultCapacity)
                    return round(sum+DefaultCapacity);
                if (sum<=(DefaultCapacity*(1<<5)))
                    return round(sum<<1);
                if (sum<=(DefaultCapacity*(1<<10)))
                    return round(sum+(sum>>1));
                if (sum<=(DefaultCapacity*(1<<15)))
                    return round(sum+(sum>>2));
                if (sum<=(DefaultCapacity*(1<<18)))
                    return round(sum+(sum>>3));
                return round(sum+(sum>>4));
            }
            if (sum<=DefaultCapacity || this._size>sum)
            {
                if (this._size<=(DefaultCapacity*(1<<10)))
                {
                    if (this._size>sum)
                        return round(this._size<<1);
                    return round(this._size+(this._size>>1)+sum);
                }
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>2)+sum);
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>3)+sum);
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>4)+sum);
                return round(this._size+(this._size>>5)+sum);
            }
            if (this._size<=(DefaultCapacity*(1<<10)))
                return round(this._size+(this._size>>3)+sum);
            if (this._size<=(DefaultCapacity*(1<<15)))
                return round(this._size+(this._size>>4)+sum);
            if (this._size<=(DefaultCapacity*(1<<18)))
                return round(this._size+(this._size>>5)+sum);
            return round(this._size+sum);
        }

        /// <summary>
        /// Genera un nuevo array en base al tamao mnimo especificado,
        /// sumando la mitad de DefaultCapacity y redondeando.
        /// Tambin actualiza capacidad.
        /// A diferencia de newArraySumItems no compara en algorritmo 
        /// el crecimiento sobre el array actual.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Nuevo array.</returns>
        private ushort[] newArrayMin(int minItems)
        {
            return new ushort[this._cap=round(minItems)];
        }


        /// <summary>
        /// Genera un nuevo array en base al crecimiento requerido y aplicando algorritmo.
        /// Tambin actualiza capacidad.
        /// </summary>
        /// <param name="sumItems">Crecimiento requerido.</param>
        private ushort[] newArraySum(int sumItems)
        {
            return new ushort[this._cap=newMinCap(sumItems)];
        }

        /// <summary>
        /// Actualiza la capacidad para aumento.
        /// </summary>
        /// <param name="items">Nueva cantidad.</param>
        private void setCapacity(int items)
        {
            ushort[] array2=new ushort[items];
            if (this._size !=0)
            {
                fixed (ushort* po=this._items, pd=array2)
                    General.Copy(po, pd, this._size);
            }
            this._items=array2;
            this._cap=items;
        }

        #endregion

        #region Propiedades

        /// <summary>
        /// Si true, no usar algoritmo de aumento para la coleccin, aunque igual efectuar redondeo.
        /// Por lo tanto, si este parmetro esta activo se redimensionar
        /// la coleccion cada vez que se agreguen nuevos elementos.
        /// </summary>
        public bool AppendExact
        {
            get
            {
                return (this._flags&(1U<<8))!=0;
            }
            set
            {
                if (value)
                    this._flags|=1U<<8;
                else
                    this._flags&=0xFFFFFFFF^(1U<<8);
            }
        }

        /// <summary>
        /// Obtiene el array que actualmente esta siendo usado por la coleccin (puede ser null) y,
        /// por lo tanto, suceptible de ser modificado o reemplazado por la coleccin.
        /// Al establecer, cambia la instancia del array por el especificado en parametro y 
        /// setea la cantidad de elementos en el largo del nuevo array (si es nulo ser 0).
        /// </summary>
        public ushort[] Array
        {
            get
            {
                return this._items;
            }
            set
            {
                this._cap=value==null?0:value.Length;
                this._size=this._cap;
                this._items=value;
            }
        }

        /// <summary>
        /// Obtiene un array con los elementos de la coleccin.
        /// Esta copia puede ser el mismo objeto subyacente si la cantidad de elementos coincide con el largo de array
        /// o un array vacio si la capacidad es cero.
        /// </summary>
        public ushort[] ArrayCompact
        {
            get
            {
                if (this._cap==0)
                    return ArrGeneral.ArrayEmptyUShort;
                if (this._size==this._cap)
                    return this._items;
                return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
            }
        }

        /// <summary>
        /// Obtiene una copia compacta al largo actual de la coleccin.
        /// Al establecer, copia los elementos de la instancia actual al array subyacente o, en caso que el
        /// array de entrada sea de capacidad mayor, genera una nueva instancia con la copia de los elementos.
        /// Setea la cantidad de elementos en el largo del nuevo array.
        /// </summary>
        public ushort[] ArrayCopy
        {
            get
            {
                if (this._size!=0)
                    return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
                return ArrGeneral.ArrayEmptyUShort;
            }
            set
            {
                this._size=value==null?0:value.Length;
                if (this._size !=0)
                {
                    if (this._cap < this._size)
                        this._items=newArrayMin(this._size+sumCap);
                    fixed (ushort* po=value, pd=this._items)
                        General.Copy(po, pd, this._size);
                }
            }
        }



        /// <summary>
        /// Obtiene o establece, solo para aumentar, la capacidad del Array.
        /// En caso de establecer una capacidad menor, se ignora. Si se establece capacidad mayor, el valor ser
        /// redondeado.
        /// </summary>
        public int Capacity
        {
            get
            {
                return this._cap;
            }
            set
            {
                if (value > this._cap)
                    setCapacity(round(value));
            }
        }


        /// <summary>
        /// Obtiene o establece la cantidad de elementos en la coleccin.
        /// Se permite el establecimiento de un largo mayor al actual siempre y cuanto este no exceda la capacidad
        /// del array subyacente (el contenido de los elementos excedentes no ser modificado).
        /// Por lo tanto, si se especifica una cantidad negativa o mayor a la capacidad actual,
        /// se genera una excepcin.
        /// En todo caso, para modificar el largo con mayor flexibilidad, usar el mtodo SetLength.
        /// </summary>
        public int Length
        {
            get
            {
                return this._size;
            }
            set
            {
                if (value > this._cap || value < 0)
                    throw new ArgumentOutOfRangeException("value", value, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                this._size=value;
            }
        }


        /// <summary>
        /// Obtiene o establece el elemento especificado en index.
        /// Para acceder a los elementos frecuentemente, es conveniente acceder al array directamente.
        /// Si el index se encuentra fuera de rango, genera excepcin.
        /// </summary>
        public ushort this[int index]
        {
            get
            {
                if (this._size <=index)
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
                return this._items[index];
            }
            set
            {
                if (index < this._size)
                    this._items[index]=value;
                else 
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
            }
        }


        #endregion

        #region Metodos Administrativos

        /// <summary>
        /// Agrega un nuevo elemento al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        public void Append(ushort item)
        {
            this.EnsureAppend();
            this._items[this._size++]=item;
        }

        /// <summary>
        /// Agrega elementos al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        /// <param name="count">Cantidad de repeticiones del elemento.</param>
        public void Append(ushort item, int count)
        {
            if (item==0)
            {
                this.AppendBlank(count);
                return;
            }

            if (count < 2)
            {
                if (count==1)
                    this.Append(item);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }
            this.EnsureAppend(count);
            fixed (ushort* pd=this._items)
                General.Fill(pd + this._size, count, item);
            this._size +=count;
        }

        /// <summary>
        /// Agrega un cero al final de la coleccin.
        /// </summary>
        public void AppendBlank()
        {
            this.EnsureAppend();
            this._items[this._size++]=0;
        }

        /// <summary>
        /// Agrega ceros al final de la coleccin.
        /// </summary>
        /// <param name="count">Cantidad de ceros.</param>
        public void AppendBlank(int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.AppendBlank();
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }
            this.EnsureAppend(count);
            fixed (ushort* pd=this._items)
                General.Clean(pd + this._size, count);
            this._size +=count;
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        public void Append(ushort[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (ushort* po=arr)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void Append(ushort[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (ushort* po=arr)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Append(ushort[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (ushort* po=arr)
                    this.Append(po + arrIndex, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void Append(ushort* arr, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.Append(*arr);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }

            this.EnsureAppend(count);
            fixed (ushort* pd=this._items)
                General.Copy(arr, pd + this._size, count);
            this._size +=count;
        }


        /// <summary>
        /// Elimina todos los elementos de la coleccin.
        /// </summary>
        public void Clear()
        {
            this._size=0;
        }

        /// <summary>
        /// Elimina todos los elementos de la coleccin y establece el array en null.
        /// </summary>
        public void ClearCompact()
        {
            this._size=0;
            this._cap=0;
            this._items=null;
        }

        /// <summary>
        /// Compacta la coleccin a la cantidad de elementos actual.
        /// Si la cantidad actual de elementos es cero, el array subyacente ser null.
        /// </summary>
        public void Compact()
        {
            if (this._size !=this._cap)
            {
                if (this._size!=0)
                {
                    ushort[] array2=new ushort[this._size];
                    fixed (ushort* po=this._items, pd=array2)
                        General.Copy(po, pd, this._size);
                    this._items=array2;
                }
                else
                    this._items=null;
                this._cap=this._size;
            }
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        public void CopyTo(ushort* arr)
        {
            if (this._size !=0)
                fixed (ushort* po=this._items)
                    General.Move(po, arr, this._size);
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        public void CopyTo(ushort* arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (this._size !=startIndex)
                fixed (ushort* po=this._items)
                    General.Move(po + startIndex, arr, this._size - startIndex);
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        /// <param name="count">Elementos a copiar.</param>
        public void CopyTo(ushort* arr, int startIndex, int count)
        {

            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (this._size !=startIndex)
                fixed (ushort* po=this._items)
                    General.Move(po + startIndex, arr, count);
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        public void CopyTo(ushort[] arr)
        {
            if (this._size !=0)
            {
                if (arr.Length < this._size)
                    throw new ArgumentOutOfRangeException("arr", arr.Length, Errors.ARRAY_SMALL_DEST);
                fixed (ushort* po=this._items, pd=arr)
                    General.Copy(po, pd, this._size);
            }
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        public void CopyTo(ushort[] arr, int startIndex)
        {
            int count=this._size - startIndex;
            if (startIndex > this._size || startIndex < 0 || count > arr.Length)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                fixed (ushort* po=this._items, pd=arr)
                    General.Copy(po + startIndex, pd, count);
        }


        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        /// <param name="count">Elementos a copiar.</param>
        public void CopyTo(ushort[] arr, int startIndex, int count)
        {
            if (startIndex > this._size || startIndex < 0 || count < 0 || count > arr.Length)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                fixed (ushort* po=this._items, pd=arr)
                    General.Copy(po + startIndex, pd, count);
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de 1 elemento.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        public void EnsureAppend()
        {
            if (this._size == this._cap)
                setCapacity(newMinCap(1));
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de n elementos.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        /// <param name="items">Elementos a agregar.</param>
        public void EnsureAppend(int items)
        {
            if ((items + this._size) > this._cap)
                setCapacity(newMinCap(items));
        }

        /// <summary>
        /// Recupera una copia de los elementos.
        /// </summary>
        /// <returns>Array con los elementos copiados.</returns>
        public ushort[] GetCopy()
        {
            return this.ArrayCopy;
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public ushort[] GetCopy(int count)
        {
            if (this._size < count || count < 0)
                throw new System.ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            ushort[] ret=new ushort[count];
            if (count !=0)
            {
                fixed (ushort* po=this._items, pd=ret)
                    General.Copy(po, pd, count);
            }
            return ret;
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="startIndex">Index donde se inicia la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public ushort[] GetCopy(int startIndex, int count)
        {
            if (this._size - startIndex < count || count < 0 || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            ushort[] ret=new ushort[count];
            if (count !=0)
            {
                fixed (ushort* po=this._items, pd=ret)
                    General.Copy(po + startIndex, pd, count);
            }
            return ret;
        }

        /// <summary>
        /// Quita todos los elementos del rango especificado.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        /// <param name="count">Cantidad de elementos a eliminar.</param>
        public void RemoveRange(int startIndex, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.RemoveAt(startIndex);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                return;
            }
            int max=startIndex + count;

            if (startIndex < 0 || max > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (max==this._size)
            {
                this._size=startIndex;
                return;
            }
            fixed (ushort* p=this._items)
                General.Cut(p, this._size, startIndex, count, false);
            this._size -=count;
        }

        /// <summary>
        /// Inserta el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="index">Index en la coleccin donde se insertar el elemento.</param>
        public void Insert(int index, ushort item)
        {
            this.Reserve(index);
            this._items[index]=item;
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void Insert(int startIndex, ushort item, int count)
        {
            this.Reserve(startIndex,count);
            this.Set(startIndex,item,count);
        }

        /// <summary>
        /// Inserta un cero en la ubicacin especificada.
        /// </summary>
        /// <param name="index">Index en la coleccin donde insertar.</param>
        public void InsertBlank(int index)
        {
            this.Reserve(index);
            this._items[index]=0;
        }


        /// <summary>
        /// Inserta ceros en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void InsertBlank(int startIndex, int count)
        {
            this.Reserve(startIndex,count);
            this.SetBlank(startIndex,count);
        }


        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Insert(int startIndex, ushort[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (ushort* po=arr)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Insert(int startIndex, ushort[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (ushort* po=arr)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, ushort[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (ushort* po=arr)
                    this.Insert(startIndex,po + arrIndex, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, ushort* arr, int count)
        {
            this.Reserve(startIndex,count);
            this.Set(startIndex,arr,count);
        }

        /// <summary>
        /// Quita del array el elemento especificado.
        /// </summary>
        /// <param name="index">Index del elemento.</param>
        public void RemoveAt(int index)
        {
            if (index >=this._size || index < 0)
                throw new System.ArgumentOutOfRangeException("index", index, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (--this._size !=index)
            {
                fixed (ushort* p=this._items)
                    General.Copy(p + (index + 1), p + index, this._size - index);
            }

        }

        /// <summary>
        /// Quita todos los elementos desde la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        public void RemoveRange(int startIndex)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
        }

        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, ushort[] r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex);
            else
                fixed(ushort*pr=r)
                    ReplaceRange(startIndex,pr,lr);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, ushort[] r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex,count);
            else
                fixed(ushort*pr=r)
                    ReplaceRange(startIndex,count,pr,lr);
        }

        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, ushort* pr, int lr)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
            this.Append(pr,lr);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, ushort* pr, int lr)
        {
            if (count>=lr)
            {
                Set(startIndex,pr,lr);
                if (count>lr)
                    RemoveRange(startIndex+lr,count-lr);
                return;
            }
            if (count<1)
            {
                if (count==0)
                {
                    Insert(startIndex,pr,lr);
                    return;
                }
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            Reserve(startIndex+count,lr-count);
            Set(startIndex,pr,lr);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        public void Reverse()
        {
            if (this._size > 1)
                fixed (ushort* p=this._items)
                    Sort.Reverse(p, this._size);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void Reverse(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (ushort* p=this._items)
                    Sort.Reverse(p + startIndex, count);
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de un elemento. A diferencia de la funcin Insert,
        /// no efecta ninguna copia o limpieza.
        /// </summary>
        /// <param name="index">Index de la insercin.</param>
        public void Reserve(int index)
        {
            if (index<0 || index>this._size)
                throw new ArgumentOutOfRangeException("index",index,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (index==this._size)
                this.EnsureAppend();
            else
            {
                if (this._size==this._cap)
                {
                    ushort[] array2=newArraySum(1);
                    fixed (ushort* po=this._items, pd=array2)
                    {
                        if (index!=0)
                            General.Copy(po, pd, index);
                        General.Copy(po + index, pd + (index + 1), this._size - index);
                    }
                    this._items=array2;
                }
                else
                {
                    fixed (ushort* po=this._items)
                        General.Move(po + index, po + (index + 1), this._size - index);
                }
            }
            this._size++;
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de la cantidad de elementos especificada mediante las funciones Set
        /// u otras. A diferencia de la funcin Insert, no efecta ninguna copia o limpieza
        /// sobre el area de insercin, siendo el contenido de esta indeterminado.
        /// </summary>
        /// <param name="startIndex">Index de inicio de insercin.</param>
        /// <param name="count">Cantidad de elementos a reservar.</param>
        public void Reserve(int startIndex, int count)
        {
            if (count!=0)
            {
                if (count==1)
                    this.Reserve(startIndex);
                else if (startIndex<0 || count<0)
                    throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE);
                else if (startIndex==this._size)
                {
                    this.EnsureAppend(count);
                    this._size+=count;
                }
                else if (startIndex>this._size)
                    throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.SUPERIOR_ARRAY);
                else
                {
                    int nue=this._size + count;
                    if (nue > this._cap)
                    {
                        ushort[] array2=newArraySum(count);
                        fixed (ushort* po=this._items, pd=array2)
                        {
                            if (startIndex!=0)
                                General.Copy(po, pd, startIndex);
                            General.Copy(po + startIndex, pd + (startIndex + count), this._size - startIndex);
                        }
                        this._items=array2;
                    }
                    else
                    {
                        fixed (ushort* po=this._items)
                            General.Move(po + startIndex, po + (startIndex + count), this._size - startIndex);
                    }
                    this._size=nue;
                }
            }
        }

        /// <summary>
        /// Setea el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="index">Index en la coleccin donde se setear el elemento.</param>
        public void Set(int index, ushort item)
        {
            this[index]=item;
        }


        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a setear.</param>
        public void Set(int startIndex, ushort item, int count)
        {
            if (count==1)
                this[startIndex]=item;
            else if (item==0)
                SetBlank(startIndex,count);
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (ushort* po=this._items)
                    General.Fill(po + startIndex, count, item);
        }

        /// <summary>
        /// Setea un cero en la ubicacin especificada.
        /// </summary>
        /// <param name="index">Index en la coleccin donde setear.</param>
        public void SetBlank(int index)
        {
            this[index]=0;
        }


        /// <summary>
        /// Setea ceros en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a setear.</param>
        public void SetBlank(int startIndex, int count)
        {
            if (count==1)
                this[startIndex]=0;
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (ushort* po=this._items)
                    General.Clean(po + startIndex, count);
        }

        /// <summary>
        /// Modifica la cantidad de elementos en la coleccin.
        /// </summary>
        /// <param name="count">Nueva cantidad de elementos.</param>
        /// <param name="exact">En caso de que sea necesario aumentar la capacidad, no aplica algoritmo al aumento.</param>
        /// <param name="clean">En caso de aumentar la cantidad, rellena con ceros el nuevo espacio.</param>
        public void SetLength(int count, bool exact, bool clean)
        {
            if (count <=this._cap)
            {
                if (count <=this._size)
                {
                    if (count < 0)
                        throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                }
                else if (clean)
                    fixed (ushort* p=this._items)
                        General.Clean(p + this._size, count - this._size);
            }
            else
            {
                ushort[] array2;
                if (exact)
                {
                    array2=new ushort[count];
                    this._cap=count;
                }
                else
                    array2=newArraySum(count-this._size);

                if (this._size !=0)
                {
                    fixed (ushort* po=this._items, pd=array2)
                        General.Copy(po, pd, this._size);
                }
                this._items=array2;
            }
            this._size=count;
        }

        /// <summary>
        /// Modifica la cantidad de elementos para que coincida con la capacidad actual.
        /// </summary>
        /// <param name="clean">Rellena con ceros el rea excedente.</param>
        public void SetLengthToCapacity(bool clean)
        {
            if (this._size!=this._cap)
            {
                if (clean)
                    fixed(ushort*p=this._items)
                        General.Clean(p+this._size,this._cap-this._size);
                this._size=this._cap;
            }

        }


        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Set(int startIndex, ushort[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (ushort* po=arr)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Set(int startIndex, ushort[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (ushort* po=arr)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, ushort[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (ushort* po=arr)
                    this.Set(startIndex, po + arrIndex, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, ushort* arr, int count)
        {
            if (count==1)
                this[startIndex]=*arr;
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (ushort* p=this._items)
                    General.Copy(arr,p + startIndex, count);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortAscending()
        {
            if (this._size > 1)
                fixed (ushort* p=this._items)
                    Sort.Ascending(p, this._size);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortAscending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (ushort* p=this._items)
                    Sort.Ascending(p + startIndex, count);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortDescending()
        {
            if (this._size > 1)
                fixed (ushort* p=this._items)
                    Sort.Descending(p, this._size);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortDescending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (ushort* p=this._items)
                    Sort.Descending(p + startIndex, count);
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        public void SortRandom()
        {
            if (this._size > 1)
                fixed (ushort* p=this._items)
                    Sort.Randomize(p, this._size);
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortRandom(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (ushort* p=this._items)
                    Sort.Randomize(p + startIndex, count);
        }


        #endregion

        #region Set, Insert y Append para cantidades limitadas de elementos

        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, ushort item0, ushort item1)
        {
            if (startIndex<0 || startIndex+1>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            fixed(ushort*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
            }
        }

        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, ushort item0, ushort item1, ushort item2)
        {
            if (startIndex<0 || startIndex+2>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            fixed(ushort*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
            }
        }


        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, ushort item0, ushort item1, ushort item2, ushort item3)
        {
            if (startIndex<0 || startIndex+3>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            fixed(ushort*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
                p[startIndex+3]=item3;
            }
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, ushort item0, ushort item1)
        {
            this.Reserve(startIndex,2);
            fixed(ushort*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
            }
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, ushort item0, ushort item1, ushort item2)
        {
            this.Reserve(startIndex,3);
            fixed(ushort*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
            }
        }


        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, ushort item0, ushort item1, ushort item2, ushort item3)
        {
            this.Reserve(startIndex,4);
            fixed(ushort*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
                p[startIndex+3]=item3;
            }
        }

        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        public void AppendItems(ushort item0, ushort item1)
        {
            this.EnsureAppend(2);
            fixed(ushort*p=this._items)
            {
                p[this._size]=item0;
                p[this._size+1]=item1;
                this._size+=2;
            }
        }

        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        public void AppendItems(ushort item0, ushort item1, ushort item2)
        {
            this.EnsureAppend(3);
            fixed(ushort*p=this._items)
            {
                p[this._size]=item0;
                p[this._size+1]=item1;
                p[this._size+2]=item2;
                this._size+=3;
            }
        }


        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        public void AppendItems(ushort item0, ushort item1, ushort item2, ushort item3)
        {
            this.EnsureAppend(4);
            fixed(ushort*p=this._items)
            {
                p[this._size]=item0;
                p[this._size+1]=item1;
                p[this._size+2]=item2;
                p[this._size+3]=item3;
                this._size+=4;

            }
        }

        #endregion

        #region Metodos de Bsqueda, reemplazo y comparacin.
  
        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="first">Se asegura de que la ubicacin de retorno corresponda a la primer aparicin del elemento especificado.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// La coleccin debe estar ordenada en forma ascendente.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public int BinarySearch(ushort item, bool first)
        {
            if (this._size==0)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.BinarySearch(p, this._size, item);
                if (first && ret > 0)
                    while (p[ret - 1]==item && --ret !=0);
                return ret;
            }
        }

        /// <summary>
        /// Efecta una comparacin con la coleccin especificada.
        /// </summary>
        /// <param name="col2">Coleccin de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en esta coleccin que en col2.
        /// - Si todos los elementos comparados son iguales y col2 contiene ms elementos que esta coleccin.
        /// Retorna 0:
        /// - Si esta coleccin y col2 representan al mismo objeto.
        /// - Si todos los elementos comparados son iguales siempre y cuando esta coleccin y col2 tengan la misma cantidad de elementos.
        /// Retorna 1:
        /// - Si col2 es null.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en esta coleccin que en col2.
        /// - Si todos los elementos comparados son iguales y col2 contiene menos elementos que esta coleccin.
        /// </returns>
        public int CompareTo(UShorts col2)
        {
            if (Object.ReferenceEquals(this,col2))
                return 0;
            if (col2==null)
                return 1;
            if (col2._size==0)
                return this._size!=0?1:0;
            if (this._size==0)
                return -1;
            int eqComp,count;
            if (col2._size>this._size)
            {
                count=this._size;
                eqComp=-1;
            }
            else if (col2._size!=this._size)
            {
                count=col2._size;
                eqComp=1;
            }
            else
            {
                count=this._size;
                eqComp=0;
            }
            fixed(ushort*p1=this._items, p2=col2._items)
            {
                count=Comp.Compare(p1,p2,count);
                if (count==-1)
                    return eqComp;
                return (p1[count]<p2[count])?-1:1;
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y array2 contiene ms elementos que la coleccin.
        /// Retorna 0: 
        /// - Si todos los elementos comparados son iguales siempre y cuando la coleccin y array2 tengan la misma cantidad de elementos.
        /// Retorna 1:
        /// - Si array2 es null.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y array2 contiene menos elementos que la coleccin.
        /// </returns>
        public int CompareTo(ushort[] array2)
        {
            if (array2==null)
                return 1;
            int count=array2.Length;
            if (count==0)
                return this._size!=0?1:0;
            if (this._size==0)
                return -1;
            int eqComp;
            if (count>this._size)
            {
                count=this._size;
                eqComp=-1;
            }
            else if (count!=this._size)
                eqComp=1;
            else
                eqComp=0;
            fixed(ushort*p1=this._items, p2=array2)
            {
                count=Comp.Compare(p1,p2,count);
                if (count==-1)
                    return eqComp;
                return (p1[count]<p2[count])?-1:1;
            }
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="arrIndex">Inicio de la comparacin en array2.</param>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si arrIndex esta dentro del rango y startIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y el rango comparable es mayor en array2 que en la coleccin.
        /// Retorna 0: 
        /// - Si tanto startIndex como arrIndex estn fuera de rango.
        /// - Si todos los elementos comparados son iguales siempre y cuando la coleccin y array2 tengan el mismo rango comparable.
        /// Retorna 1:
        /// - Si startIndex esta dentro del rango y arrIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y el rango comparable es menor en array2 que en la coleccin.
        /// </returns>
        public int CompareTo(ushort[] array2, int startIndex, int arrIndex)
        {
            if (startIndex<0 || arrIndex<0)
                throw new ArgumentOutOfRangeException("startIndex,arrIndex",Errors.NEGATIVE);
            int eqComp=array2.Length;
            if (arrIndex>=eqComp)
            {
                if (startIndex>=this._size)
                    return 0;
                return 1;
            }
            if (startIndex>=this._size)
                return -1;
            
            int count=this._size-startIndex;
            if (count<(eqComp-=arrIndex))
                eqComp=-1;
            else if(count!=eqComp)
            {
                count=eqComp;
                eqComp=1;
            }
            else
                eqComp=0;
            fixed(ushort*p1=this._items, p2=array2)
            {
                count=Comp.Compare(p1+startIndex,p2+arrIndex,count);
                if (count==-1)
                    return eqComp;
                return (p1[startIndex+count]<p2[arrIndex+count])?-1:1;
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="arrIndex">Inicio de la comparacin en array2.</param>
        /// <param name="count">Mximo de elementos a comparar.</param>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si count es mayor a cero, arrIndex esta dentro del rango y startIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales siempre y cuando count haya sido reducido total o en mayor medida para ajustarse a la cantidad de elementos de la coleccin.
        /// Retorna 0: 
        /// - Si count es cero.
        /// - Si tanto startIndex como arrIndex estn fuera de rango.
        /// - Si todos los elementos comparados son iguales siempre y cuando count no haya sido reducido o que la reduccin se deba a un mismo excedente en la coleccin y array2.
        /// Retorna 1:
        /// - Si count es mayor a cero, startIndex esta dentro del rango y arrIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales siempre y cuando count haya sido reducido total o en mayor medida para ajustarse al largo de array2.
        /// </returns>
        public int CompareTo(ushort[] array2, int startIndex, int arrIndex, int count)
        {
            if (startIndex<0 || arrIndex<0)
                throw new ArgumentOutOfRangeException("startIndex,arrIndex",Errors.NEGATIVE);
            if (count<1)
            {
                if (count==0)
                    return 0;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            int eqComp=array2.Length;
            if (arrIndex>=eqComp)
            {
                if (startIndex>=this._size)
                    return 0;
                return 1;
            }
            if (startIndex>=this._size)
                return -1;
            eqComp-=arrIndex;
            int rest1=this._size-startIndex;
            if (rest1<count)
            {
                if (eqComp<count)
                {
                    if (rest1<eqComp)
                    {
                        count=rest1;
                        eqComp=-1;
                    }
                    else if (rest1!=eqComp)
                    {
                        count=eqComp;
                        eqComp=1;
                    }
                    else
                    {
                        count=rest1;
                        eqComp=0;
                    }
                }
                else
                {
                    count=rest1;
                    eqComp=-1;
                }
            }
            else if (eqComp<count)
            {
                count=eqComp;
                eqComp=1;
            }
            else
                eqComp=0;
            fixed(ushort*p1=this._items, p2=array2)
            {
                count=Comp.Compare(p1+startIndex,p2+arrIndex,count);
                if (count==-1)
                    return eqComp;
                return (p1[startIndex+count]<p2[arrIndex+count])?-1:1;
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="count">Mximo de elementos a comparar.</param>
        /// <param name="p2">Puntero de comparacin. Debe contener a count.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si count es mayor a cero y startIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en p2.
        /// - Si todos los elementos comparados son iguales siempre y cuando count haya sido reducido para ajustarse a la cantidad de elementos de la coleccin.
        /// Retorna 0: 
        /// - Si count es cero.
        /// - Si todos los elementos comparados son iguales siempre y cuando count no haya sido reducido.
        /// Retorna 1:
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en p2.
        /// </returns>
        public int CompareTo(ushort* p2, int startIndex, int count)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            if (count<1)
            {
                if (count==0)
                    return 0;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (startIndex>=this._size)
                return -1;
            int eqComp=this._size-startIndex;
            if (eqComp<count)
            {
                count=eqComp;
                eqComp=-1;
            }
            else
                eqComp=0;
            fixed(ushort*p1=this._items)
            {
                count=Comp.Compare(p1+startIndex,p2,count);
                if (count==-1)
                    return eqComp;
                return (p1[startIndex+count]<p2[count])?-1:1;
            }
        }

        /// <summary>
        /// Retorna true si contiene el elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(ushort item,bool last)
        {
            if (this._size==0)
                return false;
            fixed (ushort* p=this._items)
            {
                if (!last)
                    return Comp.Exist(p, this._size, item);
                return Comp.LastIndexOf(p,this._size,item)!=-1;
            }
        }



        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(ushort item)
        {
            if (this._size !=0)
            {
                fixed (ushort* p=this._items)
                    return Comp.Count(p, this._size, item);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(ushort item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (ushort* p=this._items)
                return Comp.Count(p + startIndex, this._size - startIndex, item);
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(ushort item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (ushort* p=this._items)
                return Comp.Count(p + startIndex, count, item);
        }


        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(ushort[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (ushort* p=this._items, pf=items)
                    return Comp.CountAny(p, this._size, pf, lf, false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, ushort[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (ushort* p=this._items, pf=items)
                return Comp.CountAny(p + startIndex, this._size - startIndex, pf, lf, false);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, int count, ushort[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (ushort* p=this._items, pf=items)
                return Comp.CountAny(p + startIndex, count, pf, lf, false);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(ushort* pf, int lf)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (ushort* p=this._items)
                    return Comp.CountAny(p, this._size, pf, lf, false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, ushort* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (ushort* p=this._items)
                return Comp.CountAny(p + startIndex, this._size - startIndex, pf, lf, false);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, int count, ushort* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (ushort* p=this._items)
                return Comp.CountAny(p + startIndex, count, pf, lf, false);
        }


        /// <summary>
        /// Efecta una comparacin con la coleccin especificada. 
        /// </summary>
        /// <param name="col2">Coleccin a comparar.</param>
        /// <returns>Retorna true si ambas coleccin contienen la misma cantidad de elementos y su contenido es igual.</returns>
        public bool Equals(UShorts col2)
        {
            if (Object.ReferenceEquals(this,col2))
                return true;
            if (col2==null || this._size!=col2._size)
                return false;
            if (this._size<2)
            {
                if (this._size==0 || this._items[0]==col2._items[0])
                    return true;
                return false;
            }
            fixed(ushort*p=this._items, p2=col2._items)
                return Comp.Equal(p,p2,this._size);
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="array2">Array a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de array2 son iguales a los la coleccin (comparando desde el inicio) o array2 tiene largo 0.</returns>
        public bool Equals(ushort[] array2)
        {
            int count=array2.Length;
            if (count<2)
            {
                if (count==0 || (this._size!=0 && array2[0]==this._items[0]))
                    return true;
                return false;
            }
            if (count>this._size)
                return false;
            fixed(ushort*p=this._items, p2=array2)
                return Comp.Equal(p,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="array2">Array a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <returns>Solo retorna true si todos los elementos de array2 son iguales a los la coleccin (comparando desde startIndex) o array2 tiene largo 0.</returns>
        public bool Equals(ushort[] array2, int startIndex)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            int count=array2.Length;
            if (count<2)
            {
                if (count==0 || (this._size>startIndex && array2[0]==this._items[startIndex]))
                    return true;
                return false;
            }
            if (startIndex+count>this._size)
                return false;
            fixed(ushort*p=this._items, p2=array2)
                return Comp.Equal(p+startIndex,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="p2">Array a comparar.</param>
        /// <param name="count">Cantidad de elementos a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de p2 son iguales a los la coleccin (comparando desde el inicio) o count es 0.</returns>
        public bool Equals(ushort* p2, int count)
        {
            if (count<2)
            {
                if (count==1)
                {
                    if (this._size!=0 && this._items[0]==*p2)
                        return true;
                    return false;
                }
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (count>this._size)
                return false;
            fixed(ushort*p=this._items)
                return Comp.Equal(p,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="p2">Array a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="count">Cantidad de elementos a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de p2 son iguales a los la coleccin (comparando desde el startIndex) o count es 0.</returns>
        public bool Equals(ushort* p2, int startIndex, int count)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            if (count<2)
            {
                if (count==1)
                {
                    if (this._size>startIndex && this._items[startIndex]==*p2)
                        return true;
                    return false;
                }
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (startIndex+count>this._size)
                return false;
            fixed(ushort*p=this._items)
                return Comp.Equal(p+startIndex,p2,count);
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(ushort item)
        {
            if (this._size !=0)
            {
                fixed (ushort* p=this._items)
                    return Comp.IndexOf(p, this._size, item);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(ushort item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            int ret;
            fixed (ushort* p=this._items)
                ret=Comp.IndexOf(p + startIndex, this._size - startIndex, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(ushort item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            int ret;
            fixed (ushort* p=this._items)
                ret=Comp.IndexOf(p + startIndex, count, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }


        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOfAny(ushort[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (ushort* p=this._items, pf=items)
                    return Comp.IndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, ushort[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            int ret;
            fixed (ushort* p=this._items, pf=items)
                ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, int count, ushort[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            int ret;
            fixed (ushort* p=this._items, pf=items)
                ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOfAny(ushort* pf, int lf)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (ushort* p=this._items)
                    return Comp.IndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, ushort* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            int ret;
            fixed (ushort* p=this._items)
                ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, int count, ushort* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            int ret;
            fixed (ushort* p=this._items)
                ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(ushort item)
        {
            if (this._size !=0)
            {
                fixed (ushort* p=this._items)
                    return Comp.LastIndexOf(p, this._size, item);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(ushort item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            int ret;
            fixed (ushort* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, this._size - startIndex, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(ushort item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            int ret;
            fixed (ushort* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, count, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOfAny(ushort[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (ushort* p=this._items, pf=items)
                    return Comp.LastIndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, ushort[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            int ret;
            fixed (ushort* p=this._items, pf=items)
                ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, int count, ushort[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            int ret;
            fixed (ushort* p=this._items, pf=items)
                ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOfAny(ushort* pf, int lf)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (ushort* p=this._items)
                    return Comp.LastIndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, ushort* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            int ret;
            fixed (ushort* p=this._items)
                ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, int count, ushort* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            int ret;
            fixed (ushort* p=this._items)
                ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(ushort item)
        {
            if (this._size==0)
                return 0;

            fixed (ushort* p=this._items)
            {
                return Comp.Remove(false, p, p, this._size, out this._size, item, false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(ushort item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (ushort* p=this._items)
            {
                int lFinal;
                ushort* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, this._size - startIndex, out lFinal, item, false);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(ushort item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (ushort* p=this._items)
            {
                int lFinal;
                ushort* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, count, out lFinal, item, false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(ushort[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return 0;
            fixed (ushort* p=this._items, pf=items)
            {
                return Comp.RemoveAny(false, p, p, this._size, out this._size, pf, lf, false, false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, ushort[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (ushort* p=this._items, pf=items)
            {
                int lFinal;
                ushort* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, this._size - startIndex, out lFinal, pf, lf, false, false);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, int count, ushort[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (ushort* p=this._items, pf=items)
            {
                int lFinal;
                ushort* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, count, out lFinal, pf, lf, false, false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(ushort* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return 0;

            fixed (ushort* p=this._items)
            {
                return Comp.RemoveAny(false, p, p, this._size, out this._size, pf, lf, false, false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, ushort* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (ushort* p=this._items)
            {
                int lFinal;
                ushort* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, this._size - startIndex, out lFinal, pf, lf, false, false);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, int count, ushort* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (ushort* p=this._items)
            {
                int lFinal;
                ushort* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, count, out lFinal, pf, lf, false, false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }


        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(ushort item)
        {
            if (this._size==0)
                return -1;

            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, item);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(ushort item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(ushort item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(ushort[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;

            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, ushort[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, int count, ushort[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(ushort* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, ushort* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, int count, ushort* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(ushort item)
        {
            if (this._size==0)
                return -1;

            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, item);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(ushort item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(ushort item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOfAny(ushort[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;

            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, ushort[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, int count, ushort[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOfAny(ushort* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, ushort* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, int count, ushort* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(ushort item, ushort rep)
        {
            if (this._size==0)
                return 0;

            fixed (ushort* p=this._items)
            {
                return Comp.Replace(p, this._size, item, rep);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(ushort item, ushort rep, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (ushort* p=this._items)
            {
                return Comp.Replace(p + startIndex, this._size - startIndex, item, rep);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(ushort item, ushort rep, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (ushort* p=this._items)
            {
                return Comp.Replace(p + startIndex, count, item, rep);
            }
        }


        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(ushort rep, ushort[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return 0;
            fixed (ushort* p=this._items, pf=items)
            {
                return Comp.ReplaceAny(p, this._size, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, ushort rep, ushort[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (ushort* p=this._items, pf=items)
            {
                return Comp.ReplaceAny(p + startIndex, this._size - startIndex, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, int count, ushort rep, ushort[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (ushort* p=this._items, pf=items)
            {

                return Comp.ReplaceAny(p + startIndex, count, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(ushort rep, ushort* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return 0;

            fixed (ushort* p=this._items)
            {
                return Comp.ReplaceAny(p, this._size, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, ushort rep, ushort* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (ushort* p=this._items)
            {
                return Comp.ReplaceAny(p + startIndex, this._size - startIndex, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, int count, ushort rep, ushort* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (ushort* p=this._items)
            {
                return Comp.ReplaceAny(p + startIndex, count, pf, lf, rep, false);
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(ushort item, ushort rep)
        {
            if (this._size==0)
                return -1;

            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, item);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(ushort item, ushort rep, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(ushort item, ushort rep, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(ushort rep, ushort[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;

            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, ushort rep, ushort[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, int count, ushort rep, ushort[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(ushort rep, ushort* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, ushort rep, ushort* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, int count, ushort rep, ushort* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOf(ushort item, ushort rep)
        {
            if (this._size==0)
                return -1;

            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, item);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(ushort item, ushort rep, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(ushort item, ushort rep, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(ushort rep, ushort[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;

            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, ushort rep, ushort[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, int count, ushort rep, ushort[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (ushort* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(ushort rep, ushort* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, ushort rep, ushort* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, int count, ushort rep, ushort* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (ushort* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }


        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los del array de comparacin.
        /// </summary>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los del array de comparacin.</returns>
        public bool EndsWith(ushort[] array2)
        {

            int count=array2.Length;
            if (count<2)
            {
                if (count==0 || (this._size!=0 && array2[0]==this._items[this._size-1]))
                    return true;
                return false;
            }
            if (count>this._size)
                return false;
            fixed(ushort*p=this._items, p2=array2)
                return Comp.Equal(p+(this._size-count),p2,count);
        }

        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los del puntero de comparacin.
        /// </summary>
        /// <param name="p2">Puntero de comparacin.</param>
        /// <param name="count">Largo del puntero de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los del puntero de comparacin.</returns>
        public bool EndsWith(ushort* p2, int count)
        {
            if (count<2)
            {
                if (count==1)
                {
                    if (this._size!=0 && this._items[this._size-1]==*p2)
                        return true;
                    return false;
                }
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (count>this._size)
                return false;
            fixed(ushort*p=this._items)
                return Comp.Equal(p+(this._size-count),p2,count);
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los del array de comparacin.
        /// </summary>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los del array de comparacin.</returns>
        public bool StartsWith(ushort[] array2)
        {
            return Equals(array2);
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los del puntero de comparacin.
        /// </summary>
        /// <param name="p2">Puntero de comparacin.</param>
        /// <param name="count">Largo del puntero de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los del puntero de comparacin.</returns>
        public bool StartsWith(ushort* p2, int count)
        {
            return Equals(p2,count);
        }

        #endregion

        #region Funciones con Streams NO Bytes NO Chars

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream en forma directa y sin decodificar.
        /// Los elementos son insertados al final de la coleccin.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="maxCount">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int Append(System.IO.Stream stream, int maxCount, bool close)
        {
            const int sof=sizeof(ushort);
            Serializer.CalcStreamReadItems(stream,ref maxCount,sof);
            if (maxCount!=0)
            {
                this.EnsureAppend(maxCount);
                fixed(ushort*p=this._items)
                    maxCount=Serializer.StreamReadItems(stream,close,p+this._size,maxCount,sof,null,0);
                this._size+=maxCount;
            }
            else if (close)
                stream.Close();
            return maxCount;
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="maxCount">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int Insert(int startIndex, System.IO.Stream stream, int maxCount, bool close)
        {
            if (startIndex==this._size)
                return Append(stream,maxCount,close);
            if (startIndex<0 || startIndex>this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            const int sof=sizeof(ushort);
            Serializer.CalcStreamReadItems(stream, ref maxCount,sof);
            if (maxCount!=0)
            {
                this.Reserve(startIndex,maxCount);
                fixed(ushort*p=this._items)
                {
                    int ret=Serializer.StreamReadItems(stream,close,p+startIndex,maxCount,sof,null,0);
                    if (ret!=maxCount)
                        this.RemoveRange(startIndex+ret,maxCount-ret);
                    return ret;
                }
            }
            else if (close)
                stream.Close();
            return 0;
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada eliminando todos los elementos
        /// existentes a partir de esta ubicacin.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="maxLR">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int ReplaceRange( int startIndex, System.IO.Stream stream, int maxLR, bool close)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
            return this.Append(stream,maxLR,close);
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream en forma directa y sin decodificar.
        /// Los elementos copiados reemplazarn al rea comprendida entre startIndex y count.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="count">Area a reemplazar con el contenido del Stream.</param>
        /// <param name="maxLR">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int ReplaceRange( int startIndex, int count, System.IO.Stream stream, int maxLR, bool close)
        {
            if (count<1)
            {
                if (count==0)
                    return Insert(startIndex,stream,maxLR,close);
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            if (startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE);
            
            int l=startIndex+count;
            if (l>=this._size)
            {
                if (l==this._size)
                    return ReplaceRange(startIndex,stream,maxLR,close);
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.SUPERIOR_ARRAY);
            }
            
            const int sof=sizeof(ushort);
            Serializer.CalcStreamReadItems(stream, ref maxLR,sof);
            if (maxLR<=count)
            {
                if (maxLR!=0)
                {
                    fixed(ushort*p=this._items)
                        maxLR=Serializer.StreamReadItems(stream,close,p+startIndex,maxLR,sof,null,0);
                }
                else if (close)
                    stream.Close();
                if (maxLR<count)
                    this.RemoveRange(startIndex+maxLR,count-maxLR);
                return maxLR;
            }
            this.Reserve(startIndex+count,maxLR-count);
            fixed(ushort*p=this._items)
                l=Serializer.StreamReadItems(stream,close,p+startIndex,maxLR,sof,null,0);
            if (l!=maxLR)
                this.RemoveRange(startIndex+l,maxLR-l);
            return l;
        }

 
        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de un Stream en forma directa y sin codificar.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="close">Cierra el Stream luego de la escritura.</param>
        /// <returns>Bytes escritos.</returns>
        public int StreamWrite(System.IO.Stream stream,bool close)
        {
            if (this._size!=0)
            {
                fixed(ushort*p=this._items)
                    return Serializer.StreamWrite(stream,close,(byte*)p,this._size*sizeof(ushort),null,0);
            }
            else if (close)
                stream.Close();
            return 0;
        }

        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de un Stream en forma directa y sin codificar.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">Indice de comienzo de la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="close">Cierra el Stream luego de la escritura.</param>
        /// <returns>Bytes escritos.</returns>
        public int StreamWrite(System.IO.Stream stream, int startIndex, int count,bool close)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                fixed(ushort*p=this._items)
                    return Serializer.StreamWrite(stream,close,(byte*)(p+startIndex),count*sizeof(ushort),null,0);
            }
            else if (close)
                stream.Close();
            return 0;
        }

        #endregion

        #region Funciones con Arrays de Bytes  NO Bytes NO Chars

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos son insertados al final de la coleccin.
        /// </summary>
        /// <param name="bytes">Array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Append(byte[] bytes)
        {
            int bytesCount=bytes.Length;
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return Append(p,bytesCount);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos son insertados al final de la coleccin.
        /// </summary>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesIndex">Index en el array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Append(byte[] bytes, int bytesIndex, int bytesCount)
        {
            if (bytesIndex<0 || bytesCount<0 || bytesIndex+bytesCount>bytes.Length)
                throw new ArgumentOutOfRangeException("bytesIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return Append(p+bytesIndex,bytesCount);
            return 0;
        }


        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Insert(int startIndex, byte[] bytes)
        {
            int bytesCount=bytes.Length;
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return Insert(startIndex,p,bytesCount);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesIndex">Index en el array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Insert(int startIndex, byte[] bytes, int bytesIndex, int bytesCount)
        {
            if (bytesIndex<0 || bytesCount<0 || bytesIndex+bytesCount>bytes.Length)
                throw new ArgumentOutOfRangeException("bytesIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return Insert(startIndex,p+bytesIndex,bytesCount);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada eliminando todos los elementos
        /// existentes a partir de esta ubicacin.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange(int startIndex, byte[] bytes)
        {
            int bytesCount=bytes.Length;
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return ReplaceRange(startIndex,p,bytesCount);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada eliminando todos los elementos
        /// existentes a partir de esta ubicacin.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesIndex">Index en el array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange( int startIndex, byte[] bytes, int bytesIndex, int bytesCount)
        {
            if (bytesIndex<0 || bytesCount<0 || bytesIndex+bytesCount>bytes.Length)
                throw new ArgumentOutOfRangeException("bytesIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return ReplaceRange(startIndex,p+bytesIndex,bytesCount);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos copiados reemplazarn al rea comprendida entre startIndex y count.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="count">Area a reemplazar con el contenido del Array.</param>
        /// <param name="bytes">Array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange( int startIndex, int count, byte[] bytes)
        {
            int bytesCount=bytes.Length;
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return ReplaceRange(startIndex,count,p,bytesCount);
            return 0;
        }
      

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos copiados reemplazarn al rea comprendida entre startIndex y count.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="count">Area a reemplazar con el contenido del Array.</param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesIndex">Index en el array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange(int startIndex, int count, byte[] bytes, int bytesIndex, int bytesCount)
        {
            if (bytesIndex<0 || bytesCount<0 || bytesIndex+bytesCount>bytes.Length)
                throw new ArgumentOutOfRangeException("bytesIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return ReplaceRange(startIndex,count,p+bytesIndex,bytesCount);
            return 0;
        }

        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de una coleccin de bytes en forma directa y sin codificar.
        /// </summary>
        /// <returns>Bytes escritos.</returns>
        public int BytesWrite(Bytes col)
        {
            if (this._size!=0)
            {
                int count=this._size*sizeof(ushort);
                fixed(ushort*p=this._items)
                    col.Append((byte*)p,count);
                return count;
            }
            return 0;
        }

        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de una coleccin de bytes en forma directa y sin codificar.
        /// </summary>
        /// <param name="startIndex">Indice de comienzo de la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="col">Coleccin destino.</param>
        /// <returns>Bytes escritos.</returns>
        public int BytesWrite(Bytes col, int startIndex, int count)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                count*=sizeof(ushort);
                fixed(ushort*p=this._items)
                    col.Append((byte*)(p+startIndex),count);
            }
            return count;
        }

        #endregion

        #region Funciones con Punteros de Bytes  NO Bytes NO Chars

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos son insertados al final de la coleccin.
        /// </summary>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Append(byte*bytes, int bytesCount)
        {
            bytesCount/=sizeof(ushort);
            if (bytesCount!=0)
                this.Append((ushort*)bytes,bytesCount);
            return bytesCount;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Insert(int startIndex, byte*bytes, int bytesCount)
        {
            bytesCount/=sizeof(ushort);
            if (bytesCount!=0)
                this.Insert(startIndex,(ushort*)bytes,bytesCount);
            return bytesCount;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada eliminando todos los elementos
        /// existentes a partir de esta ubicacin.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange(int startIndex, byte*bytes, int bytesCount)
        {
            bytesCount/=sizeof(ushort);
            if (bytesCount!=0)
                this.ReplaceRange(startIndex,(ushort*)bytes,bytesCount);
            return bytesCount;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes en forma directa y sin decodificar.
        /// Los elementos copiados reemplazarn al rea comprendida entre startIndex y count.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="count">Area a reemplazar con el contenido del Array.</param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange(int startIndex, int count, byte*bytes, int bytesCount)
        {
            bytesCount/=sizeof(ushort);
            if (bytesCount!=0)
                this.ReplaceRange(startIndex,count,(ushort*)bytes,bytesCount);
            return bytesCount;
        }

        #endregion


    }
}
