using System;
using EnPalermo.Lit;
using EnPalermo.Txt;
using EnPalermo.Mem;

namespace EnPalermo.Int
{
	/// <summary>
	/// Representa una IP v4.
	/// </summary>
    public struct IP:IComparable<IP>,IEquatable<IP>
	{
		#region Campos

        /// <summary>
        /// IP cero.
        /// </summary>
        public static readonly IP Zero=new IP();


		/// <summary>
		/// Nmero UINT que representa la IP.
		/// </summary>
		public uint IPNumber;

		#endregion

		#region Constructores
		/// <summary>
		/// Inicializa la IP en base a una IPAddress.
		/// </summary>
		/// <param name="ip">IPAddress.</param>
		public IP(System.Net.IPAddress ip)
		{
			if (ip.AddressFamily==System.Net.Sockets.AddressFamily.InterNetwork)
            {
// # pragma warning disable 618
                IPNumber =Numerics.Invert((uint)ip.Address);
// # pragma warning restore 618
            }
			else IPNumber=0;
		}

		/// <summary>
		/// Inicializa la IP en base a un numero.
		/// </summary>
		/// <param name="n">Numero (debe estar en el rango uint).</param>
		public IP(long n)
		{
			if (n>0 && n<=UInt32.MaxValue)
			{
				IPNumber=(uint)n;
			}
			else IPNumber=0;
		}

		/// <summary>
		/// Inicializa la IP en base a un numero.
		/// </summary>
		/// <param name="n">Numero.</param>
		public IP(int n)
		{
			IPNumber=(uint)n;
		}

		/// <summary>
		/// Inicializa la IP en base a un numero.
		/// </summary>
		/// <param name="n">Numero.</param>
		public IP(uint n)
		{
			IPNumber=n;
		}

		/// <summary>
		/// Inicializa la IP en base a una cadena.
		/// </summary>
		/// <param name="strict">
		/// Si true: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// Si false: Recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// </param>
		/// <param name="str">Cadena.</param>
		public IP(string str, bool strict)
		{
			int l=str==null?0:str.Length;
			if (l>6)
			{
				unsafe
				{
					fixed(char*p=str) IPNumber=recognize(p,l,strict);
				}
			}
			else IPNumber=0;
		}

		/// <summary>
		/// Inicializa la IP en base a una cadena.
		/// </summary>
		/// <param name="str">Cadena.</param>
		/// <param name="i">Inicio de la zona de reconocimiento. Mayor al rango no genera error, simplemente genera una ip invlida.</param>
		/// <param name="l">Largo de la zona de reconocimiento. -1 o mayor al rango implica reconocer sobre el resto de la cadena.</param>
		/// <param name="strict">
		/// Si true: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// Si false: Recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// </param>
		public IP(string str, int i, int l,bool strict)
		{
			int lc=str.Length;
            if (i<0)
                throw new ArgumentOutOfRangeException("i",i,Errors.NEGATIVE);
            if (l<7)
            {
                if (l==-1)
                {
                    l=lc-i;
                    if (l>6)
                        goto siga;
                }
                else if (l<0)
                    throw new ArgumentOutOfRangeException("l",l,Errors.MINOR_ONE_NEGATIVE);
                IPNumber=0;
                return;
            }
            if (i+l>lc)
            {
                l=lc-i;
                if (l<7)
                {
                    IPNumber=0;
                    return;
                }
            }

            siga:

                unsafe
                {
                    fixed(char*p=str) 
                        IPNumber=recognize(p+i,l,strict);
                    return;
                }

		}

		/// <summary>
		/// Inicializa la IP en base a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		/// <param name="strict">
		/// Si true: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// Si false: Recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// </param>
		public unsafe IP(char*p, int l,bool strict)
		{
			IPNumber=recognize(p,l,strict);
		}

		/// <summary>
		/// Inicializa la IP en base a un puntero de bytes Ascii o Latin1.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		/// <param name="strict">
		/// Si true: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// Si false: Recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// </param>
		public unsafe IP(byte*p, int l,bool strict)
		{
			IPNumber=recognize(p,l,strict);
		}
		#endregion

		#region Propiedades

		/// <summary>
		/// Indica que la IP es vlida (aunque puede ser de tipo local).
		/// </summary>
		public bool IsValid
		{
			get
			{
				return IPNumber!=0 && IPNumber!=UInt32.MaxValue;
			}
		}

		/// <summary>
		/// Indica que la IP es vlida y fuera de los rangos considerados como IP locales.
		/// </summary>
		public bool IsReal
		{
			get
			{
				if  (IPNumber < 16777216U || IPNumber > 4294901759U || 
					(IPNumber > 167772159U && IPNumber < 184549376U) ||
					(IPNumber > 3232235519U && IPNumber < 3232301056U) ||
					(IPNumber > 2130706431U && IPNumber < 2130706688U) ||
					(IPNumber > 2851995647U && IPNumber < 2852126720U) ||
					(IPNumber > 2886729727U && IPNumber < 2887778304U)) return false;
				return true;
			}
		}
		
		/// <summary>
		/// Retorna Array de Bytes.
		/// </summary>
		public unsafe byte[] Bytes
		{
			get
			{
				return Numerics.BytesInv(this.IPNumber);
			}
		}

		/// <summary>
		/// Retorna objeto IPAddress.
		/// </summary>
		public System.Net.IPAddress IPAddress
		{
			get
			{
				return new System.Net.IPAddress(Numerics.Invert(IPNumber));
			}
		}

		#endregion

		#region Metodos de Object


		/// <summary>
		/// Retorna la cadena correspondiente a esta IP.
		/// </summary>
		/// <returns>Cadena correspondiente a esta IP.</returns>
		public override string ToString()
		{
            unsafe
            {
                char*pc=stackalloc char[15];
                return Mutables.Create(pc,FillPointer(pc));
            }
	
		}

		/// <summary>
		/// True si son iguales.
		/// </summary>
		/// <param name="obj">Objeto.</param>
		/// <returns>True si son iguales.</returns>
		public override bool Equals(object obj)
		{
			return obj is IP && ((IP)obj).IPNumber==this.IPNumber;
		}

		/// <summary>
		/// True si son iguales.
		/// </summary>
		/// <param name="b">IP.</param>
		/// <returns>True si son iguales.</returns>
		public bool Equals(IP b)
		{
			return b.IPNumber==this.IPNumber;
		}

		/// <summary>
		/// Obtiene Hash.
		/// </summary>
		/// <returns>Cdigo Hash.</returns>
		public override int GetHashCode()
		{
			return (int)this.IPNumber;
		}

		#endregion

		#region Operadores

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IP a.</param>
		/// <param name="b">IP b.</param>
		/// <returns>True si son iguales.</returns>
		public static bool operator ==(IP a, IP b)
		{
			return a.IPNumber==b.IPNumber;
		}
		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IP a.</param>
		/// <param name="b">IP b.</param>
		/// <returns>True si son diferentes.</returns>
		public static bool operator !=(IP a, IP b)
		{
			return a.IPNumber!=b.IPNumber;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IP a.</param>
		/// <param name="b">IP b.</param>
		/// <returns>True si A es mayor a B.</returns>
		public static bool operator >(IP a, IP b)
		{
			return a.IPNumber>b.IPNumber;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IP a.</param>
		/// <param name="b">IP b.</param>
		/// <returns>True si A es menor a B.</returns>
		public static bool operator <(IP a, IP b)
		{
			return a.IPNumber<b.IPNumber;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IP a.</param>
		/// <param name="b">IP b.</param>
		/// <returns>True si A es mayor o igual a B.</returns>
		public static bool operator >=(IP a, IP b)
		{
			return a.IPNumber>=b.IPNumber;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IP a.</param>
		/// <param name="b">IP b.</param>
		/// <returns>True si A es menor o igual a B.</returns>
		public static bool operator <=(IP a, IP b)
		{
			return a.IPNumber<=b.IPNumber;
		}

		/// <summary>
		/// Conversin a IP.
		/// </summary>
		/// <param name="obj">IPAddress.</param>
		/// <returns>IP.</returns>
		public static implicit operator IP(System.Net.IPAddress obj){return new IP(obj);}
		/// <summary>
		/// Conversin a IP (estricta).
		/// </summary>
		/// <param name="obj">Cadena.</param>
		/// <returns>IP.</returns>
		public static implicit operator IP(string obj){return new IP(obj,true);}
		/// <summary>
		/// Conversin a IP.
		/// </summary>
		/// <param name="obj">Nmero IP.</param>
		/// <returns>IP.</returns>
		public static implicit operator IP(uint obj){return new IP(obj);}
		/// <summary>
		/// Conversin a IP.
		/// </summary>
		/// <param name="obj">Nmero IP.</param>
		/// <returns>IP.</returns>
		public static implicit operator IP(long obj){return new IP(obj);}
		/// <summary>
		/// Conversin a String.
		/// </summary>
		/// <param name="ip">IP.</param>
		/// <returns>String.</returns>
		public static implicit operator string(IP ip){return ip.ToString();}
		/// <summary>
		/// Conversin a Nmero.
		/// </summary>
		/// <param name="ip">IP.</param>
		/// <returns>Nmero.</returns>
		public static explicit operator uint(IP ip){return ip.IPNumber;}
		/// <summary>
		/// Conversin a Nmero.
		/// </summary>
		/// <param name="ip">IP.</param>
		/// <returns>Nmero.</returns>
		public static explicit operator long(IP ip){return ip.IPNumber;}
		/// <summary>
		/// Conversin a IPAddress.
		/// </summary>
		/// <param name="ip">IP.</param>
		/// <returns>IPAddress.</returns>
		public static implicit operator System.Net.IPAddress(IP ip)	{return ip.IPAddress;}
        
		#endregion

		#region Miembros de IComparable

		/// <summary>
		/// Implementacin de CompareTo.
		/// </summary>
		/// <param name="b">IP.</param>
		/// <returns>0 si son iguales.</returns>
		public int CompareTo(IP b)
		{
			if (b.IPNumber>this.IPNumber) return -1;
			if (b.IPNumber<this.IPNumber) return 1;
			return 0;
		}

		#endregion

		#region Metodos

        /// <summary>
        /// Guarda en un puntero la cadena correspondiente a esta IP.
        /// </summary>
        /// <param name="pc">Cadena a rellenar.</param>
        /// <returns>Carcteres utilizados en la cadena de relleno.</returns>
        public unsafe int FillPointer(char*pc)
        {
            fixed(uint* pint=&IPNumber)
            {
                byte*p=(byte*)pint+3;
                int a=0;
                int n;
                do
                {
                    n=*p;
                    switch(n/100)
                    {
                        case 1:
                            pc[a++]='1';
                            n-=100;
                            break;
                        case 2:
                            pc[a++]='2';
                            n-=200;
                            break;
                    }
								
                    switch(n/10)
                    {
                        case 0:
                            if (*p>99) pc[a++]='0';
                            break;
                        case 1:
                            pc[a++]='1';
                            n-=10;
                            break;
                        case 2:
                            pc[a++]='2';
                            n-=20;
                            break;
                        case 3:
                            pc[a++]='3';
                            n-=30;
                            break;
                        case 4:
                            pc[a++]='4';
                            n-=40;
                            break;
                        case 5:
                            pc[a++]='5';
                            n-=50;
                            break;
                        case 6:
                            pc[a++]='6';
                            n-=60;
                            break;
                        case 7:
                            pc[a++]='7';
                            n-=70;
                            break;
                        case 8:
                            pc[a++]='8';
                            n-=80;
                            break;
                        case 9:
                            pc[a++]='9';
                            n-=90;
                            break;
                    }

                    switch(n)
                    {
                        case 0:
                            pc[a++]='0';
                            break;
                        case 1:
                            pc[a++]='1';
                            break;
                        case 2:
                            pc[a++]='2';
                            break;
                        case 3:
                            pc[a++]='3';
                            break;
                        case 4:
                            pc[a++]='4';
                            break;
                        case 5:
                            pc[a++]='5';
                            break;
                        case 6:
                            pc[a++]='6';
                            break;
                        case 7:
                            pc[a++]='7';
                            break;
                        case 8:
                            pc[a++]='8';
                            break;
                        case 9:
                            pc[a++]='9';
                            break;
                    }

                    if (pint!=p)
                    {
                        pc[a++]='.';
                        p--;
                    }
                    else return a; 
                }while(true);
            }
        }

		/// <summary>
		/// Guarda en un puntero la cadena correspondiente a esta IP.
		/// </summary>
		/// <param name="pc">Cadena a rellenar.</param>
		/// <returns>Carcteres utilizados en la cadena de relleno.</returns>
		public unsafe int FillPointer(byte*pc)
		{
			fixed(uint* pint=&IPNumber)
			{
				byte*p=(byte*)pint+3;
				int a=0;
				int n;
				do
				{
					n=*p;
					switch(n/100)
					{
						case 1:
							pc[a++]=49;
							n-=100;
							break;
						case 2:
							pc[a++]=50;
							n-=200;
							break;
					}
								
					switch(n/10)
					{
						case 0:
							if (*p>99) pc[a++]=48;
							break;
						case 1:
							pc[a++]=49;
							n-=10;
							break;
						case 2:
							pc[a++]=50;
							n-=20;
							break;
						case 3:
							pc[a++]=51;
							n-=30;
							break;
						case 4:
							pc[a++]=52;
							n-=40;
							break;
						case 5:
							pc[a++]=53;
							n-=50;
							break;
						case 6:
							pc[a++]=54;
							n-=60;
							break;
						case 7:
							pc[a++]=55;
							n-=70;
							break;
						case 8:
							pc[a++]=56;
							n-=80;
							break;
						case 9:
							pc[a++]=57;
							n-=90;
							break;
					}

					switch(n)
					{
						case 0:
							pc[a++]=48;
							break;
						case 1:
							pc[a++]=49;
							break;
						case 2:
							pc[a++]=50;
							break;
						case 3:
							pc[a++]=51;
							break;
						case 4:
							pc[a++]=52;
							break;
						case 5:
							pc[a++]=53;
							break;
						case 6:
							pc[a++]=54;
							break;
						case 7:
							pc[a++]=55;
							break;
						case 8:
							pc[a++]=56;
							break;
						case 9:
							pc[a++]=57;
							break;
					}

					if (pint!=p)
					{
						pc[a++]=46;
						p--;
					}
					else return a; 
				}while(true);
			}
		}

		#endregion

		#region Rutina de Deteccin

		/// <summary>
		/// Funcin interna de reconocimiento.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		/// <param name="strict">
		/// Si true: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// Si false: Recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// </param>
		/// <returns>IP o 0.</returns>
		private unsafe static uint recognize(char*p,int l,bool strict)
		{
			Mem.PChar pc=new Mem.PChar(p,l,true,true);
			if(pc.Length<7) return 0;
			Mem.PChar pc2;
			uint ip=0;
			byte* pip=(byte*)&ip;
			int zona=3, n;
			if (strict)
			{
				do
				{
					pc2=pc.Foward('.',false);
					if (zona!=0)
					{
						if (pc.Length!=0 && pc2.Length!=0 && pc2.Length<4)
						{
							l=0;
							while(pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0' && ++l!=pc2.Length);
							if (pc2.Length==l)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
								if (n!=-1)
								{
									pip[zona--]=(byte)n;
									continue;
								}
							}
						}
					}
					else if (pc2.Length!=0)
					{
						l=0;
						while(pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0' && ++l!=4 && l!=pc2.Length);
						if (l!=0 && l!=4)
						{
                            n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                            if (n!=-1)
                            {
                                *pip=(byte)n;
                                return ip;
                            }
						}
					}
					return 0;
				}while(true);
			}
			do
			{
				pc2=pc.Foward('.',true);
				switch(zona)
				{
					case 3:
						if (pc.Length<5) return 0;
						l=pc2.Length;
						while(--l!=-1 && pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0');
						if (++l!=pc2.Length)
						{
							if (l!=0)
							{
                                pc2.Pointer+=l;
								pc2.Length-=l;
							}
							if (pc2.Length<13)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,pc2.Length,0,255);
                                if (n!=-1)
                                    pip[zona--]=(byte)n;
                            }
						}
						continue;
					case 2:
					case 1:
						if (pc.Length==0) return 0;
						if (pc2.Length<13)
						{
							l=pc2.Length;
							while(--l!=-1 && pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0');
							if (l==-1)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,pc2.Length,0,255);
                                if (n!=-1)
                                {
                                    pip[zona--]=(byte)n;
                                    continue;
                                }
							} 
							else if (++l!=pc2.Length)
							{
                                pc2.Length-=l;
								pc2.Pointer+=l;
								zona=3;
								ip=0;
								goto case 3;
							}
							if (pc.Length<7) return 0;
							zona=3;
							ip=0;
							continue;
						} 
						zona=3;
						ip=0;
						goto case 3;
					default:
						if (pc2.Length!=0)
						{
							l=0;
							while(pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0' && ++l!=pc2.Length);
							if (l==0)
							{
								zona=3;
								ip=0;
								if (pc2.Length==1) 
								{
									if (pc.Length<7) return 0;
									continue;
								}
								pc2.Length--;
								pc2.Pointer++;
								goto case 3;
							}
							
							if (l<13)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                                if (n!=-1)
                                {
                                    *pip=(byte)n;
                                    return ip;
                                }
							}
							pc2.Pointer+=l;
							pc.Length+=(int)(pc.Pointer-pc2.Pointer);
							if (pc.Length>6)
							{
								pc.Pointer=pc2.Pointer;
								zona=3;
								ip=0;
								continue;
							}
						} 
						return 0;
				}
			}while(true);
		}


		/// <summary>
		/// Funcin interna de reconocimiento.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		/// <param name="strict">
		/// Si true: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// Si false: Recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// </param>
		/// <returns>IP o 0.</returns>
		private unsafe static uint recognize(byte*p,int l,bool strict)
		{
			Mem.PByte pc=new Mem.PByte(p,l,true,true,true);
			if(pc.Length<7) return 0;
			Mem.PByte pc2;
			uint ip=0;
			byte* pip=(byte*)&ip;
			int zona=3,n;
			if (strict)
			{
				do
				{
					pc2=pc.Foward(46,false);
					if (zona!=0)
					{
						if (pc.Length!=0 && pc2.Length!=0 && pc2.Length<4)
						{
							l=0;
							while(pc2.Pointer[l]<58 && pc2.Pointer[l]>47 && ++l!=pc2.Length);
							if (pc2.Length==l)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                                if (n!=-1)
                                {
                                    pip[zona--]=(byte)n;
                                    continue;
                                }
							}
						}
					}
					else if (pc2.Length!=0)
					{
						l=0;
						while(pc2.Pointer[l]<58 && pc2.Pointer[l]>47 && ++l!=4 && l!=pc2.Length);
						if (l!=0 && l!=4)
						{
                            n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                            if (n!=-1)
                            {
                                *pip=(byte)n;
                                return ip;
                            }
						}
					}
					return 0;
				}while(true);
			}
			do
			{
				pc2=pc.Foward(46,true);
				switch(zona)
				{
					case 3:
						if (pc.Length<5) return 0;
						l=pc2.Length;
						while(--l!=-1 && pc2.Pointer[l]<58 && pc2.Pointer[l]>47);
						if (++l!=pc2.Length)
						{
							if (l!=0)
							{
								pc2.Pointer+=l;
								pc2.Length-=l;
							}
							if (pc2.Length<13)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,pc2.Length,0,255);
                                if (n!=-1)
                                    pip[zona--]=(byte)n;
							}
						}
						continue;
					case 2:
					case 1:
						if (pc.Length==0) return 0;
						if (pc2.Length<13)
						{
							l=pc2.Length;
							while(--l!=-1 && pc2.Pointer[l]<58 && pc2.Pointer[l]>47);
							if (l==-1)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,pc2.Length,0,255);
                                if (n!=-1)
                                {
                                    pip[zona--]=(byte)n;
                                    continue;
                                }
							} 
							else if (++l!=pc2.Length)
							{
								pc2.Length-=l;
								pc2.Pointer+=l;
								zona=3;
								ip=0;
								goto case 3;
							}
							if (pc.Length<7) return 0;
							zona=3;
							ip=0;
							continue;
						} 
						zona=3;
						ip=0;
						goto case 3;
					default:
						if (pc2.Length!=0)
						{
							l=0;
							while(pc2.Pointer[l]<58 && pc2.Pointer[l]>47 && ++l!=pc2.Length);
							if (l==0)
							{
								zona=3;
								ip=0;
								if (pc2.Length==1) 
								{
									if (pc.Length<7) return 0;
									continue;
								}
								pc2.Length--;
								pc2.Pointer++;
								goto case 3;
							}
							
							if (l<13)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                                if (n!=-1)
                                {
                                    *pip=(byte)n;
                                    return ip;
                                }
							}
							pc2.Pointer+=l;
							pc.Length+=(int)(pc.Pointer-pc2.Pointer);
							if (pc.Length>6)
							{
								pc.Pointer=pc2.Pointer;
								zona=3;
								ip=0;
								continue;
							}
						} 
						return 0;
				}
			}while(true);
		}

		#endregion

	}
}
