using System;
using System.Runtime.InteropServices;
using EnPalermo.Lit;
using EnPalermo.Txt;

namespace EnPalermo.Int
{

	/// <summary>
	/// Representa una IP versin 4 y un puerto (similar a IPEndPoint).
	/// </summary>
    [StructLayout(LayoutKind.Sequential)]
	public struct IPPort:IComparable<IPPort>,IEquatable<IPPort>
	{

		#region Campos

		/// <summary>
		/// IP.
		/// </summary>
		public Int.IP IP;

		/// <summary>
		/// Puerto.
		/// </summary>
		public int Port;

        /// <summary>
        /// IPPort cero.
        /// </summary>
        public static readonly IPPort Zero=new IPPort();

		#endregion

		#region Constructores

		/// <summary>
		/// Contructor.
		/// </summary>
		/// <param name="ip">IP.</param>
		/// <param name="port">Puerto.</param>
		public IPPort(IP ip,int port)
		{
			IP=ip;
			Port=port;
		}
        
		/// <summary>
		/// Inicializa la IP en base a una IPAddress.
		/// </summary>
		/// <param name="ip">IPAddress.</param>
		/// <param name="port">Puerto.</param>
		public IPPort(System.Net.IPAddress ip, int port)
		{
			IP=new IP(ip);
			Port=port;
		}

		/// <summary>
		/// Inicializa la IP en base a un IPEndPoint.
		/// </summary>
		/// <param name="point">IPEndPoint.</param>
		public IPPort(System.Net.IPEndPoint point)
		{
            IP=new IP(point.Address);
            if (IP.IPNumber!=0)
            {
                Port=point.Port;
            }
            else Port=0;
		}

		/// <summary>
		/// Inicializa la IP en base a un numero.
		/// </summary>
		/// <param name="n">Numero (debe estar en el rango uint).</param>
		/// <param name="port">Puerto.</param>
		public IPPort(long n,int port)
		{
			IP=new IP(n);
			Port=port;
		}

		/// <summary>
		/// Inicializa la IP en base a un numero.
		/// </summary>
		/// <param name="n">Numero.</param>
		/// <param name="port">Puerto.</param>
		public IPPort(int n, int port)
		{
			IP=new IP(n);
			Port=port;
		}

		/// <summary>
		/// Inicializa la IP en base a un numero.
		/// </summary>
		/// <param name="n">Numero.</param>
		/// <param name="port">Puerto.</param>
		public IPPort(uint n,int port)
		{
			IP=new IP(n);
			Port=port;
		}

		/// <summary>
		/// Inicializa el IPPort en base a una cadena.
		/// </summary>
		/// <param name="str">Cadena.
		/// Modo de reconocimiento: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe ser menor a 65536 y encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// </param>
		public IPPort(string str)
		{
			int l=str==null?0:str.Length;
			if (l>6)
			{
				unsafe
				{
					fixed(char*p=str) 
					{
						IP=new IP(recognize(p,l,false,false,0,out Port));
					}
				}
			}
			else 
			{
				IP=Int.IP.Zero;
				Port=0;
			}
		}
        
		/// <summary>
		/// Inicializa la IP en base a una cadena.
		/// </summary>
		/// <param name="minPort">
		/// Puerto Minimo:
		/// Cualquier valor menor a este es tomado como invlido y se contina con el reconocimiento,
		/// en caso de que right y/o left sean true, o se interrumpe en caso contrario.
		/// La aparicin de un valor mayor a 65535 implica la interrupcin del reconocimiento a excepcin
		/// de que left y right esten en true y se este buscando a la derecha. En esta situacin se cambia la
		/// bsqueda hacia la izquierda.
		/// </param>
		/// <param name="right">
		/// Efectua reconocimiento a la derecha.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="left">
		/// Efectua reconocimiento a la izquierda.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="portDef">Valor por defecto que tendr el puerto si falla su reconocimiento.</param>
		/// <param name="str">Cadena.</param>
		public IPPort(string str,bool left, bool right,int minPort,int portDef)
		{
			int l=str==null?0:str.Length;
			if (l>6)
			{
				unsafe
				{
						fixed(char*p=str) 
						{
							IP=new IP(recognize(p,l,left,right,minPort,out Port));
                            if (IP.IPNumber!=0 && Port==0) Port=portDef;
						}
				}
			}
			else 
			{
                IP=Int.IP.Zero;
				Port=0;
			}
		}

		/// <summary>
		/// Inicializa la IP en base a una cadena.
		/// </summary>
		/// <param name="str">Cadena.</param>
        /// <param name="i">Inicio de la zona de reconocimiento. Mayor al rango no genera error, simplemente genera una ip invlida.</param>
        /// <param name="l">Largo de la zona de reconocimiento. -1 o mayor al rango implica reconocer sobre el resto de la cadena.</param>
        /// <param name="minPort">
		/// Puerto Minimo:
		/// Cualquier valor menor a este es tomado como invlido y se contina con el reconocimiento,
		/// en caso de que right y/o left sean true, o se interrumpe en caso contrario.
		/// La aparicin de un valor mayor a 65535 implica la interrupcin del reconocimiento a excepcin
		/// de que left y right esten en true y se este buscando a la derecha. En esta situacin se cambia la
		/// bsqueda hacia la izquierda.
		/// </param>
		/// <param name="right">
		/// Efectua reconocimiento a la derecha.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="left">
		/// Efectua reconocimiento a la izquierda.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="portDef">Valor por defecto que tendr el puerto si falla el reconocimiento.</param>
		public IPPort(string str, int i, int l,bool left, bool right,int minPort,int portDef)
		{
            int lc=str.Length;
            if (i<0)
                throw new ArgumentOutOfRangeException("i",i,Errors.NEGATIVE);
            if (l<7)
            {
                if (l==-1)
                {
                    l=lc-i;
                    if (l>6)
                        goto siga;
                }
                else if (l<0)
                    throw new ArgumentOutOfRangeException("l",l,Errors.MINOR_ONE_NEGATIVE);
                IP=Int.IP.Zero;
                Port=0;
                return;
            }
            if (i+l>lc)
            {
                l=lc-i;
                if (l<7)
                {
                    IP=Int.IP.Zero;
                    Port=0;
                    return;
                }
            }

            siga:

                unsafe
                {
                    fixed(char*p=str) 
                    {
                        IP=new IP(recognize(p+i,l,left,right,minPort,out Port));
                        if (IP.IPNumber!=0 && Port==0) Port=portDef;
                    }
                    return;
                }

			
		}

		/// <summary>
		/// Inicializa la IP en base a una cadena.
		/// </summary>
		/// <param name="str">Cadena.
		/// Modo de reconocimiento: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe ser menor a 65536 y encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// </param>
        /// <param name="i">Inicio de la zona de reconocimiento. Mayor al rango no genera error, simplemente genera una ip invlida.</param>
        /// <param name="l">Largo de la zona de reconocimiento. -1 o mayor al rango implica reconocer sobre el resto de la cadena.</param>
        public IPPort(string str, int i, int l):this(str,i,l,false,false,0,0)
		{}

		/// <summary>
		/// Inicializa la IP en base a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		/// <param name="minPort">
		/// Puerto Minimo:
		/// Cualquier valor menor a este es tomado como invlido y se contina con el reconocimiento,
		/// en caso de que right y/o left sean true, o se interrumpe en caso contrario.
		/// La aparicin de un valor mayor a 65535 implica la interrupcin del reconocimiento a excepcin
		/// de que left y right esten en true y se este buscando a la derecha. En esta situacin se cambia la
		/// bsqueda hacia la izquierda.
		/// </param>
		/// <param name="right">
		/// Efectua reconocimiento a la derecha.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="left">
		/// Efectua reconocimiento a la izquierda.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="portDef">Valor por defecto que tendr el puerto si falla el reconocimiento.</param>
		public unsafe IPPort(char*p, int l,bool left, bool right,int minPort,int portDef)
		{
			IP=new IP(recognize(p,l,left,right,minPort,out Port));
			if (IP.IPNumber!=0 && Port==0) Port=portDef;
		}

		/// <summary>
		/// Inicializa la IP en base a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero.
		/// Modo de reconocimiento: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe ser menor a 65536 y encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// </param>
		/// <param name="l">Largo.</param>
		public unsafe IPPort(char*p, int l)
		{
			IP=new IP(recognize(p,l,false,false,0,out Port));
		}

		/// <summary>
		/// Inicializa la IP en base a un puntero de bytes Ascii o Latin1.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		/// <param name="minPort">
		/// Puerto Minimo:
		/// Cualquier valor menor a este es tomado como invlido y se contina con el reconocimiento,
		/// en caso de que right y/o left sean true, o se interrumpe en caso contrario.
		/// La aparicin de un valor mayor a 65535 implica la interrupcin del reconocimiento a excepcin
		/// de que left y right esten en true y se este buscando a la derecha. En esta situacin se cambia la
		/// bsqueda hacia la izquierda.
		/// </param>
		/// <param name="right">
		/// Efectua reconocimiento a la derecha.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="left">
		/// Efectua reconocimiento a la izquierda.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="portDef">Valor por defecto que tendr el puerto si falla el reconocimiento.</param>
		public unsafe IPPort(byte*p, int l,bool left, bool right,int minPort,int portDef)
		{
			IP=new IP(recognize(p,l,left,right,minPort,out Port));
			if (IP.IPNumber!=0 && Port==0) Port=portDef;
		}


		/// <summary>
		/// Inicializa la IP en base a un puntero de bytes Ascii o Latin1.
		/// </summary>
		/// <param name="p">Puntero.
		/// Modo de reconocimiento: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe ser menor a 65536 y encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// </param>
		/// <param name="l">Largo.</param>
		public unsafe IPPort(byte*p, int l)
		{
			IP=new IP(recognize(p,l,false,false,0,out Port));
		}


		#endregion

		#region Propiedades

		/// <summary>
		/// Indica que la IP y el Puerto son vlidos (aunque puede ser de tipo local).
		/// </summary>
		public bool IsValid
		{
			get
			{
				return IP.IPNumber!=0 && IP.IPNumber!=UInt32.MaxValue && Port>0 && Port<65536;
			}
		}

		/// <summary>
		/// Indica que la IP y el Puerto son vlidos y la IP se encuentra fuera de los rangos considerados como IP locales.
		/// </summary>
		public bool IsReal
		{
			get
			{
				if  (Port<1 || Port>65535 || IP.IPNumber < 16777216U || IP.IPNumber > 4294901759U || 
					(IP.IPNumber > 167772159U && IP.IPNumber < 184549376U) ||
					(IP.IPNumber > 3232235519U && IP.IPNumber < 3232301056U) ||
					(IP.IPNumber > 2130706431U && IP.IPNumber < 2130706688U) ||
					(IP.IPNumber > 2851995647U && IP.IPNumber < 2852126720U) ||
					(IP.IPNumber > 2886729727U && IP.IPNumber < 2887778304U)) return false;
				return true;
			}
		}
		
		/// <summary>
		/// Retorna objeto IPEndPoint.
		/// </summary>
		public System.Net.IPEndPoint IPEndPoint
		{
			get
			{
				return new System.Net.IPEndPoint(IP.IPAddress,Port);
			}
		}

		#endregion

		#region Metodos

        /// <summary>
        /// Guarda en un puntero la cadena correspondiente a este IPPort.
        /// </summary>
        /// <param name="pc">Cadena a rellenar.</param>
        /// <returns>Carcteres utilizados en la cadena de relleno.</returns>
		public unsafe int FillPointer(char*pc)
		{
            fixed(uint* pint=&IP.IPNumber)
            {
                byte*p=(byte*)pint+3;
                int a=0;
                int n;
                do
                {
                    n=*p;
                    switch(n/100)
                    {
                        case 1:
                            pc[a++]='1';
                            n-=100;
                            break;
                        case 2:
                            pc[a++]='2';
                            n-=200;
                            break;
                    }
								
                    switch(n/10)
                    {
                        case 0:
                            if (*p>99) pc[a++]='0';
                            break;
                        case 1:
                            pc[a++]='1';
                            n-=10;
                            break;
                        case 2:
                            pc[a++]='2';
                            n-=20;
                            break;
                        case 3:
                            pc[a++]='3';
                            n-=30;
                            break;
                        case 4:
                            pc[a++]='4';
                            n-=40;
                            break;
                        case 5:
                            pc[a++]='5';
                            n-=50;
                            break;
                        case 6:
                            pc[a++]='6';
                            n-=60;
                            break;
                        case 7:
                            pc[a++]='7';
                            n-=70;
                            break;
                        case 8:
                            pc[a++]='8';
                            n-=80;
                            break;
                        case 9:
                            pc[a++]='9';
                            n-=90;
                            break;
                    }

                    switch(n)
                    {
                        case 0:
                            pc[a++]='0';
                            break;
                        case 1:
                            pc[a++]='1';
                            break;
                        case 2:
                            pc[a++]='2';
                            break;
                        case 3:
                            pc[a++]='3';
                            break;
                        case 4:
                            pc[a++]='4';
                            break;
                        case 5:
                            pc[a++]='5';
                            break;
                        case 6:
                            pc[a++]='6';
                            break;
                        case 7:
                            pc[a++]='7';
                            break;
                        case 8:
                            pc[a++]='8';
                            break;
                        case 9:
                            pc[a++]='9';
                            break;
                    }

                    if (pint!=p)
                    {
                        pc[a++]='.';
                        p--;
                    }
                    else break; 
                }while(true);
                if (Port>0 && Port<65536)
                {
                    pc[a++]=':';
                    n=Port;
                    if (n>10000)
                    {
                        pc[a++]=(char)(n/10000+48);
                        n=n%10000;
                        pc[a++]=(char)(n/1000+48);
                        n=n%1000;
                        pc[a++]=(char)(n/100+48);
                        n=n%100;
                        pc[a++]=(char)(n/10+48);
                        pc[a++]=(char)((n%10)+48);
                    }
                    else if (n>1000)
                    {
                        pc[a++]=(char)(n/1000+48);
                        n=n%1000;
                        pc[a++]=(char)(n/100+48);
                        n=n%100;
                        pc[a++]=(char)(n/10+48);
                        pc[a++]=(char)((n%10)+48);
                    }
                    else if (n>100)
                    {
                        pc[a++]=(char)(n/100+48);
                        n=n%100;
                        pc[a++]=(char)(n/10+48);
                        pc[a++]=(char)((n%10)+48);
                    }
                    else if (n>10)
                    {
                        pc[a++]=(char)(n/10+48);
                        pc[a++]=(char)((n%10)+48);
                    }
                    else
                    {
                        pc[a++]=(char)(n+48);
                    }
                }
                return a;
            }
		}

        /// <summary>
        /// Guarda en un puntero la cadena correspondiente a este IPPort.
        /// </summary>
        /// <param name="pc">Cadena a rellenar.</param>
        /// <returns>Carcteres utilizados en la cadena de relleno.</returns>
        public unsafe int FillPointer(byte*pc)
		{
			fixed(uint* pint=&IP.IPNumber)
			{
				byte*p=(byte*)pint+3;
				int a=0;
				int n;
				do
				{
					n=*p;
					switch(n/100)
					{
						case 1:
							pc[a++]=49;
							n-=100;
							break;
						case 2:
							pc[a++]=50;
							n-=200;
							break;
					}
								
					switch(n/10)
					{
						case 0:
							if (*p>99) pc[a++]=48;
							break;
						case 1:
							pc[a++]=49;
							n-=10;
							break;
						case 2:
							pc[a++]=50;
							n-=20;
							break;
						case 3:
							pc[a++]=51;
							n-=30;
							break;
						case 4:
							pc[a++]=52;
							n-=40;
							break;
						case 5:
							pc[a++]=53;
							n-=50;
							break;
						case 6:
							pc[a++]=54;
							n-=60;
							break;
						case 7:
							pc[a++]=55;
							n-=70;
							break;
						case 8:
							pc[a++]=56;
							n-=80;
							break;
						case 9:
							pc[a++]=57;
							n-=90;
							break;
					}

					switch(n)
					{
						case 0:
							pc[a++]=48;
							break;
						case 1:
							pc[a++]=49;
							break;
						case 2:
							pc[a++]=50;
							break;
						case 3:
							pc[a++]=51;
							break;
						case 4:
							pc[a++]=52;
							break;
						case 5:
							pc[a++]=53;
							break;
						case 6:
							pc[a++]=54;
							break;
						case 7:
							pc[a++]=55;
							break;
						case 8:
							pc[a++]=56;
							break;
						case 9:
							pc[a++]=57;
							break;
					}

					if (pint!=p)
					{
						pc[a++]=46;
						p--;
					}
					else break; 
				}while(true);
				if (Port>0 && Port<65536)
				{
					pc[a++]=58;
					n=Port;
					if (n>10000)
					{
						pc[a++]=(byte)(n/10000+48);
						n=n%10000;
						pc[a++]=(byte)(n/1000+48);
						n=n%1000;
						pc[a++]=(byte)(n/100+48);
						n=n%100;
						pc[a++]=(byte)(n/10+48);
						pc[a++]=(byte)((n%10)+48);
					}
					else if (n>1000)
					{
						pc[a++]=(byte)(n/1000+48);
						n=n%1000;
						pc[a++]=(byte)(n/100+48);
						n=n%100;
						pc[a++]=(byte)(n/10+48);
						pc[a++]=(byte)((n%10)+48);
					}
					else if (n>100)
					{
						pc[a++]=(byte)(n/100+48);
						n=n%100;
						pc[a++]=(byte)(n/10+48);
						pc[a++]=(byte)((n%10)+48);
					}
					else if (n>10)
					{
						pc[a++]=(byte)(n/10+48);
						pc[a++]=(byte)((n%10)+48);
					}
					else
					{
						pc[a++]=(byte)(n+48);
					}
				}
				return a;
			}
			
		}


		#endregion

		#region Metodos de Object

		/// <summary>
		/// Retorna la cadena correspondiente a esta IP.
		/// </summary>
		/// <returns>Cadena correspondiente a esta IP.</returns>
		public override string ToString()
		{
            unsafe
            {
                char*pc=stackalloc char[21];
                return Mutables.Create(pc,FillPointer(pc));
            }
		}

		/// <summary>
		/// True si son iguales.
		/// </summary>
		/// <param name="obj">Objeto.</param>
		/// <returns>True si son iguales.</returns>
		public override bool Equals(object obj)
		{
			if (obj is IPPort)
			{
				IPPort b=(IPPort)obj;
				return b.IP.IPNumber==this.IP.IPNumber && b.Port==this.Port;
			}
			return false;
		}

		/// <summary>
		/// Obtiene Hash.
		/// </summary>
		/// <returns>Cdigo Hash.</returns>
		public override int GetHashCode()
		{
			return (int)this.IP.IPNumber ^ (this.Port<<16);
		}

		#endregion

		#region Operadores

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IPPort a.</param>
		/// <param name="b">IPPort b.</param>
		/// <returns>True si son iguales.</returns>
		public static bool operator ==(IPPort a, IPPort b)
		{
			return a.IP.IPNumber==b.IP.IPNumber && a.Port==b.Port;
		}
		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IPPort a.</param>
		/// <param name="b">IPPort b.</param>
		/// <returns>True si son diferentes.</returns>
		public static bool operator !=(IPPort a, IPPort b)
		{
			return a.IP.IPNumber!=b.IP.IPNumber || a.Port!=b.Port;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IPPort a.</param>
		/// <param name="b">IPPort b.</param>
		/// <returns>True si A es mayor a B.</returns>
		public static bool operator >(IPPort a, IPPort b)
		{
			return a.IP.IPNumber>b.IP.IPNumber || (a.IP.IPNumber==b.IP.IPNumber && a.Port>b.Port);
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IPPort a.</param>
		/// <param name="b">IPPort b.</param>
		/// <returns>True si A es menor a B.</returns>
		public static bool operator <(IPPort a, IPPort b)
		{
			return a.IP.IPNumber<b.IP.IPNumber || (a.IP.IPNumber==b.IP.IPNumber && a.Port<b.Port);
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IPPort a.</param>
		/// <param name="b">IPPort b.</param>
		/// <returns>True si A es mayor o igual a B.</returns>
		public static bool operator >=(IPPort a, IPPort b)
		{
			return a.IP.IPNumber>b.IP.IPNumber || (a.IP.IPNumber==b.IP.IPNumber && a.Port>=b.Port);
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">IPPort a.</param>
		/// <param name="b">IPPort b.</param>
		/// <returns>True si A es menor o igual a B.</returns>
		public static bool operator <=(IPPort a, IPPort b)
		{
			return a.IP.IPNumber<b.IP.IPNumber || (a.IP.IPNumber==b.IP.IPNumber && a.Port<=b.Port);
		}


		/// <summary>
		/// Conversin a IPPort.
		/// </summary>
		/// <param name="obj">IPEndPoint.</param>
		/// <returns>IPPort.</returns>
		public static implicit operator IPPort(System.Net.IPEndPoint obj){return new IPPort(obj);}
		/// <summary>
		/// Conversin a IPPort (strict).
		/// </summary>
		/// <param name="obj">Cadena.</param>
		/// <returns>IPPort.</returns>
		public static implicit operator IPPort(string obj){return new IPPort(obj);}
		/// <summary>
		/// Conversin a String.
		/// </summary>
		/// <param name="p">IPPort.</param>
		/// <returns>String.</returns>
		public static implicit operator string(IPPort p){return p.ToString();}
		/// <summary>
		/// Conversin a IPEndPoint.
		/// </summary>
		/// <param name="p">IPPort.</param>
		/// <returns>IPEndPoint.</returns>
		public static implicit operator System.Net.IPEndPoint(IPPort p){return p.IPEndPoint;}
        
		#endregion

		#region Rutina de Deteccin

		/// <summary>
		/// Funcin interna de reconocimiento.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		/// <param name="port">
		/// Variable de salida con el nmero de puerto.
		/// Siempre es 0 si ip es 0.
		/// </param>
		/// <param name="minPort">
		/// Puerto Minimo:
		/// Cualquier valor menor a este es tomado como invlido y se contina con el reconocimiento,
		/// en caso de que right y/o left sean true, o se interrumpe en caso contrario.
		/// La aparicin de un valor mayor a 65535 implica la interrupcin del reconocimiento a excepcin
		/// de que left y right esten en true y se este buscando a la derecha. En esta situacin se cambia la
		/// bsqueda hacia la izquierda.
		/// </param>
		/// <param name="right">
		/// Efectua reconocimiento a la derecha.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="left">
		/// Efectua reconocimiento a la izquierda.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <returns>IP o 0.</returns>
		private unsafe static uint recognize(char*p,int l,bool left, bool right,int minPort,out int port)
		{
			Mem.PChar pc=new Mem.PChar(p,l,true,true);
			port=0;
			if(pc.Length<7) return 0;
			Mem.PChar pc2;
			uint ip=0;
			byte* pip=(byte*)&ip;
			int zona=3,n;
			if (!right && !left)
			{
				do
				{
					pc2=pc.Foward('.',false);
					if (zona!=0)
					{
						if (pc.Length!=0 && pc2.Length!=0 && pc2.Length<4)
						{
							l=0;
							while(pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0' && ++l!=pc2.Length);
							if (pc2.Length==l)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                                if (n!=-1)
                                {
                                    pip[zona--]=(byte)n;
                                    continue;
                                }
							}
						}
					}
					else if (pc2.Length!=0)
					{
						l=0;
						while(pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0' && ++l!=4 && l!=pc2.Length);
						if (l!=0 && l!=4)
						{
                            n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                            if (n!=-1)
							{
								*pip=(byte)n;
								pc2.Length-=l;
								if (pc2.Length>1)
								{
									pc2.Pointer+=l;
									do
									{
										switch(*pc2.Pointer)
										{
											case ' ':
											case '\t':
											case '\xa0':
											case '\x85':
											case '\0':
												break;
											case ':':
												if (right) return ip;
												right=true;
												break;
											case '0':
											case '1':
											case '2':
											case '3':
											case '4':
											case '5':
											case '6':
											case '7':
											case '8':
											case '9':
												l=1;
												while(l!=pc2.Length && l!=6 && pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0') l++;
												if (l!=6)
												{
                                                    n=StrToNum.StrToPositiveInt(pc2.Pointer,l,minPort,65535);
                                                    if (n!=-1)
                                                        port=n;
												}
												return ip;
											default:
												return ip;
										}
										if (--pc2.Length==0) return ip;
										pc2.Pointer++;
									}while(true);
								}
								return ip;
							}
						}
					}
					return 0;
				}while(true);
			}

			p=pc.Pointer;
			int lant=0;
			do
			{
				pc2=pc.Foward('.',true);
				switch(zona)
				{
					case 3:
						if (pc.Length<5) return 0;
						l=pc2.Length;
						while(--l!=-1 && pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0');
						if (++l!=pc2.Length)
						{
							if (l!=0)
							{
								pc2.Pointer+=l;
								pc2.Length-=l;
							}
							if (pc2.Length<13)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,pc2.Length,0,255);
                                if (n!=-1)
                                {
                                    pip[zona--]=(byte)n;
                                    if (left)
                                    {
                                        if (pc2.Pointer==p) lant=0;
                                        else lant=(int)(pc2.Pointer-p)-1;
                                    }
                                }
							}
						}
						continue;
					case 2:
					case 1:
						if (pc.Length==0) return 0;
						if (pc2.Length<13)
						{
							l=pc2.Length;
							while(--l!=-1 && pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0');
							if (l==-1)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,pc2.Length,0,255);
                                if (n!=-1)
                                {
                                    pip[zona--]=(byte)n;
                                    continue;
                                }

							} 
							else if (++l!=pc2.Length)
							{
								pc2.Length-=l;
								pc2.Pointer+=l;
								zona=3;
								ip=0;
								goto case 3;
							}
							if (pc.Length<7) return 0;
							zona=3;
							ip=0;
							continue;
						} 
						zona=3;
						ip=0;
						goto case 3;
					default:
						if (pc2.Length!=0)
						{
							l=0;
							while(pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0' && ++l!=pc2.Length);
							if (l==0)
							{
								zona=3;
								ip=0;
								if (pc2.Length==1) 
								{
									if (pc.Length<7) return 0;
									continue;
								}
								pc2.Length--;
								pc2.Pointer++;
								goto case 3;
							}
							
							if (l<13)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                                if (n!=-1)
								{
									*pip=(byte)n;
									if (right)
									{
										if (pc.Length!=0) pc.Pointer+=pc.Length;
										else pc.Pointer=pc2.Pointer+pc2.Length;
										pc2.Pointer+=++l;
										pc2.Length=(int)(pc.Pointer-pc2.Pointer);
										if (pc2.Length>0)
										{
											if (pc2.Pointer[-1]==':' && *pc2.Pointer<='9' && *pc2.Pointer>='0')
											{
												l=1;
												while(l!=pc2.Length && l!=13 && pc2.Pointer[l]<='9' && pc2.Pointer[l]>='0') l++;
												if (l!=13)
												{
                                                    n=StrToNum.StrToPositiveInt(pc2.Pointer,l,minPort,65535);
                                                    if (n!=-1)
                                                        port=n;
												}
												return ip;
											}
											do
											{
												pc=pc2.FowardNumber(true);
												if (pc.Length!=0 && pc.Length<13)
												{
													if (pc.Pointer[-1]=='.' || (pc2.Length>1 && pc2.Pointer[-1]=='.' && ((*pc2.Pointer<='9' && *pc2.Pointer>='0') || (*pc2.Pointer>='a' && *pc2.Pointer<='z') || (*pc2.Pointer>='A' && *pc2.Pointer<='Z')))) continue;
                                                    n=StrToNum.StrToPositiveInt(pc.Pointer,pc.Length,minPort,Int32.MaxValue);
                                                    if(n>65535)
                                                        break;
                                                    if (n==-1)
                                                        continue;
													port=n;
													return ip;
												}
                                                break;
											}while(true);
										}
									}
									if (lant!=0)
									{
										pc2.Pointer=p;
										pc2.Length=lant;
										do
										{
											pc=pc2.BackwardNumber(true);
											if (pc.Length!=0 && pc.Length<13)
											{
												if (pc.Pointer[pc.Length]=='.')
												{
													char c=pc.Pointer[pc.Length+1];
													if ((c<='9' && c>='0') || (c>='a' && c<='z') || (c>='A' && c<'Z')) continue;
												}
												else if (pc2.Length!=0 && pc2.Pointer[pc2.Length]==46) continue;
                                                n=StrToNum.StrToPositiveInt(pc.Pointer,pc.Length,minPort,Int32.MaxValue);
                                                if (n>65535)
                                                    return ip;
                                                if (n==-1)
                                                    continue;
												port=n;
												return ip;
											}
											break;
										}while(true);
									}
									return ip;
								}
							}
							pc2.Pointer+=l;
							pc.Length+=(int)(pc.Pointer-pc2.Pointer);
							if (pc.Length>6)
							{
								pc.Pointer=pc2.Pointer;
								zona=3;
								ip=0;
								continue;
							}
						} 
						return 0;
				}
			}while(true);
		}

		/// <summary>
		/// Funcin interna de reconocimiento.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		/// <param name="port">
		/// Variable de salida con el nmero de puerto.
		/// Siempre es 0 si ip es 0.
		/// </param>
		/// <param name="minPort">
		/// Puerto Minimo:
		/// Cualquier valor menor a este es tomado como invlido y se contina con el reconocimiento,
		/// en caso de que right y/o left sean true, o se interrumpe en caso contrario.
		/// La aparicin de un valor mayor a 65535 implica la interrupcin del reconocimiento a excepcin
		/// de que left y right esten en true y se este buscando a la derecha. En esta situacin se cambia la
		/// bsqueda hacia la izquierda.
		/// </param>
		/// <param name="right">
		/// Efectua reconocimiento a la derecha.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <param name="left">
		/// Efectua reconocimiento a la izquierda.
		/// - Si left y right son false: Luego de efectuar un trim izquierdo debe encontrarse una IP
		/// correctamente formateada al inicio de la cadena.
		/// El puerto debe encontrarse inmediatamente a la derecha de la ip,
		/// solo aceptndose espacios y/o ':' como separadores.
		/// En el resto de los casos recorre toda la cadena en bsqueda de una IP.
		/// aceptando puntos duplicados y cadenas numricas con ms de 3 carcteres.
		/// - Si left y right son true: 
		/// Iniciar la bsqueda del puerto comenzando por la derecha (hacia adelante) y luego
		/// por la izquierda (hacia atras).
		/// - Si right es true: Iniciar la bsqueda del puerto a la derecha.
		/// - Si left es true: Iniciar la bsqueda del puerto a la izquierda (bscando hacia atras).
		/// </param>
		/// <returns>IP o 0.</returns>
		private unsafe static uint recognize(byte*p,int l,bool left, bool right,int minPort,out int port)
		{
			Mem.PByte pc=new Mem.PByte(p,l,true,true,true);
			port=0;
			if(pc.Length<7) return 0;
			Mem.PByte pc2;
			uint ip=0;
			byte* pip=(byte*)&ip;
			int zona=3,n;
			if (!right && !left)
			{
				do
				{
					pc2=pc.Foward(46,false);
					if (zona!=0)
					{
						if (pc.Length!=0 && pc2.Length!=0 && pc2.Length<4)
						{
							l=0;
							while(pc2.Pointer[l]<58 && pc2.Pointer[l]>47 && ++l!=pc2.Length);
							if (pc2.Length==l)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                                if (n!=-1)
                                {
                                    pip[zona--]=(byte)n;
                                    continue;
                                }

							}
						}
					}
					else if (pc2.Length!=0)
					{
						l=0;
						while(pc2.Pointer[l]<58 && pc2.Pointer[l]>47 && ++l!=4 && l!=pc2.Length);
						if (l!=0 && l!=4)
						{
                            n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                            if (n!=-1)
							{
								*pip=(byte)n;
								pc2.Length-=l;
								if (pc2.Length>1)
								{
									pc2.Pointer+=l;
									do
									{
										switch(*pc2.Pointer)
										{
											case 20:
											case 9:
											case 160:
											case 0:
												break;
											case 58:
												if (right) return ip;
												right=true;
												break;
											case 48:
											case 49:
											case 50:
											case 51:
											case 52:
											case 53:
											case 54:
											case 55:
											case 56:
											case 57:
												l=1;
												while(l!=pc2.Length && l!=6 && pc2.Pointer[l]<58 && pc2.Pointer[l]>47) l++;
												if (l!=6)
												{
                                                    n=StrToNum.StrToPositiveInt(pc2.Pointer,l,minPort,65535);
                                                    if (n!=-1)
                                                        port=n;
												}
												return ip;
											default:
												return ip;
										}
										if (--pc2.Length==0) return ip;
										pc2.Pointer++;
									}while(true);
								}
								return ip;
							}
						}
					}
					return 0;
				}while(true);
			}

			p=pc.Pointer;
			int lant=0;
			do
			{
				pc2=pc.Foward(46,true);
				switch(zona)
				{
					case 3:
						if (pc.Length<5) return 0;
						l=pc2.Length;
						while(--l!=-1 && pc2.Pointer[l]<58 && pc2.Pointer[l]>47);
						if (++l!=pc2.Length)
						{
							if (l!=0)
							{
								pc2.Pointer+=l;
								pc2.Length-=l;
							}
							if (pc2.Length<13)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,pc2.Length,0,255);
                                if (n!=-1)
								{
									pip[zona--]=(byte)n;
									if (left)
									{
										if (pc2.Pointer==p) lant=0;
										else lant=(int)(pc2.Pointer-p)-1;
									}
								}
							}
						}
						continue;
					case 2:
					case 1:
						if (pc.Length==0) return 0;
						if (pc2.Length<13)
						{
							l=pc2.Length;
							while(--l!=-1 && pc2.Pointer[l]<58 && pc2.Pointer[l]>47);
							if (l==-1)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,pc2.Length,0,255);
                                if (n!=-1)
                                {
                                    pip[zona--]=(byte)n;
                                    continue;
                                }
							} 
							else if (++l!=pc2.Length)
							{
								pc2.Length-=l;
								pc2.Pointer+=l;
								zona=3;
								ip=0;
								goto case 3;
							}
							if (pc.Length<7) return 0;
							zona=3;
							ip=0;
							continue;
						} 
						zona=3;
						ip=0;
						goto case 3;
					default:
						if (pc2.Length!=0)
						{
							l=0;
							while(pc2.Pointer[l]<58 && pc2.Pointer[l]>47 && ++l!=pc2.Length);
							if (l==0)
							{
								zona=3;
								ip=0;
								if (pc2.Length==1) 
								{
									if (pc.Length<7) return 0;
									continue;
								}
								pc2.Length--;
								pc2.Pointer++;
								goto case 3;
							}
							
							if (l<13)
							{
                                n=StrToNum.StrToPositiveInt(pc2.Pointer,l,0,255);
                                if (n!=-1)
								{
									*pip=(byte)n;
									if (right)
									{
										if (pc.Length!=0) pc.Pointer+=pc.Length;
										else pc.Pointer=pc2.Pointer+pc2.Length;
										pc2.Pointer+=++l;
										pc2.Length=(int)(pc.Pointer-pc2.Pointer);
										if (pc2.Length>0)
										{
											if (pc2.Pointer[-1]==58 && *pc2.Pointer<58 && *pc2.Pointer>47)
											{
												l=1;
												while(l!=pc2.Length && l!=13 && pc2.Pointer[l]<58 && pc2.Pointer[l]>47) l++;
												if (l!=13)
												{
                                                    n=StrToNum.StrToPositiveInt(pc2.Pointer,l,minPort,65535);
                                                    if (n!=-1)
                                                        port=n;
												}
												return ip;
											}
											do
											{
												pc=pc2.FowardNumber(true);
												if (pc.Length!=0 && pc.Length<13)
												{
													if (pc.Pointer[-1]==46 || (pc2.Length>1 && pc2.Pointer[-1]==46 && ((*pc2.Pointer<58 && *pc2.Pointer>47) || (*pc2.Pointer>96 && *pc2.Pointer<123) || (*pc2.Pointer>64 && *pc2.Pointer<91)))) continue;
                                                    n=StrToNum.StrToPositiveInt(pc.Pointer,pc.Length,minPort,Int32.MaxValue);
                                                    if (n>65535)
                                                        break;
                                                    if (n==-1)
                                                        continue;
													port=n;
													return ip;
												}
												break;
											}while(true);
										}
									}
									if (lant!=0)
									{
										pc2.Pointer=p;
										pc2.Length=lant;
										do
										{
											pc=pc2.BackwardNumber(true);
											if (pc.Length!=0 && pc.Length<13)
											{
												if (pc.Pointer[pc.Length]==46)
												{
													byte c=pc.Pointer[pc.Length+1];
													if ((c<58 && c>47) || (c>96 && c<123) || (c>64 && c<91)) continue;
												}
												else if (pc2.Length!=0 && pc2.Pointer[pc2.Length]==46) continue;
                                                n=StrToNum.StrToPositiveInt(pc.Pointer,pc.Length,minPort,Int32.MaxValue);
                                                if (n>65535)
                                                    return ip;
                                                if (n==-1)
                                                    continue;
                                                port=n;
												return ip;
											}
											break;
										}while(true);
									}
									return ip;
								}
							}
							pc2.Pointer+=l;
							pc.Length+=(int)(pc.Pointer-pc2.Pointer);
							if (pc.Length>6)
							{
								pc.Pointer=pc2.Pointer;
								zona=3;
								ip=0;
								continue;
							}
						} 
						return 0;
				}
			}while(true);
		}


		#endregion


        #region IComparable<IPPort> Members

        /// <summary>
        /// Implementacin de CompareTo.
        /// </summary>
        /// <param name="b">IP.</param>
        /// <returns>0 si son iguales.</returns>
        public int CompareTo(IPPort b)
        {
            if (this.IP.IPNumber < b.IP.IPNumber) return -1;
            if (this.IP.IPNumber > b.IP.IPNumber) return 1;
            if (this.Port < b.Port) return -1;
            if (this.Port > b.Port) return 1;
            return 0;
        }

        #endregion

        #region IEquatable<IPPort> Members

        /// <summary>
        /// True si son iguales.
        /// </summary>
        /// <param name="b">IP.</param>
        /// <returns>True si son iguales.</returns>
        public bool Equals(IPPort b)
        {
            return b.IP.IPNumber == this.IP.IPNumber && b.Port == this.Port;
        }

        #endregion
}
}
