using System;

namespace EnPalermo.Mem
{

    /// <summary>
    /// Clase esttica con las funciones bsicas para trabajo con punteros (Copy, Move, Clean, Fill, Cut y Paste).
    /// </summary>
    public unsafe static class General 
    {

        #region Constantes y constructor esttico

        /// <summary>
        /// Cantidad Maxima en bytes para copiar a Stack con bajo riesgo de producir un StackOverflow.
        /// </summary>
        public const int MaxStackBytes = 2048;
        /// <summary>
        /// Cantidad Maxima en chars para copiar a Stack con bajo riesgo de producir un StackOverflow.
        /// </summary>
        public const int MaxStackChars = MaxStackBytes >> 1;
        /// <summary>
        /// Cantidad Maxima en ints para copiar a Stack con bajo riesgo de producir un StackOverflow.
        /// </summary>
        public const int MaxStackInts = MaxStackBytes >> 2;

        /// <summary>
        /// Cantidad Maxima en longs para copiar a Stack con bajo riesgo de producir un StackOverflow.
        /// </summary>
        public const int MaxStackLongs = MaxStackBytes >> 3;

        /// <summary>
        /// Si un puntero tiene un valor mayor a este, indica que puede encontrarse en el Heap.
        /// </summary>
        public static readonly void* MaximumStack;

        /// <summary>
        /// Si alguna variable local tiene un valor menor a este, no usar copia a Stack.
        /// </summary>
        public static readonly void* MinimumStack;

        /// <summary>
        /// Si true, NetFrameWork esta corriendo en 64 bits.
        /// Si false, NetFrameWork corre en 32 bits (nativo o WoW64).
        /// </summary>
        public static readonly bool FrameWork64;

        /// <summary>
        /// Desplazamiento segn plataforma: 3 para 64bits o 2 para 32bits.
        /// </summary>
        public static readonly int DespVoid;

        /// <summary>
        /// Cantidad de Bytes por cada PByte o PChar.
        /// </summary>
        public static readonly int MultPByte;

        /// <summary>
        /// Cantidad Maxima en longs para copiar a Stack con bajo riesgo de producir un StackOverflow.
        /// </summary>
        public static readonly int MaxStackVoids;

        static General()
        {
            long min = 0;
            MaximumStack = ((byte*)&min)+131072;
            if (IntPtr.Size == 8)
            {
                FrameWork64 = true;
                DespVoid = 3;
                MaxStackVoids = MaxStackBytes >> 3;
                min = (long)((ulong)MaximumStack) - (131072L << 2);
                MinimumStack=(min <= (131072L << 2))?((void*)(131072L << 2)):((void*)min);
                MultPByte=12;
            }
            else
            {
                FrameWork64 = false;
                DespVoid = 2;
                MaxStackVoids = MaxStackBytes >> 2;
                min = (long)((int)MaximumStack - (131072 << 2));
                MinimumStack=(min <= (131072L << 2))?((void*)(131072 << 2)):((void*)(int)min);
                MultPByte=8;
            }
        }

        #endregion

        #region Copy

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(byte* po, byte* pd, int l)
        {
            if (po == pd)
                return;
            if (l > 63)
            {
                if (!FrameWork64)
                {
                    do
                    {
                        *(int*)pd = *((int*)po);
                        *(int*)(pd + 4) = *(int*)(po + 4);
                        *(int*)(pd + 8) = *(int*)(po + 8);
                        *(int*)(pd + 12) = *(int*)(po + 12);
                        *(int*)(pd + 16) = *(int*)(po + 16);
                        *(int*)(pd + 20) = *(int*)(po + 20);
                        *(int*)(pd + 24) = *(int*)(po + 24);
                        *(int*)(pd + 28) = *(int*)(po + 28);
                        *(int*)(pd + 32) = *(int*)(po + 32);
                        *(int*)(pd + 36) = *(int*)(po + 36);
                        *(int*)(pd + 40) = *(int*)(po + 40);
                        *(int*)(pd + 44) = *(int*)(po + 44);
                        *(int*)(pd + 48) = *(int*)(po + 48);
                        *(int*)(pd + 52) = *(int*)(po + 52);
                        *(int*)(pd + 56) = *(int*)(po + 56);
                        *(int*)(pd + 60) = *(int*)(po + 60);
                        pd += 64;
                        po += 64;
                    }
                    while ((l -= 64) > 63);
                }
                else
                {
                    do
                    {
                        *(long*)pd = *((long*)po);
                        *(long*)(pd + 8) = *(long*)(po + 8);
                        *(long*)(pd + 16) = *(long*)(po + 16);
                        *(long*)(pd + 24) = *(long*)(po + 24);
                        *(long*)(pd + 32) = *(long*)(po + 32);
                        *(long*)(pd + 40) = *(long*)(po + 40);
                        *(long*)(pd + 48) = *(long*)(po + 48);
                        *(long*)(pd + 56) = *(long*)(po + 56);
                        pd += 64;
                        po += 64;
                    }
                    while ((l -= 64) > 63);
                }
            }
            if (l > 0)
            {
                if ((l & 32) != 0)
                {
                    *(long*)pd = *(long*)po;
                    *(long*)(pd + 8) = *(long*)(po + 8);
                    *(long*)(pd + 16) = *(long*)(po + 16);
                    *(long*)(pd + 24) = *(long*)(po + 24);
                    pd += 32;
                    po += 32;
                }
                if ((l & 16) != 0)
                {
                    *(long*)pd = *(long*)po;
                    *(long*)(pd + 8) = *(long*)(po + 8);
                    pd += 16;
                    po += 16;
                }
                if ((l & 8) != 0)
                {
                    *(long*)pd = *(long*)po;
                    pd += 8;
                    po += 8;
                }
                if ((l & 4) != 0)
                {
                    *(int*)pd = *(int*)po;
                    pd += 4;
                    po += 4;
                }
                if ((l & 2) != 0)
                {
                    *(short*)pd = *(short*)po;
                    pd += 2;
                    po += 2;
                }
                if ((l & 1) != 0) 
                    *pd = *po;
            }
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(sbyte* po, sbyte* pd, int l) 
        {
            Copy((byte*)po, (byte*)pd, l);
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(ushort* po, ushort* pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l<<1);
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(short* po, short* pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l << 1);
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(char* po, char* pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l << 1);
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(uint* po, uint* pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l << 2);
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(int* po, int* pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l << 2);
        }


        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(ulong* po, ulong* pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l << 3);
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(PByte* po, PByte* pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l * MultPByte);
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(PChar* po, PChar* pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l * MultPByte);
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(long* po, long* pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l << 3);
        }

        /// <summary>
        /// Copia memoria. Nota: Los bloques no pueden estar solapados. Si lo pueden estar, usar Move.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Copy(void** po, void** pd, int l)
        {
            Copy((byte*)po, (byte*)pd, l << DespVoid);
        }

        #endregion

        #region Move

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(byte* po, byte* pd, int l)
        {
            if (po >= pd)
            {
                Copy(po, pd, l);
                return;
            }
            po += l;
            pd += l;
            if (l > 63)
            {
                if (!FrameWork64)
                {
                    do
                    {
                        *(int*)(pd - 4) = *(int*)(po - 4);
                        *(int*)(pd - 8) = *(int*)(po - 8);
                        *(int*)(pd - 12) = *(int*)(po - 12);
                        *(int*)(pd - 16) = *(int*)(po - 16);
                        *(int*)(pd - 20) = *(int*)(po - 20);
                        *(int*)(pd - 24) = *(int*)(po - 24);
                        *(int*)(pd - 28) = *(int*)(po - 28);
                        *(int*)(pd - 32) = *(int*)(po - 32);
                        *(int*)(pd - 36) = *(int*)(po - 36);
                        *(int*)(pd - 40) = *(int*)(po - 40);
                        *(int*)(pd - 44) = *(int*)(po - 44);
                        *(int*)(pd - 48) = *(int*)(po - 48);
                        *(int*)(pd - 52) = *(int*)(po - 52);
                        *(int*)(pd - 56) = *(int*)(po - 56);
                        *(int*)(pd - 60) = *(int*)(po - 60);
                        *(int*)(pd -= 64) = *(int*)(po -= 64);
                    } while ((l -= 64) > 63);
                }
                else
                {
                    do
                    {
                        *(long*)(pd - 8) = *(long*)(po - 8);
                        *(long*)(pd - 16) = *(long*)(po - 16);
                        *(long*)(pd - 24) = *(long*)(po - 24);
                        *(long*)(pd - 32) = *(long*)(po - 32);
                        *(long*)(pd - 40) = *(long*)(po - 40);
                        *(long*)(pd - 48) = *(long*)(po - 48);
                        *(long*)(pd - 56) = *(long*)(po - 56);
                        *(long*)(pd -= 64) = *(long*)(po -= 64);
                    } while ((l -= 64) > 63);
                }
            }
            if (l > 0)
            {
                if ((l & 32) != 0)
                {
                    *(long*)(pd - 8) = *(long*)(po - 8);
                    *(long*)(pd - 16) = *(long*)(po - 16);
                    *(long*)(pd - 24) = *(long*)(po - 24);
                    *(long*)(pd -= 32) = *(long*)(po -= 32);
                }
                if ((l & 16) != 0)
                {
                    *(long*)(pd - 8) = *(long*)(po - 8);
                    *(long*)(pd -= 16) = *(long*)(po -= 16);
                }
                if ((l & 8) != 0)
                {
                    *(long*)(pd -= 8) = *(long*)(po -= 8);
                }
                if ((l & 4) != 0)
                {
                    *(int*)(pd -= 4) = *(int*)(po -= 4);
                }
                if ((l & 2) != 0)
                {
                    *(short*)(pd -= 2) = *(short*)(po -= 2);
                }
                if ((l & 1) != 0)
                {
                    pd[-1] = po[-1];
                }
            }
        }

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(sbyte* po, sbyte* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l);
        }

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(ushort* po, ushort* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l<<1);
        }

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(short* po, short* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l<<1);
        }

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(char* po, char* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l << 1);
        }

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(uint* po, uint* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l << 2);
        }

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(int* po, int* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l << 2);
        }


        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(ulong* po, ulong* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l << 3);
        }

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(PByte* po, PByte* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l * MultPByte);
        }

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(PChar* po, PChar* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l * MultPByte);
        }

        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(long* po, long* pd, int l)
        {
            Move((byte*)po, (byte*)pd, l << 3);
        }


        /// <summary>
        /// Mueve memoria.
        /// La diferencia con copiar es que verifica si los bloques
        /// estan solapados y actua en consecuencia.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino.</param>
        /// <param name="l">Largo.</param>
        public static void Move(void** po, void** pd, int l)
        {
            Move((byte*)po, (byte*)pd, l << DespVoid);
        }


        #endregion

        #region Clean

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(byte* p, int l)
        {
            if (l > 63)
            {
                do
                {
                    *(long*)p = 0L;
                    *(long*)(p + 8) = 0L;
                    *(long*)(p + 16) = 0L;
                    *(long*)(p + 24) = 0L;
                    *(long*)(p + 32) = 0L;
                    *(long*)(p + 40) = 0L;
                    *(long*)(p + 48) = 0L;
                    *(long*)(p + 56) = 0L;
                    p += 64;
                }
                while ((l -= 64) > 63);
            }
            if (l > 0)
            {
                if ((l & 32) != 0)
                {
                    *(long*)p = 0L;
                    *(long*)(p + 8) = 0L;
                    *(long*)(p + 16) = 0L;
                    *(long*)(p + 24) = 0L;
                    p += 32;
                }
                if ((l & 16) != 0)
                {
                    *(long*)p = 0L;
                    *(long*)(p + 8) = 0L;
                    p += 16;
                }
                if ((l & 8) != 0)
                {
                    *(long*)p = 0L;
                    p += 8;
                }
                if ((l & 4) != 0)
                {
                    *(int*)p = 0;
                    p += 4;
                }
                if ((l & 2) != 0)
                {
                    *(short*)p = 0;
                    p += 2;
                }
                if ((l & 1) != 0) *p = 0;
            }
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(sbyte* p, int l)
        {
            Clean((byte*)p, l);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(ushort* p, int l)
        {
            Clean((byte*)p, l<<1);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(short* p, int l)
        {
            Clean((byte*)p, l << 1);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(char* p, int l)
        {
            Clean((byte*)p, l << 1);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(uint* p, int l)
        {
            Clean((byte*)p, l << 2);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(int* p, int l)
        {
            Clean((byte*)p, l << 2);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(ulong* p, int l)
        {
            Clean((byte*)p, l << 3);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(PByte* p, int l)
        {
            Clean((byte*)p, l * MultPByte);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(PChar* p, int l)
        {
            Clean((byte*)p, l * MultPByte);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(long* p, int l)
        {
            Clean((byte*)p, l << 3);
        }

        /// <summary>
        /// Rellena memoria con ceros.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        public static void Clean(void** p, int l)
        {
            Clean((byte*)p, l << DespVoid);
        }

        #endregion

        #region Fill

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(byte* p, int l, byte fill)
        {
            if (l < 8)
            {
                switch (l)
                {
                    case 1:
                        *p = fill;
                        return;
                    case 2:
                        *p = fill;
                        p[1] = fill;
                        return;
                    case 3:
                        *p = fill;
                        p[1] = fill;
                        p[2] = fill;
                        return;
                    case 4:
                        *p = fill;
                        p[1] = fill;
                        *(short*)(p + 2) = *(short*)p;
                        return;
                    case 5:
                        *p = fill;
                        p[1] = fill;
                        *(short*)(p + 2) = *(short*)p;
                        p[4] = fill;
                        return;
                    case 6:
                        *p = fill;
                        p[1] = fill;
                        *(short*)(p + 2) = *(short*)p;
                        *(short*)(p + 4) = *(short*)p;
                        return;
                    case 7:
                        *p = fill;
                        p[1] = fill;
                        *(short*)(p + 2) = *(short*)p;
                        *(short*)(p + 4) = *(short*)p;
                        p[6] = fill;
                        return;
                }
                return;
            }
            byte* pr = stackalloc byte[8];
            *pr = fill;
            pr[1] = fill;
            *(short*)(pr + 2) = *(short*)pr;
            *(int*)(pr + 4) = *(int*)pr;
            if (l > 63)
            {
                do
                {
                    *(long*)p = *(long*)pr;
                    *(long*)(p + 8) = *(long*)pr;
                    *(long*)(p + 16) = *(long*)pr;
                    *(long*)(p + 24) = *(long*)pr;
                    *(long*)(p + 32) = *(long*)pr;
                    *(long*)(p + 40) = *(long*)pr;
                    *(long*)(p + 48) = *(long*)pr;
                    *(long*)(p + 56) = *(long*)pr;
                    p += 64;
                }
                while ((l -= 64) > 63);
            }
            if (l > 0)
            {
                if ((l & 32) != 0)
                {
                    *(long*)p = *(long*)pr;
                    *(long*)(p + 8) = *(long*)pr;
                    *(long*)(p + 16) = *(long*)pr;
                    *(long*)(p + 24) = *(long*)pr;
                    p += 32;
                }
                if ((l & 16) != 0)
                {
                    *(long*)p = *(long*)pr;
                    *(long*)(p + 8) = *(long*)pr;
                    p += 16;
                }
                if ((l & 8) != 0)
                {
                    *(long*)p = *(long*)pr;
                    p += 8;
                }
                if ((l & 4) != 0)
                {
                    *(int*)p = *(int*)pr;
                    p += 4;
                }
                if ((l & 2) != 0)
                {
                    *(short*)p = *(short*)pr;
                    p += 2;
                }
                if ((l & 1) != 0) 
                    *p = fill;
            }
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(char* p, int l, char fill)
        {
            if (l < 8)
            {
                switch (l)
                {
                    case 1:
                        *p = fill;
                        return;
                    case 2:
                        *p = fill;
                        p[1] = fill;
                        return;
                    case 3:
                        *p = fill;
                        p[1] = fill;
                        p[2] = fill;
                        return;
                    case 4:
                        *p = fill;
                        p[1] = fill;
                        *(int*)(p + 2) = *(int*)p;
                        return;
                    case 5:
                        *p = fill;
                        p[1] = fill;
                        *(int*)(p + 2) = *(int*)p;
                        p[4] = fill;
                        return;
                    case 6:
                        *p = fill;
                        p[1] = fill;
                        *(int*)(p + 2) = *(int*)p;
                        *(int*)(p + 4) = *(int*)p;
                        return;
                    case 7:
                        *p = fill;
                        p[1] = fill;
                        *(int*)(p + 2) = *(int*)p;
                        *(int*)(p + 4) = *(int*)p;
                        p[6] = fill;
                        return;
                }
                return;
            }
            char* pr = stackalloc char[4];
            *pr = fill;
            pr[1] = fill;
            *(int*)(pr + 2) = *(int*)pr;
            if (l > 31)
            {
                do
                {
                    *(long*)p = *(long*)pr;
                    *(long*)(p + 4) = *(long*)pr;
                    *(long*)(p + 8) = *(long*)pr;
                    *(long*)(p + 12) = *(long*)pr;
                    *(long*)(p + 16) = *(long*)pr;
                    *(long*)(p + 20) = *(long*)pr;
                    *(long*)(p + 24) = *(long*)pr;
                    *(long*)(p + 28) = *(long*)pr;
                    p += 32;
                }
                while ((l -= 32) > 31);
            }
            if (l > 0)
            {
                if ((l & 16) != 0)
                {
                    *(long*)p = *(long*)pr;
                    *(long*)(p + 4) = *(long*)pr;
                    *(long*)(p + 8) = *(long*)pr;
                    *(long*)(p + 12) = *(long*)pr;
                    p += 16;
                }
                if ((l & 8) != 0)
                {
                    *(long*)p = *(long*)pr;
                    *(long*)(p + 4) = *(long*)pr;
                    p += 8;
                }
                if ((l & 4) != 0)
                {
                    *(long*)p = *(long*)pr;
                    p += 4;
                }
                if ((l & 2) != 0)
                {
                    *(int*)p = *(int*)pr;
                    p += 2;
                }
                if ((l & 1) != 0)
                    *p = fill;
            }
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(int* p, int l, int fill)
        {
            if (l < 8)
            {
                switch (l)
                {
                    case 1:
                        *p = fill;
                        return;
                    case 2:
                        *p = fill;
                        p[1] = fill;
                        return;
                    case 3:
                        *p = fill;
                        p[1] = fill;
                        p[2] = fill;
                        return;
                    case 4:
                        *p = fill;
                        p[1] = fill;
                        *(long*)(p + 2) = *(long*)p;
                        return;
                    case 5:
                        *p = fill;
                        p[1] = fill;
                        *(long*)(p + 2) = *(long*)p;
                        p[4] = fill;
                        return;
                    case 6:
                        *p = fill;
                        p[1] = fill;
                        *(long*)(p + 2) = *(long*)p;
                        *(long*)(p + 4) = *(long*)p;
                        return;
                    case 7:
                        *p = fill;
                        p[1] = fill;
                        *(long*)(p + 2) = *(long*)p;
                        *(long*)(p + 4) = *(long*)p;
                        p[6] = fill;
                        return;
                }
                return;
            }
            int* pr = stackalloc int[2];
            *pr = fill;
            pr[1] = fill;
            if (l > 15)
            {
                do
                {
                    *(long*)p = *(long*)pr;
                    *(long*)(p + 2) = *(long*)pr;
                    *(long*)(p + 4) = *(long*)pr;
                    *(long*)(p + 6) = *(long*)pr;
                    *(long*)(p + 8) = *(long*)pr;
                    *(long*)(p + 10) = *(long*)pr;
                    *(long*)(p + 12) = *(long*)pr;
                    *(long*)(p + 14) = *(long*)pr;
                    p += 16;
                }
                while ((l -= 16) > 15);
            }
            if (l > 0)
            {
                if ((l & 8) != 0)
                {
                    *(long*)p = *(long*)pr;
                    *(long*)(p + 2) = *(long*)pr;
                    *(long*)(p + 4) = *(long*)pr;
                    *(long*)(p + 6) = *(long*)pr;
                    p += 8;
                }
                if ((l & 4) != 0)
                {
                    *(long*)p = *(long*)pr;
                    *(long*)(p + 2) = *(long*)pr;
                    p += 4;
                }
                if ((l & 2) != 0)
                {
                    *(long*)p = *(long*)pr;
                    p += 2;
                }
                if ((l & 1) != 0)
                    *p = fill;
            }
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(long* p, int l, long fill)
        {
            if (l > 7)
            {
                do
                {
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    p[4] = fill;
                    p[5] = fill;
                    p[6] = fill;
                    p[7] = fill;
                    p += 8;
                }
                while ((l -= 8) > 7);
            }
            switch (l) 
            {
                case 1:
                    *p = fill;
                    return;
                case 2:
                    *p = fill;
                    p[1] = fill;
                    return;
                case 3:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    return;
                case 4:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    return;
                case 5:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    p[4] = fill;
                    return;
                case 6:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    p[4] = fill;
                    p[5] = fill;
                    return;
                case 7:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    p[4] = fill;
                    p[5] = fill;
                    p[6] = fill;
                    return;
            }
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(PByte* p, int l, PByte fill)
        {
            if (l > 7)
            {
                do
                {
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    p[4] = fill;
                    p[5] = fill;
                    p[6] = fill;
                    p[7] = fill;
                    p += 8;
                }
                while ((l -= 8) > 7);
            }
            switch (l) 
            {
                case 1:
                    *p = fill;
                    return;
                case 2:
                    *p = fill;
                    p[1] = fill;
                    return;
                case 3:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    return;
                case 4:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    return;
                case 5:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    p[4] = fill;
                    return;
                case 6:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    p[4] = fill;
                    p[5] = fill;
                    return;
                case 7:
                    *p = fill;
                    p[1] = fill;
                    p[2] = fill;
                    p[3] = fill;
                    p[4] = fill;
                    p[5] = fill;
                    p[6] = fill;
                    return;
            }
        }


        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(sbyte* p, int l, sbyte fill)
        {
            Fill((byte*)p, l, (byte)fill);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(ushort* p, int l, ushort fill)
        {
            Fill((char*)p, l, (char)fill);
        }
        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(short* p, int l, short fill)
        {
            Fill((char*)p, l, (char)fill);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(uint* p, int l, uint fill)
        {
            Fill((int*)p, l, (int)fill);
        }
        
        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(ulong* p, int l, ulong fill)
        {
            Fill((long*)p, l, (long)fill);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Relleno.</param>
        public static void Fill(PChar* p, int l, PChar fill)
        {
            Fill((PByte*)p, l, *(PByte*)&fill);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(byte* p, byte* pr, int lr, int tot, bool noCopStack)
        {
            int a;
            if (lr < 9) 
            {
                switch (lr)
                {
                    case 1:
                        Fill(p, tot, *pr);
                        return;
                    case 2:
                        Fill((char*)p, tot, *(char*)pr);
                        return;
                    case 4:
                        Fill((int*)p, tot, *(int*)pr);
                        return;
                    case 8:
                        Fill((long*)p, tot, *(long*)pr);
                        return;
                    case 3:
                        a = 0;
                        do
                        {
                            *(short*)p = *(short*)pr;
                            p[2] = pr[2];
                            p += 3;
                        } while (++a < tot);
                        return;
                    case 5:
                        a = 0;
                        do
                        {
                            *(int*)p = *(int*)pr;
                            p[4] = pr[4];
                            p += 5;
                        } while (++a < tot);
                        return;
                    case 6:
                        a = 0;
                        do
                        {
                            *(int*)p = *(int*)pr;
                            *(short*)(p + 4) = *(short*)(pr + 4);
                            p += 6;
                        } while (++a < tot);
                        return;
                    case 7:
                        a = 0;
                        do
                        {
                            *(int*)p = *(int*)pr;
                            *(short*)(p + 4) = *(short*)(pr + 4);
                            p[6] = pr[6];
                            p += 7;
                        } while (++a < tot);
                        return;
                }
                return;
            }
            a = 0;
            if (!noCopStack && tot > 15 && pr > MaximumStack && lr < MaxStackBytes && &a>MinimumStack)
            {
                byte* pcops = stackalloc byte[lr];
                Copy(pr, pcops, lr);
                pr = pcops;
            } 
            switch (lr) 
            {
                case 9:
                    do
                    {
                        *(long*)p = *(long*)pr;
                        p[8] = pr[8];
                        p += 9;
                    } while (++a != tot);
                    return;
                case 10:
                    do
                    {
                        *(long*)p = *(long*)pr;
                        *(short*)(p + 8) = *(short*)(pr + 8);
                        p += 10;
                    } while (++a != tot);
                    return;
                case 11:
                    do
                    {
                        *(long*)p = *(long*)pr;
                        *(short*)(p + 8) = *(short*)(pr + 8);
                        p[10] = pr[10];
                        p += 11;
                    } while (++a != tot);
                    return;
                case 12:
                    do
                    {
                        *(long*)p = *(long*)pr;
                        *(int*)(p + 8) = *(int*)(pr + 8);
                        p += 12;
                    } while (++a != tot);
                    return;
                case 13:
                    do
                    {
                        *(long*)p = *(long*)pr;
                        *(int*)(p + 8) = *(int*)(pr + 8);
                        p[12] = pr[12];
                        p += 13;
                    } while (++a != tot);
                    return;
                case 14:
                    do
                    {
                        *(long*)p = *(long*)pr;
                        *(int*)(p + 8) = *(int*)(pr + 8);
                        *(short*)(p + 12) = *(short*)(pr + 12);
                        p += 14;
                    } while (++a != tot);
                    return;
                case 15:
                    do
                    {
                        *(long*)p = *(long*)pr;
                        *(int*)(p + 8) = *(int*)(pr + 8);
                        *(short*)(p + 12) = *(short*)(pr + 12);
                        p[14] = pr[14];
                        p += 15;
                    } while (++a != tot);
                    return;
                case 16:
                    do
                    {
                        *(long*)p = *(long*)pr;
                        *(long*)(p + 8) = *(long*)(pr + 8);
                        p += 16;
                    } while (++a != tot);
                    return;
            }
            do
            {
                Copy(pr, p, lr);
                p += lr;
            } while (++a != tot);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        public static void Fill(sbyte* p, sbyte* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr, tot, noCopStack);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(ushort* p, ushort* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr << 1, tot, noCopStack);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(short* p, short* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr << 1, tot, noCopStack);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(char* p, char* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr << 1, tot, noCopStack);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(uint* p, uint* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr << 2, tot, noCopStack);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(int* p, int* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr << 2, tot, noCopStack);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(ulong* p, ulong* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr << 3, tot, noCopStack);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(PByte* p, PByte* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr * MultPByte, tot, noCopStack);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(PChar* p, PChar* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr * MultPByte, tot, noCopStack);
        }

        /// <summary>
        /// Rellena memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="pr">Puntero de relleno (puede copiarlo al stack).</param>
        /// <param name="lr">Largo del puntero de relleno.</param>
        /// <param name="noCopStack">No intentar copiar al Stack el puntero de relleno.</param>
        /// <param name="tot">Total rellenos.</param>
        public static void Fill(long* p, long* pr, int lr, int tot, bool noCopStack)
        {
            Fill((byte*)p, (byte*)pr, lr << 3, tot, noCopStack);
        }

        #endregion

        #region Cut

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(byte* p, int l, int iCut, int lCut, bool clean)
        {
            p += iCut;
            Copy(p + lCut, p, l -= (iCut + lCut));
            if (clean)
                Clean(p + lCut, l);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(sbyte* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l, iCut, lCut, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(ushort* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l << 1, iCut << 1, lCut << 1, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(short* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l << 1, iCut << 1, lCut << 1, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(char* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l << 1, iCut << 1, lCut << 1, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(uint* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l << 2, iCut << 2, lCut << 2, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(int* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l << 2, iCut << 2, lCut << 2, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(ulong* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l << 3, iCut << 3, lCut << 3, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(PByte* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l * MultPByte, iCut * MultPByte, lCut * MultPByte, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(PChar* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l * MultPByte, iCut * MultPByte, lCut * MultPByte, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(long* p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l << 3, iCut << 3, lCut << 3, clean);
        }

        /// <summary>
        /// Elimina elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iCut">Inicio del corte (relativo a p).</param>
        /// <param name="lCut">Largo del corte.</param>
        /// <param name="clean">Si true, limpia con ceros los elementos excedentes.</param>
        public static void Cut(void** p, int l, int iCut, int lCut, bool clean)
        {
            Cut((byte*)p, l << DespVoid, iCut << DespVoid, lCut << DespVoid, clean);
        }

        #endregion

        #region Paste

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(byte* p, int l, int iInsert, int lInsert, bool clean)
        {
            Move(p + iInsert, p + lInsert, l - iInsert);
            if (clean)
                Clean(p + iInsert, lInsert);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(sbyte* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l, iInsert, lInsert, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(ushort* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l << 1, iInsert << 1, lInsert << 1, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(short* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l << 1, iInsert << 1, lInsert << 1, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(char* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l << 1, iInsert << 1, lInsert << 1, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(uint* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l << 2, iInsert << 2, lInsert << 2, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(int* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l << 2, iInsert << 2, lInsert << 2, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(ulong* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l << 3, iInsert << 3, lInsert << 3, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(PByte* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l * MultPByte, iInsert * MultPByte, lInsert * MultPByte, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(PChar* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l * MultPByte, iInsert * MultPByte, lInsert * MultPByte, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(long* p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l << 3, iInsert << 3, lInsert << 3, clean);
        }

        /// <summary>
        /// Inserta elementos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo total.</param>
        /// <param name="iInsert">Inicio de la insercin (relativo a p).</param>
        /// <param name="lInsert">Largo de la insercin.</param>
        /// <param name="clean">Si true, limpia con ceros la zona de insercin.</param>
        public static void Paste(void** p, int l, int iInsert, int lInsert, bool clean)
        {
            Paste((byte*)p, l << DespVoid, iInsert << DespVoid, lInsert << DespVoid, clean);
        }

        #endregion

    }
}
