using System;
using System.Security;
using System.Runtime.InteropServices;
using EnPalermo.Sys;
using EnPalermo.Txt;
using EnPalermo.Lit;

using System.Runtime.ConstrainedExecution;

namespace EnPalermo.Mem
{
    /// <summary>
    /// Representa una zona de memoria asignada en heap.
    /// </summary>
    [SuppressUnmanagedCodeSecurity]
    public unsafe sealed class Heap: CriticalFinalizerObject,IDisposable
    {

        #region campos y mtodos estticos.

        /// <summary>
        /// Obtiene el identificador de proceso para las funciones Heap.
        /// </summary>
        /// <returns>Identificador.</returns>
        [DllImport(Api.KERNEL32)]
        private extern static void* GetProcessHeap();

        /// <summary>
        /// Identificador de este proceso para las funciones Heap.
        /// </summary>
        private static readonly void* ProcHeap;


        /// <summary>
        /// Modifica opciones generales del Heap para este proceso.
        /// Requiere XP o superior y por el momento solo es para activar LFH. Ver SDK.
        /// </summary>
        /// <param name="heapHandle">Handle to the heap whose information is to be set. This handle is returned by either the HeapCreate or GetProcessHeap function.</param>
        /// <param name="alwaysZero">HeapCompatibilityInformation (0).</param>
        /// <param name="alwaysPointTo2">Heap information buffer. The format of this data depends on the value of the HeapInformationClass parameter.</param>
        /// <param name="always4">Size of the HeapInformation buffer, in bytes.</param>
        /// <returns>True Si modifico las opciones.</returns>
        [DllImport(Api.KERNEL32)]
        private static extern bool HeapSetInformation(void* heapHandle, int alwaysZero, uint* alwaysPointTo2, int always4);

        private static bool lfh;

        /// <summary>
        /// True si LFH (Low Fragmentation Heap) se encuentra activado.
        /// </summary>
        public static bool LFH 
        {
            get 
            {
                return lfh;
            }
        }

        /// <summary>
        /// Intenta activar el modo LFH (Low Fragmentation Heap).
        /// Solo XP o superior.
        /// </summary>
        /// <returns>True si LFH fue activado.</returns>
        public static bool EnableLFH()
        {
            if (!lfh && OsInfo.MinimumXP) 
            {
                uint tmp = 2;
                lfh = HeapSetInformation(ProcHeap, 0, &tmp, 4);
            }
            return lfh;
        }

        /// <summary>
        /// Intenta compactar el heap correspondiente a este proceso.
        /// </summary>
        /// <param name="hHeap">Identificador de proceso obtenido con GetProcessHeap.</param>
        /// <param name="flags">Opciones (siempre cero).</param>
        /// <returns>Tamao del bloque libre ms largo.</returns>
        [DllImport(Api.KERNEL32, SetLastError=true)]
        private static extern void* HeapCompact(void* hHeap, int flags);

        /// <summary>
        /// Intenta compactar los bloques libres del monton asignado a este proceso.
        /// </summary>
        /// <returns>Tamao mximo de bloque libre.</returns>
        public static long Compact() 
        {
            void* p = HeapCompact(ProcHeap, 0);
            if (p == null)
            {
                int err = System.Runtime.InteropServices.Marshal.GetLastWin32Error();
                if (err == 0) return 0;
                throw new ExternalException(Texts.Format(Errors.STR_FORMAT_HEAP_COMPACT, err.ToString()),err);
            }
            return (long)p;
        }

        /// <summary>
        /// Asigna memoria del heap.
        /// </summary>
        /// <param name="hHeap">Identificador de proceso obtenido con GetProcessHeap.</param>
        /// <param name="flags">Opciones (ver SDK). 8 para ZERO_MEMORY.</param>
        /// <param name="size">Tamao.</param>
        /// <returns>Puntero a la memoria asignada o null.</returns>
        [DllImport(Api.KERNEL32)]
        private static extern void* HeapAlloc(void* hHeap, int flags, void* size);

        /// <summary>
        /// Libera memoria del heap.
        /// </summary>
        /// <param name="hHeap">Identificador de proceso obtenido con GetProcessHeap.</param>
        /// <param name="flags">Opciones (ver SDK).</param>
        /// <param name="block">Puntero a liberar.</param>
        /// <returns>True si fue liberado correctamente.</returns>
        [DllImport(Api.KERNEL32)]
        private static extern bool HeapFree(void* hHeap, int flags, void* block);

        /// <summary>
        /// Cambia el tamao de la memoria asignada.
        /// </summary>
        /// <param name="hHeap">Identificador de proceso obtenido con GetProcessHeap.</param>
        /// <param name="flags">Opciones (ver SDK). 8 para ZERO_MEMORY.</param>
        /// <param name="block">Puntero.</param>
        /// <param name="size">Tamao.</param>
        /// <returns>Puntero reasignado (puede ser el mismo o no que el original).</returns>
        [DllImport(Api.KERNEL32)]
        private static extern void* HeapReAlloc(void* hHeap, int flags, void* block, void* size);


        /// <summary>
        /// Obtiene el tamao de un puntero asignado.
        /// </summary>
        /// <param name="hHeap">Identificador de proceso obtenido con GetProcessHeap.</param>
        /// <param name="flags">Opciones (ver SDK).</param>
        /// <param name="block">Puntero.</param>
        /// <returns>Tamao de memoria asignada.</returns>
        [DllImport(Api.KERNEL32)]
        private static extern void* HeapSize(void* hHeap, int flags, void* block);

        #endregion

        #region Constructores

        /// <summary>
        /// Constructor esttico (inicializa el ProcHeap).
        /// </summary>
        static Heap()
        {
            lfh = false;
            ProcHeap = GetProcessHeap();
            if (ProcHeap == null)
                throw new ExternalException(Errors.PROCHEAP_ZERO);
        }


        private Heap(void* size, bool clean) 
        {
            pointer = HeapAlloc(ProcHeap, clean ? 8 : 0, size);
            if (pointer == null)
                throw new OutOfMemoryException(Texts.Format(Errors.STR_FORMAT_OUT_OF_MEMORY, ((long)size).ToString()));
        }

        /// <summary>
        /// Destructor.
        /// </summary>
        ~Heap() 
        {
            if (pointer != null)
            {
                HeapFree(ProcHeap, 0, pointer);
                pointer = null;
            }
        }

        /// <summary>
        /// Inicializa un heap en base al tamao especificado.
        /// </summary>
        /// <param name="size">Tamao (bytes).</param>
        /// <param name="clean">Si true, rellena el heap con ceros.</param>
        public Heap(int size, bool clean) : this((void*)size, clean) { }
        /// <summary>
        /// Inicializa un heap en base al tamao especificado.
        /// </summary>
        /// <param name="size">Tamao (bytes).</param>
        /// <param name="clean">Si true, rellena el heap con ceros.</param>
        public Heap(uint size, bool clean):this((void*)size,clean) { }
        /// <summary>
        /// Inicializa un heap en base al tamao especificado.
        /// </summary>
        /// <param name="size">Tamao (bytes).</param>
        /// <param name="clean">Si true, rellena el heap con ceros.</param>
        public Heap(long size, bool clean) : this(General.FrameWork64 ? (void*)size : (void*)((int)size), clean) { }
        
        /// <summary>
        /// Inicializa un heap en base al tamao especificado.
        /// </summary>
        /// <param name="size">Tamao (bytes).</param>
        /// <param name="clean">Si true, rellena el heap con ceros.</param>
        public Heap(ulong size, bool clean) : this(General.FrameWork64 ? (void*)size : (void*)((uint)size), clean) { }

        #endregion

        #region Campos y Propiedades

        /// <summary>
        /// Puntero en Heap.
        /// </summary>
        private void* pointer;

        /// <summary>
        /// Obtiene el puntero actual.
        /// </summary>
        public void* Pointer
        {
            get
            {
                return pointer;
            }
        }

        /// <summary>
        /// Obtiene el tamao asignado al puntero actual.
        /// Si hay error, retorna -1.
        /// </summary>
        public long Size 
        {
            get 
            {
                if (General.FrameWork64) return (long)((ulong)HeapSize(ProcHeap, 0, pointer));
                return (long)((int)HeapSize(ProcHeap, 0, pointer));
            }
        }

        #endregion

        #region Metodos

        private void* changeSize(void* size, bool clean) 
        {
            pointer = HeapReAlloc(ProcHeap, clean ? 8 : 0, pointer, size);
            if (pointer == null) 
                throw new OutOfMemoryException(Texts.Format(Errors.STR_FORMAT_OUT_OF_MEMORY,((long)size).ToString()));
            return pointer;
        }

        /// <summary>
        /// Cambia el tamao asignado al puntero actual, reubicando memoria si es necesario.
        /// </summary>
        /// <param name="size">Nuevo tamao.</param>
        /// <param name="clean">Si el nuevo tamao es mayor al actual, limpia con ceros la zona agregada.</param>
        /// <returns>Nuevo puntero (puede ser diferente al actual si reubico memoria).</returns>
        public void* ChangeSize(int size, bool clean)
        {
            return changeSize((void*)size, clean);
        }

        /// <summary>
        /// Cambia el tamao asignado al puntero actual, reubicando memoria si es necesario.
        /// </summary>
        /// <param name="size">Nuevo tamao.</param>
        /// <param name="clean">Si el nuevo tamao es mayor al actual, limpia con ceros la zona agregada.</param>
        /// <returns>Nuevo puntero (puede ser diferente al actual si reubico memoria).</returns>
        public void* ChangeSize(uint size, bool clean)
        {
            return changeSize((void*)size, clean);
        }

        /// <summary>
        /// Cambia el tamao asignado al puntero actual, reubicando memoria si es necesario.
        /// </summary>
        /// <param name="size">Nuevo tamao.</param>
        /// <param name="clean">Si el nuevo tamao es mayor al actual, limpia con ceros la zona agregada.</param>
        /// <returns>Nuevo puntero (puede ser diferente al actual si reubico memoria).</returns>
        public void* ChangeSize(long size, bool clean) 
        {
            return changeSize(General.FrameWork64 ? (void*)size : (void*)((int)size), clean);
        }

        /// <summary>
        /// Cambia el tamao asignado al puntero actual, reubicando memoria si es necesario.
        /// </summary>
        /// <param name="size">Nuevo tamao.</param>
        /// <param name="clean">Si el nuevo tamao es mayor al actual, limpia con ceros la zona agregada.</param>
        /// <returns>Nuevo puntero (puede ser diferente al actual si reubico memoria).</returns>
        public void* ChangeSize(ulong size, bool clean)
        {
            return changeSize(General.FrameWork64 ? (void*)size : (void*)((uint)size), clean);
        }

        /// <summary>
        /// Libera el puntero actual.
        /// </summary>
        public void Dispose()
        {
            if (pointer != null) 
            {
                HeapFree(ProcHeap, 0, pointer);
                pointer = null;
            }
            System.GC.SuppressFinalize(this);
        }

        #endregion
        
    }
}
