using System;

namespace EnPalermo.Mem
{
    /// <summary>
    /// Funciones estticas de tipo inseguro (numricas).
    /// </summary>
    public unsafe static class Numerics
    {
        /// <summary>
        /// Invierte los bytes del nmero de entrada y devuelve el resultado.
        /// </summary>
        /// <param name="n">Nmero a convertir.</param>
        /// <returns>Nmero invertido.</returns>
        public static short Invert(short n)
        {
            byte* p = (byte*)&n;
            byte b0 = *p;
            *p = p[1];
            p[1] = b0;
            return n;
        }

        /// <summary>
        /// Invierte los bytes del nmero de entrada y devuelve el resultado.
        /// </summary>
        /// <param name="n">Nmero a convertir.</param>
        /// <returns>Nmero invertido.</returns>
        public static ushort Invert(ushort n)
        {
            byte* p = (byte*)&n;
            byte b0 = *p;
            *p = p[1];
            p[1] = b0;
            return n;
        }

        /// <summary>
        /// Obtiene los bytes invertidos del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] BytesInv(short n)
        {
            byte* pn = (byte*)&n;
            byte[] r = new byte[2];
            r[0] = pn[1];
            r[1] = *pn;
            return r;
        }

        /// <summary>
        /// Obtiene los bytes invertidos del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] BytesInv(ushort n)
        {
            byte* pn = (byte*)&n;
            byte[] r = new byte[2];
            r[0] = pn[1];
            r[1] = *pn;
            return r;
        }

        /// <summary>
        /// Obtiene los bytes del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] Bytes(short n)
        {
            byte[] r = new byte[2];
            fixed (byte* p = r)
            {
                *((short*)p) = n;
            }
            return r;
        }

        /// <summary>
        /// Obtiene los bytes del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] Bytes(ushort n)
        {
            byte[] r = new byte[2];
            fixed (byte* p = r)
            {
                *((ushort*)p) = n;
            }
            return r;
        }

        /// <summary>
        /// Invierte los bytes del nmero de entrada y devuelve el resultado.
        /// </summary>
        /// <param name="n">Nmero a convertir.</param>
        /// <returns>Nmero invertido.</returns>
        public static int Invert(int n)
        {
            byte* p = (byte*)&n;
            byte b0 = *p;
            byte b1 = p[1];
            *p = p[3];
            p[1] = p[2];
            p[2] = b1;
            p[3] = b0;
            return n;
        }

        /// <summary>
        /// Invierte los bytes del nmero de entrada y devuelve el resultado.
        /// </summary>
        /// <param name="n">Nmero a convertir.</param>
        /// <returns>Nmero invertido.</returns>
        public static uint Invert(uint n)
        {
            byte* p = (byte*)&n;
            byte b0 = *p;
            byte b1 = p[1];
            *p = p[3];
            p[1] = p[2];
            p[2] = b1;
            p[3] = b0;
            return n;
        }

        /// <summary>
        /// Invierte los bytes del nmero de entrada y devuelve el resultado.
        /// </summary>
        /// <param name="n">Nmero a convertir.</param>
        /// <returns>Nmero invertido.</returns>
        public static long Invert(long n)
        {
            byte* p = (byte*)&n;
            byte b0 = *p;
            byte b1 = p[1];
            byte b2 = p[2];
            byte b3 = p[3];
            *p = p[7];
            p[1] = p[6];
            p[2] = p[5];
            p[3] = p[4];
            p[4] = b3;
            p[5] = b2;
            p[6] = b1;
            p[7] = b0;
            return n;
        }


        /// <summary>
        /// Invierte los bytes del nmero de entrada y devuelve el resultado.
        /// </summary>
        /// <param name="n">Nmero a convertir.</param>
        /// <returns>Nmero invertido.</returns>
        public static ulong Invert(ulong n)
        {
            byte* p = (byte*)&n;
            byte b0 = *p;
            byte b1 = p[1];
            byte b2 = p[2];
            byte b3 = p[3];
            *p = p[7];
            p[1] = p[6];
            p[2] = p[5];
            p[3] = p[4];
            p[4] = b3;
            p[5] = b2;
            p[6] = b1;
            p[7] = b0;
            return n;
        }

        /// <summary>
        /// Obtiene los bytes invertidos del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] BytesInv(int n)
        {
            byte* pn = (byte*)&n;
            byte[] r = new byte[4];
            r[0] = pn[3];
            r[1] = pn[2];
            r[2] = pn[1];
            r[3] = *pn;
            return r;
        }

        /// <summary>
        /// Obtiene los bytes invertidos del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] BytesInv(uint n)
        {
            byte* pn = (byte*)&n;
            byte[] r = new byte[4];
            r[0] = pn[3];
            r[1] = pn[2];
            r[2] = pn[1];
            r[3] = *pn;
            return r;
        }

        /// <summary>
        /// Obtiene los bytes invertidos del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] BytesInv(long n)
        {
            byte* pn = (byte*)&n;
            byte[] r = new byte[8];
            r[0] = pn[7];
            r[1] = pn[6];
            r[2] = pn[5];
            r[3] = pn[4];
            r[4] = pn[3];
            r[5] = pn[2];
            r[6] = pn[1];
            r[7] = *pn;
            return r;
        }

        /// <summary>
        /// Obtiene los bytes invertidos del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] BytesInv(ulong n)
        {
            byte* pn = (byte*)&n;
            byte[] r = new byte[8];
            r[0] = pn[7];
            r[1] = pn[6];
            r[2] = pn[5];
            r[3] = pn[4];
            r[4] = pn[3];
            r[5] = pn[2];
            r[6] = pn[1];
            r[7] = *pn;
            return r;
        }

        /// <summary>
        /// Obtiene los bytes del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] Bytes(int n)
        {
            byte[] r = new byte[4];
            fixed (byte* p = r)
            {
                *((int*)p) = n;
            }
            return r;
        }

        /// <summary>
        /// Obtiene los bytes del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] Bytes(uint n)
        {
            byte[] r = new byte[4];
            fixed (byte* p = r)
            {
                *((uint*)p) = n;
            }
            return r;
        }

        /// <summary>
        /// Obtiene los bytes del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] Bytes(long n)
        {
            byte[] r = new byte[8];
            fixed (byte* p = r)
            {
                *((long*)p) = n;
            }
            return r;
        }

        /// <summary>
        /// Obtiene los bytes del nmero de entrada.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <returns>Array de Bytes.</returns>
        public static byte[] Bytes(ulong n)
        {
            byte[] r = new byte[8];
            fixed (byte* p = r)
            {
                *((ulong*)p) = n;
            }
            return r;
        }


    }
}
