using System;
using System.Runtime.InteropServices;
using EnPalermo.Txt;
using EnPalermo.Lit;

namespace EnPalermo.Mem
{
	/// <summary>
	/// Representa una zona de memoria basada en un puntero Byte.
	/// Tambin provee un enumerador simple que se autoconsume.
	/// </summary>
    [StructLayout(LayoutKind.Sequential)]
	public unsafe struct PByte:IComparable<PByte>,IEquatable<PByte>
    {


		#region Campos

		/// <summary>
		/// Puntero.
		/// </summary>
		public byte* Pointer;

		/// <summary>
		/// Largo.
		/// </summary>
		public int Length;

        /// <summary>
        /// PByte cero.
        /// </summary>
        public static readonly PByte Zero=new PByte();

		#endregion

		#region Constructores

		/// <summary>
		/// Inicializa la estructura.
		/// </summary>
		/// <param name="p">Puntero.</param>
		/// <param name="l">Largo.</param>
		public PByte(byte * p, int l)
		{
			Pointer=p;Length=l;
		}
        
		/// <summary>
		/// Inicializa la estructura en base a otro PByte.
		/// </summary>
		/// <param name="pByte">PByte.</param>
		/// <param name="i">Inicio.</param>
		/// <param name="l">Largo.</param>
		public PByte(PByte pByte,int i, int l)
		{
			Pointer=pByte.Pointer+i;Length=l;
		}

        /// <summary>
        /// Remueve comillas y efectua Trim con el filtro de todos los carcteres en blanco a un puntero de cadena.
        /// </summary>
        /// <param name="p">Puntero de cadena.</param>
        /// <param name="l">Largo del puntero.</param>
        /// <param name="afterTrimR">Luego del procesamiento de comillas, efecta trim derecho.</param>
        /// <param name="afterTrimL">Luego del procesamiento de comillas, efecta trim izquierdo.</param>
        /// <param name="mode">Opciones de eliminacin de comillas.</param>
        /// <param name="lat1">
        /// Adems de las comillas estndar (34 y 39) incluye comillas Latin1 (171 y 187).
        /// En el trim, incluye el carcter 160.
        /// </param>
        public PByte(byte * p, int l,RemoveQuotesMode mode, bool afterTrimL, bool afterTrimR, bool lat1)
        {
            start:
                if (l<2)
                {
                    if (l==1 && (afterTrimL || afterTrimR))
                    {
                        switch(*p)
                        {
                            case 32:
                            case 9:
                            case 10:
                            case 13:
                            case 0:
                            case 11:
                            case 12:
                                l=0;
                                break;
                        }
                    }
                    Pointer=p;
                    Length=l;
                    return;
                }
            PByte ret;            
            switch(mode)
            {
                case RemoveQuotesMode.RemoveOne:
                case RemoveQuotesMode.RemoveAll:
                    ret=new PByte(p,l,true,true,lat1);
                    if (ret.Length<2)
                        goto end;
                    break;
                case RemoveQuotesMode.RemoveOneNoTrim:
                case RemoveQuotesMode.RemoveAllNoTrim:
                    ret=new PByte(p,l);
                    break;
                default:
                    if (afterTrimL || afterTrimR)
                    {
                        ret=new PByte(p,l,afterTrimL,afterTrimR,lat1);
                        goto end;
                    }
                    Pointer=p;
                    Length=l;
                    return;
            }
            switch(*ret.Pointer)
            {
                case 34:
                    if (ret.Pointer[ret.Length-1]==34) 
                        break;
                    goto default;
                case 39:
                    if (ret.Pointer[ret.Length-1]==39) 
                        break;
                    goto default;

                case 171:
                    if (lat1 && ret.Pointer[ret.Length-1]==187) 
                        break;
                    goto default;
                default:
                    if ((afterTrimL || afterTrimR) && ((int)mode&4)==0)
                        ret=new PByte(ret.Pointer,ret.Length,afterTrimL,afterTrimR,lat1);
                    goto end;
            }
            if (((int)mode&2)!=0)
            {
                p=ret.Pointer+1;
                l=ret.Length-2;
                goto start;
            }
            ret.Pointer++;
            ret.Length-=2;
            if (ret.Length!=0 && (afterTrimL || afterTrimR))
                ret=new PByte(ret.Pointer,ret.Length,afterTrimL,afterTrimR,lat1);
           
            end:
                Pointer=ret.Pointer;
            Length=ret.Length;
        }

		/// <summary>
		/// Efectua Trim con el filtro de todos los carcteres en blanco a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero de cadena.</param>
		/// <param name="l">Largo del puntero.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 superiores.</param>
		public PByte(byte * p, int l,bool left, bool right, bool lat1)
		{
            if (l>0)
            {
                if (left)
                {
                reswitch1:
                    switch(*p)
                    {
                        case 32:
                        case 9:
                        case 10:
                        case 13:
                        case 0:
                        case 11:
                        case 12:
                            p++;
                            if (--l!=0)
                                goto reswitch1;
                            goto end;
                        case 160:
                            if (lat1)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }
                            break;
                    }
                    if (l==1)
                        goto end;
                }
                if (right)
                {
                reswitch2:
                    switch(p[l-1])
                    {
                        case 32:
                        case 9:
                        case 10:
                        case 13:
                        case 0:
                        case 11:
                        case 12:
                            if (--l!=0)
                                goto reswitch2;
                            break;
                        case 160:
                            if (!lat1 || --l==0)
                                break;
                            goto reswitch2;                            
                    }                
                }
            }
            end:
                Pointer=p;
            Length=l;
		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero de cadena.</param>
		/// <param name="l">Largo del puntero.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
		/// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
		/// <param name="tab">Incluye en filtro el taburador (\t).</param>
		public PByte(byte * p, int l,bool left, bool right, bool sp,bool lat1, bool tab, bool crlf, bool control)
		{
            if (l>0)
            {
                if (left)
                {
                reswitch1:
                    switch(*p)
                    {
                        case 32:
                            if (sp)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }
                            break;

                        case 9:
                            if (tab)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }
                            break;

                        case 10:
                        case 13:
                            if (crlf)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }
                            break;

                        case 0:
                        case 11:
                        case 12:
                            if (control)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }
                            break;
                        case 160:
                            if (lat1)
                            {
                                p++;
                                if (--l!=0)
                                    goto reswitch1;
                                goto end;
                            }
                            break;
                    }
                    if (l==1)
                        goto end;
                }
                if (right)
                {
                reswitch2:
                    switch(p[l-1])
                    {
                        case 32:
                            if (!sp || --l==0)
                                break;
                            goto reswitch2;                            
                        case 9:
                            if (!tab || --l==0)
                                break;
                            goto reswitch2;                            
                        case 10:
                        case 13:
                            if (!crlf || --l==0)
                                break;
                            goto reswitch2;                            
                        case 0:
                        case 11:
                        case 12:
                            if (!control || --l==0)
                                break;
                            goto reswitch2;                            
                        case 160:
                            if (!lat1 || --l==0)
                                break;
                            goto reswitch2;                            
                    }                
                }
            }
            end:
                Pointer=p;
            Length=l;


		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero de cadena.</param>
		/// <param name="l">Largo del puntero.</param>
		/// <param name="pTrim">Puntero de carcteres de filtro.</param>
		/// <param name="lTrim">Largo del puntero de carcteres de filtro.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		public PByte (byte * p, int l,byte * pTrim, int lTrim, bool left, bool right)
		{
			if(l>0)
			{
				if(left)
				{
					int a=0;
					while(Comp.Exist(pTrim,lTrim,p[a]) && ++a!=l);
					if (a==l) l=0;
					else
					{
						p+=a;
						l-=a;
						if(right && l!=1)
						{
							while(Comp.Exist(pTrim,lTrim,p[--l]) && l!=0);
							l++;
						}
					}
				}
				else if (right)
				{
					while(Comp.Exist(pTrim,lTrim,p[--l]) && l!=-1);
					l++;
				}
			}
			Pointer=p;
			Length=l;
		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="p">Puntero de cadena.</param>
		/// <param name="l">Largo del puntero.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <param name="cTrim">Carcteres para Trim.</param>
		public PByte (byte * p, int l,bool left, bool right, byte[] cTrim)
		{
			int ltrim=cTrim.Length;
			if (ltrim!=0)
			{
				fixed(byte* ptrim=cTrim)
				{
					PByte pc=new PByte(p,l,ptrim,ltrim,left,right);
					Pointer=pc.Pointer;
					Length=pc.Length;
				} 
			}
			else
			{
				Pointer=p;
				Length=l;
			}
		}



		#endregion

		#region Enumeradores

		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// actual y proximo separador encontrado.
		/// </summary>
		/// <param name="c">Separador.</param>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PByte Foward(byte c, bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			if (ignoreEmpty)
			{
				while(Pointer[a]==c && ++a!=Length);
				if(a==Length)
				{
					Pointer=null;
					Length=0;
					return this;
				}
				if (a!=0)
				{
					Pointer+=a;
					Length-=a;
				}
				if (Length==1)
				{
					ret=this;
					Pointer=null;
					Length=0;
					return ret;
				}
				a=1;
			}
			while(Pointer[a]!=c && ++a!=Length);

			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// del byte siguiente al ltimo separador del bloque y el final del bloque.
		/// </summary>
		/// <param name="c">Separador.</param>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PByte Backward(byte c, bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			if (ignoreEmpty)
			{
				while(--a!=-1 && Pointer[a]==c);
				if(a<1)
				{
					if (a==-1)
					{
						Pointer=null;
						Length=0;
						return this;
					}
					ret=new PByte(Pointer,1);
					Pointer=null;
					Length=0;
					return ret;
				}
				Length=a+1;
			}
			while(--a!=-1 && Pointer[a]!=c);
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}

		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// actual y proximo separador encontrado.
		/// </summary>
		/// <param name="p">Puntero de separadores.</param>
		/// <param name="l">Largo del puntero de separadores.</param>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PByte Foward(byte * p, int l, bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			if (ignoreEmpty)
			{
				while(Comp.Exist(p,l,Pointer[a]) && ++a!=Length);
				if(a==Length)
				{
					Pointer=null;
					Length=0;
					return this;
				}
				if (a!=0)
				{
					Pointer+=a;
					Length-=a;
				}
				if (Length==1)
				{
					ret=this;
					Pointer=null;
					Length=0;
					return ret;
				}
				a=1;
			}
			while(!Comp.Exist(p,l,Pointer[a]) && ++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// del char siguiente al ltimo separador del bloque y el final del bloque.
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <param name="p">Puntero de separadores.</param>
		/// <param name="l">Largo del puntero de separadores.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PByte Backward(byte * p, int l, bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			if (ignoreEmpty)
			{
				while(--a!=-1 && Comp.Exist(p,l,Pointer[a]));
				if(a<1)
				{
					if (a==-1)
					{
						Pointer=null;
						Length=0;
						return this;
					}
					ret=new PByte(Pointer,1);
					Pointer=null;
					Length=0;
					return ret;
				}
				Length=a+1;
			}
			while(--a!=-1 && !Comp.Exist(p,l,Pointer[a]));
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}

        /// <summary>
        /// Enumerador que devuelve el PByte del bloque entre la posicin
        /// actual y proximo CR y/o LF encontrado.
        /// Se efectua trim derecho a cada bloque, y, si este es vacio, se ignora.
        /// </summary>
        /// <param name="lat1">
        /// Incluye en el filtro de trim derecho a los carcteres en blanco Latin1 superiores.
        /// </param>
        /// <param name="doubleLine">Es considerado parrafo solo cuando es seguido de un segundo
        ///  salto de lnea. Los bloques no considerados parrafos se unen.
        ///  </param>
        /// <returns>
        /// - Si el separador es detectado y es considerado parrafo (luego de trim derecho a los bloques):
        ///   PByte con el parrafo.
        ///   El puntero se setea al inicio del siguiente parrafo y Length se setea en el resto del PByte 
        ///   (que puede ser cero).
        /// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
        ///   Pchar actual luego de trim derecho. El puntero se setea en null y Length se setea en 0.
        /// - Si el puntero es nulo:
        ///   PByte con puntero nulo.
        ///   El puntero nulo o Length cero permite indicar que el enumerador se ha consumido.
        /// </returns>
        public PByte FowardParagraph(bool doubleLine, bool lat1)
        {
            if(Pointer==null)
                return this;
            PByte ret;
            if(Length<1) 
            {
                ret=this;
                Pointer=null;
                Length=0;
                return ret;
            }

            int a,rest=0;
            otra:
            a=0;
            do
            {
                if (Pointer[a]==13)
                {
                    if (a!=Length-1 && Pointer[a+1]==10)
                        rest=2;
                    else
                        rest=1;
                    break;
                }
                else if (Pointer[a]==10)
                {
                    rest=1;
                    break;
                }
            }while(++a!=Length);
            ret=new PByte(Pointer,a,false,true, lat1);
            if (a==Length)
            {
                Length=0;
                Pointer=null;
                return ret;
            }
            Length-=(a+=rest);
            if (Length==0)
            {
                Pointer=null;
                return ret;
            }
            Pointer+=a;
            if (ret.Length==0)
                goto otra;
            if (!doubleLine)
                return ret;
            PByte ret2;
            otra2:
            a=0;
            do
            {
                if (Pointer[a]==13)
                {
                    if (a!=Length-1 && Pointer[a+1]==10)
                        rest=2;
                    else
                        rest=1;
                    break;
                }
                else if (Pointer[a]==10)
                {
                    rest=1;
                    break;
                }
            }while(++a!=Length);
            ret2=new PByte(Pointer,a,false,true,lat1);
            if (a==Length)
            {
                Length=0;
                Pointer=null;
                goto fin;

            }
            Length-=(a+=rest);
            if (Length==0)
            {
                Pointer=null;
                goto fin;
            }
            Pointer+=a;
            if (ret2.Length==0)
                return ret;
            ret.Length=(int)(ret2.Pointer-ret.Pointer)+ret2.Length;
            goto otra2;
            fin:
                if (ret2.Length!=0)
                    ret.Length=(int)(ret2.Pointer-ret.Pointer)+ret2.Length;
            return ret;
        }


        /// <summary>
        /// Enumerador que devuelve el PByte del bloque considerado como una parte de nombre de archivo
        /// e ignorando los separadores consecutivos.
        /// Se considera un bloque no vacio a las cadenas ".", ".." y todas las que contengan carcteres
        /// luego de trim derecho de espacios y puntos.
        /// Para obtener el PByte con un nombre de archivo verificado, usar la funcin
        /// Sys.FileHelpers.GetFileNameBlock().
        /// </summary>
        /// <param name="lat1">Incluye en el trim derecho a los carcteres altos Latin1.</param>
        /// <returns>
        /// - Si el separador es detectado:
        ///   PByte con Length==Ubicacin separador.
        ///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
        /// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
        ///   PByte actual. El puntero se setea en null y Length se setea en 0.
        /// - Si el puntero es nulo:
        ///   PByte con puntero nulo.
        ///   El puntero nulo permite indicar que el enumerador se ha consumido.
        /// </returns>
        public PByte FowardFNPart(bool lat1)
        {
            if(Pointer==null) 
                return this;
            int a, l;
            byte*p;
            otra:
                if(Length<1) 
                {
                    PByte ret=this;
                    Pointer=null;
                    Length=0;
                    return ret;
                }
            if (*Pointer==92 || *Pointer==47)
            {
                Pointer++;
                Length--;
                goto otra;
            }
            a=0;
            l=Length;
            while(++a!=l && Pointer[a]!=92 && Pointer[a]!=47);
            p=Pointer;
            if (a!=l)
            {
                Pointer+=a+1;
                Length-=a+1;
            }
            else
            {
                Pointer+=a;
                Length=0;
            }
            while(--a!=-1 && ((p[a]==32) || (p[a]==46 && a!=0 && (a!=1 || *p!=46)) || (lat1 && p[a]==160)));
            if (a==-1)
                goto otra;
            return new PByte(p,a+1);
        }


		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// actual y proximo CR y/o LF encontrado.
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PByte FowardCrLf(bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;

			if (ignoreEmpty)
			{
				while((Pointer[a]==13 || Pointer[a]==10) && ++a!=Length);
				if(a==Length)
				{
					Pointer=null;
					Length=0;
					return this;
				}
				if (a!=0)
				{
					Pointer+=a;
					Length-=a;
				}
				if (Length==1)
				{
					ret=this;
					Pointer=null;
					Length=0;
					return ret;
				}
				a=1;
			}
			do
			{
				switch(Pointer[a])
				{
					case 13:
						ret=new PByte(Pointer,a);
						if(++a!=Length && Pointer[a]==10) ++a;
						Pointer+=a;
						Length-=a;
						return ret;
					case 10:
						ret=new PByte(Pointer,a);
						Pointer+=++a;
						Length-=a;
						return ret;
				}
			}while(++a<Length);
			ret=this;
			Pointer=null;
			Length=0;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// del char siguiente al ltimo CR y/o LF del bloque y el final del bloque.
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PByte BackwardCrLf(bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;

			if (ignoreEmpty)
			{
				while(--a!=-1 && (Pointer[a]==10 || Pointer[a]==13));
				if(a<1)
				{
					if (a==-1)
					{
						Pointer=null;
						Length=0;
						return this;
					}
					ret=new PByte(Pointer,1);
					Pointer=null;
					Length=0;
					return ret;
				}
				Length=a+1;
			}

			while(--a!=-1)
			{
				switch(Pointer[a])
				{
					case 10:
						ret=new PByte(Pointer+(a+1),Length-a-1);
						if(a!=0 && Pointer[a-1]==13) --a;
						Length=a;
						return ret;
					case 13:
						ret=new PByte(Pointer+(a+1),Length-a-1);
						Length=a;
						return ret;
				}
			}
			ret=this;
			Pointer=null;
			Length=0;
			return ret;
		}



		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// actual y proximo caracter en blanco encontrado.
		/// </summary>
		/// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
		/// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
		/// <param name="tab">Incluye en filtro el taburador (\t).</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		public PByte FowardBlank(bool sp,bool lat1, bool tab, bool crlf, bool control)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			do
			{
				switch(Pointer[a])
				{
					case 32:
						if (sp) break;
						continue;
					case 9:
						if (tab) break;
						continue;
					case 10:
					case 13:
						if (crlf) break;
						continue;
					case 160:
						if (lat1) break;
						continue;
					case 0:
					case 11:
					case 12:
						if (control) break;
						continue;
					default:
						continue;
				}
				break;
			}while(++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// del byte siguiente al ltimo caracter en blanco del bloque y el final del bloque.
		/// </summary>
		/// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
		/// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
		/// <param name="tab">Incluye en filtro el taburador (\t).</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		public PByte BackwardBlank(bool sp,bool lat1, bool tab, bool crlf, bool control)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			while(--a!=-1)
			{
				switch(Pointer[a])
				{
					case 32:
						if (sp) break;
						continue;
					case 9:
						if (tab) break;
						continue;
					case 10:
					case 13:
						if (crlf) break;
						continue;
					case 160:
						if (lat1) break;
						continue;
					case 0:
					case 11:
					case 12:
						if (control) break;
						continue;
					default:
						continue;
				}
				break;
			}
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// actual y proximo caracter en blanco encontrado, incluyendo los Latin1 superiores.
		/// </summary>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		public PByte FowardBlank()
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			do
			{
				switch(Pointer[a])
				{
					case 32:
					case 9:
					case 10:
					case 13:
					case 160:
					case 0:
					case 11:
					case 12:
						break;
					default:
						continue;
				}
				break;
			}while(++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}

		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// actual y proximo caracter en blanco encontrado (Incluyendo los Latin1 superiores) + el caracter que se especifique.
		/// </summary>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		/// <param name="c">Caracter adicional a los carcteres en blanco.</param>
		public PByte AvanzarBlancoByte(byte c)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			do
			{
				switch(Pointer[a])
				{
					case 32:
					case 9:
					case 10:
					case 13:
					case 160:
					case 0:
					case 11:
					case 12:
						break;
					default:
						if (Pointer[a]==c) break;
						continue;
				}
				break;
			}while(++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}


		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// del byte siguiente al ltimo caracter en blanco (Incluyendo los Latin1 superiores) del bloque y el final del bloque.
		/// </summary>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con el puntero situado en Ubicacin separador + 1 y el Length=Length-Ubicacin separador-1.
		///   El puntero no se modifica y el Length se setea en Ubicacin separador.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// </returns>
		public PByte BackwardBlank()
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			while(--a!=-1)
			{
				switch(Pointer[a])
				{
					case 32:
					case 9:
					case 10:
					case 13:
					case 160:
					case 0:
					case 11:
					case 12:
						break;
					default:
						continue;
				}
				break;
			}
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}

		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// actual y proximo separador encontrado (determinado por cualquier elemento no-numerico).
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PByte FowardNumber(bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=0;
			if (ignoreEmpty)
			{
				while((Pointer[a]>57 || Pointer[a]<48) && ++a!=Length);
				if(a==Length)
				{
					Pointer=null;
					Length=0;
					return this;
				}
				if(a!=0)
				{
					Pointer+=a;
					Length-=a;
				}
				if (Length==1)
				{
					ret=this;
					Pointer=null;
					Length=0;
					return ret;
				}
				a=1;
			}
			while(Pointer[a]<58 && Pointer[a]>47 && ++a!=Length);
			if(a==Length)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer,a);
			Pointer+=++a;
			Length-=a;
			return ret;
		}

		/// <summary>
		/// Enumerador que devuelve el PByte del bloque entre la posicin
		/// del char siguiente al ltimo separador encontrado (determinado por cualquier elemento no-numerico).
		/// y el final del bloque.
		/// </summary>
		/// <param name="ignoreEmpty">Mltiples separadores son considerados uno solo.</param>
		/// <returns>
		/// - Si el separador es detectado:
		///   PByte con Length==Ubicacin separador.
		///   El puntero se setea en Pointer+Ubicacin separador+1 y Length se setea en Length-Ubicacin separador-1.
		/// - Si el separador no es detectado o el largo actual es 0 (con puntero no-nulo):
		///   PByte actual. El puntero se setea en null y Length se setea en 0.
		/// - Si el puntero es nulo:
		///   PByte con puntero nulo.
		///   El puntero nulo permite indicar que el enumerador se ha consumido.
		/// - Si ignoreEmpty es true, solo retornar punteros con Length mayor a 0,
		///   siendo el ltimo retorno de largo 0 y puntero null.
		/// </returns>
		public PByte BackwardNumber(bool ignoreEmpty)
		{
			if(Pointer==null) return this;
			PByte ret;
			if(Length<1) 
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			int a=Length;
			if (ignoreEmpty)
			{
				while(--a!=-1 && (Pointer[a]>57 || Pointer[a]<48));
				if(a<1)
				{
					if (a==-1)
					{
						Pointer=null;
						Length=0;
						return this;
					}
					ret=new PByte(Pointer,1);
					Pointer=null;
					Length=0;
					return ret;
				}
				Length=a+1;
			}
			while(--a!=-1 && Pointer[a]<58 && Pointer[a]>47);
			if(a==-1)
			{
				ret=this;
				Pointer=null;
				Length=0;
				return ret;
			}
			ret=new PByte(Pointer+(a+1),Length-a-1);
			Length=a;
			return ret;
		}


		#endregion

		#region Metodos

		
		/// <summary>
		/// Efectua trim de carcteres en blanco a derecha e izquierda.
		/// </summary>
		/// <returns>Pbyte en base a trim (el PByte actual no es modificado).</returns>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 superiores.</param>
        public PByte Trim(bool lat1)
		{
			return new PByte(Pointer,Length,true,true,lat1);
		}

		/// <summary>
		/// Efectua Trim con el filtro de todos los carcteres en blanco a un puntero de cadena.
		/// </summary>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 superiores.</param>
		/// <returns>Pbyte en base a trim (el PByte actual no es modificado).</returns>
		public PByte Trim(bool left, bool right, bool lat1)
		{
			return new PByte(Pointer,Length,left,right, lat1);
		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
		/// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
		/// <param name="tab">Incluye en filtro el taburador (\t).</param>
		/// <returns>Pbyte en base a trim (el PByte actual no es modificado).</returns>
		public PByte Trim(bool left, bool right, bool sp,bool lat1, bool tab, bool crlf, bool control)
		{
            return new PByte(Pointer,Length,left,right,sp,lat1,tab,crlf,control);
		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="pTrim">Puntero de carcteres de filtro.</param>
		/// <param name="lTrim">Largo del puntero de carcteres de filtro.</param>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <returns>Pbyte en base a trim (el PByte actual no es modificado).</returns>
		public PByte Trim(byte * pTrim, int lTrim, bool left, bool right)
		{
			return new PByte(Pointer,Length,pTrim,lTrim,left,right);
		}

		/// <summary>
		/// Efectua Trim a un puntero de cadena.
		/// </summary>
		/// <param name="right">Trim derecho.</param>
		/// <param name="left">Trim izquierdo.</param>
		/// <param name="cTrim">Carcteres para Trim.</param>
		/// <returns>Pbyte en base a trim (el PByte actual no es modificado).</returns>
		public PByte Trim(bool left, bool right, byte[] cTrim)
		{
			int ltrim=cTrim.Length;
			if (ltrim!=0)
			{
                fixed(byte* ptrim=cTrim)
                {
                    return new PByte(Pointer,Length,ptrim,ltrim,left,right);
                } 
            }
			else return this;
		}


		/// <summary>
		/// Efectua un reconocimiento simple de texto encomillado:
		/// - Efectua Trim al bloque.
		/// - Si encuentra comillas coincidentes al comienzo y al final del bloque resultante, las descarta.
		/// </summary>
        /// <param name="afterTrimR">Luego del procesamiento de comillas, efecta trim derecho.</param>
        /// <param name="afterTrimL">Luego del procesamiento de comillas, efecta trim izquierdo.</param>
        /// <param name="mode">Opciones de eliminacin de comillas.</param>
        /// <param name="lat1">
        /// Adems de las comillas estndar (34 y 39) incluye comillas Latin1 (171 y 187).
        /// En el trim, incluye el carcter 160.
        /// </param>
        /// <returns>PByte con el bloque de texto vlido para extraccin (el PByte actual no es modificado).</returns>
		public PByte RemoveQuotes(RemoveQuotesMode mode, bool afterTrimL, bool afterTrimR, bool lat1)
		{
			return new PByte(Pointer,Length,mode,afterTrimL,afterTrimR,lat1);
		}

		#endregion

		#region Operadores

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PByte a.</param>
		/// <param name="b">PByte b.</param>
		/// <returns>True si son iguales.</returns>
		public static bool operator ==(PByte a, PByte b)
		{
            return a.CompareTo(b) == 0;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PByte a.</param>
		/// <param name="b">PByte b.</param>
		/// <returns>True si son diferentes.</returns>
		public static bool operator !=(PByte a, PByte b)
		{
            return a.CompareTo(b) != 0;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PByte a.</param>
		/// <param name="b">PByte b.</param>
		/// <returns>True si A es mayor a B.</returns>
		public static bool operator >(PByte a, PByte b)
		{
            return a.CompareTo(b) == 1;
		}
		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PByte a.</param>
		/// <param name="b">PByte b.</param>
		/// <returns>True si A es menor a B.</returns>
		public static bool operator <(PByte a, PByte b)
		{
            return a.CompareTo(b) == -1;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PByte a.</param>
		/// <param name="b">PByte b.</param>
		/// <returns>True si A es mayor o igual a B.</returns>
		public static bool operator >=(PByte a, PByte b)
		{
            return a.CompareTo(b) != -1;
		}

		/// <summary>
		/// Comparador.
		/// </summary>
		/// <param name="a">PByte a.</param>
		/// <param name="b">PByte b.</param>
		/// <returns>True si A es menor o igual a B.</returns>
		public static bool operator <=(PByte a, PByte b)
		{
            return a.CompareTo(b) != 1;
		}


		#endregion

		#region Metodos de Object

        /// <summary>
        /// Obtiene cdigo hash en base a FNV32.
        /// </summary>
        /// <returns></returns>
        public override int GetHashCode()
        {
            return (int)EnPalermo.Uti.Crypto.Fnv32(this.Pointer, this.Length);
        }

		/// <summary>
		/// True si son iguales.
		/// </summary>
		/// <param name="obj">Objeto.</param>
		/// <returns>True si son iguales.</returns>
		public override bool Equals(object obj)
		{
			if (obj is PByte)
			{
                return this.Equals((PByte)obj);
			}
			return false;
		}

		/// <summary>
		/// True si son iguales.
		/// </summary>
		/// <param name="b">PByte.</param>
		/// <returns>True si son iguales.</returns>
		public bool Equals(PByte b)
		{
            return this.CompareTo(b) == 0;
		}

        /// <summary>
        /// Genera una cadena mutable Latin1 en base al rango de memoria.
        /// </summary>
        /// <returns>Cadena mutable o vacia si el largo es cero.</returns>
        public override string ToString()
        {
            if (Length<1) return Others.Empty;
            return Mutables.Create(Pointer,Length);
        }
        /// <summary>
        /// Genera una cadena mutable Latin1 en base al rango de memoria.
        /// </summary>
        /// <param name="length">Largo de la cadena.</param>
        /// <returns>Cadena mutable o vacia si el largo es cero.</returns>
        public string ToString(int length)
        {
            if (length<1) return Others.Empty;
            return Mutables.Create(Pointer,length);
        }
        /// <summary>
        /// Genera una cadena mutable Latin1 en base al rango de memoria.
        /// </summary>
        /// <param name="index">Inicio.</param>
        /// <param name="length">Largo de la cadena.</param>
        /// <returns>Cadena mutable o vacia si el largo es cero.</returns>
        public string ToString(int index, int length)
        {
            if (length<1) return Others.Empty;
            return Mutables.Create(Pointer + index,length);
        }

		#endregion

        #region IComparable<PByte> Members

        /// <summary>
        /// Compara este objeto con otro en base al contenido.
        /// </summary>
        /// <param name="other">Otro objeto.</param>
        /// <returns>-1 si menor, 0 si igual y 1 si mayor.</returns>
        public int CompareTo(PByte other)
        {
            if (this.Pointer == other.Pointer)
            {
                if (this.Length == other.Length)
                    return 0;
                if (this.Length > other.Length)
                    return 1;
                return -1;
            }
            int comp;
            if (this.Length >= other.Length)
            {
                comp = Mem.Comp.Compare(this.Pointer, other.Pointer, other.Length);
                if (comp == -1)
                    return (this.Length > other.Length) ? 1 : 0;
                return this.Pointer[comp] > other.Pointer[comp] ? 1 : -1;
            }
            comp = Mem.Comp.Compare(this.Pointer, other.Pointer, this.Length);
            if (comp == -1 || this.Pointer[comp] < other.Pointer[comp])
                return -1;
            return 1;
        }


        #endregion

 
    }
}
