using System;
using System.Globalization;
using System.Reflection;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;
using EnPalermo.Lit;

namespace EnPalermo.Sys
{
    /// <summary>
    /// Permite acceder a informacin variada sobre el ensamblado.
    /// </summary>
	public sealed class AssemblyInfo
	{
        #region Campos

        /// <summary>
        /// Ensamblado.
        /// </summary>
        public readonly Assembly Assembly;

        /// <summary>
        /// AssemblyName.
        /// </summary>
        public readonly AssemblyName AssemblyName;

        /// <summary>
        /// Atributos del ensamblado.
        /// </summary>
        private object[] attribs;

        #endregion


        #region Constructores

        private AssemblyInfo(){}

        /// <summary>
        /// Constructor basado en tipo.
        /// </summary>
        /// <param name="type">Tipo.</param>
        public AssemblyInfo(Type type):this(type.Assembly){}

        /// <summary>
        /// Constructor basado en ensamblado.
        /// </summary>
        /// <param name="ass">Ensamblado.</param>
        public AssemblyInfo(Assembly ass)
        {
            this.Assembly=ass;
            this.AssemblyName=ass.GetName(false);
            this.attribs=null;
        }

        /// <summary>
        /// Constructor basado en objeto.
        /// </summary>
        /// <param name="obj">Objeto.</param>
        public AssemblyInfo(object obj):this(obj.GetType().Assembly){}


        #endregion


        #region Propiedades

        /// <summary>
        /// Nombre sencillo.
        /// </summary>
        public string Name 
        {
            get 
            {
                return AssemblyName.Name;
            }
        }

        /// <summary>
        /// Obtiene par de claves criptogrficas pblica y privada.
        /// </summary>
        public StrongNameKeyPair KeyPair 
        {
            get 
            {
                return AssemblyName.KeyPair;
            }
        }

        /// <summary>
        /// URL CodeBase.
        /// </summary>
        public string CodeBase
        {
            get 
            {
                return AssemblyName.CodeBase;
            }
        }

        /// <summary>
        /// CultureInfo.
        /// </summary>
        public CultureInfo CultureInfo
        {
            get 
            {
                return AssemblyName.CultureInfo;
            }
        }

        /// <summary>
        /// Nombre completo.
        /// </summary>
        public string FullName
        {
            get 
            {
                return Assembly.FullName;
            }
        }

        /// <summary>
        /// Version.
        /// </summary>
        public Version Version
        {
            get 
            {
                return AssemblyName.Version;
            }
        }

        /// <summary>
        /// Clave Pblica.
        /// </summary>
        public byte[] PublicKey
        {
            get 
            {
                return AssemblyName.GetPublicKey();
            }
        }

        /// <summary>
        /// Clave Pblica (Token).
        /// </summary>
        public byte[] PublicKeyToken
        {
            get 
            {
                return AssemblyName.GetPublicKeyToken();
            }
        }

        /// <summary>
        /// Identificador URI, incluidos carcteres de escape.
        /// </summary>
        public string EscapedCodeBase
        {
            get 
            {
                return AssemblyName.EscapedCodeBase;
            }
        }

        /// <summary>
        /// Obtiene la ubicacin original en formato CodeBase.
        /// </summary>
        public string Location
        {
            get 
            {
                return Assembly.Location;
            }
        }

        /// <summary>
        /// Obtiene el punto de entrada.
        /// </summary>
        public MethodInfo EntryPoint
        {
            get 
            {
                return Assembly.EntryPoint;
            }
        }


        /// <summary>
        /// True si se encuentra en el cache de ensamblados.
        /// </summary>
        public bool GlobalAssemblyCache
        {
            get 
            {
                return Assembly.GlobalAssemblyCache;
            }
        }

        /// <summary>
        /// Obtiene los atributos del ensamblado (conserva el array en cache luego de la primer llamada).
        /// </summary>
        public object[] CustomAttributes
        {
            get
            {
                if (attribs==null)
                    attribs=Assembly.GetCustomAttributes(false);
                return attribs;
            }
        }


        /// <summary>
        /// Obtiene el Ttulo del ensamblado o una cadena vacia.
        /// </summary>
        public string Title 
        {

            get 
            {
                object[] at=this.CustomAttributes;
                for(int a=0;a<attribs.Length;a++)
                    if(at[a] is AssemblyTitleAttribute)
                        return (at[a] as AssemblyTitleAttribute).Title;
                return Others.Empty;
            }

        }

        /// <summary>
        /// Obtiene la Descripcin del ensamblado o una cadena vacia.
        /// </summary>
        public string Description 
        {

            get 
            {
                object[] at=this.CustomAttributes;
                for(int a=0;a<attribs.Length;a++)
                    if(at[a] is AssemblyDescriptionAttribute)
                        return (at[a] as AssemblyDescriptionAttribute).Description;
                return Others.Empty;
            }

        }

        /// <summary>
        /// Obtiene el Copyright del ensamblado o una cadena vacia.
        /// </summary>
        public string Copyright
        {

            get 
            {
                object[] at=this.CustomAttributes;
                for(int a=0;a<attribs.Length;a++)
                    if(at[a] is AssemblyCopyrightAttribute)
                        return (at[a] as AssemblyCopyrightAttribute).Copyright;
                return Others.Empty;
            }

        }

        /// <summary>
        /// Obtiene el nombre del fabricante del ensamblado o una cadena vacia.
        /// </summary>
        public string Company
        {

            get 
            {
                object[] at=this.CustomAttributes;
                for(int a=0;a<attribs.Length;a++)
                    if(at[a] is AssemblyCompanyAttribute)
                        return (at[a] as AssemblyCompanyAttribute).Company;
                return Others.Empty;
            }

        }

        /// <summary>
        /// Obtiene el nombre de producto del ensamblado o una cadena vacia.
        /// </summary>
        public string Product
        {

            get 
            {
                object[] at=this.CustomAttributes;
                for(int a=0;a<attribs.Length;a++)
                    if(at[a] is AssemblyProductAttribute)
                        return (at[a] as AssemblyProductAttribute).Product;
                return Others.Empty;
            }

        }

        /// <summary>
        /// Obtiene la informacin de configuracin del ensamblado o una cadena vacia.
        /// </summary>
        public string Configuration
        {

            get 
            {
                object[] at=this.CustomAttributes;
                for(int a=0;a<attribs.Length;a++)
                    if(at[a] is AssemblyConfigurationAttribute)
                        return (at[a] as AssemblyConfigurationAttribute).Configuration;
                return Others.Empty;
            }

        }

        /// <summary>
        /// Obtiene la marca comercial del ensamblado o una cadena vacia.
        /// </summary>
        public string Trademark
        {

            get 
            {
                object[] at=this.CustomAttributes;
                for(int a=0;a<attribs.Length;a++)
                    if(at[a] is AssemblyTrademarkAttribute)
                        return (at[a] as AssemblyTrademarkAttribute).Trademark;
                return Others.Empty;
            }
        }

        /// <summary>
        /// Obtiene AssemblyVersion del ensamblado o una cadena vacia.
        /// </summary>
        public string AssemblyVersion
        {

            get 
            {
                object[] at=this.CustomAttributes;
                for(int a=0;a<attribs.Length;a++)
                    if(at[a] is AssemblyVersionAttribute)
                        return (at[a] as AssemblyVersionAttribute).Version;
                return Others.Empty;
            }
        }

        /// <summary>
        /// Obtiene AssemblyFileVersion del ensamblado o una cadena vacia.
        /// </summary>
        public string AssemblyFileVersion
        {

            get 
            {
                object[] at=this.CustomAttributes;
                for(int a=0;a<attribs.Length;a++)
                    if(at[a] is AssemblyFileVersionAttribute)
                        return (at[a] as AssemblyFileVersionAttribute).Version;
                return Others.Empty;
            }
        }



        #endregion



	}
}
