using System;
using System.Text;
using System.IO;
using System.Security;
using System.Runtime.InteropServices;
using EnPalermo.Mem;
using EnPalermo.Txt;
using EnPalermo.Lit;

namespace EnPalermo.Sys
{

    #region Enums

    /// <summary>
    /// Describe las funcionalidades del sistema de archivos en la unidad especificada.
    /// </summary>
    [Flags()]
    public enum FileSystemFlags
    {
        /// <summary>
        /// The file system supports case-sensitive file names.
        /// </summary>
        CaseSensitiveSearch    =  0x00000001,

        /// <summary>
        /// The file system preserves the case of file names when it places a name on disk.
        /// </summary>
        CasePreservedNames     =  0x00000002,

        /// <summary>
        /// The file system supports Unicode in file names as they appear on disk.
        /// </summary>
        UnicodeOnDisk          =  0x00000004,

        /// <summary>
        /// The file system preserves and enforces access control lists (ACL). For example, the NTFS file system preserves and enforces ACLs, and the FAT file system does not.
        /// </summary>
        PersistentAcls          =  0x00000008,

        /// <summary>
        /// The file system supports file-based compression.
        /// </summary>
        FileCompression         =  0x00000010,

        /// <summary>
        /// The file system supports disk quotas.
        /// </summary>
        VolumeQuotas            =  0x00000020,

        /// <summary>
        /// The file system supports sparse files.
        /// </summary>
        SupportsSparseFiles    =  0x00000040,

        /// <summary>
        /// The file system supports re-parse points.
        /// </summary>
        SupportsReparsePoints  =  0x00000080,

        /// <summary>
        /// Undocumented...
        /// </summary>
        SupportsRemoteStorage  =  0x00000100,

        /// <summary>
        /// The specified volume is a compressed volume, for example, a DoubleSpace volume.
        /// </summary>
        VolumeIsCompressed     =  0x00008000,

        /// <summary>
        /// The file system supports object identifiers.
        /// </summary>
        SupportsObjectIds      =  0x00010000,

        /// <summary>
        /// The file system supports the Encrypted File System (EFS).
        /// </summary>
        SupportsEncryption      =  0x00020000,

        /// <summary>
        /// The file system supports named streams.
        /// </summary>
        NamedStreams            =  0x00040000,

        /// <summary>
        /// The specified volume is read-only. Windows 2000/NT and Windows Me/98/95:  This value is not supported.
        /// </summary>
        ReadOnlyVolume         =  0x00080000

    }



    /// <summary>
    /// Describe el tipo de unidad.
    /// </summary>
    public enum DriveType
    {
        /// <summary>
        /// The drive type cannot be determined.
        /// </summary>
        Unknown     = 0,
        /// <summary>
        /// The root path is invalid, for example, no volume is mounted at the path.
        /// </summary>
        NoRootDir = 1,
        /// <summary>
        /// The drive is a type that has removable media, for example, a floppy drive or removable hard disk.
        /// </summary>
        Removable   = 2,
        /// <summary>
        /// The drive is a type that cannot be removed, for example, a fixed hard drive.
        /// </summary>
        Fixed       = 3,
        /// <summary>
        /// The drive is a remote (network) drive.
        /// </summary>
        Remote      = 4,
        /// <summary>
        /// The drive is a CD-ROM drive.
        /// </summary>
        Cdrom       = 5,
        /// <summary>
        /// The drive is a RAM disk.
        /// </summary>
        Ramdisk     = 6
    }


    #endregion



	/// <summary>
	/// Funciones estticas y constantes relativas al manejo de archivos.
	/// </summary>
    [SuppressUnmanagedCodeSecurity]
    public unsafe static class FileHelpers
    {

        #region Constantes y campos estticos

        /// <summary>
        /// Representa en un entero los bytes de la cadena Ascii "A:\".
        /// </summary>
        public const uint AnsiDriveA=0x5C3A41;

        /// <summary>
        /// Las 26 letras de unidad en formato "A:\".
        /// </summary>
        public static readonly string[] DrvLetters;

        /// <summary>
        /// Las 26 letras de unidad en formato "\\?\A:\".
        /// </summary>
        public static readonly string[] DrvLettersUnicode;

        /// <summary>
        /// Constructor esttico.
        /// </summary>
        static FileHelpers()
        {
            DrvLetters= new string[26]{"A:\\","B:\\","C:\\","D:\\","E:\\","F:\\","G:\\","H:\\","I:\\","J:\\","K:\\","L:\\","M:\\","N:\\","O:\\","P:\\","Q:\\","R:\\","S:\\","T:\\","U:\\","V:\\","W:\\","X:\\","Y:\\","Z:\\"};
            DrvLettersUnicode= new string[26]{"\\\\?\\A:\\","\\\\?\\B:\\","\\\\?\\C:\\","\\\\?\\D:\\","\\\\?\\E:\\","\\\\?\\F:\\","\\\\?\\G:\\","\\\\?\\H:\\","\\\\?\\I:\\","\\\\?\\J:\\","\\\\?\\K:\\","\\\\?\\L:\\","\\\\?\\M:\\","\\\\?\\N:\\","\\\\?\\O:\\","\\\\?\\P:\\","\\\\?\\Q:\\","\\\\?\\R:\\","\\\\?\\S:\\","\\\\?\\T:\\","\\\\?\\U:\\","\\\\?\\V:\\","\\\\?\\W:\\","\\\\?\\X:\\","\\\\?\\Y:\\","\\\\?\\Z:\\"};
        }


        #endregion

        #region API

        /// <summary>
        /// The GetDriveType function determines whether a disk drive is a removable, fixed, CD-ROM, RAM disk, or network drive. 
        /// </summary>
        /// <param name="drive">Pointer to a null-terminated string that specifies the root directory of the disk to return information about. A trailing backslash is required. If this parameter is NULL, the function uses the root of the current directory.</param>
        /// <returns>The return value specifies the type of drive.</returns>
        [DllImport(Api.KERNEL32, ExactSpelling = true, SetLastError = false)]
        public static unsafe extern DriveType GetDriveTypeA(byte* drive);

        /// <summary>
        /// The GetDriveType function determines whether a disk drive is a removable, fixed, CD-ROM, RAM disk, or network drive. 
        /// </summary>
        /// <param name="drive">Pointer to a null-terminated string that specifies the root directory of the disk to return information about. A trailing backslash is required. If this parameter is NULL, the function uses the root of the current directory.</param>
        /// <returns>The return value specifies the type of drive.</returns>
        [DllImport(Api.KERNEL32, ExactSpelling = true, SetLastError = false)]
        public static unsafe extern DriveType GetDriveTypeW(char* drive);

        /// <summary>
        /// The GetDriveType function determines whether a disk drive is a removable, fixed, CD-ROM, RAM disk, or network drive. 
        /// </summary>
        /// <param name="drive">Pointer to a null-terminated string that specifies the root directory of the disk to return information about. A trailing backslash is required. If this parameter is NULL, the function uses the root of the current directory.</param>
        /// <returns>The return value specifies the type of drive.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = false)]
        public static extern DriveType GetDriveType(string drive);

        /// <summary>
        /// The GetVolumeInformation function retrieves information about a file system and volume whose root directory is specified.
        /// </summary>
        /// <param name="drive">Pointer to a string that contains the root directory of the volume to be described. If this parameter is NULL, the root of the current directory is used. A trailing backslash is required. For example, you would specify \\MyServer\MyShare as \\MyServer\MyShare\, or the C drive as "C:\".</param>
        /// <param name="volumeName">Pointer to a buffer that receives the name of the specified volume.</param>
        /// <param name="volumeNameBufLen">Length of the volume name buffer, in TCHARs. This parameter is ignored if the volume name buffer is not supplied.</param>
        /// <param name="volSerialNumber">Pointer to a variable that receives the volume serial number. This parameter can be NULL if the serial number is not required.</param>
        /// <param name="maxFileNameLen">Pointer to a variable that receives the maximum length, in TCHARs, of a file name component supported by the specified file system. A file name component is that portion of a file name between backslashes. </param>
        /// <param name="fileSystemFlags">Pointer to a variable that receives flags associated with the specified file system. This parameter can be one or more of the following flags; however, FS_FILE_COMPRESSION and FS_VOL_IS_COMPRESSED are mutually exclusive.</param>
        /// <param name="fileSystemName">Pointer to a buffer that receives the name of the file system (such as FAT or NTFS).</param>
        /// <param name="fileSystemNameBufLen">Length of the file system name buffer, in TCHARs. This parameter is ignored if the file system name buffer is not supplied.</param>
        /// <returns>If not all the requested information is retrieved, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, ExactSpelling = true, SetLastError = false)]
        public static unsafe extern bool GetVolumeInformationA(byte* drive, byte* volumeName, int volumeNameBufLen, out uint volSerialNumber, out int maxFileNameLen, out FileSystemFlags fileSystemFlags, byte* fileSystemName, int fileSystemNameBufLen);

        
        /// <summary>
        /// The GetVolumeInformation function retrieves information about a file system and volume whose root directory is specified.
        /// </summary>
        /// <param name="drive">Pointer to a string that contains the root directory of the volume to be described. If this parameter is NULL, the root of the current directory is used. A trailing backslash is required. For example, you would specify \\MyServer\MyShare as \\MyServer\MyShare\, or the C drive as "C:\".</param>
        /// <param name="volumeName">Pointer to a buffer that receives the name of the specified volume.</param>
        /// <param name="volumeNameBufLen">Length of the volume name buffer, in TCHARs. This parameter is ignored if the volume name buffer is not supplied.</param>
        /// <param name="volSerialNumber">Pointer to a variable that receives the volume serial number. This parameter can be NULL if the serial number is not required.</param>
        /// <param name="maxFileNameLen">Pointer to a variable that receives the maximum length, in TCHARs, of a file name component supported by the specified file system. A file name component is that portion of a file name between backslashes. </param>
        /// <param name="fileSystemFlags">Pointer to a variable that receives flags associated with the specified file system. This parameter can be one or more of the following flags; however, FS_FILE_COMPRESSION and FS_VOL_IS_COMPRESSED are mutually exclusive.</param>
        /// <param name="fileSystemName">Pointer to a buffer that receives the name of the file system (such as FAT or NTFS).</param>
        /// <param name="fileSystemNameBufLen">Length of the file system name buffer, in TCHARs. This parameter is ignored if the file system name buffer is not supplied.</param>
        /// <returns>If not all the requested information is retrieved, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, ExactSpelling = true, SetLastError = false)]
        public static unsafe extern bool GetVolumeInformationW(char* drive, char* volumeName, int volumeNameBufLen, out uint volSerialNumber, out int maxFileNameLen, out FileSystemFlags fileSystemFlags, char* fileSystemName, int fileSystemNameBufLen);


        /// <summary>
        /// The GetVolumeInformation function retrieves information about a file system and volume whose root directory is specified.
        /// </summary>
        /// <param name="drive">Pointer to a string that contains the root directory of the volume to be described. If this parameter is NULL, the root of the current directory is used. A trailing backslash is required. For example, you would specify \\MyServer\MyShare as \\MyServer\MyShare\, or the C drive as "C:\".</param>
        /// <param name="volumeName">Pointer to a buffer that receives the name of the specified volume.</param>
        /// <param name="volumeNameBufLen">Length of the volume name buffer, in TCHARs. This parameter is ignored if the volume name buffer is not supplied.</param>
        /// <param name="volSerialNumber">Pointer to a variable that receives the volume serial number. This parameter can be NULL if the serial number is not required.</param>
        /// <param name="maxFileNameLen">Pointer to a variable that receives the maximum length, in TCHARs, of a file name component supported by the specified file system. A file name component is that portion of a file name between backslashes. </param>
        /// <param name="fileSystemFlags">Pointer to a variable that receives flags associated with the specified file system. This parameter can be one or more of the following flags; however, FS_FILE_COMPRESSION and FS_VOL_IS_COMPRESSED are mutually exclusive.</param>
        /// <param name="fileSystemName">Pointer to a buffer that receives the name of the file system (such as FAT or NTFS).</param>
        /// <param name="fileSystemNameBufLen">Length of the file system name buffer, in TCHARs. This parameter is ignored if the file system name buffer is not supplied.</param>
        /// <returns>If not all the requested information is retrieved, the return value is zero. To get extended error information, call GetLastError.</returns>
        [DllImport(Api.KERNEL32, CharSet = CharSet.Auto, SetLastError = false)]
        public static extern bool GetVolumeInformation(string drive, System.Text.StringBuilder volumeName, int volumeNameBufLen, out uint volSerialNumber, out int maxFileNameLen, out FileSystemFlags fileSystemFlags, System.Text.StringBuilder fileSystemName, int fileSystemNameBufLen);


        #endregion

        #region Metodos

        /// <summary>
        /// Prepara una cadena para su uso como nombre de archivo.
        /// Efectua las siguientes operaciones:
        /// - Efectua trim de espacios y carcteres invlidos a la izquierda.
        /// - Efectua trim de espacios, puntos y carcteres invlidos a la derecha.
        /// - Verifica si la cadena resultante no contiene carcteres invlidos o wildCards.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="allowDirDots">Acepta las cadenas ".." y ".".</param>
        /// <param name="allowWildCards">En la etapa de verificacin, acepta las wildcard. Independientemente de este parmetro, el
        /// inicio con "\\?\" o "//?/" siempre es aceptado.</param>
        /// <param name="noVerif">No efectua verificacin final de carcteres invlidos.</param>
        /// <returns>PChar resultante o nulo si la cadena result vacia luego de las depuraciones o no super la verificacin final.</returns>
        public static PChar GetFileNameBlock(char*p, int l, bool noVerif,bool allowDirDots,bool allowWildCards)
        {
            if (l<1)
                return PChar.Zero;
            reswitch1:

                switch(*p)
                {
                    case '"':
                    case '<':
                    case '>':
                    case '|':
                        if (--l!=0)
                        {
                            p++;
                            goto reswitch1;
                        }
                        return PChar.Zero;
                    default:
                        if (*p<=' ')
                            goto case '|';
                        break;

                }
            reswitch2:
                switch(p[l-1])
                {
                    case '"':
                    case '<':
                    case '>':
                    case '|':
                    case '\xa0':
                    case '\x85':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if (--l!=0)
                            goto reswitch2;
                        return PChar.Zero;
                    case '.':
                        if (allowDirDots && (l==1 || (l==2 && *p=='.')))
                            return new PChar(p,l);
                        goto case '\ufeff';
                    default:
                        if (p[l]<=' ')
                            goto case '\ufeff';
                        break;
                }
            if (!noVerif)
            {
                int a=1;
                bool volsep=false;
                do
                {
                    switch(p[a])
                    {
                        case '"':
                        case '<':
                        case '>':
                        case '|':
                            return PChar.Zero;
                        case '?':
                            if (allowWildCards || (a==2 && l>3 && (*p=='\\' || *p=='/') && (p[1]=='\\' || p[1]=='/') && (p[3]=='\\' || p[3]=='/') ))
                                continue;
                            return PChar.Zero;
                        case '*':
                            if (allowWildCards)
                                continue;
                            return PChar.Zero;
                        case ':':
                            if (!volsep && ((a==1 && Texts.IsAsciiLetter(*p)) || (a>4 && (*p=='\\' || *p=='/') && (p[1]=='\\' || p[1]=='/')  && (p[2]=='?' || p[2]=='.') && (p[3]=='\\' || p[3]=='/') && Texts.IsAsciiLetter(p[a-1]) && (p[a-2]=='\\' || p[a-2]=='/'))) && (a+1==l || p[a+1]=='\\' || p[a+1]=='/'))
                            {
                                volsep=true;
                                continue;
                            }
                            return PChar.Zero;
                        default:
                            if (p[a]<' ')
                                return PChar.Zero;
                            continue;
                    }
                }while(++a!=l);
            }
            return new PChar(p,l);
        }

        /// <summary>
        /// Prepara una cadena para su uso como nombre de archivo.
        /// Efectua las siguientes operaciones:
        /// - Efectua trim de espacios y carcteres invlidos a la izquierda.
        /// - Efectua trim de espacios, puntos y carcteres invlidos a la derecha.
        /// - Verifica si la cadena resultante no contiene carcteres invlidos o wildCards.
        /// </summary>
        /// <param name="p">Puntero de bytes.</param>
        /// <param name="l">Largo.</param>
        /// <param name="allowDirDots">Acepta las cadenas ".." y ".".</param>
        /// <param name="allowWildCards">En la etapa de verificacin, acepta las wildcard. Independientemente de este parmetro, el
        /// inicio con "\\?\" o "//?/" siempre es aceptado.</param>
        /// <param name="noVerif">No efectua verificacin final de carcteres invlidos.</param>
        /// <param name="lat1">Incluye en trim derecho a los carcteres altos Latin1.</param>
        /// <returns>PByte resultante o nulo si la cadena result vacia luego de las depuraciones o no super la verificacin final.</returns>
        public static PByte GetFileNameBlock(byte*p, int l, bool noVerif,bool allowDirDots,bool allowWildCards, bool lat1)
        {
            if (l<1)
                return PByte.Zero;
            reswitch1:

                switch(*p)
                {
                    case 34:
                    case 60:
                    case 62:
                    case 124:
                        if (--l!=0)
                        {
                            p++;
                            goto reswitch1;
                        }
                        return PByte.Zero;
                    default:
                        if (*p<33)
                            goto case 124;
                        break;

                }
            reswitch2:
                switch(p[l-1])
                {
                    case 34:
                    case 60:
                    case 62:
                    case 124:
                        if (--l!=0)
                            goto reswitch2;
                        return PByte.Zero;
                    case 46:
                        if (allowDirDots && (l==1 || (l==2 && *p==46)))
                            return new PByte(p,l);
                        goto case 124;
                    case 160:
                        if (lat1)
                            goto case 124;
                        break;

                    default:
                        if (p[l]<33)
                            goto case 124;
                        break;
                }
            if (!noVerif)
            {
                int a=1;
                bool volsep=false;
                do
                {
                    switch(p[a])
                    {
                        case 34:
                        case 60:
                        case 62:
                        case 124:
                            return PByte.Zero;
                        case 63:
                            if (allowWildCards || (a==2 && l>3 && (*p==92 || *p==47) && (p[1]==92 || p[1]==47) && (p[3]==92 || p[3]==47) ))
                                continue;
                            return PByte.Zero;
                        case 42:
                            if (allowWildCards)
                                continue;
                            return PByte.Zero;
                        case 58:
                            if (!volsep && ((a==1 && Texts.IsAsciiLetter((char)*p)) || (a>4 && (*p==92 || *p==47) && (p[1]==92 || p[1]==47)  && (p[2]==63 || p[2]==46) && (p[3]==92 || p[3]==47) && Texts.IsAsciiLetter((char)p[a-1]) && (p[a-2]==92 || p[a-2]==47))) && (a+1==l || p[a+1]==92 || p[a+1]==47))
                            {
                                volsep=true;
                                continue;
                            }
                            return PByte.Zero;
                        default:
                            if (p[a]<32)
                                return PByte.Zero;
                            continue;
                    }
                }while(++a!=l);
            }
            return new PByte(p,l);
        }


        /// <summary>
        /// Obtiene full path del nombre de archivo sin generar excepciones.
        /// </summary>
        /// <param name="fname">Nombre de archivo.</param>
        /// <param name="emptyIfNotExist">Si true y el archivo no existe, retornar cadena vacia aunque el nombre sea correcto.</param>
        /// <returns>Cadena con el FullPath o vacia.</returns>
        public static string GetFullPath(string fname, bool emptyIfNotExist)
        {
            int l=fname==null?0:fname.Length;
            if (l==0)
                return Others.Empty;
            try
            {
                fname=System.IO.Path.GetFullPath(fname);
                if (emptyIfNotExist && !File.Exists(fname))
                    return Others.Empty;
                return fname;
            }
            catch
            {
                return Others.Empty;
            }
        }


        /// <summary>
        /// Devuelve true se el path especificado existe y tiene el atributo Directory esperado.
        /// </summary>
        /// <param name="path">Path.</param>
        /// <param name="isDirectory">True si se espera un directorio.</param>
        /// <returns>True se el path especificado existe y tiene el atributo Directory esperado.</returns>
        public static bool PathExist(string path, bool isDirectory)
        {
            return PathExist(Api.GetFileAttributes(path),isDirectory);
        }

        /// <summary>
        /// Devuelve true si el valor indica que el path existe y tiene el atributo Directory esperado.
        /// </summary>
        /// <param name="att">Atributos generados con GetFileAttributes() o GetFileAttributesEx().</param>
        /// <param name="isDirectory">True si se espera un directorio.</param>
        /// <returns>True si el valor indica que el path existe y tiene el atributo Directory esperado.</returns>
        public static bool PathExist(int att, bool isDirectory)
        {
            return isDirectory ? ((att & 16) != 0 && att != -1) : ((att & 16) == 0);
        }


        #endregion



	}
}
