using System;
using System.Security;
using System.Runtime.InteropServices;
using EnPalermo.Uti;

namespace EnPalermo.Sys
{
    /// <summary>
    /// Representa un FileTime.
    /// </summary>
    [StructLayout(LayoutKind.Sequential)]
    public unsafe struct FileTime
    {

        /// <summary>
        /// FileTime Cero.
        /// </summary>
        public static readonly FileTime Zero=new FileTime();

        /// <summary>
        /// Ticks desde el 01/01/1601 UTC.
        /// </summary>
        public long TicksUTC;

        /// <summary>
        /// Constructor a partir de un DateTime.
        /// </summary>
        /// <param name="dt">DateTime</param>
        /// <param name="utc">Si true, no convierte a UTC.</param>
        public FileTime(DateTime dt, bool utc)
        {
            if (utc) TicksUTC = dt.Ticks - Time.Ticks1601UTC;
            else
            {
                long ft = dt.Ticks - Time.Ticks1601UTC;
                long ft2 = 0L;
                LocalFileTimeToFileTime(&ft, &ft2);
                this.TicksUTC = ft2;
            }
        }

        /// <summary>
        /// Constructor a partir de un DateTime Local.
        /// </summary>
        /// <param name="dt">DateTime local.</param>
        public FileTime(DateTime dt)
        {
            long ft = dt.Ticks - Time.Ticks1601UTC;
            long ft2 = 0L;
            LocalFileTimeToFileTime(&ft, &ft2);
            this.TicksUTC = ft2;
        }

        /// <summary>
        /// Constructor a partir de un SystemTime.
        /// </summary>
        /// <param name="st">SystemTime.</param>
        /// <param name="utc">Si true, no convierte a UTC.</param>
        public FileTime(SystemTime st, bool utc)
        {
            int dia = st.Day;
            int ao = st.Year;
            switch (st.Month - 1)
            {
                case 0:
                    break;
                case 1:
                    dia += 31;
                    break;
                case 2:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 59;
                    else dia += 60;
                    break;
                case 3:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 90;
                    else dia += 91;
                    break;
                case 4:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 120;
                    else dia += 121;
                    break;
                case 5:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 151;
                    else dia += 152;
                    break;
                case 6:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 181;
                    else dia += 182;
                    break;
                case 7:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 212;
                    else dia += 213;
                    break;
                case 8:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 243;
                    else dia += 244;
                    break;
                case 9:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 273;
                    else dia += 274;
                    break;
                case 10:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 304;
                    else dia += 305;
                    break;
                default:
                    if (ao % 4 != 0 || (ao % 100 == 0 && ao % 400 != 0)) dia += 334;
                    else dia += 335;
                    break;
            }
            dia += (--ao * 365) + (ao / 4) + (ao / 400) - (ao / 100) - 1;
            ao = ((((int)st.Hour * 3600000) + ((int)st.Minute * 60000) + (int)st.Second) * 1000) + (int)st.Milisecond;
            if (utc) TicksUTC = ((long)dia * 864000000000L) + ((long)ao * 10000L) - Time.Ticks1601UTC;
            else
            {
                long ft = ((long)dia * 864000000000L) + ((long)ao * 10000L) - Time.Ticks1601UTC;
                long ft2 = 0L;
                LocalFileTimeToFileTime(&ft, &ft2);
                TicksUTC = ft2;
            }
        }

        /// <summary>
        /// Constructor a partir de Ticks.
        /// </summary>
        /// <param name="ftTicksUTC">Ticks desde 1/1/1601 UTC.</param>
        public FileTime(long ftTicksUTC)
        {
            this.TicksUTC = ftTicksUTC;
        }

        /// <summary>
        /// Obtiene DateTime Local.
        /// </summary>
        public DateTime DateTimeLocal
        {
            get
            {
                long ldt = 0L;
                FileTimeToLocalFileTime(ref this, &ldt);
                ldt += Time.Ticks1601UTC;
                return *(DateTime*)&ldt;
            }
        }

        /// <summary>
        /// Obtiene DateTime UTC.
        /// </summary>
        public DateTime DateTimeUTC
        {
            get
            {
                long ldt = TicksUTC + Time.Ticks1601UTC;
                return *(DateTime*)&ldt;
            }
        }

        /// <summary>
        /// Obtiene un SystemTime Local.
        /// </summary>
        public SystemTime SystemTimeLocal
        {
            get
            {
                FileTime ft = Zero;
                FileTimeToLocalFileTime(ref this, (long*)&ft);
                SystemTime st = SystemTime.Zero;
                FileTimeToSystemTime(ref ft, &st);
                return st;
            }
        }

        /// <summary>
        /// Obtiene un SystemTime UTC.
        /// </summary>
        public SystemTime SystemTimeUTC
        {
            get
            {
                SystemTime st = SystemTime.Zero;
                FileTimeToSystemTime(ref this, &st);
                return st;
            }
        }

        /// <summary>
        /// Devuelve FileTime del momento actual (UTC).
        /// </summary>
        public static FileTime UtcNow
        {
            get
            {
                FileTime ft = Zero;
                GetSystemTimeAsFileTime(ref ft);
                return ft;
            }
        }

        /// <summary>
        /// Convierte el actual FileTime Local a UTC.
        /// </summary>
        public FileTime LocalToUTC
        {
            get
            {
                long loc = this.TicksUTC;
                FileTime ft = Zero;
                LocalFileTimeToFileTime(&loc, (long*)&ft);
                return ft;
            }
        }

        /// <summary>
        /// Convierte el actual FileTime UTC a Local.
        /// </summary>
        public FileTime UTCToLocal
        {
            get
            {
                FileTime ft = Zero;
                FileTimeToLocalFileTime(ref this, (long*)&ft);
                return ft;
            }
        }

        [SuppressUnmanagedCodeSecurity,DllImport(Api.KERNEL32)]
        private static extern void GetSystemTimeAsFileTime(ref FileTime ft);

        [SuppressUnmanagedCodeSecurity,DllImport(Api.KERNEL32)]
        private static extern void FileTimeToLocalFileTime(ref FileTime ftUtc, long* ft);

        [SuppressUnmanagedCodeSecurity,DllImport(Api.KERNEL32)]
        private static extern void LocalFileTimeToFileTime(long* ftLocal, long* ft);

        [SuppressUnmanagedCodeSecurity,DllImport(Api.KERNEL32)]
        private static extern bool FileTimeToSystemTime(ref FileTime ft, SystemTime* st);

    }
}
