using System;
using System.Security;
using System.Runtime.InteropServices;
using EnPalermo.Lit;

namespace EnPalermo.Sys
{
	/// <summary>
	/// Provee informacin del estado actual de la memoria.
	/// </summary>
    [SuppressUnmanagedCodeSecurity]
	public sealed class MemInfo
    {
        #region API

        /// <summary>
        /// Informacin de memoria. Para uso con Windows 98.
        /// </summary>
        /// <param name="buffer">Estructura MEMORYSTATUS.</param>
        /// <returns>True si ok.</returns>
        [DllImport(Api.KERNEL32)]
        private static unsafe extern bool GlobalMemoryStatus(uint* buffer);

        /// <summary>
        /// Informacin de memoria. Para uso con NT.
        /// </summary>
        /// <param name="buffer">Estructura MEMORYSTATUSEX.</param>
        /// <returns>True si ok.</returns>
        [DllImport(Api.KERNEL32)]
        private static unsafe extern bool GlobalMemoryStatusEx(long* buffer);


        #endregion

        /// <summary>
        /// Total de memoria utilizable. Suele ser un poco menos de la suma de la memoria fsica y la virtual.
        /// </summary>
        private long _total;

        /// <summary>
        /// Total de memoria disponible. Incluye la memoria fisica y la virtual.
        /// </summary>
        private long _free;

        /// <summary>
        /// Total de memoria fsica del sistema.
        /// </summary>
        private long _physicalTotal;

        /// <summary>
        /// Total de memoria fsica disponible.
        /// </summary>
        private long _physicalFree;

        /// <summary>
        /// Total de memoria de usuario utilizable en este proceso.
        /// </summary>
        private long _virtualTotal;

        /// <summary>
        /// Total de memoria de usuario disponible en este proceso.
        /// </summary>
        private long _virtualFree;

        /// <summary>
        /// Total del espacio de memoria virtual extendida que se encuentra disponible para este proceso.
        /// </summary>
        private long _extendedFree;

        /// <summary>
        /// Porcentaje aproximado del uso de memoria.
        /// </summary>
        private int _percentUse;

		/// <summary>
		/// Total de memoria utilizable. Suele ser un poco menos de la suma de la memoria fsica y la virtual.
		/// </summary>
        public long Total
        {
            get
            {
                return _total;
            }
        }

		/// <summary>
		/// Total de memoria disponible. Incluye la memoria fisica y la virtual.
		/// </summary>
        public long Free
        {
            get
            {
                return _free;
            }
        }


		/// <summary>
		/// Total de memoria fsica del sistema.
		/// </summary>
        public long PhysicalTotal
        {
            get
            {
                return _physicalTotal;
            }
        }

		/// <summary>
		/// Total de memoria fsica disponible.
		/// </summary>
        public long PhysicalFree
        {
            get
            {
                return _physicalFree;
            }
        }


		/// <summary>
		/// Total de memoria de usuario utilizable en este proceso.
		/// </summary>
        public long VirtualTotal
        {
            get
            {
                return _virtualTotal;
            }
        }


		/// <summary>
		/// Total de memoria de usuario disponible en este proceso.
		/// </summary>
        public long VirtualFree
        {
            get
            {
                return _virtualFree;
            }
        }


		/// <summary>
		/// Total del espacio de memoria virtual extendida que se encuentra disponible para este proceso.
		/// </summary>
        public long ExtendedFree
        {
            get
            {
                return _extendedFree;
            }
        }

		/// <summary>
		/// Porcentaje aproximado del uso de memoria.
		/// </summary>
        public int PercentUse
        {
            get
            {
                return _percentUse;
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public MemInfo()
        {
            refresh(out _percentUse, out _physicalTotal, out _physicalFree, out _total, out _free, out _virtualTotal, out _virtualFree, out _extendedFree);
        }

        /// <summary>
        /// Actualiza la informacin de memoria.
        /// </summary>
        public void Refresh()
        {
            refresh(out _percentUse, out _physicalTotal, out _physicalFree, out _total, out _free, out _virtualTotal, out _virtualFree, out _extendedFree);
        }

        /// <summary>
        /// Actualizador.
        /// </summary>
        private static unsafe void refresh(out int _percentUse, out long _physicalTotal, out long _physicalFree, out long _total, out long _free, out long _virtualTotal, out long _virtualFree, out long _extendedFree)
        {
            /*
[StructLayout(LayoutKind.Sequential)]
internal sealed class MEMORYSTATUS
{
internal int length=32;
internal int memoryLoad=0;
internal uint totalPhys=0;
internal uint availPhys=0;
internal uint totalPageFile=0;
internal uint availPageFile=0;
internal uint totalVirtual=0;
internal uint availVirtual=0;
}

[StructLayout(LayoutKind.Sequential)]
internal struct MEMORYSTATUSEX
{
internal int length=64;
internal int memoryLoad=0;
internal long totalPhys=0;
internal long availPhys=0;
internal long totalPageFile=0;
internal long availPageFile=0;
internal long totalVirtual=0;
internal long availVirtual=0;
internal long availExtendedVirtual=0;
}
*/
            if (OsInfo.Minimum2K)
            {
                long* p = stackalloc long[8];
                *(int*)p = 64;
                if (!GlobalMemoryStatusEx(p))
                    throw new ExternalException(Errors.MEMINFO_API);
                _percentUse = ((int*)p)[1];
                _physicalTotal = p[1];
                _physicalFree = p[2];
                _total = p[3];
                _free = p[4];
                _virtualTotal = p[5];
                _virtualFree = p[6];
                _extendedFree = p[7];
            }
            else
            {
                uint* p = stackalloc uint[8];
                *p = 32U;
                if (!GlobalMemoryStatus(p))
                    throw new ExternalException(Errors.MEMINFO_API);
                _percentUse = (int)p[1];
                _physicalTotal = p[2];
                _physicalFree = p[3];
                _total = p[4];
                _free = p[5];
                _virtualTotal = p[6];
                _virtualFree = p[7];
                _extendedFree = 0;
            }

        }


	}
}
