using System;
using System.Runtime.InteropServices;
using System.Security;
using EnPalermo.Txt;
using EnPalermo.Lit;

namespace EnPalermo.Sys
{
    #region Enumeradores


    /// <summary>
    /// Suite de Windows.
    /// </summary>
    [Flags()]
    public enum SuiteWindows : short
    {
        /// <summary>
        /// No definido.
        /// </summary>
        Undefined = 0,
        /// <summary>
        /// Back Office Suite.
        /// </summary>
        BackOffice = 0x00000004,
        /// <summary>
        /// Edicin Web.
        /// </summary>
        Blade = 0x00000400,
        /// <summary>
        /// Communications Suite.
        /// </summary>
        Communications = 0x00000008,
        /// <summary>
        /// Data Center.
        /// </summary>
        DataCenter = 0x00000080,
        /// <summary>
        /// NT Embedded.
        /// </summary>
        EmbeddedNT = 0x00000040,
        /// <summary>
        /// NT Embedded restringido.
        /// </summary>
        EmbeddedRestricted = 0x00000800,
        /// <summary>
        /// Enterprise.
        /// </summary>
        Enterprise = 0x00000002,
        /// <summary>
        /// Edicin Personal o Home.
        /// </summary>
        Personal = 0x00000200,
        /// <summary>
        /// Security Appliance Suite.
        /// </summary>
        SecurityAppliance = 0x00001000,
        /// <summary>
        /// Terminal Server para un solo usuario.
        /// </summary>
        SingleUserTS = 0x00000100,
        /// <summary>
        /// Small Business.
        /// </summary>
        SmallBusiness = 0x00000001,
        /// <summary>
        /// Small Business restringido.
        /// </summary>
        SmallBusinessRestricted = 0x00000020,
        /// <summary>
        /// Terminal Server.
        /// </summary>
        Terminal = 0x00000010
    }


    /// <summary>
    /// Producto Windows.
    /// </summary>
    public enum WindowsProduct
    {
        /// <summary>
        /// No se puede determinar!
        /// </summary>
        Indeterminate = 0,
        /// <summary>
        /// Windows 2000.
        /// </summary>
        Windows2000 = 22,
        /// <summary>
        /// Windows 2003 Server.
        /// </summary>
        Windows2003Server = 24,
        /// <summary>
        /// Windows Vista (Provisorio).
        /// </summary>
        WindowsVista = 25,
        /// <summary>
        /// Windows 95.
        /// </summary>
        Windows95 = 10,
        /// <summary>
        /// Windows 98.
        /// </summary>
        Windows98 = 12,
        /// <summary>
        /// Windows Me.
        /// </summary>
        Windows98Me = 14,
        /// <summary>
        /// Windows 98 SE.
        /// </summary>
        Windows98Se = 13,
        /// <summary>
        /// Windows CE.
        /// </summary>
        WindowsCE = 30,
        /// <summary>
        /// Windows NT 3.x.
        /// </summary>
        WindowsNT3x = 20,
        /// <summary>
        /// Windows NT 4.
        /// </summary>
        WindowsNT4 = 21,
        /// <summary>
        /// Windows XP.
        /// </summary>
        WindowsXP = 23
    }

    #endregion

    /// <summary>
    /// Campos estticos que muestran informacin relativa al sistema operativo.
    /// </summary>
    [SuppressUnmanagedCodeSecurity]
    public static class OsInfo
    {
        /// <summary>
        /// Obtiene informacin del sistema.
        /// </summary>
        /// <param name="ver">Estructura OSVERSIONINFO u OSVERSIONINFOEX.</param>
        /// <returns>True si ok.</returns>
        [DllImport(Api.KERNEL32, ExactSpelling = true, SetLastError = false)]
        private static unsafe extern bool GetVersionExA(byte* ver);


        #region Campos

        /// <summary>
        /// Cadena de Service Pack.
        /// </summary>
        public static readonly string ServicePack;

        /// <summary>
        /// Cadena descriptiva del Sistema.
        /// </summary>
        public static readonly string System;

        /// <summary>
        /// Nmero de Compilacin.
        /// </summary>
        public static readonly int VersionBuild;
        /// <summary>
        /// Nmero de Versin Mayor.
        /// </summary>
        public static readonly int VersionMajor;
        /// <summary>
        /// Nmero de Versin Menor.
        /// </summary>
        public static readonly int VersionMinor;

        /// <summary>
        /// Versin de Service Pack.
        /// </summary>
        public static readonly short ServicePackMajor;
        /// <summary>
        /// Versin menor de Service Pack.
        /// </summary>
        public static readonly short ServicePackMinor;

        /// <summary>
        /// Suite de Windows.
        /// </summary>
        public static readonly SuiteWindows Suite;

        /// <summary>
        /// True si es Dominio NT.
        /// </summary>
        public static readonly bool NTDomain;
        /// <summary>
        /// True si es Servidor.
        /// </summary>
        public static readonly bool NTServer;
        /// <summary>
        /// True si es Estacin de Trabajo NT.
        /// </summary>
        public static readonly bool NTWorkStation;
        /// <summary>
        /// True si es Windows 95 / 98 / Me.
        /// </summary>
        public static readonly bool Win9x;
        /// <summary>
        /// True si es Windows CE.
        /// </summary>
        public static readonly bool WinCE;
        /// <summary>
        /// True si es Windows NT / 2k / XP o superior.
        /// </summary>
        public static readonly bool WinNT;
        /// <summary>
        /// True si es Windows 2000 o superior.
        /// </summary>
        public static readonly bool Minimum2K;
        /// <summary>
        /// True si es Windows XP o superior.
        /// </summary>
        public static readonly bool MinimumXP;
        /// <summary>
        /// Producto Windows.
        /// </summary>
        public static readonly WindowsProduct Windows;

        #endregion

        #region Propiedades

        /// <summary>
        /// Cadena de la versin de Windows.
        /// </summary>
        public static string Version
        {
            get
            {
                if (VersionBuild != 0)
                {
                    if (ServicePack.Length != 0) return VersionMajor.ToString() + "." + VersionMinor.ToString() + "." + VersionBuild.ToString() + " " + ServicePack;
                    return VersionMajor.ToString() + "." + VersionMinor.ToString() + "." + VersionBuild.ToString();
                }
                if (ServicePack.Length != 0) return VersionMajor.ToString() + "." + VersionMinor.ToString() + " " + ServicePack;
                return VersionMajor.ToString() + "." + VersionMinor.ToString();
            }
        }

        /// <summary>
        /// True si es Back Office Suite.
        /// </summary>
        public static bool SuiteBackOffice { get { return (Suite & SuiteWindows.BackOffice) != 0; } }
        /// <summary>
        /// True si es Edicin Web.
        /// </summary>
        public static bool SuiteBlade { get { return (Suite & SuiteWindows.Blade) != 0; } }
        /// <summary>
        /// True si es Communications Suite.
        /// </summary>
        public static bool SuiteCommunications { get { return (Suite & SuiteWindows.Communications) != 0; } }
        /// <summary>
        /// True si es Data Center.
        /// </summary>
        public static bool SuiteDataCenter { get { return (Suite & SuiteWindows.DataCenter) != 0; } }
        /// <summary>
        /// True si es NT Embedded.
        /// </summary>
        public static bool SuiteEmbeddedNT { get { return (Suite & SuiteWindows.EmbeddedNT) != 0; } }
        /// <summary>
        /// True si es NT Embedded restringido.
        /// </summary>
        public static bool SuiteEmbeddedRestricted { get { return (Suite & SuiteWindows.EmbeddedRestricted) != 0; } }
        /// <summary>
        /// True si es Enterprise.
        /// </summary>
        public static bool SuiteEnterprise { get { return (Suite & SuiteWindows.Enterprise) != 0; } }
        /// <summary>
        /// True si es Edicin Personal o Home.
        /// </summary>
        public static bool SuitePersonal { get { return (Suite & SuiteWindows.Personal) != 0; } }
        /// <summary>
        /// True si es Security Appliance Suite.
        /// </summary>
        public static bool SuiteSecurityAppliance { get { return (Suite & SuiteWindows.SecurityAppliance) != 0; } }
        /// <summary>
        /// True si es Terminal Server para un solo usuario.
        /// </summary>
        public static bool SuiteSingleUserTS { get { return (Suite & SuiteWindows.SingleUserTS) != 0; } }
        /// <summary>
        /// True si es Small Business.
        /// </summary>
        public static bool SuiteSmallBusiness { get { return (Suite & SuiteWindows.SmallBusiness) != 0; } }
        /// <summary>
        /// True si es Small Business restringido.
        /// </summary>
        public static bool SuiteSmallBusinessRestricted { get { return (Suite & SuiteWindows.SmallBusinessRestricted) != 0; } }
        /// <summary>
        /// True si es Terminal Server.
        /// </summary>
        public static bool SuiteTerminal { get { return (Suite & SuiteWindows.Terminal) != 0; } }

        #endregion


        static OsInfo()
        {
            /*
        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        internal sealed class OSVERSIONINFO
        {
            internal int OSVersionInfoSize;
            internal int MajorVersion=0;
            internal int MinorVersion=0;
            internal int BuildNumber=0;
            internal int PlatformId=0;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 0x80)]
            internal string CSDVersion=null;
            internal OSVERSIONINFO()
            {
                this.OSVersionInfoSize = Marshal.SizeOf(this);
            }
        }

        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Auto)]
        internal sealed class OSVERSIONINFOEX
        {
            internal int OSVersionInfoSize;
            internal int MajorVersion=0;
            internal int MinorVersion=0;
            internal int BuildNumber=0;
            internal int PlatformId=0;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 0x80)]
            internal string CSDVersion=null;
            internal short ServicePackMajor=0;
            internal short ServicePackMinor=0;
            internal short SuiteMask=0;
            internal byte ProductType=0;
            internal byte Reserved=0;
            internal OSVERSIONINFOEX()
            {
                this.OSVersionInfoSize = Marshal.SizeOf(this);
            }
        }
        */
            unsafe
            {
                byte* p = stackalloc byte[156];
                *(int*)p = 156;
                *(int*)(p + 20) = 0;

                if (GetVersionExA(p))
                {
                    ServicePackMajor = *(short*)(p + 148);
                    ServicePackMinor = *(short*)(p + 150);
                    Suite = (SuiteWindows)(*(short*)(p + 152));
                    switch (p[154])
                    {
                        case 1:
                            NTWorkStation = true;
                            NTDomain = false;
                            NTServer = false;
                            break;
                        case 2:
                            NTWorkStation = false;
                            NTDomain = true;
                            NTServer = false;
                            break;
                        case 3:
                            NTWorkStation = false;
                            NTDomain = false;
                            NTServer = true;
                            break;
                        default:
                            NTWorkStation = false;
                            NTDomain = false;
                            NTServer = false;
                            break;
                    }
                }
                else
                {
                    *(int*)p = 148;
                    if (!GetVersionExA(p)) throw new PlatformNotSupportedException(Errors.NO_PLATFORM);
                    ServicePackMajor = 0;
                    ServicePackMinor = 0;
                    Suite = SuiteWindows.Undefined;
                    NTWorkStation = false;
                    NTDomain = false;
                    NTServer = false;
                }

                VersionMajor = *(int*)(p + 4);
                VersionMinor = *(int*)(p + 8);
                VersionBuild = *(int*)(p + 12);
                switch (*(int*)(p + 16))
                {
                    case 1:
                        Win9x = true;
                        WinNT = false;
                        WinCE = false;
                        break;
                    case 2:
                        Win9x = false;
                        WinNT = true;
                        WinCE = false;
                        break;
                    case 3:
                        Win9x = false;
                        WinNT = false;
                        WinCE = true;
                        break;
                    default:
                        Win9x = false;
                        WinNT = false;
                        WinCE = false;
                        break;
                }

                if (p[20] == 0) ServicePack = Others.Empty;
                else
                {
                    char* pc = stackalloc char[128];
                    p += 20;
                    int a = 0;
                    do
                    {
                        pc[a] = (char)p[a];
                    } while (p[++a] != 0 && a != 128);
                    ServicePack = Mutables.Create(pc,a);
                }
            }

            if (WinNT)
            {
                if (VersionMajor > 4)
                {
                    Minimum2K = true;
                    if (VersionMajor > 5)
                    {
                        MinimumXP = true;
                        System = "Windows Vista";
                        Windows = WindowsProduct.WindowsVista;
                        return;
                    }

                    if (VersionMinor >= 2)
                    {
                        MinimumXP = true;
                        System = "Windows 2003 Server";
                        Windows = WindowsProduct.Windows2003Server;
                        return;
                    }
                    if (VersionMinor == 1)
                    {
                        MinimumXP = true;
                        System = "Windows XP";
                        Windows = WindowsProduct.WindowsXP;
                        return;
                    }
                    MinimumXP = false;
                    System = "Windows 2000";
                    Windows = WindowsProduct.Windows2000;
                    return;
                }
                Minimum2K = false;
                MinimumXP = false;
                if (VersionMajor == 4)
                {
                    System = "Windows NT 4";
                    Windows = WindowsProduct.WindowsNT4;
                    return;
                }
                System = "Windows NT 3.x";
                Windows = WindowsProduct.WindowsNT3x;
                return;
            }

            Minimum2K = false;
            MinimumXP = false;

            if (Win9x)
            {
                if (VersionMinor == 10)
                {
                    if (ServicePack.Length > 1 && ServicePack[1] == 'A')
                    {
                        System = "Windows 98 SE";
                        Windows = WindowsProduct.Windows98Se;
                        return;
                    }
                    System = "Windows 98";
                    Windows = WindowsProduct.Windows98;
                    return;
                }
                if (VersionMinor == 90)
                {
                    System = "Windows 98 ME";
                    Windows = WindowsProduct.Windows98Me;
                    return;
                }
                System = "Windows 95";
                Windows = WindowsProduct.Windows95;
                return;
            }

            if (WinCE)
            {
                System = "Windows CE";
                Windows = WindowsProduct.WindowsCE;
                return;
            }

            System = Errors.INDETERMINATE;
            Windows = WindowsProduct.Indeterminate;
        }

    }
}
