using System;
using EnPalermo.Lit;
using Microsoft.Win32;
using EnPalermo.Txt;
using EnPalermo.Mem;
namespace EnPalermo.Sys
{
    /// <summary>
    /// Wrappers de funciones para uso con el registro.
    /// </summary>
    public static class Registry
    {
        /// <summary>
        /// Obtiene una clave base del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="hive">Hive del registro.</param>
        /// <returns>Clave base.</returns>
        public static RegistryKey GetKey(RegistryHive hive)
        {
            switch (hive)
            {
                case Microsoft.Win32.RegistryHive.ClassesRoot:
                    return Microsoft.Win32.Registry.ClassesRoot;
                case Microsoft.Win32.RegistryHive.CurrentConfig:
                    return Microsoft.Win32.Registry.CurrentConfig;
                case Microsoft.Win32.RegistryHive.CurrentUser:
                    return Microsoft.Win32.Registry.CurrentUser;
                case Microsoft.Win32.RegistryHive.DynData:
                    return Microsoft.Win32.Registry.DynData;
                case Microsoft.Win32.RegistryHive.LocalMachine:
                    return Microsoft.Win32.Registry.LocalMachine;
                case Microsoft.Win32.RegistryHive.PerformanceData:
                    return Microsoft.Win32.Registry.PerformanceData;
                case Microsoft.Win32.RegistryHive.Users:
                    return Microsoft.Win32.Registry.Users;
                default:
                    throw new ArgumentException(Errors.INVALID_ENUM,"hive");
            }
        }

        /// <summary>
        /// Obtiene una clave base del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="computer">Nombre de la computadora (si es null, cadena vacia, "." o "\\." se la considerar computadora actual).</param>
        /// <param name="hive">Hive del registro.</param>
        /// <returns>Clave o null si la computadora no existe o no es accesible.</returns>
        public static RegistryKey GetKey(string computer, RegistryHive hive)
        {
            if (Info.IsMyComputer(computer))
                return GetKey(hive);
            try
            {
                return RegistryKey.OpenRemoteBaseKey(hive, computer);
            }
            catch(System.ArgumentException)
            {
                return null;
            }
            catch ( System.Security.SecurityException)
            {
                return null;
            }
            catch (System.UnauthorizedAccessException)
            {
                return null;
            }            
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// Obtiene o crea una clave del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="hive">Hive del registro.</param>
        /// <param name="key">Clave.</param>
        /// <param name="writeMode">Modo de escritura. Tmbien implica que si la clave no existe esta ser creada.</param>
        /// <returns>Clave o null si la clave no existe, no es accesible o no puede ser creada.</returns>
        public static RegistryKey GetKey(RegistryHive hive, string key, bool writeMode)
        {
            return GetKey(GetKey(hive),key,writeMode);
        }

        /// <summary>
        /// Obtiene o crea una clave del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="hive">Hive del registro.</param>
        /// <param name="computer">Nombre de la computadora (si es null, cadena vacia, "." o "\\." se la considerar computadora actual).</param>
        /// <param name="key">Clave.</param>
        /// <param name="writeMode">Modo de escritura. Tmbien implica que si la clave no existe esta ser creada.</param>
        /// <returns>Clave o null si la clave no existe, no es accesible o no puede ser creada.</returns>
        public static RegistryKey GetKey(string computer, RegistryHive hive, string key, bool writeMode)
        {
            RegistryKey ret=GetKey(computer,hive);
            if (ret==null)
            {
                if (key==null || key.Length==0)
                    throw new ArgumentException(Errors.INVALID_KEY,"key");
                return null;
            }
            return GetKey(ret,key,writeMode);
        }

       

        
        /// <summary>
        /// Obtiene o crea una clave del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave (absoluta).</param>
        /// <param name="writeMode">Modo de escritura. Tmbien implica que si la clave no existe esta ser creada.</param>
        /// <returns>Clave o null si la clave no existe, no es accesible o no puede ser creada.</returns>
        public static RegistryKey GetKey(string key, bool writeMode)
        {
            int l=key==null?0:key.Length;
            if (l<4)
            {
                if (l==3 && TxtComp.EqualNCInvariant(key,"HKU",false))
                    return Microsoft.Win32.Registry.Users;
                throw new ArgumentException(Errors.INVALID_KEY,"key");
            }

            unsafe
            {
                string computer=null;
                string subkey=null;
                int ubi;
                fixed(char*pk=key)
                {
                    char*p=pk;
                otra:
                    if ((*p=='H' || *p=='h') && (p[1]=='K' || p[1]=='k'))
                        goto test;
                    if (*p=='\\')
                    {
                        if (--l>2)
                        {
                            p++;
                            goto otra;
                        }
                        throw new ArgumentException(Errors.INVALID_KEY,"key");
                    }
                    if (l<5)
                        throw new ArgumentException(Errors.INVALID_KEY,"key");
                    ubi=Comp.IndexOf(p+1,l-4,'\\');
                    if (ubi==-1)
                        throw new ArgumentException(Errors.INVALID_KEY,"key");
                    computer=Mutables.Create(p,++ubi);
                    p+=++ubi;
                    l-=ubi;
                    if ((*p=='H' || *p=='h') && (p[1]=='K' || p[1]=='k'))
                        goto test;
                    throw new ArgumentException(Errors.INVALID_KEY,"key");
                test:
                    l-=2;
                    p+=2;
                    ubi=Comp.IndexOf(p+1,l-1,'\\');
                    if (ubi!=-1)
                    {
                        if (ubi!=l-2)
                        {
                            subkey=Mutables.Create(p+(ubi+=2),l-ubi);
                            l=ubi-1;
                        }
                        else
                            l--;
                    }

                    RegistryHive hive;
                    char c=*p;
                    switch(l)
                    {
                        case 1:
                            if (c=='U' || c=='u')
                            {
                                hive=Microsoft.Win32.RegistryHive.Users;
                                break;
                            }
                            goto default;
                        case 2:
                        switch(c)
                        {
                            case 'C':
                            case 'c':
                                c=p[1];
                                if (c=='U' || c=='u')
                                {
                                    hive=Microsoft.Win32.RegistryHive.CurrentUser;
                                    break;
                                }
                                else if (c=='R' || c=='r')
                                {
                                    hive=Microsoft.Win32.RegistryHive.ClassesRoot;
                                    break;
                                }
                                else if (c=='C' || c=='c')
                                {
                                    hive=Microsoft.Win32.RegistryHive.CurrentConfig;
                                    break;
                                }
                                goto default;

                            case 'L':
                            case 'l':
                                if (p[1]=='M' || p[1]=='m')
                                {
                                    hive=Microsoft.Win32.RegistryHive.LocalMachine;
                                    break;
                                }
                                goto default;

                            case 'P':
                            case 'p':
                                if (p[1]=='D' || p[1]=='d')
                                {
                                    hive=Microsoft.Win32.RegistryHive.PerformanceData;
                                    break;
                                }
                                goto default;

                            case 'D':
                            case 'd':
                                if (p[1]=='D' || p[1]=='d')
                                {
                                    hive=Microsoft.Win32.RegistryHive.DynData;
                                    break;
                                }
                                goto default;

                            default:
                                throw new ArgumentException(Errors.INVALID_KEY,"key");
                        }
                            break;

                        case 8:
                        case 11:
                        case 15:
                        case 16:
                        case 17:
                        case 19:
                            if ((c=='E' || c=='e') && (p[1]=='Y' || p[1]=='y') && p[2]=='_')
                            {
                                p+=3;
                                switch(l-=3)
                                {
                                    case 5:
                                        if ((*p=='U' || *p=='u') && (p[1]=='S' || p[1]=='s') && (p[2]=='E' || p[2]=='e') && (p[3]=='R' || p[3]=='r') && (p[4]=='S' || p[4]=='s'))
                                        {
                                            hive=Microsoft.Win32.RegistryHive.Users;
                                            break;
                                        }
                                        goto default;
                                    case 8:
                                        if ((*p=='D' || *p=='d') && (p[1]=='Y' || p[1]=='y') && (p[2]=='N' || p[2]=='n') && (p[3]=='_') && (p[4]=='D' || p[4]=='d') && (p[5]=='A' || p[5]=='a') && (p[6]=='T' || p[6]=='t') && (p[7]=='A' || p[7]=='a'))
                                        {
                                            hive=Microsoft.Win32.RegistryHive.DynData;
                                            break;
                                        }
                                        goto default;
                                    case 12:
                                        if ((*p=='C' || *p=='c'))
                                        {
                                            if ((p[1]=='U' || p[1]=='u') && (p[2]=='R' || p[2]=='r') && (p[3]=='R' || p[3]=='r') && (p[4]=='E' || p[4]=='e') && (p[5]=='N' || p[5]=='n') && (p[6]=='T' || p[6]=='t') && (p[7]=='_') && (p[8]=='U' || p[8]=='u') && (p[9]=='S' || p[9]=='s') && (p[10]=='E' || p[10]=='e') && (p[11]=='R' || p[11]=='r'))
                                            {
                                                hive=Microsoft.Win32.RegistryHive.CurrentUser;
                                                break;
                                            }
                                            if ((p[1]=='L' || p[1]=='l') && (p[2]=='A' || p[2]=='a') && (p[3]=='S' || p[3]=='s') && (p[4]=='S' || p[4]=='s') && (p[5]=='E' || p[5]=='e') && (p[6]=='S' || p[6]=='s') && (p[7]=='_') && (p[8]=='R' || p[8]=='r') && (p[9]=='O' || p[9]=='o') && (p[10]=='O' || p[10]=='o') && (p[11]=='T' || p[11]=='t'))
                                            {
                                                hive=Microsoft.Win32.RegistryHive.ClassesRoot;
                                                break;
                                            }                                        
                                        }
                                        
                                        goto default;
                                    case 13:
                                        if ((*p=='L' || *p=='L') && (p[1]=='O' || p[1]=='o') && (p[2]=='C' || p[2]=='c') && (p[3]=='A' || p[3]=='a') && (p[4]=='L' || p[4]=='l') && (p[5]=='_') && (p[6]=='M' || p[6]=='m') && (p[7]=='A' || p[7]=='a') && (p[8]=='C' || p[8]=='c') && (p[9]=='H' || p[9]=='h') && (p[10]=='I' || p[10]=='i') && (p[11]=='N' || p[11]=='n') && (p[12]=='E' || p[12]=='e'))
                                        {
                                            hive=Microsoft.Win32.RegistryHive.LocalMachine;
                                            break;
                                        }
                                        goto default;
                                    case 14:
                                        if ((*p=='C' || *p=='c') && (p[1]=='U' || p[1]=='u') && (p[2]=='R' || p[2]=='r') && (p[3]=='R' || p[3]=='r') && (p[4]=='E' || p[4]=='e') && (p[5]=='N' || p[5]=='n') && (p[6]=='T' || p[6]=='t') && (p[7]=='_') && (p[8]=='C' || p[8]=='c') && (p[9]=='O' || p[9]=='o') && (p[10]=='N' || p[10]=='n') && (p[11]=='F' || p[11]=='f') && (p[12]=='I' || p[12]=='i') && (p[13]=='G' || p[13]=='g'))
                                        {
                                            hive=Microsoft.Win32.RegistryHive.CurrentConfig;
                                            break;
                                        }
                                        goto default;
                                    case 16:
                                        if ((*p=='P' || *p=='p') && (p[1]=='E' || p[1]=='e') && (p[2]=='R' || p[2]=='r') && (p[3]=='F' || p[3]=='f') && (p[4]=='O' || p[4]=='o') && (p[5]=='R' || p[5]=='r') && (p[6]=='M' || p[6]=='m') && (p[7]=='A' || p[7]=='a') && (p[8]=='N' || p[8]=='n') && (p[9]=='C' || p[9]=='c') && (p[10]=='E' || p[10]=='e') && (p[11]=='_') && (p[12]=='D' || p[12]=='d') && (p[13]=='A' || p[13]=='a') && (p[14]=='T' || p[14]=='t') && (p[15]=='A' || p[15]=='a'))
                                        {
                                            hive=Microsoft.Win32.RegistryHive.PerformanceData;
                                            break;
                                        }
                                        goto default;
                                    default:
                                        throw new ArgumentException(Errors.INVALID_KEY,"key");
                                }

                            }
                            else 
                                goto default;
                            break;
                        default:
                            throw new ArgumentException(Errors.INVALID_KEY,"key");
                    }

                    if (subkey==null)
                    {
                        if (computer==null)
                            return GetKey(hive);
                        return GetKey(computer,hive);
                    }
                    if (computer==null)
                        return GetKey(hive,subkey,writeMode);
                    return GetKey(computer,hive,subkey,writeMode);
                }
            }
        }

        /// <summary>
        /// Obtiene o crea una clave del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="baseKey">Clave base.</param>
        /// <param name="key">Clave.</param>
        /// <param name="writeMode">Modo de escritura. Tmbien implica que si la clave no existe esta ser creada.</param>
        /// <returns>Clave o null si la clave no existe, no es accesible o no puede ser creada.</returns>
        public static RegistryKey GetKey(RegistryKey baseKey, string key, bool writeMode)
        {
            if (baseKey==null)
                throw new ArgumentNullException("baseKey",Errors.NULL_OBJECT);
            if (key==null || key.Length==0)
                throw new ArgumentException(Errors.INVALID_KEY,"key");

            try
            {
                return writeMode?(baseKey.CreateSubKey(key)):(baseKey.OpenSubKey(key,false));
            }
            catch (System.UnauthorizedAccessException)
            {
                return null;
            }
            catch ( System.Security.SecurityException)
            {
                return null;
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// Elimina una subclave en forma recursiva.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="baseKey">Clave base (debe estar habilitada para escritura).</param>
        /// <param name="key">Clave.</param>
        /// <returns>True si la sub clave no existe o fue eliminada.</returns>
        public static bool DeleteSubKeyTree(RegistryKey baseKey, string key)
        {
            if (baseKey==null)
                throw new ArgumentNullException("baseKey",Errors.NULL_OBJECT);
            if (key==null || key.Length==0)
                throw new ArgumentException(Errors.INVALID_KEY,"key");

            try
            {
                baseKey.DeleteSubKeyTree(key);
                return true;
            }
            catch(System.ArgumentException)
            {
                return true;
            }
            catch(System.Security.SecurityException)
            {
                return false;
            }
            catch(System.UnauthorizedAccessException)
            {
                return false;
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// Elimina un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <returns>True si el valor no existe o fue eliminado.</returns>
        public static bool DeleteValue(RegistryKey key, string val)
        {
            if (key==null)
                throw new ArgumentNullException("key",Errors.NULL_OBJECT);
            if (val==null)
                throw new ArgumentNullException("val",Errors.NULL_STRING);

            try
            {
                key.DeleteValue(val,false);
                return true;
            }
            catch(System.Security.SecurityException)
            {
                return false;
            }
            catch(System.UnauthorizedAccessException)
            {
                return false;
            }
            catch
            {
                throw;
            }
        }


        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        private static object getValue(RegistryKey key, string val, object def)
        {
            if (key==null)
                throw new ArgumentNullException("key",Errors.NULL_OBJECT);
            if (val==null)
                throw new ArgumentNullException("val",Errors.NULL_STRING);
            try
            {
                return key.GetValue(val,def);
            }
            catch(System.Security.SecurityException)
            {
                return def;
            }
            catch (System.IO.IOException)
            {
                return def;
            }
            catch (System.ArgumentException)
            {
                return def;
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static byte[] GetValue(RegistryKey key, string val, byte[] def)
        {
            return getValue(key,val,def as object) as byte[];
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static string GetValue(RegistryKey key, string val, string def)
        {
            return getValue(key,val,def as object) as string;
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static string[] GetValue(RegistryKey key, string val, string[] def)
        {
            return getValue(key,val,def as object) as string[];
        }

        /// <summary>
        /// Obtiene un valor del registro, expandiendo variables.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static string GetValueExpand(RegistryKey key, string val, string def)
        {
            def=GetValue(key,val,def);
            if (def!=null)
                def=Environment.ExpandEnvironmentVariables(def);
            return def;
        }

        /// <summary>
        /// Obtiene un valor del registro, expandiendo variables.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static string[] GetValueExpand(RegistryKey key, string val, string[] def)
        {
            def=GetValue(key,val,def);
            if (def!=null)
            {
                for(int a=0;a<def.Length;a++)
                    def[a]=Environment.ExpandEnvironmentVariables(def[a]);
            }
            return def;
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static int GetValue(RegistryKey key, string val, int def)
        {
            object ret=getValue(key,val,null as object);
            if (ret is int)
                return (int)ret;
            return def;
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static uint GetValue(RegistryKey key, string val, uint def)
        {
            object ret=getValue(key,val,null as object);
            if (ret is int)
                return (uint)(int)ret;
            return def;
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static float GetValue(RegistryKey key, string val, float def)
        {
            object ret=getValue(key,val,null as object);
            if (ret is int)
            {
                int i=(int)ret;
                unsafe
                {
                    return *(float*)&i;
                }
            }
            return def;
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static byte GetValue(RegistryKey key, string val, byte def)
        {
            return (byte)GetValue(key,val,(int)def);
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static sbyte GetValue(RegistryKey key, string val, sbyte def)
        {
            return (sbyte)GetValue(key,val,(int)def);
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static bool GetValue(RegistryKey key, string val, bool def)
        {
            return GetValue(key,val,def?1:0)!=0?true:false;
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static short GetValue(RegistryKey key, string val, short def)
        {
            return (short)GetValue(key,val,(int)def);
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static ushort GetValue(RegistryKey key, string val, ushort def)
        {
            return (ushort)GetValue(key,val,(int)def);
        }


        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static ushort GetValue(RegistryKey key, string val, char def)
        {
            return (char)GetValue(key,val,(int)def);
        }


        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static long GetValue(RegistryKey key, string val, long def)
        {
            object ret=getValue(key,val,null as object);
            if (ret is byte[])
            {
                byte[] b = ret as byte[];
                if(b.Length!=8)
                    return def;
                unsafe
                {
                    fixed(byte*p=b)
                        return *(long*)p;
                }
            }
            return def;
        }


        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static ulong GetValue(RegistryKey key, string val, ulong def)
        {
            object ret=getValue(key,val,null as object);
            if (ret is byte[])
            {
                byte[] b = ret as byte[];
                if(b.Length!=8)
                    return def;
                unsafe
                {
                    fixed(byte*p=b)
                        return *(ulong*)p;
                }
            }
            return def;
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static double GetValue(RegistryKey key, string val, double def)
        {
            object ret=getValue(key,val,null as object);
            if (ret is byte[])
            {
                byte[] b = ret as byte[];
                if(b.Length!=8)
                    return def;
                unsafe
                {
                    fixed(byte*p=b)
                        return *(double*)p;
                }
            }
            return def;
        }
 
        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static decimal GetValue(RegistryKey key, string val, decimal def)
        {
            object ret=getValue(key,val,null as object);
            if (ret is byte[])
            {
                byte[] b = ret as byte[];
                if(b.Length!=16)
                    return def;
                unsafe
                {
                    fixed(byte*p=b)
                        return *(decimal*)p;
                }
            }
            return def;
        }

        /// <summary>
        /// Obtiene un valor del registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="def">
        /// Valor por defecto.
        /// Retorna def si el valor no existe u ocurri un error no critico.
        /// </param>
        /// <returns>
        /// Retorna def si el valor no existe u ocurri un error no critico,
        /// De lo contrario, devuelve el contenido del valor.
        /// </returns>
        public static DateTime GetValue(RegistryKey key, string val, DateTime def)
        {
            try
            {
                return new DateTime(GetValue(key,val,def.Ticks));
            }
            catch
            {
                return def;
            }
            
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="objValue">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        private static bool writeValue(RegistryKey key, string val, object objValue)
        {
            if (key==null)
                throw new ArgumentNullException("key",Errors.NULL_OBJECT);
            if (val==null)
                throw new ArgumentNullException("val",Errors.NULL_STRING);
            if (objValue==null)
                throw new ArgumentNullException("objValue",Errors.NULL_OBJECT);

            try
            {
                key.SetValue(val,objValue);
                return true;
            }
            catch(System.Security.SecurityException)
            {
                return false;
            }
            catch(System.UnauthorizedAccessException)
            {
                return false;
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, int valSet)
        {
            return writeValue(key,val,valSet);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, bool valSet)
        {
            return writeValue(key,val,valSet?1:0);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, byte valSet)
        {
            return writeValue(key,val,(int)valSet);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, sbyte valSet)
        {
            return writeValue(key,val,(int)valSet);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, short valSet)
        {
            return writeValue(key,val,(int)valSet);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, ushort valSet)
        {
            return writeValue(key,val,(int)valSet);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, char valSet)
        {
            return writeValue(key,val,(int)valSet);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, uint valSet)
        {
            return writeValue(key,val,(int)valSet);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, float valSet)
        {
            unsafe
            {
                return writeValue(key,val,*(int*)&valSet);
            }
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, long valSet)
        {
            unsafe
            {
                byte[] b= new byte[8];
                fixed(byte*p=b)
                    *(long*)p=valSet;
                return writeValue(key,val,b);
            }
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, ulong valSet)
        {
            unsafe
            {
                byte[] b= new byte[8];
                fixed(byte*p=b)
                    *(ulong*)p=valSet;
                return writeValue(key,val,b);
            }
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, double valSet)
        {
            unsafe
            {
                byte[] b= new byte[8];
                fixed(byte*p=b)
                    *(double*)p=valSet;
                return writeValue(key,val,b);
            }
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, DateTime valSet)
        {
            unsafe
            {
                byte[] b= new byte[8];
                fixed(byte*p=b)
                    *(DateTime*)p=valSet;
                return writeValue(key,val,b);
            }
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, decimal valSet)
        {
            unsafe
            {
                byte[] b= new byte[16];
                fixed(byte*p=b)
                    *(decimal*)p=valSet;
                return writeValue(key,val,b);
            }
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, string valSet)
        {
            return writeValue(key,val,valSet);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, string[] valSet)
        {
            return writeValue(key,val,valSet);
        }

        /// <summary>
        /// Escribe un valor en el registro.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <param name="val">Valor.</param>
        /// <param name="valSet">Contenido del valor a escribr en el registro.</param>
        /// <returns>True si el valor fue escrito.</returns>
        public static bool WriteValue(RegistryKey key, string val, byte[] valSet)
        {
            return writeValue(key,val,valSet);
        }

        
        /// <summary>
        /// Obtiene un array con los nombres de subclaves.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <returns>Array con subclaves o null si ocurri un error no critico.</returns>
        public static string[] GetSubKeyNames(RegistryKey key)
        {
            if (key==null)
                throw new ArgumentNullException("key",Errors.NULL_OBJECT);
            try
            {
                return key.GetSubKeyNames();
            }
            catch(System.Security.SecurityException)
            {
                return null;
            }
            catch
            {
                throw;
            }
        }

        /// <summary>
        /// Obtiene un array con los nombres de valores.
        /// Esta funcin no genera excepciones de apertura (aunque si de argumentos o crticas).
        /// </summary>
        /// <param name="key">Clave.</param>
        /// <returns>Array con nombre de valores o null si ocurri un error no critico.</returns>
        public static string[] GetValueNames(RegistryKey key)
        {
            if (key==null)
                throw new ArgumentNullException("key",Errors.NULL_OBJECT);
            try
            {
                return key.GetValueNames();
            }
            catch(System.Security.SecurityException)
            {
                return null;
            }
            catch
            {
                throw;
            }
        }

    }
}
