using System;
using System.Text;
using System.IO;
using System.Security;
using System.Runtime.InteropServices;
using EnPalermo.Sys;
using EnPalermo.Lit;
using EnPalermo.Col;
using EnPalermo.Mem;
using EnPalermo.Uti;

namespace EnPalermo.Txt
{
	/// <summary>
	/// Representa un cdigo de pgina especifico.
	/// </summary>
    [StructLayout(LayoutKind.Sequential)]
	public unsafe struct CodePage:IComparable<CodePage>,IEquatable<CodePage>
	{

        #region Campos

        /// <summary>
        /// Cdigo de pgina.
        /// Si es cero indica que el cdigo de pgina es invlido.
        /// </summary>
        private ushort codePage;


        /// <summary>
        /// Tamao mximo de bytes por carcter.
        /// Si es cero o mayor a 15, indica que el cdigo de pgina no es soportado.
        /// Por esta razn, este campo no debe exportarse fuera de este equipo.
        /// </summary>
        private ushort codeSize;

        #endregion

        #region Pginas de cdigo de uso frecuente.

        /// <summary>
        /// CodePage Cero.
        /// </summary>
        public static readonly CodePage Zero;

        /// <summary>
        /// Pgina Ansi del sistema.
        /// </summary>
        public static readonly CodePage AnsiDefault;

        /// <summary>
        /// Pgina Oem del sistema.
        /// </summary>
        public static readonly CodePage OemDefault;

        /// <summary>
        /// Pgina Mac del sistema (puede ser invlida si no es soportada en este sistema).
        /// </summary>
        public static readonly CodePage MacDefault;

        /// <summary>
        /// Pgina Oem Latin1 (puede ser invlida si no es soportada en este sistema).
        /// </summary>
        public static readonly CodePage Latin1Oem;
        
        /// <summary>
        /// Pgina Unicode Little Endian de 2 bytes. Arquitectura x86.
        /// </summary>
        public static readonly CodePage UnicodeLE;
        
        /// <summary>
        /// Pgina Unicode Big Endian de 2 bytes. Algunas arquitecturas no Intel.
        /// </summary>
        public static readonly CodePage UnicodeBE;
        
        
        /// <summary>
        /// Pgina Ansi Latin1.
        /// </summary>
        public static readonly CodePage Latin1Ansi;
        
        /// <summary>
        /// Pgina Ascii (solo 7 bit).
        /// </summary>
        public static readonly CodePage Ascii;
        
        /// <summary>
        /// Pgina Unicode UTF7 (puede ser invlida si no es soportada en este sistema).
        /// </summary>
        public static readonly CodePage UTF7;
        
        /// <summary>
        /// Pgina Unicode UTF8 (puede ser invlida si no es soportada en este sistema).
        /// </summary>
        public static readonly CodePage UTF8;

        /// <summary>
        /// Pgina Ansi correspondiente a la tarea actual.
        /// </summary>
        public static CodePage AnsiCurrent
        {
            get
            {
                return new CodePage(CP_ThreadAnsi);
            }
        }

        /// <summary>
        /// Pgina Oem correspondiente a la tarea actual.
        /// </summary>
        public static CodePage OemCurrent
        {
            get
            {
                return new CodePage(CP_ThreadOem);
            }
        }

        /// <summary>
        /// Pgina Mac correspondiente a la tarea actual.
        /// </summary>
        public static CodePage MacCurrent
        {
            get
            {
                CodePage cp = new CodePage(System.Threading.Thread.CurrentThread.CurrentCulture.TextInfo.MacCodePage);
                if (!cp.IsSupported)
                    return MacDefault;
                return cp;
            }
        }



        #endregion

        #region Prembulos Unicode

        /// <summary>
        /// Prembulo de la codificacin Unicode LittleEndian.
        /// </summary>
        public const char PRE_LittleEndian = '\xFEFF';
        /// <summary>
        /// Prembulo de la codificacin Unicode BigEndian.
        /// </summary>
        public const char PRE_BigEndian = '\xFFFE';

        /// <summary>
        /// Prembulo de la codificacin Unicode LittleEndian de 32 bits.
        /// </summary>
        public const uint PRE_LittleEndian32 = 0x0000FEFF;
        /// <summary>
        /// Prembulo de la codificacin Unicode BigEndian de 32 bits.
        /// </summary>
        public const uint PRE_BigEndian32 = 0xFFFE0000;

        #endregion

        #region Constantes privadas y constructor esttico.

        /// <summary>
        /// Pgina Ansi Latin1.
        /// </summary>
        internal const ushort CP_Latin1Ansi =  1252;

        /// <summary>
        /// Pgina Oem Latin1.
        /// </summary>
        private const ushort CP_Latin1Oem =  850;

        /// <summary>
        /// Pgina Unicode Little Endian de 2 bytes. Arquitectura x86.
        /// </summary>
        private const ushort CP_UnicodeLE =  1200;
        
        /// <summary>
        /// Pgina Unicode Big Endian de 2 bytes. Algunas arquitecturas no Intel.
        /// </summary>
        private const ushort CP_UnicodeBE =  1201;
        
        /// <summary>
        /// Pgina Unicode Little Endian de 4 bytes. Arquitectura x86.
        /// </summary>
        private const ushort CP_Unicode32LE =  12000;
        
        /// <summary>
        /// Pgina Unicode Big Endian de 4 bytes. Algunas arquitecturas no Intel.
        /// </summary>
        private const ushort CP_Unicode32BE =  12001;
        
        /// <summary>
        /// Pgina Ascii (solo 7 bit).
        /// </summary>
        private const ushort CP_Ascii =  20127;
        
        /// <summary>
        /// Pgina Unicode UTF7.
        /// </summary>
        private const ushort CP_UTF7 =  65000;
        
        /// <summary>
        /// Pgina Unicode UTF8.
        /// </summary>
        private const ushort CP_UTF8  = 65001;

        /// <summary>
        /// Pgina Ansi del Sistema (para uso mediante API).
        /// </summary>
        private const ushort CP_SystemAnsi =  0;
        
        /// <summary>
        /// Pgina Oem del Sistema (para uso mediante API).
        /// </summary>
        private const ushort CP_SystemOem =  1;
        
        /// <summary>
        /// Pgina Mac del Sistema (para uso mediante API).
        /// </summary>
        private const ushort CP_SystemMac =  2;
        
        /// <summary>
        /// Pgina Ansi de la tarea actual (para uso mediante API). En sistemas anteriores a Windows 2000, equivale a CP_SystemAnsi.
        /// </summary>
        private const ushort CP_ThreadAnsi = 3;
        
        /// <summary>
        /// Pgina Oem de la tarea actual (no compatible directamente con API). En sistemas anteriores a Windows 2000, equivale a CP_SystemOem.
        /// </summary>
        private const ushort CP_ThreadOem = 4;
        
        /// <summary>
        /// Symbol. Solo Windows 2000 o superior.
        /// </summary>
        private const ushort CP_Symbol= 42;

        /// <summary>
        /// Opcion NoBestFitChars para API.
        /// </summary>
        private const uint NO_BEST_FIT_CHARS = 0x00000400;

        /// <summary>
        /// Chache de cdigos de pgina.
        /// </summary>
        private readonly static UShorts cacheCodePages;

        private static bool isInit;

        static CodePage()
        {
            
            cacheCodePages=new UShorts();
            Zero=new CodePage();
            isInit=false;
            Latin1Ansi=new CodePage((int)CP_Latin1Ansi);
            if (!Latin1Ansi.IsSupported)
                throw new NotSupportedException(Errors.CP_L1_UNSUPPORTED);
            
            AnsiDefault = new CodePage(GetACP());
            OemDefault = new CodePage(GetOEMCP());
            MacDefault = new CodePage((int)CP_SystemMac);
            Latin1Oem=new CodePage((int)CP_Latin1Oem);
            UTF7=new CodePage((int)CP_UTF7);
            UTF8=new CodePage((int)CP_UTF8);
            isInit=true;
            Ascii=new CodePage(CP_Ascii,1);
            UnicodeLE=new CodePage(CP_UnicodeLE,2);
            UnicodeBE=new CodePage(CP_UnicodeBE,2);
            
        }



        #endregion

        #region Constructores

        /// <summary>
        /// Constructor interno para inicializacin directa.
        /// </summary>
        /// <param name="codePage">Cdigo de pgina.</param>
        /// <param name="codeSize">Tamao mximo de bytes por carcter.</param>
        internal CodePage(ushort codePage, ushort codeSize)
        {
            this.codePage=codePage;
            this.codeSize=codeSize;
        }


        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="codePage">
        /// Cdigo de pgina. Si el valor no es soportado por el sistema, se inicializar
        /// como invlido.
        /// </param>
        public CodePage(int codePage)
        {
            if (codePage<0 || codePage>65535)
                throw new ArgumentOutOfRangeException(Texts.Format(Errors.FMT_CP_RANGE,codePage));
            switch(codePage)
            {
                case CP_Latin1Ansi:
                    this.codePage=CP_Latin1Ansi;
                    if (isInit)
                    {
                        this.codeSize=1;
                        return;
                    }
                    break;
                case CP_Ascii:
                    this.codePage=CP_Ascii;
                    this.codeSize=1;
                    return;
                case CP_Latin1Oem:
                    this.codePage=CP_Latin1Oem;
                    if (isInit)
                    {
                        this.codeSize=Latin1Oem.codeSize;
                        return;
                    }
                    break;
                case CP_UTF7:
                    this.codePage=CP_UTF7;
                    if (isInit)
                    {
                        this.codeSize=UTF7.codeSize;
                        return;
                    }
                    break;
                case CP_UTF8:
                    this.codePage=CP_UTF8;
                    if (isInit)
                    {
                        this.codeSize=UTF8.codeSize;
                        return;
                    }
                    break;
                case CP_SystemAnsi:
                    if (isInit)
                    {
                        this.codePage=AnsiDefault.codePage;
                        this.codeSize=AnsiDefault.codeSize;
                        return;
                    }    
                    this.codePage=(ushort)GetACP();
                    break;
                case CP_SystemOem:
                    if (isInit)
                    {
                        this.codePage=OemDefault.codePage;
                        this.codeSize=OemDefault.codeSize;
                        return;
                    }    
                    this.codePage=(ushort)GetACP();
                    break;
                case CP_SystemMac:
                    if (isInit)
                    {
                        this.codePage=MacDefault.codePage;
                        this.codeSize=MacDefault.codeSize;
                        return;
                    }    
                    this.codePage=CP_SystemMac;
                    break;
                case CP_ThreadAnsi:
                    this.codePage=(ushort)System.Threading.Thread.CurrentThread.CurrentCulture.TextInfo.ANSICodePage;
                    if (this.codePage==CP_Latin1Ansi)
                    {
                        this.codeSize=1;
                        return;
                    }
                    if (this.codePage==AnsiDefault.codePage)
                    {
                        this.codeSize=AnsiDefault.codeSize;
                        return;
                    }
                    if (this.codePage==0)
                    {
                        this.codePage=AnsiDefault.codePage;
                        this.codeSize=AnsiDefault.codeSize;
                        return;
                    }
                    break;
                case CP_ThreadOem:
                    this.codePage=(ushort)System.Threading.Thread.CurrentThread.CurrentCulture.TextInfo.OEMCodePage;
                    if (this.codePage==CP_Latin1Oem)
                    {
                        this.codeSize=Latin1Oem.codeSize;
                        return;
                    }
                    if (this.codePage==OemDefault.codePage)
                    {
                        this.codeSize=OemDefault.codeSize;
                        return;
                    }
                    if (this.codePage==0)
                    {
                        this.codePage=OemDefault.codePage;
                        this.codeSize=OemDefault.codeSize;
                        return;
                    }
                    break;
                case CP_Symbol:
                    this.codePage=CP_Symbol;
                    this.codeSize=OsInfo.Minimum2K?(ushort)1:(ushort)0;
                    return;
                case CP_UnicodeLE:
                    this.codePage=CP_UnicodeLE;
                    this.codeSize=2;
                    return;

                case CP_UnicodeBE:
                    this.codePage=CP_UnicodeBE;
                    this.codeSize=2;
                    return;
                case CP_Unicode32LE:
                    this.codePage=CP_Unicode32LE;
                    this.codeSize=4;
                    return;
                case CP_Unicode32BE:
                    this.codePage=CP_Unicode32BE;
                    this.codeSize=4;
                    return;
                default:
                    this.codePage=(ushort)codePage;
                    break;
            }
            System.Threading.Monitor.Enter(cacheCodePages);
            int l=cacheCodePages.Length;
            if (l!=0)
            {
                int a=0;
                fixed(ushort* arr=cacheCodePages.Array)
                {
                    do
                    {
                        if (this.codePage==arr[a])
                        {
                            this.codeSize=arr[a+1];
                            System.Threading.Monitor.Exit(cacheCodePages);
                            return;
                        }
                    }while((a+=2)!=l);
                }
            }
            /*
             * struct _cpinfo { 
             *  UINT MaxCharSize; 
             *  BYTE DefaultChar[2]; 
             *  BYTE LeadByte[12]; 
             * } CPINFO, *LPCPINFO; 
             * */
            ushort* inf = stackalloc ushort[10];
            if (GetCPInfo(codePage,inf)==0)
            {
                System.Threading.Monitor.Exit(cacheCodePages);
                this.codeSize=0;
                return;
            }
            cacheCodePages.AppendItems(this.codePage,*inf);
            System.Threading.Monitor.Exit(cacheCodePages);
            this.codeSize=*inf;
            return;
        }

        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="stream">Stream sobre el cual intentar detectar la firma de cdigo 
        /// de pgina en la posicin actual.</param>
        /// <param name="noZeroSign">
        /// Si false, solo detectar firma si la posicin actual del stream es cero.
        /// </param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        public CodePage(Stream stream, bool noZeroSign, CodePage def)
        {

            long l=stream.Position;
            if (noZeroSign || l==0L)
            {
                l=stream.Length-l;
                if (l>1L)
                {
                    int sign;
                    if (l>5L)
                        sign=5;
                    else
                        sign=(int)l;
                    byte[] bytes=new byte[sign];
                    l=(long)stream.Read(bytes,0,sign);
                    fixed(byte*p=bytes)
                        def=new CodePage(p,(int)l,def, out sign);
                    if ((int)l!=sign)
                        stream.Position=stream.Position-(l+(long)sign);
                }
                
            }
            this.codePage=def.codePage;
            this.codeSize=def.codeSize;
        }

        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="stream">Stream sobre el cual intentar detectar la firma de cdigo 
        /// de pgina en la posicin actual.</param>
        /// <param name="noZeroSign">
        /// Si false, solo detectar firma si la posicin actual del stream es cero.
        /// </param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        /// <param name="sign">Variable de salida. Si es mayor a cero, indica la zona de firma. Si
        /// es cero, indica que se utiliz el cdigo de pgina por defecto.
        /// Un valor diferente a cero tambin indica que el stream se desplaz al byte siguiente a la firma.
        /// </param>
        public CodePage(Stream stream, bool noZeroSign, CodePage def, out int sign)
        {
            long l=stream.Position;
            if (noZeroSign || l==0L)
            {
                l=stream.Length-l;
                if (l>1L)
                {
                    if (l>5L)
                        sign=5;
                    else
                        sign=(int)l;
                    byte[] bytes=new byte[sign];
                    l=(long)stream.Read(bytes,0,sign);
                    fixed(byte*p=bytes)
                        def=new CodePage(p,(int)l,def,out sign);
                    if ((int)l!=sign)
                        stream.Position=stream.Position-(l+(long)sign);
                }
                else
                    sign=0;
            }
            else
                sign=0;
            this.codePage=def.codePage;
            this.codeSize=def.codeSize;
        }


        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="arr">Array sobre el cual intentar detectar firma de cdigo de pgina.</param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        public CodePage(byte[] arr, CodePage def):this(arr,0,arr.Length,def)
        {}

        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="arr">Array sobre el cual intentar detectar firma de cdigo de pgina.</param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        /// <param name="sign">Variable de salida. Si es mayor a cero, indica la zona de firma. Si
        /// es cero, indica que se utiliz el cdigo de pgina por defecto.</param>
        public CodePage(byte[] arr, CodePage def, out int sign):this(arr,0,arr.Length,def,out sign)
        {}


        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="arr">Array sobre el cual intentar detectar firma de cdigo de pgina.</param>
        /// <param name="count">Largo de la zona de deteccin.</param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        public CodePage(byte[] arr, int count, CodePage def):this(arr,0,count,def)
        {}

        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="arr">Array sobre el cual intentar detectar firma de cdigo de pgina.</param>
        /// <param name="count">Largo de la zona de deteccin.</param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        /// <param name="sign">Variable de salida. Si es mayor a cero, indica la zona de firma. Si
        /// es cero, indica que se utiliz el cdigo de pgina por defecto.</param>
        public CodePage(byte[] arr, int count, CodePage def, out int sign):this(arr,0,count,def,out sign)
        {}


        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="arr">Array sobre el cual intentar detectar firma de cdigo de pgina.</param>
        /// <param name="index">Inicio de la zona de deteccin.</param>
        /// <param name="count">Largo de la zona de deteccin.</param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        public CodePage(byte[] arr, int index, int count, CodePage def)
        {
            if (index<0 || count<0 || index+count>arr.Length)
                throw new ArgumentOutOfRangeException("index,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
                fixed(byte*p=arr)
                    def=new CodePage(p+index,count,def);
            this.codePage=def.codePage;
            this.codeSize=def.codeSize;
        }

        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="arr">Array sobre el cual intentar detectar firma de cdigo de pgina.</param>
        /// <param name="index">Inicio de la zona de deteccin.</param>
        /// <param name="count">Largo de la zona de deteccin.</param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        /// <param name="sign">Variable de salida. Si es mayor a cero, indica la zona de firma. Si
        /// es cero, indica que se utiliz el cdigo de pgina por defecto.</param>
        public CodePage(byte[] arr, int index, int count, CodePage def, out int sign)
        {
            if (index<0 || count<0 || index+count>arr.Length)
                throw new ArgumentOutOfRangeException("index,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
                fixed(byte*p=arr)
                    def=new CodePage(p+index,count,def,out sign);
            else
                sign=0;
            this.codePage=def.codePage;
            this.codeSize=def.codeSize;
        }

        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="p">Puntero sobre el cual intentar detectar firma de cdigo de pgina.</param>
        /// <param name="l">Largo del puntero de deteccin.</param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        public CodePage(byte*p, int l, CodePage def)
        {
            switch(l)
            {
                case 0:
                case 1:
                    break;
                case 2:
                switch(*(char*)p)
                {
                    case PRE_LittleEndian:
                        this.codeSize=2;
                        this.codePage=CP_UnicodeLE;
                        return;
                    case PRE_BigEndian:
                        this.codeSize=2;
                        this.codePage=CP_UnicodeBE;
                        return;
                }
                    break;
                case 3:
                    if(*(ushort*)p==0xBBEF && p[2]==0xBF) 
                    {
                        this.codeSize=UTF8.codeSize;
                        this.codePage=CP_UTF8;
                        return;
                    }
                    goto case 2;
                case 4:
                switch(*(uint*)p)
                {
                    case PRE_LittleEndian32:
                        this.codeSize=4;
                        this.codePage=CP_Unicode32LE;
                        return;
                    case PRE_BigEndian32:
                        this.codeSize=4;
                        this.codePage=CP_Unicode32BE;
                        return;
                }
                    if(*(ushort*)p == 0x2F2B && p[2] == 0x76) 
                    {
                        /*
                         * UTF-7: Initial U+FEFF is encoded as +/v8  or  +/v9  or  +/v+  or  +/v/
                         * depending on the second UTF-16 code unit.
                         * Detect the entire, closed Unicode mode sequence +/v8- for only U+FEFF
                         * if it occurs.
                         *
                         * So far we have +/v
                         */
                        switch(p[3])
                        {
                            case 0x38:
                                if (l>4 && p[4] == 0x2D)
                                {
                                    this.codeSize=UTF7.codeSize;
                                    this.codePage=CP_UTF7;
                                    return;
                                }
                                goto case 0x2F;
                            case 0x39:
                            case 0x2B:
                            case 0x2F:
                                this.codeSize=UTF7.codeSize;
                                this.codePage=CP_UTF7;
                                return;
                        }
                    }
                    goto case 3;
                default:
                    if (l<0)
                        throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
                    goto case 4;
            }

            this.codePage=def.codePage;
            this.codeSize=def.codeSize;
            return;
        }

        /// <summary>
        /// Constructor con deteccin de firma.
        /// </summary>
        /// <param name="p">Puntero sobre el cual intentar detectar firma de cdigo de pgina.</param>
        /// <param name="l">Largo del puntero de deteccin.</param>
        /// <param name="def">Cdigo de pgina por defecto a ser usado cuando no se detecta firma.</param>
        /// <param name="sign">Variable de salida. Si es mayor a cero, indica la zona de firma. Si
        /// es cero, indica que se utiliz el cdigo de pgina por defecto.</param>
        public CodePage(byte*p, int l, CodePage def, out int sign)
        {
            switch(l)
            {
                case 0:
                case 1:
                    break;
                case 2:
                switch(*(char*)p)
                {
                    case PRE_LittleEndian:
                        sign=2;
                        this.codeSize=2;
                        this.codePage=CP_UnicodeLE;
                        return;
                    case PRE_BigEndian:
                        sign=2;
                        this.codeSize=2;
                        this.codePage=CP_UnicodeBE;
                        return;
                }
                    break;
                case 3:
                    if(*(ushort*)p==0xBBEF && p[2]==0xBF) 
                    {
                        sign=3;
                        this.codeSize=UTF8.codeSize;
                        this.codePage=CP_UTF8;
                        return;
                    }
                    goto case 2;
                case 4:
                switch(*(uint*)p)
                {
                    case PRE_LittleEndian32:
                        sign=4;
                        this.codeSize=4;
                        this.codePage=CP_Unicode32LE;
                        return;
                    case PRE_BigEndian32:
                        sign=4;
                        this.codeSize=4;
                        this.codePage=CP_Unicode32BE;
                        return;
                }
                    if(*(ushort*)p == 0x2F2B && p[2] == 0x76) 
                    {
                        /*
                         * UTF-7: Initial U+FEFF is encoded as +/v8  or  +/v9  or  +/v+  or  +/v/
                         * depending on the second UTF-16 code unit.
                         * Detect the entire, closed Unicode mode sequence +/v8- for only U+FEFF
                         * if it occurs.
                         *
                         * So far we have +/v
                         */
                        switch(p[3])
                        {
                            case 0x38:
                                if (l>4 && p[4] == 0x2D)
                                {
                                    sign=5;
                                    this.codeSize=UTF7.codeSize;
                                    this.codePage=CP_UTF7;
                                    return;
                                }
                                goto case 0x2F;
                            case 0x39:
                            case 0x2B:
                            case 0x2F:
                                sign=4;
                                this.codeSize=UTF7.codeSize;
                                this.codePage=CP_UTF7;
                                return;
                        }
                    }
                    goto case 3;
                default:
                    if (l<0)
                        throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
                    goto case 4;
            }

            sign=0;
            this.codePage=def.codePage;
            this.codeSize=def.codeSize;
            return;
        }



        #endregion

        #region Propiedades

        /// <summary>
        /// Obtiene el nmero de cdigo de pgina.
        /// Si es cero indica que el cdigo de pgina es invlido.
        /// </summary>
        public int Number
        {
            get
            {
                return (int)this.codePage;
            }
            
        }

        /// <summary>
        /// Obtiene el tamao mximo de bytes por carcter.
        /// Si es cero o mayor a 15, indica que el cdigo de pgina es invlido.
        /// </summary>
        public int MaxSize
        {
            get
            {
                return (int)this.codeSize;
            }
        }

        /// <summary>
        /// Retorna true si el cdigo de pgina es considerado vlido y soportado por el sistema.
        /// </summary>
        public bool IsSupported
        {
            get
            {
                return this.codeSize!=0 && this.codeSize<16;
            }
        }

        /// <summary>
        /// Retorna true si tanto el Number como MaxSize son cero. 
        /// </summary>
        public bool IsNull
        {
            get
            {
                return this.codePage==0 && this.codeSize==0;
            }
        }

        /// <summary>
        /// Retorna true si el cdigo de pgina es Ansi Latin1.
        /// </summary>
        public bool IsLatin1
        {
            get
            {
                return this.codePage==CP_Latin1Ansi;
            }
        }

        /// <summary>
        /// Retorna true si el cdigo de pgina es Ascii 7 bits.
        /// </summary>
        public bool IsAscii
        {
            get
            {
                return this.codePage==CP_Ascii;
            }
        }

        /// <summary>
        /// Retorna true si el cdigo de pgina es Unicode LE de 16 bits.
        /// </summary>
        public bool IsUnicodeLE
        {
            get
            {
                return this.codePage==CP_UnicodeLE;
            }
        }

        /// <summary>
        /// Retorna true si el tamao mximo de bytes por caracter es tambin el mnimo (tamao fijo).
        /// El retorno true no necesariamente significa que el tamao decodificado sea previsible. 
        /// </summary>
        public bool IsFixedSize
        {
            get
            {
                if (this.codeSize==1)
                    return true;
                switch(this.codePage)
                {
                    case CP_UnicodeLE:
                    case CP_UnicodeBE:
                    case CP_Unicode32LE:
                    case CP_Unicode32BE:
                        return true;
                }
                return false;
            }

        }

        /// <summary>
        /// Obtiene el tamao de prembulo nesario para firmar archivos con este cdigo de pgina.
        /// Si es cero, indica que el cdigo de pgina no tiene firma establecida o esta
        /// no es soportada.
        /// </summary>
        public int SignSize
        {
            get
            {
                switch(codePage)
                {
                    case CP_UnicodeLE:
                    case CP_UnicodeBE:
                        return 2;
                    case CP_UTF8:
                        return 3;
                    case CP_UTF7:
                    case CP_Unicode32LE:
                    case CP_Unicode32BE:
                        return 4;
                }
                return 0;
            }
        }

        #endregion

        #region Operadores

        /// <summary>
        /// Comparador.
        /// </summary>
        /// <param name="a">CodePage a.</param>
        /// <param name="b">CodePage b.</param>
        /// <returns>True si son iguales.</returns>
        public static bool operator ==(CodePage a, CodePage b)
        {
            return a.codePage==b.codePage;
        }

        /// <summary>
        /// Comparador.
        /// </summary>
        /// <param name="a">CodePage a.</param>
        /// <param name="b">CodePage b.</param>
        /// <returns>True si son diferentes.</returns>
        public static bool operator !=(CodePage a, CodePage b)
        {
            return a.codePage!=b.codePage;
        }

        /// <summary>
        /// Comparador.
        /// </summary>
        /// <param name="a">CodePage a.</param>
        /// <param name="b">CodePage b.</param>
        /// <returns>True si A es mayor a B.</returns>
        public static bool operator >(CodePage a, CodePage b)
        {
            return a.codePage>b.codePage;
        }
        /// <summary>
        /// Comparador.
        /// </summary>
        /// <param name="a">CodePage a.</param>
        /// <param name="b">CodePage b.</param>
        /// <returns>True si A es menor a B.</returns>
        public static bool operator <(CodePage a, CodePage b)
        {
            return a.codePage<b.codePage;
        }

        /// <summary>
        /// Comparador.
        /// </summary>
        /// <param name="a">CodePage a.</param>
        /// <param name="b">CodePage b.</param>
        /// <returns>True si A es mayor o igual a B.</returns>
        public static bool operator >=(CodePage a, CodePage b)
        {
            return a.codePage>=b.codePage;
        }

        /// <summary>
        /// Comparador.
        /// </summary>
        /// <param name="a">CodePage a.</param>
        /// <param name="b">CodePage b.</param>
        /// <returns>True si A es menor o igual a B.</returns>
        public static bool operator <=(CodePage a, CodePage b)
        {
            return a.codePage<=b.codePage;
        }


        #endregion

        #region Metodos de Object

        /// <summary>
        /// True si son iguales.
        /// </summary>
        /// <param name="obj">Objeto.</param>
        /// <returns>True si son iguales.</returns>
        public override bool Equals(object obj)
        {
            if (obj is CodePage)
            {
                return this.codePage == ((CodePage)obj).codePage;
            }
            return false;
        }

        /// <summary>
        /// Obtiene Hash.
        /// </summary>
        /// <returns>Cdigo Hash.</returns>
        public override int GetHashCode()
        {
            return (int)this.codePage;
        }

        /// <summary>
        /// Obtiene la cadena descriptiva del cdigo de pgina.
        /// Si el cdigo de pgina y el tamao es cero, retorna la cadena Invalid CodePage.
        /// </summary>
        /// <returns>
        /// Cadena descriptiva del cdigo de pgina.
        /// Si el cdigo de pgina y el tamao es cero, retorna la cadena Invalid CodePage.
        /// </returns>
        public override string ToString()
        {
            if (this.IsNull)
                return Others.CP_INVALID;

            switch(codePage)
            {
                case CP_UnicodeLE:
                    return "1200  (Unicode UCS-2 Little Endian)";
                case CP_UnicodeBE:
                    return "1201  (Unicode UCS-2 Big Endian)";
                case CP_Unicode32LE:
                    return "12000 (Unicode UCS-4 Little Endian)";
                case CP_Unicode32BE:
                    return "12001 (Unicode UCS-4 Big Endian)";
            }
            if (OsInfo.Minimum2K)
            {
                char*p=stackalloc char[12+Api.MAX_PATH];
                if (GetCPInfoExW(codePage,0,p)!=0)
                    return Mutables.Create(p+12);
            }
            else
            {
                byte*p=stackalloc byte[24+Api.MAX_PATH];
                if (GetCPInfoExA(codePage,0,p)!=0)
                    return Mutables.Create(AnsiDefault,p+24);
            }
            switch(codePage)
            {
                case CP_Latin1Ansi:
                    return Others.CP_LATIN1_ANSI;
                case CP_Ascii:
                    return Others.CP_USA_ASCII;
                case CP_Symbol:
                    if (OsInfo.Minimum2K)
                        return "42    (Symbol)";
                    return Others.CP_INVALID;
            }
            return Others.CP_INVALID;
        }

        #endregion

        #region IComparable<CodePage> Members

        /// <summary>
        /// Compara este objeto con otro.
        /// </summary>
        /// <param name="other">Otro objeto.</param>
        /// <returns>-1 si menor, 0 si igual y 1 si mayor.</returns>
        public int CompareTo(CodePage other)
        {
            if (this.codePage>other.codePage)
                return 1;
            if (this.codePage!=other.codePage)
                return -1;
            return 0;
        }

        #endregion

        #region IEquatable<CodePage> Members

        /// <summary>
        /// Verifica igualdad.
        /// </summary>
        /// <param name="other">Objecto a comparar.</param>
        /// <returns>True si son iguales.</returns>
        public bool Equals(CodePage other)
        {
            return this.codePage==other.codePage;
        }

        #endregion

        #region API

        [SuppressUnmanagedCodeSecurity, DllImport(Api.KERNEL32, EntryPoint="WideCharToMultiByte",  SetLastError = true, ExactSpelling=true)]
        private static extern int wideCharToMultiByte(int codePage, uint dwFlags, char* lpWideCharStr, int cchWideChar, byte* lpMultiByteStr, int cbMultiByte, char* lpDefaultChar, int* lpUsedDefaultChar);

        [SuppressUnmanagedCodeSecurity, DllImport(Api.KERNEL32, EntryPoint="MultiByteToWideChar", SetLastError = true, ExactSpelling=true)]
        private static extern int multiByteToWideChar(int codePage, uint dwFlags, byte* lpMultiByteStr, int cbMultiByte, char* lpWideCharStr, int cchWideChar);

        [SuppressUnmanagedCodeSecurity, DllImport(Api.KERNEL32, ExactSpelling=true)]
        private static extern int GetACP();

        [SuppressUnmanagedCodeSecurity, DllImport(Api.KERNEL32, ExactSpelling=true)]
        private static extern int GetOEMCP();

        [SuppressUnmanagedCodeSecurity, DllImport(Api.KERNEL32, ExactSpelling=true)]
        private static extern int GetCPInfo(int codePage, ushort* lpCPInfo);

        [SuppressUnmanagedCodeSecurity, DllImport(Api.KERNEL32, ExactSpelling=true)]
        private static extern int GetCPInfoExA(int codePage,int cero, byte* lpCPInfoEx);

        [SuppressUnmanagedCodeSecurity, DllImport(Api.KERNEL32, ExactSpelling=true)]
        private static extern int GetCPInfoExW(int codePage,int cero, char* lpCPInfoEx);

        #endregion

        #region SignCP

        /// <summary>
        /// Aade firma de cdigo de pgina a un stream, siempre y cuando el cdigo de pgina lo requiera.
        /// </summary>
        /// <param name="stream">Stream.</param>
        /// <param name="noZeroSign">Si false, solo firmar si el stream se encuentra en posicin cero.</param>
        /// <returns>Bytes escritos en el stream o cero si no escribi.</returns>
        public int SignCP(System.IO.Stream stream, bool noZeroSign)
        {
            if (noZeroSign || stream.Position==0L)
            {
                byte[] tmp;
                switch(codePage)
                {
                    case CP_UnicodeLE:
                        tmp=new byte[2];
                        fixed(byte*p=tmp)
                            *(char*)p=PRE_LittleEndian;
                        stream.Write(tmp,0,2);
                        return 2;
                    case CP_UnicodeBE:
                        tmp=new byte[2];
                        fixed(byte*p=tmp)
                            *(char*)p=PRE_BigEndian;
                        stream.Write(tmp,0,2);
                        return 2;
                    case CP_UTF8:
                        tmp=new byte[3];
                        fixed(byte*p=tmp)
                        {
                            *(ushort*)p=0xBBEF;
                            p[2]=0xBF;
                        }
                        stream.Write(tmp,0,3);
                        return 3;
                    case CP_UTF7:
                        tmp=new byte[4];
                        fixed(byte*p=tmp)
                            *(uint*)p=0x38762F2B;
                        stream.Write(tmp,0,4);
                        return 4;
                    case CP_Unicode32LE:
                        tmp=new byte[4];
                        fixed(byte*p=tmp)
                            *(uint*)p=PRE_LittleEndian32;
                        stream.Write(tmp,0,4);
                        return 4;
                    case CP_Unicode32BE:
                        tmp=new byte[4];
                        fixed(byte*p=tmp)
                            *(uint*)p=PRE_BigEndian32;
                        stream.Write(tmp,0,4);
                        return 4;
                }
            }
            return 0;
        }

        /// <summary>
        /// Aade firma de cdigo de pgina a un Array, siempre y cuando el cdigo de pgina lo requiera.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <returns>Bytes escritos en el array o cero si no escribi.</returns>
        public int SignCP(byte[] arr)
        {
            int l=arr.Length;
            if (l!=0)
                fixed(byte*p=arr)
                    return SignCP(p,l);
            return 0;
        }


        /// <summary>
        /// Aade firma de cdigo de pgina a un Array, siempre y cuando el cdigo de pgina lo requiera.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="index">Indice en el array donde comenzar la firma.</param>
        /// <param name="maxCount">Mximo de bytes a escribr. Si es menor al tamao de la firma, no escribir.</param>
        /// <returns>Bytes escritos en el array o cero si no escribi.</returns>
        public int SignCP(byte[] arr, int index, int maxCount)
        {
            if (index<0 || maxCount<0 || index+maxCount>arr.Length)
                throw new ArgumentOutOfRangeException("index,maxCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (maxCount!=0)
                fixed(byte*p=arr)
                    return SignCP(p+index,maxCount);
            return 0;
        }

        /// <summary>
        /// Aade firma de cdigo de pgina a un puntero, siempre y cuando el cdigo de pgina lo requiera.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Bytes escritos en el puntero o cero si no escribi.</returns>
        public int SignCP(byte*p, int l)
        {
            switch(codePage)
            {
                case CP_UnicodeLE:
                    if (l>1)
                    {
                        *(char*)p=PRE_LittleEndian;
                        return 2;
                    }
                    return 0;
                case CP_UnicodeBE:
                    if (l>1)
                    {
                        *(char*)p=PRE_BigEndian;
                        return 2;
                    }
                    return 0;
                case CP_UTF8:
                    if (l>2)
                    {
                        *(ushort*)p=0xBBEF;
                        p[2]=0xBF;
                        return 3;
                    }
                    return 0;
                case CP_UTF7:
                    if (l>3)
                    {
                        *(uint*)p=0x38762F2B;
                        return 4;
                    }
                    return 0;
                case CP_Unicode32LE:
                    if (l>3)
                    {
                        *(uint*)p=PRE_LittleEndian32;
                        return 4;
                    }
                    return 0;
                case CP_Unicode32BE:
                    if (l>3)
                    {
                        *(uint*)p=PRE_BigEndian32;
                        return 4;
                    }
                    return 0;
            }
            return 0;
        }


        #endregion

        #region CharsToBytes y BytesToChars

        internal void ThrowIfUnSupported()
        {
            if(!this.IsSupported)
                throw new InvalidOperationException(Texts.Format(Errors.FMT_CP_UNSUPPORTED,this.codePage));
        }


        private static void errorWCMB(bool allowIB)
        {
            int err = Marshal.GetLastWin32Error();
            /*
             ERROR_INSUFFICIENT_BUFFER=122;
             ERROR_INVALID_FLAGS=1004;
             ERROR_INVALID_PARAMETER=87;
             ERROR_NO_UNICODE_TRANSLATION=1113;
            */
            switch (err)
            {
                case 87:
                    throw new ExternalException(Errors.INVALID_PARAMETER,err);
                case 122:
                    if (allowIB)
                        return;
                    throw new ArgumentOutOfRangeException("pd",Errors.DEST_MINOR);
                case 1004:
                    throw new ExternalException(Errors.INVALID_COMBINATION,err);
                case 1113:
                    throw new ExternalException(Errors.CP_NO_UNI_TRANS,err);
                default:
                    throw new ExternalException(Errors.UNEXPECTED,err);
            }
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="strOrig">Cadena origen.</param>
        /// <param name="arrDest">Array destino. Se debe asegurar que la cadena contenga a strOrig.</param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>Bytes escritos en arrDest.</returns>
        public int CharsToBytes(bool noBestFitChar, string strOrig, byte[] arrDest)
        {
            return CharsToBytes(noBestFitChar,strOrig,arrDest,null,null,false);
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="strOrig">Cadena origen.</param>
        /// <param name="arrDest">Array destino. Se debe asegurar que la cadena contenga a strOrig (a no ser que allowIB sea true).</param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="def">
        /// Array a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Array boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <param name="allowIB">
        /// Si true: 
        /// En caso de que el largo del array destino no pueda contener el total de la conversin,
        /// codificar hasta completar y retornar por charsCount tanto el valor bytesCount.
        /// En codificaciones Unicode LE y BE de 16 y 32 bits debe tomarse la precaucin de que bytesCount
        /// sea multiplo del ancho de caracter (2 o 4) dado que el valor de retorno siempre ser multiplo,
        /// pudiendo ser menor a bytesCount si este no es. Por ej: si bytesCount es 15, el valor de retorno ser 14 o 12 en
        /// Unicode 16 y 32 respectivamente.
        /// En otras codificaciones multibyte se rellenar con ceros los ltimos carcteres de bytesCount, 
        /// de acuerdo a la propiedad MaxSize, para facilitar la deteccin del largo final dado que
        /// el retorno siempre ser igual a bytesCount.
        /// Si false:
        /// En la situacin descripta se genera una excepcin ArgumentOutOfRangeException.
        /// </param>
        /// <returns>Bytes escritos en arrDest.</returns>
        public int CharsToBytes(bool noBestFitChar, string strOrig, byte[] arrDest, char* def, int* rep, bool allowIB)
        {
            int charsCount=strOrig==null?0:strOrig.Length;
            if (charsCount==0)
            {
                if (rep!=null && *rep!=0)
                    *rep=0;
                return 0;
            }

            int bytesCount=arrDest.Length;
            if (bytesCount==0)
            {
                if (allowIB)
                {
                    if (rep!=null && *rep!=0)
                        *rep=0;
                    return 0;
                }
                throw new ArgumentOutOfRangeException("arrDest",Errors.ARRAY_EMPTY);
            }

            fixed(char*po=strOrig)
                fixed(byte*pd=arrDest)
                    return CharsToBytes(noBestFitChar,po,charsCount,pd,bytesCount,def,rep,allowIB);
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="arrOrig">Array origen.</param>
        /// <param name="arrDest">Array destino. Se debe asegurar que la cadena contenga a arrOrig.</param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>Bytes escritos en arrDest.</returns>
        public int CharsToBytes(bool noBestFitChar, char[] arrOrig, byte[] arrDest)
        {
            return CharsToBytes(noBestFitChar,arrOrig,arrDest,null,null,false);
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="arrOrig">Array origen.</param>
        /// <param name="arrDest">Array destino. Se debe asegurar que la cadena contenga a arrOrig (a no ser que allowIB sea true).</param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="def">
        /// Array a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Array boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <param name="allowIB">
        /// Si true: 
        /// En caso de que el largo del array destino no pueda contener el total de la conversin,
        /// codificar hasta completar y retornar por charsCount tanto el valor bytesCount.
        /// En codificaciones Unicode LE y BE de 16 y 32 bits debe tomarse la precaucin de que bytesCount
        /// sea multiplo del ancho de caracter (2 o 4) dado que el valor de retorno siempre ser multiplo,
        /// pudiendo ser menor a bytesCount si este no es. Por ej: si bytesCount es 15, el valor de retorno ser 14 o 12 en
        /// Unicode 16 y 32 respectivamente.
        /// En otras codificaciones multibyte se rellenar con ceros los ltimos carcteres de bytesCount, 
        /// de acuerdo a la propiedad MaxSize, para facilitar la deteccin del largo final dado que
        /// el retorno siempre ser igual a bytesCount.
        /// Si false:
        /// En la situacin descripta se genera una excepcin ArgumentOutOfRangeException.
        /// </param>
        /// <returns>Bytes escritos en arrDest.</returns>
        public int CharsToBytes(bool noBestFitChar, char[] arrOrig, byte[] arrDest, char* def, int* rep, bool allowIB)
        {
            int charsCount=arrOrig.Length;
            if (charsCount==0)
            {
                if (rep!=null && *rep!=0)
                    *rep=0;
                return 0;
            }

            int bytesCount=arrDest.Length;
            if (bytesCount==0)
            {
                if (allowIB)
                {
                    if (rep!=null && *rep!=0)
                        *rep=0;
                    return 0;
                }
                throw new ArgumentOutOfRangeException("arrDest",Errors.ARRAY_EMPTY);
            }

            fixed(char*po=arrOrig)
                fixed(byte*pd=arrDest)
                    return CharsToBytes(noBestFitChar,po,charsCount,pd,bytesCount,def,rep,allowIB);
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="strOrig">Cadena origen.</param>
        /// <param name="origIndex">Inicio de la copia en la cadena origen.</param>
        /// <param name="charsCount">
        /// Largo de la cadena origen.
        /// Si es mayor a bytesCount, genera excepcin.
        /// </param>
        /// <param name="arrDest">Array destino.</param>
        /// <param name="destIndex">Inicio de la copia en el array destino.</param>
        /// <param name="bytesCount">
        /// Largo de copia del array destino.
        /// Se debe asegurar que la cadena contenga a charsCount.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>Bytes escritos en arrDest.</returns>
        public int CharsToBytes(bool noBestFitChar, string strOrig, int origIndex, int charsCount, byte[] arrDest, int destIndex, int bytesCount)
        {
            return CharsToBytes(noBestFitChar,strOrig,origIndex,charsCount,arrDest,destIndex,bytesCount,null,null,false);
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="strOrig">Cadena origen.</param>
        /// <param name="origIndex">Inicio de la copia en la cadena origen.</param>
        /// <param name="charsCount">
        /// Largo de la cadena origen.
        /// Si es mayor a bytesCount (a no ser que allowIB sea true), genera excepcin.
        /// </param>
        /// <param name="arrDest">Array destino.</param>
        /// <param name="destIndex">Inicio de la copia en el array destino.</param>
        /// <param name="bytesCount">
        /// Largo de copia del array destino.
        /// Se debe asegurar que la cadena contenga a charsCount (a no ser que allowIB sea true).
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="def">
        /// Array a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Array boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <param name="allowIB">
        /// Si true: 
        /// En caso de que bytesCount no pueda contener el total de la conversin,
        /// codificar hasta completar y retornar por charsCount tanto el valor bytesCount.
        /// En codificaciones Unicode LE y BE de 16 y 32 bits debe tomarse la precaucin de que bytesCount
        /// sea multiplo del ancho de caracter (2 o 4) dado que el valor de retorno siempre ser multiplo,
        /// pudiendo ser menor a bytesCount si este no es. Por ej: si bytesCount es 15, el valor de retorno ser 14 o 12 en
        /// Unicode 16 y 32 respectivamente.
        /// En otras codificaciones multibyte se rellenar con ceros los ltimos carcteres de bytesCount, 
        /// de acuerdo a la propiedad MaxSize, para facilitar la deteccin del largo final dado que
        /// el retorno siempre ser igual a bytesCount.
        /// Si false:
        /// En la situacin descripta se genera una excepcin ArgumentOutOfRangeException.
        /// </param>
        /// <returns>Bytes escritos en arrDest.</returns>
        public int CharsToBytes(bool noBestFitChar, string strOrig, int origIndex, int charsCount, byte[] arrDest, int destIndex, int bytesCount, char* def, int* rep, bool allowIB)
        {
            if (origIndex<0 || charsCount<0 || origIndex+charsCount>strOrig.Length)
                throw new ArgumentOutOfRangeException("origIndex,charsCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (charsCount==0)
            {
                if (rep!=null && *rep!=0)
                    *rep=0;
                return 0;
            }

            if (destIndex<0 || bytesCount<0 || destIndex+bytesCount>arrDest.Length)
                throw new ArgumentOutOfRangeException("destIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount==0)
            {
                if (allowIB)
                {
                    if (rep!=null && *rep!=0)
                        *rep=0;
                    return 0;
                }
                throw new ArgumentOutOfRangeException("bytesCount",Errors.MINOR_ONE);
            }

            fixed(char*po=strOrig)
                fixed(byte*pd=arrDest)
                    return CharsToBytes(noBestFitChar,po+origIndex,charsCount,pd+destIndex,bytesCount,def,rep,allowIB);
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="arrOrig">Array origen.</param>
        /// <param name="origIndex">Inicio de la copia en el array origen.</param>
        /// <param name="charsCount">
        /// Largo de copia del array origen.
        /// Si es mayor a bytesCount, genera excepcin.
        /// </param>
        /// <param name="arrDest">Array destino.</param>
        /// <param name="destIndex">Inicio de la copia en el array destino.</param>
        /// <param name="bytesCount">
        /// Largo de copia del array destino.
        /// Se debe asegurar que la cadena contenga a charsCount.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>Bytes escritos en arrDest.</returns>
        public int CharsToBytes(bool noBestFitChar, char[] arrOrig, int origIndex, int charsCount, byte[] arrDest, int destIndex, int bytesCount)
        {
            return CharsToBytes(noBestFitChar,arrOrig,origIndex,charsCount,arrDest,destIndex,bytesCount,null,null,false);
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="arrOrig">Array origen.</param>
        /// <param name="origIndex">Inicio de la copia en el array origen.</param>
        /// <param name="charsCount">
        /// Largo de copia del array origen.
        /// Si es mayor a bytesCount (a no ser que allowIB sea true), genera excepcin.
        /// </param>
        /// <param name="arrDest">Array destino.</param>
        /// <param name="destIndex">Inicio de la copia en el array destino.</param>
        /// <param name="bytesCount">
        /// Largo de copia del array destino.
        /// Se debe asegurar que la cadena contenga a charsCount (a no ser que allowIB sea true).
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="def">
        /// Array a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Array boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <param name="allowIB">
        /// Si true: 
        /// En caso de que bytesCount no pueda contener el total de la conversin,
        /// codificar hasta completar y retornar por charsCount tanto el valor bytesCount.
        /// En codificaciones Unicode LE y BE de 16 y 32 bits debe tomarse la precaucin de que bytesCount
        /// sea multiplo del ancho de caracter (2 o 4) dado que el valor de retorno siempre ser multiplo,
        /// pudiendo ser menor a bytesCount si este no es. Por ej: si bytesCount es 15, el valor de retorno ser 14 o 12 en
        /// Unicode 16 y 32 respectivamente.
        /// En otras codificaciones multibyte se rellenar con ceros los ltimos carcteres de bytesCount, 
        /// de acuerdo a la propiedad MaxSize, para facilitar la deteccin del largo final dado que
        /// el retorno siempre ser igual a bytesCount.
        /// Si false:
        /// En la situacin descripta se genera una excepcin ArgumentOutOfRangeException.
        /// </param>
        /// <returns>Bytes escritos en arrDest.</returns>
        public int CharsToBytes(bool noBestFitChar, char[] arrOrig, int origIndex, int charsCount, byte[] arrDest, int destIndex, int bytesCount, char* def, int* rep, bool allowIB)
        {
            if (origIndex<0 || charsCount<0 || origIndex+charsCount>arrOrig.Length)
                throw new ArgumentOutOfRangeException("origIndex,charsCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (charsCount==0)
            {
                if (rep!=null && *rep!=0)
                    *rep=0;
                return 0;
            }

            if (destIndex<0 || bytesCount<0 || destIndex+bytesCount>arrDest.Length)
                throw new ArgumentOutOfRangeException("destIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount==0)
            {
                if (allowIB)
                {
                    if (rep!=null && *rep!=0)
                        *rep=0;
                    return 0;
                }
                throw new ArgumentOutOfRangeException("bytesCount",Errors.MINOR_ONE);
            }

            fixed(char*po=arrOrig)
                fixed(byte*pd=arrDest)
                    return CharsToBytes(noBestFitChar,po+origIndex,charsCount,pd+destIndex,bytesCount,def,rep,allowIB);
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="po">Puntero origen.
        /// Si es igual a pd copiar a un puntero temporal en caso de ser necesario.</param>
        /// <param name="lo">
        /// Largo del puntero origen. Si es -1 el largo se determina automticamente.
        /// Si es positivo pero mayor a ld (a no ser que ld sea 0), genera excepcin.
        /// </param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="ld">
        /// Largo del puntero destino.
        /// Se debe asegurar que la cadena contenga a lo.
        /// Si es 0, la funcin simplemente retornar la cantidad necesaria de bytes para efectuar la conversin sin usar el puntero destino.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>Bytes escritos en pd.</returns>
        public int CharsToBytes(bool noBestFitChar, char* po, int lo, byte* pd, int ld)
        {
            return CharsToBytes(noBestFitChar,po,lo,pd,ld,null,null,false);
        }

        /// <summary>
        /// Efecta conversin desde Unicode a MultiByte.
        /// </summary>
        /// <param name="po">Puntero origen.
        /// Si es igual a pd copiar a un puntero temporal en caso de ser necesario.</param>
        /// <param name="lo">
        /// Largo del puntero origen. Si es -1 el largo se determina automticamente.
        /// Si es positivo pero mayor a ld (a no ser que ld sea 0 o allowIB sea true), genera excepcin.
        /// </param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="ld">
        /// Largo del puntero destino.
        /// Se debe asegurar que la cadena contenga a lo (a no ser que allowIB sea true).
        /// Si es 0, la funcin simplemente retornar la cantidad necesaria de bytes para efectuar la conversin sin usar el puntero destino.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <param name="allowIB">
        /// Si true: 
        /// En caso de que ld, siendo diferente a cero, no pueda contener el total de la conversin,
        /// codificar hasta completar y retornar por lo tanto el valor ld.
        /// En codificaciones Unicode LE y BE de 16 y 32 bits debe tomarse la precaucin de que ld
        /// sea multiplo del ancho de caracter (2 o 4) dado que el valor de retorno siempre ser multiplo,
        /// pudiendo ser menor a ld si este no es. Por ej: si ld es 15, el valor de retorno ser 14 o 12 en
        /// Unicode 16 y 32 respectivamente.
        /// En otras codificaciones multibyte se rellenar con ceros los ltimos carcteres de ld, 
        /// de acuerdo a la propiedad MaxSize, para facilitar la deteccin del largo final dado que
        /// el retorno siempre ser igual a ld.
        /// Si false:
        /// En la situacin descripta se genera una excepcin ArgumentOutOfRangeException.
        /// </param>
        /// <returns>Bytes escritos en pd.</returns>
        public int CharsToBytes(bool noBestFitChar, char* po, int lo, byte* pd, int ld, char* def, int* rep, bool allowIB)
        {
            if (rep!=null && *rep!=0)
                *rep=0;
            this.ThrowIfUnSupported();
            if (lo==-1)
            {
                lo=Comp.StrLen(po);
                if (lo==0)
                    return 0;
            }
            else if (lo<1)
            {
                if (lo==0)
                    return 0;
                throw new ArgumentOutOfRangeException("lo",lo,Errors.MINOR_ONE_NEGATIVE);
            }
            if (ld<1)
            {
                if (ld!=0)
                    throw new ArgumentOutOfRangeException("ld",ld,Errors.NEGATIVE);
                if (this.IsFixedSize)
                    return lo*this.MaxSize;
            }
            switch(codePage)
            {
                case CP_Ascii:
                    if (!noBestFitChar)
                        return wcToMBAsc(po,lo,pd,ld,allowIB,(def!=null)?(*(byte*)def):(byte)63,rep);
                    break;
                case CP_UnicodeLE:
                case CP_UnicodeBE:
                    return mbUni((byte*)po,lo<<1,pd,ld,allowIB);
                case CP_Unicode32LE:
                case CP_Unicode32BE:
                    return wcToMB32(po,lo,pd,ld,allowIB);
                case CP_UTF7:
                case CP_UTF8:
                case CP_Symbol:
                case 50220:
                case 50221:
                case 50222:
                case 50225:
                case 50227:
                case 50229:
                case 52936:
                case 54936:
                case 57002:
                case 57003:
                case 57004:
                case 57005:
                case 57006:
                case 57007:
                case 57008:
                case 57009:
                case 57010:
                case 57011:
                    if (noBestFitChar) 
                        noBestFitChar=false;
                    if (def!=null)
                        def=null;
                    if (rep!=null)
                        rep=null;
                    break;
            }

            if (ld<lo)
            {
                if (ld==0)
                    pd=null;
                else if (!allowIB)
                { 
                    if (this.codeSize == 1)
                        throw new ArgumentOutOfRangeException("ld", ld, Errors.DEST_MINOR);
                }
                else if (this.codeSize != 1)
                {
                    if (ld <= this.MaxSize)
                        General.Clean(pd, ld);
                    else
                        General.Clean(pd + (ld - this.MaxSize), this.MaxSize);
                }
                else
                    lo = ld;
            }
            int ldIB=ld;
            if (po!=pd)
                ld = wideCharToMultiByte((int)codePage, noBestFitChar?NO_BEST_FIT_CHARS:0, po, lo, pd, ld, def, rep);
            else if (&ld>General.MinimumStack && lo<General.MaxStackChars)
            {
                
                char* potmp=stackalloc char[lo];
                General.Copy(po,potmp,lo);
                ld = wideCharToMultiByte((int)codePage, noBestFitChar?NO_BEST_FIT_CHARS:0, potmp, lo, pd, ld, def, rep);
            }
            else
            {
                Mem.Heap heap=new Mem.Heap(lo<<1,false);
                General.Copy(po,(char*)heap.Pointer,lo);
                ld = wideCharToMultiByte((int)codePage, noBestFitChar?NO_BEST_FIT_CHARS:0, (char*)heap.Pointer, lo, pd, ld, def, rep);
                heap.Dispose();
            }
            if (ld == 0) 
            {
                errorWCMB(allowIB);
                return ldIB;
            }
            return ld;
        }


        private static int mbToWCAsc(byte* po, int lo, char* pd, int ld, bool allowIB)
        {
            if (ld<lo)
            {
                if (!allowIB)
                    throw new ArgumentOutOfRangeException("ld",ld,Errors.DEST_MINOR);
                lo=ld;
            }
            ld=0;
            if (po==pd)
            {
                if (&lo>General.MinimumStack && lo<General.MaxStackBytes)
                {
                    byte* potmp=stackalloc byte[lo];
                    General.Copy(po,potmp,lo);
                    po=potmp;
                }
                else
                {
                    Heap heap=new Mem.Heap(lo,false);
                    General.Copy(po,(byte*)heap.Pointer,lo);
                    po=(byte*)heap.Pointer;
                    do
                    {
                        pd[ld]=(char)((int)po[ld]&127);
                    }while(++ld!=lo);
                    heap.Dispose();
                    return lo;
                }
            }
            do
            {
                pd[ld]=(char)(po[ld]&127);
            }while(++ld!=lo);
            return lo;            
        }
      

        private static int wcToMBAsc(char* po, int lo, byte* pd, int ld, bool allowIB, byte def, int*rep)
        {
            if (ld<lo)
            {
                if (!allowIB)
                    throw new ArgumentOutOfRangeException("ld",ld,Errors.DEST_MINOR);
                lo=ld;
            }
            ld=0;
            char c;
            if (rep!=null)
            {
                do
                {
                    c=po[ld];
                    if (c>'\x7f')
                    {
                        pd[ld]=def;
                        *rep=1;
                        if (++ld!=lo)
                            goto nonull;
                        return lo;
                    }
                    pd[ld]=(byte)c;
                }while(++ld!=lo);
                return lo;
            }
            
            nonull:
                do
                {
                    c=po[ld];
                    if (c>'\x7f')
                        pd[ld]=def;
                    pd[ld]=(byte)c;
                }while(++ld!=lo);
            return lo;            
        }
      
        private int wcToMB32(char* po, int lo, byte* pd, int ld, bool allowIB)
        {
            int ldi=lo<<2;
            if (ld<ldi)
            {
                if (!allowIB)
                    throw new ArgumentOutOfRangeException("ld",ld,Errors.DEST_MINOR);
                lo=ld>>2;
                ldi=lo<<2;
                ld=ldi;
            }
            Mem.Heap heap;
            if (po!=pd)
                heap=null;
            else if (&ldi>General.MinimumStack && lo<General.MaxStackChars)
            {
                heap=null;
                char* potmp=stackalloc char[lo];
                General.Copy(po,potmp,lo);
                po=potmp;
            }
            else
            {
                heap=new Mem.Heap(lo<<1,false);
                General.Copy(po,(char*)heap.Pointer,lo);
                po=(char*)heap.Pointer;
            }
            if (codePage==CP_Unicode32LE)
            {
                ld=0;
                int*pi=(int*)pd;
                do
                {
                    pi[ld]=po[ld];
                }while(++ld!=lo);
            }
            else
            {
                ld=0;
                lo=0;
                byte*pob=(byte*)po;
                do
                {
                    *(char*)(pd+lo)='\0';
                    pd[lo+2]=pob[ld+1];
                    pd[lo+3]=pob[ld];
                    ld+=2;
                }while((lo+=4)!=ldi);
            }
            if (heap!=null)
                heap.Dispose();
            return ldi;
        }

        private int mbUni(byte* po, int lo, byte* pd, int ld, bool allowIB)
        {
            if (ld<lo)
            {
                if (!allowIB)
                    throw new ArgumentOutOfRangeException("ld",ld,Errors.DEST_MINOR);
                if ((ld&1)!=0)
                    lo=ld-1;
                else
                    lo=ld;
            }
            if (codePage==CP_UnicodeLE)
            {
                General.Move(po,pd,lo);
                return lo;
            }
            int sum;
            ld=0;
            if (po!=pd)
            {
                do
                {
                    sum=ld+1;
                    pd[ld]=po[sum];
                    pd[sum]=po[ld];
                }while((ld+=2)!=lo);
            }
            else
            {
                byte tmp;
                do
                {
                    sum=ld+1;
                    tmp=pd[ld];
                    pd[ld]=po[sum];
                    pd[sum]=tmp;
                }while((ld+=2)!=lo);
            }
            return lo;
        }
        
        private int mbToWC32(byte*po, int lo, char*pd, int ld, bool allowIB)
        {
            lo>>=2;
            if (ld<lo)
            {
                if (!allowIB)
                    throw new ArgumentOutOfRangeException("ld",ld,Errors.DEST_MINOR);
                lo=ld;
            }
            ld=0;
            if (codePage==CP_Unicode32LE)
            {
                int* poi=(int*)po;
                do
                {
                    pd[ld]=*(char*)(poi+ld);
                }while(++ld!=lo);
            }
            else
            {
                byte*pdb=(byte*)pd;
                do
                {
                    po+=4;
                    pdb[ld<<1]=po[-1];
                    pdb[(ld<<1)+1]=po[-2];
                }while(++ld!=lo);
            }
            return lo;
        }

        /// <summary>
        /// Efecta conversin desde MultiByte a Unicode.
        /// </summary>
        /// <param name="arrOrig">Array origen.</param>
        /// <param name="arrDest">Array destino.</param>
        /// <returns>Chars escritos en arrDest.</returns>
        public int BytesToChars(byte[] arrOrig, char[] arrDest)
        {
            return BytesToChars(arrOrig,arrDest,false);
        }

        /// <summary>
        /// Efecta conversin desde MultiByte a Unicode.
        /// </summary>
        /// <param name="arrOrig">Array origen.</param>
        /// <param name="arrDest">Array destino.</param>
        /// <param name="allowIB">
        /// Si true: 
        /// En caso de que el array destino no pueda contener el total de la conversin,
        /// decodificar hasta completar y retornar por bytesCount tanto el valor charsCount.
        /// Si false:
        /// En la situacin descripta se genera una excepcin ArgumentOutOfRangeException.
        /// </param>
        /// <returns>Chars escritos en arrDest.</returns>
        public int BytesToChars(byte[] arrOrig, char[] arrDest, bool allowIB)
        {
            int bytesCount=arrOrig.Length;
            if (bytesCount==0)
                return 0;

            int charsCount=arrDest.Length;
            if (charsCount==0)
            {
                if (allowIB)
                    return 0;
                throw new ArgumentOutOfRangeException("arrDest",Errors.ARRAY_EMPTY);
            }
            fixed(byte*po=arrOrig)
                fixed(char*pd=arrDest)
                    return BytesToChars(po,bytesCount,pd,charsCount,allowIB);
        }

        /// <summary>
        /// Efecta conversin desde MultiByte a Unicode.
        /// </summary>
        /// <param name="arrOrig">Array origen.</param>
        /// <param name="origIndex">Inicio de la copia en el array origen.</param>
        /// <param name="bytesCount">
        /// Largo de copia del array origen.
        /// Si es mayor a charsCount, genera excepcin.
        /// </param>
        /// <param name="arrDest">Array destino.</param>
        /// <param name="destIndex">Inicio de la copia en el array destino.</param>
        /// <param name="charsCount">
        /// Largo de copia del array destino.
        /// Se debe asegurar que la cadena contenga a bytesCount.
        /// </param>
        /// <returns>Chars escritos en arrDest.</returns>
        public int BytesToChars(byte[] arrOrig, int origIndex, int bytesCount, char[] arrDest, int destIndex, int charsCount)
        {
            return BytesToChars(arrOrig,origIndex,bytesCount,arrDest,destIndex,charsCount,false);
        }

        /// <summary>
        /// Efecta conversin desde MultiByte a Unicode.
        /// </summary>
        /// <param name="arrOrig">Array origen.</param>
        /// <param name="origIndex">Inicio de la copia en el array origen.</param>
        /// <param name="bytesCount">
        /// Largo de copia del array origen.
        /// Si es positivo pero mayor a charsCount (a no ser que allowIB sea true), genera excepcin.
        /// </param>
        /// <param name="arrDest">Array destino.</param>
        /// <param name="destIndex">Inicio de la copia en el array destino.</param>
        /// <param name="charsCount">
        /// Largo de copia del array destino.
        /// Se debe asegurar que la cadena contenga a bytesCount (a no ser que allowIB sea true).
        /// </param>
        /// <param name="allowIB">
        /// Si true: 
        /// En caso de que charsCount no pueda contener el total de la conversin,
        /// decodificar hasta completar y retornar por bytesCount tanto el valor charsCount.
        /// Si false:
        /// En la situacin descripta se genera una excepcin ArgumentOutOfRangeException.
        /// </param>
        /// <returns>Chars escritos en arrDest.</returns>
        public int BytesToChars(byte[] arrOrig, int origIndex, int bytesCount, char[] arrDest, int destIndex, int charsCount, bool allowIB)
        {
            if (origIndex<0 || bytesCount<0 || origIndex+bytesCount>arrOrig.Length)
                throw new ArgumentOutOfRangeException("origIndex,destIndex",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount==0)
                return 0;

            if (destIndex<0 || charsCount<0 || destIndex+charsCount>arrDest.Length)
                throw new ArgumentOutOfRangeException("destIndex,destIndex",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (charsCount==0)
            {
                if (allowIB)
                    return 0;
                throw new ArgumentOutOfRangeException("charsCount",Errors.MINOR_ONE);
            }
            fixed(byte*po=arrOrig)
                fixed(char*pd=arrDest)
                    return BytesToChars(po+origIndex,bytesCount,pd+destIndex,charsCount,allowIB);
        }

        
        /// <summary>
        /// Efecta conversin desde MultiByte a Unicode.
        /// </summary>
        /// <param name="po">Puntero origen. Si es igual a pd copiar a un puntero temporal en caso de ser necesario.</param>
        /// <param name="lo">
        /// Largo del puntero origen. Si es -1 el largo se determina automticamente.
        /// Si es positivo pero mayor a ld (a no ser que ld sea 0), genera excepcin.
        /// </param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="ld">
        /// Largo del puntero destino.
        /// Se debe asegurar que la cadena contenga a lo.
        /// Si es 0, la funcin simplemente retornar la cantidad necesaria de bytes para efectuar la conversin sin usar el puntero destino.
        /// </param>
        /// <returns>Chars escritos en pd.</returns>
        public int BytesToChars(byte* po, int lo, char* pd, int ld)
        {
            return BytesToChars(po,lo,pd,ld,false);
        }

        /// <summary>
        /// Efecta conversin desde MultiByte a Unicode.
        /// </summary>
        /// <param name="po">Puntero origen. Si es igual a pd copiar a un puntero temporal en caso de ser necesario.</param>
        /// <param name="lo">
        /// Largo del puntero origen. Si es -1 el largo se determina automticamente.
        /// Si es positivo pero mayor a ld (a no ser que ld sea 0 o allowIB sea true), genera excepcin.
        /// </param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="ld">
        /// Largo del puntero destino.
        /// Se debe asegurar que la cadena contenga a lo (a no ser que allowIB sea true).
        /// Si es 0, la funcin simplemente retornar la cantidad necesaria de bytes para efectuar la conversin sin usar el puntero destino.
        /// </param>
        /// <param name="allowIB">
        /// Si true: 
        /// En caso de que ld, siendo diferente a cero, no pueda contener el total de la conversin,
        /// decodificar hasta completar y retornar por lo tanto el valor ld.
        /// Si false:
        /// En la situacin descripta se genera una excepcin ArgumentOutOfRangeException.
        /// </param>
        /// <returns>Chars escritos en pd.</returns>
        public int BytesToChars(byte* po, int lo, char* pd, int ld, bool allowIB)
        {
            this.ThrowIfUnSupported();

            if (lo==-1)
            {
                lo=this.StrLen(po);
                if (lo==0)
                    return 0;
            }
            else if (lo<this.MaxSize)
            {
                if (lo<0)
                    throw new ArgumentOutOfRangeException("lo",lo,Errors.MINOR_ONE_NEGATIVE);
                if (lo==0 || this.IsFixedSize)
                    return 0;
            }
            
            if (ld<1)
            {
                if (ld!=0)
                    throw new ArgumentOutOfRangeException("ld",ld,Errors.NEGATIVE);
                if (this.IsFixedSize)
                    return lo/this.MaxSize;
            }
            

            switch(codePage)
            {
                case CP_Ascii:
                    return mbToWCAsc(po,lo,pd,ld,allowIB);
                case CP_UnicodeLE:
                case CP_UnicodeBE:
                    return mbUni(po,(lo>>1)<<1,(byte*)pd,ld<<1,allowIB)>>1;
                case CP_Unicode32LE:
                case CP_Unicode32BE:
                    return mbToWC32(po,(lo>>2)<<2,pd,ld,allowIB);
            }

            if (ld<lo)
            {
                if (ld==0)
                    pd=null;
                else if (this.codeSize == 1)
                {
                    if (!allowIB)
                        throw new ArgumentOutOfRangeException("ld", ld, Errors.DEST_MINOR);
                    lo = ld;
                }
            }

            int ldIB=ld;

            if (po!=pd)
                ld = multiByteToWideChar((int)codePage, 0, po, lo, pd, ld);
            else if (&ld>General.MinimumStack && lo<General.MaxStackBytes)
            {
                byte* potmp=stackalloc byte[lo];
                General.Copy(po,potmp,lo);
                ld = multiByteToWideChar((int)codePage, 0, potmp, lo, pd, ld);
            }
            else
            {
                Mem.Heap heap=new Mem.Heap(lo,false);
                General.Copy(po,(byte*)heap.Pointer,lo);
                ld = multiByteToWideChar((int)codePage, 0, (byte*)heap.Pointer, lo, pd, ld);
                heap.Dispose();
            }
            if (ld == 0) 
            {
                errorWCMB(allowIB);
                return ldIB;
            }
            return ld;
        }


        #endregion

        #region CharToInt e IntToChar


        /// <summary>
        /// Convierte un carcter individual en MultiByte.
        /// </summary>
        /// <param name="c">Carcter.</param>
        /// <param name="noBestFitChar">
        /// En el caso de que el caracter no se traduzca directamente a multibyte devolver -1 sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>Caracter convertido o -1.</returns>
        public int CharToInt(char c, bool noBestFitChar)
        {
            
            switch (codePage)
            {
                case CP_UnicodeLE:
                case CP_Unicode32LE:
                    return c;
                case CP_UnicodeBE:
                    return Numerics.Invert((ushort)c);
                case CP_Unicode32BE:
                    return Numerics.Invert((int)c);
                case CP_Symbol:
                case CP_UTF7:
                case CP_UTF8:
                case 50220:
                case 50221:
                case 50222:
                case 50225:
                case 50227:
                case 50229:
                case 52936:
                case 54936:
                case 57002:
                case 57003:
                case 57004:
                case 57005:
                case 57006:
                case 57007:
                case 57008:
                case 57009:
                case 57010:
                case 57011:
                    if (noBestFitChar) 
                        noBestFitChar=false;
                    break;
                case CP_Ascii:
                    if (c<'\x80')
                        return c;
                    if (noBestFitChar)
                        return -1;
                    break;
                case CP_Latin1Ansi:
                    if (c<'\x100')
                    {
                        if (c<'\x80' || c>'\x9f')
                            return c;
                    }
                    else if (noBestFitChar)
                        return -1;
                    break;
            }

            this.ThrowIfUnSupported();
            int rep=0;
            int ret=0;
            if (wideCharToMultiByte(codePage, noBestFitChar?NO_BEST_FIT_CHARS:0, &c, 1, (byte*)&ret, 4, null, &rep) == 0) 
                errorWCMB(false);
            if (rep!=0)
                return -1;
            return ret;

        }

        /// <summary>
        /// Convierte un carcter MultiByte individual en char.
        /// </summary>
        /// <param name="c">Carcter.</param>
        /// <returns>Caracter convertido.</returns>
        public char IntToChar(int c)
        {
            switch (codePage)
            {
                case CP_UnicodeLE:
                case CP_Unicode32LE:
                    return (char)c;
                case CP_UnicodeBE:
                    return (char)Numerics.Invert((ushort)c);
                case CP_Unicode32BE:
                    return (char)Numerics.Invert(c);
                case CP_Ascii:
                    return (char)(c&127);
                case CP_Latin1Ansi:
                    if (c<128 || c>159)
                        return (char)(c&255);
                    break;
            }
            this.ThrowIfUnSupported();
            int ret=0;
            if (multiByteToWideChar((int)codePage, 0, (byte*)&c, (c&0xFFFFFF00)==0?1:((c&0xFFFF0000)==0?2:4), (char*)&ret, 2) == 0) 
                errorWCMB(false);
            return (char)ret;
        }

        #endregion

        #region Clculos

        /// <summary>
        /// Calcula largo de cadena de bytes de acuerdo a la codificacin.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="maxBytes">
        /// Largo mximo y zona de deteccin, si no contiene cero final se retorna este valor.
        /// Si es -1, se efectuar una busqueda de cero final sin limite de memoria.
        /// </param>
        /// <returns>
        /// Tamao de cadena, en bytes, si se encontr un cero final. De lo contrario, maxBytes.
        /// </returns>
        public int StrLen(byte*p, int maxBytes)
        {
            if (this.codeSize!=1)
            {
                this.ThrowIfUnSupported();
                if (this.IsFixedSize)
                {
                    int sof=this.MaxSize;
                    return Comp.StrLen(sof,p,maxBytes/sof)*sof;
                }
            }
            return Comp.StrLen(p,maxBytes);
        }

        /// <summary>
        /// Calcula largo de cadena de bytes de acuerdo a la codificacin.
        /// </summary>
        /// <param name="p">Puntero, deber tener efectivamente un cero final para no provocar una violacin de memoria.</param>
        /// <returns>
        /// Tamao de cadena.
        /// </returns>
        public int StrLen(byte*p)
        {
            if (this.codeSize!=1)
            {
                if (this.IsFixedSize)
                {
                    int sof=this.MaxSize;
                    return Comp.StrLen(sof,p)*sof;
                }
                else
                    this.ThrowIfUnSupported();
            }
            return Comp.StrLen(p);
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de bytes para efectuar una codificacin.
        /// </summary>
        /// <param name="chars">Cantidad de chars a calcular.</param>
        /// <returns>Cantidad mxima de bytes necesaria para codificar los chars.</returns>
        public int CalcMaxBytes(int chars)
        {
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(false,false,chars,sof))
                return 0;
            return chars*sof;
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de bytes para efectuar una codificacin.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="exact">Efecta estimacin exacta efectuando una codificacin tentativa si es necesario.</param>
        /// <returns>Cantidad mxima de bytes necesaria para codificar los chars.</returns>
        public int CalcMaxBytes(string str, bool exact)
        {
            int count=str==null?0:str.Length;
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(false,false,count,sof))
                return 0;
            if (!exact || this.IsFixedSize)
                return count*sof;
            fixed(char*p=str)
                return this.CharsToBytes(false,p,count,null,0,null,null,false);
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de bytes para efectuar una codificacin.
        /// </summary>
        /// <param name="arr">Array de chars.</param>
        /// <param name="exact">Efecta estimacin exacta efectuando una codificacin tentativa si es necesario.</param>
        /// <returns>Cantidad mxima de bytes necesaria para codificar los chars.</returns>
        public int CalcMaxBytes(char[] arr, bool exact)
        {
            int count=arr.Length;
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(false,false,count,sof))
                return 0;
            if (!exact || this.IsFixedSize)
                return count*sof;
            fixed(char*p=arr)
                return this.CharsToBytes(false,p,count,null,0,null,null,false);
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de bytes para efectuar una codificacin.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="exact">Efecta estimacin exacta efectuando una codificacin tentativa si es necesario.</param>
        /// <param name="index">Inicio de la zona a calcular.</param>
        /// <param name="count">Largo de la zona a calcular (-1 no es aceptado).</param>
        /// <returns>Cantidad mxima de bytes necesaria para codificar los chars.</returns>
        public int CalcMaxBytes(string str, int index, int count, bool exact)
        {
            if (index<0 || count<0 || index+count>str.Length)
                throw new ArgumentOutOfRangeException("index,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(false,false,count,sof))
                return 0;
            if (!exact || this.IsFixedSize)
                return count*sof;
            fixed(char*p=str)
                return this.CharsToBytes(false,p+index,count,null,0,null,null,false);
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de bytes para efectuar una codificacin.
        /// </summary>
        /// <param name="arr">Array de chars.</param>
        /// <param name="exact">Efecta estimacin exacta efectuando una codificacin tentativa si es necesario.</param>
        /// <param name="index">Inicio de la zona a calcular.</param>
        /// <param name="count">Largo de la zona a calcular (-1 no es aceptado).</param>
        /// <returns>Cantidad mxima de bytes necesaria para codificar los chars.</returns>
        public int CalcMaxBytes(char[] arr, int index, int count, bool exact)
        {
            if (index<0 || count<0 || index+count>arr.Length)
                throw new ArgumentOutOfRangeException("index,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(false,false,count,sof))
                return 0;
            if (!exact || this.IsFixedSize)
                return count*sof;
            fixed(char*p=arr)
                return this.CharsToBytes(false,p+index,count,null,0,null,null,false);
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de bytes para efectuar una codificacin.
        /// </summary>
        /// <param name="p">Puntero de chars.</param>
        /// <param name="l">Largo del puntero (si es -1 ser detectado segn cero final.</param>
        /// <param name="exact">Efecta estimacin exacta efectuando una codificacin tentativa si es necesario.</param>
        /// <returns>Cantidad mxima de bytes necesaria para codificar los chars.</returns>
        public int CalcMaxBytes(char*p, int l, bool exact)
        {
            if (l==-1)
                l=Comp.StrLen(p);
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(false,false,l,sof))
                return 0;
            if (!exact || this.IsFixedSize)
                return l*sof;
            return this.CharsToBytes(false,p,l,null,0,null,null,false);
        }
        
        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de chars para efectuar una decodificacin.
        /// </summary>
        /// <param name="bytes">Cantidad de bytes a calcular.</param>
        /// <returns>Cantidad mxima de chars necesaria para decodificar los bytes.</returns>
        public int CalcMaxChars(int bytes)
        {
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(bytes,false,true,sof))
            {
                if (bytes==0 || this.IsFixedSize)
                    return 0;
            }
            else if (sof!=1 && this.IsFixedSize)
                return bytes/sof;
            return bytes;
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de chars para efectuar una decodificacin.
        /// </summary>
        /// <param name="arr">Array de bytes.</param>
        /// <param name="exact">Efecta estimacin exacta efectuando una decodificacin tentativa si es necesario.</param>
        /// <returns>Cantidad mxima de chars necesaria para decodificar los bytes.</returns>
        public int CalcMaxChars(byte[] arr, bool exact)
        {
            int count=arr.Length;
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(count,false,true,sof))
            {
                if (count==0  || this.IsFixedSize)
                    return 0;
            }
            else if (this.IsFixedSize)
            {
                if (sof!=1)
                    return count/sof;
                return count;
            }
            if (!exact)
                return count;
            fixed(byte*p=arr)
                return this.BytesToChars(p,count,null,0,false);
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de chars para efectuar una decodificacin.
        /// </summary>
        /// <param name="arr">Array de bytes.</param>
        /// <param name="index">Inicio de la zona a calcular.</param>
        /// <param name="count">Largo de la zona a calcular (-1 no es aceptado).</param>
        /// <param name="exact">Efecta estimacin exacta efectuando una decodificacin tentativa si es necesario.</param>
        /// <returns>Cantidad mxima de chars necesaria para decodificar los bytes.</returns>
        public int CalcMaxChars(byte[] arr, int index, int count, bool exact)
        {
            if (index<0 || count<0 || index+count>arr.Length)
                throw new ArgumentOutOfRangeException("index,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(count,false,true,sof))
            {
                if (count==0  || this.IsFixedSize)
                    return 0;
            }
            else if (this.IsFixedSize)
            {
                if (sof!=1)
                    return count/sof;
                return count;
            }
            if (!exact)
                return count;
            fixed(byte*p=arr)
                return this.BytesToChars(p+index,count,null,0,false);
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de chars para efectuar una decodificacin.
        /// </summary>
        /// <param name="p">Puntero de bytes.</param>
        /// <param name="l">Largo del puntero (si es -1 ser detectado segn cero final.</param>
        /// <param name="exact">Efecta estimacin exacta efectuando una decodificacin tentativa si es necesario.</param>
        /// <returns>Cantidad mxima de chars necesaria para decodificar los bytes (sin incluir cero final cuando l es -1).</returns>
        public int CalcMaxChars(byte*p, int l, bool exact)
        {
            if (l==-1)
                l=this.StrLen(p);
            int sof=this.MaxSize;
            if (!Serializer.CheckParams(l,false,true,sof))
            {
                if (l==0  || this.IsFixedSize)
                    return 0;
            }
            else if (this.IsFixedSize)
            {
                if (sof!=1)
                    return l/sof;
                return l;
            }
            if (!exact)
                return l;
            return this.BytesToChars(p,l,null,0,false);
        }

        /// <summary>
        /// Verifica parmetros y determina el mximo en Chars requerido para la lectura de un stream.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// Si maxChars es -1 y el largo recuperable del stream supera 2GB - 1 bytes,
        /// genera excepcin.
        /// </param>
        /// <param name="maxChars">
        /// En la entrada, establece un limite en chars decodificados para la lectura.
        /// En la salida actualiza la variable de acuerdo a la capacidad del stream.
        /// Si es -1, se establece inicialmente en 1GB - 1. En cambio, si es mayor a 1GB - 1, genera excepcin.
        /// </param>
        /// <returns>
        /// True si las estimaciones son exactas. 
        /// Esto solo ocurre si la codificacion es fija o si se ha establecido un lmite de cero
        /// dado que nunca se lee el contenido del stream.
        /// </returns>
        public bool CalcStreamRead(Stream stream, ref int maxChars)
        {
            bool fixSize=this.IsFixedSize;
            if (fixSize)
            {
                int siz=this.MaxSize;
                if (siz!=1)
                    Serializer.CalcStreamReadItems(stream,ref maxChars,siz);
                else
                    maxChars=Serializer.CalcStreamRead(stream,maxChars);
            }
            else
            {
                this.ThrowIfUnSupported();
                maxChars=Serializer.CalcStreamRead(stream,maxChars);
            }
            if (maxChars>Int32.MaxValue>>1)
                throw new ArgumentOutOfRangeException("maxChars",maxChars,Errors.BIG_VALUE);
            return fixSize;
        }
        
        /// <summary>
        /// Verifica parmetros y determina el mximo en Chars y Bytes requerido para la lectura de un stream.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// Si tanto maxBytes y maxChars son -1 y el largo recuperable del stream supera 2GB - 1 bytes,
        /// genera excepcin.
        /// </param>
        /// <param name="maxBytes">
        /// En la entrada, establece un limite en bytes para la lectura.
        /// En la salida actualiza la variable y la sincroniza con maxChars de acuerdo a la capacidad del stream.
        /// Si es -1, se establece inicialmente en 2GB - 1.
        /// </param>
        /// <param name="maxChars">
        /// En la entrada, establece un limite en chars decodificados para la lectura.
        /// En la salida actualiza la variable y la sincroniza con maxBytes de acuerdo a la capacidad del stream.
        /// Si es -1, se establece inicialmente en (1GB - 1)/(sof*2). En cambio, si es mayor a (1GB - 1)/(sof*2), genera excepcin.
        /// </param>
        /// <returns>
        /// True si las estimaciones son exactas. 
        /// Esto solo ocurre si la codificacion es fija o si se han establecido limites de cero
        /// dado que nunca se lee el contenido del stream.
        /// </returns>
        public bool CalcStreamRead(Stream stream, ref int maxBytes, ref int maxChars)
        {
            bool fixSize=this.IsFixedSize;
            maxBytes=Serializer.CalcStreamReadItems(stream,maxBytes,ref maxChars,fixSize?this.MaxSize:1);
            if (maxChars>Int32.MaxValue>>1)
                throw new ArgumentOutOfRangeException("maxChars",maxChars,Errors.BIG_VALUE);
            return fixSize;
        }

        #endregion

        #region StreamRead

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual, decodificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrDest">
        /// Array destino.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Chars recuperados desde el Stream.
        /// </returns>
        public int StreamRead(Stream stream, bool close, char[] arrDest)
        {
            int maxChars=arrDest.Length;
            if (maxChars==0)
            {
                if (close)
                    stream.Close();
                return 0;
            }
            fixed(char*p=arrDest)
                return StreamRead(stream,close,p,maxChars,null,0);
        }

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual, decodificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrDest">
        /// Array destino. Debe tener una capacidad coherente con maxChars.
        /// </param>
        /// <param name="destIndex">Index en el array destino donde comenzar la copia (en chars).</param>
        /// <param name="maxChars">
        /// Cantidad mxima en chars a decodificar.
        /// La capacidad de arrDest debe ser coherente con este valor.
        /// Para obtener un valor adecuado, usar el mtodo CalcStreamRead() antes de iniciar la lectura.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Chars recuperados desde el Stream.
        /// </returns>
        public int StreamRead(Stream stream, bool close, char[] arrDest, int destIndex, int maxChars)
        {
            if (destIndex<0 || maxChars<0 || destIndex+maxChars>arrDest.Length)
                throw new ArgumentOutOfRangeException("destIndex,maxChars",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (maxChars==0)
            {
                if (close)
                    stream.Close();
                return 0;
            }
            fixed(char*p=arrDest)
                return StreamRead(stream,close,p+destIndex,maxChars,null,0);
        }

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual, decodificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrDest">
        /// Array destino. Debe tener una capacidad coherente con maxChars.
        /// </param>
        /// <param name="destIndex">Index en el array destino donde comenzar la copia (en chars).</param>
        /// <param name="arrTemp">
        /// Array de bytes temporal. Si es null o su capacidad til no es practica para esta funcionalidad, se crear uno.
        /// </param>
        /// <param name="tempIndex">
        /// Index en el array donde comienza el espacio utilizable para copia temporal (en bytes).
        /// </param>
        /// <param name="maxChars">
        /// Cantidad mxima en chars a decodificar.
        /// La capacidad de arrDest debe ser coherente con este valor.
        /// Para obtener un valor adecuado, usar el mtodo CalcStreamRead() antes de iniciar la lectura.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Chars recuperados desde el Stream.
        /// </returns>
        public int StreamRead(Stream stream, bool close, char[] arrDest, int destIndex, int maxChars, byte[] arrTemp, int tempIndex)
        {
            if (destIndex<0 || maxChars<0 || destIndex+maxChars>arrDest.Length)
                throw new ArgumentOutOfRangeException("destIndex,maxChars",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (maxChars==0)
            {
                if (close)
                    stream.Close();
                return 0;
            }
            fixed(char*p=arrDest)
                return StreamRead(stream,close,p+destIndex,maxChars,arrTemp,tempIndex);
        }

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual, decodificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe tener una capacidad coherente con maxChars.
        /// </param>
        /// <param name="maxChars">
        /// Cantidad mxima en chars a decodificar.
        /// La capacidad de pd debe ser coherente con este valor.
        /// Para obtener un valor adecuado, usar el mtodo CalcStreamRead() antes de iniciar la lectura.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Chars recuperados desde el Stream.
        /// </returns>
        public int StreamRead(Stream stream, bool close, char* pd, int maxChars)
        {
            return StreamRead(stream,close,pd,maxChars,null,0);
        }

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual, decodificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe tener una capacidad coherente con maxChars.
        /// </param>
        /// <param name="arrTemp">
        /// Array de bytes temporal. Si es null o su capacidad til no es practica para esta funcionalidad, se crear uno.
        /// </param>
        /// <param name="tempIndex">
        /// Index en el array donde comienza el espacio utilizable para copia temporal (en bytes).
        /// </param>
        /// <param name="maxChars">
        /// Cantidad mxima en chars a decodificar.
        /// La capacidad de pd debe ser coherente con este valor.
        /// Para obtener un valor adecuado, usar el mtodo CalcStreamRead() antes de iniciar la lectura.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Chars recuperados desde el Stream.
        /// </returns>
        public int StreamRead(Stream stream, bool close, char* pd, int maxChars, byte[] arrTemp, int tempIndex)
        {
            if (this.codePage==CP_UnicodeLE)
                return Serializer.StreamReadItems(stream,close,pd,maxChars,2,arrTemp,tempIndex);

            int maxBytes=this.CalcMaxBytes(maxChars);

            if (maxBytes==0)
                goto end;
            
            if (pd==null)
                throw new ArgumentNullException("pd",Errors.NULL_POINTER);

            if (tempIndex<0)
                throw new ArgumentOutOfRangeException("tempIndex",tempIndex,Errors.NEGATIVE);

            int maxBuf=(arrTemp!=null?arrTemp.Length:0)-tempIndex;
            int sof=this.MaxSize;
            int reads;

            if (maxBuf>=maxBytes)
                goto onePass;

            if (!this.IsFixedSize)
            {
                tempIndex=0;
                arrTemp=new byte[maxBytes];
                goto onePass;
            }
            
            if (maxBuf<32 || (maxBuf<512 && maxBytes/maxBuf>127))
            {
                if (maxBuf<0)
                    throw new ArgumentOutOfRangeException("tempIndex",tempIndex,Errors.SUPERIOR_ARRAY);
                tempIndex=0;
                if (maxBytes>=1<<20)
                    maxBuf=16384;
                else if (maxBytes>=1<<18)
                    maxBuf=8192;
                else if (maxBytes>=1<<16)
                    maxBuf=4096;
                else if (maxBytes<=8192)
                {
                    arrTemp=new byte[maxBytes];
                    goto onePass;
                }
                else
                    maxBuf=2048;
            }

            int chBuf;
            if (sof!=1)
            {
                chBuf=maxBuf/sof;
                maxBuf=chBuf*sof;
            }
            else
                chBuf=maxBuf;
            arrTemp=new byte[maxBuf];

            fixed(byte*p=arrTemp)
            {
                byte*po=p+tempIndex;
                if (po==pd)
                    throw new ArgumentException(Errors.SER_STREAM_P_EQUAL_ARRAY,"arrTemp");

                int ret=0;
                while((reads=stream.Read(arrTemp,tempIndex,maxBuf))!=0)
                {
                    if (reads!=maxBuf)
                    {
                        if (sof!=1)
                        {
                            if (reads<sof)
                                break;
                            chBuf=reads/sof;
                            reads=chBuf*sof;
                        }
                        else
                            chBuf=reads;
                        ret+=this.BytesToChars(po,reads,pd+ret,chBuf,false);
                        break;
                    }
                    ret+=this.BytesToChars(po,reads,pd+ret,chBuf,false);
                    if (maxChars-ret<chBuf)
                    {
                        if (maxChars==ret)
                            break;
                        chBuf=maxChars-ret;
                        maxBuf=chBuf*sof;
                    }
                }
                maxChars=ret;
                goto end;
            }

            onePass:

                fixed(byte*p=arrTemp)
                {
                    reads=stream.Read(arrTemp,tempIndex,maxBytes);
                    if (reads!=maxBytes)
                    {
                        if (sof==1 || !this.IsFixedSize)
                        {
                            maxChars=reads;
                            if (reads==0)
                                goto end;
                        }
                        else if (reads<sof)
                        {
                            maxChars=0;
                            goto end;
                        }
                        else
                        {
                            maxChars=reads/sof;
                            reads=maxChars*sof;
                        }
                    }
                    maxChars=this.BytesToChars(p+tempIndex,reads,pd,maxChars,false);
                }

            end:
                if (close)
                    stream.Close();
            return maxChars;
        }



        #endregion

        #region StreamWrite

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="strOrig">
        /// Cadena origen.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, string strOrig, bool noBestFitChar)
        {
            int charsCount=strOrig==null?0:strOrig.Length;
            if (charsCount==0)
            {
                if (close)
                    stream.Close();
                return 0;
            }
            fixed(char*p=strOrig)
                return StreamWrite(stream,signCP,noZeroSign,close,p,charsCount,noBestFitChar,null,null,null,0);
        }

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, char[] arrOrig, bool noBestFitChar)
        {
            int charsCount=arrOrig.Length;
            if (charsCount==0)
            {
                if (close)
                    stream.Close();
                return 0;
            }

            fixed(char*p=arrOrig)
                return StreamWrite(stream,signCP,noZeroSign,close,p,charsCount,noBestFitChar,null,null,null,0);
        }

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="strOrig">
        /// Cadena origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en la cadena origen donde comenzar la copia (en chars).</param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de strOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, string strOrig, int origIndex, int charsCount, bool noBestFitChar)
        {
            return StreamWrite(stream,signCP,noZeroSign,close,strOrig,origIndex,charsCount,noBestFitChar,null,null,null,0);
        }

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en el array origen donde comenzar la copia (en chars).</param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de arrOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, char[] arrOrig, int origIndex, int charsCount, bool noBestFitChar)
        {
            return StreamWrite(stream,signCP,noZeroSign,close,arrOrig,origIndex,charsCount,noBestFitChar,null,null,null,0);
        }


        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="po">
        /// Puntero origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de po debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, char* po, int charsCount, bool noBestFitChar)
        {
            return StreamWrite(stream,signCP,noZeroSign,close,po,charsCount,noBestFitChar,null,null,null,0);
        }


        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="strOrig">
        /// Cadena origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en la cadena origen donde comenzar la copia (en chars).</param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de strOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, string strOrig, int origIndex, int charsCount, bool noBestFitChar, char* def, int* rep)
        {
            return StreamWrite(stream,signCP,noZeroSign,close,strOrig,origIndex,charsCount,noBestFitChar,def,rep,null,0);
        }


        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="strOrig">
        /// Cadena origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en la cadena origen donde comenzar la copia (en chars).</param>
        /// <param name="arrTemp">
        /// Array de bytes temporal. Si es null o su capacidad til no es practica para esta funcionalidad, se crear uno.
        /// </param>
        /// <param name="tempIndex">
        /// Index en el array donde comienza el espacio utilizable para copia temporal (en bytes).
        /// </param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de strOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, string strOrig, int origIndex, int charsCount, bool noBestFitChar, char* def, int* rep, byte[] arrTemp, int tempIndex)
        {
            if (origIndex<0 || charsCount<0 || origIndex+charsCount>strOrig.Length)
                throw new ArgumentOutOfRangeException("origIndex,charsCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (charsCount==0)
            {
                if (rep!=null && *rep!=0)
                    *rep=0;
                if (close)
                    stream.Close();
                return 0;
            }

            fixed(char*p=strOrig)
                return StreamWrite(stream,signCP,noZeroSign,close,p+origIndex,charsCount,noBestFitChar,def,rep,arrTemp,tempIndex);
        }


        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en el array origen donde comenzar la copia (en chars).</param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de arrOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, char[] arrOrig, int origIndex, int charsCount, bool noBestFitChar, char* def, int* rep)
        {
            return StreamWrite(stream,signCP,noZeroSign,close,arrOrig,origIndex,charsCount,noBestFitChar,def,rep,null,0);
        }


        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en el array origen donde comenzar la copia (en chars).</param>
        /// <param name="arrTemp">
        /// Array de bytes temporal. Si es null o su capacidad til no es practica para esta funcionalidad, se crear uno.
        /// </param>
        /// <param name="tempIndex">
        /// Index en el array donde comienza el espacio utilizable para copia temporal (en bytes).
        /// </param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de arrOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, char[] arrOrig, int origIndex, int charsCount, bool noBestFitChar, char* def, int* rep, byte[] arrTemp, int tempIndex)
        {
            if (origIndex<0 || charsCount<0 || origIndex+charsCount>arrOrig.Length)
                throw new ArgumentOutOfRangeException("origIndex,charsCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (charsCount==0)
            {
                if (rep!=null && *rep!=0)
                    *rep=0;
                if (close)
                    stream.Close();
                return 0;
            }
            fixed(char*p=arrOrig)
                return StreamWrite(stream,signCP,noZeroSign,close,p+origIndex,charsCount,noBestFitChar,def,rep,arrTemp,tempIndex);
        }

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="po">
        /// Puntero origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de po debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, char* po, int charsCount, bool noBestFitChar, char* def, int* rep)
        {
            return StreamWrite(stream,signCP,noZeroSign,close,po,charsCount,noBestFitChar,def,rep,null,0);
        }

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="po">
        /// Puntero origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="arrTemp">
        /// Array de bytes temporal. Si es null o su capacidad til no es practica para esta funcionalidad, se crear uno.
        /// </param>
        /// <param name="tempIndex">
        /// Index en el array donde comienza el espacio utilizable para copia temporal (en bytes).
        /// </param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de po debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Aade firma de cdigo de pgina al stream, siempre y cuando el cdigo de pgina lo requiera
        /// y la cantidad de chars a codificar sea mayor a cero.
        /// </param>
        /// <param name="noZeroSign">
        /// Ignorado si signCP es false. Si true, firmar en cualquier parte del stream, en
        /// cambio, si false solo firmar si el stream se encuentra en posicin cero.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream, incluyendo firma.
        /// </returns>
        public int StreamWrite(Stream stream, bool signCP, bool noZeroSign, bool close, char* po, int charsCount, bool noBestFitChar, char* def, int* rep, byte[] arrTemp, int tempIndex)
        {
            if (po==null)
                throw new ArgumentNullException("po",Errors.NULL_POINTER);

            if (tempIndex<0)
                throw new ArgumentOutOfRangeException("tempIndex",tempIndex,Errors.NEGATIVE);

            
            int bytesCount=this.CalcMaxBytes(charsCount);
            int totBytes=0;
            if (bytesCount==0)
            {
                if (rep!=null && *rep!=0)
                    *rep=0;
                goto end;
            }

            
            if (signCP)
                totBytes=this.SignCP(stream,noZeroSign);

            if (this.codePage==CP_UnicodeLE)
            {
                if (rep!=null && *rep!=0)
                    *rep=0;
                return Serializer.StreamWrite(stream,close,(byte*)po,bytesCount,arrTemp,tempIndex)+totBytes;
            }

            int maxBuf=(arrTemp!=null?arrTemp.Length:0)-tempIndex;
            
            if (maxBuf>=bytesCount)
                goto onePass;

            if (maxBuf<32 || (maxBuf<512 && bytesCount/maxBuf>127))
            {
                if (maxBuf<0)
                    throw new ArgumentOutOfRangeException("tempIndex",tempIndex,Errors.SUPERIOR_ARRAY);
                tempIndex=0;
                if (bytesCount>=1<<20)
                    maxBuf=16384;
                else if (bytesCount>=1<<18)
                    maxBuf=8192;
                else if (bytesCount>=1<<16)
                    maxBuf=4096;
                else if (bytesCount<=8192)
                {
                    arrTemp=new byte[bytesCount];
                    goto onePass;
                }
                else
                    maxBuf=2048;
                arrTemp=new byte[maxBuf];
            }

            int chBuf=maxBuf/this.MaxSize;

            fixed(byte*p=arrTemp)
            {
                byte*pd=p+tempIndex;
                if (pd==po)
                    throw new ArgumentException(Errors.SER_STREAM_P_EQUAL_ARRAY,"arrTemp");
                int chTot=0;
                do
                {
                    bytesCount=this.CharsToBytes(noBestFitChar,po+chTot,chBuf,pd,maxBuf,def,rep,false);
                    if (bytesCount!=0)
                    { 
                        stream.Write(arrTemp,tempIndex,bytesCount);
                        totBytes+=bytesCount;
                        if (rep!=null && *rep!=0)
                            rep=null;
                    }
                    chTot+=chBuf;
                    if (charsCount-chTot<chBuf)
                    {
                        if (charsCount==chTot)
                            break;
                        chBuf=charsCount-chTot;
                    }

                }while(true);
                goto end;
            }

            onePass:

                fixed(byte*p=arrTemp)
                {
                    bytesCount=this.CharsToBytes(noBestFitChar,po,charsCount,p+tempIndex,bytesCount,def,rep,false);
                    if (bytesCount!=0)
                    { 
                        stream.Write(arrTemp,tempIndex,bytesCount);
                        totBytes+=bytesCount;
                    }
                }

            end:
                if (close)
                    stream.Close();
            return totBytes;

        }



        #endregion

        #region GetString

        /// <summary>
        /// Genera una cadena a partir de la lectura de un Stream, desde su posicin actual y decodificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cadena con los carcteres decodificados desde el Stream.
        /// </returns>
        public string GetString(Stream stream, bool close)
        {
            return GetString(stream,close,-1);
        }
        
        /// <summary>
        /// Genera una cadena a partir de la lectura de un Stream, desde su posicin actual y decodificando carcteres. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <param name="maxChars">
        /// Cantidad mxima en chars a decodificar. Si es -1 recuperar todo el contenido pendiente de lectura.
        /// </param>
        /// <returns>
        /// Cadena con los carcteres decodificados desde el Stream.
        /// </returns>
        public string GetString(Stream stream, bool close, int maxChars)
        {
            if(maxChars<1)
            {
                if (maxChars==-1)
                    this.CalcStreamRead(stream,ref maxChars);
                if (maxChars==0)
                {
                    if (close)
                        stream.Close();
                    return Others.Empty;
                }
                throw new ArgumentOutOfRangeException("maxChars",maxChars,Errors.MINOR_ONE_NEGATIVE);
            }
            string ret=Mutables.Create(0,maxChars);
            fixed(char*p=ret)
            {
                int l=this.StreamRead(stream,close,p,maxChars,null,0);
                if (l==0)
                    return Others.Empty;
                ((int*)p)[-1] = l;
            }
            return ret;
        }

        /// <summary>
        /// Genera una cadena a partir de la lectura de los bytes de entrada, decodificando carcteres. 
        /// </summary>
        /// <param name="po">Puntero de la cadena original. El Largo es detectado segn el cero final.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        public string GetString(byte* po)
        {
            return Mutables.Create(this,po,-1,0);
        }

        /// <summary>
        /// Genera una cadena a partir de la lectura de los bytes de entrada, decodificando carcteres. 
        /// </summary>
        /// <param name="po">Puntero de la cadena original.</param>
        /// <param name="bytesCount">Largo de la copia en bytes. Si es -1 se determina automticamente.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        public string GetString(byte* po, int bytesCount)
        {
            return Mutables.Create(this,po,bytesCount,0);
        }


        /// <summary>
        /// Genera una cadena a partir de la lectura de los bytes de entrada, decodificando carcteres. 
        /// </summary>
        /// <param name="arrOrig">Cadena original. El largo de copia es determinado segn largo de array.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        public string GetString(byte[] arrOrig)
        {
            return Mutables.Create(this,arrOrig);
        }


        /// <summary>
        /// Genera una cadena a partir de la lectura de los bytes de entrada, decodificando carcteres. 
        /// </summary>
        /// <param name="arrOrig">Cadena original.</param>
        /// <param name="origIndex">Inicio de la copia en la cadena original (en bytes).</param>
        /// <param name="bytesCount">Largo de la copia (en bytes). Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        public string GetString(byte[] arrOrig, int origIndex, int bytesCount)
        {
            return Mutables.Create(this,arrOrig,origIndex,bytesCount);
        }

        #endregion

        #region BytesWrite

        /// <summary>
        /// Efecta escritura en una coleccin de Bytes, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="col">
        /// Coleccin de Bytes. Si su codificacin subyacente es diferente a la de esta instancia, la actualizar,
        /// si el largo de la misma es cero, o de lo contrario generar una excepcin.
        /// </param>
        /// <param name="strOrig">
        /// Cadena origen.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en la coleccin.
        /// </returns>
        public int BytesWrite(Bytes col, string strOrig, bool noBestFitChar)
        {
            int charsCount=strOrig==null?0:strOrig.Length;
            if (charsCount==0)
                return 0;
            fixed(char*p=strOrig)
                return BytesWrite(col,p,charsCount,noBestFitChar,null,null);
        }

        /// <summary>
        /// Efecta escritura en una coleccin de Bytes, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="col">
        /// Coleccin de Bytes. Si su codificacin subyacente es diferente a la de esta instancia, la actualizar,
        /// si el largo de la misma es cero, o de lo contrario generar una excepcin.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en la coleccin.
        /// </returns>
        public int BytesWrite(Bytes col, char[] arrOrig, bool noBestFitChar)
        {
            int charsCount=arrOrig.Length;
            if (charsCount==0)
                return 0;
            fixed(char*p=arrOrig)
                return BytesWrite(col,p,charsCount,noBestFitChar,null,null);
        }

        /// <summary>
        /// Efecta escritura en una coleccin de Bytes, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="col">
        /// Coleccin de Bytes. Si su codificacin subyacente es diferente a la de esta instancia, la actualizar,
        /// si el largo de la misma es cero, o de lo contrario generar una excepcin.
        /// </param>
        /// <param name="strOrig">
        /// Cadena origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en la cadena origen donde comenzar la copia (en chars).</param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de strOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en la coleccin.
        /// </returns>
        public int BytesWrite(Bytes col, string strOrig, int origIndex, int charsCount, bool noBestFitChar)
        {
            return BytesWrite(col,strOrig,origIndex,charsCount,noBestFitChar,null,null);
        }

        /// <summary>
        /// Efecta escritura en una coleccin de Bytes, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="col">
        /// Coleccin de Bytes. Si su codificacin subyacente es diferente a la de esta instancia, la actualizar,
        /// si el largo de la misma es cero, o de lo contrario generar una excepcin.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en el array origen donde comenzar la copia (en chars).</param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de arrOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en la coleccin.
        /// </returns>
        public int BytesWrite(Bytes col, char[] arrOrig, int origIndex, int charsCount, bool noBestFitChar)
        {
            return BytesWrite(col,arrOrig,origIndex,charsCount,noBestFitChar,null,null);
        }

        /// <summary>
        /// Efecta escritura en una coleccin de Bytes, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="col">
        /// Coleccin de Bytes. Si su codificacin subyacente es diferente a la de esta instancia, la actualizar,
        /// si el largo de la misma es cero, o de lo contrario generar una excepcin.
        /// </param>
        /// <param name="strOrig">
        /// Cadena origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en la cadena origen donde comenzar la copia (en chars).</param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de strOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en la coleccin.
        /// </returns>
        public int BytesWrite(Bytes col, string strOrig, int origIndex, int charsCount, bool noBestFitChar, char* def, int* rep)
        {
            if (origIndex<0 || charsCount<0 || origIndex+charsCount>strOrig.Length)
                throw new ArgumentOutOfRangeException("origIndex,charsCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (charsCount==0)
                return 0;
            fixed(char*p=strOrig)
                return BytesWrite(col,p+origIndex,charsCount,noBestFitChar,def,rep);
        }

        /// <summary>
        /// Efecta escritura en una coleccin de Bytes, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="col">
        /// Coleccin de Bytes. Si su codificacin subyacente es diferente a la de esta instancia, la actualizar,
        /// si el largo de la misma es cero, o de lo contrario generar una excepcin.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="origIndex">Index en el array origen donde comenzar la copia (en chars).</param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de arrOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en la coleccin.
        /// </returns>
        public int BytesWrite(Bytes col, char[] arrOrig, int origIndex, int charsCount, bool noBestFitChar, char* def, int* rep)
        {
            if (origIndex<0 || charsCount<0 || origIndex+charsCount>arrOrig.Length)
                throw new ArgumentOutOfRangeException("origIndex,charsCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (charsCount==0)
                return 0;
            fixed(char*p=arrOrig)
                return BytesWrite(col,p+origIndex,charsCount,noBestFitChar,def,rep);
        }

        /// <summary>
        /// Efecta escritura en una coleccin de Bytes, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="col">
        /// Coleccin de Bytes. Si su codificacin subyacente es diferente a la de esta instancia, la actualizar,
        /// si el largo de la misma es cero, o de lo contrario generar una excepcin.
        /// </param>
        /// <param name="po">
        /// Puntero origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de po debe ser coherente con este valor.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en la coleccin.
        /// </returns>
        public int BytesWrite(Bytes col, char* po, int charsCount, bool noBestFitChar)
        {
            return BytesWrite(col,po,charsCount,noBestFitChar,null,null);
        }

        /// <summary>
        /// Efecta escritura en una coleccin de Bytes, desde su posicin actual, codificando carcteres. 
        /// </summary>
        /// <param name="col">
        /// Coleccin de Bytes. Si su codificacin subyacente es diferente a la de esta instancia, la actualizar,
        /// si el largo de la misma es cero, o de lo contrario generar una excepcin.
        /// </param>
        /// <param name="po">
        /// Puntero origen. Debe tener una capacidad coherente con charsCount.
        /// </param>
        /// <param name="charsCount">
        /// Cantidad de chars a escribr.
        /// La capacidad de po debe ser coherente con este valor.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter
        /// por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="def">
        /// Puntero a caracter de reemplazo.
        /// Si es nulo (mejor rendimiento) se usar el caracter de reemplazo determinado por el sistema
        /// ('?' en el caso de Ansi Latin1).
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a null sin generar excepcin.
        /// </param>
        /// <param name="rep">
        /// Puntero boleano que se setea diferente a 0 si ocurren reemplazos por el caracter por defecto
        /// (determinado en parametro o por el sistema).
        /// Puede ser nulo (mejor rendimiento) si no se desea determinar si ocurrieron reemplazos.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a 0 sin generar excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en la coleccin.
        /// </returns>
        public int BytesWrite(Bytes col, char* po, int charsCount, bool noBestFitChar, char* def, int* rep)
        {
            if (po==null)
                throw new ArgumentNullException("po",Errors.NULL_POINTER);

            int l=col.Length;
            if (l==0 || !col.IsDefinedCP)
            {
                this.ThrowIfUnSupported();
                col.CodePage=this;
            }
            else if (col.CodePage!=this)
                throw new InvalidOperationException(Errors.COL_CP_INCOMP);
            int bytesCount=this.CalcMaxBytes(po,charsCount,true);
            if (bytesCount==0)
            {
                if (rep!=null && *rep!=0)
                    *rep=0;
                return 0;
            }
            if (this.IsUnicodeLE)
            {
                col.Append((byte*)po,bytesCount);
                return bytesCount;
            }
            col.EnsureAppend(bytesCount);
            fixed(byte*p=col.Array)
            {
                bytesCount=this.CharsToBytes(noBestFitChar,po,charsCount,p+l,bytesCount,def,rep,false);
                col.Length=l+bytesCount;
            }
            return bytesCount;
        }


        #endregion

	}
}
