using System;
using EnPalermo.Lit;
using EnPalermo.Mem;

namespace EnPalermo.Txt
{
    /// <summary>
    /// Funciones estticas relativas al tratamiento de archivos CSV.
    /// </summary>
    public unsafe static class Csv
    {
        #region Generacion de CSV

        /// <summary>
        /// Genera texto Csv a partir de un array escalonado.
        /// La conversin se efectuar de este modo:
        /// - Los elementos que contengan separador sern encomillados.
        /// - Si se encuentran cr y lfs estos sern reemplazados por repCrLf.
        /// </summary>
        /// <param name="csv">Array Escalonado.</param>
        /// <param name="sep">Separador de elementos. No puede ser igual CR, LF, iQuote, fQuote o repCrLf.</param>
        /// <param name="iQuote">Comilla inicial. No puede ser igual a CR, LF, sep o repCrLf.</param>
        /// <param name="fQuote">Comilla final. No puede ser igual a CR, LF, sep o repCrLf.</param>
        /// <param name="repCrLf">
        /// Caracter de reemplazo para las combinaciones encontradas de Cr y/o Lf.
        /// No puede ser igual a CR, LF, sep, iQuote o fQuote.</param>
        /// <param name="commText">
        /// Si true, todos los elementos no considerados numeros (incluyendo los vacios) sern encomillados.
        /// Para la verificacin numrica se usa la funcin IsNumber.
        /// </param>
        /// <param name="decimalDot">
        /// Si true, se considera el punto como separador decimal en la verificacin numrica. Si false, se usa la coma.
        /// Si se da coincidencia entre separador decimal y separador (sep==',' y false o sep=='.' y true)
        /// todos los nmeros que contengan separador decimal, tambin sern considerados textos.
        /// </param>
        /// <param name="allowBool">
        /// Si true, en la verificacin numrica considera a las cadenas "t", "true", "f" y "false" como boleanas (ignorando May/Min).
        /// </param>
        /// <param name="ignoreEmpty">Si true, ignora registros vacios. Si false, inserta un Cr/Lf adicional por cada registro vacio.</param>
        /// <param name="unix">Si true, se usara solo el caracter LF como separador de lneas.</param>
        /// <returns>Texto Csv o cadena vacia.</returns>
        public static string CreateCsv(string[][] csv,char sep,char iQuote,char fQuote,char repCrLf,bool commText,bool decimalDot, bool allowBool,bool unix,bool ignoreEmpty)
        {
            int lReg=csv.Length;
            if (lReg==0) return Others.Empty;
		
            unsafe
            {
                Mem.Heap hReg = new Mem.Heap(lReg << 2, true);
                int * iReg=(int*)hReg.Pointer;
                int a=0;
                string[] actReg;
                int lactReg;
                int lFinal=0;
                int totElem=0;
                int sumLine=unix?1:2;
                // primera pasada: determina el total de elementos en cada registro.
                do
                {
                    actReg=csv[a];
                    lactReg=actReg==null?0:actReg.Length;
					
                    if(lactReg!=0)
                    {
                        iReg[a]=lactReg;
                        totElem+=lactReg;
                        lFinal+=lactReg+sumLine-1;
                    } 
                    else if(!ignoreEmpty) lFinal+=sumLine;
                }while(++a!=lReg);
                if (lFinal==0) 
                {
                    hReg.Dispose();
                    return Others.Empty;
                }
                if (
                    sep==iQuote || sep==fQuote || sep==repCrLf ||
                    iQuote==repCrLf || fQuote==repCrLf || sep=='\r' ||
                    sep=='\n' || iQuote=='\r' || iQuote=='\n' ||
                    fQuote=='\r' || fQuote=='\n' || repCrLf=='\r' ||
                    repCrLf=='\n')
                    throw new FormatException(Errors.CSV_ITEMS);

                Mem.Heap hcomis = new Mem.Heap(totElem, true);
                bool*comis=(bool*)hcomis.Pointer;

                totElem=-1;
                a=0;
                string actElem;
                int lactElem;
                int b;
				
                // segunda pasada: determina los elementos que requieren encomillado y finaliza la determinacin del largo final.
                do
                {
                    lactReg=iReg[a];
                    if(lactReg!=0)
                    {
                        actReg=csv[a];
                        b=0;
                        do
                        {
                            totElem++;
                            actElem=actReg[b];
                            lactElem=actElem==null?0:actElem.Length;
                            if (lactElem!=0)
                            {
                                fixed(char*pelem=actElem)
                                {
                                    if (Comp.Exist(pelem,lactElem,sep) || (commText && !Validate.IsNumber(true,pelem,lactElem,decimalDot,allowBool)))
                                    {
                                        if (lactElem==1 || *pelem!=iQuote || pelem[lactElem-1]!=fQuote)
                                        {
                                            comis[totElem]=true;
                                            lFinal+=2;
                                        }
                                    }
                                }
                                lFinal+=lactElem;
                            } 
                            else if (commText) lFinal+=2;
                        }while(++b!=lactReg);
                    }
                }while(++a!=lReg);

                string final=Mutables.Create(lFinal);
                a=0;
                totElem=-1;
                int cop=0;
                int c;
                char cact;
                fixed(char *pfinal=final)
                {
                    // Tercera Pasada, efectua la conversin...
                    do
                    {
                        lactReg=iReg[a];
                        if(lactReg!=0)
                        {
                            actReg=csv[a];
                            b=0;
                            do
                            {
                                totElem++;
                                actElem=actReg[b];
                                lactElem=actElem==null?0:actElem.Length;
                                if (lactElem!=0)
                                {
                                    fixed(char*pelem=actElem)
                                    {
                                        if (comis[totElem]) pfinal[cop++]=iQuote;
                                        c=0;
                                        do
                                        {
                                            cact=pelem[c];
                                            switch(cact)
                                            {
                                                case '\r':
                                                    pfinal[cop++]=repCrLf;
                                                    int prox=c+1;
                                                    if (prox==lactElem) break;
                                                    if (pelem[prox]=='\n') c++;
                                                    continue;
                                                case '\n':
                                                    pfinal[cop++]=repCrLf;
                                                    continue;
                                                default:
                                                    pfinal[cop++]=cact;
                                                    continue;
                                            }
                                            break;
                                        }while(++c!=lactElem);
                                        if (comis[totElem]) pfinal[cop++]=fQuote;
                                    }
											
                                }
                                else if(commText)
                                {
                                    pfinal[cop++]=iQuote;
                                    pfinal[cop++]=fQuote;
                                }
                                if (++b==lactReg) break;
                                pfinal[cop++]=sep;
                            }while(true);
                            if (unix) pfinal[cop++]='\n';
                            else
                            {
                                *(int*)(pfinal+cop)=Texts.CharsCrLf;
                                cop+=2;
                            }
                        }
                        else if(!ignoreEmpty)
                        {
                            if (unix) pfinal[cop++]='\n';
                            else
                            {
                                *(int*)(pfinal+cop)=Texts.CharsCrLf;
                                cop+=2;
                            }
                        }
                    }while(++a!=lReg);
                    if (lFinal!=cop)
                    {
                        *(int*)(pfinal-2)=cop;
                    }
                    hReg.Dispose();
                    hcomis.Dispose();
                    return final;
                }
            }
        }

        #endregion

	}
}
