using System;
using EnPalermo.Mem;
using EnPalermo.Lit;

namespace EnPalermo.Txt
{
    /// <summary>
    /// Cadenas mutables.
    /// Las cadenas mutables pueden modificarse directamente en cdigo inseguro con la garanta de que
    /// no se afectar la tabla de cadenas de Net - Framework.
    /// Por lo tanto, estas cadenas son similares a las creadas con StringBuilder.
    /// </summary>
    public unsafe static class Mutables
    {


        #region Campos y Constructor esttico.
        /// <summary>
        /// Diferencia en Ints entre el primer dato en array y el comienzo del objeto.
        /// 2 en sistemas de 32 bits, 3 en sistemas de 64 bits, 0 cuando es modo compatible (Mono y otras donde no pasa el test).
        /// </summary>
        private static readonly int arrayDataInt;

        /// <summary>
        /// Handle del Tipo String.
        /// </summary>
        private static readonly IntPtr typeString;

        /// <summary>
        /// Verifica compatibilidad e inicializa el handle del Tipo String.
        /// </summary>
        static Mutables()
        {
            arrayDataInt = IntPtr.Size == 8 ? 3 : 2;
            int zof = sizeof(void*);
            // Evita verificacin en plataforma 64...
            if (zof == 8)
                goto compatible;
            string tmpstr;
            if (IntPtr.Size != zof)
                goto compatible;
            if (sizeof(System.TypedReference) != zof << 1)
                goto compatible;
            if (System.Runtime.CompilerServices.RuntimeHelpers.OffsetToStringData != zof + 8)
                goto compatible;
            typeString = typeof(string).TypeHandle.Value;
            tmpstr = Others.Empty;
            TypedReference tr = __makeref(tmpstr);
            if (((IntPtr*)&tr)[1] != typeString)
                goto compatible;
            int[] tmp = new int[5];
            fixed (int* pi = tmp)
            {
                int* p = pi - arrayDataInt;
                if (p[-1] != 0)
                    goto compatible;
                tr = __makeref(tmp);
                System.TypedReference* ptr = &tr;

                if ((**(IntPtr**)ptr).ToPointer() != p)
                    goto compatible;
                if (pi[-1] != 5)
                    goto compatible;
                *(IntPtr*)p = typeString;
                pi[-1] = 8;
                ((IntPtr*)ptr)[1] = typeString;
                tmpstr = __refvalue( tr,string);
                tmp = null;
            }
            fixed (char* p = tmpstr)
            {
                *(long*)p = 0x65007500720050;
                p[-2] = '\x4';
            }
            if (tmpstr.Length != 4)
                goto compatible;
            if (tmpstr != "Prue")
                goto compatible;
            return;

            // Modo compatible...

            compatible:
            arrayDataInt = 0;
            typeString = IntPtr.Zero;
            tmpstr = new string('\0',8);
            if (!IsMutable(tmpstr))
                 throw new System.PlatformNotSupportedException(Errors.ERROR_MUTABLES + '1');
            fixed (char* p = tmpstr)
            {
                *(long*)p = 0x65007500720050;
                p[-2] = '\x4';
            }
            if (tmpstr.Length != 4)
                throw new System.PlatformNotSupportedException(Errors.ERROR_MUTABLES + '2');
            if (tmpstr != "Prue")
                throw new System.PlatformNotSupportedException(Errors.ERROR_MUTABLES + '3');
        }


        #endregion

        #region Generadores bsicos.

        /// <summary>
        /// Genera una cadena mutable: Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="l">
        /// Largo final de la cadena. Debe ser mayor a -1 y menor a 1073741820.
        /// </param>
        /// <returns>Nueva cadena rellena de ceros con el largo de l y una capacidad
        /// (arrayLength) del siguiente nmero par mayor a l.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(int l)
        {
            if (l<1)
            {
                if (l!=0)
                    throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
                return Others.Empty;
            }
            if (arrayDataInt!=0)
            {
                int extraCap=(l&1)!=0?(l+1):(l+2);
                int[] tmp = new int[(extraCap >> 1) + 1];
                fixed (int* p = tmp)
                {
                    *(IntPtr*)(p - arrayDataInt) = typeString;
                    p[-1] = extraCap;
                    *p = l;
                    TypedReference tr = __makeref(tmp);
                    ((IntPtr*)&tr)[1] = typeString;
                    return __refvalue( tr,string);
                }
            }
            return new string('\0',l);
            
        }

        /// <summary>
        /// Genera una cadena mutable: Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="l">
        /// Largo de la cadena. Debe ser mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <returns>
        /// Nueva cadena rellena de ceros con el largo de l y una capacidad
        /// (arrayLength) del nmero par mayor a extraCap + l.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(int l, int extraCap)
        {
            if (extraCap<1)
            {
                if (extraCap!=0)
                    throw new ArgumentOutOfRangeException("extraCap",extraCap,Errors.NEGATIVE);
                return Create(l);
            }
            if (arrayDataInt!=0)
            {
                if (l<1)
                {
                    if (l!=0)
                        throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
                    if ((extraCap&1)!=0)
                        extraCap++;
                    else
                        extraCap+=2;
                }
                else
                {
                    extraCap+=l+1;
                    if ((extraCap&1)!=0)
                        extraCap++;
                }

                int[] tmp = new int[(extraCap >> 1) + 1];
                fixed (int* p = tmp)
                {
                    *(IntPtr*)(p - arrayDataInt) = typeString;
                    p[-1] = extraCap;
                    if(l!=0)
                        *p = l;
                    TypedReference tr = __makeref(tmp);
                    ((IntPtr*)&tr)[1] = typeString;
                    return __refvalue( tr,string);
                }
            }
            if (l<0)
                throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
            string ret=new string('\0',l+extraCap);
            fixed(char*p=ret)
                ((int*)p)[-1]=l;
            return ret;
            
        }

        /// <summary>
        /// Genera una cadena mutable: Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="l">
        /// Largo final de la cadena. Debe ser mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno.</param>
        /// <returns>Nueva cadena rellena con el caracter especificado y con el largo de l y una capacidad
        /// (arrayLength) del siguiente nmero par mayor a l.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(int l, char fill)
        {
            if (arrayDataInt!=0)
            {
                if (l<1)
                {
                    if (l!=0)
                        throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
                    return Others.Empty;
                }
                int extraCap=(l&1)!=0?(l+1):(l+2);
                int[] tmp = new int[(extraCap >> 1) + 1];
                fixed (int* p = tmp)
                {
                    *(IntPtr*)(p - arrayDataInt) = typeString;
                    p[-1] = extraCap;
                    *p = l;
                    if (fill!='\0')
                        General.Fill((char*)(p + 1),l,fill);
                    TypedReference tr = __makeref(tmp);
                    ((IntPtr*)&tr)[1] = typeString;
                    return __refvalue( tr,string);
                }
            }
            return new string(fill,l);
        }

        /// <summary>
        /// Genera una cadena mutable: Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="l">
        /// Largo de la cadena. Debe ser mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <param name="fill">Caracter de relleno.</param>
        /// <returns>
        /// Nueva cadena rellena con el caracter especificado y con el largo de l y una capacidad
        /// (arrayLength) del nmero par mayor a extraCap + l.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(int l, int extraCap, char fill)
        {
            if (extraCap<1)
            {
                if (extraCap!=0)
                    throw new ArgumentOutOfRangeException("extraCap",extraCap,Errors.NEGATIVE);
                return Create(l,fill);
            }
            if (arrayDataInt!=0)
            {
                if (l<1)
                {
                    if (l!=0)
                        throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
                    if ((extraCap&1)!=0)
                        extraCap++;
                    else
                        extraCap+=2;
                }
                else
                {
                    extraCap+=l+1;
                    if ((extraCap&1)!=0)
                        extraCap++;
                }
                int[] tmp = new int[(extraCap >> 1) + 1];
                fixed (int* p = tmp)
                {
                    *(IntPtr*)(p - arrayDataInt) = typeString;
                    p[-1] = extraCap;
                    if(l!=0)
                    {
                        *p = l;
                        if (fill!='\0')
                            General.Fill((char*)(p + 1),l,fill);
                    }
                    TypedReference tr = __makeref(tmp);
                    ((IntPtr*)&tr)[1] = typeString;
                    return __refvalue( tr,string);
                }
            }
            if (l<0)
                throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
            string ret=new string(fill,l+extraCap);
            fixed(char*p=ret)
            {
                ((int*)p)[-1]=l;
                if (fill!='\0')
                    General.Clean(p+l,extraCap);
            }
            return ret;
        }


        #endregion

        #region Generadores desde punteros chars

        /// <summary>
        /// A partir de la copia de un puntero de cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original. El Largo es detectado segn el cero final.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char* pcad)
        {
            return Create(pcad,Comp.StrLen(pcad));
        }

        /// <summary>
        /// A partir de la copia de un puntero de cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char* pcad, int lcop)
        {
            string ret=Create(lcop);
            if (lcop!=0)
            {
                fixed(char*p=ret)
                    General.Copy(pcad, p, lcop);
            }
            return ret;
        }

        /// <summary>
        /// A partir de la copia de un puntero de cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char* pcad, int lcop, int extraCap)
        {
            string ret=Create(lcop,extraCap);
            if (lcop!=0)
            {
                fixed(char*p=ret)
                    General.Copy(pcad, p, lcop);
            }
            return ret;
            
        }

        /// <summary>
        /// A partir de la copia de un puntero de cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char* pcad, int lcop, int l, char fill)
        {
            if (lcop>=l)
            {
                if (lcop!=l)
                    throw new ArgumentOutOfRangeException("lcop",lcop,Errors.SUPERIOR_ARRAY);
                return Create(pcad,lcop);
            }
            if (lcop<1)
            {
                if (lcop!=0)
                    throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE);
                return Create(l,fill);
            }
            string ret=Create(l);
            fixed(char*p=ret)
            {
                General.Copy(pcad, p, lcop);
                if (fill!='\0')
                    General.Fill(p+lcop,l-lcop,fill);
            }
            return ret;
        }

        /// <summary>
        /// A partir de la copia de un puntero de cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char* pcad, int lcop, int inew, int l, char fill)
        {
            if (inew<1)
            {
                if (inew!=0)
                    throw new ArgumentOutOfRangeException("inew",inew,Errors.NEGATIVE);
                return Create(pcad,lcop,l,fill);
            }

            if (lcop<1)
            {
                if (lcop!=0)
                    throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE);
                return Create(l,fill);
            }
            int tcop=lcop+inew;
            if (tcop>l)
                throw new ArgumentOutOfRangeException("lcop,inew",Errors.SUPERIOR_ARRAY);

            string ret=Create(l);
            fixed(char*pc=ret)
            {
                if (fill!='\0')
                {
                    General.Fill(pc,inew,fill);
                    General.Copy(pcad, pc + inew, lcop);
                    if (tcop!=l)
                        General.Fill(pc+tcop,l-tcop,fill);
                }
                else
                    General.Copy(pcad, pc + inew, lcop);
            }
            return ret;
        }

        /// <summary>
        /// A partir de la copia de un puntero de cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char* pcad, int lcop, int inew, int l, int extraCap, char fill)
        {
            
            if (extraCap<1)
            {
                if (extraCap!=0)
                    throw new ArgumentOutOfRangeException("extraCap",extraCap,Errors.NEGATIVE);
                return Create(pcad,lcop,inew,l,fill);
            }

            if (lcop<1)
            {
                if (lcop!=0)
                    throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE);
                return Create(l,extraCap,fill);
            }

            int tcop;
            if (inew<1)
            {
                if (inew!=0)
                    throw new ArgumentOutOfRangeException("inew",inew,Errors.NEGATIVE);
                tcop=lcop;
            }
            else
                tcop=lcop+inew;
            if (tcop>l)
                throw new ArgumentOutOfRangeException("lcop,inew",Errors.SUPERIOR_ARRAY);

            
            string ret=Create(l,extraCap);
            fixed(char*pc=ret)
            {
                if (fill!='\0')
                {
                    General.Fill(pc,inew,fill);
                    General.Copy(pcad, pc + inew, lcop);
                    if (tcop!=l)
                        General.Fill(pc+tcop,l-tcop,fill);
                }
                else
                    General.Copy(pcad, pc + inew, lcop);
            }
            return ret;
        }


        #endregion

        #region Generadores desde punteros bytes latin1

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original. El Largo es detectado segn el cero final.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte* pcad)
        {
            return Create(pcad,Comp.StrLen(pcad));
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte* pcad, int lcop)
        {
            string ret=Create(lcop);
            if (lcop!=0)
            {
                fixed(char*p=ret)
                    Texts.Latin1ToChars(pcad, p, lcop);
            }
            return ret;
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte* pcad, int lcop, int extraCap)
        {
            string ret=Create(lcop,extraCap);
            if (lcop!=0)
            {
                fixed(char*p=ret)
                    Texts.Latin1ToChars(pcad, p, lcop);
            }
            return ret;
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte* pcad, int lcop, int l, char fill)
        {
            if (lcop>=l)
            {
                if (lcop!=l)
                    throw new ArgumentOutOfRangeException("lcop",lcop,Errors.SUPERIOR_ARRAY);
                return Create(pcad,lcop);
            }
            if (lcop<1)
            {
                if (lcop!=0)
                    throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE);
                return Create(l,fill);
            }
            string ret=Create(l);
            fixed(char*p=ret)
            {
                Texts.Latin1ToChars(pcad, p, lcop);
                if (fill!='\0')
                    General.Fill(p+lcop,l-lcop,fill);
            }
            return ret;
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte* pcad, int lcop, int inew, int l, char fill)
        {
            if (inew<1)
            {
                if (inew!=0)
                    throw new ArgumentOutOfRangeException("inew",inew,Errors.NEGATIVE);
                return Create(pcad,lcop,l,fill);
            }

            if (lcop<1)
            {
                if (lcop!=0)
                    throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE);
                return Create(l,fill);
            }
            int tcop=lcop+inew;
            if (tcop>l)
                throw new ArgumentOutOfRangeException("lcop,inew",Errors.SUPERIOR_ARRAY);

            string ret=Create(l);
            fixed(char*pc=ret)
            {
                if (fill!='\0')
                {
                    General.Fill(pc,inew,fill);
                    Texts.Latin1ToChars(pcad, pc + inew, lcop);
                    if (tcop!=l)
                        General.Fill(pc+tcop,l-tcop,fill);
                }
                else
                    Texts.Latin1ToChars(pcad, pc + inew, lcop);
            }
            return ret;
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte* pcad, int lcop, int inew, int l, int extraCap, char fill)
        {

            if (extraCap<1)
            {
                if (extraCap!=0)
                    throw new ArgumentOutOfRangeException("extraCap",extraCap,Errors.NEGATIVE);
                return Create(pcad,lcop,inew,l,fill);
            }

            if (lcop<1)
            {
                if (lcop!=0)
                    throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE);
                return Create(l,extraCap,fill);
            }

            int tcop;
            if (inew<1)
            {
                if (inew!=0)
                    throw new ArgumentOutOfRangeException("inew",inew,Errors.NEGATIVE);
                tcop=lcop;
            }
            else
                tcop=lcop+inew;
            if (tcop>l)
                throw new ArgumentOutOfRangeException("lcop,inew",Errors.SUPERIOR_ARRAY);

            
            string ret=Create(l,extraCap);
            fixed(char*pc=ret)
            {
                if (fill!='\0')
                {
                    General.Fill(pc,inew,fill);
                    Texts.Latin1ToChars(pcad, pc + inew, lcop);
                    if (tcop!=l)
                        General.Fill(pc+tcop,l-tcop,fill);
                }
                else
                    Texts.Latin1ToChars(pcad, pc + inew, lcop);
            }
            return ret;
        }


        #endregion

        #region Generadores desde punteros sbytes Latin1

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original. El Largo es detectado segn el cero final.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(sbyte* pcad)
        {
            return Create((byte*)pcad);
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(sbyte* pcad, int lcop)
        {
            return Create((byte*)pcad,lcop);
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(sbyte* pcad, int lcop, int extraCap)
        {
            return Create((byte*)pcad,lcop,extraCap);
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(sbyte* pcad, int lcop, int l, char fill)
        {
            return Create((byte*)pcad,lcop,l,fill);
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(sbyte* pcad, int lcop, int inew, int l, char fill)
        {
            return Create((byte*)pcad,lcop,inew,l,fill);
        }

        /// <summary>
        /// A partir de la copia de un puntero Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(sbyte* pcad, int lcop, int inew, int l, int extraCap, char fill)
        {
            return Create((byte*)pcad,lcop,inew,l,extraCap,fill);
        }

        #endregion

        #region Generadores desde punteros bytes con codificacin

        /// <summary>
        /// A partir de la copia de un puntero con la codificacin especificada, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="codePage">
        /// Cdigo de pgina.
        /// </param>
        /// <param name="pcad">Puntero de la cadena original. El Largo es detectado segn el cero final.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(CodePage codePage, byte* pcad)
        {
            return Create(codePage,pcad,-1,0);
        }

        /// <summary>
        /// A partir de la copia de un puntero con la codificacin especificada, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="codePage">
        /// Cdigo de pgina.
        /// </param>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lBytes">Largo de la copia en bytes. Si es -1 se determina automticamente.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(CodePage codePage, byte* pcad, int lBytes)
        {
            return Create(codePage,pcad,lBytes,0);
        }

        /// <summary>
        /// A partir de la copia de un puntero con la codificacin especificada, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="codePage">
        /// Cdigo de pgina.
        /// </param>
        /// <param name="pcad">Puntero de la cadena original.</param>
        /// <param name="lBytes">Largo de la copia en bytes. Si es -1 se determina automticamente.</param>
        /// <param name="extraCap">
        /// Capacidad extra en chars a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(CodePage codePage, byte* pcad, int lBytes, int extraCap)
        {
            if (lBytes==-1)
                lBytes=codePage.StrLen(pcad);
            int lcop=codePage.CalcMaxChars(pcad,lBytes,true);
            string ret=Create(lcop,extraCap);
            if (lcop!=0)
            {
                fixed(char*p=ret)
                    ((int*)p)[-1] = codePage.BytesToChars(pcad,lBytes,p,lcop,false);
            }
            return ret;
        }

        #endregion

        #region Generadores desde arrays de bytes Latin1

        /// <summary>
        /// A partir de la copia de un array Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original. El largo de copia es determinado segn largo de array.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte[] str)
        {
            int lcop=str.Length;
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(pcad,lcop);
            return Others.Empty;
        }

        /// <summary>
        /// A partir de la copia de un array Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte[] str, int lcop)
        {
            if (lcop<0 || lcop>str.Length)
                throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(pcad,lcop);
            return Others.Empty;        
        }

        /// <summary>
        /// A partir de la copia de un array Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte[] str, int icop, int lcop)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(pcad+icop,lcop);
            return Others.Empty;
        }

        /// <summary>
        /// A partir de la copia de un array Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte[] str, int icop, int lcop, int extraCap)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(pcad+icop,lcop,extraCap);
            return Create(0,extraCap);
        }




        /// <summary>
        /// A partir de la copia de un array Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte[] str, int icop, int lcop, int l, char fill)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(pcad+icop,lcop,l,fill);
            return Create(l,fill);
        }

        /// <summary>
        /// A partir de la copia de un array Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte[] str, int icop, int lcop, int inew, int l, char fill)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(pcad+icop,lcop,inew,l,fill);
            return Create(l,fill);

        }

        /// <summary>
        /// A partir de la copia de un array Latin1, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(byte[] str, int icop, int lcop, int inew, int l, int extraCap, char fill)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(pcad+icop,lcop,inew,l,extraCap,fill);
            return Create(l,extraCap,fill);
        }

        #endregion

        #region Generadores desde arrays de bytes con codificacin

        /// <summary>
        /// A partir de la copia de un array de bytes con la cdificacin especificada, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="codePage">
        /// Cdigo de pgina.
        /// </param>
        /// <param name="str">Cadena original. El largo de copia es determinado segn largo de array.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(CodePage codePage, byte[] str)
        {
            int lcop=str.Length;
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(codePage,pcad,lcop,0);
            return Others.Empty;
        }

        /// <summary>
        /// A partir de la copia de un array de bytes con la cdificacin especificada, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="codePage">
        /// Cdigo de pgina.
        /// </param>
        /// <param name="str">Cadena original.</param>
        /// <param name="lcop">Largo de la copia (en bytes). Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(CodePage codePage, byte[] str, int lcop)
        {
            if (lcop<0 || lcop>str.Length)
                throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(codePage,pcad,lcop,0);
            return Others.Empty;        
        }

        /// <summary>
        /// A partir de la copia de un array de bytes con la cdificacin especificada, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="codePage">
        /// Cdigo de pgina.
        /// </param>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original (en bytes).</param>
        /// <param name="lcop">Largo de la copia (en bytes). Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(CodePage codePage, byte[] str, int icop, int lcop)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(codePage,pcad+icop,lcop,0);
            return Others.Empty;
        }

        /// <summary>
        /// A partir de la copia de un array de bytes con la cdificacin especificada, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="codePage">
        /// Cdigo de pgina.
        /// </param>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original (en bytes).</param>
        /// <param name="lcop">Largo de la copia (en bytes). Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(CodePage codePage, byte[] str, int icop, int lcop, int extraCap)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(byte*pcad=str)
                    return Create(codePage,pcad+icop,lcop,extraCap);
            return Create(0,extraCap);
        }


        #endregion

        #region Generadores desde arrays de chars

        /// <summary>
        /// A partir de la copia de un array de chars, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original. El largo de copia es determinado segn largo de array.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char[] str)
        {
            int lcop=str.Length;
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad,lcop);
            return Others.Empty;
        }

        /// <summary>
        /// A partir de la copia de un array de chars, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char[] str, int lcop)
        {
            if (lcop<0 || lcop>str.Length)
                throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad,lcop);
            return Others.Empty;        
        }

        /// <summary>
        /// A partir de la copia de un array de chars, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char[] str, int icop, int lcop)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop);
            return Others.Empty;
        }

        /// <summary>
        /// A partir de la copia de un array de chars, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char[] str, int icop, int lcop, int extraCap)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop,extraCap);
            return Create(0,extraCap);
        }

        /// <summary>
        /// A partir de la copia de un array de chars, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char[] str, int icop, int lcop, int l, char fill)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop,l,fill);
            return Create(l,fill);
        }

        /// <summary>
        /// A partir de la copia de un array de chars, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char[] str, int icop, int lcop, int inew, int l, char fill)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop,inew,l,fill);
            return Create(l,fill);

        }

        /// <summary>
        /// A partir de la copia de un array de chars, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(char[] str, int icop, int lcop, int inew, int l, int extraCap, char fill)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop,inew,l,extraCap,fill);
            return Create(l,extraCap,fill);
        }

        #endregion

        #region Generadores desde strings

        /// <summary>
        /// A partir de la copia de otra cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original. El largo de copia es determinado segn largo de cadena.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(string str)
        {
            int lcop=str==null?0:str.Length;
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad,lcop);
            return Others.Empty;
        }

        /// <summary>
        /// A partir de la copia de otra cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(string str, int lcop)
        {
            if (lcop<0 || lcop>str.Length)
                throw new ArgumentOutOfRangeException("lcop",lcop,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad,lcop);
            return Others.Empty;        
        }

        /// <summary>
        /// A partir de la copia de otra cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(string str, int icop, int lcop)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop);
            return Others.Empty;
        }

        /// <summary>
        /// A partir de la copia de otra cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia y de la nueva cadena. Debe ser mayor a -1 y menor a 1073741820.</param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <returns>Nueva cadena con el contenido de la original.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(string str, int icop, int lcop, int extraCap)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop,extraCap);
            return Create(0,extraCap);
        }

        /// <summary>
        /// A partir de la copia de otra cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(string str, int icop, int lcop, int l, char fill)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop,l,fill);
            return Create(l,fill);
        }

        /// <summary>
        /// A partir de la copia de otra cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(string str, int icop, int lcop, int inew, int l, char fill)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop,inew,l,fill);
            return Create(l,fill);

        }

        /// <summary>
        /// A partir de la copia de otra cadena, genera una cadena mutable:
        /// Puede ser modificada mediante punteros
        /// sin alterar otros objetos de cadena.
        /// </summary>
        /// <param name="str">Cadena original.</param>
        /// <param name="icop">Inicio de la copia en la cadena original.</param>
        /// <param name="lcop">Largo de la copia.
        /// </param>
        /// <param name="inew">Inicio de la copia en la nueva cadena.</param>
        /// <param name="l">Largo total de la nueva cadena. Debe ser ser coherente con los otros parmetros,
        /// mayor a -1 y menor a 1073741820.
        /// </param>
        /// <param name="extraCap">
        /// Capacidad extra a agregar al final de la cadena sin inclur el cero final.
        /// </param>
        /// <param name="fill">Caracter de relleno para las zonas vacias.</param>
        /// <returns>Nueva cadena con el contenido especificado.</returns>
        /// <remarks>
        /// Nota 1: En los casos donde el largo de array de la cadena y el largo pretendido
        /// es cero, la funcin devolver la constante de cadena vacia estandr ("") en vez de una
        /// cadena mutable.
        /// Nota 2: Para operar con esta cadena, siempre tener en cuenta
        /// que debe respetarse el largo de array (ubicado en ((int*)p)-2)
        /// y modificar el largo de cadena (ubicado en ((int*)p)-1)
        /// para establecer el tamao.
        /// </remarks>
        public static string Create(string str, int icop, int lcop, int inew, int l, int extraCap, char fill)
        {
            if (lcop<0 || icop<0 || icop+lcop>str.Length)
                throw new ArgumentOutOfRangeException("icop,lcop",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lcop!=0)
                fixed(char*pcad=str)
                    return Create(pcad+icop,lcop,inew,l,extraCap,fill);
            return Create(l,extraCap,fill);
        }

        #endregion

        #region Utilidades


        /// <summary>
        /// Si retorna true, significa que esta operando en modo compatible con Mono.
        /// Dado que Mono no guarda informacin de largo de array, el mtodo ArrayLength() siempre 
        /// retornar el largo de cadena+1 y el mtodo Capacity() retornar directamente
        /// el largo de cadena.
        /// </summary>
        public static bool CompatibleMode
        {
            get
            {
                return arrayDataInt==0;
            }
        }


        /// <summary>
        /// Modifica el largo en una cadena mutable.
        /// </summary>
        /// <param name="str">Cadena. Debe ser mutable!</param>
        /// <param name="l">Largo.</param>
        /// <param name="zeroEnd">Inserta '\0' al final.</param>
        /// <returns>La misma instancia de cadena.</returns>
        public static string SetLength(string str, int l, bool zeroEnd)
        {
            fixed (char* p = str)
            {
                ((int*)p)[-1] = l;
                if (zeroEnd)
                    p[l] = '\0';
            }
            return str;
        }

        /// <summary>
        /// Obtiene el largo de array de la cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>Largo de array de la cadena (incluyendo 0 final).</returns>
        public static int ArrayLength(string str)
        {
            if (arrayDataInt!=0)
                fixed (char* p = str)
                    return ((int*)p)[-2];
            return str.Length+1;
        }

        /// <summary>
        /// Obtiene el largo de array de la cadena - 1.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>Largo de array de la cadena (sin incluir 0 final).</returns>
        public static int Capacity(string str)
        {
            if (arrayDataInt!=0)
                fixed (char* p = str)
                    return ((int*)p)[-2]-1;
            return str.Length;
        }


        /// <summary>
        /// Obtiene el campo Length de la cadena. No usar como sustituto de String.Length
        /// dado que los 2 bits mas significativos del campo pueden ser usados como flags.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>Campo Length correspondiente a la cadena.</returns>
        public static int FieldLength(string str)
        {
            fixed (char* p = str)
                return ((int*)p)[-1];
        }

        /// <summary>
        /// Determina si la cadena de entrada es mutable (no interned).
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>True si la cadena de entrada es mutable (no interned).</returns>
        public static bool IsMutable(string str)
        {
            return str != null && !Object.ReferenceEquals(String.IsInterned(str), str);
        }

        #endregion

    }
}
