using System;
using EnPalermo.Lit;

namespace EnPalermo.Txt
{
    /// <summary>
    /// Opcin de prefix para cadenas Hex.
    /// </summary>
    public enum HexPrefix
    {
        /// <summary>
        /// Sin Prefijo.
        /// </summary>
        Nothing,
        /// <summary>
        /// Prefijo "x".
        /// </summary>
        PrefixX,
        /// <summary>
        /// Prefijo "0x".
        /// </summary>
        Prefix0x,
        /// <summary>
        /// Prefijo "u".
        /// </summary>
        PrefixU
    }

    /// <summary>
    /// Funciones estticas relativas a la generacin de cadenas numricas.
    /// </summary>
    public static class NumToStr
    {

        #region BitsToString

        /// <summary>
        /// Rellena puntero con los bits de un byte.
        /// </summary>
        /// <param name="i">Entero (solo es usado el primer byte).</param>
        /// <param name="p">Puntero a rellenar con los 8 bits del primer byte del entero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        public static unsafe void BitsToString(char* p, int i,bool bigEndian)
        {
            if (!bigEndian)
            {
                if ((i & 1) != 0)
                    *p = '1';
                else
                    *p = '0';

                if ((i & 2) != 0)
                    p[1] = '1';
                else
                    p[1] = '0';

                if ((i & 4) != 0)
                    p[2] = '1';
                else
                    p[2] = '0';

                if ((i & 8) != 0)
                    p[3] = '1';
                else
                    p[3] = '0';

                if ((i & 16) != 0)
                    p[4] = '1';
                else
                    p[4] = '0';

                if ((i & 32) != 0)
                    p[5] = '1';
                else
                    p[5] = '0';

                if ((i & 64) != 0)
                    p[6] = '1';
                else
                    p[6] = '0';

                if ((i & 128) != 0)
                    p[7] = '1';
                else
                    p[7] = '0';
            }
            else
            {
                if ((i & 128) != 0)
                    *p = '1';
                else
                    *p = '0';

                if ((i & 64) != 0)
                    p[1] = '1';
                else
                    p[1] = '0';

                if ((i & 32) != 0)
                    p[2] = '1';
                else
                    p[2] = '0';

                if ((i & 16) != 0)
                    p[3] = '1';
                else
                    p[3] = '0';

                if ((i & 8) != 0)
                    p[4] = '1';
                else
                    p[4] = '0';

                if ((i & 4) != 0)
                    p[5] = '1';
                else
                    p[5] = '0';

                if ((i & 2) != 0)
                    p[6] = '1';
                else
                    p[6] = '0';

                if ((i & 1) != 0)
                    p[7] = '1';
                else
                    p[7] = '0';
            }
        }

        /// <summary>
        /// Genera cadena con los bits de un byte.
        /// </summary>
        /// <param name="b">byte.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <returns>Cadena con los bits de un byte.</returns>
        public unsafe static string BitsToString(byte b, bool bigEndian)
        {
            string ret = Mutables.Create(8);
            fixed (char* p = ret)
            {
                BitsToString(p, b, bigEndian);
            }
            return ret;
        }

        /// <summary>
        /// Genera cadena con los bits de un sbyte.
        /// </summary>
        /// <param name="b">sbyte.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <returns>Cadena con los bits de un sbyte.</returns>
        public unsafe static string BitsToString(sbyte b, bool bigEndian)
        {
            return BitsToString((byte)b, bigEndian);
        }

        /// <summary>
        /// Genera cadena con los bits de un puntero de bytes.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Cadena con los bits del puntero.</returns>
        public unsafe static string BitsToString(byte* p, int l, bool bigEndian,bool spaces)
        {
            if (l < 1)
                return Others.Empty;
            int a;
            if (spaces)
            {
                if (l == 1)
                {
                    spaces = false;
                    a = l << 3;
                }
                else
                {
                    a = (l << 3) + l - 1;
                }
            }
            else
                a = l << 3;
            string ret=Mutables.Create(a);
            fixed (char* pret = ret)
            {
                char* pc = pret;
                
                if (!bigEndian)
                {
                    a = 0;
                    do
                    {
                        BitsToString(pc,p[a], false);
                        if (++a == l)
                            break;
                        if (!spaces)
                        {
                            pc += 8;
                        }
                        else
                        {
                            pc[8] = ' ';
                            pc += 9;
                        }
                    } while (true);

                }
                else
                {
                    do
                    {
                        BitsToString(pc, p[--l], true);
                        if (l == 0)
                            break;
                        if (!spaces)
                        {
                            pc += 8;
                        }
                        else
                        {
                            pc[8] = ' ';
                            pc += 9;
                        }
                    } while (true);
                }
            }
            return ret;
        }

        /// <summary>
        /// Genera cadena con los bits de un array de bytes.
        /// </summary>
        /// <param name="arr">Array.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <returns>Cadena con los bits del array.</returns>
        public unsafe static string BitsToString(byte[] arr, bool bigEndian, bool spaces)
        {
            int l = arr.Length;
            if (l == 0)
                return Others.Empty;
            fixed (byte* p = arr)
                return BitsToString(p, l, bigEndian, spaces);
        }

        /// <summary>
        /// Genera cadena con los bits de un nmero.
        /// </summary>
        /// <param name="n">Numero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <returns>Cadena con los bits del nmero.</returns>
        public unsafe static string BitsToString(short n, bool bigEndian, bool spaces)
        {
            byte* p = (byte*)&n;
            return BitsToString(p, 2, bigEndian, spaces);
        }

        /// <summary>
        /// Genera cadena con los bits de un nmero.
        /// </summary>
        /// <param name="n">Numero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <returns>Cadena con los bits del nmero.</returns>
        public unsafe static string BitsToString(ushort n, bool bigEndian, bool spaces)
        {
            byte* p = (byte*)&n;
            return BitsToString(p, 2, bigEndian, spaces);
        }

        /// <summary>
        /// Genera cadena con los bits de un nmero.
        /// </summary>
        /// <param name="n">Numero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <returns>Cadena con los bits del nmero.</returns>
        public unsafe static string BitsToString(int n, bool bigEndian, bool spaces)
        {
            byte* p = (byte*)&n;
            return BitsToString(p, 4, bigEndian, spaces);
        }

        /// <summary>
        /// Genera cadena con los bits de un nmero.
        /// </summary>
        /// <param name="n">Numero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <returns>Cadena con los bits del nmero.</returns>
        public unsafe static string BitsToString(uint n, bool bigEndian, bool spaces)
        {
            byte* p = (byte*)&n;
            return BitsToString(p, 4, bigEndian, spaces);
        }

        /// <summary>
        /// Genera cadena con los bits de un nmero.
        /// </summary>
        /// <param name="n">Numero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <returns>Cadena con los bits del nmero.</returns>
        public unsafe static string BitsToString(float n, bool bigEndian, bool spaces)
        {
            byte* p = (byte*)&n;
            return BitsToString(p, 4, bigEndian, spaces);
        }

        /// <summary>
        /// Genera cadena con los bits de un nmero.
        /// </summary>
        /// <param name="n">Numero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <returns>Cadena con los bits del nmero.</returns>
        public unsafe static string BitsToString(long n, bool bigEndian, bool spaces)
        {
            byte* p = (byte*)&n;
            return BitsToString(p, 8, bigEndian, spaces);
        }

        /// <summary>
        /// Genera cadena con los bits de un nmero.
        /// </summary>
        /// <param name="n">Numero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <returns>Cadena con los bits del nmero.</returns>
        public unsafe static string BitsToString(ulong n, bool bigEndian, bool spaces)
        {
            byte* p = (byte*)&n;
            return BitsToString(p, 8, bigEndian, spaces);
        }

        /// <summary>
        /// Genera cadena con los bits de un nmero.
        /// </summary>
        /// <param name="n">Numero.</param>
        /// <param name="bigEndian">Modo BigEndian: Los bits ms significativos a la izquierda.</param>
        /// <param name="spaces">Si true, cada grupo de 8 bits es separado por un espacio.</param>
        /// <returns>Cadena con los bits del nmero.</returns>
        public unsafe static string BitsToString(double n, bool bigEndian, bool spaces)
        {
            byte* p = (byte*)&n;
            return BitsToString(p, 8, bigEndian, spaces);
        }

        #endregion

        #region HexToString

        /// <summary>
        /// Convierte numero en cadena Hex.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <param name="prefix">Opciones de prefijo.</param>
        /// <param name="trim">No incluye ceros excedentes.</param>
        /// <param name="lower">Letras A-F en minscula (los prefijos siempre son en minscula).</param>
        /// <returns>Cadena Hex.</returns>
        public static string HexToString(byte n, HexPrefix prefix, bool trim, bool lower)
        {
            if (n == 0)
            {
                switch (prefix)
                {
                    case HexPrefix.PrefixX:
                        if (trim)
                            return "x0";
                        return "x00";
                    case HexPrefix.Prefix0x:
                        if (trim)
                            return "0x0";
                        return "0x00";
                    case HexPrefix.PrefixU:
                        if (trim)
                            return "u0";
                        return "u00";
                }
                if (trim)
                    return "0";
                return "00";
            }

            unsafe
            {
                string ret = Mutables.Create(prefix == HexPrefix.Prefix0x ? 4 : 3);
                fixed (char* p = ret)
                {
                    int l;
                    switch (prefix)
                    {
                        case HexPrefix.Prefix0x:
                            *(int*)p = 0x00780030;
                            l = 2;
                            break;
                        case HexPrefix.PrefixX:
                            *p = 'x';
                            l = 1;
                            break;
                        case HexPrefix.PrefixU:
                            *p = 'u';
                            l = 1;
                            break;
                        default:
                            l = 0;
                            break;
                    }
                    bool alto = true;
                    bool pasoNum = false;
                    do
                    {
                        switch (alto ? (n >> 4) : (n & 15))
                        {
                            case 0:
                                if (pasoNum || !trim)
                                    p[l++] = '0';
                                continue;
                            case 1:
                                p[l++] = '1';
                                pasoNum = true;
                                continue;
                            case 2:
                                p[l++] = '2';
                                pasoNum = true;
                                continue;
                            case 3:
                                p[l++] = '3';
                                pasoNum = true;
                                continue;
                            case 4:
                                p[l++] = '4';
                                pasoNum = true;
                                continue;
                            case 5:
                                p[l++] = '5';
                                pasoNum = true;
                                continue;
                            case 6:
                                p[l++] = '6';
                                pasoNum = true;
                                continue;
                            case 7:
                                p[l++] = '7';
                                pasoNum = true;
                                continue;
                            case 8:
                                p[l++] = '8';
                                pasoNum = true;
                                continue;
                            case 9:
                                p[l++] = '9';
                                pasoNum = true;
                                continue;
                            case 10:
                                p[l++] = lower ? 'a' : 'A';
                                pasoNum = true;
                                continue;
                            case 11:
                                p[l++] = lower ? 'b' : 'B';
                                pasoNum = true;
                                continue;
                            case 12:
                                p[l++] = lower ? 'c' : 'C';
                                pasoNum = true;
                                continue;
                            case 13:
                                p[l++] = lower ? 'd' : 'D';
                                pasoNum = true;
                                continue;
                            case 14:
                                p[l++] = lower ? 'e' : 'E';
                                pasoNum = true;
                                continue;
                            case 15:
                                p[l++] = lower ? 'f' : 'F';
                                pasoNum = true;
                                continue;
                        }
                    } while (!(alto = !alto));
                    ((int*)p)[-1] = l;
                    return ret;
                }
            }
        }



        /// <summary>
        /// Convierte numero en cadena Hex.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <param name="prefix">Opciones de prefijo.</param>
        /// <param name="trim">No incluye ceros excedentes.</param>
        /// <param name="lower">Letras A-F en minscula (los prefijos siempre son en minscula).</param>
        /// <returns>Cadena Hex.</returns>
        public static string HexToString(ushort n, HexPrefix prefix, bool trim, bool lower)
        {
            if (n == 0)
            {
                switch (prefix)
                {
                    case HexPrefix.PrefixX:
                        if (trim)
                            return "x0";
                        return "x0000";
                    case HexPrefix.Prefix0x:
                        if (trim)
                            return "0x0";
                        return "0x0000";
                    case HexPrefix.PrefixU:
                        if (trim)
                            return "u0";
                        return "u0000";
                }
                if (trim)
                    return "0";
                return "0000";
            }

            unsafe
            {
                byte* pn = (byte*)&n;
                string ret = Mutables.Create(prefix == HexPrefix.Prefix0x ? 6 : 5);
                fixed (char* p = ret)
                {
                    int l;
                    switch (prefix)
                    {
                        case HexPrefix.Prefix0x:
                            *(int*)p = 0x00780030;
                            l = 2;
                            break;
                        case HexPrefix.PrefixX:
                            *p = 'x';
                            l = 1;
                            break;
                        case HexPrefix.PrefixU:
                            *p = 'u';
                            l = 1;
                            break;
                        default:
                            l = 0;
                            break;
                    }
                    int b = 1;
                    bool alto;
                    bool pasoNum = false;
                    do
                    {
                        alto = true;
                        do
                        {
                            switch (alto ? (pn[b] >> 4) : (pn[b] & 15))
                            {
                                case 0:
                                    if (pasoNum || !trim)
                                        p[l++] = '0';
                                    continue;
                                case 1:
                                    p[l++] = '1';
                                    pasoNum = true;
                                    continue;
                                case 2:
                                    p[l++] = '2';
                                    pasoNum = true;
                                    continue;
                                case 3:
                                    p[l++] = '3';
                                    pasoNum = true;
                                    continue;
                                case 4:
                                    p[l++] = '4';
                                    pasoNum = true;
                                    continue;
                                case 5:
                                    p[l++] = '5';
                                    pasoNum = true;
                                    continue;
                                case 6:
                                    p[l++] = '6';
                                    pasoNum = true;
                                    continue;
                                case 7:
                                    p[l++] = '7';
                                    pasoNum = true;
                                    continue;
                                case 8:
                                    p[l++] = '8';
                                    pasoNum = true;
                                    continue;
                                case 9:
                                    p[l++] = '9';
                                    pasoNum = true;
                                    continue;
                                case 10:
                                    p[l++] = lower ? 'a' : 'A';
                                    pasoNum = true;
                                    continue;
                                case 11:
                                    p[l++] = lower ? 'b' : 'B';
                                    pasoNum = true;
                                    continue;
                                case 12:
                                    p[l++] = lower ? 'c' : 'C';
                                    pasoNum = true;
                                    continue;
                                case 13:
                                    p[l++] = lower ? 'd' : 'D';
                                    pasoNum = true;
                                    continue;
                                case 14:
                                    p[l++] = lower ? 'e' : 'E';
                                    pasoNum = true;
                                    continue;
                                case 15:
                                    p[l++] = lower ? 'f' : 'F';
                                    pasoNum = true;
                                    continue;
                            }
                        } while (!(alto = !alto));
                    } while (--b != -1);
                    ((int*)p)[-1] = l;
                    return ret;
                }
            }
        }

        /// <summary>
        /// Convierte numero en cadena Hex.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <param name="prefix">Opciones de prefijo.</param>
        /// <param name="trim">No incluye ceros excedentes.</param>
        /// <param name="lower">Letras A-F en minscula (los prefijos siempre son en minscula).</param>
        /// <returns>Cadena Hex.</returns>
        public static string HexToString(uint n, HexPrefix prefix, bool trim, bool lower)
        {
            if (n == 0)
            {
                switch (prefix)
                {
                    case HexPrefix.PrefixX:
                        if (trim)
                            return "x0";
                        return "x00000000";
                    case HexPrefix.Prefix0x:
                        if (trim)
                            return "0x0";
                        return "0x00000000";
                    case HexPrefix.PrefixU:
                        if (trim)
                            return "u0";
                        return "u00000000";
                }
                if (trim)
                    return "0";
                return "00000000";
            }

            unsafe
            {
                byte* pn = (byte*)&n;

                string ret = Mutables.Create(prefix == HexPrefix.Prefix0x ? 10 : 9);
                fixed (char* p = ret)
                {
                    int l;
                    switch (prefix)
                    {
                        case HexPrefix.Prefix0x:
                            *(int*)p = 0x00780030;
                            l = 2;
                            break;
                        case HexPrefix.PrefixX:
                            *p = 'x';
                            l = 1;
                            break;
                        case HexPrefix.PrefixU:
                            *p = 'u';
                            l = 1;
                            break;
                        default:
                            l = 0;
                            break;
                    }
                    int b = 3;
                    bool alto;
                    bool pasoNum = false;
                    do
                    {
                        alto = true;
                        do
                        {
                            switch (alto ? (pn[b] >> 4) : (pn[b] & 15))
                            {
                                case 0:
                                    if (pasoNum || !trim)
                                        p[l++] = '0';
                                    continue;
                                case 1:
                                    p[l++] = '1';
                                    pasoNum = true;
                                    continue;
                                case 2:
                                    p[l++] = '2';
                                    pasoNum = true;
                                    continue;
                                case 3:
                                    p[l++] = '3';
                                    pasoNum = true;
                                    continue;
                                case 4:
                                    p[l++] = '4';
                                    pasoNum = true;
                                    continue;
                                case 5:
                                    p[l++] = '5';
                                    pasoNum = true;
                                    continue;
                                case 6:
                                    p[l++] = '6';
                                    pasoNum = true;
                                    continue;
                                case 7:
                                    p[l++] = '7';
                                    pasoNum = true;
                                    continue;
                                case 8:
                                    p[l++] = '8';
                                    pasoNum = true;
                                    continue;
                                case 9:
                                    p[l++] = '9';
                                    pasoNum = true;
                                    continue;
                                case 10:
                                    p[l++] = lower ? 'a' : 'A';
                                    pasoNum = true;
                                    continue;
                                case 11:
                                    p[l++] = lower ? 'b' : 'B';
                                    pasoNum = true;
                                    continue;
                                case 12:
                                    p[l++] = lower ? 'c' : 'C';
                                    pasoNum = true;
                                    continue;
                                case 13:
                                    p[l++] = lower ? 'd' : 'D';
                                    pasoNum = true;
                                    continue;
                                case 14:
                                    p[l++] = lower ? 'e' : 'E';
                                    pasoNum = true;
                                    continue;
                                case 15:
                                    p[l++] = lower ? 'f' : 'F';
                                    pasoNum = true;
                                    continue;
                            }
                        } while (!(alto = !alto));
                    } while (--b != -1);
                    ((int*)p)[-1] = l;
                    return ret;
                }
            }
        }

        /// <summary>
        /// Convierte numero en cadena Hex.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <param name="prefix">Opciones de prefijo.</param>
        /// <param name="trim">No incluye ceros excedentes.</param>
        /// <param name="lower">Letras A-F en minscula (los prefijos siempre son en minscula).</param>
        /// <returns>Cadena Hex.</returns>
        public static string HexToString(ulong n, HexPrefix prefix, bool trim, bool lower)
        {
            if (n == 0)
            {
                switch (prefix)
                {
                    case HexPrefix.PrefixX:
                        if (trim)
                            return "x0";
                        return "x0000000000000000";
                    case HexPrefix.Prefix0x:
                        if (trim)
                            return "0x0";
                        return "0x0000000000000000";
                    case HexPrefix.PrefixU:
                        if (trim)
                            return "u0";
                        return "u0000000000000000";
                }
                if (trim)
                    return "0";
                return "0000000000000000";
            }

            unsafe
            {
                byte* pn = (byte*)&n;

                string ret = Mutables.Create(prefix == HexPrefix.Prefix0x ? 18 : 17);
                fixed (char* p = ret)
                {
                    int l;
                    switch (prefix)
                    {
                        case HexPrefix.Prefix0x:
                            *(int*)p = 0x00780030;
                            l = 2;
                            break;
                        case HexPrefix.PrefixX:
                            *p = 'x';
                            l = 1;
                            break;
                        case HexPrefix.PrefixU:
                            *p = 'u';
                            l = 1;
                            break;
                        default:
                            l = 0;
                            break;
                    }
                    int b = 7;
                    bool alto;
                    bool pasoNum = false;
                    do
                    {
                        alto = true;
                        do
                        {
                            switch (alto ? (pn[b] >> 4) : (pn[b] & 15))
                            {
                                case 0:
                                    if (pasoNum || !trim)
                                        p[l++] = '0';
                                    continue;
                                case 1:
                                    p[l++] = '1';
                                    pasoNum = true;
                                    continue;
                                case 2:
                                    p[l++] = '2';
                                    pasoNum = true;
                                    continue;
                                case 3:
                                    p[l++] = '3';
                                    pasoNum = true;
                                    continue;
                                case 4:
                                    p[l++] = '4';
                                    pasoNum = true;
                                    continue;
                                case 5:
                                    p[l++] = '5';
                                    pasoNum = true;
                                    continue;
                                case 6:
                                    p[l++] = '6';
                                    pasoNum = true;
                                    continue;
                                case 7:
                                    p[l++] = '7';
                                    pasoNum = true;
                                    continue;
                                case 8:
                                    p[l++] = '8';
                                    pasoNum = true;
                                    continue;
                                case 9:
                                    p[l++] = '9';
                                    pasoNum = true;
                                    continue;
                                case 10:
                                    p[l++] = lower ? 'a' : 'A';
                                    pasoNum = true;
                                    continue;
                                case 11:
                                    p[l++] = lower ? 'b' : 'B';
                                    pasoNum = true;
                                    continue;
                                case 12:
                                    p[l++] = lower ? 'c' : 'C';
                                    pasoNum = true;
                                    continue;
                                case 13:
                                    p[l++] = lower ? 'd' : 'D';
                                    pasoNum = true;
                                    continue;
                                case 14:
                                    p[l++] = lower ? 'e' : 'E';
                                    pasoNum = true;
                                    continue;
                                case 15:
                                    p[l++] = lower ? 'f' : 'F';
                                    pasoNum = true;
                                    continue;
                            }
                        } while (!(alto = !alto));
                    } while (--b != -1);
                    ((int*)p)[-1] = l;
                    return ret;
                }
            }
        }

        /// <summary>
        /// Convierte numero en cadena Hex.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <param name="prefix">Opciones de prefijo.</param>
        /// <param name="trim">No incluye ceros excedentes.</param>
        /// <param name="lower">Letras A-F en minscula (los prefijos siempre son en minscula).</param>
        /// <returns>Cadena Hex.</returns>
        public static string HexToString(sbyte n, HexPrefix prefix, bool trim, bool lower)
        {
            unsafe
            {
                return HexToString((byte)n, prefix, trim, lower);
            }
        }

        /// <summary>
        /// Convierte numero en cadena Hex.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <param name="prefix">Opciones de prefijo.</param>
        /// <param name="trim">No incluye ceros excedentes.</param>
        /// <param name="lower">Letras A-F en minscula (los prefijos siempre son en minscula).</param>
        /// <returns>Cadena Hex.</returns>
        public static string HexToString(short n, HexPrefix prefix, bool trim, bool lower)
        {
            unsafe
            {
                return HexToString((ushort)n, prefix, trim, lower);
            }
        }

        /// <summary>
        /// Convierte numero en cadena Hex.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <param name="prefix">Opciones de prefijo.</param>
        /// <param name="trim">No incluye ceros excedentes.</param>
        /// <param name="lower">Letras A-F en minscula (los prefijos siempre son en minscula).</param>
        /// <returns>Cadena Hex.</returns>
        public static string HexToString(int n, HexPrefix prefix, bool trim, bool lower)
        {
            unsafe
            {
                return HexToString((uint)n, prefix, trim, lower);
            }
        }

        /// <summary>
        /// Convierte numero en cadena Hex.
        /// </summary>
        /// <param name="n">Nmero.</param>
        /// <param name="prefix">Opciones de prefijo.</param>
        /// <param name="trim">No incluye ceros excedentes.</param>
        /// <param name="lower">Letras A-F en minscula (los prefijos siempre son en minscula).</param>
        /// <returns>Cadena Hex.</returns>
        public static string HexToString(long n, HexPrefix prefix, bool trim, bool lower)
        {
            unsafe
            {
                return HexToString((ulong)n, prefix, trim, lower);
            }
        }


        #endregion

    }
}
