using System;
using System.Globalization;
using System.Runtime.InteropServices;
using EnPalermo.Lit;
using EnPalermo.Mem;

namespace EnPalermo.Txt
{

    #region Enums

    /// <summary>
    /// Modo de reconocimiento de StrToNum
    /// </summary>
    [Flags()]
    public enum NumericMode
    {
        /// <summary>
        /// Modo de nmero Real. El modo Real puede combinarse con Bool, las opciones de separador decimal y Negative.
        /// </summary>
        Real=0,
        /// <summary>
        /// Modo de nmero Entero. El modo Integer puede combinarse con Bool y Negative. Este modo no acepta separadores
        /// de miles ni punto o coma decimal (si no estan restringidos se interrumpe el reconocimiento al encontrarlos).
        /// </summary>
        Integer=1,
        /// <summary>
        /// Modo de nmero Hex. El modo Hex no acepta otras combinacines.
        /// </summary>
        Hex=2,
        /// <summary>
        /// Solo en modos Real e Integer (sin otras combinaciones, usa modo Real).
        /// Efecta un trim inicial (lo que implica que implicitamente se aceptarn los caracteres trim externamente) y considera a las cadenas "t", "true", "f" y "false", "1" y "0"  como boleanas (ignorando May/Min).
        /// En el reconocimiento booleano no se toman en cuenta las opciones accept.
        /// </summary>
        Bool=4,
        /// <summary>
        /// Solo en modos Real e Integer (sin otras combinaciones, usa modo Real).
        /// Detecta nmeros negativos en base al signo.
        /// </summary>
        Negative=8,
        /// <summary>
        /// Solo en modos Real e Integer (sin otras combinaciones, usa modo Real).
        /// Combina las opciones Bool y Negative.
        /// </summary>
        BoolNegative=Bool+Negative,
        /// <summary>
        /// Combina las opciones Integer y Bool.
        /// </summary>
        IntegerBool=Integer+Bool,
        /// <summary>
        /// Combina las opciones Integer y Negative.
        /// </summary>
        IntegerNegative=Integer+Negative,
        /// <summary>
        /// Combina las opciones Integer, Bool y Negative.
        /// </summary>
        IntegerBoolNegative=Integer+BoolNegative
    }


    /// <summary>
    /// Opciones para el modo ToString.
    /// </summary>
    [Flags()]
    public enum ToStringMode
    {
        /// <summary>
        /// Solo modo Real: Usar punto decimal. Si DecimalDot y DecimalComma no estn especificados (o ambos lo estn), usar Current.
        /// </summary>
        DecimalDot=1,
        /// <summary>
        /// Solo modo Real: Usar coma decimal. Si DecimalDot y DecimalComma no estn especificados (o ambos lo estn), usar Current.
        /// </summary>
        DecimalComma=2,
        /// <summary>
        /// Solo en modo Hex: Usar letras maysculas.
        /// </summary>
        UpperHex=4,
    }

    /// <summary>
    /// Opciones de separador decimal.
    /// </summary>
    public enum NumericDecimal
    {
        /// <summary>
        /// No efectuar deteccin: Se considera la coma como separador decimal.
        /// </summary>
        UseComma,
        /// <summary>
        /// No efectuar deteccin: Se considera al punto como separador decimal.
        /// </summary>
        UseDot,
        /// <summary>
        /// No efectuar deteccin: Se considera la cultura actual (detectada a la primer llamada).
        /// Cualquier separador decimal diferente a comma de la cultura actual, ser considerado punto.
        /// </summary>
        UseCurrent,
        /// <summary>
        /// Se usar deteccin. En caso de encontrarse un separador decimal probable (es nico, precedido de una a 3 cifras y seguido de 3 cifras)
        /// se lo acepta como separador decimal.
        /// </summary>
        IsDecimalOnProbable,
        /// <summary>
        /// Se usar deteccin. En caso de encontrarse un separador decimal probable (es nico, precedido de una a 3 cifras y seguido de 3 cifras)
        /// se lo acepta como separador de miles.
        /// </summary>
        IsThousandsOnProbable,
        /// <summary>
        /// Se usar deteccin. En caso de encontrarse un separador decimal probable (es nico, precedido de una a 3 cifras y seguido de 3 cifras)
        /// se considerar siempre al punto como separador decimal.
        /// </summary>
        DotOnProbable,
        /// <summary>
        /// Se usar deteccin. En caso de encontrarse un separador decimal probable (es nico, precedido de una a 3 cifras y seguido de 3 cifras)
        /// se considerar siempre a la coma como separador decimal.
        /// </summary>
        CommaOnProbable,
        /// <summary>
        /// Se usar deteccin. En caso de encontrarse un separador decimal probable (es nico, precedido de una a 3 cifras y seguido de 3 cifras)
        /// se usar DefaultDot (Cualquier separador decimal de la cultura actual diferente a comma, ser considerado punto).
        /// </summary>
        CurrentOnProbable
    }

 

    /// <summary>
    /// Carcteres no nmericos a tolerar en el reconocimiento.
    /// </summary>
    [Flags()]
    public enum NumericAccept
    {
        /// <summary>
        /// No acepta ningn carcter no numrico. Si estos aparecen se considera al nmero no convertible.
        /// </summary>
        None=0,
        /// <summary>
        /// Acepta carcteres en blanco (incluyendo espacios), de control, separadores de lneas, prrafos y otros fuera de la zona numrica (detiene el reconocimiento ante su aparicin).
        /// </summary>
        ExternalBlanksAndMarks=1,
        /// <summary>
        /// Acepta puntuaciones fuera de la zona de reconocimiento (detiene el reconocimiento ante su aparicin).
        /// Los smbolos Mayor y Menor son considerados tanto smbolos como puntuacin.
        /// </summary>
        ExternalPunctuation=2,
        /// <summary>
        /// Acepta smbolos fuera de la zona de reconocimiento (detiene el reconocimiento ante su aparicin).
        /// Los smbolos Mayor y Menor son considerados tanto smbolos como puntuacin.
        /// </summary>
        ExternalSymbols=4,
        /// <summary>
        /// Acepta letras y otros nmeros fuera de la zona de reconocimiento (detiene el reconocimiento ante su aparicin).
        /// </summary>
        ExternalLettersNumbers=8,
        /// <summary>
        /// Acepta espacios (32 y 160) dentro de la zona de reconocimiento (debe estar dentro de la zona y el carcter siguiente no puede ser espacio).
        /// </summary>
        InternalSpaces=16,
        /// <summary>
        /// Solo en modo Hex o Entero. Acepta '-' y '\xAD' dentro de la zona de reconocimiento (debe estar dentro de la zona y debe estar precedido y seguido de un nmero o combinacin espacio y nmero si estos se aceptan).
        /// </summary>
        InternalDashs=32,
        /// <summary>
        /// Solo en modo Hex o Entero. Acepta '.' y '\xB7' dentro de la zona de reconocimiento (debe estar dentro de la zona y debe estar precedido y seguido de un nmero o combinacin espacio y nmero si estos se aceptan).
        /// </summary>
        InternalDot=64,
        /// <summary>
        /// Solo en modo Hex o Entero. Acepta ',' dentro de la zona de reconocimiento (debe estar dentro de la zona y debe estar precedido y seguido de un nmero o combinacin espacio y nmero si estos se aceptan).
        /// </summary>
        InternalComma=128,
        /// <summary>
        /// Solo en modo Hex o Entero. Acepta ':' y ';' dentro de la zona de reconocimiento  (debe estar dentro de la zona y debe estar precedido y seguido de un nmero o combinacin espacio y nmero si estos se aceptan).
        /// </summary>
        InternalColons=256,
        /// <summary>
        /// Solo en modo Hex o Entero. Acepta '/' y '\\' dentro de la zona de reconocimiento (debe estar dentro de la zona y debe estar precedido y seguido de un nmero o combinacin espacio y nmero si estos se aceptan).
        /// </summary>
        InternalSlashs=512,
        /// <summary>
        /// Solo en modo Hex o Entero. Acepta '|' y '\xA6' dentro de la zona de reconocimiento (debe estar dentro de la zona y debe estar precedido y seguido de un nmero o combinacin espacio y nmero si estos se aceptan).
        /// </summary>
        InternalBars=1024
    }



    #endregion
  
	/// <summary>
	/// Estructura y funciones estticas para la conversin de cadenas a nmeros.
	/// </summary>
	/// <remarks>
	/// Ventajas en el uso de la estructura:
	/// - Flexibilidad.
	/// - Dado que la conversin se valida al consultar propiedades, no genera excepciones (a no ser las de argumento).
	/// - Deteccin inteligente de coma decimal y separador de miles.
	/// - Apta para el reconocimiento de cadenas no estndar.
	/// - En el caso de enteros, soporta todo el rango long para numeros negativos y ulong para positivos. En
	/// el caso de numeros mas grandes o con decimales, usa double.
	/// - Acepta encabezados Hex 0x, 0X, x y X (aunque se debe especificar que la conversin es hex).
	/// Desventajas:
	/// - Velocidad: En enteros StrToPositiveInt es mucho mas rapida. En las otras conversiones la velocidad es similar
	/// o un poco mejor a las funciones Parse.
	/// - No acepta exponentes y los parentesis y smbolos monetarios deben ser reemplazados por el soporte de puntuaciones y smbolos.
	/// </remarks>
    [StructLayout(LayoutKind.Sequential)]
	public struct StrToNum
	{
        /// <summary>
        /// Tipo de nmero
        /// </summary>
        [Flags()]
        private enum NumericType
        {
            /// <summary>
            /// No es un nmero (no convertido).
            /// </summary>
            NoNumber=0,
            /// <summary>
            /// Es cero entero (puede haber doubles que conviertan a cero).
            /// </summary>
            Zero=1,
            /// <summary>
            /// Es positivo diferente de cero o en todo caso double que redondea a cero.
            /// </summary>
            Positive=2,
            /// <summary>
            /// Es negativo (diferente de cero).
            /// </summary>
            Negative=4,
            /// <summary>
            /// Contiene coma flotante o es un nmero grande. Por lo tanto, el nmero subyacente es un Double.
            /// </summary>
            Double=8,
            /// <summary>
            /// En la deteccin boleana se detect true. Siempre se combina con Positive dado que convierte a 1. 
            /// </summary>
            True=16,
            /// <summary>
            /// En la deteccin boleana se detect true. Siempre se combina con Cero dado que convierte a 0 entero. 
            /// </summary>
            False=32,
        }

        #region Campos

        /// <summary>
        /// Nmero subyacente.
        /// Si la Flag Double es true, ser un nmero double, en cambio puede ser long o ulong.
        /// </summary>
        private long number;

        /// <summary>
        /// Muestra el tipo de nmero y especifica como es convertible.
        /// </summary>
        private NumericType numericType;

        /// <summary>
        /// Si true, usar el punto decimal en las opciones NumericDecimal.CurrentOnProbable y NumericDecimal.CommaOnProbable. 
        /// Si false, usar la coma.
        /// Por performance, este campo se setea en la primer llamada.
        /// </summary>
        public static readonly bool DefaultDot;

        /// <summary>
        /// Mxima cantidad de nmeros aceptable.
        /// </summary>
        private const int maxL=320;

        /// <summary>
        /// Mxima cantidad de nmeros aceptable - 1.
        /// </summary>
        private const int maxLi=maxL-1;



        #endregion

        #region Constructores y asistentes

        static StrToNum()
        {
            string sep=System.Globalization.NumberFormatInfo.CurrentInfo.NumberDecimalSeparator;
            DefaultDot=sep==null||sep.Length!=1||sep[0]!=',';
        }


        private static unsafe NumericType Init(byte*p, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, out long number, char* str, ToStringMode strMode)
        {

            if (l<1)
                goto fallo;
            int modi;
            switch(mode&(NumericMode.Integer|NumericMode.Hex))
            {
                case NumericMode.Real:
                    if (accept>=NumericAccept.InternalDashs)
                        goto default;
                    modi=0;
                    detBool:
                        if ((mode&NumericMode.Bool)!=0)
                        {
                    
                            PByte pc =new PByte(p,l,true,true,true);
                            p=pc.Pointer;
                            l=pc.Length;
                            switch(l)
                            {
                                case 0:
                                    goto fallo;
                                case 1:
                        
                                    if (*p=='T' || *p=='t')
                                        goto boolTrue;
                                    if (*p=='F' || *p=='f')
                                        goto boolFalse;
                                    if (*p<='9' && *p>='0')
                                    {
                                        if (*p=='0')
                                            goto boolFalse;
                                        if (*p=='1')
                                            goto boolTrue;
                                        if (str!=null)
                                            *(int*)str=(int)*p;
                                        number=(long)(((int)*p)-48);
                                        return NumericType.Positive;
                                    }
                                    goto fallo;
                                case 4:
                                    if ((*p=='T' || *p=='t') && (p[1]=='R' || p[1]=='r') && (p[2]=='U' || p[2]=='u') && (p[3]=='E' || p[3]=='e'))
                                        goto boolTrue;
                                    break;
                                case 5:
                                    if ((*p=='F' || *p=='f') && (p[1]=='A' || p[1]=='a') && (p[2]=='L' || p[2]=='l') && (p[3]=='S' || p[3]=='s') && (p[4]=='E' || p[4]=='e'))
                                        goto boolFalse;
                                    break;
                            }

                        }
                    break;
                case NumericMode.Integer:
                    modi=1;
                    goto detBool;
                case NumericMode.Hex:
                    if ((mode&(NumericMode.Bool|NumericMode.Negative))!=0)
                        goto default;
                    modi=2;
                    break;
                default:
                    throw new ArgumentException(Errors.INVALID_ENUM,"mode");
            }

            byte*pb=stackalloc byte[l>=maxL?maxL:l];
            int a=0,b=-1, finalComma=-1, dot=-1,isSep=-1;
            
            if (modi==0)
            {
                switch(floatPoint)
                {
                    case NumericDecimal.UseComma:
                        dot=0;
                        break;
                    case NumericDecimal.UseDot:
                        dot=1;
                        break;
                    case NumericDecimal.UseCurrent:
                        dot=(DefaultDot)?1:0;
                        break;
                }
            }
            
            bool neg=false, lastDot=false;
            char c;


            /*
             * Datos Bytes:
             * nmeros: 0 a 15
             * Espacio: 32 (no reemplaza a sep).
             * Separador: 33.
             * */
            

            do
            {
                c=(char)p[a];
                switch(c)
                {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        if (b!=-1)
                        {
                            if (pb[b]<32 && b++==maxLi)
                                goto fallo;
                        }
                        else
                            b=0;
                        pb[b]=(byte)(((int)c)-48);
                        continue;

                    case 'a':
                    case 'b':
                    case 'c':
                    case 'd':
                    case 'e':
                    case 'f':
                        if (modi!=2)
                        {
                            if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                goto external;
                            goto fallo;
                        }
                        if (b!=-1)
                        {
                            if (pb[b]<32 && b++==maxLi)
                                goto fallo;
                        }
                        else
                            b=0;
                        pb[b]=(byte)(((int)c)-87);
                        continue;

                    case 'A':
                    case 'B':
                    case 'C':
                    case 'D':
                    case 'E':
                    case 'F':
                        if (modi!=2)
                        {
                            if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                goto external;
                            goto fallo;
                        }
                        if (b!=-1)
                        {
                            if (pb[b]<32 && b++==maxLi)
                                goto fallo;
                        }
                        else
                            b=0;
                        pb[b]=(byte)(((int)c)-55);
                        continue;

                    case 'x':
                    case 'X':
                        if (modi!=2 || b>0)
                        {
                            if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                goto external;
                            goto fallo;
                        }

                        if (b==-1)
                        {
                            if (a+1==l)
                                goto fallo;
                            if (!Validate.OnlyHex((char)p[a+1]))
                                goto default;
                        }
                        else if (*pb!=0)
                            goto default;
                        b=-1;
                        continue;
                    
                    case ' ':
                    case '\xA0':
                        if (b==-1 || (accept&NumericAccept.InternalSpaces)==0)
                            goto case '\f';
                        if (pb[b]<32)
                        {
                            if (b==maxLi)
                                goto fallo;
                            pb[++b]=(byte)32;
                        }
                        continue;

                    case '\t':
                    case '\n':
                    case '\r':
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                        if ((accept&NumericAccept.ExternalBlanksAndMarks)!=0)
                            goto external;
                        goto fallo;

                    case '!':
                    case '"':
                    case '#':
                    case '%':
                    case '&':
                    case '\'':
                    case '(':
                    case ')':
                    case '?':
                    case '*':
                    case '@':
                    case '[':
                    case ']':
                    case '_':
                    case '{':
                    case '}':
                        if ((accept&NumericAccept.ExternalPunctuation)!=0)
                            goto external;
                        goto fallo;
                   
                    
                    case '$':
                    case '+':
                    case '=':
                    case '^':
                    case '`':
                    case '~':
                        if ((accept&NumericAccept.ExternalSymbols)!=0)
                            goto external;
                        goto fallo;

                    case ',':
                        if (modi!=0)
                        {
                            if (b==-1 || (accept&NumericAccept.InternalComma)==0)
                                goto case '}';
                            goto proSep;
                        }
                        
                        if (finalComma!=-1)
                        {
                            if(pb[b]==32)
                                b--;
                            if ((((b+1)-finalComma)%3)!=0)
                                goto fallo;
                            switch(isSep)
                            {
                                case 0:
                                    goto fallo;
                                case 1:
                                    if (lastDot)
                                        goto default;
                                    break;
                                default:
                                switch(dot)
                                {
                                    case 0:
                                        if (!lastDot)
                                            goto fallo;
                                        isSep=0;
                                        finalComma=b+1;
                                        break;

                                    case 1:
                                        if (lastDot)
                                            goto fallo;
                                        isSep=1;
                                        break;

                                    default:
                                        if (lastDot)
                                        {
                                            dot=0;
                                            isSep=0;
                                            finalComma=b+1;
                                        }
                                        else
                                        {
                                            dot=1;
                                            isSep=1;
                                        }
                                        break;
                                }
                                    break;
                            }
                        }
                        else
                        {
                            if(b!=-1 && pb[b]==32)
                                b--;
                            switch(dot)
                            {
                                case 0:
                                    isSep=0;
                                    break;
                                case 1:
                                    if (b==-1)
                                        goto fallo;
                                    isSep=1;
                                    break;
                                default:
                                    if (b==-1)
                                    {
                                        isSep=0;
                                        dot=0;
                                    }
                                    break;
                            }
                            lastDot=false;
                            finalComma=b+1;
                        }
                        continue;

                    case '.':
                        if (modi!=0)
                        {
                            if (b==-1 || (accept&NumericAccept.InternalDot)==0)
                                goto case '}';
                            goto proSep;
                        }
                        if (finalComma!=-1)
                        {
                            if(pb[b]==32)
                                b--;
                            if ((((b+1)-finalComma)%3)!=0)
                                goto fallo;
                            switch(isSep)
                            {
                                case 0:
                                    goto fallo;
                                case 1:
                                    if (!lastDot)
                                        goto default;
                                    break;
                                default:
                                switch(dot)
                                {
                                    case 0:
                                        if (!lastDot)
                                            goto fallo;
                                        isSep=1;
                                        break;

                                    case 1:
                                        if (lastDot)
                                            goto fallo;
                                        isSep=0;
                                        finalComma=b+1;
                                        break;

                                    default:
                                        if (lastDot)
                                        {
                                            dot=0;
                                            isSep=1;
                                        }
                                        else
                                        {
                                            dot=1;
                                            isSep=0;
                                            finalComma=b+1;
                                        }
                                        break;
                                }
                                    break;
                            }
                        }
                        else
                        {
                            if(b!=-1 && pb[b]==32)
                                b--;

                            switch(dot)
                            {
                                case 0:
                                    if (b==-1)
                                        goto fallo;
                                    isSep=1;
                                    break;
                                case 1:
                                    isSep=0;
                                    break;
                                default:
                                    if (b==-1)
                                    {
                                        isSep=0;
                                        dot=1;
                                    }
                                    break;
                            }
                            lastDot=true;
                            finalComma=b+1;
                        }
                        continue;

                    case '\xB7':
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalDot)==0)
                            goto case '}';
                        goto proSep;

                    case '-':
                        if ((mode&NumericMode.Negative)==0)
                            goto case '\xAD'; 
                        if (b!=-1)
                            goto case '}';
                        if ((accept&NumericAccept.InternalSpaces)!=0)
                        {
                            int x=a;
                            while(++x!=l)
                            {
                                if (p[x]!=' ' && p[x]!='\xA0')
                                {
                                    if (p[x]>='0' && p[x]<='9')
                                    {
                                        neg=true;
                                        b=0;
                                        *pb=(byte)(((int)p[x])-48);
                                        a=x;
                                    }
                                    break;
                                }
                            }
                            if(neg)
                                continue;
                        }
                        else if (a+1!=l && Validate.OnlyDigits((char)p[a+1]))
                        {
                            neg=true;
                            continue;
                        }
                        goto case '}';

                    case '\xAD':
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalDashs)==0)
                            goto case '}';
                        goto proSep;

                    case ':':
                    case ';':
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalColons)==0)
                            goto case '}';
                        goto proSep;

                    case '/':
                    case '\\':
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalSlashs)==0)
                            goto case '}';
                        proSep:
                            if (pb[b]>31)
                            {
                                if (pb[b]!=32)
                                    goto fallo;
                            }
                            else if (b++==maxLi)
                                goto fallo;
                        pb[b]=(byte)33;
                        continue;


                    case '|':
                    case '\xA6':                    
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalBars)==0)
                            goto case '~';
                        if (pb[b]>31)
                        {
                            if (pb[b]!=32)
                                goto fallo;
                        }
                        else if (b++==maxLi)
                            goto fallo;
                        pb[b]=(byte)34;
                        continue;

                    case '<':
                    case '>':
                        if ((accept&NumericAccept.ExternalSymbols)!=0 || (accept&NumericAccept.ExternalPunctuation)!=0)
                            goto external;
                        goto fallo;

                    default:
                        if (c<'\xA0')
                        {
                            if (c>'\x7E')
                            {
                                if ((accept&NumericAccept.ExternalBlanksAndMarks)!=0)
                                    goto external;
                            }
                            else if (c>'\x40')
                            {
                                if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                    goto external;
                            }
                            else if ((accept&NumericAccept.ExternalBlanksAndMarks)!=0)
                                goto external;
                            goto fallo;
                        }
                    switch(Texts.GetUnicodeCategory(c))
                    {
                        case UnicodeCategory.ClosePunctuation:
                        case UnicodeCategory.ConnectorPunctuation:
                        case UnicodeCategory.DashPunctuation:
                        case UnicodeCategory.FinalQuotePunctuation:
                        case UnicodeCategory.InitialQuotePunctuation:
                        case UnicodeCategory.OpenPunctuation:
                        case UnicodeCategory.OtherPunctuation:
                            if ((accept&NumericAccept.ExternalPunctuation)!=0)
                                goto external;
                            goto fallo;

                        case UnicodeCategory.CurrencySymbol:
                        case UnicodeCategory.MathSymbol:
                        case UnicodeCategory.ModifierSymbol:
                        case UnicodeCategory.OtherSymbol:
                            if ((accept&NumericAccept.ExternalSymbols)!=0)
                                goto external;
                            goto fallo;

                        case UnicodeCategory.DecimalDigitNumber:
                        case UnicodeCategory.LetterNumber:
                        case UnicodeCategory.LowercaseLetter:
                        case UnicodeCategory.ModifierLetter:
                        case UnicodeCategory.OtherLetter:
                        case UnicodeCategory.OtherNumber:
                        case UnicodeCategory.TitlecaseLetter:
                        case UnicodeCategory.UppercaseLetter:
                            if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                goto external;
                            goto fallo;


                        default:
                            if ((accept&NumericAccept.ExternalBlanksAndMarks)!=0)
                                goto external;
                            goto fallo;

                    }

                        external:
                            if (b==-1)
                                continue;
                        break;
                        
                }
                break;
            }while(++a!=l);


            if (b==-1)
                goto fallo;
            if (pb[b]>31)
            {
                switch(pb[b])
                {
                    case 32:
                        if ((accept&NumericAccept.ExternalBlanksAndMarks)==0)
                            goto fallo;
                        break;
                    case 33:
                        if ((accept&NumericAccept.ExternalPunctuation)==0)
                            goto fallo;
                        break;
                    default:
                        if ((accept&NumericAccept.ExternalSymbols)==0)
                            goto fallo;
                        break;
                }
                if (b==0)
                    goto fallo;
                b--;
            }
            switch(modi)
            {
                case 0:
                    if (finalComma!=-1)
                    {
                        if (isSep==-1)
                        {
                            if ((((b+1)-finalComma)%3)!=0)
                                isSep=0;
                            else
                            {
                                switch(floatPoint)
                                {
                                    case NumericDecimal.IsDecimalOnProbable:
                                        isSep=0;
                                        break;
                                    case NumericDecimal.CommaOnProbable:
                                        if (!lastDot)
                                            isSep=0;
                                        break;
                                       
                                    case NumericDecimal.CurrentOnProbable:
                                        if (lastDot==DefaultDot)
                                            isSep=0;
                                        break;
                                    case NumericDecimal.DotOnProbable:
                                        if (lastDot)
                                            isSep=0;
                                        break;
                                }
                            }
                        }
                        else if (isSep==1)
                        {
                            if ((((b+1)-finalComma)%3)!=0)
                                goto fallo;
                        }
                        if (isSep!=0 || finalComma>b)
                            goto case 1;
                        while(pb[isSep]==0 && ++isSep!=finalComma);
                        if (isSep!=0)
                        {
                            b-=isSep;
                            finalComma-=isSep;
                        }
                        return InitDouble(pb+isSep,b+1,out number,finalComma,neg,str,strMode);
                    }
                    goto case 1;
                case 1:
                    isSep=0;
                    b++;
                    while(pb[isSep]==0 && ++isSep!=b);
                    if (isSep!=0)
                    {
                        if (b==isSep)
                            goto Cero;
                        b-=isSep;
                    }
                    if (b>19 || (neg && b==19))
                        return InitDouble(pb+isSep,b,out number,maxL+1,neg,str,strMode);
                    return InitLong(pb+isSep,b,out number,neg,str);
 
                default:
                    isSep=0;
                    b++;
                    while(pb[isSep]==0 && ++isSep!=b);
                    if (isSep!=0)
                    {
                        if (b==isSep)
                            goto Cero;
                        b-=isSep;
                    }
                    if (b>16)
                        return InitDoubleHex(pb+isSep,b,out number,str,strMode);
                    return InitLongHex(pb+isSep,b,out number,str,strMode);
            }
            fallo:
                number=0L;
            return NumericType.NoNumber;
            boolTrue:
                if (str!=null)
                    *(int*)str=49;
            number=1L;
            return NumericType.True|NumericType.Positive;
            boolFalse:
                if (str!=null)
                    *(int*)str=48;
            number=0L;
            return NumericType.False|NumericType.Zero;
            Cero:
                if (str!=null)
                    *(int*)str=48;
            number=0L;
            return NumericType.Zero;
        }
        
 
        private static unsafe NumericType Init(char*p, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, out long number, char* str, ToStringMode strMode)
        {

            if (l<1)
                goto fallo;
            int modi;
            switch(mode&(NumericMode.Integer|NumericMode.Hex))
            {
                case NumericMode.Real:
                    if (accept>=NumericAccept.InternalDashs)
                        goto default;
                    modi=0;
                    detBool:
                    if ((mode&NumericMode.Bool)!=0)
                    {
                    
                        PChar pc =new PChar(p,l,true,true);
                        p=pc.Pointer;
                        l=pc.Length;
                        switch(l)
                        {
                            case 0:
                                goto fallo;
                            case 1:
                        
                                if (*p=='T' || *p=='t')
                                    goto boolTrue;
                                if (*p=='F' || *p=='f')
                                    goto boolFalse;
                                if (*p<='9' && *p>='0')
                                {
                                    if (*p=='0')
                                        goto boolFalse;
                                    if (*p=='1')
                                        goto boolTrue;
                                    if (str!=null)
                                        *(int*)str=(int)*p;
                                    number=(long)(((int)*p)-48);
                                    return NumericType.Positive;
                                }
                                goto fallo;
                            case 4:
                                if ((*p=='T' || *p=='t') && (p[1]=='R' || p[1]=='r') && (p[2]=='U' || p[2]=='u') && (p[3]=='E' || p[3]=='e'))
                                    goto boolTrue;
                                break;
                            case 5:
                                if ((*p=='F' || *p=='f') && (p[1]=='A' || p[1]=='a') && (p[2]=='L' || p[2]=='l') && (p[3]=='S' || p[3]=='s') && (p[4]=='E' || p[4]=='e'))
                                    goto boolFalse;
                                break;
                        }

                    }
                    break;
                case NumericMode.Integer:
                    modi=1;
                    goto detBool;
                case NumericMode.Hex:
                    if ((mode&(NumericMode.Bool|NumericMode.Negative))!=0)
                        goto default;
                    modi=2;
                    break;
                default:
                    throw new ArgumentException(Errors.INVALID_ENUM,"mode");
            }

            byte*pb=stackalloc byte[l>=maxL?maxL:l];
            int a=0,b=-1, finalComma=-1, dot=-1,isSep=-1;
            
            if (modi==0)
            {
                switch(floatPoint)
                {
                    case NumericDecimal.UseComma:
                        dot=0;
                        break;
                    case NumericDecimal.UseDot:
                        dot=1;
                        break;
                    case NumericDecimal.UseCurrent:
                        dot=(DefaultDot)?1:0;
                        break;
                }
            }
            
            bool neg=false, lastDot=false;
            char c;


            /*
             * Datos Bytes:
             * nmeros: 0 a 15
             * Espacio: 32 (no reemplaza a sep).
             * Separador: 33.
             * */
            

            do
            {
                c=p[a];
                switch(c)
                {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        if (b!=-1)
                        {
                            if (pb[b]<32 && b++==maxLi)
                                goto fallo;
                        }
                        else
                            b=0;
                        pb[b]=(byte)(((int)c)-48);
                        continue;

                    case 'a':
                    case 'b':
                    case 'c':
                    case 'd':
                    case 'e':
                    case 'f':
                        if (modi!=2)
                        {
                            if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                goto external;
                            goto fallo;
                        }
                        if (b!=-1)
                        {
                            if (pb[b]<32 && b++==maxLi)
                                goto fallo;
                        }
                        else
                            b=0;
                        pb[b]=(byte)(((int)c)-87);
                        continue;

                    case 'A':
                    case 'B':
                    case 'C':
                    case 'D':
                    case 'E':
                    case 'F':
                        if (modi!=2)
                        {
                            if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                goto external;
                            goto fallo;
                        }
                        if (b!=-1)
                        {
                            if (pb[b]<32 && b++==maxLi)
                                goto fallo;
                        }
                        else
                            b=0;
                        pb[b]=(byte)(((int)c)-55);
                        continue;

                    case 'x':
                    case 'X':
                        if (modi!=2 || b>0)
                        {
                            if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                goto external;
                            goto fallo;
                        }

                        if (b==-1)
                        {
                            if (a+1==l)
                                goto fallo;
                            if (!Validate.OnlyHex(p[a+1]))
                                goto default;
                        }
                        else if (*pb!=0)
                            goto default;
                        b=-1;
                        continue;
                    
                    case ' ':
                    case '\xA0':
                        if (b==-1 || (accept&NumericAccept.InternalSpaces)==0)
                                    goto case '\ufeff';
                        if (pb[b]<32)
                        {
                            if (b==maxLi)
                                goto fallo;
                            pb[++b]=(byte)32;
                        }
                        continue;

                    case '\t':
                    case '\n':
                    case '\r':
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if ((accept&NumericAccept.ExternalBlanksAndMarks)!=0)
                            goto external;
                        goto fallo;

                    case '!':
                    case '"':
                    case '#':
                    case '%':
                    case '&':
                    case '\'':
                    case '(':
                    case ')':
                    case '?':
                    case '*':
                    case '@':
                    case '[':
                    case ']':
                    case '_':
                    case '{':
                    case '}':
                        if ((accept&NumericAccept.ExternalPunctuation)!=0)
                            goto external;
                        goto fallo;
                   
                    
                    case '$':
                    case '+':
                    case '=':
                    case '^':
                    case '`':
                    case '~':
                        if ((accept&NumericAccept.ExternalSymbols)!=0)
                            goto external;
                        goto fallo;

                    case ',':
                        if (modi!=0)
                        {
                            if (b==-1 || (accept&NumericAccept.InternalComma)==0)
                                goto case '}';
                            goto proSep;
                        }
                        
                        if (finalComma!=-1)
                        {
                            if(pb[b]==32)
                                b--;
                            if ((((b+1)-finalComma)%3)!=0)
                                goto fallo;
                            switch(isSep)
                            {
                                case 0:
                                    goto fallo;
                                case 1:
                                    if (lastDot)
                                        goto default;
                                    break;
                                default:
                                switch(dot)
                                {
                                    case 0:
                                        if (!lastDot)
                                            goto fallo;
                                        isSep=0;
                                        finalComma=b+1;
                                        break;

                                    case 1:
                                        if (lastDot)
                                            goto fallo;
                                        isSep=1;
                                        break;

                                    default:
                                        if (lastDot)
                                        {
                                            dot=0;
                                            isSep=0;
                                            finalComma=b+1;
                                        }
                                        else
                                        {
                                            dot=1;
                                            isSep=1;
                                        }
                                        break;
                                }
                                    break;
                            }
                        }
                        else
                        {
                            if(b!=-1 && pb[b]==32)
                                b--;
                            switch(dot)
                            {
                                case 0:
                                    isSep=0;
                                    break;
                                case 1:
                                    if (b==-1)
                                        goto fallo;
                                    isSep=1;
                                    break;
                                default:
                                    if (b==-1)
                                    {
                                        isSep=0;
                                        dot=0;
                                    }
                                    break;
                            }
                            lastDot=false;
                            finalComma=b+1;
                        }
                        continue;

                    case '.':
                        if (modi!=0)
                        {
                            if (b==-1 || (accept&NumericAccept.InternalDot)==0)
                                goto case '}';
                            goto proSep;
                        }
                        if (finalComma!=-1)
                        {
                            if(pb[b]==32)
                                b--;
                            if ((((b+1)-finalComma)%3)!=0)
                                goto fallo;
                            switch(isSep)
                            {
                                case 0:
                                    goto fallo;
                                case 1:
                                    if (!lastDot)
                                        goto default;
                                    break;
                                default:
                                switch(dot)
                                {
                                    case 0:
                                        if (!lastDot)
                                            goto fallo;
                                        isSep=1;
                                        break;

                                    case 1:
                                        if (lastDot)
                                            goto fallo;
                                        isSep=0;
                                        finalComma=b+1;
                                        break;

                                    default:
                                        if (lastDot)
                                        {
                                            dot=0;
                                            isSep=1;
                                        }
                                        else
                                        {
                                            dot=1;
                                            isSep=0;
                                            finalComma=b+1;
                                        }
                                        break;
                                }
                                    break;
                            }
                        }
                        else
                        {
                            if(b!=-1 && pb[b]==32)
                                b--;

                            switch(dot)
                            {
                                case 0:
                                    if (b==-1)
                                        goto fallo;
                                    isSep=1;
                                    break;
                                case 1:
                                    isSep=0;
                                    break;
                                default:
                                    if (b==-1)
                                    {
                                        isSep=0;
                                        dot=1;
                                    }
                                    break;
                            }
                            lastDot=true;
                            finalComma=b+1;
                        }
                        continue;

                    case '\xB7':
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalDot)==0)
                            goto case '}';
                        goto proSep;

                    case '-':
                        if ((mode&NumericMode.Negative)==0)
                            goto case '\xAD'; 
                        if (b!=-1)
                            goto case '}';
                        if ((accept&NumericAccept.InternalSpaces)!=0)
                        {
                            int x=a;
                            while(++x!=l)
                            {
                                if (p[x]!=' ' && p[x]!='\xA0')
                                {
                                    if (p[x]>='0' && p[x]<='9')
                                    {
                                        neg=true;
                                        b=0;
                                        *pb=(byte)(((int)p[x])-48);
                                        a=x;
                                    }
                                    break;
                                }
                            }
                            if(neg)
                                continue;
                        }
                        else if (a+1!=l && Validate.OnlyDigits(p[a+1]))
                        {
                            neg=true;
                            continue;
                        }
                        goto case '}';

                    case '\xAD':
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalDashs)==0)
                            goto case '}';
                        goto proSep;

                    case ':':
                    case ';':
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalColons)==0)
                            goto case '}';
                        goto proSep;

                    case '/':
                    case '\\':
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalSlashs)==0)
                            goto case '}';
                        proSep:
                            if (pb[b]>31)
                            {
                                if (pb[b]!=32)
                                    goto fallo;
                            }
                            else if (b++==maxLi)
                                goto fallo;
                        pb[b]=(byte)33;
                        continue;


                    case '|':
                    case '\xA6':                    
                        if (modi==0 || b==-1 || (accept&NumericAccept.InternalBars)==0)
                            goto case '~';
                            if (pb[b]>31)
                            {
                                if (pb[b]!=32)
                                    goto fallo;
                            }
                            else if (b++==maxLi)
                                goto fallo;
                        pb[b]=(byte)34;
                        continue;

                    case '<':
                    case '>':
                        if ((accept&NumericAccept.ExternalSymbols)!=0 || (accept&NumericAccept.ExternalPunctuation)!=0)
                            goto external;
                        goto fallo;

                    default:
                        if (c<'\xA0')
                        {
                            if (c>'\x7E')
                            {
                                if ((accept&NumericAccept.ExternalBlanksAndMarks)!=0)
                                    goto external;
                            }
                            else if (c>'\x40')
                            {
                                if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                    goto external;
                            }
                            else if ((accept&NumericAccept.ExternalBlanksAndMarks)!=0)
                                goto external;
                            goto fallo;
                        }
                    switch(Texts.GetUnicodeCategory(c))
                    {
                        case UnicodeCategory.ClosePunctuation:
                        case UnicodeCategory.ConnectorPunctuation:
                        case UnicodeCategory.DashPunctuation:
                        case UnicodeCategory.FinalQuotePunctuation:
                        case UnicodeCategory.InitialQuotePunctuation:
                        case UnicodeCategory.OpenPunctuation:
                        case UnicodeCategory.OtherPunctuation:
                            if ((accept&NumericAccept.ExternalPunctuation)!=0)
                                goto external;
                            goto fallo;

                        case UnicodeCategory.CurrencySymbol:
                        case UnicodeCategory.MathSymbol:
                        case UnicodeCategory.ModifierSymbol:
                        case UnicodeCategory.OtherSymbol:
                            if ((accept&NumericAccept.ExternalSymbols)!=0)
                                goto external;
                            goto fallo;

                        case UnicodeCategory.DecimalDigitNumber:
                        case UnicodeCategory.LetterNumber:
                        case UnicodeCategory.LowercaseLetter:
                        case UnicodeCategory.ModifierLetter:
                        case UnicodeCategory.OtherLetter:
                        case UnicodeCategory.OtherNumber:
                        case UnicodeCategory.TitlecaseLetter:
                        case UnicodeCategory.UppercaseLetter:
                            if ((accept&NumericAccept.ExternalLettersNumbers)!=0)
                                goto external;
                            goto fallo;


                        default:
                            if ((accept&NumericAccept.ExternalBlanksAndMarks)!=0)
                                goto external;
                            goto fallo;

                    }

                        external:
                            if (b==-1)
                                continue;
                        break;
                        
                }
                break;
            }while(++a!=l);


            if (b==-1)
                goto fallo;
            if (pb[b]>31)
            {
                switch(pb[b])
                {
                    case 32:
                        if ((accept&NumericAccept.ExternalBlanksAndMarks)==0)
                            goto fallo;
                        break;
                    case 33:
                        if ((accept&NumericAccept.ExternalPunctuation)==0)
                            goto fallo;
                        break;
                    default:
                        if ((accept&NumericAccept.ExternalSymbols)==0)
                            goto fallo;
                        break;
                }
                if (b==0)
                    goto fallo;
                b--;
            }
            switch(modi)
            {
                case 0:
                    if (finalComma!=-1)
                    {
                        if (isSep==-1)
                        {
                            if ((((b+1)-finalComma)%3)!=0)
                                isSep=0;
                            else
                            {
                                switch(floatPoint)
                                {
                                    case NumericDecimal.IsDecimalOnProbable:
                                        isSep=0;
                                        break;
                                    case NumericDecimal.CommaOnProbable:
                                        if (!lastDot)
                                            isSep=0;
                                        break;
                                       
                                    case NumericDecimal.CurrentOnProbable:
                                        if (lastDot==DefaultDot)
                                            isSep=0;
                                        break;
                                    case NumericDecimal.DotOnProbable:
                                        if (lastDot)
                                            isSep=0;
                                        break;
                                }
                            }
                        }
                        else if (isSep==1)
                        {
                            if ((((b+1)-finalComma)%3)!=0)
                                goto fallo;
                        }
                        if (isSep!=0 || finalComma>b)
                            goto case 1;
                        while(pb[isSep]==0 && ++isSep!=finalComma);
                        if (isSep!=0)
                        {
                            b-=isSep;
                            finalComma-=isSep;
                        }
                        return InitDouble(pb+isSep,b+1,out number,finalComma,neg,str,strMode);
                    }
                    goto case 1;
                case 1:
                    isSep=0;
                    b++;
                    while(pb[isSep]==0 && ++isSep!=b);
                    if (isSep!=0)
                    {
                        if (b==isSep)
                            goto Cero;
                        b-=isSep;
                    }
                    if (b>19 || (neg && b==19))
                        return InitDouble(pb+isSep,b,out number,maxL+1,neg,str,strMode);
                    return InitLong(pb+isSep,b,out number,neg,str);
 
                default:
                    isSep=0;
                    b++;
                    while(pb[isSep]==0 && ++isSep!=b);
                    if (isSep!=0)
                    {
                        if (b==isSep)
                            goto Cero;
                        b-=isSep;
                    }
                    if (b>16)
                        return InitDoubleHex(pb+isSep,b,out number,str,strMode);
                    return InitLongHex(pb+isSep,b,out number,str,strMode);
            }
            fallo:
                number=0L;
            return NumericType.NoNumber;
            boolTrue:
                if (str!=null)
                    *(int*)str=49;
            number=1L;
            return NumericType.True|NumericType.Positive;
            boolFalse:
                if (str!=null)
                    *(int*)str=48;
            number=0L;
            return NumericType.False|NumericType.Zero;
            Cero:
                if (str!=null)
                    *(int*)str=48;
            number=0L;
            return NumericType.Zero;
        }
        
        private static unsafe NumericType InitDouble(byte*p, int l, out long number, int pos, bool neg, char* str, ToStringMode strMode)
        {
            while(pos<l && p[l-1]==0)
                l--;
            int a;
            byte*psub;
            if (pos<l)
            {
                if (pos==0)
                {
                    psub=p;
                    pos=l;
                    l=0;
                }
                else
                {
                    psub=p+pos;
                    a=l-pos;
                    l=pos;
                    pos=a;
                }
            }
            else if (l==0)
            {
                number=0L;
                if (str!=null)
                    *(int*)str=48;
                return NumericType.Zero;
            }
            else
            {
                if (l<21)
                {
                    if (!neg)
                    {
                        if (l<20)
                            goto tolong;
                        if (*p==1)
                        {
                            if (p[1]<8)
                                goto tolong;

                            // 18446744073709551615
                            if (p[1]==8)
                            {
                                if (p[2]<4)
                                    goto tolong;

                                // 18446744073709551615
                                if (p[2]==4)
                                {
                                    if (p[3]<4)
                                        goto tolong;

                                    // 18446744073709551615
                                    if (p[3]==4)
                                    {
                                        if (p[4]<6)
                                            goto tolong;

                                        // 18446744073709551615
                                        if (p[4]==6)
                                        {
                                            if (p[5]<7)
                                                goto tolong;

                                            // 18446744073709551615
                                            if (p[5]==7)
                                            {
                                                if (p[6]<4)
                                                    goto tolong;

                                                // 18446744073709551615
                                                if (p[6]==4)
                                                {
                                                    if (p[7]<4)
                                                        goto tolong;

                                                    // 18446744073709551615
                                                    if (p[7]==4 && p[8]==0)
                                                    {
                                                        if (p[9]<7)
                                                            goto tolong;

                                                        // 18446744073709551615
                                                        if (p[9]==7)
                                                        {
                                                            if (p[10]<3)
                                                                goto tolong;

                                                            // 18446744073709551615
                                                            if (p[10]==3)
                                                            {
                                                                if (p[11]<7)
                                                                    goto tolong;

                                                                // 18446744073709551615
                                                                if (p[11]==7 && p[12]==0)
                                                                {
                                                                    if (p[13]!=9 || p[14]<5)
                                                                        goto tolong;

                                                                    // 18446744073709551615
                                                                    if (p[14]==5)
                                                                    {
                                                                        if (p[15]<5)
                                                                            goto tolong;

                                                                        // 18446744073709551615
                                                                        if (p[15]==5)
                                                                        {
                                                                            if (p[16]==0)
                                                                                goto tolong;

                                                                            // 18446744073709551615
                                                                            if (p[16]==1)
                                                                            {
                                                                                if (p[17]<6)
                                                                                    goto tolong;

                                                                                // 18446744073709551615
                                                                                if (p[17]==6)
                                                                                {
                                                                                    if (p[18]==0)
                                                                                        goto tolong;

                                                                                    // 18446744073709551615
                                                                                    if (p[18]==1)
                                                                                    {
                                                                                        if (p[19]<6)
                                                                                        {
                                                                                            number=(long)(18446744073709551610UL+(ulong)p[19]);
                                                                                            if (str!=null)
                                                                                                stampStr(p,l,false,null,0,str,0);
                                                                                            return NumericType.Positive;
                                                                                        }
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    else if (l<19)
                        goto tolong;
                    else if (l==19)
                    {
                        if (*p!=9 || p[1]<2)
                            goto tolong;
                        if (p[1]==2)
                        {
                            if (p[2]<2)
                                goto tolong;
                            if (p[2]==2)
                            {
                                if (p[3]<3)
                                    goto tolong;

                                // 9223372036854775807
                                if (p[3]==3)
                                {
                                    if (p[4]<3)
                                        goto tolong;
                                    // 9223372036854775807
                                    if (p[4]==3)
                                    {
                                        if (p[5]<7)
                                            goto tolong;
                                        // 9223372036854775807
                                        if (p[5]==7)
                                        {
                                            if (p[6]<2)
                                                goto tolong;
                                            // 9223372036854775807
                                            if (p[6]==2 && p[7]==0)
                                            {
                                                if (p[8]<3)
                                                    goto tolong;
                                                // 9223372036854775807
                                                if (p[8]==3)
                                                {
                                                    if (p[9]<6)
                                                        goto tolong;
                                                    // 9223372036854775807
                                                    if (p[9]==6)
                                                    {
                                                        if (p[10]<8)
                                                            goto tolong;
                                                        // 9223372036854775807
                                                        if (p[10]==8)
                                                        {
                                                            if (p[11]<5)
                                                                goto tolong;
                                                            // 9223372036854775807
                                                            if (p[11]==5)
                                                            {
                                                                if (p[12]<4)
                                                                    goto tolong;
                                                                // 9223372036854775807
                                                                if (p[12]==4)
                                                                {
                                                                    if (p[13]<7)
                                                                        goto tolong;
                                                                    // 9223372036854775807
                                                                    if (p[13]==7)
                                                                    {
                                                                        if (p[14]<7)
                                                                            goto tolong;
                                                                        // 9223372036854775807
                                                                        if (p[14]==7)
                                                                        {
                                                                            if (p[15]<5)
                                                                                goto tolong;
                                                                            // 9223372036854775807
                                                                            if (p[15]==5)
                                                                            {
                                                                                if (p[16]<8)
                                                                                    goto tolong;
                                                                                // 9223372036854775807
                                                                                if (p[16]==8 && p[17]==0)
                                                                                {
                                                                                    if (p[18]<9)
                                                                                    {
                                                                                        if (p[18]==8)
                                                                                            number=Int64.MinValue;
                                                                                        else
                                                                                            number=-9223372036854775800L-(long)p[18];
                                                                                        if (str!=null)
                                                                                            stampStr(p,l,true,null,0,str,0);
                                                                                        return NumericType.Negative;
                                                                                    }
                                                                                }
                                                                            }
                                                                        }
                                                                    }
                                                                }
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
                psub=null;
                pos=0;
            }
            if (str!=null)
                stampStr(p,l,neg,psub,pos,str,strMode);
            double num=0d, mult;

            if (pos!=0)
            {
                mult=1d;
                while(--pos!=-1)
                {
                    num+=(double)psub[pos]*mult;
                    mult*=10d;
                }
                num/=mult;
            }
            if (l!=0)
            {
                mult=1d;
                while(--l!=-1)
                {
                    num+=(double)p[l]*mult;
                    mult*=10d;
                }
            }
            if (neg)
            {
                num=-num;
                number=*(long*)&num;
                return NumericType.Double|NumericType.Negative;
            }
            number=*(long*)&num;
            return NumericType.Double|NumericType.Positive;
            tolong:
                return InitLong(p,l,out number,neg,str);

        }

        private static unsafe void stampStrHex(byte* p, int l, char* str, ToStringMode strMode)
        {
            int a=0, sumLet=((strMode&ToStringMode.UpperHex)!=0)?55:87;
            do
            {
                if (p[a]>9)
                    str[a]=(char)(p[a]+sumLet);
                else
                    str[a]=(char)(p[a]+48);
            }while(++a!=l);
            str[l]='\0';
        }
        private static unsafe void stampStr(byte* p, int l, bool neg, byte* psep, int lsep, char* str, ToStringMode strMode)
        {
            int sum,a;
            if (neg)
            {
                *str='-';
                sum=1;
            }
            else
                sum=0;
            
            if (l!=0)
            {
                a=0;
                do
                {
                    str[sum++]=(char)(p[a]+48);
                }while(++a!=l);
            
            }
            else
                str[sum++]='0';
            if (lsep!=0)
            {
                switch(strMode&(ToStringMode.DecimalDot|ToStringMode.DecimalComma))
                {
                    case ToStringMode.DecimalDot:
                        str[sum++]='.';
                        break;
                    case ToStringMode.DecimalComma:
                        str[sum++]=',';
                        break;
                    default:
                        str[sum++]=DefaultDot?'.':',';
                        break;
                }
                a=0;
                do
                {
                    str[sum++]=(char)(psep[a]+48);
                }while(++a!=lsep);
            }
            str[sum]='\0';
        }

        private static unsafe NumericType InitLong(byte*p, int l, out long number, bool neg, char* str)
        {
            if (str!=null)
                stampStr(p,l,neg,null,0,str,0);
            if (!General.FrameWork64 && l<10)
                return InitInt(p,l,out number,neg);
            long n=0L,mult=1L;
            while(--l!=-1)
            {
                n+=(long)p[l]*mult;
                mult*=10L;
            }
            if (n==0L)
            {
                number=0L;
                return NumericType.Zero;
            }
            if (neg)
            {
                number=-n;
                return NumericType.Negative;
            }
            number=n;
            return NumericType.Positive;
        }

        private static unsafe NumericType InitInt(byte*p, int l, out long number, bool neg)
        {
            int n=0,mult=1;
            while(--l!=-1)
            {
                n+=(int)p[l]*mult;
                mult*=10;
            }
            if (n==0)
            {
                number=0L;
                return NumericType.Zero;
            }
            if (neg)
            {
                number=(long)-n;
                return NumericType.Negative;
            }
            number=(long)n;
            return NumericType.Positive;
        }

        private static unsafe NumericType InitLongHex(byte*p, int l, out long number, char* str, ToStringMode strMode)
        {
            if (str!=null)
                stampStrHex(p,l,str,strMode);
            long n=0L;
            byte*pn=(byte*)&n;
            int a=0;
            bool par=true;
            while(--l!=-1)
            {
                if (par)
                {
                    pn[a]=p[l];
                    par=false;
                }
                else
                {
                    pn[a]=(byte)((int)pn[a]+((int)p[l]<<4));
                    par=true;
                    a++;
                }
                   
            }
            if (n==0L)
            {
                number=0L;
                return NumericType.Zero;
            }
            number=n;
            return NumericType.Positive;
        }


        private static unsafe NumericType InitDoubleHex(byte*p, int l, out long number, char* str, ToStringMode strMode)
        {
            if (str!=null)
                stampStrHex(p,l,str,strMode);
            double num=0d, mult=1d;
            while(--l!=-1)
            {
                num+=p[l]*mult;
                mult*=16d;
            }
            number=*(long*)&num;
            return NumericType.Double|NumericType.Positive;
        }


        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="str">Cadena a reconocer.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        public StrToNum(string str, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept)
        {
            int l=str==null?0:str.Length;
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                unsafe
                {
                    fixed(char*p=str)
                        numericType=Init(p,l,mode,floatPoint,accept,out number,null,0);
                }
            }

        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="str">Cadena a reconocer (no puede ser nula).</param>
        /// <param name="i">Inicio de la zona de reconocimiento.</param>
        /// <param name="l">Largo de la zona de reconocimiento.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        public StrToNum(string str, int i, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept)
        {
            if (i<0 || l<0 || i+l>str.Length)
                throw new ArgumentOutOfRangeException("i,l",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                unsafe
                {
                    fixed(char*p=str)
                        numericType=Init(p+i,l,mode,floatPoint,accept,out number,null,0);
                }
            }

        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="arr">Array a reconocer (no puede ser nulo).</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        public StrToNum(char[] arr, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept)
        {
            int l=arr.Length;
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                unsafe
                {
                    fixed(char*p=arr)
                        numericType=Init(p,l,mode,floatPoint,accept,out number,null,0);
                }
            }

        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="arr">Array a reconocer (no puede ser nulo).</param>
        /// <param name="i">Inicio de la zona de reconocimiento.</param>
        /// <param name="l">Largo de la zona de reconocimiento.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        public StrToNum(char[] arr, int i, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept)
        {
            if (i<0 || l<0 || i+l>arr.Length)
                throw new ArgumentOutOfRangeException("i,l",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                unsafe
                {
                    fixed(char*p=arr)
                        numericType=Init(p+i,l,mode,floatPoint,accept,out number,null,0);
                }
            }

        }

        
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="p">Cadena a reconocer.</param>
        /// <param name="l">Largo.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        public unsafe StrToNum(char*p, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept)
        {
            numericType=Init(p,l,mode,floatPoint,accept,out number,null,0);
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="arr">Array a reconocer (no puede ser nulo). El contenido del array ser tomado como Latin1.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        public StrToNum(byte[] arr, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept)
        {
            int l=arr.Length;
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                unsafe
                {
                    fixed(byte*p=arr)
                        numericType=Init(p,l,mode,floatPoint,accept,out number,null,0);
                }
            }

        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="arr">Array a reconocer (no puede ser nulo). El contenido del array ser tomado como Latin1.</param>
        /// <param name="i">Inicio de la zona de reconocimiento.</param>
        /// <param name="l">Largo de la zona de reconocimiento.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        public StrToNum(byte[] arr, int i, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept)
        {
            if (i<0 || l<0 || i+l>arr.Length)
                throw new ArgumentOutOfRangeException("i,l",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                unsafe
                {
                    fixed(byte*p=arr)
                        numericType=Init(p+i,l,mode,floatPoint,accept,out number,null,0);
                }
            }

        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="p">Cadena a reconocer (el puntero de bytes ser tomado como una cadena Latin1).</param>
        /// <param name="l">Largo.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        public unsafe StrToNum(byte*p, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept)
        {
            numericType=Init(p,l,mode,floatPoint,accept,out number,null,0);
        }


        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="str">Cadena a reconocer.</param>
        /// <param name="strMode">Opciones para el modo ToString.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="toString">
        /// Si el puntero toString es diferente a null y la conversin es exitosa, se copiar la representacin de cadena en este puntero, que puede tener la misma direccion que la cadena de reconocimiento, insertando un caracter nulo al final.
        /// Por lo tanto, el largo de este puntero deber ser equivalente a la cadena de entrada + 1 (+ 2 si se inicia con coma decimal sin cero).
        ///  </param>
        public unsafe StrToNum(string str, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, char* toString, ToStringMode strMode)
        {
            int l=str==null?0:str.Length;
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                fixed(char*p=str)
                {
                    numericType=Init(p,l,mode,floatPoint,accept,out number,toString,strMode);
                }
            }

        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="str">Cadena a reconocer (no puede ser nula).</param>
        /// <param name="i">Inicio de la zona de reconocimiento.</param>
        /// <param name="l">Largo de la zona de reconocimiento.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="strMode">Opciones para el modo ToString.</param>
        /// <param name="toString">
        /// Si el puntero toString es diferente a null y la conversin es exitosa, se copiar la representacin de cadena en este puntero, que puede tener la misma direccion que la cadena de reconocimiento, insertando un caracter nulo al final.
        /// Por lo tanto, el largo de este puntero deber ser equivalente a la cadena de entrada + 1 (+ 2 si se inicia con coma decimal sin cero).
        ///  </param>
        public unsafe StrToNum(string str, int i, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, char* toString, ToStringMode strMode)
        {
            if (i<0 || l<0 || i+l>str.Length)
                throw new ArgumentOutOfRangeException("i,l",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                fixed(char*p=str)
                {
                    numericType=Init(p+i,l,mode,floatPoint,accept,out number,toString,strMode);
                }            
            }

        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="arr">Array a reconocer (no puede ser nulo).</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="strMode">Opciones para el modo ToString.</param>
        /// <param name="toString">
        /// Si el puntero toString es diferente a null y la conversin es exitosa, se copiar la representacin de cadena en este puntero, que puede tener la misma direccion que la cadena de reconocimiento, insertando un caracter nulo al final.
        /// Por lo tanto, el largo de este puntero deber ser equivalente a la cadena de entrada + 1 (+ 2 si se inicia con coma decimal sin cero).
        ///  </param>
        public unsafe StrToNum(char[] arr, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, char* toString, ToStringMode strMode)
        {
            int l=arr.Length;
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                fixed(char*p=arr)
                {
                    numericType=Init(p,l,mode,floatPoint,accept,out number,toString,strMode);
                }            
            }

        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="arr">Array a reconocer (no puede ser nulo).</param>
        /// <param name="i">Inicio de la zona de reconocimiento.</param>
        /// <param name="l">Largo de la zona de reconocimiento.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="strMode">Opciones para el modo ToString.</param>
        /// <param name="toString">
        /// Si el puntero toString es diferente a null y la conversin es exitosa, se copiar la representacin de cadena en este puntero, que puede tener la misma direccion que la cadena de reconocimiento, insertando un caracter nulo al final.
        /// Por lo tanto, el largo de este puntero deber ser equivalente a la cadena de entrada + 1 (+ 2 si se inicia con coma decimal sin cero).
        ///  </param>
        public unsafe StrToNum(char[] arr, int i, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, char* toString, ToStringMode strMode)
        {
            if (i<0 || l<0 || i+l>arr.Length)
                throw new ArgumentOutOfRangeException("i,l",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                fixed(char*p=arr)
                {
                    numericType=Init(p+i,l,mode,floatPoint,accept,out number,toString,strMode);
                }            
            }
        }

        
        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="p">Cadena a reconocer.</param>
        /// <param name="l">Largo.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="strMode">Opciones para el modo ToString.</param>
        /// <param name="toString">
        /// Si el puntero toString es diferente a null y la conversin es exitosa, se copiar la representacin de cadena en este puntero, que puede tener la misma direccion que la cadena de reconocimiento, insertando un caracter nulo al final.
        /// Por lo tanto, el largo de este puntero deber ser equivalente a la cadena de entrada + 1 (+ 2 si se inicia con coma decimal sin cero).
        ///  </param>
        public unsafe StrToNum(char*p, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, char* toString, ToStringMode strMode)
        {
            numericType=Init(p,l,mode,floatPoint,accept,out number,toString,strMode);
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="arr">Array a reconocer (no puede ser nulo). El contenido del array ser tomado como Latin1.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="strMode">Opciones para el modo ToString.</param>
        /// <param name="toString">
        /// Si el puntero toString es diferente a null y la conversin es exitosa, se copiar la representacin de cadena en este puntero, que puede tener la misma direccion que la cadena de reconocimiento, insertando un caracter nulo al final.
        /// Por lo tanto, el largo de este puntero deber ser equivalente a la cadena de entrada + 1 (+ 2 si se inicia con coma decimal sin cero).
        ///  </param>
        public unsafe StrToNum(byte[] arr, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, char* toString, ToStringMode strMode)
        {
            int l=arr.Length;
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                fixed(byte*p=arr)
                {
                    numericType=Init(p,l,mode,floatPoint,accept,out number,toString,strMode);
                }            
            }
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="arr">Array a reconocer (no puede ser nulo). El contenido del array ser tomado como Latin1.</param>
        /// <param name="i">Inicio de la zona de reconocimiento.</param>
        /// <param name="l">Largo de la zona de reconocimiento.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="strMode">Opciones para el modo ToString.</param>
        /// <param name="toString">
        /// Si el puntero toString es diferente a null y la conversin es exitosa, se copiar la representacin de cadena en este puntero, que puede tener la misma direccion que la cadena de reconocimiento, insertando un caracter nulo al final.
        /// Por lo tanto, el largo de este puntero deber ser equivalente a la cadena de entrada + 1 (+ 2 si se inicia con coma decimal sin cero).
        ///  </param>
        public unsafe StrToNum(byte[] arr, int i, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, char* toString, ToStringMode strMode)
        {
            if (i<0 || l<0 || i+l>arr.Length)
                throw new ArgumentOutOfRangeException("i,l",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (l==0)
            {
                number=0L;
                numericType=NumericType.NoNumber;
            }
            else
            {
                fixed(byte*p=arr)
                {
                    numericType=Init(p+i,l,mode,floatPoint,accept,out number,toString,strMode);
                }            
            }

        }

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="p">Cadena a reconocer (el puntero de bytes ser tomado como una cadena Latin1).</param>
        /// <param name="l">Largo.</param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="strMode">Opciones para el modo ToString.</param>
        /// <param name="toString">
        /// Si el puntero toString es diferente a null y la conversin es exitosa, se copiar la representacin de cadena en este puntero, que puede tener la misma direccion que la cadena de reconocimiento, insertando un caracter nulo al final.
        /// Por lo tanto, el largo de este puntero deber ser equivalente a la cadena de entrada + 1 (+ 2 si se inicia con coma decimal sin cero).
        ///  </param>
        public unsafe StrToNum(byte*p, int l, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, char* toString, ToStringMode strMode)
        {
            numericType=Init(p,l,mode,floatPoint,accept,out number,toString,strMode);
        }


        #endregion

        #region Propiedades de la estructura

        /// <summary>
        /// True si fue recuperado exitosamente.
        /// </summary>
        public bool IsConverted{get{return numericType!=NumericType.NoNumber;}}

        /// <summary>
        /// True si es Cero Entero.
        /// </summary>
        public bool IsZero{get{return (numericType&NumericType.Zero)!=0;}}

        /// <summary>
        /// Retorna true si el nmero subyacente es un entero vlido que se encuentra en el rango ulong (si es positivo) o long (si es negativo).
        /// </summary>
        public bool IsConvertedLong{get{return (numericType!=NumericType.NoNumber && (numericType&NumericType.Double)==0);}}

        /// <summary>
        /// Retorna true si el nmero subyacente es un nmero con decimales o un entero grande (positivo mayor al rango ulong o negativo mayor al rango long).
        /// </summary>
        public bool IsConvertedDouble{get{return (numericType&NumericType.Double)!=0;}}

        /// <summary>
        /// True si el nmero es Positivo.
        /// </summary>
        public bool IsPositive{get{return (numericType&NumericType.Positive)!=0;}}

        /// <summary>
        /// True si el nmero es Negativo.
        /// </summary>
        public bool IsNegative{get{return (numericType&NumericType.Negative)!=0;}}

        /// <summary>
        /// True si el nmero es fue interpretado como boolean (solo cuando fue inicializado con NumericMode.Bool y se detecta true o false).
        /// </summary>
        public bool IsRealBool{get{return (numericType&(NumericType.True|NumericType.False))!=0;}}

        /// <summary>
        /// True si el nmero es fue interpretado como True (solo cuando fue inicializado con NumericMode.Bool y se detecta true).
        /// </summary>
        public bool IsRealTrue{get{return (numericType&NumericType.True)!=0;}}

        /// <summary>
        /// True si el nmero es fue interpretado como False (solo cuando fue inicializado con NumericMode.Bool y se detecta false).
        /// </summary>
        public bool IsRealFalse{get{return (numericType&NumericType.False)!=0;}}

        /// <summary>
        /// True en cualquier valor diferente de 0.
        /// </summary>
        public bool GetBool{get{return number!=0L;}}

        /// <summary>
        /// Obtiene el valor o NaN si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public float GetSingle
        {
            get
            {
                return ValueOrDefault(Single.NaN);
            }
        }

        /// <summary>
        /// Recupera el nmero subyacente en forma directa.
        /// El nmero subyacente puede encontrarse en formato long o double.
        /// </summary>
        public long GetInternalNumber
        {
            get
            {
                return number;
            }
        }


        /// <summary>
        /// Obtiene el valor o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public decimal GetDecimal
        {
            get
            {
                return ValueOrDefault(Decimal.Zero);
            }
        }


        /// <summary>
        /// Obtiene el valor o NaN si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public double GetDouble
        {
            get
            {
                return ValueOrDefault(Double.NaN);
            }
        }



        /// <summary>
        /// Obtiene el valor o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public byte GetByte
        {
            get
            {
                return ValueOrDefault(Byte.MinValue);
            }
        }

        /// <summary>
        /// Obtiene el valor o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public sbyte GetSByte
        {
            get
            {
                return ValueOrDefault((sbyte)0);
            }
        }

        /// <summary>
        /// Obtiene el valor o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public ushort GetUShort
        {
            get
            {
                return ValueOrDefault((ushort)0);
            }
        }


        /// <summary>
        /// Obtiene el valor o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public short GetShort
        {
            get
            {
                return ValueOrDefault((short)0);
            }
        }

        /// <summary>
        /// Obtiene el valor o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public uint GetUInt
        {
            get
            {
                return ValueOrDefault(0U);
            }
        }

        /// <summary>
        /// Obtiene el valor o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public int GetInt
        {
            get
            {
                return ValueOrDefault(0);
            }
        }

        /// <summary>
        /// Obtiene el valor o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public ulong GetULong
        {
            get
            {
                return ValueOrDefault(0UL);
            }
        }

        /// <summary>
        /// Obtiene el valor o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public long GetLong
        {
            get
            {
                return ValueOrDefault(0L);
            }
        }



        /// <summary>
        /// Obtiene el valor (redondeado si el subyacente es Double) o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public byte GetByteRound
        {
            get
            {
                return ValueOrDefaultRound(Byte.MinValue);
            }
        }

        /// <summary>
        /// Obtiene el valor (redondeado si el subyacente es Double) o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public sbyte GetSByteRound
        {
            get
            {
                return ValueOrDefaultRound((sbyte)0);
            }
        }

        /// <summary>
        /// Obtiene el valor (redondeado si el subyacente es Double) o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public ushort GetUShortRound
        {
            get
            {
                return ValueOrDefaultRound((ushort)0);
            }
        }


        /// <summary>
        /// Obtiene el valor (redondeado si el subyacente es Double) o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public short GetShortRound
        {
            get
            {
                return ValueOrDefaultRound((short)0);
            }
        }

        /// <summary>
        /// Obtiene el valor (redondeado si el subyacente es Double) o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public uint GetUIntRound
        {
            get
            {
                return ValueOrDefaultRound(0U);
            }
        }

        /// <summary>
        /// Obtiene el valor (redondeado si el subyacente es Double) o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public int GetIntRound
        {
            get
            {
                return ValueOrDefaultRound(0);
            }
        }

        /// <summary>
        /// Obtiene el valor (redondeado si el subyacente es Double) o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public ulong GetULongRound
        {
            get
            {
                return ValueOrDefaultRound(0UL);
            }
        }

        /// <summary>
        /// Obtiene el valor (redondeado si el subyacente es Double) o cero si este no se encuentra en el rango o no es convertible.
        /// </summary>
        public long GetLongRound
        {
            get
            {
                return ValueOrDefaultRound(0L);
            }
        }


        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Byte (sin redondear).
        /// </summary>
        public unsafe bool IsByte
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<256L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<256d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>-1d;
                    default:
                        return false;
                }
            }
        }


        /// <summary>
        /// True si el nmero es vlido y convertible en el rango UShort (sin redondear).
        /// </summary>
        public unsafe bool IsUShort
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<65536L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<65536d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>-1d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango UInt (sin redondear).
        /// </summary>
        public unsafe bool IsUInt
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<4294967296L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<4294967296d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>-1d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango ULong (sin redondear).
        /// </summary>
        public unsafe bool IsULong
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                    case NumericType.Positive:
                        return true;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<18446744073709551616d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>-1d;
                    default:
                        return false;
                }
            }
        }


        /// <summary>
        /// True si el nmero es vlido y convertible en el rango SByte (sin redondear).
        /// </summary>
        public unsafe bool IsSByte
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<128L;
                    case NumericType.Negative:
                        return number>-129L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<128d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>-129d;
                    default:
                        return false;
                }
            }
        }


        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Short (sin redondear).
        /// </summary>
        public unsafe bool IsShort
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<32768L;
                    case NumericType.Negative:
                        return number>-32769L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<32768d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>-32769d;
                    default:
                        return false;
                }
            }
        }

   
        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Int32 (sin redondear).
        /// </summary>
        public unsafe bool IsInt
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<2147483648L;
                    case NumericType.Negative:
                        return number>-2147483649L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<2147483648d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>-2147483649d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Int64 (sin redondear).
        /// </summary>
        public unsafe bool IsLong
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                    case NumericType.Negative:
                        return true;
                    case NumericType.Positive:
                        return number>-1L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<9223372036854775808d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>-9223372036854775809d;
                    default:
                        return false;
                }
            }
        }


        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Byte (efectuando redondeo si el nmero subyacente es double).
        /// </summary>
        public unsafe bool IsByteRound
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<256L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<255.5d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-0.5d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango UShort (efectuando redondeo si el nmero subyacente es double).
        /// </summary>
        public unsafe bool IsUShortRound
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<65536L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<65535.5d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-0.5d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango UInt32 (efectuando redondeo si el nmero subyacente es double).
        /// </summary>
        public unsafe bool IsUIntRound
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<4294967296L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<4294967295.5d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-0.5d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango UInt64 (efectuando redondeo si el nmero subyacente es double).
        /// </summary>
        public unsafe bool IsULongRound
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                    case NumericType.Positive:
                        return true;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<18446744073709551615.5d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-0.5d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango SByte (efectuando redondeo si el nmero subyacente es double).
        /// </summary>
        public unsafe bool IsSByteRound
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<128L;
                    case NumericType.Negative:
                        return number>-129L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<127.5d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-128.5d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Short (efectuando redondeo si el nmero subyacente es double).
        /// </summary>
        public unsafe bool IsShortRound
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<32768L;
                    case NumericType.Negative:
                        return number>-32769L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<32767.5d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-32768.5d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Int32 (efectuando redondeo si el nmero subyacente es double).
        /// </summary>
        public unsafe bool IsIntRound
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                        return true;
                    case NumericType.Positive:
                        return number>-1L && number<2147483648L;
                    case NumericType.Negative:
                        return number>-2147483649L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<2147483647.5d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-2147483648.5d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Int64 (efectuando redondeo si el nmero subyacente es double).
        /// </summary>
        public unsafe bool IsLongRound
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                    case NumericType.Negative:
                        return true;
                    case NumericType.Positive:
                        return number>-1L;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<9223372036854775807.5d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-9223372036854775808.5d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Boolean.
        /// </summary>
        public bool IsBool
        {
            get
            {
                return numericType!=NumericType.NoNumber;
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Double.
        /// </summary>
        public bool IsDouble
        {
            get
            {
                return numericType!=NumericType.NoNumber;
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Single (sin redondear).
        /// </summary>
        public unsafe bool IsSingle
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                    case NumericType.Positive:
                    case NumericType.Negative:
                        return true;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<=3.40282346638529E+38d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-3.40282346638529E+38d;
                    default:
                        return false;
                }
            }
        }

        /// <summary>
        /// True si el nmero es vlido y convertible en el rango Decimal (sin redondear).
        /// </summary>
        public unsafe bool IsDecimal
        {
            get
            {
                switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
                {
                    case NumericType.Zero:
                    case NumericType.Positive:
                    case NumericType.Negative:
                        return true;
                    case (NumericType.Double|NumericType.Positive):
                        fixed(long*p=&number)
                            return (*(double*)p)<=7.92281625142643E+28d;
                    case (NumericType.Double|NumericType.Negative):
                        fixed(long*p=&number)
                            return (*(double*)p)>=-7.92281625142643E+28d;
                    default:
                        return false;
                }
            }
        }

        #endregion

        #region Mtodos de la estructura.

        /// <summary>
        /// Obtiene el valor boleano o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public bool ValueOrDefault(bool def)
        {
            if (numericType==NumericType.NoNumber)
                return def;
            return number!=0L;
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe float ValueOrDefault(float def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0f;
                case NumericType.Positive:
                    if (number<0L)
                        return (float)(ulong)number;
                    return (float)number;
                case NumericType.Negative:
                    return (float)number;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<=3.40282346638529E+38d)
                            return (float)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-3.40282346638529E+38d)
                            return (float)(*(double*)p);
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe decimal ValueOrDefault(decimal def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0m;
                case NumericType.Positive:
                    if (number<0L)
                        return (decimal)(ulong)number;
                    return (decimal)number;
                case NumericType.Negative:
                    return (decimal)number;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<=7.92281625142643E+28d)
                            return (decimal)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-7.92281625142643E+28d)
                            return (decimal)(*(double*)p);
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe double ValueOrDefault(double def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0d;
                case NumericType.Positive:
                    if (number<0L)
                        return (double)(ulong)number;
                    return (double)number;
                case NumericType.Negative:
                    return (double)number;
                case (NumericType.Double|NumericType.Positive):
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        return (*(double*)p);
                default:
                    return def;
            }
        }


        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe byte ValueOrDefault(byte def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return (byte)0;
                case NumericType.Positive:
                    if (number>-1L && number<256L)
                        return (byte)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<256d)
                            return (byte)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>-1d)
                            return (byte)0;
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe ushort ValueOrDefault(ushort def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return (ushort)0;
                case NumericType.Positive:
                    if (number>-1L && number<65536L)
                        return (ushort)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<65536d)
                            return (ushort)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>-1d)
                            return (ushort)0;
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe uint ValueOrDefault(uint def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0U;
                case NumericType.Positive:
                    if (number>-1L && number<4294967296L)
                        return (uint)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<4294967296d)
                            return (uint)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>-1d)
                            return 0U;
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe ulong ValueOrDefault(ulong def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0UL;
                case NumericType.Positive:
                    return (ulong)number;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<18446744073709551616d)
                            return (ulong)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>-1d)
                            return 0UL;
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe sbyte ValueOrDefault(sbyte def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return (sbyte)0;
                case NumericType.Positive:
                    if (number>-1L && number<128L)
                        return (sbyte)number;
                    return def;
                case NumericType.Negative:
                    if (number>-129L)
                        return (sbyte)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<128d)
                            return (sbyte)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>-129d)
                            return (sbyte)(*(double*)p);
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe short ValueOrDefault(short def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return (short)0;
                case NumericType.Positive:
                    if (number>-1L && number<32768L)
                        return (short)number;
                    return def;
                case NumericType.Negative:
                    if (number>-32769L)
                        return (short)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<32768d)
                            return (short)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>-32769d)
                            return (short)(*(double*)p);
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe int ValueOrDefault(int def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0;
                case NumericType.Positive:
                    if (number>-1L && number<2147483648L)
                        return (int)number;
                    return def;
                case NumericType.Negative:
                    if (number>-2147483649L)
                        return (int)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<2147483648d)
                            return (int)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>-2147483649d)
                            return (int)(*(double*)p);
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe long ValueOrDefault(long def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0L;
                case NumericType.Positive:
                    if (number>-1L)
                        return number;
                    return def;
                case NumericType.Negative:
                    return number;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<9223372036854775808d)
                            return (long)(*(double*)p);
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>-9223372036854775809d)
                            return (long)(*(double*)p);
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// En caso de que el valor subyacente sea double, el valor convertido se redondea.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe byte ValueOrDefaultRound(byte def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return (byte)0;
                case NumericType.Positive:
                    if (number>-1L && number<256L)
                        return (byte)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<255.5d)
                        {
                            int v=(int)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif>0.5d || (dif==0.5d && (v&1)!=0))
                                v++;
                            return (byte)v;
                        }
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-0.5d)
                            return (byte)0;
                    return def;
                default:
                    return def;
            }
        }


        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// En caso de que el valor subyacente sea double, el valor convertido se redondea.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe ushort ValueOrDefaultRound(ushort def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return (ushort)0;
                case NumericType.Positive:
                    if (number>-1L && number<65536L)
                        return (ushort)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<65535.5d)
                        {
                            int v=(int)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif>0.5d || (dif==0.5d && (v&1)!=0))
                                v++;
                            return (ushort)v;
                        }
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-0.5d)
                            return (ushort)0;
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// En caso de que el valor subyacente sea double, el valor convertido se redondea.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe uint ValueOrDefaultRound(uint def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0U;
                case NumericType.Positive:
                    if (number>-1L && number<4294967296L)
                        return (uint)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<4294967295.5d)
                        {
                            uint v=(uint)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif>0.5d || (dif==0.5d && (v&1)!=0))
                                v++;
                            return v;
                        }
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-0.5d)
                            return 0U;
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// En caso de que el valor subyacente sea double, el valor convertido se redondea.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe ulong ValueOrDefaultRound(ulong def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0UL;
                case NumericType.Positive:
                    return (ulong)number;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<18446744073709551615.5d)
                        {
                            ulong v=(ulong)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif>0.5d || (dif==0.5d && (v&1)!=0))
                                v++;
                            return v;
                        }
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-0.5d)
                            return 0UL;
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// En caso de que el valor subyacente sea double, el valor convertido se redondea.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe sbyte ValueOrDefaultRound(sbyte def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return (sbyte)0;
                case NumericType.Positive:
                    if (number>-1L && number<128L)
                        return (sbyte)number;
                    return def;
                case NumericType.Negative:
                    if (number>-129L)
                        return (sbyte)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<127.5d)
                        {
                            int v=(int)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif>0.5d || (dif==0.5d && (v&1)!=0))
                                v++;
                            return (sbyte)v;
                        }
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-128.5d)
                        {
                            int v=(int)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif<-0.5d || (dif==-0.5d && (v&1)!=0))
                                v--;
                            return (sbyte)v;
                        }
                    return def;
                default:
                    return def;
            }

        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// En caso de que el valor subyacente sea double, el valor convertido se redondea.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe short ValueOrDefaultRound(short def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return (short)0;
                case NumericType.Positive:
                    if (number>-1L && number<32768L)
                        return (short)number;
                    return def;
                case NumericType.Negative:
                    if (number>-32769L)
                        return (short)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<32767.5d)
                        {
                            int v=(int)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif>0.5d || (dif==0.5d && (v&1)!=0))
                                v++;
                            return (short)v;
                        }
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-32768.5d)
                        {
                            int v=(int)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif<-0.5d || (dif==-0.5d && (v&1)!=0))
                                v--;
                            return (short)v;
                        }
                    return def;
                default:
                    return def;
            }

        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// En caso de que el valor subyacente sea double, el valor convertido se redondea.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe int ValueOrDefaultRound(int def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0;
                case NumericType.Positive:
                    if (number>-1L && number<2147483648L)
                        return (int)number;
                    return def;
                case NumericType.Negative:
                    if (number>-2147483649L)
                        return (int)number;
                    return def;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<2147483647.5d)
                        {
                            int v=(int)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif>0.5d || (dif==0.5d && (v&1)!=0))
                                v++;
                            return v;
                        }
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-2147483648.5d)
                        {
                            int v=(int)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif<-0.5d || (dif==-0.5d && (v&1)!=0))
                                v--;
                            return v;
                        }
                    return def;
                default:
                    return def;
            }

        }

        /// <summary>
        /// Obtiene el valor convertido o el valor por defecto.
        /// En caso de que el valor subyacente sea double, el valor convertido se redondea.
        /// </summary>
        /// <param name="def">Valor por defecto (solo es retornado si el nmero no es convertible o representa un valor fuera de rango).</param>
        /// <returns>Valor convertido o el valor por defecto.</returns>
        public unsafe long ValueOrDefaultRound(long def)
        {
            switch(numericType&(NumericType.Zero|NumericType.Double|NumericType.Negative|NumericType.Positive))
            {
                case NumericType.Zero:
                    return 0L;
                case NumericType.Positive:
                    if (number>-1L)
                        return number;
                    return def;
                case NumericType.Negative:
                    return number;
                case (NumericType.Double|NumericType.Positive):
                    fixed(long*p=&number)
                        if ((*(double*)p)<9223372036854775807.5d)
                        {
                            long v=(long)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif>0.5d || (dif==0.5d && (v&1)!=0))
                                v++;
                            return v;
                        }
                    return def;
                case (NumericType.Double|NumericType.Negative):
                    fixed(long*p=&number)
                        if ((*(double*)p)>=-9223372036854775808.5d)
                        {
                            long v=(long)(*(double*)p);
                            double dif=(*(double*)p)-v;
                            if (dif<-0.5d || (dif==-0.5d && (v&1)!=0))
                                v--;
                            return v;
                        }
                    return def;
                default:
                    return def;
            }
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref bool val)
        {
            if (numericType==NumericType.NoNumber)
                return false;
            val=number!=0L;
            return true;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref float val, float min, float max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            float ret=ValueOrDefault(val);
            if (ret==val)
                return IsSingle;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref decimal val, decimal min, decimal max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            decimal ret=ValueOrDefault(val);
            if (ret==val)
                return IsDecimal;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref double val, double min, double max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            double ret=ValueOrDefault(val);
            if (ret==val)
                return IsDouble;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref byte val, byte min, byte max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            byte ret=ValueOrDefault(val);
            if (ret==val)
                return IsByte;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref sbyte val, sbyte min, sbyte max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            sbyte ret=ValueOrDefault(val);
            if (ret==val)
                return IsSByte;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref ushort val, ushort min, ushort max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            ushort ret=ValueOrDefault(val);
            if (ret==val)
                return IsUShort;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref short val, short min, short max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            short ret=ValueOrDefault(val);
            if (ret==val)
                return IsShort;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref uint val, uint min, uint max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            uint ret=ValueOrDefault(val);
            if (ret==val)
                return IsUInt;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref int val, int min, int max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            int ret=ValueOrDefault(val);
            if (ret==val)
                return IsInt;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref long val, long min, long max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            long ret=ValueOrDefault(val);
            if (ret==val)
                return IsLong;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRange(ref ulong val, ulong min, ulong max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            ulong ret=ValueOrDefault(val);
            if (ret==val)
                return IsULong;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }



        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar. En caso de que el valor subyacente sea Double, la conversin se efecta con redondeo.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRangeRound(ref byte val, byte min, byte max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            byte ret=ValueOrDefaultRound(val);
            if (ret==val)
                return IsByteRound;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar. En caso de que el valor subyacente sea Double, la conversin se efecta con redondeo.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRangeRound(ref sbyte val, sbyte min, sbyte max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            sbyte ret=ValueOrDefaultRound(val);
            if (ret==val)
                return IsSByteRound;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar. En caso de que el valor subyacente sea Double, la conversin se efecta con redondeo.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRangeRound(ref ushort val, ushort min, ushort max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            ushort ret=ValueOrDefaultRound(val);
            if (ret==val)
                return IsUShortRound;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar. En caso de que el valor subyacente sea Double, la conversin se efecta con redondeo.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRangeRound(ref short val, short min, short max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            short ret=ValueOrDefaultRound(val);
            if (ret==val)
                return IsShortRound;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar. En caso de que el valor subyacente sea Double, la conversin se efecta con redondeo.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRangeRound(ref uint val, uint min, uint max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            uint ret=ValueOrDefaultRound(val);
            if (ret==val)
                return IsUIntRound;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar. En caso de que el valor subyacente sea Double, la conversin se efecta con redondeo.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRangeRound(ref int val, int min, int max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            int ret=ValueOrDefaultRound(val);
            if (ret==val)
                return IsIntRound;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar. En caso de que el valor subyacente sea Double, la conversin se efecta con redondeo.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRangeRound(ref long val, long min, long max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            long ret=ValueOrDefaultRound(val);
            if (ret==val)
                return IsLongRound;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }

        /// <summary>
        /// Reemplaza el valor de la referencia solo si el nmero fue convertido exitosamente y
        /// este se encuentra dentro del rango especificado.
        /// </summary>
        /// <param name="val">Valor a reemplazar. En caso de que el valor subyacente sea Double, la conversin se efecta con redondeo.</param>
        /// <param name="min">Valor mnimo (inclusive).</param>
        /// <param name="max">Valor mximo (inclusive).</param>
        /// <returns>True si el valor de la referencia fue reemplazado.</returns>
        public bool ReplaceIfRangeRound(ref ulong val, ulong min, ulong max)
        {
            if (min>max)
                throw new ArgumentOutOfRangeException("min,max",Errors.MIN_MAJOR_MAX);
            ulong ret=ValueOrDefaultRound(val);
            if (ret==val)
                return IsULongRound;
            if (ret>=min && ret<=max)
            {
                val=ret;
                return true;
            }
            return false;
        }


        #endregion

        #region Reconocimiento de cadenas de Bits Little y Big Endian

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static ulong LittleEndianToULong(string bits)
        {
            int l=bits==null?0:bits.Length;
            if (l == 0)
                return 0;
            unsafe
            {
                int a = 0;
                int b = 0;
                ulong res = 0;
                fixed (char* p = bits)
                {
                    do
                    {
                        switch (p[a])
                        {
                            case '1':
                            case 'v':
                            case 'V':
                            case 't':
                            case 'T':
                                res += 1UL << b;
                                if (++b == 64)
                                    break;
                                continue;
                            case '0':
                            case 'f':
                            case 'F':
                                if (++b == 64)
                                    break;
                                continue;
                            case ' ':
                            case '\t':
                            case '\x85':
                            case '\xa0':
                            case '\0':
                                continue;
                            default:
                                throw new FormatException(Errors.BAD_FORMAT);
                        }
                        break;
                    } while (++a != l);
                }
                return res;
            }
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static ulong BigEndianToULong(string bits)
        {
            int l=bits==null?0:bits.Length;
            if (l == 0)
                return 0;
            unsafe
            {
                int b = 0;
                ulong res = 0;
                fixed (char* p = bits)
                {
                    do
                    {
                        switch (p[--l])
                        {
                            case '1':
                            case 'v':
                            case 'V':
                            case 't':
                            case 'T':
                                res += 1UL << b;
                                if (++b == 64)
                                    break;
                                continue;
                            case '0':
                            case 'f':
                            case 'F':
                                if (++b == 64)
                                    break;
                                continue;
                            case ' ':
                            case '\t':
                            case '\x85':
                            case '\xa0':
                            case '\0':
                                continue;
                            default:
                                throw new FormatException(Errors.BAD_FORMAT);
                        }
                        break;
                    } while (l!=0);
                }
                return res;
            }
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static long LittleEndianToLong(string bits)
        {
            return (long)LittleEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static long BigEndianToLong(string bits)
        {
            return (long)BigEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static uint LittleEndianToUInt(string bits)
        {
            return (uint)LittleEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static uint BigEndianToUInt(string bits)
        {
            return (uint)BigEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static int LittleEndianToInt(string bits)
        {
            return (int)LittleEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static int BigEndianToInt(string bits)
        {
            return (int)BigEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static ushort LittleEndianToUShort(string bits)
        {
            return (ushort)LittleEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static ushort BigEndianToUShort(string bits)
        {
            return (ushort)BigEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static short LittleEndianToShort(string bits)
        {
            return (short)LittleEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static short BigEndianToShort(string bits)
        {
            return (short)BigEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static byte LittleEndianToByte(string bits)
        {
            return (byte)LittleEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static byte BigEndianToByte(string bits)
        {
            return (byte)BigEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static sbyte LittleEndianToSByte(string bits)
        {
            return (sbyte)LittleEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static sbyte BigEndianToSByte(string bits)
        {
            return (sbyte)BigEndianToULong(bits);
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static float LittleEndianToSingle(string bits)
        {
            unsafe
            {
                ulong z = LittleEndianToULong(bits);
                return *(float*)&z;
            }
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static float BigEndianToSingle(string bits)
        {
            unsafe
            {
                ulong z = BigEndianToULong(bits);
                return *(float*)&z;
            }
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al menos significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static double LittleEndianToDouble(string bits)
        {
            unsafe
            {
                ulong z = LittleEndianToULong(bits);
                return *(double*)&z;
            }
        }

        /// <summary>
        /// Obtiene valor generado a partir de una cadena con Bits ordenados
        /// en base al ms significativo a la izquierda.
        /// </summary>
        /// <param name="bits">
        /// Cadena con los bits:
        /// Se reconoce 1-V-v-T-t como 1 y 0-F-f como 0.
        /// Espacios y taburadores son ignorados. Cualquier otro caracter genera excepcin.
        /// </param>
        /// <returns>
        /// Nmero generado a partir de la cadena de bits.
        /// Los bits que excedan el rango del tipo son ignorados.
        /// </returns>
        public static double BigEndianToDouble(string bits)
        {
            unsafe
            {
                ulong z = BigEndianToULong(bits);
                return *(double*)&z;
            }
        }


        #endregion

        #region StrToBoolSpEn

        /// <summary>
        /// Convierte una cadena en espaol o ingls a valor booleano (0 o 1) o -1 si no puede ser reconocida.
        /// El reconocimiento ignora May/Min y toma la primer palabra de la cadena (hace trim y descarta carcteres no vlidos).
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="aceptarCancelar">Reconoce las palabras aceptar y cancelar.</param>
        /// <param name="numbers">Reconoce al 0 como false y a cualquier nmero mayor a cero como true.</param>
        /// <param name="okCancel">Reconoce las palabras ok y cancel.</param>
        /// <param name="siNo">Reconoce las palabras si, s, s, no y n.</param>
        /// <param name="trueFalse">Reconoce las palabras true, t, false y f.</param>
        /// <param name="verdaderoFalso">Reconoce las palabras verdadero, v, falso y f.</param>
        /// <param name="yesNo">Reconoce las palabras yes, y, no y n.</param>
        /// <returns>0 si false, 1 si true o -1 si no es reconocida.</returns>
        public static int StrToBoolSpEn(string str, bool siNo, bool yesNo, bool verdaderoFalso, bool trueFalse, bool okCancel, bool aceptarCancelar, bool numbers)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (char* p = str)
                {
                    Mem.PChar pc = new EnPalermo.Mem.PChar(p, l, true, false).FowardWord(false);
                    if (pc.Length == 0)
                        return -1;
                    if (pc.Length == 1)
                    {
                        switch (*pc.Pointer)
                        {

                            case 'n':
                            case 'N':
                                if (siNo || yesNo)
                                    return 0;
                                return -1;

                            case 's':
                            case 'S':
                                if (siNo)
                                    return 1;
                                return -1;

                            case 'y':
                            case 'Y':
                                if (yesNo)
                                    return 1;
                                return -1;

                            case '0':
                                if (numbers)
                                    return 0;
                                return -1;

                            case '1':
                            case '2':
                            case '3':
                            case '4':
                            case '5':
                            case '6':
                            case '7':
                            case '8':
                            case '9':
                                if (numbers)
                                    return 1;
                                return -1;

                            case 'f':
                            case 'F':
                                if (verdaderoFalso || trueFalse)
                                    return 0;
                                return -1;

                            case 'v':
                            case 'V':
                                if (verdaderoFalso)
                                    return 1;
                                return -1;

                            case 't':
                            case 'T':
                                if (trueFalse)
                                    return 1;
                                return -1;
                        }
                        return -1;
                    }
                    string str2 = Mutables.Create(pc.Length);
                    fixed (char* p2 = str2)
                        Texts.ToUpperInvariant(pc.Pointer ,p2, pc.Length);
                    switch (str2)
                    {
                        case "NO":
                            if (siNo || yesNo)
                                return 0;
                            return -1;

                        case "SI":
                        case "S":
                            if (siNo)
                                return 1;
                            return -1;

                        case "YES":
                            if (yesNo)
                                return 1;
                            return -1;

                        case "FALSO":
                            if (verdaderoFalso)
                                return 0;
                            return -1;

                        case "FALSE":
                            if (trueFalse)
                                return 0;
                            return -1;

                        case "VERDADERO":
                            if (verdaderoFalso)
                                return 1;
                            return -1;

                        case "TRUE":
                            if (trueFalse)
                                return 1;
                            return -1;

                        case "CANCELAR":
                            if (aceptarCancelar)
                                return 0;
                            return -1;

                        case "CANCEL":
                            if (okCancel)
                                return 0;
                            return -1;

                        case "ACEPTAR":
                            if (aceptarCancelar)
                                return 1;
                            return -1;

                        case "OK":
                            if (okCancel)
                                return 1;
                            return -1;

                    }
                    if (numbers)
                    {
                        l =StrToPositiveInt(pc.Pointer,pc.Length,0,Int32.MaxValue);
                        if (l!=-1)
                            return l==0?0:1;
                    }
                    return -1;
                }
            }
        }

        #endregion

        #region StrToPositiveInt

        /// <summary>
        /// Obtiene un nmero Int32 positivo o -1 si no es convertible.
        /// La conversin es rpida y se realiza de acuerdo a estas reglas:
        /// Los espacios estandar son ignorados en cualquier ubicacin.
        /// Puntos o comas implican el reinicio del reconocimiento (de derecha a izquierda).
        /// Cualquier otro carcter diferente a dgito invalida el reconocimiento.
        /// </summary>
        /// <param name="p">Cadena.</param>
        /// <param name="l">Largo.</param>
        /// <param name="min">Mnimo valido (inclusivo, no puede ser negativo).</param>
        /// <param name="max">Mximo valido (inclusivo, no puede ser menor a min).</param>
        /// <returns>Entero positivo si el nmero traducido es vlido y dentro del rango, de lo contrario -1.</returns>
        public unsafe static int StrToPositiveInt(byte*p, int l, int min, int max)
        {
            if (min<0)
                throw new ArgumentOutOfRangeException("min",min,Errors.NEGATIVE);
            if (max<min)
                throw new ArgumentOutOfRangeException("max",max,Errors.MIN_MAJOR_MAX);
            if (l<1)
            {
                if (l!=0)
                    throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
                return -1;
            }
            bool zona=false;
            int ret=-1;
            int mult=1;
            do
            {
                switch(p[--l])
                {
                    case 48:
                    case 49:
                    case 50:
                    case 51:
                    case 52:
                    case 53:
                    case 54:
                    case 55:
                    case 56:
                    case 57:
                        if (!zona)
                        {
                            ret=(int)p[l]-48;
                            zona=true;
                        }
                        else if (p[l]=='0')
                        {
                            if (mult<1000000000)
                                mult*=10;
                        }
                        else if (mult>=100000000)
                        {
                            if (mult==1000000000 || p[l]>'2')
                                return -1;
                            mult=1000000000;
                            ret+=((int)p[l]-48)*mult;
                        }
                        else
                        {
                            mult*=10;
                            ret+=((int)p[l]-48)*mult;
                        }
                        continue;
                    case 32:
                        continue;
                    case 46:
                    case 44:
                        if (zona)
                        {
                            zona=false;
                            mult=1;
                            ret=0;
                        }
                        break;
                    default:
                        return -1;
                }
            }while(l!=0);
            if (ret<min || ret>max)
                return -1;
            return ret;
        }

        /// <summary>
        /// Obtiene un nmero Int32 positivo o -1 si no es convertible.
        /// La conversin es rpida y se realiza de acuerdo a estas reglas:
        /// Los espacios estandar son ignorados en cualquier ubicacin.
        /// Puntos o comas implican el reinicio del reconocimiento (de derecha a izquierda).
        /// Cualquier otro carcter diferente a dgito invalida el reconocimiento.
        /// </summary>
        /// <param name="p">Cadena.</param>
        /// <param name="l">Largo.</param>
        /// <param name="min">Mnimo valido (inclusivo, no puede ser negativo).</param>
        /// <param name="max">Mximo valido (inclusivo, no puede ser menor a min).</param>
        /// <returns>Entero positivo si el nmero traducido es vlido y dentro del rango, de lo contrario -1.</returns>
        public unsafe static int StrToPositiveInt(char*p, int l, int min, int max)
        {
            if (min<0)
                throw new ArgumentOutOfRangeException("min",min,Errors.NEGATIVE);
            if (max<min)
                throw new ArgumentOutOfRangeException("max",max,Errors.MIN_MAJOR_MAX);
            if (l<1)
            {
                if (l!=0)
                    throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
                return -1;
            }
            bool zona=false;
            int ret=-1;
            int mult=1;
            do
            {
                switch(p[--l])
                {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        if (!zona)
                        {
                            ret=(int)p[l]-48;
                            zona=true;
                        }
                        else if (p[l]=='0')
                        {
                            if (mult<1000000000)
                                mult*=10;
                        }
                        else if (mult>=100000000)
                        {
                            if (mult==1000000000 || p[l]>'2')
                                return -1;
                            mult=1000000000;
                            ret+=((int)p[l]-48)*mult;
                        }
                        else
                        {
                            mult*=10;
                            ret+=((int)p[l]-48)*mult;
                        }
                        continue;
                    case ' ':
                        continue;
                    case '.':
                    case ',':
                        if (zona)
                        {
                            zona=false;
                            mult=1;
                            ret=0;
                        }
                        break;
                    default:
                        return -1;
                }
            }while(l!=0);
            if (ret<min || ret>max)
                return -1;
            return ret;
        }

        /// <summary>
        /// Obtiene un nmero Int32 positivo o -1 si no es convertible.
        /// La conversin es rpida y se realiza de acuerdo a estas reglas:
        /// Los espacios estandar son ignorados en cualquier ubicacin.
        /// Puntos o comas implican el reinicio del reconocimiento (de derecha a izquierda).
        /// Cualquier otro carcter diferente a dgito invalida el reconocimiento.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="min">Mnimo valido (inclusivo, no puede ser negativo).</param>
        /// <param name="max">Mximo valido (inclusivo, no puede ser menor a min).</param>
        /// <returns>Entero positivo si el nmero traducido es vlido y dentro del rango, de lo contrario -1.</returns>
        public static int StrToPositiveInt(string str, int min, int max)
        {
            int l=str==null?0:str.Length;
            if (l==0)
                return -1;
            unsafe
            {
                fixed(char*p=str)
                    return StrToPositiveInt(p,l,min,max);
            }
            
        }

        #endregion

 	}
}
