using System;
using System.Globalization;
using EnPalermo.Mem;
using EnPalermo.Lit;
using EnPalermo.Arr;
using EnPalermo.Col;

namespace EnPalermo.Txt
{
    #region Enums

    /// <summary>
    /// Opciones de filtro para NormalizeString.
    /// </summary>
    [Flags()]
    public enum NormalizeFilter
    {
        /// <summary>
        /// Ninguna modificacin.
        /// </summary>
        None=0,
        /// <summary>
        /// Trim a la izquierda.
        /// </summary>
        TrimLeft=1,
        /// <summary>
        /// Trim a la derecha.
        /// </summary>
        TrimRight=2,
        /// <summary>
        /// Trim a la izquierda y derecha.
        /// </summary>
        TrimFull=TrimLeft+TrimRight,
        /// <summary>
        /// Remueve comillas en todas las anidaciones (equivale a RemoveQuotesMode.RemoveAll).
        /// Se efecta un trim derecho e izquierdo antes de intentar la remocin.
        /// </summary>
        RemoveQuotes=4,
        /// <summary>
        /// Remueve comillas en todas las anidaciones (equivale a RemoveQuotesMode.RemoveAll).
        /// Se efecta un trim derecho e izquierdo antes de intentar la remocin y al finalizar efecta un TrimFull.
        /// </summary>
        RemoveQuotesTrimFull=RemoveQuotes+TrimFull,
        /// <summary>
        /// Filtra carcteres blancos.
        /// Se actua de acuerdo a la funcin RemoveBlanks().
        /// Esta opcin no es combinable con NormalizeBlanks y toda combinacin con trim es innecesaria.
        /// </summary>
        RemoveBlanks=8,
        /// <summary>
        /// Remueve comillas en todas las anidaciones (equivale a RemoveQuotesMode.RemoveAll).
        /// Luego, elimina los carcteres blancos.
        /// </summary>
        RemoveQuotesBlanks=RemoveQuotes+RemoveBlanks,
        /// <summary>
        /// Efecta una normalizacin de carcteres blancos, convirtiendolos a espacio o un caracter especifico y eliminando duplicados.
        /// Se actua de acuerdo a la funcin NormalizeBlanks().
        /// Esta opcin no es combinable con RemoveBlanks.
        /// En caso de combinar esta opcin con las de remocin de comillas o trim, la normalizacin de blancos se efecta posteriormente. 
        /// </summary>
        NormalizeBlanks=16,
        /// <summary>
        /// Para utilizar individualmente (se ignoran todas las otras opciones). Filtra todos los carcteres no dgitos.
        /// </summary>
        FilterNoDigits=32,
        /// <summary>
        /// Para utilizar individualmente (se ignoran todas las otras opciones). Filtra todos los carcteres no dgitos en modo Hex.
        /// </summary>
        FilterNoDigitsHex=64
    }

    /// <summary>
    /// Opciones de normalizacin Case para NormalizeString.
    /// </summary>
    public enum NormalizeCase
    {
        /// <summary>
        /// Ninguna modificacin Case.
        /// </summary>
        None,
        /// <summary>
        /// A maysculas usando la tabla actual.
        /// </summary>
        ToUpperCurrent,
        /// <summary>
        /// A maysculas usando la tabla invariable.
        /// </summary>
        ToUpperInvariant,
        /// <summary>
        /// A minsculas usando la tabla actual.
        /// </summary>
        ToLowerCurrent,
        /// <summary>
        /// A minsculas usando la tabla invariable.
        /// </summary>
        ToLowerInvariant,
        /// <summary>
        /// A ttulo usando la tabla actual.
        /// </summary>
        ToProperCurrent,
        /// <summary>
        /// A ttulo usando la tabla invariable.
        /// </summary>
        ToProperInvariant
    }


    /// <summary>
    /// Grupos de carcteres.
    /// </summary>
    [Flags()]
    public enum CharGroup
    {
        /// <summary>
        /// Ninguno.
        /// </summary>
        Nothing=0,
        /// <summary>
        /// El espacio estndar (32).
        /// </summary>
        Space=1,
        /// <summary>
        /// El espacio 160.
        /// </summary>
        SpaceUpper=2,
        /// <summary>
        /// Los espacios 32 y 160.
        /// </summary>
        AllSpaces=Space+SpaceUpper,
        /// <summary>
        /// Taburador (\t).
        /// </summary>
        Tab=4,
        /// <summary>
        /// Cr y Lf (\n).
        /// </summary>
        CrLf=8,
        /// <summary>
        /// Punto ('.').
        /// </summary>
        Dot=16,
        /// <summary>
        /// Coma (',')
        /// </summary>
        Comma=32,
        /// <summary>
        /// Comillas ('"','\'','\xab','\xbb','\u2039','\u203a','\u2018','\u2019','\u201c', '\u201d')
        /// </summary>
        Quotes=64,
        /// <summary>
        /// Dgitos estndar del 0 al 9.
        /// </summary>
        Digits09=128,
        /// <summary>
        /// Otros espacios en blanco estndar.
        /// </summary>
        OtherBlanks=256,
        /// <summary>
        /// Todos los espacios en blanco estndar.
        /// </summary>
        AllBlanks=AllSpaces+Tab+CrLf+OtherBlanks,
        /// <summary>
        /// Otros Caracteres de control 0 - 31 y 127 - 159 excluyendo Tab y Crlf.
        /// </summary>
        OtherControls=512,
        /// <summary>
        /// Caracteres de control 0 - 31 y 127 - 159.
        /// </summary>
        AllControls=Tab+CrLf+OtherControls,
        /// <summary>
        /// Letras minsculas.
        /// </summary>
        LettersLower=1024,
        /// <summary>
        /// Letras maysculas.
        /// </summary>
        LettersUpper=2048,
        /// <summary>
        /// Otros carcteres considerados letras y nmeros no incluidos en LettersLower, LettersUpper y Digits09.
        /// </summary>
        OthersLettersNumbers=4096,
        /// <summary>
        /// Todos los carcteres considerados letras y nmeros.
        /// </summary>
        AllLettersNumbers=LettersLower+LettersUpper+OthersLettersNumbers+Digits09,
        /// <summary>
        /// Grupo de carcteres personalizado (ignorado si no hay array de carcteres definido).
        /// </summary>
        Custom=1<<29,
        /// <summary>
        /// Todos los carcteres que no se encuadran en ninguna de estas categoras (incluyendo carcteres personalizados).
        /// </summary>
        Others=1<<30,
    } 



    /// <summary>
    /// Procedimiento de limpieza de comillas.
    /// </summary>
    public enum RemoveQuotesMode
    {
        /// <summary>
        /// No remueve comillas.
        /// </summary>
        NoRemove=0,
        /// <summary>
        /// Remueve una sola vez, sin Trim.
        /// </summary>
        RemoveOneNoTrim=1,
        /// <summary>
        /// Remueve todas las anidaciones, sin Trim.
        /// </summary>
        RemoveAllNoTrim=2,
        /// <summary>
        /// Previo Trim derecho e izquierdo, remueve una sola vez.
        /// </summary>
        RemoveOne=5,
        /// <summary>
        /// Remueve todas las anidaciones. Se efecta un trim derecho e izquierdo cada vez.
        /// </summary>
        RemoveAll=6
    }


    /// <summary>
    /// Opciones de parrafo para depurar texto.
    /// </summary>
    public enum ParagraphFmt
    {
        /// <summary>
        /// Cada salto de lnea individual es considerado un parrafo.
        /// Por otra parte se respetan los saltos de lnea doble (a excepcin del parrafo final),
        /// eliminndose lneas vacias solo a partir de la segunda.
        /// </summary>
        Normal,

        /// <summary>
        /// Cada salto de lnea individual es considerado un parrafo.
        /// Todas las lneas vacias son eliminadas.
        /// </summary>
        NoEmptysSingle,

        /// <summary>
        /// Cada salto de lnea individual es considerado un parrafo.
        /// Todas las lneas vacias son eliminadas y luego cada
        /// parrafo es finalizado con un doble salto de lnea (a excepcin del parrafo final).
        /// </summary>
        NoEmptysDouble,

        /// <summary>
        /// Cada salto de lnea individual es considerado un wrap, por lo
        /// que se convierte a espacio.
        /// Solo los saltos de lnea doble son considerados parrafos.
        /// Todas las lneas vacias son eliminadas.
        /// </summary>
        ParagraphSingle,

        /// <summary>
        /// Cada salto de lnea individual es considerado un wrap, por lo
        /// que se convierte a espacio.
        /// Solo los saltos de lnea doble son considerados parrafos.
        /// Todas las lneas vacias son eliminadas y luego cada
        /// parrafo es finalizado con un doble salto de lnea (a excepcin del parrafo final).
        /// </summary>
        ParagraphDouble
    }

    /// <summary>
    /// Modo de reconocimiento de parrafos.
    /// </summary>
    public enum ParagraphMode
    {
        /// <summary>
        /// Sin reconocimiento de parrafos ni oraciones.
        /// </summary>
        Nothing,
        /// <summary>
        /// Sin reconocimiento de parrafos. Todo el bloque es considerado un parrafo (que puede contener una o varias oraciones).
        /// </summary>
        Sentence,
        /// <summary>
        /// Cada salto de lnea individual es considerado un parrafo (que puede contener una o varias oraciones).
        /// </summary>
        SingleLine,
        /// <summary>
        /// Solo los saltos de lnea doble son considerados parrafos (que pueden contener una o varias oraciones).
        /// </summary>
        DoubleLine,
    }

    /// <summary>
    /// Clasificacin de un TextInfo de acuerdo a su tabla de conversin May/Min.
    /// </summary>
    public enum TableCase
    {
        /// <summary>
        /// La tabla de conversin May/Min es del tipo Invariant.
        /// </summary>
        Invariant,
        /// <summary>
        /// La tabla de conversin May/Min es estndar no - turco.
        /// </summary>
        Normal,
        /// <summary>
        /// La tabla de conversin May/Min es estndar turco.
        /// </summary>
        Turkish
    }


    #endregion


    /// <summary>
    /// Clase esttica con funciones relativas al uso de cadenas y clasificacin de carcteres individuales.
    /// </summary>
    public static class Texts
    {

        #region Campos, Propiedades y Constructor esttico

        /// <summary>
        /// CultureInfo Invariant.
        /// Es el usado por las funciones de esta libreria que requieren cultura invariable.
        /// </summary>
        public static readonly System.Globalization.CultureInfo CulInfoInvariant;

        /// <summary>
        /// Utilidades de texto basado en Invariant.
        /// Es el usado por las funciones de esta libreria que requieren cultura invariable.
        /// </summary>
        public static readonly System.Globalization.TextInfo TInfoInvariant;

        /// <summary>
        /// CultureInfo Current (wrapper).
        /// </summary>
        public static System.Globalization.CultureInfo CulInfoCurrent
        {
            get
            {
                return System.Threading.Thread.CurrentThread.CurrentCulture;
            }
        }

        /// <summary>
        /// Utilidades de texto basado en CurrentCulture (wrapper).
        /// </summary>
        public static System.Globalization.TextInfo TInfoCurrent
        {
            get
            {
                return System.Threading.Thread.CurrentThread.CurrentCulture.TextInfo;
            }
        }

        /// <summary>
        /// Obtiene el tipo de tabla de conversin May/Min correspondiente a la cultura actual.
        /// </summary>
        public static TableCase TableCurrent
        {
            get
            {
                return GetTableCase(System.Threading.Thread.CurrentThread.CurrentCulture);
            }
        }

        /// <summary>
        /// Representacin de los carcteres CrLf en un entero.
        /// </summary>
        public const int CharsCrLf = 0x000A000D;

        /// <summary>
        /// Representacin de los bytes CrLf en un entero.
        /// </summary>
        public const short BytesCrLf = 0x0A0D;

        /// <summary>
        /// Categorias Unicode de los primeros 536 Chars.
        /// </summary>
        public static readonly byte[] CatsUnicode536;

        /// <summary>
        /// Minsculas invariant entre el char 192 y 536.
        /// </summary>
        private static readonly char[] iLow344;
        /// <summary>
        /// Maysculas invariant entre el char 192 y 536.
        /// </summary>
        private static readonly char[] iUpp344;
        /// <summary>
        /// Minsculas estndar no - turcas entre el char 192 y 536.
        /// </summary>
        private static readonly char[] xLow344;
        /// <summary>
        /// Maysculas estndar no - turcas entre el char 192 y 536.
        /// </summary>
        private static readonly char[] xUpp344;

        static unsafe Texts()
        {
            CulInfoInvariant = System.Globalization.CultureInfo.InvariantCulture;
            TInfoInvariant = CulInfoInvariant.TextInfo;

            CatsUnicode536 = new byte[536]{
                                              14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
                                              14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
                                              11,24,24,24,26,24,24,24,20,21,24,25,24,19,24,24,
                                              8,8,8,8,8,8,8,8,8,8,24,24,25,25,25,24,
                                              24,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                                              0,0,0,0,0,0,0,0,0,0,0,20,24,21,27,18,
                                              27,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
                                              1,1,1,1,1,1,1,1,1,1,1,20,25,21,25,14,
                                              14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
                                              14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
                                              11,24,26,26,26,26,28,28,27,28,1,22,25,19,28,27,
                                              28,25,10,10,27,1,28,24,27,10,1,23,10,10,10,24,
                                              0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                                              0,0,0,0,0,0,0,25,0,0,0,0,0,0,0,1,
                                              1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
                                              1,1,1,1,1,1,1,25,1,1,1,1,1,1,1,1,
                                              0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
                                              0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
                                              0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
                                              0,1,0,1,0,1,0,1,1,0,1,0,1,0,1,0,
                                              1,0,1,0,1,0,1,0,1,1,0,1,0,1,0,1,
                                              0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
                                              0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
                                              0,1,0,1,0,1,0,1,0,0,1,0,1,0,1,1,
                                              1,0,0,1,0,1,0,0,1,0,0,0,1,1,0,0,
                                              0,0,1,0,0,1,0,0,0,1,1,1,0,0,1,0,
                                              0,1,0,1,0,1,0,0,1,0,1,1,0,1,0,0,
                                              1,0,0,0,1,0,1,0,0,1,1,4,0,1,1,1,
                                              4,4,4,4,0,2,1,0,2,1,0,2,1,0,1,0,
                                              1,0,1,0,1,0,1,0,1,0,1,0,1,1,0,1,
                                              0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
                                              1,0,2,1,0,1,0,0,0,1,0,1,0,1,0,1,
                                              0,1,0,1,0,1,0,1,0,1,0,1,0,1,0,1,
                                              0,1,0,1,0,1,0,1};

            iLow344 = new char[344]{
                                        '\xe0','\xe1','\xe2','\xe3','\xe4','\xe5','\xe6','\xe7','\xe8','\xe9','\xea','\xeb','\xec','\xed','\xee','\xef',
                                        '\xf0','\xf1','\xf2','\xf3','\xf4','\xf5','\xf6','\xd7','\xf8','\xf9','\xfa','\xfb','\xfc','\xfd','\xfe','\xdf',
                                        '\xe0','\xe1','\xe2','\xe3','\xe4','\xe5','\xe6','\xe7','\xe8','\xe9','\xea','\xeb','\xec','\xed','\xee','\xef',
                                        '\xf0','\xf1','\xf2','\xf3','\xf4','\xf5','\xf6','\xf7','\xf8','\xf9','\xfa','\xfb','\xfc','\xfd','\xfe','\xff',
                                        '\x101','\x101','\x103','\x103','\x105','\x105','\x107','\x107','\x109','\x109','\x10b','\x10b','\x10d','\x10d','\x10f','\x10f',
                                        '\x111','\x111','\x113','\x113','\x115','\x115','\x117','\x117','\x119','\x119','\x11b','\x11b','\x11d','\x11d','\x11f','\x11f',
                                        '\x121','\x121','\x123','\x123','\x125','\x125','\x127','\x127','\x129','\x129','\x12b','\x12b','\x12d','\x12d','\x12f','\x12f',
                                        '\x130','\x131','\x133','\x133','\x135','\x135','\x137','\x137','\x138','\x13a','\x13a','\x13c','\x13c','\x13e','\x13e','\x140',
                                        '\x140','\x142','\x142','\x144','\x144','\x146','\x146','\x148','\x148','\x149','\x14b','\x14b','\x14d','\x14d','\x14f','\x14f',
                                        '\x151','\x151','\x153','\x153','\x155','\x155','\x157','\x157','\x159','\x159','\x15b','\x15b','\x15d','\x15d','\x15f','\x15f',
                                        '\x161','\x161','\x163','\x163','\x165','\x165','\x167','\x167','\x169','\x169','\x16b','\x16b','\x16d','\x16d','\x16f','\x16f',
                                        '\x171','\x171','\x173','\x173','\x175','\x175','\x177','\x177','\xff','\x17a','\x17a','\x17c','\x17c','\x17e','\x17e','\x17f',
                                        '\x180','\x253','\x183','\x183','\x185','\x185','\x254','\x188','\x188','\x256','\x257','\x18c','\x18c','\x18d','\x1dd','\x259',
                                        '\x25b','\x192','\x192','\x260','\x263','\x195','\x269','\x268','\x199','\x199','\x19a','\x19b','\x26f','\x272','\x19e','\x275',
                                        '\x1a1','\x1a1','\x1a3','\x1a3','\x1a5','\x1a5','\x1a6','\x1a8','\x1a8','\x283','\x1aa','\x1ab','\x1ad','\x1ad','\x288','\x1b0',
                                        '\x1b0','\x28a','\x28b','\x1b4','\x1b4','\x1b6','\x1b6','\x292','\x1b9','\x1b9','\x1ba','\x1bb','\x1bd','\x1bd','\x1be','\x1bf',
                                        '\x1c0','\x1c1','\x1c2','\x1c3','\x1c6','\x1c5','\x1c6','\x1c9','\x1c8','\x1c9','\x1cc','\x1cb','\x1cc','\x1ce','\x1ce','\x1d0',
                                        '\x1d0','\x1d2','\x1d2','\x1d4','\x1d4','\x1d6','\x1d6','\x1d8','\x1d8','\x1da','\x1da','\x1dc','\x1dc','\x1dd','\x1df','\x1df',
                                        '\x1e1','\x1e1','\x1e3','\x1e3','\x1e5','\x1e5','\x1e7','\x1e7','\x1e9','\x1e9','\x1eb','\x1eb','\x1ed','\x1ed','\x1ef','\x1ef',
                                        '\x1f0','\x1f3','\x1f2','\x1f3','\x1f5','\x1f5','\x1f6','\x1f7','\x1f8','\x1f9','\x1fb','\x1fb','\x1fd','\x1fd','\x1ff','\x1ff',
                                        '\x201','\x201','\x203','\x203','\x205','\x205','\x207','\x207','\x209','\x209','\x20b','\x20b','\x20d','\x20d','\x20f','\x20f',
                                        '\x211','\x211','\x213','\x213','\x215','\x215','\x217','\x217'};

            iUpp344 = new char[344]{
                                        '\xc0','\xc1','\xc2','\xc3','\xc4','\xc5','\xc6','\xc7','\xc8','\xc9','\xca','\xcb','\xcc','\xcd','\xce','\xcf',
                                        '\xd0','\xd1','\xd2','\xd3','\xd4','\xd5','\xd6','\xd7','\xd8','\xd9','\xda','\xdb','\xdc','\xdd','\xde','\xdf',
                                        '\xc0','\xc1','\xc2','\xc3','\xc4','\xc5','\xc6','\xc7','\xc8','\xc9','\xca','\xcb','\xcc','\xcd','\xce','\xcf',
                                        '\xd0','\xd1','\xd2','\xd3','\xd4','\xd5','\xd6','\xf7','\xd8','\xd9','\xda','\xdb','\xdc','\xdd','\xde','\x178',
                                        '\x100','\x100','\x102','\x102','\x104','\x104','\x106','\x106','\x108','\x108','\x10a','\x10a','\x10c','\x10c','\x10e','\x10e',
                                        '\x110','\x110','\x112','\x112','\x114','\x114','\x116','\x116','\x118','\x118','\x11a','\x11a','\x11c','\x11c','\x11e','\x11e',
                                        '\x120','\x120','\x122','\x122','\x124','\x124','\x126','\x126','\x128','\x128','\x12a','\x12a','\x12c','\x12c','\x12e','\x12e',
                                        '\x130','\x131','\x132','\x132','\x134','\x134','\x136','\x136','\x138','\x139','\x139','\x13b','\x13b','\x13d','\x13d','\x13f',
                                        '\x13f','\x141','\x141','\x143','\x143','\x145','\x145','\x147','\x147','\x149','\x14a','\x14a','\x14c','\x14c','\x14e','\x14e',
                                        '\x150','\x150','\x152','\x152','\x154','\x154','\x156','\x156','\x158','\x158','\x15a','\x15a','\x15c','\x15c','\x15e','\x15e',
                                        '\x160','\x160','\x162','\x162','\x164','\x164','\x166','\x166','\x168','\x168','\x16a','\x16a','\x16c','\x16c','\x16e','\x16e',
                                        '\x170','\x170','\x172','\x172','\x174','\x174','\x176','\x176','\x178','\x179','\x179','\x17b','\x17b','\x17d','\x17d','\x17f',
                                        '\x180','\x181','\x182','\x182','\x184','\x184','\x186','\x187','\x187','\x189','\x18a','\x18b','\x18b','\x18d','\x18e','\x18f',
                                        '\x190','\x191','\x191','\x193','\x194','\x195','\x196','\x197','\x198','\x198','\x19a','\x19b','\x19c','\x19d','\x19e','\x19f',
                                        '\x1a0','\x1a0','\x1a2','\x1a2','\x1a4','\x1a4','\x1a6','\x1a7','\x1a7','\x1a9','\x1aa','\x1ab','\x1ac','\x1ac','\x1ae','\x1af',
                                        '\x1af','\x1b1','\x1b2','\x1b3','\x1b3','\x1b5','\x1b5','\x1b7','\x1b8','\x1b8','\x1ba','\x1bb','\x1bc','\x1bc','\x1be','\x1bf',
                                        '\x1c0','\x1c1','\x1c2','\x1c3','\x1c4','\x1c5','\x1c4','\x1c7','\x1c8','\x1c7','\x1ca','\x1cb','\x1ca','\x1cd','\x1cd','\x1cf',
                                        '\x1cf','\x1d1','\x1d1','\x1d3','\x1d3','\x1d5','\x1d5','\x1d7','\x1d7','\x1d9','\x1d9','\x1db','\x1db','\x18e','\x1de','\x1de',
                                        '\x1e0','\x1e0','\x1e2','\x1e2','\x1e4','\x1e4','\x1e6','\x1e6','\x1e8','\x1e8','\x1ea','\x1ea','\x1ec','\x1ec','\x1ee','\x1ee',
                                        '\x1f0','\x1f1','\x1f2','\x1f1','\x1f4','\x1f4','\x1f6','\x1f7','\x1f8','\x1f9','\x1fa','\x1fa','\x1fc','\x1fc','\x1fe','\x1fe',
                                        '\x200','\x200','\x202','\x202','\x204','\x204','\x206','\x206','\x208','\x208','\x20a','\x20a','\x20c','\x20c','\x20e','\x20e',
                                        '\x210','\x210','\x212','\x212','\x214','\x214','\x216','\x216'};

            xLow344 = new char[344]{
                                        '\xe0','\xe1','\xe2','\xe3','\xe4','\xe5','\xe6','\xe7','\xe8','\xe9','\xea','\xeb','\xec','\xed','\xee','\xef',
                                        '\xf0','\xf1','\xf2','\xf3','\xf4','\xf5','\xf6','\xd7','\xf8','\xf9','\xfa','\xfb','\xfc','\xfd','\xfe','\xdf',
                                        '\xe0','\xe1','\xe2','\xe3','\xe4','\xe5','\xe6','\xe7','\xe8','\xe9','\xea','\xeb','\xec','\xed','\xee','\xef',
                                        '\xf0','\xf1','\xf2','\xf3','\xf4','\xf5','\xf6','\xf7','\xf8','\xf9','\xfa','\xfb','\xfc','\xfd','\xfe','\xff',
                                        '\x101','\x101','\x103','\x103','\x105','\x105','\x107','\x107','\x109','\x109','\x10b','\x10b','\x10d','\x10d','\x10f','\x10f',
                                        '\x111','\x111','\x113','\x113','\x115','\x115','\x117','\x117','\x119','\x119','\x11b','\x11b','\x11d','\x11d','\x11f','\x11f',
                                        '\x121','\x121','\x123','\x123','\x125','\x125','\x127','\x127','\x129','\x129','\x12b','\x12b','\x12d','\x12d','\x12f','\x12f',
                                        '\x69','\x131','\x133','\x133','\x135','\x135','\x137','\x137','\x138','\x13a','\x13a','\x13c','\x13c','\x13e','\x13e','\x140',
                                        '\x140','\x142','\x142','\x144','\x144','\x146','\x146','\x148','\x148','\x149','\x14b','\x14b','\x14d','\x14d','\x14f','\x14f',
                                        '\x151','\x151','\x153','\x153','\x155','\x155','\x157','\x157','\x159','\x159','\x15b','\x15b','\x15d','\x15d','\x15f','\x15f',
                                        '\x161','\x161','\x163','\x163','\x165','\x165','\x167','\x167','\x169','\x169','\x16b','\x16b','\x16d','\x16d','\x16f','\x16f',
                                        '\x171','\x171','\x173','\x173','\x175','\x175','\x177','\x177','\xff','\x17a','\x17a','\x17c','\x17c','\x17e','\x17e','\x17f',
                                        '\x180','\x253','\x183','\x183','\x185','\x185','\x254','\x188','\x188','\x256','\x257','\x18c','\x18c','\x18d','\x1dd','\x259',
                                        '\x25b','\x192','\x192','\x260','\x263','\x195','\x269','\x268','\x199','\x199','\x19a','\x19b','\x26f','\x272','\x19e','\x275',
                                        '\x1a1','\x1a1','\x1a3','\x1a3','\x1a5','\x1a5','\x1a6','\x1a8','\x1a8','\x283','\x1aa','\x1ab','\x1ad','\x1ad','\x288','\x1b0',
                                        '\x1b0','\x28a','\x28b','\x1b4','\x1b4','\x1b6','\x1b6','\x292','\x1b9','\x1b9','\x1ba','\x1bb','\x1bd','\x1bd','\x1be','\x1bf',
                                        '\x1c0','\x1c1','\x1c2','\x1c3','\x1c6','\x1c6','\x1c6','\x1c9','\x1c9','\x1c9','\x1cc','\x1cc','\x1cc','\x1ce','\x1ce','\x1d0',
                                        '\x1d0','\x1d2','\x1d2','\x1d4','\x1d4','\x1d6','\x1d6','\x1d8','\x1d8','\x1da','\x1da','\x1dc','\x1dc','\x1dd','\x1df','\x1df',
                                        '\x1e1','\x1e1','\x1e3','\x1e3','\x1e5','\x1e5','\x1e7','\x1e7','\x1e9','\x1e9','\x1eb','\x1eb','\x1ed','\x1ed','\x1ef','\x1ef',
                                        '\x1f0','\x1f3','\x1f3','\x1f3','\x1f5','\x1f5','\x1f6','\x1f7','\x1f8','\x1f9','\x1fb','\x1fb','\x1fd','\x1fd','\x1ff','\x1ff',
                                        '\x201','\x201','\x203','\x203','\x205','\x205','\x207','\x207','\x209','\x209','\x20b','\x20b','\x20d','\x20d','\x20f','\x20f',
                                        '\x211','\x211','\x213','\x213','\x215','\x215','\x217','\x217'};

            xUpp344 = new char[344]{
                                        '\xc0','\xc1','\xc2','\xc3','\xc4','\xc5','\xc6','\xc7','\xc8','\xc9','\xca','\xcb','\xcc','\xcd','\xce','\xcf',
                                        '\xd0','\xd1','\xd2','\xd3','\xd4','\xd5','\xd6','\xd7','\xd8','\xd9','\xda','\xdb','\xdc','\xdd','\xde','\xdf',
                                        '\xc0','\xc1','\xc2','\xc3','\xc4','\xc5','\xc6','\xc7','\xc8','\xc9','\xca','\xcb','\xcc','\xcd','\xce','\xcf',
                                        '\xd0','\xd1','\xd2','\xd3','\xd4','\xd5','\xd6','\xf7','\xd8','\xd9','\xda','\xdb','\xdc','\xdd','\xde','\x178',
                                        '\x100','\x100','\x102','\x102','\x104','\x104','\x106','\x106','\x108','\x108','\x10a','\x10a','\x10c','\x10c','\x10e','\x10e',
                                        '\x110','\x110','\x112','\x112','\x114','\x114','\x116','\x116','\x118','\x118','\x11a','\x11a','\x11c','\x11c','\x11e','\x11e',
                                        '\x120','\x120','\x122','\x122','\x124','\x124','\x126','\x126','\x128','\x128','\x12a','\x12a','\x12c','\x12c','\x12e','\x12e',
                                        '\x130','\x49','\x132','\x132','\x134','\x134','\x136','\x136','\x138','\x139','\x139','\x13b','\x13b','\x13d','\x13d','\x13f',
                                        '\x13f','\x141','\x141','\x143','\x143','\x145','\x145','\x147','\x147','\x149','\x14a','\x14a','\x14c','\x14c','\x14e','\x14e',
                                        '\x150','\x150','\x152','\x152','\x154','\x154','\x156','\x156','\x158','\x158','\x15a','\x15a','\x15c','\x15c','\x15e','\x15e',
                                        '\x160','\x160','\x162','\x162','\x164','\x164','\x166','\x166','\x168','\x168','\x16a','\x16a','\x16c','\x16c','\x16e','\x16e',
                                        '\x170','\x170','\x172','\x172','\x174','\x174','\x176','\x176','\x178','\x179','\x179','\x17b','\x17b','\x17d','\x17d','\x17f',
                                        '\x180','\x181','\x182','\x182','\x184','\x184','\x186','\x187','\x187','\x189','\x18a','\x18b','\x18b','\x18d','\x18e','\x18f',
                                        '\x190','\x191','\x191','\x193','\x194','\x195','\x196','\x197','\x198','\x198','\x19a','\x19b','\x19c','\x19d','\x19e','\x19f',
                                        '\x1a0','\x1a0','\x1a2','\x1a2','\x1a4','\x1a4','\x1a6','\x1a7','\x1a7','\x1a9','\x1aa','\x1ab','\x1ac','\x1ac','\x1ae','\x1af',
                                        '\x1af','\x1b1','\x1b2','\x1b3','\x1b3','\x1b5','\x1b5','\x1b7','\x1b8','\x1b8','\x1ba','\x1bb','\x1bc','\x1bc','\x1be','\x1bf',
                                        '\x1c0','\x1c1','\x1c2','\x1c3','\x1c4','\x1c4','\x1c4','\x1c7','\x1c7','\x1c7','\x1ca','\x1ca','\x1ca','\x1cd','\x1cd','\x1cf',
                                        '\x1cf','\x1d1','\x1d1','\x1d3','\x1d3','\x1d5','\x1d5','\x1d7','\x1d7','\x1d9','\x1d9','\x1db','\x1db','\x18e','\x1de','\x1de',
                                        '\x1e0','\x1e0','\x1e2','\x1e2','\x1e4','\x1e4','\x1e6','\x1e6','\x1e8','\x1e8','\x1ea','\x1ea','\x1ec','\x1ec','\x1ee','\x1ee',
                                        '\x1f0','\x1f1','\x1f1','\x1f1','\x1f4','\x1f4','\x1f6','\x1f7','\x1f8','\x1f9','\x1fa','\x1fa','\x1fc','\x1fc','\x1fe','\x1fe',
                                        '\x200','\x200','\x202','\x202','\x204','\x204','\x206','\x206','\x208','\x208','\x20a','\x20a','\x20c','\x20c','\x20e','\x20e',
                                        '\x210','\x210','\x212','\x212','\x214','\x214','\x216','\x216'};

        }

        #endregion

        #region Clasificacion de Cultura

        /// <summary>
        /// Obtiene el tipo de tabla para las conversiones May/Min.
        /// </summary>
        /// <param name="cul">Cultura.</param>
        /// <returns>Tipo de tabla.</returns>
        public static TableCase GetTableCase(CultureInfo cul)
        {
            switch (cul.LCID & 1023)
            {
                case 127:
                    return TableCase.Invariant;
                case 31:
                case 44:
                    return cul.TextInfo.EBCDICCodePage == 20905?TableCase.Turkish:TableCase.Normal;
            }
            return TableCase.Normal;
        }


        /// <summary>
        /// Obtiene el tipo de tabla para las conversiones May/Min.
        /// </summary>
        /// <param name="tex">TextInfo.</param>
        /// <returns>Tipo de tabla.</returns>
        public static TableCase GetTableCase(TextInfo tex)
        {
            switch (tex.LCID & 1023)
            {
                case 127:
                    return TableCase.Invariant;
                case 31:
                case 44:
                    return tex.EBCDICCodePage == 20905 ? TableCase.Turkish : TableCase.Normal;
            }
            return TableCase.Normal;
        }

        #endregion

        #region Clasificacin de carcteres

        /// <summary>
        /// Determina si la categora Unicode corresponde a una letra.
        /// </summary>
        /// <param name="uc">Categora unicode.</param>
        /// <returns>True si es letra.</returns>
        public static bool IsLetter(UnicodeCategory uc)
        {
            switch (uc)
            {
                case UnicodeCategory.UppercaseLetter:
                case UnicodeCategory.LowercaseLetter:
                case UnicodeCategory.TitlecaseLetter:
                case UnicodeCategory.ModifierLetter:
                case UnicodeCategory.OtherLetter:
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Determina si la categora Unicode corresponde a una letra mayscula.
        /// </summary>
        /// <param name="uc">Categora unicode.</param>
        /// <returns>True si es letra mayscula.</returns>
        public static bool IsUpper(UnicodeCategory uc)
        {
            return uc==UnicodeCategory.UppercaseLetter;
        }

        /// <summary>
        /// Determina si la categora Unicode corresponde a una letra minscula.
        /// </summary>
        /// <param name="uc">Categora unicode.</param>
        /// <returns>True si es letra minscula.</returns>
        public static bool IsLower(UnicodeCategory uc)
        {
            return uc==UnicodeCategory.LowercaseLetter;
        }

        /// <summary>
        /// Determina si la categora Unicode corresponde a un nmero.
        /// </summary>
        /// <param name="uc">Categora unicode.</param>
        /// <returns>True si es nmero.</returns>
        public static bool IsNumber(System.Globalization.UnicodeCategory uc)
        {
            switch (uc)
            {
                case UnicodeCategory.DecimalDigitNumber:
                case UnicodeCategory.LetterNumber:
                case UnicodeCategory.OtherNumber:
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Determina si la categora Unicode corresponde a un dgito.
        /// </summary>
        /// <param name="uc">Categora unicode.</param>
        /// <returns>True si es dgito.</returns>
        public static bool IsDigit(System.Globalization.UnicodeCategory uc)
        {
            return uc == UnicodeCategory.DecimalDigitNumber;
        }

        /// <summary>
        /// Determina si la categora Unicode corresponde a una letra o nmero.
        /// </summary>
        /// <param name="uc">Categora unicode.</param>
        /// <returns>True si es letra o nmero.</returns>
        public static bool IsLetterNumber(System.Globalization.UnicodeCategory uc)
        {
            switch (uc)
            {
                case UnicodeCategory.UppercaseLetter:
                case UnicodeCategory.LowercaseLetter:
                case UnicodeCategory.TitlecaseLetter:
                case UnicodeCategory.ModifierLetter:
                case UnicodeCategory.OtherLetter:
                case UnicodeCategory.DecimalDigitNumber:
                case UnicodeCategory.LetterNumber:
                case UnicodeCategory.OtherNumber:
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Determina si la categora Unicode corresponde a una letra o dgito.
        /// </summary>
        /// <param name="uc">Categora unicode.</param>
        /// <returns>True si es letra o dgito.</returns>
        public static bool IsLetterDigit(System.Globalization.UnicodeCategory uc)
        {
            switch (uc)
            {
                case UnicodeCategory.UppercaseLetter:
                case UnicodeCategory.LowercaseLetter:
                case UnicodeCategory.TitlecaseLetter:
                case UnicodeCategory.ModifierLetter:
                case UnicodeCategory.OtherLetter:
                case UnicodeCategory.DecimalDigitNumber:
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Obtiene la categora unicode del caracter.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <returns>Categora Unicode.</returns>
        public static UnicodeCategory GetUnicodeCategory(char c)
        {
            if (c < '\x218')
                return (UnicodeCategory)CatsUnicode536[c];
            return Char.GetUnicodeCategory(c);
        }

        /// <summary>
        /// Determina si el caracter es una letra.
        /// </summary>
        ///<param name="c">Caracter.</param>
        /// <returns>True si es letra.</returns>
        public static bool IsLetter(char c)
        {
            if (c <= 'z')
                return c >= 'a' || (c <= 'Z' && c >= 'A');
            if (c < '\x218')
                return IsLetter((UnicodeCategory)CatsUnicode536[c]);
            return IsLetter(Char.GetUnicodeCategory(c));
        }

        /// <summary>
        /// Determina si el caracter es una letra minscula.
        /// </summary>
        ///<param name="c">Caracter.</param>
        /// <returns>True si es letra minscula.</returns>
        public static bool IsLower(char c)
        {
            if (c < '\xAA')
                return c >= 'a' && c<='z';
            if (c < '\x218')
                return (UnicodeCategory)CatsUnicode536[c]==UnicodeCategory.LowercaseLetter;
            return Char.GetUnicodeCategory(c)==UnicodeCategory.LowercaseLetter;
        }

        /// <summary>
        /// Determina si el caracter es una letra mayscula.
        /// </summary>
        ///<param name="c">Caracter.</param>
        /// <returns>True si es letra mayscula.</returns>
        public static bool IsUpper(char c)
        {
            if (c < '\xC0')
                return c >= 'A' && c<='Z';
            if (c < '\x218')
                return (UnicodeCategory)CatsUnicode536[c]==UnicodeCategory.UppercaseLetter;
            return Char.GetUnicodeCategory(c)==UnicodeCategory.UppercaseLetter;
        }


        /// <summary>
        /// Determina si el caracter es una letra Ascii.
        /// </summary>
        ///<param name="c">Caracter.</param>
        /// <returns>True si es letra.</returns>
        public static bool IsAsciiLetter(char c)
        {
            if (c <= 'z')
                return c >= 'a' || (c <= 'Z' && c >= 'A');
            return false;
        }

        /// <summary>
        /// Determina si el caracter es un nmero.
        /// </summary>
        ///<param name="c">Caracter.</param>
        /// <returns>True si es nmero.</returns>
        public static bool IsNumber(char c)
        {
            if (c <= '9')
                return c >= '0';
            if (c < '\x218')
                return IsNumber((UnicodeCategory)CatsUnicode536[c]);
            return IsNumber(Char.GetUnicodeCategory(c));
        }

        /// <summary>
        /// Determina si el caracter es un dgito.
        /// </summary>
        ///<param name="c">Caracter.</param>
        /// <returns>True si es dgito.</returns>
        public static bool IsDigit(char c)
        {
            if (c < '\x218')
                return c <= '9' && c >= '0';
            return IsDigit(Char.GetUnicodeCategory(c));
        }

        /// <summary>
        /// Determina si el caracter es una letra o nmero.
        /// </summary>
        ///<param name="c">Caracter.</param>
        /// <returns>True si es letra o nmero.</returns>
        public static bool IsLetterNumber(char c)
        {
            if (c <= 'z')
                return c >= 'a' || (c <= 'Z' && c >= 'A') || (c <= '9' && c >= '0');
            if (c < '\x218')
                return IsLetterNumber((UnicodeCategory)CatsUnicode536[c]);
            return IsLetterNumber(Char.GetUnicodeCategory(c));
        }

        /// <summary>
        /// Determina si el caracter es una letra o dgito.
        /// </summary>
        ///<param name="c">Caracter.</param>
        /// <returns>True si es letra o dgito.</returns>
        public static bool IsLetterDigit(char c)
        {
            if (c <= 'z')
                return c >= 'a' || (c <= 'Z' && c >= 'A') || (c <= '9' && c >= '0');
            if (c < '\x218')
                return IsLetterDigit((UnicodeCategory)CatsUnicode536[c]);
            return IsLetterDigit(Char.GetUnicodeCategory(c));
        }

        /// <summary>
        /// Determina si el caracter es espacio en blanco.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro el taburador (\t).</param>
        /// <returns>True si coincide con el filtro.</returns>
        public static bool IsBlank(char c, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control)
        {
            switch (c)
            {
                case ' ':
                    return sp;
                case '\t':
                    return tab;
                case '\n':
                case '\r':
                    return crlf;
                case '\xa0':
                case '\x85':
                    return lat1;
                case '\0':
                case '\v':
                case '\f':
                    return control;
                case '\u1680':
                case '\u2000':
                case '\u2001':
                case '\u2002':
                case '\u2003':
                case '\u2004':
                case '\u2005':
                case '\u2006':
                case '\u2007':
                case '\u2008':
                case '\u2009':
                case '\u200a':
                case '\u200b':
                case '\u2028':
                case '\u2029':
                case '\u202f':
                case '\u3000':
                case '\ufeff':
                    return uni;
            }
            return false;
        }

        /// <summary>
        /// Determina si el caracter es espacio en blanco.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <returns>True si es caracter en blanco.</returns>
        public static bool IsBlank(char c)
        {
            switch (c)
            {
                case ' ':
                case '\t':
                case '\n':
                case '\r':
                case '\xa0':
                case '\x85':
                case '\0':
                case '\v':
                case '\f':
                case '\u1680':
                case '\u2000':
                case '\u2001':
                case '\u2002':
                case '\u2003':
                case '\u2004':
                case '\u2005':
                case '\u2006':
                case '\u2007':
                case '\u2008':
                case '\u2009':
                case '\u200a':
                case '\u200b':
                case '\u2028':
                case '\u2029':
                case '\u202f':
                case '\u3000':
                case '\ufeff':
                    return true;
            }
            return false;
        }


        /// <summary>
        /// Comprueba si el carcter se encuadra en el grupo especificado.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <returns>True si se encuadra.</returns>
        public unsafe static bool InGroup(char c, CharGroup group, char*pch, int lch)
        {
            if ((group&(CharGroup.Dot|CharGroup.Comma|CharGroup.Quotes|CharGroup.Digits09|CharGroup.AllBlanks))!=0)
            {
                switch (c)
                {
                    case ' ':
                        if ((group&CharGroup.Space)!=0)
                            return true;
                        break;
                    case '\t':
                        if ((group&CharGroup.Tab)!=0)
                            return true;
                        break;
                    case '\n':
                    case '\r':
                        if ((group&CharGroup.CrLf)!=0)
                            return true;
                        break;
                    case '\xa0':
                        if ((group&CharGroup.SpaceUpper)!=0)
                            return true;
                        break;
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if ((group&CharGroup.OtherBlanks)!=0)
                            return true;
                        break;
                    case '.':
                        if ((group&CharGroup.Dot)!=0)
                            return true;
                        break;
                    case ',':
                        if ((group&CharGroup.Comma)!=0)
                            return true;
                        break;
                    case '"':
                    case '\'':
                    case '\xab':
                    case '\xbb':
                    case '\u2039':
                    case '\u203a':
                    case '\u2018':
                    case '\u2019':
                    case '\u201c':
                    case '\u201d':
                        if ((group&CharGroup.Quotes)!=0)
                            return true;
                        break;
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        if ((group&CharGroup.Digits09)!=0)
                            return true;
                        break;
                }            
            }
            if ((group&CharGroup.OtherControls)!=0 && c<'\xA0' && ((c<' ' && c!='\t' && c!='\n' && c!='\r') || c>'\x7E'))
                return true;

            switch(group&(CharGroup.LettersLower|CharGroup.LettersUpper|CharGroup.OthersLettersNumbers))
            {
                case CharGroup.LettersLower:
                    if (c < '\xAA')
                    {
                        if (c >= 'a' && c<='z')
                            return true;
                    }
                    else if (c < '\x218')
                    {
                        if ((UnicodeCategory)CatsUnicode536[c]==UnicodeCategory.LowercaseLetter)
                            return true;
                    }
                    else if (Char.GetUnicodeCategory(c)==UnicodeCategory.LowercaseLetter)
                        return true;
                    break;

                case CharGroup.LettersUpper:
                    if (c < '\xC0')
                    {
                        if (c >= 'A' && c<='Z')
                            return true;
                    }
                    else if (c < '\x218')
                    {
                        if ((UnicodeCategory)CatsUnicode536[c]==UnicodeCategory.UppercaseLetter)
                            return true;
                    }
                    else if (Char.GetUnicodeCategory(c)==UnicodeCategory.UppercaseLetter)
                        return true;
                    break;
                case CharGroup.OthersLettersNumbers:
                    if (c>'\xB1')
                    {
                        switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                        {
                            case UnicodeCategory.TitlecaseLetter:
                            case UnicodeCategory.ModifierLetter:
                            case UnicodeCategory.OtherLetter:
                            case UnicodeCategory.DecimalDigitNumber:
                            case UnicodeCategory.LetterNumber:
                            case UnicodeCategory.OtherNumber:
                                return true;
                        }
                    }
                    break;
                case (CharGroup.LettersLower|CharGroup.LettersUpper):
                    if (c < '\xAA')
                    {
                        if (c<='z' && (c >= 'a' || (c<='Z' && c>='A')))
                            return true;
                    }
                    else
                    {
                        switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                        {
                            case UnicodeCategory.LowercaseLetter:
                            case UnicodeCategory.UppercaseLetter:
                                return true;
                        }
                    }
                    break;
                case (CharGroup.LettersLower|CharGroup.OthersLettersNumbers):
                    if (c<'\xB2')
                        goto case CharGroup.LettersLower;
                switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                {
                    case UnicodeCategory.LowercaseLetter:
                    case UnicodeCategory.TitlecaseLetter:
                    case UnicodeCategory.ModifierLetter:
                    case UnicodeCategory.OtherLetter:
                    case UnicodeCategory.DecimalDigitNumber:
                    case UnicodeCategory.LetterNumber:
                    case UnicodeCategory.OtherNumber:
                        return true;
                }
                    break;

                case (CharGroup.LettersUpper|CharGroup.OthersLettersNumbers):
                    if (c<'\xB2')
                        goto case CharGroup.LettersUpper;
                switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                {
                    case UnicodeCategory.UppercaseLetter:
                    case UnicodeCategory.TitlecaseLetter:
                    case UnicodeCategory.ModifierLetter:
                    case UnicodeCategory.OtherLetter:
                    case UnicodeCategory.DecimalDigitNumber:
                    case UnicodeCategory.LetterNumber:
                    case UnicodeCategory.OtherNumber:
                        return true;
                }
                    break;

                case (CharGroup.LettersLower|CharGroup.LettersUpper|CharGroup.OthersLettersNumbers):
                    if (c<'\xB2')
                        goto case (CharGroup.LettersLower|CharGroup.LettersUpper);
                switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                {
                    case UnicodeCategory.LowercaseLetter:
                    case UnicodeCategory.UppercaseLetter:
                    case UnicodeCategory.TitlecaseLetter:
                    case UnicodeCategory.ModifierLetter:
                    case UnicodeCategory.OtherLetter:
                    case UnicodeCategory.DecimalDigitNumber:
                    case UnicodeCategory.LetterNumber:
                    case UnicodeCategory.OtherNumber:
                        return true;
                }
                    break;
            }

            if (lch>0 && (group&CharGroup.Custom)!=0 && Comp.Exist(pch,lch,c))
                return true;

            if ((group&CharGroup.Others)!=0 && !InGroup(c,~group,pch,lch))
                return true;
            return false;
        }



        /// <summary>
        /// Comprueba si el carcter se encuadra en el grupo especificado.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="custom">Array de carcteres en categora custom.</param>
        /// <returns>True si se encuadra.</returns>
        public static bool InGroup(char c, CharGroup group, params char[] custom)
        {
            if ((group&(CharGroup.Dot|CharGroup.Comma|CharGroup.Quotes|CharGroup.Digits09|CharGroup.AllBlanks))!=0)
            {
                switch (c)
                {
                    case ' ':
                        if ((group&CharGroup.Space)!=0)
                            return true;
                        break;
                    case '\t':
                        if ((group&CharGroup.Tab)!=0)
                            return true;
                        break;
                    case '\n':
                    case '\r':
                        if ((group&CharGroup.CrLf)!=0)
                            return true;
                        break;
                    case '\xa0':
                        if ((group&CharGroup.SpaceUpper)!=0)
                            return true;
                        break;
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if ((group&CharGroup.OtherBlanks)!=0)
                            return true;
                        break;
                    case '.':
                        if ((group&CharGroup.Dot)!=0)
                            return true;
                        break;
                    case ',':
                        if ((group&CharGroup.Comma)!=0)
                            return true;
                        break;
                    case '"':
                    case '\'':
                    case '\xab':
                    case '\xbb':
                    case '\u2039':
                    case '\u203a':
                    case '\u2018':
                    case '\u2019':
                    case '\u201c':
                    case '\u201d':
                        if ((group&CharGroup.Quotes)!=0)
                            return true;
                        break;
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        if ((group&CharGroup.Digits09)!=0)
                            return true;
                        break;
                }            
            }
            if ((group&CharGroup.OtherControls)!=0 && c<'\xA0' && ((c<' ' && c!='\t' && c!='\n' && c!='\r') || c>'\x7E'))
                return true;

            switch(group&(CharGroup.LettersLower|CharGroup.LettersUpper|CharGroup.OthersLettersNumbers))
            {
                case CharGroup.LettersLower:
                    if (c < '\xAA')
                    {
                        if (c >= 'a' && c<='z')
                            return true;
                    }
                    else if (c < '\x218')
                    {
                        if ((UnicodeCategory)CatsUnicode536[c]==UnicodeCategory.LowercaseLetter)
                            return true;
                    }
                    else if (Char.GetUnicodeCategory(c)==UnicodeCategory.LowercaseLetter)
                        return true;
                    break;

                case CharGroup.LettersUpper:
                    if (c < '\xC0')
                    {
                        if (c >= 'A' && c<='Z')
                            return true;
                    }
                    else if (c < '\x218')
                    {
                        if ((UnicodeCategory)CatsUnicode536[c]==UnicodeCategory.UppercaseLetter)
                            return true;
                    }
                    else if (Char.GetUnicodeCategory(c)==UnicodeCategory.UppercaseLetter)
                        return true;
                    break;
                case CharGroup.OthersLettersNumbers:
                    if (c>'\xB1')
                    {
                        switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                        {
                            case UnicodeCategory.TitlecaseLetter:
                            case UnicodeCategory.ModifierLetter:
                            case UnicodeCategory.OtherLetter:
                            case UnicodeCategory.DecimalDigitNumber:
                            case UnicodeCategory.LetterNumber:
                            case UnicodeCategory.OtherNumber:
                                return true;
                        }
                    }
                    break;
                case (CharGroup.LettersLower|CharGroup.LettersUpper):
                    if (c < '\xAA')
                    {
                        if (c<='z' && (c >= 'a' || (c<='Z' && c>='A')))
                            return true;
                    }
                    else
                    {
                        switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                        {
                            case UnicodeCategory.LowercaseLetter:
                            case UnicodeCategory.UppercaseLetter:
                                return true;
                        }
                    }
                    break;
                case (CharGroup.LettersLower|CharGroup.OthersLettersNumbers):
                    if (c<'\xB2')
                        goto case CharGroup.LettersLower;
                switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                {
                    case UnicodeCategory.LowercaseLetter:
                    case UnicodeCategory.TitlecaseLetter:
                    case UnicodeCategory.ModifierLetter:
                    case UnicodeCategory.OtherLetter:
                    case UnicodeCategory.DecimalDigitNumber:
                    case UnicodeCategory.LetterNumber:
                    case UnicodeCategory.OtherNumber:
                        return true;
                }
                    break;

                case (CharGroup.LettersUpper|CharGroup.OthersLettersNumbers):
                    if (c<'\xB2')
                        goto case CharGroup.LettersUpper;
                switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                {
                    case UnicodeCategory.UppercaseLetter:
                    case UnicodeCategory.TitlecaseLetter:
                    case UnicodeCategory.ModifierLetter:
                    case UnicodeCategory.OtherLetter:
                    case UnicodeCategory.DecimalDigitNumber:
                    case UnicodeCategory.LetterNumber:
                    case UnicodeCategory.OtherNumber:
                        return true;
                }
                    break;

                case (CharGroup.LettersLower|CharGroup.LettersUpper|CharGroup.OthersLettersNumbers):
                    if (c<'\xB2')
                        goto case (CharGroup.LettersLower|CharGroup.LettersUpper);
                switch(c < '\x218'?(UnicodeCategory)CatsUnicode536[c]:Char.GetUnicodeCategory(c))
                {
                    case UnicodeCategory.LowercaseLetter:
                    case UnicodeCategory.UppercaseLetter:
                    case UnicodeCategory.TitlecaseLetter:
                    case UnicodeCategory.ModifierLetter:
                    case UnicodeCategory.OtherLetter:
                    case UnicodeCategory.DecimalDigitNumber:
                    case UnicodeCategory.LetterNumber:
                    case UnicodeCategory.OtherNumber:
                        return true;
                }
                    break;
            }

            if (custom!=null && (group&CharGroup.Custom)!=0 && ArrComp.Exist(custom,c))
                return true;

            if ((group&CharGroup.Others)!=0 && !InGroup(c,~group,custom))
                return true;
            return false;
        }
        

        #endregion
        
        #region ToUpper/ToLower Invariant


        /// <summary>
        /// Convierte a Minsculas usando cultura invariable.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <returns>Caracter en Minscula.</returns>
        public unsafe static char ToLowerInvariant(char c)
        {
            if (c < '\xC0')
            {
                if (c < '[' && c > '@')
                    return (c += ' ');
                return c;
            }
            if (c < '\x218')
                return iLow344[c - 192];
            return TInfoInvariant.ToLower(c);
        }

        /// <summary>
        /// Convierte a Maysculas usando cultura invariable.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <returns>Caracter en Mayscula.</returns>
        public unsafe static char ToUpperInvariant(char c)
        {
            if (c < '\xC0')
            {
                if (c < '{' && c > '\x60')
                    return (c -= ' ');
                return c;
            }
            if (c < '\x218')
                return iUpp344[c - 192];
            return TInfoInvariant.ToUpper(c);
        }

        /// <summary>
        /// Convierte a Minsculas usando cultura invariable.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si el puntero fue modificado.</returns>
        public unsafe static bool ToLowerInvariant(char* p, int l)
        {

            bool ret = false;
            char c;
            if (l > 0)
            {
                int a = 0;
                do
                {
                    c = p[a];
                    if (c < '\xC0')
                    {
                        if (c < '[' && c > '@')
                        {
                            p[a] = (c += ' ');
                            if (!ret)
                                ret = true;
                        }
                    }
                    else
                    {
                        if (c < '\x218')
                            c = iLow344[c - 192];
                        else
                            c = TInfoInvariant.ToLower(c);
                        if (c != p[a])
                        {
                            p[a] = c;
                            if (!ret)
                                ret = true;
                        }
                    }
                } while (++a != l);
            }
            return ret;
        }

        /// <summary>
        /// Convierte a Maysculas usando cultura invariable.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si el puntero fue modificado.</returns>
        public unsafe static bool ToUpperInvariant(char* p, int l)
        {
            bool ret = false;
            char c;
            if (l > 0)
            {
                int a = 0;
                do
                {
                    c = p[a];
                    if (c < '\xC0')
                    {
                        if (c < '{' && c > '\x60')
                        {
                            p[a] = (c -= ' ');
                            if (!ret)
                                ret = true;
                        }
                    }
                    else
                    {
                        if (c < '\x218')
                            c = iUpp344[c - 192];
                        else
                            c = TInfoInvariant.ToUpper(c);
                        if (c != p[a])
                        {
                            p[a] = c;
                            if (!ret)
                                ret = true;
                        }
                    }
                } while (++a != l);
            }
            return ret;
        }


        /// <summary>
        /// Copia convirtiendo a Minsculas usando cultura invariable.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="l">Largo.</param>
        public unsafe static void ToLowerInvariant(char* po,char*pd, int l)
        {
            if (l > 0)
            {
                int a = 0;
                do
                {
                    if (po[a] < '\xC0')
                    {
                        if (po[a] < '[' && po[a] > '@')
                            pd[a] = (char)(po[a] + 32);
                        else
                            pd[a] = po[a];
                    }
                    else if (po[a] < '\x218')
                        pd[a] = iLow344[po[a] - 192];
                    else 
                        pd[a] = TInfoInvariant.ToLower(po[a]);

                } while (++a != l);
            }
        }

        /// <summary>
        /// Copia convirtiendo a Maysculas usando cultura invariable.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="l">Largo.</param>
        public unsafe static void ToUpperInvariant(char* po, char* pd, int l)
        {
            if (l > 0)
            {
                int a = 0;
                do
                {
                    if (po[a] < '\xC0')
                    {
                        if (po[a] < '{' && po[a] > '\x60')
                            pd[a] = (char)(po[a] - 32);
                        else
                            pd[a] = po[a];
                    }
                    else if (po[a] < '\x218')
                        pd[a] = iUpp344[po[a] - 192];
                    else
                        pd[a] = TInfoInvariant.ToUpper(po[a]);

                } while (++a != l);
            }
        }

        /// <summary>
        /// Convierte a Minsculas usando cultura invariable.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>Cadena mutable en minsculas o la misma instancia de cadena si no ocurrieron modificaciones.</returns>
        public unsafe static string ToLowerInvariant(string str)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            string ret = Mutables.Create(str);
            fixed (char* p = ret)
            {
                if (ToLowerInvariant(p, l))
                    return ret;
            }
            return str;
        }

        /// <summary>
        /// Convierte a Maysculas usando cultura invariable.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>Cadena mutable en maysculas o la misma instancia de cadena si no ocurrieron modificaciones.</returns>
        public unsafe static string ToUpperInvariant(string str)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            string ret = Mutables.Create(str);
            fixed (char* p = ret)
            {
                if (ToUpperInvariant(p, l))
                    return ret;
            }
            return str;
        }

        #endregion

        #region ToUpper/ToLower basadas en tabla


        /// <summary>
        /// Convierte a Minsculas usando el tipo de tabla especificado.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <returns>Caracter en Minscula.</returns>
        public unsafe static char ToLower(char c,TableCase t)
        {
            if (c < '\xC0')
            {
                if (c < '[' && c > '@')
                {
                    if (c != 'I')
                        return (c += ' ');
                    if (t == TableCase.Turkish)
                        return '\x131';
                    return 'i';
                }
                return c;
            }
            if (c < '\x218')
            {
                if (t != TableCase.Invariant)
                    return xLow344[c - 192];
                return iLow344[c - 192];
            }
            if (c > '\u03d4' || c < '\u03d2' || t == TableCase.Invariant)
            {
                return TInfoInvariant.ToLower(c);
            }
            if (c == '\u03d4')
            {
                return '\u03cb';
            }
            if (c == '\u03d3')
            {
                return '\u03cd';
            }
            return '\u03c5';
        }

        /// <summary>
        /// Convierte a Maysculas usando el tipo de tabla especificado.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <returns>Caracter en Mayscula.</returns>
        public unsafe static char ToUpper(char c, TableCase t)
        {
            if (c < '\xC0')
            {
                if (c < '{' && c > '\x60')
                {
                    if (c != 'i')
                        return (c -= ' ');
                    if (t == TableCase.Turkish)
                        return '\x130';
                    return 'I';
                }
                return c;
            }
            if (c < '\x218')
            {
                if (t != TableCase.Invariant)
                    return xUpp344[c - 192];
                return iUpp344[c - 192];
            }
            if (t != TableCase.Invariant)
            {
                switch (c)
                {
                    case '\x390':
                        return '\x3aa';
                    case '\x3b0':
                        return '\x3ab';
                    case '\x3d0':
                        return '\x392';
                    case '\x3d1':
                        return '\x398';
                    case '\x3d5':
                        return '\x3a6';
                    case '\x3d6':
                        return '\x3a0';
                    case '\x3f0':
                        return '\x39a';
                    case '\x3f1':
                        return '\x3a1';
                }
            }
            return TInfoInvariant.ToUpper(c);
        }

        /// <summary>
        /// Convierte a Minsculas usando el tipo de tabla especificado.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <returns>True si el puntero fue modificado.</returns>
        public unsafe static bool ToLower(char* p, int l, TableCase t)
        {
            if (t == TableCase.Invariant)
                return ToLowerInvariant(p, l);
            bool ret = false;

            char c;
            if (l > 0)
            {
                int a = 0;
                do
                {
                    c = p[a];
                    if (c < '\xC0')
                    {
                        if (c < '[' && c > '@')
                        {
                            if (c != 'I' || t != TableCase.Turkish)
                                p[a] = (c += ' ');
                            else
                                p[a] = '\x131';
                            if (!ret)
                                ret = true;
                        }
                    }
                    else
                    {
                        if (c < '\x218')
                            c = xLow344[c - 192];
                        else if (c > '\u03d4' || c < '\u03d2')
                            c = TInfoInvariant.ToLower(c);
                        else if (c == '\u03d4')
                            c = '\u03cb';
                        else if (c == '\u03d3')
                            c = '\u03cd';
                        else
                            c = '\u03c5';
                        if (c != p[a])
                        {
                            p[a] = c;
                            if (!ret)
                                ret = true;
                        }
                    }
                } while (++a != l);
            }
            return ret;
        }

        /// <summary>
        /// Copia convirtiendo a Maysculas usando el tipo de tabla especificado.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        public unsafe static void ToUpper(char* po,char* pd, int l, TableCase t)
        {
            if (t != TableCase.Invariant)
            {
                if (l > 0)
                {
                    int a = 0;
                    do
                    {
                        if (po[a] < '\xC0')
                        {
                            if (po[a] < '{' && po[a] > '\x60')
                            {
                                if (po[a] != 'i' || t != TableCase.Turkish)
                                    pd[a] = (char)(po[a] - 32);
                                else
                                    pd[a] = '\x130';
                            }
                            else
                                pd[a] = po[a];
                        }
                        else if (po[a] < '\x218')
                            pd[a] = xUpp344[po[a] - 192];
                        else
                        {
                            switch (po[a])
                            {
                                case '\x390':
                                    pd[a] = '\x3aa';
                                    break;
                                case '\x3b0':
                                    pd[a] = '\x3ab';
                                    break;
                                case '\x3d0':
                                    pd[a] = '\x392';
                                    break;
                                case '\x3d1':
                                    pd[a] = '\x398';
                                    break;
                                case '\x3d5':
                                    pd[a] = '\x3a6';
                                    break;
                                case '\x3d6':
                                    pd[a] = '\x3a0';
                                    break;
                                case '\x3f0':
                                    pd[a] = '\x39a';
                                    break;
                                case '\x3f1':
                                    pd[a] = '\x3a1';
                                    break;
                                default:
                                    pd[a] = TInfoInvariant.ToUpper(po[a]);
                                    break;
                            }
                        }
                    } while (++a != l);
                }
            }
            else ToUpperInvariant(po, pd, l);
        }

        /// <summary>
        /// Copia convirtiendo a Minsculas usando el tipo de tabla especificado.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        public unsafe static void ToLower(char* po, char* pd, int l, TableCase t)
        {
            if (t != TableCase.Invariant)
            {
                if (l > 0)
                {
                    int a = 0;
                    do
                    {
                        if (po[a] < '\xC0')
                        {
                            if (po[a] < '[' && po[a] > '@')
                            {
                                if (po[a] != 'I' || t != TableCase.Turkish)
                                    pd[a] = (char)(po[a] + 32);
                                else
                                    pd[a] = '\x131';
                            }
                            else
                                pd[a] = po[a];
                        }
                        else if (po[a] < '\x218')
                            pd[a] = xLow344[po[a] - 192];
                        else if (po[a] > '\u03d4' || po[a] < '\u03d2')
                            pd[a] = TInfoInvariant.ToLower(po[a]);
                        else if (po[a] == '\u03d4')
                            pd[a] = '\u03cb';
                        else if (po[a] == '\u03d3')
                            pd[a] = '\u03cd';
                        else
                            pd[a] = '\u03c5';
                    } while (++a != l);
                }
            }
            else
                ToLowerInvariant(po, pd, l);
        }

        /// <summary>
        /// Convierte a Maysculas usando el tipo de tabla especificado.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <returns>True si el puntero fue modificado.</returns>
        public unsafe static bool ToUpper(char* p, int l, TableCase t)
        {
            if (t == TableCase.Invariant)
                return ToUpperInvariant(p, l);
            bool ret = false;

            char c;
            if (l > 0)
            {
                int a = 0;
                do
                {
                    c = p[a];
                    if (c < '\xC0')
                    {
                        if (c < '{' && c > '\x60')
                        {
                            if (c != 'i' || t != TableCase.Turkish)
                                p[a] = (c -= ' ');
                            else
                                p[a] = '\x130';
                            if (!ret)
                                ret = true;
                        }
                    }
                    else
                    {
                        if (c < '\x218')
                            c = xUpp344[c - 192];
                        else
                        {
                            switch (c)
                            {
                                case '\x390':
                                    c= '\x3aa';
                                    break;
                                case '\x3b0':
                                    c = '\x3ab';
                                    break;
                                case '\x3d0':
                                    c = '\x392';
                                    break;
                                case '\x3d1':
                                    c = '\x398';
                                    break;
                                case '\x3d5':
                                    c = '\x3a6';
                                    break;
                                case '\x3d6':
                                    c = '\x3a0';
                                    break;
                                case '\x3f0':
                                    c = '\x39a';
                                    break;
                                case '\x3f1':
                                    c = '\x3a1';
                                    break;
                                default:
                                    c = TInfoInvariant.ToUpper(c);
                                    break;
                            }
                        }
                        if (c != p[a])
                        {
                            p[a] = c;
                            if (!ret)
                                ret = true;
                        }
                    }
                } while (++a != l);
            }
            return ret;
        }

        /// <summary>
        /// Convierte a Minsculas usando el tipo de tabla especificado.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <returns>Cadena mutable en minsculas o la misma instancia de cadena si no ocurrieron modificaciones.</returns>
        public unsafe static string ToLower(string str,TableCase t)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            string ret = Mutables.Create(str);
            fixed (char* p = ret)
            {
                if (ToLower(p, l, t))
                    return ret;
            }
            return str;
        }

        /// <summary>
        /// Convierte a Maysculas usando el tipo de tabla especificado.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <returns>Cadena mutable en maysculas o la misma instancia de cadena si no ocurrieron modificaciones.</returns>
        public unsafe static string ToUpper(string str, TableCase t)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            string ret = Mutables.Create(str);
            fixed (char* p = ret)
            {
                if (ToUpper(p, l, t))
                    return ret;
            }
            return str;
        }

 
        #endregion

        #region ToProper

        /// <summary>
        /// Convierte la cadena de entrada en texto titulado.
        /// </summary>
        /// <param name="str">
        /// Cadena.
        /// </param>
        /// <param name="t">Tabla de conversin May/Min. 
        /// Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.</param>
        /// <returns>Cadena con texto titulado. Si no existen modificaciones retornar la misma instancia de cadena.</returns>
        public static string ToProper(string str, TableCase t)
        {
            
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            string str2 = Mutables.Create(str);
            unsafe
            {
                fixed (char* p = str2)
                    return ToProper(p, l, t) ? str2 : str;
            }
        }

        /// <summary>
        /// Convierte la cadena en texto titulado.
        /// </summary>
        /// <param name="p">
        /// Cadena.
        /// </param>
        /// <param name="l">Largo.</param>
        /// <param name="t">Tabla de conversin May/Min. 
        /// Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.</param>
        /// <returns>True si la cadena fue modificada.</returns>
        public unsafe static bool ToProper(char* p, int l, TableCase t)
        {
            if (l<1)
                return false;
            bool mod=false;
            PChar pc=new PChar(p,l), pc2;
            while((pc2=pc.FowardWord(true)).Length!=0)
            {
                char c=*pc2.Pointer;
                if ((c <= 'Z' && c >= 'A') || (c > 127 && GetUnicodeCategory(c) == System.Globalization.UnicodeCategory.UppercaseLetter))
                {
                    if (pc2.Length!=1)
                    {
                        if (ToLower(pc2.Pointer+1,pc2.Length-1,t) && !mod)
                            mod=true;
                    }
                    continue;
                }
                
                *pc2.Pointer=ToUpper(c,t);
                if (!mod && c!=*pc2.Pointer)
                    mod=true;
                if (pc2.Length!=1 && ToLower(pc2.Pointer+1,pc2.Length-1,t) && !mod)
                    mod=true;
            }
            return mod;
        }

        /// <summary>
        /// Convierte la cadena de entrada en texto titulado.
        /// </summary>
        /// <param name="str">
        /// Cadena.
        /// </param>
        /// <param name="mPar">
        /// Opciones para la deteccin de prrafo y oraciones.
        /// - Nothing: Efectuar la conversion en todas las palabras y breakFirstUppSecLow se aplica una sola vez
        /// a toda la cadena.
        /// - Sentence: Solo convierte la primer palabra de cada oracin. Toda la cadena es considerada
        /// un parrafo nico por lo que no se toman en cuenta los saltos de lnea como cambio de oracin.
        /// SingleLine y DoubleLine: Solo convierte la primer palabra de cada oracin. Los saltos de lnea (simples o dobles)
        /// son considerados cambios de oracin.
        /// </param>
        /// <param name="t">Tabla de conversin May/Min. 
        /// Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.</param>
        /// <param name="breakFirstUppSecLow">
        /// Si la primer letra de una oracin es mayscula y la segunda es minscula,
        /// no efectuar modificacin alguna a esa oracin (o a la cadena completa si mPar es Nothing).
        /// </param>
        /// <returns>Cadena con texto titulado. Si no existen modificaciones retornar la misma instancia de cadena.</returns>
        public static string ToProper(string str, ParagraphMode mPar, bool breakFirstUppSecLow, TableCase t)
        {
            
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            string str2 = Mutables.Create(str);
            unsafe
            {
                fixed (char* p = str2)
                    return ToProper(p, l, mPar, breakFirstUppSecLow, t) ? str2 : str;
            }
        }

        /// <summary>
        /// Convierte la cadena en texto titulado.
        /// </summary>
        /// <param name="p">
        /// Cadena.
        /// </param>
        /// <param name="l">Largo.</param>
        /// <param name="mPar">
        /// Opciones para la deteccin de prrafo y oraciones.
        /// - Nothing: Efectuar la conversion en todas las palabras y breakFirstUppSecLow se aplica una sola vez
        /// a toda la cadena.
        /// - Sentence: Solo convierte la primer palabra de cada oracin. Toda la cadena es considerada
        /// un parrafo nico por lo que no se toman en cuenta los saltos de lnea como cambio de oracin.
        /// SingleLine y DoubleLine: Solo convierte la primer palabra de cada oracin. Los saltos de lnea (simples o dobles)
        /// son considerados cambios de oracin.
        /// </param>
        /// <param name="t">Tabla de conversin May/Min. 
        /// Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.</param>
        /// <param name="breakFirstUppSecLow">
        /// Si la primer letra de una oracin es mayscula y la segunda es minscula,
        /// no efectuar modificacin alguna a esa oracin (o a la cadena completa si mPar es Nothing).
        /// </param>
        /// <returns>True si la cadena fue modificada.</returns>
        public unsafe static bool ToProper(char* p, int l, ParagraphMode mPar, bool breakFirstUppSecLow, TableCase t)
        {
            if (l<1)
                return false;
            bool mod=false;
            PChar pc=new PChar(p,l), pc2;
            switch(mPar)
            {
                case ParagraphMode.Sentence:
                    properParr(ref mod,pc,breakFirstUppSecLow,t);
                    return mod;
                case ParagraphMode.SingleLine:
                    while((pc2=pc.FowardParagraph(false)).Length!=0)
                        properParr(ref mod,pc2,breakFirstUppSecLow,t);
                    return mod;
                case ParagraphMode.DoubleLine:
                    while((pc2=pc.FowardParagraph(true)).Length!=0)
                        properParr(ref mod,pc2,breakFirstUppSecLow,t);
                    return mod;
            }
            while((pc2=pc.FowardWord(true)).Length!=0)
            {
                char c=*pc2.Pointer;
                if ((c <= 'Z' && c >= 'A') || (c > 127 && GetUnicodeCategory(c) == System.Globalization.UnicodeCategory.UppercaseLetter))
                {
                    if (pc2.Length!=1)
                    {
                        if (breakFirstUppSecLow)
                        {
                            c=pc2.Pointer[1];
                            if ((c <= 'z' && c >= 'a') || (c > 127 && GetUnicodeCategory(c) == System.Globalization.UnicodeCategory.LowercaseLetter))
                                return false;
                            breakFirstUppSecLow=false;
                        }
                        if (ToLower(pc2.Pointer+1,pc2.Length-1,t) && !mod)
                            mod=true;
                    }
                    else if (breakFirstUppSecLow)
                        breakFirstUppSecLow=false;
                    continue;
                }
                
                if (breakFirstUppSecLow)
                    breakFirstUppSecLow=false;

                *pc2.Pointer=ToUpper(c,t);
                if (!mod && c!=*pc2.Pointer)
                    mod=true;
                if (pc2.Length!=1 && ToLower(pc2.Pointer+1,pc2.Length-1,t) && !mod)
                    mod=true;
            }
            return mod;
        }

        private unsafe static void properParr(ref bool mod, PChar pc, bool breakFirstUppSecLow, TableCase t)
        {
            int a=0;
            char c;
            UnicodeCategory uc;
            do
            {
            otra:
                c = pc.Pointer[a];
                if (c < 128)
                {
                    switch (c)
                    {
                        case '.':
                        case ':':
                        case '!':
                        case '?':
                        case '(':
                        case '[':
                        case '{':
                            break;
                        default:
                            continue;
                    }
                }
                else if ((uc = GetUnicodeCategory(c)) != UnicodeCategory.OtherPunctuation && uc != UnicodeCategory.OpenPunctuation && uc != UnicodeCategory.InitialQuotePunctuation)
                    continue;
                if (a!=0)
                    properSent(ref mod,pc.Pointer,a,breakFirstUppSecLow,t);
                pc.Length-=++a;
                if (pc.Length==0)
                    return;
                pc.Pointer+=a;
                a=0;
                goto otra;

            }while(++a!=pc.Length);
            properSent(ref mod,pc.Pointer,a,breakFirstUppSecLow,t);
        }

       
        private unsafe static void properSent(ref bool mod, char*p, int l, bool breakFirstUppSecLow, TableCase t)
        {
            int ubi=TxtComp.IndexOfLetter(p,l);
            if (ubi!=-1)
            {
                char c=p[ubi];
                if ((c <= 'Z' && c >= 'A') || (c > 127 && GetUnicodeCategory(c) == System.Globalization.UnicodeCategory.UppercaseLetter))
                {
                    if (++ubi!=l)
                    {
                        if (breakFirstUppSecLow)
                        {
                            c=p[ubi];
                            if ((c <= 'z' && c >= 'a') || (c > 127 && GetUnicodeCategory(c) == System.Globalization.UnicodeCategory.LowercaseLetter))
                                return;
                        }
                        if (ToLower(p+ubi,l-ubi,t) && !mod)
                            mod=true;
                    }
                    return;
                }
                p[ubi]=ToUpper(c,t);
                if (!mod && c!=p[ubi])
                    mod=true;
                if (++ubi!=l && ToLower(p+ubi,l-ubi,t) && !mod)
                    mod=true;
            }
        }
        
        #endregion

        #region Trim

        /// <summary>
        /// Efectua Trim a una cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="cTrim">Array de parametro con carcteres de filtro.</param>
        /// <param name="right">Efectua Trim a la derecha.</param>
        /// <param name="left">Efectua Trim a la izquierda.</param>
        /// <returns>Cadena modificada, vacia o la misma instancia de cadena en caso de no efectuar modificaciones.</returns>
        public static string Trim(string str, bool left, bool right, params char[] cTrim)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            int lTrim = cTrim.Length;
            if (lTrim != 0)
            {
                unsafe
                {
                    fixed (char* p = str, pTrim = cTrim)
                    {
                        Mem.PChar pchar = new Mem.PChar(p, l, pTrim, lTrim, left, right);
                        if (l!=pchar.Length)
                            return pchar.ToString();
                    }
                }
            }
            return str;
        }

        /// <summary>
        /// Efectua Trim a una cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="custom">Array de carcteres en categora custom.</param>
        /// <param name="right">Efectua Trim a la derecha.</param>
        /// <param name="left">Efectua Trim a la izquierda.</param>
        /// <returns>Cadena modificada, vacia o la misma instancia de cadena en caso de no efectuar modificaciones.</returns>
        public static string Trim(string str, bool left, bool right, CharGroup group, params char[] custom)
        {
            int l=str==null?0:str.Length;
            if (l != 0)
            {
                int lch = custom==null?0:custom.Length;
                unsafe
                {
                    fixed (char* p = str)
                    {
                        Mem.PChar pchar;
                        if (lch != 0)
                            fixed (char* pch = custom)
                                pchar = new Mem.PChar(p, l, group, pch, lch, left, right);
                        else
                            pchar = new Mem.PChar(p, l, group, null, 0, left, right);
                    
                        if (l!=pchar.Length)
                            return pchar.ToString();
                    }
                }
            }
            return str;
        }


        /// <summary>
        /// Efectua Trim a una cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="right">Efectua Trim a la derecha.</param>
        /// <param name="left">Efectua Trim a la izquierda.</param>
        /// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro el taburador (\t).</param>
        /// <returns>Cadena modificada, vacia o la misma instancia de cadena en caso de no efectuar modificaciones.</returns>
        public static string Trim(string str, bool left, bool right, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            unsafe
            {
                fixed (char* p = str)
                {
                    Mem.PChar pchar = new Mem.PChar(p, l, left, right, sp, lat1, uni, tab, crlf, control);
                    if (l!=pchar.Length)
                        return pchar.ToString();
                    return str;
                }
            }
         }



        /// <summary>
        /// Efectua Trim con el filtro de todos los carcteres en blanco a una cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="right">Efectua Trim a la derecha.</param>
        /// <param name="left">Efectua Trim a la izquierda.</param>
        /// <returns>Cadena modificada, vacia o la misma instancia de cadena en caso de no efectuar modificaciones.</returns>
        public static string Trim(string str, bool left, bool right)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            unsafe
            {
                fixed (char* p = str)
                {
                    Mem.PChar pchar = new Mem.PChar(p, l, left, right);
                    if (l!=pchar.Length)
                        return pchar.ToString();
                    return str;
                }
            }
        }

        /// <summary>
        /// Efectua Trim de left y right con el filtro de todos los carcteres en blanco a una cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>Cadena modificada, vacia o la misma instancia de cadena en caso de no efectuar modificaciones.</returns>
        public static string Trim(string str)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return str;
            unsafe
            {
                fixed (char* p = str)
                {
                    Mem.PChar pchar = new Mem.PChar(p, l, true, true);
                    if (l!=pchar.Length)
                        return pchar.ToString();
                    return str;

                }
            }
        }

        #endregion

        #region Join


        /// <summary>
        /// Similar a String.Join, une arrays de cadenas.
        /// </summary>
        /// <param name="sep">Caracter Separador.</param>
        /// <param name="strings">Representaciones de cadena a unir.</param>
        /// <param name="ignoreEmpty">Ignora cadenas vacias.</param>
        /// <param name="sepAtEnd">
        /// Inserta un ltimo separador al final. Este separador solo es insertado si la cadena final ya tiene
        /// algun contenido.
        /// </param>
        /// <returns>
        /// Cadena con los elementos seleccionados del Array.
        /// </returns>
        public static string Join<T>(char sep, bool ignoreEmpty, bool sepAtEnd, params T[] strings)
        {
            return Join(sep, ignoreEmpty, sepAtEnd, ArrGeneral.Strings(strings));
        }

        /// <summary>
        /// Similar a String.Join, une arrays de cadenas.
        /// </summary>
        /// <param name="sep">Caracter Separador.</param>
        /// <param name="strings">Representaciones de cadena a unir.</param>
        /// <param name="ignoreEmpty">Ignora cadenas vacias.</param>
        /// <param name="sepAtEnd">
        /// Inserta un ltimo separador al final. Este separador solo es insertado si la cadena final ya tiene
        /// algun contenido.
        /// </param>
        /// <param name="f">Cadena de formato. Null para usar el formato por defecto.</param>
        /// <param name="p">Proveedor de formato.</param>
        /// <returns>
        /// Cadena con los elementos seleccionados del Array.
        /// </returns>
        public static string Join<T>(char sep, bool ignoreEmpty, bool sepAtEnd, string f, IFormatProvider p, params T[] strings) where T : IFormattable
        {
            return Join(sep, ignoreEmpty, sepAtEnd, ArrGeneral.Strings(strings, f, p));
        }

        /// <summary>
        /// Similar a String.Join, une arrays de cadenas.
        /// </summary>
        /// <param name="crlf">
        /// Si true, usar la cadena '\r\n' como separador.
        /// Si false, no usar ningun separador.
        /// </param>
        /// <param name="strings">Representaciones de cadena a unir.</param>
        /// <param name="ignoreEmpty">
        /// Ignora cadenas vacias. Solo tomado en cuenta si crlf es true.
        /// </param>
        /// <param name="sepAtEnd">
        /// Inserta un ltimo crlf al final. Este separador solo es insertado si la cadena final ya tiene
        /// algun contenido. Solo tomado en cuenta si crlf es true.
        /// </param>
        /// <returns>
        /// Cadena con los elementos seleccionados del Array.
        /// </returns>
        public static string Join<T>(bool crlf, bool ignoreEmpty, bool sepAtEnd, params T[] strings)
        {
            return Join(crlf, ignoreEmpty, sepAtEnd, ArrGeneral.Strings(strings));
        }

        /// <summary>
        /// Similar a String.Join, une arrays de cadenas.
        /// </summary>
        /// <param name="crlf">
        /// Si true, usar la cadena '\r\n' como separador.
        /// Si false, no usar ningun separador.
        /// </param>
        /// <param name="strings">Representaciones de cadena a unir.</param>
        /// <param name="ignoreEmpty">
        /// Ignora cadenas vacias. Solo tomado en cuenta si crlf es true.
        /// </param>
        /// <param name="f">Cadena de formato. Null para usar el formato por defecto.</param>
        /// <param name="p">Proveedor de formato.</param>
        /// <param name="sepAtEnd">
        /// Inserta un ltimo crlf al final. Este separador solo es insertado si la cadena final ya tiene
        /// algun contenido. Solo tomado en cuenta si crlf es true.
        /// </param>
        /// <returns>
        /// Cadena con los elementos seleccionados del Array.
        /// </returns>
        public static string Join<T>(bool crlf, bool ignoreEmpty, bool sepAtEnd, string f, IFormatProvider p, params T[] strings) where T : IFormattable
        {
            return Join(crlf, ignoreEmpty, sepAtEnd, ArrGeneral.Strings(strings, f, p));
        }


        /// <summary>
        /// Similar a String.Join, une arrays de cadenas.
        /// </summary>
        /// <param name="sep">Caracter Separador.</param>
        /// <param name="strings">Cadenas a unir.</param>
        /// <param name="ignoreEmpty">Ignora cadenas vacias.</param>
        /// <param name="sepAtEnd">
        /// Inserta un ltimo separador al final. Este separador solo es insertado si la cadena final ya tiene
        /// algun contenido.
        /// </param>
        /// <returns>
        /// Cadena con los elementos seleccionados del Array.
        /// </returns>
        public static string Join(char sep, bool ignoreEmpty, bool sepAtEnd, params string[] strings)
        {
            int la = strings.Length;
            if (la == 0)
                return Others.Empty;
            int lTot = 0;
            int l;
            string act;
            int a = 0;
            Mem.Heap heap;
            unsafe
            {
                int* iarr;
                if (la < General.MaxStackInts && &la>General.MinimumStack)
                {
                    int* ptmp = stackalloc int[la];
                    iarr = ptmp;
                    heap = null;
                }
                else
                {
                    heap = new Mem.Heap(la << 2, false);
                    iarr = (int*)heap.Pointer;
                }
                // determina largo...
                do
                {
                    act = strings[a];
                    l = act == null ? 0 : act.Length;
                    iarr[a] = l;
                    if (l != 0)
                        lTot += l + 1;
                    else if (!ignoreEmpty)
                        lTot++;
                } while (++a != la);
                if (!sepAtEnd)
                    lTot--;
                if (lTot < 1)
                {
                    if (heap != null)
                        heap.Dispose();
                    return Others.Empty;
                }
                // genera cadena...
                string ret = Mutables.Create(lTot);
                fixed (char* pret = ret)
                {
                    a = 0;
                    lTot = 0;
                    do
                    {
                        l = iarr[a];
                        if (l != 0)
                        {
                            fixed (char* pact = strings[a])
                                General.Copy(pact, pret + lTot, l);
                            lTot += l;
                            pret[lTot++] = sep;
                        }
                        else if (!ignoreEmpty)
                            pret[lTot++] = sep;
                    } while (++a != la);
                    if (!sepAtEnd)
                    {
                        pret[lTot - 1] = '\0';
                    }
                    if (heap != null)
                        heap.Dispose();
                    return ret;
                }
            }
        }


        /// <summary>
        /// Similar a String.Join, une arrays de cadenas.
        /// </summary>
        /// <param name="crlf">
        /// Si true, usar la cadena '\r\n' como separador.
        /// Si false, no usar ningun separador.
        /// </param>
        /// <param name="strings">Cadenas a unir.</param>
        /// <param name="ignoreEmpty">
        /// Ignora cadenas vacias. Solo tomado en cuenta si crlf es true.
        /// </param>
        /// <param name="sepAtEnd">
        /// Inserta un ltimo crlf al final. Este separador solo es insertado si la cadena final ya tiene
        /// algun contenido. Solo tomado en cuenta si crlf es true.
        /// </param>
        /// <returns>
        /// Cadena con los elementos seleccionados del Array.
        /// </returns>
        public static string Join(bool crlf, bool ignoreEmpty, bool sepAtEnd, params string[] strings)
        {
            int la = strings.Length;
            if (la == 0)
                return Others.Empty;
            int lTot = 0;
            int l;
            string act;
            int a = 0;
            Mem.Heap heap;
            unsafe
            {
                int* iarr;
                if (la < General.MaxStackInts && &la>General.MinimumStack)
                {
                    int* ptmp = stackalloc int[la];
                    iarr = ptmp;
                    heap = null;
                }
                else
                {
                    heap = new Mem.Heap(la << 2, false);
                    iarr = (int*)heap.Pointer;
                }
                // determina largo...
                string ret;
                if (crlf)
                {
                    do
                    {
                        act = strings[a];
                        l = act == null ? 0 : act.Length;
                        iarr[a] = l;
                        if (l != 0)
                            lTot += l + 2;
                        else if (!ignoreEmpty)
                            lTot += 2;
                    } while (++a != la);
                    if (!sepAtEnd)
                        lTot -= 2;
                    if (lTot < 1)
                    {
                        if (heap != null)
                            heap.Dispose();
                        return Others.Empty;
                    }
                    ret = Mutables.Create(lTot + 1);
                    // genera cadena...
                    fixed (char* pret = ret)
                    {
                        a = 0;
                        lTot = 0;
                        do
                        {
                            l = iarr[a];
                            if (l != 0)
                            {
                                fixed (char* pact = strings[a])
                                    General.Copy(pact, pret + lTot, l);
                                lTot += l;
                                *(int*)(pret + lTot) = CharsCrLf;
                                lTot += 2;
                            }
                            else if (!ignoreEmpty)
                            {
                                *(int*)(pret + lTot) = CharsCrLf;
                                lTot += 2;
                            }
                        } while (++a != la);
                        if (!sepAtEnd)
                        {
                            lTot -= 2;
                            *(int*)(pret + lTot) = 0;
                        }
                        *(int*)(pret - 2) = lTot;
                    }
                }
                else
                {
                    do
                    {
                        act = strings[a];
                        l = act == null ? 0 : act.Length;
                        iarr[a] = l;
                        lTot += l;
                    } while (++a != la);
                    if (lTot == 0)
                    {
                        if (heap != null)
                            heap.Dispose();
                        return Others.Empty;
                    }
                    ret = Mutables.Create(lTot);
                    // genera cadena...
                    fixed (char* pret = ret)
                    {
                        a = 0;
                        lTot = 0;
                        do
                        {
                            l = iarr[a];
                            if (l != 0)
                            {
                                fixed (char* pact = strings[a])
                                    General.Copy(pact, pret + lTot, l);
                                lTot += l;
                            }
                        } while (++a != la);
                    }
                }
                if (heap != null)
                    heap.Dispose();
                return ret;
            }
        }

        #endregion

        #region MixString

        /// <summary>
        /// Genera un array de cadenas compuesto por las combinaciones de maysculas y
        /// minsculas posibles para la cadena de entrada.
        /// </summary>
        /// <param name="str">Cadena a procesar. No debe ser nula ni mayor de 24 carcteres.</param>
        /// <returns>Coleccin con array de combinaciones.</returns>
        public static string[] MixString(string str)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
            {
                return new string[] { str };
            }
            if (l > 30)
                throw new ArgumentException(Errors.MIXSTRING_MAJOR_30, "str");
            str = ToLowerInvariant(str);
            string mcad = ToUpperInvariant(str);
            if (str == mcad)
            {
                return new string[] { str };
            }
            unsafe
            {
                char* pori = stackalloc char[l];
                char* pmori = stackalloc char[l];
                char* pdes = stackalloc char[l];
                fixed (char* px = str)
                    General.Copy(px, pori, l);
                fixed (char* px = mcad)
                    General.Copy(px, pmori, l);
                int* pbool = stackalloc int[l];
                int cop = 0;
                for (int a = 0; a != l; a++)
                {
                    if (pori[a] != pmori[a])
                    {
                        pbool[a] = 1;
                        cop++;
                    }
                    else
                        pbool[a] = 0;
                }
                int tot = 1 << l;
                string[] ret = new string[1 << cop];
                cop = 0;
                bool destok = false;
                for (int a = 0; a != tot; a++)
                {
                    if (!destok)
                    {
                        General.Copy(pori, pdes, l);
                        destok = true;
                    }
                    bool modok = true;
                    for (int b = 1, c = 0; c != l; b <<= 1, c++)
                    {
                        if ((a & b) != 0)
                        {
                            if (pbool[c] != 0)
                            {
                                pdes[c] = pmori[c];
                                destok = false;
                            }
                            else
                            {
                                modok = false;
                                break;
                            }
                        }
                    }
                    if (modok)
                        ret[cop++] = Mutables.Create(pdes,l);
                }
                return ret;
            }
        }

        #endregion

        #region Quitar Comillas

        /// <summary>
        /// Efectua un reconocimiento simple de texto encomillado:
        /// - Efectua Trim al bloque.
        /// - Si encuentra comillas coincidentes al comienzo y al final del bloque resultante, las descarta.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="afterTrimR">Luego del procesamiento de comillas, efecta trim derecho.</param>
        /// <param name="afterTrimL">Luego del procesamiento de comillas, efecta trim izquierdo.</param>
        /// <param name="mode">Opciones de eliminacin de comillas.</param>
        /// <returns>Cadena sin comillas (misma instancia si no hay modificaciones).</returns>
        public static string RemoveQuotes(string str, RemoveQuotesMode mode, bool afterTrimL, bool afterTrimR)
        {
            int l=str==null?0:str.Length;
            if (l==0)
                return str;
            unsafe 
            {
                fixed(char*p=str)
                {
                    Mem.PChar pchar=new Mem.PChar(p,l,mode,afterTrimL,afterTrimR);
                    if(pchar.Length!=l) return pchar.ToString();
                }
            }
            return str;
        }


        #endregion

        #region Normalizar Blancos y Normalizar Largo

        /// <summary>
        /// Efecta una normalizacin de cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="filter">Opciones NormalizeFilter.</param>
        /// <param name="changeCase">Opciones de cambio May/Min.</param>
        /// <param name="alwaysCopy">Siempre retornar una copia de la cadena.</param>
        /// <returns>Misma instancia de cadena si no existen modificaciones, de lo contrario una nueva cadena (mutable) o cadena vacia estndar.</returns>
        public static string NormalizeString(bool alwaysCopy, string str, NormalizeFilter filter, NormalizeCase changeCase)
        {
            int l=str==null?0:str.Length;
            if (l==0)
                return Others.Empty;
            if (filter==NormalizeFilter.None && changeCase==NormalizeCase.None)
                return (alwaysCopy)?Mutables.Create(str,0,l):str;
            unsafe
            {
                if (l<512)
                {
                    char*p=stackalloc char[l];
                    fixed(char*po=str)
                        General.Copy(po,p,l);
                    if (NormalizeString(p,l,out l,filter,changeCase,false) || alwaysCopy)
                        return Mutables.Create(p,l);
                    return str;
                }
                else
                {
                    Heap heap=new Heap(l<<1,false);
                    fixed(char*po=str)
                        General.Copy(po,(char*)heap.Pointer,l);
                    string ret;
                    if (NormalizeString((char*)heap.Pointer,l,out l,filter,changeCase,false) || alwaysCopy)
                        ret = Mutables.Create((char*)heap.Pointer,l);
                    else
                        ret=str;
                    heap.Dispose();
                    return ret;
                }
            }


        }

        /// <summary>
        /// Efecta una normalizacin de cadena.
        /// </summary>
        /// <param name="p">Puntero Origen / Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// </param>
        /// <param name="filter">Opciones NormalizeFilter.</param>
        /// <param name="changeCase">Opciones de cambio May/Min.</param>
        /// <param name="zeroFinal">Si true, estampar cero al final (si ocurrieron modificaciones).</param>
        /// <returns>True si ocurrieron modificaciones.</returns>
        public unsafe static bool NormalizeString(char* p, int l, out int lFinal, NormalizeFilter filter, NormalizeCase changeCase, bool zeroFinal)
        {
            if (l < 1)
            {
                lFinal = 0;
                return false;
            }
            bool ret;
            switch((filter&(NormalizeFilter.FilterNoDigitsHex|NormalizeFilter.FilterNoDigits)))
            {
                case NormalizeFilter.None:
                    ret=false;
                    lFinal=l;
                    break;
                case NormalizeFilter.FilterNoDigits:
                    if (TxtComp.RemoveNoDigits(false,p,p,l,out lFinal,false,false)!=0)
                    {
                        if (zeroFinal)
                            p[lFinal]='\0';
                        return true;
                    }
                    lFinal=l;
                    return false;
                case NormalizeFilter.FilterNoDigitsHex:
                    if (TxtComp.RemoveNoDigits(false,p,p,l,out lFinal,false,true)!=0)
                    {
                        if (zeroFinal)
                            p[lFinal]='\0';
                        if (lFinal==0)
                            return true;
                        ret=true;
                    }
                    else
                    {
                        lFinal=l;
                        ret=false;
                    }
                    goto cCase;
                default:
                    throw new ArgumentException(Errors.INVALID_ENUM,"filter");
            }
            if ((filter&NormalizeFilter.RemoveBlanks)!=0)
            {
                if (TxtComp.RemoveBlanks(false,p,p,lFinal,out lFinal,false)!=0)
                {
                    if (zeroFinal)
                        p[lFinal]='\0';
                    if (lFinal==0)
                        return true;
                    ret=true;
                }
                if ((filter&NormalizeFilter.RemoveQuotes)!=0)
                {
                    PChar pc=new PChar(p,lFinal,RemoveQuotesMode.RemoveAllNoTrim,false,false);
                    if (pc.Length!=lFinal)
                    {
                        lFinal=pc.Length;
                        if (pc.Length==0)
                        {
                            if (zeroFinal)
                               *p='\0';
                            return true;
                        }
                        General.Copy(pc.Pointer,p,pc.Length);
                        if (zeroFinal)
                            p[pc.Length]='\0';
                        ret=true;
                    }
                }
                goto cCase;
            }
            if ((filter&(NormalizeFilter.RemoveQuotes|NormalizeFilter.TrimFull))!=0)
            {
                PChar pc=new PChar(p,lFinal,((filter&NormalizeFilter.RemoveQuotes)!=0)?(RemoveQuotesMode.RemoveAll):(RemoveQuotesMode.NoRemove),(filter&NormalizeFilter.TrimLeft)!=0,(filter&NormalizeFilter.TrimRight)!=0);
                if (pc.Length!=lFinal)
                {
                    lFinal=pc.Length;
                    if (lFinal==0)
                    {
                        if (zeroFinal)
                            *p='\0';
                        return true;
                    }
                    General.Copy(pc.Pointer,p,lFinal);
                    if (zeroFinal)
                        p[lFinal]='\0';
                    ret=true;
                }
            }

            if ((filter&NormalizeFilter.NormalizeBlanks)!=0)
            {
                int a;
                if (!ret)
                {
                    a=0;
                    do
                    {
                        switch(p[a])
                        {
                            case '\t':
                            case '\n':
                            case '\r':
                            case '\xa0':
                            case '\x85':
                            case '\0':
                            case '\v':
                            case '\f':
                            case '\u1680':
                            case '\u2000':
                            case '\u2001':
                            case '\u2002':
                            case '\u2003':
                            case '\u2004':
                            case '\u2005':
                            case '\u2006':
                            case '\u2007':
                            case '\u2008':
                            case '\u2009':
                            case '\u200a':
                            case '\u200b':
                            case '\u2028':
                            case '\u2029':
                            case '\u202f':
                            case '\u3000':
                            case '\ufeff':
                                ret=true;
                                break;
                        }
                    }while(++a!=lFinal);
                }
                if ((a=NormalizeBlanks(p,p,lFinal,false,true,true,' '))!=lFinal)
                {
                    lFinal=a;
                    if (zeroFinal)
                        p[a]='\0';
                    if (a==0)
                        return true;
                    ret=true;
                }
            }
            cCase:
                switch(changeCase)
                {
                    case NormalizeCase.ToLowerCurrent:
                        return ToLower(p,lFinal,Texts.TableCurrent) || ret;
                    case NormalizeCase.ToLowerInvariant:
                        return ToLowerInvariant(p,lFinal) || ret;
                    case NormalizeCase.ToProperCurrent:
                        return ToProper(p,lFinal,ParagraphMode.Nothing,false,Texts.TableCurrent) || ret;
                    case NormalizeCase.ToProperInvariant:
                        return ToProper(p,lFinal,ParagraphMode.Nothing,false,TableCase.Invariant) || ret;
                    case NormalizeCase.ToUpperCurrent:
                        return ToUpper(p,lFinal,Texts.TableCurrent) || ret;
                    case NormalizeCase.ToUpperInvariant:
                        return ToUpperInvariant(p,lFinal) || ret;
                }
            return ret;
        }

        /// <summary>
        /// Copia memoria reemplazando caracteres blancos y luego eliminando apariciones contiguas del caracter de reemplazo.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).
        /// Puntero origen siempre es copiado a destino.
        /// </param>
        /// <param name="l">Largo.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir del largo final.
        /// La limpieza ocurre solo si se dan estas condiciones: El largo final es diferente al largo original y a 0.
        /// </param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="trimLeft">Efecta trim izquierdo.</param>
        /// <param name="trimRight">Efecta trim derecho.</param>
        /// <returns>Largo en pd.</returns>
        public unsafe static int NormalizeBlanks(char* po, char* pd, int l, bool clean, bool trimLeft, bool trimRight, char r)
        {
            if (l < 1)
                return 0;
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            int a=0;
            int final=0;
            char c;
            do
            {
                c=po[a];
                switch(c)
                {
                    case ' ':
                    case '\t':
                    case '\n':
                    case '\r':
                    case '\xa0':
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if (!trimLeft)
                        {
                            pd[final++]=r;
                            trimLeft=true;
                        }
                        continue;
                }
                if (c!=r)
                {
                    pd[final++]=c;
                    if (trimLeft)
                        trimLeft=false;
                }
                else if (!trimLeft)
                {
                    pd[final++]=r;
                    trimLeft=true;
                }
            }while(++a!=l);
            if (final!=0)
            {
                if (trimRight && pd[final-1]==r)
                {
                    if (--final==0)
                        return 0;
                    if (clean)
                        General.Clean(pd + final, l-final);
                    else
                        pd[final]='\0';
                }
                else if (clean && final!=l)
                    General.Clean(pd + final, l-final);
            }
            return final;
        }



        /// <summary>
        /// Copia memoria reemplazando caracteres blancos y luego eliminando apariciones contiguas del caracter de reemplazo.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).
        /// Puntero origen siempre es copiado a destino.
        /// </param>
        /// <param name="l">Largo.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir del largo final.
        /// La limpieza ocurre solo si se dan estas condiciones: El largo final es diferente al largo original y a 0.
        /// </param>
        /// <param name="control">Incluye en filtro de reemplazo y trimLeftSelect carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro de reemplazo y trimLeftSelect \r y \n.</param>
        /// <param name="lat1">Incluye en filtro de reemplazo y trimLeftSelect los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro de reemplazo y trimLeftSelect el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro de reemplazo y trimLeftSelect espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro de reemplazo y trimLeftSelect el taburador (\t).</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="trimLeftSelect">Efecta trim izquierdo sobre el caracter de reemplazo (y por lo tanto sobre los reemplazados contiguos).</param>
        /// <param name="trimRightFull">Efecta trim derecho sobre todos los caracteres blancos, aunque no esten seleccionados.</param>
        /// <returns>Largo en pd.</returns>
        public unsafe static int NormalizeBlanks(char* po, char* pd, int l, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control, bool clean, bool trimLeftSelect, bool trimRightFull, char r)
        {
            if (l < 1)
                return 0;
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            int a=0;
            int final=0;
            char c;
            do
            {
                c=po[a];
                switch(c)
                {
                    case ' ':
                        if (sp)
                        {
                            if (!trimLeftSelect)
                            {
                                pd[final++]=r;
                                trimLeftSelect=true;
                            }
                            continue;
                        }
                        break;
                    case '\t':
                        if (tab)
                        {
                            if (!trimLeftSelect)
                            {
                                pd[final++]=r;
                                trimLeftSelect=true;
                            }
                            continue;
                        }
                        break;
                    case '\n':
                    case '\r':
                        if (crlf)
                        {
                            if (!trimLeftSelect)
                            {
                                pd[final++]=r;
                                trimLeftSelect=true;
                            }
                            continue;
                        }
                        break;
                    case '\xa0':
                    case '\x85':
                        if (lat1)
                        {
                            if (!trimLeftSelect)
                            {
                                pd[final++]=r;
                                trimLeftSelect=true;
                            }
                            continue;
                        }
                        break;
                    case '\0':
                    case '\v':
                    case '\f':
                        if (control)
                        {
                            if (!trimLeftSelect)
                            {
                                pd[final++]=r;
                                trimLeftSelect=true;
                            }
                            continue;
                        }
                        break;
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if (uni)
                        {
                            if (!trimLeftSelect)
                            {
                                pd[final++]=r;
                                trimLeftSelect=true;
                            }
                            continue;
                        }
                        break;
                }
                if (c!=r)
                {
                    pd[final++]=c;
                    if (trimLeftSelect)
                        trimLeftSelect=false;
                }
                else if (!trimLeftSelect)
                {
                    pd[final++]=r;
                    trimLeftSelect=true;
                }
            }while(++a!=l);
            if (final!=0)
            {
                if (trimRightFull)
                {
                    int final2=(new PChar(pd,final,false,true)).Length;
                    if (final2!=l)
                    {
                        if (final2==0)
                            return 0;
                        if (clean)
                            General.Clean(pd + final2, l-final2);
                        else if (final2!=final)
                            General.Clean(pd + final2, final-final2);
                    }
                    return final2;
                }
                if (clean && final!=l)
                    General.Clean(pd + final, l-final);
            }
            return final;
        }

        /// <summary>
        /// Copia la cadena reemplazando caracteres blancos y luego eliminando apariciones contiguas del caracter de reemplazo.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="trimLeft">Efecta trim izquierdo.</param>
        /// <param name="trimRight">Efecta trim derecho.</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <returns>Cadena resultante de las depuraciones. Siempre es una copia mutable o cadena vacia.</returns>
        public static string NormalizeBlanks(string str, bool trimLeft, bool trimRight, char r)
        {
            int l=str==null?0:str.Length;
            if (l==0) 
                return Others.Empty;
            string ret=Mutables.Create(l);
            unsafe
            {
                fixed(char*po=str,pd=ret)
                {
                    int lfinal=NormalizeBlanks(po,pd,l,false,trimLeft,trimRight,r);
                    if (lfinal!=l)
                    {
                        if (lfinal==0)
                            return Others.Empty;
                        ((int*)pd)[-1]=lfinal;
                    }
                    return ret;
                }
            }
        }

        /// <summary>
        /// Copia la cadena reemplazando caracteres blancos y luego eliminando apariciones contiguas del caracter de reemplazo.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="control">Incluye en filtro de reemplazo y trimLeftSelect carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro de reemplazo y trimLeftSelect \r y \n.</param>
        /// <param name="lat1">Incluye en filtro de reemplazo y trimLeftSelect los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro de reemplazo y trimLeftSelect el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro de reemplazo y trimLeftSelect espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro de reemplazo y trimLeftSelect el taburador (\t).</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="trimLeftSelect">Efecta trim izquierdo sobre el caracter de reemplazo (y por lo tanto sobre los reemplazados contiguos).</param>
        /// <param name="trimRightFull">Efecta trim derecho sobre todos los caracteres blancos, aunque no esten seleccionados.</param>
        /// <returns>Cadena resultante de las depuraciones. Siempre es una copia mutable o cadena vacia.</returns>
        public static string NormalizeBlanks(string str, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control, bool trimLeftSelect, bool trimRightFull, char r)
        {
            int l=str==null?0:str.Length;
            if (l==0) 
                return Others.Empty;
            string ret=Mutables.Create(l);
            unsafe
            {
                fixed(char*po=str,pd=ret)
                {
                    int lfinal=NormalizeBlanks(po,pd,l,sp,lat1,uni,tab,crlf,control,false,trimLeftSelect,trimRightFull,r);
                    if (lfinal!=l)
                    {
                        if (lfinal==0)
                            return Others.Empty;
                        ((int*)pd)[-1]=lfinal;
                    }
                    return ret;
                }
            }
        }


        /// <summary>
        /// Convierte todos los carcteres en blanco en espacios estndar.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>Cadena procesada (misma instancia si no hay modificaciones).</returns>
        public static string BlankToSpace(string str)
        {
            int l=str==null?0:str.Length;
            if (l==0) 
                return str;
            string ret=Mutables.Create(l);
            unsafe
            {
                fixed(char*po=str,pd=ret)
                {
                    if (TxtComp.ReplaceBlanks(false,po,pd,l,' ')!=0) 
                        return ret;
                }
            }
            return str;
        }


        /// <summary>
        /// Rellena o contrae una cadena de acuerdo al largo.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="l">Largo.</param>
        /// <param name="fill">Caracter Relleno.</param>
        /// <param name="fillLeft">Fill a la izquierda. La contraccin siempre es a la derecha.</param>
        /// <returns>Cadena modificada o la misma instancia de cadena si no hay modificaciones.</returns>
        public static string NormalizeLength(string str, int l, char fill, bool fillLeft)
        {
            int lcad=str==null?0:str.Length;
            if (lcad==l)
                return str;
            if (l<1)
            {
                if (l==0)
                    return Others.Empty;
                throw new ArgumentOutOfRangeException("l",l,Errors.NEGATIVE);
            }
            if (lcad == 0) 
                return Mutables.Create(l,fill);
            if (lcad>l) 
                return Mutables.Create(str,l);
            return Mutables.Create(str,0,lcad,fillLeft?(l-lcad):0,l,fill);
        }


        #endregion

        #region Split

        /// <summary>
        /// Similar a String.Split.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="ignoreEmpty">Ignora elementos vacios.</param>
        /// <param name="trim">Hace trim sobre cada elemento.</param>
        /// <param name="blankMode">Si true, separa los carcteres en base a espacios en blanco, Si false separa lneas de texto.</param>
        /// <returns>Array de 0 o ms elementos.</returns>
        public static string[] Split(string str, bool ignoreEmpty, bool trim, bool blankMode)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
            {
                if (ignoreEmpty)
                    return ArrGeneral.ArrayEmptyString;
                return new string[1] { Others.Empty };
            }
            unsafe
            {
                fixed (char* p = str)
                    return Split(p, l, ignoreEmpty, trim, blankMode);
            }
        }


        /// <summary>
        /// Similar a String.Split.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="seps">Separadores. Si esta vacio, usara los carcteres en blanco como separador.</param>
        /// <param name="ignoreEmpty">Ignora elementos vacios.</param>
        /// <param name="trim">Hace trim sobre cada elemento.</param>
        /// <returns>Array de 0 o ms elementos.</returns>
        public static string[] Split(string str, bool ignoreEmpty, bool trim, params char[] seps)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
            {
                if (ignoreEmpty)
                    return ArrGeneral.ArrayEmptyString;
                return new string[1] { Others.Empty };
            }
            int lsep = seps==null?0:seps.Length;
            unsafe
            {
                if (lsep == 0)
                    fixed (char* p = str)
                        return Split(p, l, ignoreEmpty, trim, true);
                fixed (char* p = str, psep = seps)
                    return Split(p, l, ignoreEmpty, trim, psep, lsep, false);
            }
        }

        /// <summary>
        /// Similar a String.Split.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="lsep">Largo del puntero de separadores.</param>
        /// <param name="psep">Puntero de separadores. Si esta vacio, retorna un array de un solo elemento.</param>
        /// <param name="ignoreEmpty">Ignora elementos vacios.</param>
        /// <param name="trim">Hace trim sobre cada elemento.</param>
        /// <param name="noCopStack">No intentar copiar al Stack al puntero de separadores.</param>
        /// <returns>Array de 0 o ms elementos.</returns>
        public unsafe static string[] Split(char* p, int l, bool ignoreEmpty, bool trim, char* psep, int lsep, bool noCopStack)
        {
            if (l < 1)
            {
                if (ignoreEmpty)
                    return ArrGeneral.ArrayEmptyString;
                return new string[1] { Others.Empty };
            }
            if (lsep < 1)
            {
                if (trim)
                {
                    Mem.PChar ptrim = new Mem.PChar(p, l, true, true);
                    if (ptrim.Length == 0)
                    {
                        if (ignoreEmpty)
                            return ArrGeneral.ArrayEmptyString;
                        return new string[1] { Others.Empty };
                    }
                }
                return new string[1] { Mutables.Create(p,l) };
            }
            if (!noCopStack && psep > General.MaximumStack && l > 32 && lsep < General.MaxStackChars && &lsep > General.MinimumStack)
            {
                char* pcops = stackalloc char[lsep];
                General.Copy(psep, pcops, lsep);
                psep = pcops;
            }

            PChars zonas=new PChars((l<PChars.DefaultCapacity)?l:PChars.DefaultCapacity);
            Mem.PChar pc = new Mem.PChar(p, l);
            Mem.PChar pc2;
            
            if (lsep != 1)
            {
                if (ignoreEmpty)
                {
                    if (trim)
                    {
                        do
                        {
                            pc2 = pc.Foward(psep, lsep, true);
                            if (pc2.Pointer != null)
                            {
                                pc2 = new Mem.PChar(pc2.Pointer, pc2.Length, true, true);
                                if (pc2.Length != 0)
                                    zonas.Append(pc2);
                            }
                            else
                                break;
                        } while (true);
                    }
                    else
                    {
                        do
                        {
                            pc2 = pc.Foward(psep, lsep, true);
                            if (pc2.Pointer != null)
                                zonas.Append(pc2);
                            else
                                break;
                        } while (true);
                    }
                }
                else
                {
                    if (trim)
                    {
                        do
                        {
                            pc2 = pc.Foward(psep, lsep, false);
                            if (pc2.Pointer != null)
                            {
                                if (pc2.Length != 0)
                                    pc2 = new Mem.PChar(pc2.Pointer, pc2.Length, true, true);
                                zonas.Append(pc2);
                            }
                            else
                                break;
                        } while (true);
                    }
                    else
                    {
                        do
                        {
                            pc2 = pc.Foward(psep, lsep, false);
                            if (pc2.Pointer != null)
                                zonas.Append(pc2);
                            else
                                break;
                        } while (true);
                    }

                }
            }
            else
            {
                if (ignoreEmpty)
                {
                    if (trim)
                    {
                        do
                        {
                            pc2 = pc.Foward(*psep, true);
                            if (pc2.Pointer != null)
                            {
                                pc2 = new Mem.PChar(pc2.Pointer, pc2.Length, true, true);
                                if (pc2.Length != 0)
                                    zonas.Append(pc2);
                            }
                            else
                                break;
                        } while (true);
                    }
                    else
                    {
                        do
                        {
                            pc2 = pc.Foward(*psep, true);
                            if (pc2.Pointer != null)
                                zonas.Append(pc2);
                            else
                                break;
                        } while (true);
                    }
                }
                else
                {
                    if (trim)
                    {
                        do
                        {
                            pc2 = pc.Foward(*psep, false);
                            if (pc2.Pointer != null)
                            {
                                if (pc2.Length != 0)
                                    pc2 = new Mem.PChar(pc2.Pointer, pc2.Length, true, true);
                                zonas.Append(pc2);
                            }
                            else
                                break;
                        } while (true);
                    }
                    else
                    {
                        do
                        {
                            pc2 = pc.Foward(*psep, false);
                            if (pc2.Pointer != null)
                                zonas.Append(pc2);
                            else
                                break;
                        } while (true);
                    }

                }
            }
            lsep=zonas.Length;
            if (lsep == 0)
                return ArrGeneral.ArrayEmptyString;
            string[] ret = new string[lsep];
            l = 0;

            fixed(PChar* pzonas=zonas.Array)
            {
                if (ignoreEmpty)
                {
                    do
                    {
                        ret[l] = Mutables.Create(pzonas[l].Pointer, pzonas[l].Length);
                    } while (++l != lsep);
                }
                else
                {
                    do
                    {
                        if (pzonas[l].Length != 0)
                            ret[l] = Mutables.Create(pzonas[l].Pointer, pzonas[l].Length);
                        else
                            ret[l] = Others.Empty;
                    } while (++l != lsep);
                }
            }
            return ret;
        }



        /// <summary>
        /// Similar a String.Split.
        /// Genera un array con los parrafos (efectuando trim derecho completo). Se ignoran parrafos vacios.
        /// Si alguna de las opciones lTrimCtrl, lTrimLat1, lTrimSp, lTrimUni o lTrimTab esta activa, tambien
        /// se realizar un trim izquierdo a cada parrafo.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="doubleLine">
        /// Solo se considera parrafo cuando el bloque finaliza en un salto de lnea doble.
        /// Los saltos de lnea intermedio se convierten a espacios simples (eliminando todo espacio en blanco excedente).
        /// </param>
        /// <param name="lTrimCtrl">Incluye en filtro de trim izquierdo carcteres de control \0, \v y \f.</param>
        /// <param name="lTrimLat1">Incluye en filtro de trim izquierdo los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="lTrimSp">Incluye en filtro de trim izquierdo el espacio estndar (32).</param>
        /// <param name="lTrimUni">Incluye en filtro de trim izquierdo espacios unicode (\u1680 y superiores).</param>
        /// <param name="lTrimTab">Incluye en filtro de trim izquierdo el taburador (\t).</param>
        /// <returns>Array de 0 o ms elementos.</returns>
        public unsafe static string[] Split(char* p, int l, bool doubleLine, bool lTrimSp, bool lTrimLat1, bool lTrimUni, bool lTrimTab, bool lTrimCtrl)
        {
            int trim;
            if (lTrimSp || lTrimLat1 || lTrimUni || lTrimTab || lTrimCtrl)
            {
                if (lTrimSp && lTrimLat1 && lTrimUni && lTrimTab && lTrimCtrl)
                    trim=2;
                else
                    trim=1;
            }
            else
                trim=0;
          
            if (l < 1)
                return ArrGeneral.ArrayEmptyString;

            PChars zonas=new PChars((l<PChars.DefaultCapacity)?l:PChars.DefaultCapacity);

            Mem.PChar pc = new Mem.PChar(p, l);
            Mem.PChar pc2;
            while((pc2 = pc.FowardParagraph(doubleLine)).Length != 0)
            {
                if (trim!=0)
                {
                    if (trim!=1)
                        zonas.Append(new PChar(pc2.Pointer,pc2.Length,true,false));
                    else
                        zonas.Append(new PChar(pc2.Pointer,pc2.Length,true,false,lTrimSp,lTrimLat1,lTrimUni,lTrimTab,false,lTrimCtrl));
                }
                else
                    zonas.Append(pc2);
            }
            int tot=zonas.Length;
            if (tot == 0)
                return ArrGeneral.ArrayEmptyString;
            string[] ret = new string[tot];
            l = 0;
            fixed(PChar* pzonas=zonas.Array)
            {
                
                if (doubleLine)
                {
                    PChars dob=new PChars();
                    PChar[] arrtmp;
                    int atmp,tsum;
                    do
                    {
                        pc=pzonas[l];
                        lTrimSp=false;
                        atmp=0;
                        while((pc2 = pc.FowardParagraph(false)).Length != 0)
                        {
                            if (lTrimSp)
                                pc2=new PChar(pc2.Pointer,pc2.Length,true,false);
                            else
                                lTrimSp=true;
                            atmp+=pc2.Length;
                            dob.Append(pc2);
                        }
                        if ((trim=dob.Length)==1)
                            ret[l] = Mutables.Create(pzonas[l].Pointer, pzonas[l].Length);
                        else
                        {
                            ret[l]=Mutables.Create(atmp+trim-1);
                            fixed(char*ptmp=ret[l])
                            {
                                atmp=0;
                                tsum=0;
                                arrtmp=dob.Array;
                            otra:
                                pc=arrtmp[atmp];
                                General.Copy(pc.Pointer,ptmp+tsum,pc.Length);
                                tsum+=pc.Length;
                                if (++atmp!=trim)
                                {
                                    ptmp[tsum++]=' ';
                                    goto otra;
                                }
                            }
                        }
                        dob.Clear();
                    }while(++l!=tot);
                }
                else
                {
                    do
                    {
                        ret[l] = Mutables.Create(pzonas[l].Pointer, pzonas[l].Length);
                    }while(++l!=tot);
                }
            }
            return ret;
        }

        /// <summary>
        /// Similar a String.Split.
        /// Genera un array con los parrafos (efectuando trim derecho completo). Se ignoran parrafos vacios.
        /// Si alguna de las opciones lTrimCtrl, lTrimLat1, lTrimSp, lTrimUni o lTrimTab esta activa, tambien
        /// se realizar un trim izquierdo a cada parrafo.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="doubleLine">
        /// Solo se considera parrafo cuando el bloque finaliza en un salto de lnea doble.
        /// Los saltos de lnea intermedio se convierten a espacios simples (eliminando todo espacio en blanco excedente).
        /// </param>
        /// <param name="lTrimCtrl">Incluye en filtro de trim izquierdo carcteres de control \0, \v y \f.</param>
        /// <param name="lTrimLat1">Incluye en filtro de trim izquierdo los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="lTrimSp">Incluye en filtro de trim izquierdo el espacio estndar (32).</param>
        /// <param name="lTrimUni">Incluye en filtro de trim izquierdo espacios unicode (\u1680 y superiores).</param>
        /// <param name="lTrimTab">Incluye en filtro de trim izquierdo el taburador (\t).</param>
        /// <returns>Array de 0 o ms elementos.</returns>
        public static string[] Split(string str, bool doubleLine, bool lTrimSp, bool lTrimLat1, bool lTrimUni, bool lTrimTab, bool lTrimCtrl)
        {
            int l=str==null?0:str.Length;
            if (l == 0)
                return ArrGeneral.ArrayEmptyString;
            unsafe
            {
                fixed (char* p = str)
                    return Split(p, l,doubleLine,lTrimSp,lTrimLat1,lTrimUni,lTrimTab,lTrimCtrl);
            }

        }


        /// <summary>
        /// Similar a String.Split.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="ignoreEmpty">Ignora elementos vacios.</param>
        /// <param name="trim">Hace trim sobre cada elemento (ignorado si blankMode es true).</param>
        /// <param name="blankMode">Si true, separa los carcteres en base a espacios en blanco, Si false separa lneas de texto.</param>
        /// <returns>Array de 0 o ms elementos.</returns>
        public unsafe static string[] Split(char* p, int l, bool ignoreEmpty, bool trim, bool blankMode)
        {
            if (trim && blankMode)
                trim = false;
            if (l < 1)
            {
                if (ignoreEmpty)
                    return ArrGeneral.ArrayEmptyString;
                return new string[1] { Others.Empty };
            }

            PChars zonas=new PChars((l<PChars.DefaultCapacity)?l:PChars.DefaultCapacity);

            Mem.PChar pc = new Mem.PChar(p, l);
            Mem.PChar pc2;

            if (blankMode)
            {
                if (ignoreEmpty)
                {
                    if (trim)
                    {
                        do
                        {
                            pc2 = pc.FowardBlank();
                            if (pc2.Pointer != null)
                            {
                                if (pc2.Length != 0)
                                {
                                    pc2 = new Mem.PChar(pc2.Pointer, pc2.Length, true, true);
                                    if (pc2.Length != 0)
                                        zonas.Append(pc2);
                                }
                            }
                            else
                                break;
                        } while (true);
                    }
                    else
                    {
                        do
                        {
                            pc2 = pc.FowardBlank();
                            if (pc2.Pointer != null)
                            {
                                if (pc2.Length != 0)
                                    zonas.Append(pc2);
                            }
                            else
                                break;
                        } while (true);
                    }
                }
                else
                {
                    if (trim)
                    {
                        do
                        {
                            pc2 = pc.FowardBlank();
                            if (pc2.Pointer != null)
                            {
                                if (pc2.Length != 0)
                                    pc2 = new Mem.PChar(pc2.Pointer, pc2.Length, true, true);
                                zonas.Append(pc2);
                            }
                            else
                                break;
                        } while (true);
                    }
                    else
                    {
                        do
                        {
                            pc2 = pc.FowardBlank();
                            if (pc2.Pointer != null)
                                zonas.Append(pc2);
                            else
                                break;
                        } while (true);
                    }

                }
            }
            else
            {
                if (ignoreEmpty)
                {
                    if (trim)
                    {
                        do
                        {
                            pc2 = pc.FowardCrLf(true);
                            if (pc2.Pointer != null)
                            {
                                pc2 = new Mem.PChar(pc2.Pointer, pc2.Length, true, true);
                                if (pc2.Length != 0)
                                    zonas.Append(pc2);
                            }
                            else
                                break;
                        } while (true);
                    }
                    else
                    {
                        do
                        {
                            pc2 = pc.FowardCrLf(true);
                            if (pc2.Pointer != null)
                            {
                                zonas.Append(pc2);
                            }
                            else
                                break;
                        } while (true);
                    }
                }
                else
                {
                    if (trim)
                    {
                        do
                        {
                            pc2 = pc.FowardCrLf(false);
                            if (pc2.Pointer != null)
                            {
                                if (pc2.Length != 0)
                                    pc2 = new Mem.PChar(pc2.Pointer, pc2.Length, true, true);
                                zonas.Append(pc2);
                            }
                            else
                                break;
                        } while (true);
                    }
                    else
                    {
                        do
                        {
                            pc2 = pc.FowardCrLf(false);
                            if (pc2.Pointer != null)
                                zonas.Append(pc2);
                            else
                                break;
                        } while (true);
                    }

                }
            }
            int tot=zonas.Length;
            if (tot == 0)
                return ArrGeneral.ArrayEmptyString;
            string[] ret = new string[tot];
            l = 0;
            fixed(PChar* pzonas=zonas.Array)
            {
                if (ignoreEmpty)
                {
                    do
                    {
                        ret[l] = Mutables.Create(pzonas[l].Pointer, pzonas[l].Length);
                    } while (++l != tot);
                }
                else
                {
                    do
                    {
                        if (pzonas[l].Length != 0)
                            ret[l] = Mutables.Create(pzonas[l].Pointer, pzonas[l].Length);
                        else
                            ret[l] = Others.Empty;
                    } while (++l != tot);
                }
            }
            return ret;
        }


        #endregion

        #region Format

        /// <summary>
        /// Genera una cadena a partir del formato especificado.
        /// El comportamiento de la funcin es idntico a Texts.Format().
        /// </summary>
        /// <param name="format">Cadena de formato.</param>
        /// <param name="args">Argumentos.</param>
        /// <returns>Cadena formateada.</returns>
        public static string Format(string format, params object[] args)
        {
            SB sb=new SB();
            sb.AppendFormat(null,format,args);
            return sb.String;
        }

        /// <summary>
        /// Genera una cadena a partir del formato especificado.
        /// El comportamiento de la funcin es idntico a Texts.Format().
        /// </summary>
        /// <param name="provider">Proveedor de formato.</param>
        /// <param name="format">Cadena de formato.</param>
        /// <param name="args">Argumentos.</param>
        /// <returns>Cadena formateada.</returns>
        public static string Format (IFormatProvider provider, string format, params object[] args)
        {
            SB sb=new SB();
            sb.AppendFormat(provider,format,args);
            return sb.String;
        }




        #endregion

        #region Conversin entre Chars y Latin1


        /// <summary>
        /// Copia memoria convirtiendo chars a bytes (copiando el byte menos significativo del char) hasta encontrar 0.
        /// </summary>
        /// <param name="po">Puntero origen (Puede ser el mismo que el de destino aunque no verifica solapados).</param>
        /// <param name="pd">Puntero destino.</param>
        public unsafe static void CharsToLatin1(char* po, byte* pd)
        {
            int a=0;
            while(po[a]!=0)
                pd[a]=(byte)po[a++];
        }

        /// <summary>
        /// Copia memoria convirtiendo chars a bytes (copiando el byte menos significativo del char) hasta encontrar 0.
        /// </summary>
        /// <param name="po">Puntero origen (Puede ser el mismo que el de destino aunque no verifica solapados).</param>
        /// <param name="pd">Puntero destino.</param>
        public unsafe static void CharsToLatin1(char* po, sbyte* pd)
        {
            CharsToLatin1(po,(byte*)pd);
        }

        /// <summary>
        /// Copia memoria convirtiendo bytes a chars (Latin1) hasta encontrar cero.
        /// </summary>
        /// <param name="po">Puntero origen (No puede ser igual ni estar solapado con el destino!).</param>
        /// <param name="pd">Puntero destino.</param>
        public unsafe static void Latin1ToChars(byte* po, char* pd)
        {
            int a=0;
            while(po[a]!=0)
                pd[a]=(char)po[a++];
        }

        /// <summary>
        /// Copia memoria convirtiendo bytes a chars (Latin1) hasta encontrar cero.
        /// </summary>
        /// <param name="po">Puntero origen (No puede ser igual ni estar solapado con el destino!).</param>
        /// <param name="pd">Puntero destino.</param>
        public unsafe static void Latin1ToChars(sbyte* po, char* pd)
        {
            Latin1ToChars((byte*)po,pd);
        }

        /// <summary>
        /// Copia memoria convirtiendo chars a bytes (copiando el byte menos significativo del char).
        /// </summary>
        /// <param name="po">Puntero origen (Puede ser el mismo que el de destino aunque no verifica solapados).</param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="l">Largo.</param>
        public unsafe static void CharsToLatin1(char* po, byte* pd, int l)
        {
            if (l>0)
            {
                int a=0;
                do
                {
                    pd[a]=(byte)po[a];                
                }while(++a!=l);
            }
        }

        /// <summary>
        /// Copia memoria convirtiendo chars a bytes (copiando el byte menos significativo del char).
        /// </summary>
        /// <param name="po">Puntero origen (Puede ser el mismo que el de destino aunque no verifica solapados).</param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="l">Largo.</param>
        public unsafe static void CharsToLatin1(char* po, sbyte* pd, int l)
        {
            CharsToLatin1(po,(byte*)pd,l);
        }

        /// <summary>
        /// Copia memoria convirtiendo bytes a chars (Latin1).
        /// </summary>
        /// <param name="po">Puntero origen (No puede ser igual ni estar solapado con el destino!).</param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="l">Largo.</param>
        public unsafe static void Latin1ToChars(byte* po, char* pd, int l)
        {
            if (l>0)
            {
                int a=0;
                do
                {
                    pd[a]=(char)po[a]; 
                }while(++a!=l);
            }
        }

        /// <summary>
        /// Copia memoria convirtiendo bytes a chars (Latin1).
        /// </summary>
        /// <param name="po">Puntero origen (No puede ser igual ni estar solapado con el destino!).</param>
        /// <param name="pd">Puntero destino.</param>
        /// <param name="l">Largo.</param>
        public unsafe static void Latin1ToChars(sbyte* po, char* pd, int l)
        {
            Latin1ToChars((byte*)po,pd,l);
        }

        #endregion


    }
}
