using System;
using System.Globalization;
using EnPalermo.Mem;
using EnPalermo.Lit;
using EnPalermo.Col;

namespace EnPalermo.Txt
{

    #region Enums


    /// <summary>
    /// Resultado de la comparacin entre 2 cadenas.
    /// </summary>
    public enum CompareStrings
    {
        /// <summary>
        /// Ambas cadenas no estan vacias y son iguales.
        /// </summary>
        EqualsNoEmptys,
        /// <summary>
        /// Ambas cadenas estan vacias o nulas.
        /// </summary>
        Emptys,
        /// <summary>
        /// Cadena A es nula o vacia, la B no.
        /// </summary>
        Aempty,
        /// <summary>
        /// Cadena B es nula o vacia, la A no.
        /// </summary>
        Bempty,
        /// <summary>
        /// Ambas cadenas no estan vacias y son diferentes.
        /// </summary>
        DiffersNoEmptys
    }



    /// <summary>
    /// Resultado de la comparacin entre 3 cadenas:
    /// Cadena A (actual).
    /// Cadena B (nueva).
    /// Cadena C (testigo).
    /// </summary>
    public enum TripleCompare
    {

        /// <summary>
        /// Las 3 cadenas vlidas e iguales.
        /// </summary>
        EqualsNoEmptys,

        /// <summary>
        /// Las 3 cadenas son nulas o vacias.
        /// </summary>
        Emptys,

        /// <summary>
        /// A no est vacia. B y C nulas o vacias.
        /// </summary>
        BCemptys,

        /// <summary>
        /// B no est vacia. A y C nulas o vacias.
        /// </summary>
        ACemptys,

        /// <summary>
        /// C no est vacia. A y B nulas o vacias.
        /// </summary>
        ABemptys,

        /// <summary>
        /// A y B no vacias e iguales. C es nula o vacia.
        /// </summary>
        ABequalsCempty,

        /// <summary>
        /// A y B no vacias y diferentes entre si, C es nula o vacia.
        /// </summary>
        ABdiffersCempty,

        /// <summary>
        /// A y C no vacias e iguales. B es nula o vacia.
        /// </summary>
        ACequalsBempty,

        /// <summary>
        /// A y C no vacias y diferentes entre si, B es nula o vacia.
        /// </summary>
        ACdiffersBempty,

        /// <summary>
        /// B y C no vacias e iguales. A es nula o vacia.
        /// </summary>
        BCequalsAempty,

        /// <summary>
        /// B y C no vacias y diferentes entre si, A es nula o vacia.
        /// </summary>
        BCdiffersAempty,

        /// <summary>
        /// Las 3 cadenas no vacias, C es diferente a A y B.
        /// </summary>
        ABequalsCdiffer,

        /// <summary>
        /// Las 3 cadenas no vacias, B es diferente a A y C.
        /// </summary>
        ACequalsBdiffer,

        /// <summary>
        /// Las 3 cadenas no vacias, A es diferente a B y C.
        /// </summary>
        BCequalsAdiffer,

        /// <summary>
        /// Las 3 cadenas no vacias y diferentes entre si.
        /// </summary>
        DiffersNoEmptys

    }

    /// <summary>
    /// Opciones de comparacin ordinal.
    /// </summary>
    public enum Ordinal
    {
        /// <summary>
        /// Comparacin ordinal sin ignorar May/Min.
        /// </summary>
        Binary,
        /// <summary>
        /// Comparacin ordinal ignorando May/Min basndose en la cultura actual.
        /// </summary>
        Current,
        /// <summary>
        /// Comparacin ordinal ignorando May/Min basndose en la cultura invariable.
        /// </summary>
        Invariant

    }


    #endregion


    /// <summary>
    /// Funciones de comparacin, busqueda y remplazo en cadenas.
    /// </summary>
    public static class TxtComp
    {
 
        #region Equal e EqualNC

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres.
        /// </summary>
        /// <param name="c1">Caracter 1.</param>
        /// <param name="c2">Caracter 2.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(char c1, char c2, Ordinal comp)
        {
            if (c1 != c2)
            {
                if (comp == Ordinal.Binary)
                    return false;
                if (comp == Ordinal.Current)
                {
                    TableCase t = Texts.TableCurrent;
                    return Texts.ToUpper(c1, t) == Texts.ToUpper(c2, t);
                }
                return Texts.ToUpperInvariant(c1) == Texts.ToUpperInvariant(c2);
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres.
        /// Es un wrapper a Comp.Equal().
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="p2Upper">Si true, el puntero 2 no ser convertido
        /// a maysculas en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="l">Largo de los punteros.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>True si son iguales.</returns>
        public static unsafe bool Equal(char* p1, char* p2, bool p2Upper, int l, Ordinal comp)
        {
            if (comp == Ordinal.Binary)
                return Comp.Equal(p1, p2, l);
            if (comp == Ordinal.Current)
                return EqualNC(p1, p2,p2Upper, l, Texts.TableCurrent);
            return EqualNCInvariant(p1, p2,p2Upper, l);
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres.
        /// </summary>
        /// <param name="str1">Cadena 1.</param>
        /// <param name="str2">Cadena 2.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="nullEqualEmpty">Si true, null equivale a vacio.</param>
        /// <returns>True si son iguales.</returns>
        public static unsafe bool Equal(string str1, string str2, Ordinal comp, bool nullEqualEmpty)
        {
            if (!Object.ReferenceEquals(str1,str2))
            {
                if (str1 as object == null)
                    return nullEqualEmpty?(str2.Length == 0):false;
                if (str2 as object == null)
                    return nullEqualEmpty?(str1.Length == 0):false;
                int l = str1.Length;
                if (l != str2.Length)
                    return false;
                fixed (char* p1 = str1, p2 = str2)
                    return Equal(p1, p2,false, l, comp);
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres.
        /// </summary>
        /// <param name="str1">Cadena 1 (no se aceptan instancias nulas).</param>
        /// <param name="str2">Cadena 2 (no se aceptan instancias nulas).</param>
        /// <param name="i1">Inicio de la comparacin en cadena 1. Si supera el rango de la cadena, retorna true si i2 tambin supera el rango.</param>
        /// <param name="i2">Inicio de la comparacin en cadena 2. Si supera el rango de la cadena, retorna true si i1 tambin supera el rango.</param>
        /// <param name="l">Largo de la comparacin. Si es mayor al rango de las cadenas, se reajusta si el exceso es igual en ambas cadenas. Siempre retorna true si es 0.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>True si son iguales.</returns>
        public static unsafe bool Equal(string str1, string str2, int i1, int i2, int l, Ordinal comp)
        {
            if (l < 0 || i1 < 0 || i2 < 0)
                throw new ArgumentOutOfRangeException("l,i1,i2",Errors.NEGATIVE);
            if ((i1 != i2 || !Object.ReferenceEquals(str1, str2)) && l != 0)
            {
                int l1 = str1.Length;
                int l2 = str2.Length;
                if (i1 >= l1)
                    return i2 >= l2;
                if (i2 >= l2)
                    return false;
                int f1 = i1 + l - l1;
                int f2 = i2 + l - l2;
                if (f1 > 0 || f2 > 0)
                {
                    if (f2 != f1)
                        return false;
                    l -= f1;
                }
                fixed (char* p1 = str1, p2 = str2)
                    return Equal(p1 + i1, p2 + i2,false, l, comp);
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres.
        /// </summary>
        /// <param name="c1">Caracter 1.</param>
        /// <param name="c2">Caracter 2.</param>
        /// <returns>True si son iguales.</returns>
        public static bool Equal(char c1, char c2)
        {
            return c1 == c2;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres.
        /// Es un wrapper a Comp.Equal().
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo de los punteros.</param>
        /// <returns>True si son iguales.</returns>
        public static unsafe bool Equal(char* p1, char* p2, int l)
        {
            return Comp.Equal(p1, p2, l);
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres.
        /// </summary>
        /// <param name="str1">Cadena 1.</param>
        /// <param name="str2">Cadena 2.</param>
        /// <param name="nullEqualEmpty">Si true, null equivale a vacio.</param>
        /// <returns>True si son iguales.</returns>
        public static unsafe bool Equal(string str1, string str2,bool nullEqualEmpty)
        {
            if (!Object.ReferenceEquals(str1, str2))
            {
                if (str1 as object == null)
                    return nullEqualEmpty?(str2.Length == 0):false;
                if (str2 as object == null)
                    return nullEqualEmpty?(str1.Length == 0):false;
                int l = str1.Length;
                if (l != str2.Length)
                    return false;
                fixed (char* p1 = str1, p2 = str2)
                    return Comp.Equal(p1, p2, l);
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres.
        /// </summary>
        /// <param name="str1">Cadena 1 (no se aceptan instancias nulas).</param>
        /// <param name="str2">Cadena 2 (no se aceptan instancias nulas).</param>
        /// <param name="i1">Inicio de la comparacin en cadena 1. Si supera el rango de la cadena, retorna true si i2 tambin supera el rango.</param>
        /// <param name="i2">Inicio de la comparacin en cadena 2. Si supera el rango de la cadena, retorna true si i1 tambin supera el rango.</param>
        /// <param name="l">Largo de la comparacin. Si es mayor al rango de las cadenas, se reajusta si el exceso es igual en ambas cadenas. Siempre retorna true si es 0.</param>
        /// <returns>True si son iguales.</returns>
        public static unsafe bool Equal(string str1, string str2, int i1, int i2, int l)
        {
            if (l < 0 || i1 < 0 || i2 < 0)
                throw new ArgumentOutOfRangeException("l,i1,i2",Errors.NEGATIVE);
            if ((i1 != i2 || !Object.ReferenceEquals(str1, str2)) && l != 0)
            {
                int l1 = str1.Length;
                int l2 = str2.Length;
                if (i1 >= l1)
                    return i2 >= l2;
                if (i2 >= l2)
                    return false;
                int f1 = i1 + l - l1;
                int f2 = i2 + l - l2;
                if (f1 > 0 || f2 > 0)
                {
                    if (f2 != f1)
                        return false;
                    l -= f1;
                }
                fixed (char* p1 = str1, p2 = str2)
                    return Comp.Equal(p1 + i1, p2 + i2, l);
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres ignorando May/Min.
        /// La comparacin se efectuar de acuerdo a la cultura invariable.
        /// </summary>
        /// <param name="c1">Caracter 1.</param>
        /// <param name="c2">Caracter 2.</param>
        /// <returns>True si son iguales (ignorando May/Min).</returns>
        public static bool EqualNCInvariant(char c1, char c2)
        {
            if (c1 != c2)
                return Texts.ToUpperInvariant(c1) == Texts.ToUpperInvariant(c2);
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres ignorando May/Min.
        /// La comparacin se efectuar de acuerdo a la cultura invariable.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="p2Upper">Si true, el puntero 2 no ser convertido
        /// a maysculas en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="l">Largo de los punteros.</param>
        /// <returns>True si son iguales (ignorando May/Min).</returns>
        public static unsafe bool EqualNCInvariant(char* p1, char* p2, bool p2Upper, int l)
        {
            if (p1 != p2 && l != 0)
            {
                int a = 0;
                if (!p2Upper)
                {
                    do
                    {
                        if (p1[a] != p2[a] && Texts.ToUpperInvariant(p1[a]) != Texts.ToUpperInvariant(p2[a]))
                            return false;
                    } while (++a != l);
                }
                else
                {
                    do
                    {
                        if (Texts.ToUpperInvariant(p1[a]) != p2[a])
                            return false;
                    } while (++a != l);
                }
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres ignorando May/Min.
        /// La comparacin se efectuar de acuerdo a la cultura invariable.
        /// </summary>
        /// <param name="str1">Cadena 1.</param>
        /// <param name="str2">Cadena 2.</param>
        /// <param name="nullEqualEmpty">Si true, null equivale a vacio.</param>
        /// <returns>True si son iguales (ignorando May/Min).</returns>
        public static unsafe bool EqualNCInvariant(string str1, string str2, bool nullEqualEmpty)
        {
            if (!Object.ReferenceEquals(str1, str2))
            {
                if (str1 as object == null)
                    return nullEqualEmpty?(str2.Length == 0):false;
                if (str2 as object == null)
                    return nullEqualEmpty?(str1.Length == 0):false;
                int l = str1.Length;
                if (l != str2.Length)
                    return false;
                fixed (char* p1 = str1, p2 = str2)
                    return EqualNCInvariant(p1, p2,false, l);
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres ignorando May/Min.
        /// La comparacin se efectuar de acuerdo a la cultura invariable.
        /// </summary>
        /// <param name="str1">Cadena 1 (no se aceptan instancias nulas).</param>
        /// <param name="str2">Cadena 2 (no se aceptan instancias nulas).</param>
        /// <param name="i1">Inicio de la comparacin en cadena 1. Si supera el rango de la cadena, retorna true si i2 tambin supera el rango.</param>
        /// <param name="i2">Inicio de la comparacin en cadena 2. Si supera el rango de la cadena, retorna true si i1 tambin supera el rango.</param>
        /// <param name="l">Largo de la comparacin. Si es mayor al rango de las cadenas, se reajusta si el exceso es igual en ambas cadenas. Siempre retorna true si es 0.</param>
        /// <returns>True si son iguales (ignorando May/Min).</returns>
        public static unsafe bool EqualNCInvariant(string str1, string str2, int i1, int i2, int l)
        {
            if (l < 0 || i1 < 0 || i2 < 0)
                throw new ArgumentOutOfRangeException("l,i1,i2",Errors.NEGATIVE);
            if ((i1 != i2 || !Object.ReferenceEquals(str1, str2)) && l != 0)
            {
                int l1 = str1.Length;
                int l2 = str2.Length;
                if (i1 >= l1)
                    return i2 >= l2;
                if (i2 >= l2)
                    return false;
                int f1 = i1 + l - l1;
                int f2 = i2 + l - l2;
                if (f1 > 0 || f2 > 0)
                {
                    if (f2 != f1)
                        return false;
                    l -= f1;
                }
                fixed (char* p1 = str1, p2 = str2)
                    return EqualNCInvariant(p1 + i1, p2 + i2,false, l);
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres ignorando May/Min.
        /// La comparacin se efectuar de acuerdo a la tabla especificada.
        /// </summary>
        /// <param name="c1">Caracter 1.</param>
        /// <param name="c2">Caracter 2.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <returns>True si son iguales (ignorando May/Min).</returns>
        public static bool EqualNC(char c1, char c2, TableCase t)
        {
            if (c1 != c2)
                return Texts.ToUpper(c1, t) == Texts.ToUpper(c2, t);
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres ignorando May/Min.
        /// La comparacin se efectuar de acuerdo a la tabla especificada.
        /// </summary>
        /// <param name="p1">Puntero 1.</param>
        /// <param name="p2">Puntero 2.</param>
        /// <param name="l">Largo de los punteros.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <param name="p2Upper">Si true, el puntero 2 no ser convertido
        /// a maysculas en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <returns>True si son iguales (ignorando May/Min).</returns>
        public static unsafe bool EqualNC(char* p1, char* p2, bool p2Upper, int l, TableCase t)
        {
            if (p1 != p2 && l != 0)
            {
                int a = 0;
                if (!p2Upper)
                {
                    do
                    {
                        if (p1[a] != p2[a] && Texts.ToUpper(p1[a], t) != Texts.ToUpper(p2[a], t))
                            return false;
                    } while (++a != l);
                }
                else
                {
                    do
                    {
                        if (Texts.ToUpper(p1[a], t) != p2[a])
                            return false;
                    } while (++a != l);
                }
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres ignorando May/Min.
        /// La comparacin se efectuar de acuerdo a la tabla especificada.
        /// </summary>
        /// <param name="str1">Cadena 1.</param>
        /// <param name="str2">Cadena 2.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <param name="nullEqualEmpty">Si true, null equivale a vacio.</param>
        /// <returns>True si son iguales (ignorando May/Min).</returns>
        public static unsafe bool EqualNC(string str1, string str2, TableCase t, bool nullEqualEmpty)
        {
            if (!Object.ReferenceEquals(str1, str2))
            {
                if (str1 as object == null)
                    return nullEqualEmpty?(str2.Length == 0):false;
                if (str2 as object == null)
                    return nullEqualEmpty?(str1.Length == 0):false;
                int l = str1.Length;
                if (l != str2.Length)
                    return false;
                fixed (char* p1 = str1, p2 = str2)
                    return EqualNC(p1, p2,false, l, t);
            }
            return true;
        }

        /// <summary>
        /// Efectua una comparacin ordinal de carcteres ignorando May/Min.
        /// La comparacin se efectuar de acuerdo a la tabla especificada.
        /// </summary>
        /// <param name="str1">Cadena 1 (no se aceptan instancias nulas).</param>
        /// <param name="str2">Cadena 2 (no se aceptan instancias nulas).</param>
        /// <param name="i1">Inicio de la comparacin en cadena 1. Si supera el rango de la cadena, retorna true si i2 tambin supera el rango.</param>
        /// <param name="i2">Inicio de la comparacin en cadena 2. Si supera el rango de la cadena, retorna true si i1 tambin supera el rango.</param>
        /// <param name="l">Largo de la comparacin. Si es mayor al rango de las cadenas, se reajusta si el exceso es igual en ambas cadenas. Siempre retorna true si es 0.</param>
        /// <param name="t">
        /// Tabla de conversin May/Min. Para obtener la tabla correspondiente a la tarea actual, usar la propiedad TableCurrent.
        /// En el caso de Invariable se obtendr mayor rendimiento usando la funcin especifica.
        /// </param>
        /// <returns>True si son iguales (ignorando May/Min).</returns>
        public static unsafe bool EqualNC(string str1, string str2, int i1, int i2, int l, TableCase t)
        {
            if (l < 0 || i1 < 0 || i2 < 0)
                throw new ArgumentOutOfRangeException("l,i1,i2",Errors.NEGATIVE);
            if ((i1 != i2 || !Object.ReferenceEquals(str1, str2)) && l != 0)
            {
                int l1 = str1.Length;
                int l2 = str2.Length;
                if (i1 >= l1)
                    return i2 >= l2;
                if (i2 >= l2)
                    return false;
                int f1 = i1 + l - l1;
                int f2 = i2 + l - l2;
                if (f1 > 0 || f2 > 0)
                {
                    if (f2 != f1)
                        return false;
                    l -= f1;
                }
                fixed (char* p1 = str1, p2 = str2)
                    return EqualNC(p1 + i1, p2 + i2,false, l, t);
            }
            return true;
        }

        #endregion

        #region Comp Cadenas

        /// <summary>
        /// Realiza una comparacin rpida entre 3 cadenas.
        /// </summary>
        /// <param name="a">Cadena 1 (A o actual).</param>
        /// <param name="b">Cadena 2 (B o nueva).</param>
        /// <param name="c">Cadena 3 (C o testigo).</param>
        /// <param name="comp">Modo de comparacin ordinal.</param>
        /// <param name="breakEqualAB">
        /// Si true y las 3 cadenas son validas (no nulas) y a - b son iguales, retorna inmediatamente EqualsNoEmptys
        /// sin comparar con la tercer cadena, lo que implica que nunca retornar ABequalsCdiffer.
        /// </param>
        /// <returns>Enumerador TripleCompare.</returns>
        public static TripleCompare Compare(string a, string b, string c, bool breakEqualAB, Ordinal comp)
        {
            if (a != null && a.Length != 0)
            {
                if (b != null && b.Length != 0)
                {
                    if (c != null && c.Length != 0)
                    {
                        if (comp == Ordinal.Current)
                        {
                            TableCase t = Texts.TableCurrent;
                            if (EqualNC(a, b, t, true))
                            {
                                if (breakEqualAB || EqualNC(a, c, t, true))
                                    return TripleCompare.EqualsNoEmptys;
                                return TripleCompare.ABequalsCdiffer;
                            }
                            if (EqualNC(a, c, t, true))
                                return TripleCompare.ACequalsBdiffer;
                            if (EqualNC(b, c, t, true))
                                return TripleCompare.BCequalsAdiffer;
                            return TripleCompare.DiffersNoEmptys;
                        }
                        if (Equal(a, b, comp, true))
                        {
                            if (breakEqualAB || Equal(a, c, comp, true))
                                return TripleCompare.EqualsNoEmptys;
                            return TripleCompare.ABequalsCdiffer;
                        }
                        if (Equal(a, c, comp, true))
                            return TripleCompare.ACequalsBdiffer;
                        if (Equal(b, c, comp, true))
                            return TripleCompare.BCequalsAdiffer;
                        return TripleCompare.DiffersNoEmptys;
                    }
                    if (Equal(a, b, comp, true))
                        return TripleCompare.ABequalsCempty;
                    return TripleCompare.ABdiffersCempty;
                }
                if (c != null && c.Length != 0)
                {
                    if (Equal(a, c, comp, true))
                        return TripleCompare.ACequalsBempty;
                    return TripleCompare.ACdiffersBempty;
                }
                return TripleCompare.BCemptys;
            }
            if (b != null && b.Length != 0)
            {
                if (c != null && c.Length != 0)
                {
                    if (Equal(b, c, comp, true))
                        return TripleCompare.BCequalsAempty;
                    return TripleCompare.BCdiffersAempty;
                }
                return TripleCompare.ACemptys;
            }
            if (c != null && c.Length != 0)
                return TripleCompare.ABemptys;
            return TripleCompare.Emptys;
        }

        /// <summary>
        /// Realiza una comparacin rpida entre 2 cadenas.
        /// </summary>
        /// <param name="a">Cadena 1 (A).</param>
        /// <param name="b">Cadena 2 (B).</param>
        /// <param name="comp">Modo de comparacin ordinal.</param>
        /// <returns>Enumerador CompareStrings.</returns>
        public static CompareStrings Compare(string a, string b, Ordinal comp)
        {
            if (a != null && a.Length != 0)
            {
                if (b != null && b.Length != 0)
                {
                    if (Equal(a, b, comp, true))
                        return CompareStrings.EqualsNoEmptys;
                    return CompareStrings.DiffersNoEmptys;
                }
                return CompareStrings.Bempty;
            }
            if (b != null && b.Length != 0)
                return CompareStrings.Aempty;
            return CompareStrings.Emptys;
        }

        #endregion

        #region IndexOf Carcteres

        /// <summary>
        /// Busca carcteres no dgitos.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="hexMode">Modo Hex.</param>
        /// <returns>-1 o ubicacin.</returns>
        public unsafe static int IndexOfNoDigit(char* p, int l, bool hexMode)
        {
            int a = 0;
            while (a < l)
            {
                switch(p[a])
                {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        a++;
                        continue;
                    case 'A':
                    case 'B':
                    case 'C':
                    case 'D':
                    case 'E':
                    case 'F':
                    case 'a':
                    case 'b':
                    case 'c':
                    case 'd':
                    case 'e':
                    case 'f':
                        if (hexMode)
                        {
                            a++;
                            continue;
                        }
                        return a;
                    default:
                        return a;
                }
                
            }
            return -1;
        }

        /// <summary>
        /// Busca carcteres blancos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 o ubicacin.</returns>
        public unsafe static int IndexOfBlank(char* p, int l)
        {
            int a = 0;
            while (a < l)
            {
                switch(p[a])
                {
                    case ' ':
                    case '\t':
                    case '\n':
                    case '\r':
                    case '\xa0':
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        return a;
                }
                a++;
            }
            return -1;
        }


        /// <summary>
        /// Busca carcteres blancos en memoria.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro el taburador (\t).</param>
        /// <returns>Ubicacin o -1.</returns>
        public unsafe static int IndexOfBlank(char*p, int l, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control)
        {
            if (l>0)
            {
                int a=0;
                do
                {
                    switch (p[a])
                    {

                        case ' ':
                            if (sp) 
                                return a;
                            continue;
                        case '\t':
                            if (tab) 
                                return a;
                            continue;
                        case '\n':
                        case '\r':
                            if (crlf) 
                                return a;
                            continue;
                        case '\xa0':
                        case '\x85':
                            if (lat1) 
                                return a;
                            continue;
                        case '\0':
                        case '\v':
                        case '\f':
                            if (control) 
                                return a;
                            continue;
                        case '\u1680':
                        case '\u2000':
                        case '\u2001':
                        case '\u2002':
                        case '\u2003':
                        case '\u2004':
                        case '\u2005':
                        case '\u2006':
                        case '\u2007':
                        case '\u2008':
                        case '\u2009':
                        case '\u200a':
                        case '\u200b':
                        case '\u2028':
                        case '\u2029':
                        case '\u202f':
                        case '\u3000':
                        case '\ufeff':
                            if (uni) 
                                return a;
                            continue;
                    }
                }while(++a!=l);
            }
            return -1;
        }

        /// <summary>
        /// Busca el primer carcter que se encuadra en el grupo especificado.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <returns>Ubicacin o -1.</returns>
        public unsafe static int IndexOfGroup(char* p, int l, CharGroup group, char*pch, int lch)
        {

            if (l>0 && group!=0)
            {
                int a=0;
                do
                {
                    if (Texts.InGroup(p[a],group,pch,lch))
                        return a;
                }while(++a!=l);
            }
            return -1;
        }



        /// <summary>
        /// Busca el primer carcter con la categora unicode especificada.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="uniCat">Categora Unicode.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int IndexOfCategory(char*p, int l, UnicodeCategory uniCat)
        {
            if (l>0)
            {
                int a=0;
                char c;
                do
                {
                    c=p[a];
                    if (c < '\x218')
                    {
                        if ((UnicodeCategory)Texts.CatsUnicode536[c]==uniCat)
                            return a;
                    }
                    else if (Char.GetUnicodeCategory(c)==uniCat)
                        return a;
                }while(++a!=l);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer letra.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int IndexOfLetter(char*p, int l)
        {
            if (l>0)
            {
                int a=0;
                char c;
                do
                {
                    c=p[a];
                    if (c <= 'z')
                    {
                        if (c >= 'a' || (c <= 'Z' && c >= 'A'))
                            return a;
                    }
                    else if (c < '\x218')
                    {
                        if (Texts.IsLetter((UnicodeCategory)Texts.CatsUnicode536[c]))
                            return a;
                    }
                    else if (Texts.IsLetter(Char.GetUnicodeCategory(c)))
                        return a;
                }while(++a!=l);
            }
            return -1;
        }

        /// <summary>
        /// Busca el primer carcter numrico.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int IndexOfNumber(char*p, int l)
        {
            if (l>0)
            {
                int a=0;
                char c;
                do
                {
                    c=p[a];
                    if (c <= '9')
                    {
                        if (c >= '0')
                            return a;
                    }
                    else if (c < '\x218')
                    {
                        if (Texts.IsNumber((UnicodeCategory)Texts.CatsUnicode536[c]))
                            return a;
                    }
                    else if (Texts.IsNumber(Char.GetUnicodeCategory(c)))
                        return a;
                }while(++a!=l);
            }
            return -1;
        }

        /// <summary>
        /// Busca el primer carcter dgito.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int IndexOfDigit(char*p, int l)
        {
            if (l>0)
            {
                int a=0;
                char c;
                do
                {
                    c=p[a];
                    if (c < '\x218')
                    {
                        if (c <= '9' && c >= '0')
                            return a;
                    }
                    else if (Texts.IsDigit(Char.GetUnicodeCategory(c)))
                        return a;
                }while(++a!=l);
            }
            return -1;

        }

        /// <summary>
        /// Busca el primer carcter letra o nmero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int IndexOfLetterNumber(char*p, int l)
        {
            if (l>0)
            {
                int a=0;
                char c;
                do
                {
                    c=p[a];
                    if (c <= 'z')
                    {
                        if (c >= 'a' || (c <= 'Z' && c >= 'A') || (c <= '9' && c >= '0'))
                            return a;
                    }
                    else if (c < '\x218')
                    {
                        if (Texts.IsLetterNumber((UnicodeCategory)Texts.CatsUnicode536[c]))
                            return a;
                    }
                    else if (Texts.IsLetterNumber(Char.GetUnicodeCategory(c)))
                        return a;
                }while(++a!=l);
            }
            return -1;
        }

        /// <summary>
        /// Busca el primer carcter letra o dgito.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int IndexOfLetterDigit(char*p, int l)
        {
            if (l>0)
            {
                int a=0;
                char c;
                do
                {
                    c=p[a];
                    if (c <= 'z')
                    {
                        if (c >= 'a' || (c <= 'Z' && c >= 'A') || (c <= '9' && c >= '0'))
                            return a;
                    }
                    else if (c < '\x218')
                    {
                        if (Texts.IsLetterDigit((UnicodeCategory)Texts.CatsUnicode536[c]))
                            return a;
                    }
                    else if (Texts.IsLetterDigit(Char.GetUnicodeCategory(c)))
                        return a;
                }while(++a!=l);
            }
            return -1;
        }

        #endregion

        #region IndexOf Carcteres Ultimo
        /// <summary>
        /// Busca carcteres no dgitos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="hexMode">Modo Hex.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin relativa a p.</returns>
        public unsafe static int LastIndexOfNoDigit(char* p, int l, bool hexMode)
        {
            while (--l > -1) 
            {
                switch(p[l])
                {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        continue;
                    case 'A':
                    case 'B':
                    case 'C':
                    case 'D':
                    case 'E':
                    case 'F':
                    case 'a':
                    case 'b':
                    case 'c':
                    case 'd':
                    case 'e':
                    case 'f':
                        if (hexMode)
                            continue;
                        return l;
                    default:
                        return l;

                }
            }
            return -1;
        }


        /// <summary>
        /// Busca carcteres blancos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>-1 o ubicacin de la ltima aparicin relativa a p.</returns>
        public unsafe static int LastIndexOfBlank(char* p, int l)
        {
            while (--l > -1) 
            {
                switch(p[l])
                {
                    case ' ':
                    case '\t':
                    case '\n':
                    case '\r':
                    case '\xa0':
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        return l;
                }
            }
            return -1;
        }

        /// <summary>
        /// Busca carcteres blancos hacia atras.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro el taburador (\t).</param>
        /// <returns>Ubicacin o -1.</returns>
        public unsafe static int LastIndexOfBlank(char*p, int l, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control)
        {
            while(--l>-1)
            {
                switch (p[l])
                {

                    case ' ':
                        if (sp) 
                            return l;
                        continue;
                    case '\t':
                        if (tab) 
                            return l;
                        continue;
                    case '\n':
                    case '\r':
                        if (crlf) 
                            return l;
                        continue;
                    case '\xa0':
                    case '\x85':
                        if (lat1) 
                            return l;
                        continue;
                    case '\0':
                    case '\v':
                    case '\f':
                        if (control) 
                            return l;
                        continue;
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if (uni) 
                            return l;
                        continue;
                }
            }
            return -1;
        }


        /// <summary>
        /// Busca el ltimo carcter que se encuadra en el grupo especificado.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <returns>Ubicacin o -1.</returns>
        public unsafe static int LastIndexOfGroup(char* p, int l, CharGroup group, char*pch, int lch)
        {

            if (group!=0)
            {
                while(--l>-1)
                {
                    if (Texts.InGroup(p[l],group,pch,lch))
                        return l;
                }
            }
            return -1;
        }


        /// <summary>
        /// Busca el ltimo carcter con la categora unicode especificada.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="uniCat">Categora Unicode.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int LastIndexOfCategory(char*p, int l, UnicodeCategory uniCat)
        {
            char c;
            while(--l>-1)
            {
                c=p[l];
                if (c < '\x218')
                {
                    if ((UnicodeCategory)Texts.CatsUnicode536[c]==uniCat)
                        return l;
                }
                else if (Char.GetUnicodeCategory(c)==uniCat)
                    return l;
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltimo letra.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int LastIndexOfLetter(char*p, int l)
        {
            char c;
            while(--l>-1)
            {
                c=p[l];
                if (c <= 'z')
                {
                    if (c >= 'l' || (c <= 'Z' && c >= 'A'))
                        return l;
                }
                else if (c < '\x218')
                {
                    if (Texts.IsLetter((UnicodeCategory)Texts.CatsUnicode536[c]))
                        return l;
                }
                else if (Texts.IsLetter(Char.GetUnicodeCategory(c)))
                    return l;
            }
            return -1;
        }

        /// <summary>
        /// Busca el ltimo carcter numrico.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int LastIndexOfNumber(char*p, int l)
        {
            char c;
            while(--l>-1)
            {
                c=p[l];
                if (c <= '9')
                {
                    if (c >= '0')
                        return l;
                }
                else if (c < '\x218')
                {
                    if (Texts.IsNumber((UnicodeCategory)Texts.CatsUnicode536[c]))
                        return l;
                }
                else if (Texts.IsNumber(Char.GetUnicodeCategory(c)))
                    return l;
            }
            return -1;
        }

        /// <summary>
        /// Busca el ltimo carcter dgito.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int LastIndexOfDigit(char*p, int l)
        {
            char c;
            while(--l>-1)
            {
                c=p[l];
                if (c < '\x218')
                {
                    if (c <= '9' && c >= '0')
                        return l;
                }
                else if (Texts.IsDigit(Char.GetUnicodeCategory(c)))
                    return l;
            }
            return -1;

        }

        /// <summary>
        /// Busca el ltimo carcter letra o nmero.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int LastIndexOfLetterNumber(char*p, int l)
        {
            char c;
            while(--l>-1)
            {
                c=p[l];
                if (c <= 'z')
                {
                    if (c >= 'l' || (c <= 'Z' && c >= 'A') || (c <= '9' && c >= '0'))
                        return l;
                }
                else if (c < '\x218')
                {
                    if (Texts.IsLetterNumber((UnicodeCategory)Texts.CatsUnicode536[c]))
                        return l;
                }
                else if (Texts.IsLetterNumber(Char.GetUnicodeCategory(c)))
                    return l;
            }
            return -1;
        }

        /// <summary>
        /// Busca el ltimo carcter letra o dgito.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>Ubicacin o -1.</returns>
        public static unsafe int LastIndexOfLetterDigit(char*p, int l)
        {
            char c;
            while(--l>-1)
            {
                c=p[l];
                if (c <= 'z')
                {
                    if (c >= 'l' || (c <= 'Z' && c >= 'A') || (c <= '9' && c >= '0'))
                        return l;
                }
                else if (c < '\x218')
                {
                    if (Texts.IsLetterDigit((UnicodeCategory)Texts.CatsUnicode536[c]))
                        return l;
                }
                else if (Texts.IsLetterDigit(Char.GetUnicodeCategory(c)))
                    return l;
            }
            return -1;
        }


  
        #endregion

        #region IndexOf

        /// <summary>
        /// Busqueda ordinal de cualquiera de los carcteres en la cadena de bsqueda.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int IndexOfAny(string str, Ordinal comp, bool last,params char[] chars)
        {
            return IndexOfAny(str,0,-1,comp,last,chars);

        }

        /// <summary>
        /// Busqueda ordinal de cualquiera de los carcteres en la cadena de bsqueda.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static int IndexOfAny(string str, int i, int l, Ordinal comp, bool last, params char[] chars)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            
            int lf = chars.Length;
            
            if (lf < 2)
            {
                if (lf == 1)
                    return IndexOf(str,i,l, chars[0], comp, last);
                return -1;
            }

            int lc = str.Length;

            if (l == -1 || i + l > lc)
                l = lc - i;

            if (l<1)
                return -1;


            unsafe
            {
                if (l == 1)
                {
                    fixed (char* pf = chars)
                    {
                        if (IndexOf(pf, lf, str[i], comp, false) != -1)
                            return i;
                        return -1;
                    }
                }
                fixed (char* p = str, pf = chars)
                {
                    l=IndexOfAny(p+i, l, pf, lf,false, comp, last);
                }
            }
            if (l != -1)
                return l+i;
            return l;
        }

        /// <summary>
        /// Busqueda ordinal de cualquiera de los carcteres en el puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="isUpper">Si true, el puntero de bsqueda no ser convertido
        /// a maysculas (ni ser copiado a stack) en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static unsafe int IndexOfAny(char* p, int l, char* pf, int lf, bool isUpper, Ordinal comp, bool last)
        {
            if (comp == Ordinal.Binary)
            {
                if (!last)
                    return Comp.IndexOfAny(p, l, pf, lf,false);
                return Comp.LastIndexOfAny(p, l, pf, lf,false);
            }

            if (lf < 2)
            {
                if (lf == 1)
                    return IndexOf(p, l, *pf, comp, last);
                return -1;
            }

            if (l < 2)
            {
                if (l == 1 && IndexOf(pf, lf, *p,  comp, false) != -1)
                    return 0;
                return -1;
            }


            TableCase t = comp==Ordinal.Invariant?TableCase.Invariant:Texts.TableCurrent;

            Mem.Heap heapf;
            if (!isUpper)
            {
                if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                {
                    char* ptmp = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp, lf, t);
                    pf = ptmp;
                    heapf = null;
                }
                else
                {
                    heapf = new Mem.Heap(lf << 1, false);
                    Texts.ToUpper(pf, (char*)heapf.Pointer, lf, t);
                    pf = (char*)heapf.Pointer;
                }
            }
            else
                heapf = null;

            int a,z;
            char c;
            if (comp == Ordinal.Invariant)
            {
                if (!last)
                {
                    a = 0;
                    do
                    {
                        z = 0;
                        c = Texts.ToUpperInvariant(p[a]);
                        do
                        {
                            if (pf[z] == c)
                                goto fin;
                        } while (++z != lf);
                    } while (++a != l);
                    a = -1;
                }
                else
                {
                    while (--l != -1)
                    {
                        z = 0;
                        c = Texts.ToUpperInvariant(p[l]);
                        do
                        {
                            if (pf[z] == c)
                            {
                                a = l;
                                goto fin;
                            }
                        } while (++z != lf);
                    }
                    a = l;
                }
            }
            else 
            {
                if (!last)
                {
                    a = 0;
                    do
                    {
                        z = 0;
                        c = Texts.ToUpper(p[a],t);
                        do
                        {
                            if (pf[z] == c)
                                goto fin;
                        } while (++z != lf);
                    } while (++a != l);
                    a = -1;
                }
                else
                {
                    while (--l != -1)
                    {
                        z = 0;
                        c = Texts.ToUpper(p[l],t);
                        do
                        {
                            if (pf[z] == c)
                            {
                                a = l;
                                goto fin;
                            }
                        } while (++z != lf);
                    }
                    a = l;
                }

            }
            fin:
            if (heapf != null)
                heapf.Dispose();
            return a;

        }

        /// <summary>
        /// Busqueda ordinal de un caracter en la cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del caracter.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static unsafe int IndexOf(char* p, int l, char f, Ordinal comp, bool last)
        {
            if (comp == Ordinal.Binary)
            {
                if (!last)
                    return Comp.IndexOf(p, l, f);
                return Comp.LastIndexOf(p, l, f);
            }

            if (l < 2)
            {
                if (l == 1 && Equal(*p, f, comp))
                    return 0;
                return -1;
            }

            int a;
            if (comp == Ordinal.Invariant)
            {
                f = Texts.ToUpperInvariant(f);
                    
                if (!last)
                {
                    a = 0;
                    do
                    {
                        if (Texts.ToUpperInvariant(p[a]) == f)
                            return a;
                    } while (++a != l);
                    return -1;
                }
                while (--l != -1 && Texts.ToUpperInvariant(p[l]) != f);
                return l;
            }
            TableCase t = Texts.TableCurrent;
            f = Texts.ToUpper(f, t);
                
            if (!last)
            {
                a = 0;
                do
                {
                    if (Texts.ToUpper(p[a],t) == f)
                        return a;
                } while (++a != l);
                return -1;
            }
            while (--l != -1 && Texts.ToUpper(p[l],t) != f)
                ;
            return l;
        }


        /// <summary>
        /// Busqueda ordinal de una subcadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="isUpper">Si true, el puntero de bsqueda no ser convertido
        /// a maysculas (ni ser copiado a stack) en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la subcadena.</param>
        /// <returns>-1 o ubicacin relativa a p.</returns>
        public static unsafe int IndexOf(char* p, int l, char* pf, int lf, bool isUpper, Ordinal comp, bool last)
        {

            if (comp == Ordinal.Binary)
            {
                if (!last)
                    return Comp.IndexOf(p, l, pf, lf, false);
                return Comp.LastIndexOf(p, l, pf, lf, false);
            }

            if (lf < 2)
            {
                if (lf == 1)
                    return IndexOf(p, l, *pf, comp, last);
                return -1;
            }
            if (lf >= l)
            {
                if (lf > l)
                    return -1;
                return Equal(p, pf,isUpper, l, comp) ? 0 : -1;
            }

            TableCase t =comp==Ordinal.Invariant?TableCase.Invariant: Texts.TableCurrent;
            Mem.Heap heap, heapf;
            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                General.Copy(p, ptmp, l);
                p = ptmp;
                heap = null;
                if (!isUpper)
                {
                    char* ptmp2 = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp2, lf, t);
                    pf = ptmp2;
                }
                heapf = null;
            }
            else
            {
                heap = new EnPalermo.Mem.Heap(l << 1, false);
                General.Copy(p, (char*)heap.Pointer, l);
                p = (char*)heap.Pointer;
                if (!isUpper)
                {
                    if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                    {
                        char* ptmp = stackalloc char[lf];
                        Texts.ToUpper(pf, ptmp, lf, t);
                        pf = ptmp;
                        heapf = null;
                    }
                    else
                    {
                        heapf = new EnPalermo.Mem.Heap(lf << 1, false);
                        Texts.ToUpper(pf, (char*)heapf.Pointer, lf, t);
                        pf = (char*)heapf.Pointer;
                    }
                }
                else
                    heapf = null;
            }

            int a;
            int b;
            if (comp == Ordinal.Invariant)
            {
                if (!last)
                {
                    b = lf - 1;
                    Texts.ToUpperInvariant(p, b);
                    a = 0;
                    do
                    {
                        p[b] = Texts.ToUpperInvariant(p[b]);
                        if (Comp.Equal(p + a, pf, lf))
                            break;
                        else if (++a == l)
                        {
                            a = -1;
                            break;
                        }
                        b++;
                    } while (true);
                }
                else
                {
                    a = l - lf;
                    b = a + 1;
                    Texts.ToUpperInvariant(p + b, lf - 1);
                    do
                    {
                        p[--b] = Texts.ToUpperInvariant(p[b]);
                        if (Comp.Equal(p + a, pf, lf))
                            break;
                    } while (--a != -1);
                }
            }
            else
            {
                if (!last)
                {
                    b = lf - 1;
                    Texts.ToUpper(p, b,t);
                    a = 0;
                    do
                    {
                        p[b] = Texts.ToUpper(p[b],t);
                        if (Comp.Equal(p + a, pf, lf))
                            break;
                        else if (++a == l)
                        {
                            a = -1;
                            break;
                        }
                        b++;
                    } while (true);
                }
                else
                {
                    a = l - lf;
                    b = a + 1;
                    Texts.ToUpper(p + b, lf - 1,t);
                    do
                    {
                        p[--b] = Texts.ToUpper(p[b],t);
                        if (Comp.Equal(p + a, pf, lf))
                            break;
                    } while (--a != -1);
                }
            }
            if (heap != null)
            {
                heap.Dispose();
                if (heapf != null)
                    heapf.Dispose();
            }
            return a;
        }

        /// <summary>
        /// Busqueda ordinal de un caracter en la cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del caracter.</param>
        /// <returns>-1 o ubicacin relativa al inicio de la cadena.</returns>
        public static int IndexOf(string str, char f, Ordinal comp, bool last)
        {
            int l = str.Length;
            if (l == 0)
                return -1;
            unsafe
            {
                fixed (char* p = str)
                    return IndexOf(p, l, f, comp, last);
            }
        }

        /// <summary>
        /// Busqueda ordinal de un caracter en la cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="last">Si true, buscar la ltima ubicacin del caracter.</param>
        /// <returns>-1 o ubicacin relativa al inicio de la cadena.</returns>
        public static int IndexOf(string str,int i,int l, char f, Ordinal comp, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            int lc = str.Length;
            if (l ==-1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return -1;

            unsafe
            {
                fixed (char* p = str)
                    l=IndexOf(p+i, l, f, comp, last);
            }
            if (l != -1)
                return l+i;
            return l;
        }

        /// <summary>
        /// Busqueda ordinal de una subcadena en la cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la subcadena.</param>
        /// <returns>-1 o ubicacin relativa al inicio de la cadena.</returns>
        public static int IndexOf(string str, string f, Ordinal comp, bool last)
        {
            int lf = f.Length;
            if (lf < 2)
            {
                if (lf == 1)
                    return IndexOf(str, f[0], comp, last);
                return -1;
            }
            int l = str.Length;
            if (l < lf)
                return -1;
            unsafe
            {
                fixed (char* p = str, pf=f)
                    return IndexOf(p, l, pf,lf,false, comp, last);
            }
        }

        /// <summary>
        /// Busqueda ordinal de una subcadena en la cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la subcadena.</param>
        /// <returns>-1 o ubicacin relativa al inicio de la cadena.</returns>
        public static int IndexOf(string str, int i, int l, string f, Ordinal comp, bool last)
        {
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);

            int lf = f.Length;
            if (lf < 2)
            {
                if (lf == 1)
                    return IndexOf(str, i , l , f[0], comp, last);
                return -1;
            }
            int lc = str.Length;
            if (l ==-1 || i + l > lc)
                l = lc - i;
            if (l < lf)
                return -1;
            unsafe
            {
                fixed (char* p = str, pf = f)
                    l= IndexOf(p+i, l, pf, lf,false, comp, last);
            }
            if (l != -1)
                return l+i;
            return l;
        }

        #endregion

        #region Exist

        /// <summary>
        /// Busqueda ordinal de cualquiera de los carcteres en la cadena de bsqueda.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin.</param>
        /// <returns>True si existe.</returns>
        public static bool ExistAny(string str, Ordinal comp, bool last, params char[] chars)
        {
            return IndexOfAny(str, comp, last, chars) != -1;
        }

        /// <summary>
        /// Busqueda ordinal de cualquiera de los carcteres en la cadena de bsqueda.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <returns>True si existe.</returns>
        public static bool ExistAny(string str, int i, int l, Ordinal comp, bool last, params char[] chars)
        {
            return IndexOfAny(str,i,l, comp, last, chars) != -1;
        }

        /// <summary>
        /// Busqueda ordinal de cualquiera de los carcteres en el puntero de bsqueda.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="isUpper">Si true, el puntero de bsqueda no ser convertido
        /// a maysculas (ni ser copiado a stack) en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin.</param>
        /// <returns>True si existe.</returns>
        public static unsafe bool ExistAny(char* p, int l, char* pf, int lf, bool isUpper, Ordinal comp, bool last)
        {
            return IndexOfAny(p, l,pf,lf,isUpper, comp, last) != -1;
        }

        /// <summary>
        /// Busqueda ordinal de un caracter en la cadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del caracter.</param>
        /// <returns>True si existe.</returns>
        public static unsafe bool Exist(char* p, int l, char f, Ordinal comp, bool last)
        {
            return IndexOf(p, l, f, comp, last) != -1;
        }


        /// <summary>
        /// Busqueda ordinal de una subcadena.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="isUpper">Si true, el puntero de bsqueda no ser convertido
        /// a maysculas (ni ser copiado a stack) en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la subcadena.</param>
        /// <returns>True si existe.</returns>
        public static unsafe bool Exist(char* p, int l, char* pf, int lf,bool isUpper, Ordinal comp, bool last)
        {
            return IndexOf(p, l, pf,lf,isUpper, comp, last) != -1;
        }

        /// <summary>
        /// Busqueda ordinal de un caracter en la cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin del caracter.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(string str, char f, Ordinal comp, bool last)
        {
            return IndexOf(str,f, comp, last) != -1;
        }

        /// <summary>
        /// Busqueda ordinal de un caracter en la cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="last">Si true, buscar la ltima ubicacin del caracter.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(string str, int i, int l, char f, Ordinal comp, bool last)
        {
            return IndexOf(str,i,l, f, comp, last) != -1;
        }

        /// <summary>
        /// Busqueda ordinal de una subcadena en la cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la subcadena.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(string str, string f, Ordinal comp, bool last)
        {
            return IndexOf(str, f, comp, last) != -1;            
        }

        /// <summary>
        /// Busqueda ordinal de una subcadena en la cadena.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la subcadena.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(string str, int i, int l, string f, Ordinal comp, bool last)
        {
            return IndexOf(str,i,l, f, comp, last) != -1;
        }

        #endregion

        #region Count

        /// <summary>
        /// Contabiliza las ocurrencias de cualquiera de los carcteres especificados mediante bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Ocurrencias.</returns>
        public static int CountAny(string str, Ordinal comp, params char[] chars)
        {
            return CountAny(str,0,-1,comp,chars);
        }

        /// <summary>
        /// Contabiliza las ocurrencias de cualquiera de los carcteres especificados mediante bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <returns>Ocurrencias.</returns>
        public static int CountAny(string str, int i, int l, Ordinal comp, params char[] chars)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);

            int lf = chars.Length;
            if (lf < 2)
            {
                if (lf == 1)
                    return Count(str, i, l, chars[0], comp);
                return 0;
            }

            int lc = str.Length;

            if (l ==-1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return 0;

            unsafe
            {
                if (l == 1)
                {
                    fixed (char* pf = chars)
                    {
                        if (IndexOf(pf, lf, str[i], comp, false) != -1)
                            return 1;
                        return 0;
                    }
                }
                fixed (char* p = str, pf = chars)
                {
                    return CountAny(p + i, l, pf, lf,false, comp);
                }
            }
        }

        /// <summary>
        /// Contabiliza las ocurrencias de cualquiera de los carcteres especificados mediante bsqueda ordinal.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="isUpper">Si true, el puntero de bsqueda no ser convertido
        /// a maysculas (ni ser copiado a stack) en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Ocurrencias.</returns>
        public static unsafe int CountAny(char* p, int l, char* pf, int lf, bool isUpper, Ordinal comp)
        {
            if (comp == Ordinal.Binary)
            {
                return Comp.CountAny(p, l, pf, lf,false);
            }

            if (lf < 2)
            {
                if (lf == 1)
                    return Count(p, l, *pf, comp);
                return 0;
            }

            if (l < 2)
            {
                if (l == 1 && IndexOf(pf, lf, *p, comp, false) != -1)
                    return 1;
                return 0;
            }

            TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;

            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                Texts.ToUpper(p, ptmp, l, t);
                if (!isUpper)
                {
                    char* ptmp2 = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp2, lf, t);
                    return Comp.CountAny(ptmp, l, ptmp2, lf,true);
                }
                return Comp.CountAny(ptmp, l, pf, lf,false);
            }
            Mem.Heap heap = new EnPalermo.Mem.Heap(l << 1, false);
            Texts.ToUpper(p, (char*)heap.Pointer, l, t);
            if (!isUpper)
            {
                if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                {
                    char* ptmp = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp, lf, t);
                    l = Comp.CountAny((char*)heap.Pointer, l, ptmp, lf,true);
                    heap.Dispose();
                    return l;
                }

                Mem.Heap heapf = new EnPalermo.Mem.Heap(lf << 1, false);
                Texts.ToUpper(pf, (char*)heapf.Pointer, lf, t);
                l = Comp.CountAny((char*)heap.Pointer, l, (char*)heapf.Pointer, lf,true);
                heap.Dispose();
                heapf.Dispose();
                return l;
            }
            l = Comp.CountAny((char*)heap.Pointer, l, pf, lf,false);
            heap.Dispose();
            return l;
        }

        /// <summary>
        /// Contabiliza las ocurrencias de un caracter en la cadena mediante bsqueda ordinal.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Ocurrencias.</returns>
        public static unsafe int Count(char* p, int l, char f, Ordinal comp)
        {
            if (comp == Ordinal.Binary)
            {
                return Comp.Count(p, l, f);
            }

            if (l < 2)
            {
                if (l == 1 && Equal(*p, f, comp))
                    return 1;
                return 0;
            }


            TableCase t = comp==Ordinal.Invariant?TableCase.Invariant: Texts.TableCurrent;
            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                Texts.ToUpper(p, ptmp, l, t);
                return Comp.Count(ptmp, l, Texts.ToUpper(f, t));
            }
            Mem.Heap heap = new EnPalermo.Mem.Heap(l << 1, false);
            Texts.ToUpper(p, (char*)heap.Pointer, l, t);
            l = Comp.Count((char*)heap.Pointer, l, Texts.ToUpper(f, t));
            heap.Dispose();
            return l;
        }


        /// <summary>
        /// Contabiliza las ocurrencias de un caracter en la cadena mediante bsqueda ordinal.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="isUpper">Si true, el puntero de bsqueda no ser convertido
        /// a maysculas (ni ser copiado a stack) en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Ocurrencias.</returns>
        public static unsafe int Count(char* p, int l, char* pf, int lf, bool isUpper, Ordinal comp)
        {
            if (comp == Ordinal.Binary)
            {
                return Comp.Count(p, l, pf, lf,false);
            }

            if (lf < 2)
            {
                if (lf == 1)
                    return Count(p, l, *pf, comp);
                return 0;
            }
            if (lf >= l)
            {
                if (lf == l && Equal(p, pf,isUpper, l, comp))
                    return 1;
                return 0;
            }

            
            TableCase t = comp==Ordinal.Invariant?TableCase.Invariant:Texts.TableCurrent;

            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                Texts.ToUpper(p, ptmp, l, t);
                if (!isUpper)
                {
                    char* ptmp2 = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp2, lf, t);
                    return Comp.Count(ptmp, l, ptmp2, lf,true);
                }
                return Comp.Count(ptmp, l, pf, lf,false);
            }
            Mem.Heap heap = new EnPalermo.Mem.Heap(l << 1, true);
            Texts.ToUpper(p, (char*)heap.Pointer, l, t);
            if (!isUpper)
            {
                if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                {
                    char* ptmp = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp, lf, t);
                    l = Comp.Count((char*)heap.Pointer, l, ptmp, lf,true);
                    heap.Dispose();
                    return l;
                }
                Mem.Heap heapf = new EnPalermo.Mem.Heap(lf << 1, false);
                Texts.ToUpper(pf, (char*)heapf.Pointer, lf, t);
                l = Comp.Count((char*)heap.Pointer, l, (char*)heapf.Pointer, lf,true);
                heap.Dispose();
                heapf.Dispose();
                return l;
            }
            l = Comp.Count((char*)heap.Pointer, l, pf, lf,false);
            heap.Dispose();
            return l;
        }

        /// <summary>
        /// Contabiliza las ocurrencias de un caracter en la cadena mediante bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Ocurrencias.</returns>
        public static int Count(string str, char f, Ordinal comp)
        {
            int l = str.Length;
            if (l == 0)
                return 0;
            unsafe
            {
                fixed (char* p = str)
                    return Count(p, l, f, comp);
            }
        }

        /// <summary>
        /// Contabiliza las ocurrencias de un caracter en la cadena mediante bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <returns>Ocurrencias.</returns>
        public static int Count(string str, int i, int l, char f, Ordinal comp)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return 0;
            unsafe
            {
                fixed (char* p = str)
                    return Count(p + i, l, f, comp);
            }
        }

        /// <summary>
        /// Contabiliza las ocurrencias de una subcadena en la cadena mediante bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Ocurrencias.</returns>
        public static int Count(string str, string f, Ordinal comp)
        {
            return Count(str,0,-1,f,comp);
        }

        /// <summary>
        /// Contabiliza las ocurrencias de una subcadena en la cadena mediante bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <returns>Ocurrencias.</returns>
        public static int Count(string str, int i, int l, string f, Ordinal comp)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);

            int lf = f.Length;
            if (lf < 2)
            {
                if (lf == 1)
                    return Count(str, i, l, f[0], comp);
                return 0;
            }
            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l < lf)
                return 0;
            unsafe
            {
                fixed (char* p = str, pf = f)
                    return Count(p + i, l, pf, lf,false, comp);
            }
        }

        #endregion

        #region Locations

        /// <summary>
        /// Contabiliza las ocurrencias de cualquiera de los carcteres especificados mediante bsqueda ordinal
        /// y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo de la bsqueda es mayor a cero.
        /// </param>
        /// <param name="str">Cadena.</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>
        /// Ints nuevo o actualizado donde se almacenarn las ubicaciones.
        /// Si el Ints de entrada es nulo, la salida tambin podra serla si el largo de la bsqueda es cero.
        /// </returns>
        public static Ints LocationsAny(Ints ret, string str, Ordinal comp, params char[] chars)
        {
            return LocationsAny(ret,str,0,-1,comp,chars);
        }

        /// <summary>
        /// Contabiliza las ocurrencias de cualquiera de los carcteres especificados mediante bsqueda ordinal
        /// y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo de la bsqueda es mayor a cero.
        /// </param>
        /// <param name="str">Cadena.</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <returns>
        /// Ints nuevo o actualizado donde se almacenarn las ubicaciones.
        /// Si el Ints de entrada es nulo, la salida tambin podra serla si el largo de la bsqueda es cero.
        /// </returns>
        public static Ints LocationsAny(Ints ret, string str, int i, int l, Ordinal comp, params char[] chars)
        {
            
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            
            int lf = chars.Length;
            
            if (lf < 2)
            {
                if (lf == 1)
                    return Locations(ret,str, i, l, chars[0], comp);
                return (ret==null && l!=0)?new Ints():ret;
            }

            int lc = str.Length;

            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return ret;

            unsafe
            {
                fixed (char* p = str, pf = chars)
                    return LocationsAny(ret,i,p + i, l, pf, lf,false, comp);
            }
        }

        /// <summary>
        /// Contabiliza las ocurrencias de cualquiera de los carcteres especificados mediante bsqueda ordinal
        /// y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo de la bsqueda es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="isUpper">Si true, el puntero de bsqueda no ser convertido
        /// a maysculas (ni ser copiado a stack) en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>
        /// Ints nuevo o actualizado donde se almacenarn las ubicaciones.
        /// Si el Ints de entrada es nulo, la salida tambin podra serla si el largo de la bsqueda es cero.
        /// </returns>
        public static unsafe Ints LocationsAny(Ints ret, int sum, char* p, int l, char* pf, int lf, bool isUpper, Ordinal comp)
        {

            if (comp == Ordinal.Binary)
                return Comp.LocationsAny(ret,sum,p, l, pf, lf,false);

            if (lf < 2)
            {
                if (lf == 1)
                    return Locations(ret,sum,p, l, *pf, comp);
                return (ret==null && l>0)?new Ints():ret;
            }

            if (l < 2)
            {
                if (l==1)
                {
                    if(ret==null)
                        ret=new Ints();
                    if (IndexOf(pf, lf, *p, comp, false) != -1)
                        ret.Append(sum);
                }
                return ret;
            }

            TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;

            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                Texts.ToUpper(p, ptmp, l, t);
                if (!isUpper)
                {
                    char* ptmp2 = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp2, lf, t);
                    return Comp.LocationsAny(ret,sum,ptmp, l, ptmp2, lf,true);
                }
                return Comp.LocationsAny(ret,sum,ptmp, l, pf, lf,false);
            }
            Mem.Heap heap = new EnPalermo.Mem.Heap(l << 1, false);
            Texts.ToUpper(p, (char*)heap.Pointer, l, t);
            if (!isUpper)
            {
                if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                {
                    char* ptmp = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp, lf, t);
                    ret = Comp.LocationsAny(ret,sum,(char*)heap.Pointer, l, ptmp, lf,true);
                    heap.Dispose();
                    return ret;
                }

                Mem.Heap heapf = new EnPalermo.Mem.Heap(lf << 1, false);
                Texts.ToUpper(pf, (char*)heapf.Pointer, lf, t);
                ret = Comp.LocationsAny(ret,sum,(char*)heap.Pointer, l, (char*)heapf.Pointer, lf,true);
                heap.Dispose();
                heapf.Dispose();
                return ret;
            }
            ret = Comp.LocationsAny(ret,sum,(char*)heap.Pointer, l, pf, lf,false);
            heap.Dispose();
            return ret;
        }

        /// <summary>
        /// Contabiliza las ocurrencias de un caracter en la cadena mediante bsqueda ordinal
        /// y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo de la bsqueda es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>
        /// Ints nuevo o actualizado donde se almacenarn las ubicaciones.
        /// Si el Ints de entrada es nulo, la salida tambin podra serla si el largo de la bsqueda es cero.
        /// </returns>
        public static unsafe Ints Locations(Ints ret, int sum, char* p, int l, char f, Ordinal comp)
        {
            if (comp == Ordinal.Binary)
                return Comp.Locations(ret,sum,p, l, f);

            if (l < 2)
            {
                if (l==1)
                {
                    if(ret==null)
                        ret=new Ints();
                    if (Equal(*p, f, comp))
                        ret.Append(sum);
                }
                return ret;
            }

            TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                Texts.ToUpper(p, ptmp, l, t);
                return Comp.Locations(ret,sum,ptmp, l, Texts.ToUpper(f, t));
            }
            Mem.Heap heap = new EnPalermo.Mem.Heap(l << 1, false);
            Texts.ToUpper(p, (char*)heap.Pointer, l, t);
            ret = Comp.Locations(ret,sum,(char*)heap.Pointer, l, Texts.ToUpper(f, t));
            heap.Dispose();
            return ret;
        }


        /// <summary>
        /// Contabiliza las ocurrencias de un caracter en la cadena mediante bsqueda ordinal
        /// y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo de la bsqueda es mayor a cero.
        /// </param>
        /// <param name="sum">Suma que se har a cada ubicacin</param>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo de bsqueda.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo de la cadena de bsqueda.</param>
        /// <param name="isUpper">Si true, el puntero de bsqueda no ser convertido
        /// a maysculas (ni ser copiado a stack) en el caso de que la opcin de
        /// comparacin ignore May/Min.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>
        /// Ints nuevo o actualizado donde se almacenarn las ubicaciones.
        /// Si el Ints de entrada es nulo, la salida tambin podra serla si el largo de la bsqueda es cero.
        /// </returns>
        public static unsafe Ints Locations(Ints ret, int sum, char* p, int l, char* pf, int lf,bool isUpper, Ordinal comp)
        {
            if (comp == Ordinal.Binary)
                return Comp.Locations(ret,sum,p, l, pf, lf,false);

            if (lf < 2 || lf >= l)
            {
                if (lf == 1)
                    return Locations(ret,sum,p, l,*pf,comp);
                if (l>0)
                {
                    if (lf == l && Equal(p, pf,isUpper, l, comp))
                    {
                        if (ret==null)
                            ret=new Ints(1);
                        ret.Append(sum);
                    }
                    else if (ret==null)
                        ret=new Ints();
                }
                return ret;
            }

            TableCase t = comp==Ordinal.Invariant?TableCase.Invariant: Texts.TableCurrent;

            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                Texts.ToUpper(p, ptmp, l, t);
                if (!isUpper)
                {
                    char* ptmp2 = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp2, lf, t);
                    return Comp.Locations(ret,sum,ptmp, l, ptmp2, lf,true);
                }
                return Comp.Locations(ret,sum,ptmp, l, pf, lf,false);

            }
            Mem.Heap heap = new EnPalermo.Mem.Heap(l << 1, false);
            Texts.ToUpper(p, (char*)heap.Pointer, l, t);
            if (!isUpper)
            {
                if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                {
                    char* ptmp = stackalloc char[lf];
                    Texts.ToUpper(pf, ptmp, lf, t);
                    ret = Comp.Locations(ret,sum,(char*)heap.Pointer, l, ptmp, lf,true);
                    heap.Dispose();
                    return ret;
                }

                Mem.Heap heapf = new EnPalermo.Mem.Heap(lf << 1, false);
                Texts.ToUpper(pf, (char*)heapf.Pointer, lf, t);
                ret = Comp.Locations(ret,sum,(char*)heap.Pointer, l, (char*)heapf.Pointer, lf,true);
                heap.Dispose();
                heapf.Dispose();
                return ret;
            }
            ret = Comp.Locations(ret,sum,(char*)heap.Pointer, l, pf, lf,false);
            heap.Dispose();
            return ret;
        }

        /// <summary>
        /// Contabiliza las ocurrencias de un caracter en la cadena mediante bsqueda ordinal
        /// y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo de la bsqueda es mayor a cero.
        /// </param>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>
        /// Ints nuevo o actualizado donde se almacenarn las ubicaciones.
        /// Si el Ints de entrada es nulo, la salida tambin podra serla si el largo de la bsqueda es cero.
        /// </returns>
        public static Ints Locations(Ints ret, string str, char f, Ordinal comp)
        {
            int l = str.Length;
            if (l == 0)
                return ret;
            unsafe
            {
                fixed (char* p = str)
                    return Locations(ret,0,p, l, f, comp);
            }
        }

        /// <summary>
        /// Contabiliza las ocurrencias de un caracter en la cadena mediante bsqueda ordinal
        /// y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo de la bsqueda es mayor a cero.
        /// </param>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <returns>
        /// Ints nuevo o actualizado donde se almacenarn las ubicaciones.
        /// Si el Ints de entrada es nulo, la salida tambin podra serla si el largo de la bsqueda es cero.
        /// </returns>
        public static Ints Locations(Ints ret, string str, int i, int l, char f, Ordinal comp)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return ret;
            unsafe
            {
                fixed (char* p = str)
                    ret = Locations(ret,i,p + i, l, f, comp);
            }
            return ret;
        }

        /// <summary>
        /// Contabiliza las ocurrencias de una subcadena en la cadena mediante bsqueda ordinal
        /// y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo de la bsqueda es mayor a cero.
        /// </param>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>
        /// Ints nuevo o actualizado donde se almacenarn las ubicaciones.
        /// Si el Ints de entrada es nulo, la salida tambin podra serla si el largo de la bsqueda es cero.
        /// </returns>
        public static Ints Locations(Ints ret, string str, string f, Ordinal comp)
        {
            return Locations(ret,str,0,-1,f,comp);
        }

        /// <summary>
        /// Contabiliza las ocurrencias de una subcadena en la cadena mediante bsqueda ordinal
        /// y retorna ubicaciones.
        /// </summary>
        /// <param name="ret">
        /// Ints a usar para almacenar las ubicaciones.
        /// Si es null, crear un nuevo objeto si el largo de la bsqueda es mayor a cero.
        /// </param>
        /// <param name="str">Cadena.</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <returns>
        /// Ints nuevo o actualizado donde se almacenarn las ubicaciones.
        /// Si el Ints de entrada es nulo, la salida tambin podra serla si el largo de la bsqueda es cero.
        /// </returns>
        public static Ints Locations(Ints ret, string str, int i, int l, string f, Ordinal comp)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            
            int lf = f.Length;
            
            if (lf < 2)
            {
                if (lf == 1)
                    return Locations(ret,str, i, l, f[0], comp);
                return (ret==null && l!=0)?new Ints():ret;
            }

            int lc = str.Length;

            if (l == -1 || i + l > lc)
                l = lc - i;

            if (l<lf)
                return (ret==null && l!=0)?new Ints():ret;
            
            unsafe
            {
                fixed (char* p = str, pf = f)
                    ret = Locations(ret,i,p + i, l, pf, lf,false, comp);
            }
            return ret;
        }

        #endregion

        #region Remove

        /// <summary>
        /// Copia memoria filtrando cadenas de elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a eliminar.</param>
        /// <param name="lf">Largo de la cadena de elementos a eliminar.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static unsafe int Remove(bool alwaysCopy, char* po, char* pd, int l, out int lFinal, char* pf, int lf, bool clean, Ordinal comp)
        {
            if (comp==Ordinal.Binary)
                return Comp.Remove(alwaysCopy,po,po,pd,l,out lFinal,pf,lf,clean,false);
            if (lf>l || lf<1 || l<1)
            {
                if (po==pd)
                    lFinal=l;
                else if (!alwaysCopy || l<1)
                    lFinal=0;
                else
                {
                    lFinal=l;
                    General.Move(po,pd,l);
                }
                return 0;
            }
                
            int tot;
            char* pc,pfz;
            Heap heap, heap2;
            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                pc = ptmp;
                heap = null;
                heap2 = null;
                char* ptmp2 = stackalloc char[lf];
                pfz = ptmp2;
            }
            else
            {
                heap = new Heap(l << 1, false);
                pc = (char*)heap.Pointer;
                if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                {
                    char* ptmp2 = stackalloc char[lf];
                    pfz = ptmp2;
                    heap2 = null;
                }
                else
                {
                    heap2 = new Heap(lf << 1, false);
                    pfz = (char*)heap2.Pointer;
                }

            }
            TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
            Texts.ToUpper(po, pc, l, t);
            Texts.ToUpper(pf,pfz,lf, t);
            tot = Comp.Remove(alwaysCopy, po, pc, pd, l, out lFinal, pfz, lf, clean, true);
            if (heap != null)
            {
                heap.Dispose();
                if (heap2!=null)
                    heap2.Dispose();
            }
            return tot;
        }

        /// <summary>
        /// Elimina las ocurrencias del caracter especificado usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="refMutable">
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(ref string str, char f, Ordinal comp, bool refMutable)
        {
            return Remove(ref str,0,-1,f,comp,refMutable);
        }

        /// <summary>
        /// Elimina las ocurrencias del caracter especificado usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="refMutable">
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(ref string str, int i, int l, char f, Ordinal comp, bool refMutable)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return 0;
            string cop;
            if (refMutable)
                cop = str;
            else
                cop = Mutables.Create(lc);
            unsafe
            {
                int lFinal, tot;
                char* pc;
                Mem.Heap heap;
                fixed (char* po = str, pd = cop)
                {
                    char* poi = po + i;
                    if (comp == Ordinal.Binary)
                    {
                        heap = null;
                        tot = Comp.Remove(false, poi, pd + i, l, out lFinal, f, false);
                    }
                    else
                    {
                        if (l < General.MaxStackChars && &l>General.MinimumStack)
                        {
                            char* ptmp = stackalloc char[l];
                            pc = ptmp;
                            heap = null;
                        }
                        else
                        {
                            heap = new EnPalermo.Mem.Heap(l << 1, false);
                            pc = (char*)heap.Pointer;
                        }
                        TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
                        Texts.ToUpper(poi, pc, l, t);
                        f = Texts.ToUpper(f, t);
                        tot = Comp.Remove(false, poi, pc, pd+i, l, out lFinal, &f, 1, false, true);
                    }
                    if (tot != 0)
                    {
                        if (lFinal!=0 || l!=lc)
                        {
                            int tmp = i + l;
                            if (refMutable)
                            {
                                if (tmp != lc)
                                    General.Move(pd + tmp, pd + (i + lFinal), lc - tmp);
                                tmp = lc - tot;
                                pd[tmp]='\0';
                                ((int*)pd)[-1] = tmp;
                            }
                            else
                            {
                                if (i != 0)
                                    General.Copy(po, pd, i);
                                if (tmp != lc)
                                    General.Copy(po + tmp, pd + (i + lFinal), lc - tmp);
                                ((int*)pd)[-1] = lc - tot;
                            }
                            str = cop;
                        }
                        else str=Others.Empty;
                    }
                    if (heap != null)
                        heap.Dispose();
                    return tot;
                }
            }


        }

        /// <summary>
        /// Elimina las ocurrencias de la subcadena especificada usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="refMutable">
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(ref string str, string f, Ordinal comp, bool refMutable)
        {
            return Remove(ref str,0,-1,f,comp,refMutable);
        }

        /// <summary>
        /// Elimina las ocurrencias de la subcadena especificada usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="refMutable">
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int Remove(ref string str, int i, int l, string f, Ordinal comp, bool refMutable)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int lf = f.Length;
            if (lf < 2)
            {
                if (lf == 1)
                    return Remove(ref str,i,l, f[0], comp, refMutable);
                return 0;
            }
            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l < lf)
                return 0;
            string cop;
            if (refMutable)
                cop = str;
            else
                cop = Mutables.Create(lc);
            unsafe
            {
                int lFinal, tot;
                char* pc, pf, poi;
                Mem.Heap heap, heap2;
                fixed (char* po = str, pd = cop, pfx = f)
                {
                    poi = po + i;
                    if (comp == Ordinal.Binary)
                    {
                        heap = null;
                        heap2 = null;
                        tot = Comp.Remove(false, poi, poi, pd + i, l, out lFinal, pfx, lf, false,false);
                    }
                    else
                    {
                        if (l < General.MaxStackChars && &l>General.MinimumStack)
                        {
                            char* ptmp = stackalloc char[l];
                            pc = ptmp;
                            heap = null;
                            heap2 = null;
                            char* ptmp2 = stackalloc char[lf];
                            pf = ptmp2;
                        }
                        else
                        {
                            heap = new EnPalermo.Mem.Heap(l << 1, false);
                            pc = (char*)heap.Pointer;
                            if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                            {
                                char* ptmp2 = stackalloc char[lf];
                                pf = ptmp2;
                                heap2 = null;
                            }
                            else
                            {
                                heap2 = new EnPalermo.Mem.Heap(lf << 1, false);
                                pf = (char*)heap2.Pointer;
                            }

                        }
                        TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
                        Texts.ToUpper(poi, pc, l, t);
                        Texts.ToUpper(pfx, pf, lf, t);
                        tot = Comp.Remove(false, poi, pc, pd + i, l, out lFinal, pf, lf, false,true);
                    }
                    if (tot != 0)
                    {
                        if (lFinal!=0 || l!=lc)
                        {
                            int tmp = i + l;
                            int tot2 = tot * lf;
                            if (refMutable)
                            {
                                if (tmp != lc)
                                    General.Move(pd + tmp, pd + (i + lFinal), lc - tmp);
                                tmp = lc - tot2;
                                pd[tmp]='\0';
                                ((int*)pd)[-1] = tmp;
                            }
                            else
                            {
                                if (i != 0)
                                    General.Copy(po, pd, i);
                                if (tmp != lc)
                                    General.Copy(po + tmp, pd + (i + lFinal), lc - tmp);
                                ((int*)pd)[-1] = lc - tot2;
                            }
                            str = cop;
                        }
                        else
                            str=Others.Empty;
                    }

                    if (heap != null)
                    {
                        heap.Dispose();
                        if (heap2 != null)
                            heap2.Dispose();
                    }
                    return tot;
                }
            }
        }

        /// <summary>
        /// Copia memoria filtrando elementos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos a eliminar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public unsafe static int RemoveAny(bool alwaysCopy, char* po, char* pd, int l, out int lFinal, char* pf, int lf, bool clean, Ordinal comp)
        {
            if (comp==Ordinal.Binary)
                return Comp.RemoveAny(alwaysCopy,po,po,pd,l,out lFinal,pf,lf,clean,false);
            if (lf<1 || l<1)
            {
                if (po==pd)
                    lFinal=l;
                else if (!alwaysCopy || l<1)
                    lFinal=0;
                else
                {
                    lFinal=l;
                    General.Move(po,pd,l);
                }
                return 0;
            }
                
            int tot;
            char* pc,pfz;
            Heap heap, heap2;
            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                pc = ptmp;
                heap = null;
                heap2 = null;
                char* ptmp2 = stackalloc char[lf];
                pfz = ptmp2;
            }
            else
            {
                heap = new Heap(l << 1, false);
                pc = (char*)heap.Pointer;
                if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                {
                    char* ptmp2 = stackalloc char[lf];
                    pfz = ptmp2;
                    heap2 = null;
                }
                else
                {
                    heap2 = new Heap(lf << 1, false);
                    pfz = (char*)heap2.Pointer;
                }

            }
            TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
            Texts.ToUpper(po, pc, l, t);
            Texts.ToUpper(pf,pfz,lf, t);
            tot = Comp.RemoveAny(alwaysCopy, po, pc, pd, l, out lFinal, pfz, lf, clean, true);
            if (heap != null)
            {
                heap.Dispose();
                if (heap2!=null)
                    heap2.Dispose();
            }
            return tot;

        }

        /// <summary>
        /// Elimina las ocurrencias de cada caracter en la cadena de bsqueda usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="refMutable">
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(ref string str, Ordinal comp, bool refMutable, params char[] chars)
        {
            return RemoveAny(ref str,0,-1,comp,refMutable,chars);
        }

        /// <summary>
        /// Elimina las ocurrencias de cada caracter en la cadena de bsqueda usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="refMutable">
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveAny(ref string str, int i, int l, Ordinal comp, bool refMutable, params char[] chars)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);

            int lf = chars.Length;
            if (lf < 2)
            {
                if (lf == 1)
                    return Remove(ref str, i, l, chars[0], comp, refMutable);
                return 0;
            }
            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return 0;

            string cop;
            if (refMutable)
                cop = str;
            else
                cop = Mutables.Create(lc);

            unsafe
            {
                int lFinal, tot;
                char* pc, pf, poi;
                Mem.Heap heap, heap2;
                fixed (char* po = str, pd = cop, pfx = chars)
                {
                    poi = po + i;
                    if (comp == Ordinal.Binary)
                    {
                        heap = null;
                        heap2 = null;
                        tot = Comp.RemoveAny(false, poi, poi, pd + i, l, out lFinal, pfx, lf, false,false);
                    }
                    else
                    {
                        if (l < General.MaxStackChars && &l>General.MinimumStack)
                        {
                            char* ptmp = stackalloc char[l];
                            pc = ptmp;
                            heap = null;
                            heap2 = null;
                            char* ptmp2 = stackalloc char[lf];
                            pf = ptmp2;
                        }
                        else
                        {
                            heap = new EnPalermo.Mem.Heap(l << 1, false);
                            pc = (char*)heap.Pointer;
                            if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                            {
                                char* ptmp2 = stackalloc char[lf];
                                pf = ptmp2;
                                heap2 = null;
                            }
                            else
                            {
                                heap2 = new EnPalermo.Mem.Heap(lf << 1, false);
                                pf = (char*)heap2.Pointer;
                            }

                        }
                        TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
                        Texts.ToUpper(poi, pc, l, t);
                        Texts.ToUpper(pfx, pf, lf, t);
                        tot = Comp.RemoveAny(false, poi, pc, pd + i, l, out lFinal, pf, lf, false,true);
                    }

                    if (tot != 0)
                    {
                        if (lFinal!=0 || l!=lc)
                        {
                            int tmp = i + l;
                            if (refMutable)
                            {
                                if (tmp != lc)
                                    General.Move(pd + tmp, pd + (i + lFinal), lc - tmp);
                                tmp = lc - tot;
                                pd[tmp]='\0';
                                ((int*)pd)[-1] = tmp;
                            }
                            else
                            {
                                if (i != 0)
                                    General.Copy(po, pd, i);
                                if (tmp != lc)
                                    General.Copy(po + tmp, pd + (i + lFinal), lc - tmp);
                                ((int*)pd)[-1] = lc - tot;
                            }
                            str = cop;
                        }
                        else
                            str=Others.Empty;
                    }


                    if (heap != null)
                    {
                        heap.Dispose();
                        if (heap2 != null)
                            heap2.Dispose();
                    }
                    return tot;
                }
            }
        }

        
        /// <summary>
        /// Filtra elementos encuadrados en un grupo.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public unsafe static int RemoveGroup(CharGroup group, bool alwaysCopy, char* po, char* pd, int l, out int lFinal, char* pch, int lch, bool clean)
        {
            if (l < 2)
            {
                if (l < 1)
                {
                    lFinal = 0;
                    return 0;
                }
                else if (Texts.InGroup(*po,group,pch,lch))
                {
                    lFinal = 0;
                    if (clean)
                        *pd = '\0';
                    return 1;
                }
                else if (alwaysCopy)
                    *pd = *po;
                lFinal = 1;
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            if (pch > General.MaximumStack && lch>0 && l > 127 && (group&(CharGroup.Custom|CharGroup.Others))!=0 && lch < General.MaxStackChars && &lch > General.MinimumStack)
            {
                char* pcops = stackalloc char[lch];
                General.Copy(pch, pcops, lch);
                pch = pcops;
            }
            int tot = 0, ipo = 0, a = 0, tc;
            lFinal = 0;
            do
            {
                if (Texts.InGroup(po[a],group,pch,lch))
                {
                    tot++;
                    tc = a - ipo;
                    if (tc != 0)
                    {
                        General.Copy(po + ipo, pd + lFinal, tc);
                        lFinal += tc;
                    }
                    ipo = a + 1;
                }
            } while (++a != l);

            if (tot != 0)
            {
                tc = l - ipo;
                if (tc != 0)
                {
                    General.Copy(po + ipo, pd + lFinal, tc);
                    lFinal += tc;
                }
                if (clean && (lFinal != 0 || alwaysCopy))
                {
                    General.Clean(pd + lFinal, tot);
                }
            }
            else if (po == pd)
            {
                lFinal = l;
            }
            else if (alwaysCopy)
            {
                General.Copy(po, pd, l);
                lFinal = l;
            }
            return tot;
        }

        /// <summary>
        /// Filtra elementos encuadrados en un grupo.
        /// </summary>
        /// <param name="group">Grupo.</param>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="custom">Array de carcteres en categora custom.</param>
        /// <param name="refMutable">
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveGroup(CharGroup group, ref string str, bool refMutable, params char[] custom)
        {
            return RemoveGroup(group, ref str,0,-1,refMutable,custom);
        }

        /// <summary>
        /// Copia memoria filtrando elementos encuadrados en un grupo.
        /// </summary>
        /// <param name="group">Grupo.</param>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="custom">Array de carcteres en categora custom.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="refMutable">
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public static int RemoveGroup(CharGroup group, ref string str, int i, int l, bool refMutable, params char[] custom)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);

            
            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return 0;

            string cop;
            if (refMutable)
                cop = str;
            else
                cop = Mutables.Create(lc);

            unsafe
            {
                fixed (char* po = str, pd = cop)
                {
                    int lFinal;
                    int lch = custom==null?0:custom.Length;
                    if (lch!=0)
                        fixed (char* pch = custom)
                            lch = RemoveGroup(group, false, po+i, pd + i, l, out lFinal, pch, lch, false);
                    else
                        lch = RemoveGroup(group, false, po+i, pd + i, l, out lFinal, null, 0, false);
                    if (lch != 0)
                    {
                        if (lFinal!=0 || l!=lc)
                        {
                            int tmp = i + l;
                            if (refMutable)
                            {
                                if (tmp != lc)
                                    General.Move(pd + tmp, pd + (i + lFinal), lc - tmp);
                                tmp = lc - lch;
                                pd[tmp]='\0';
                                ((int*)pd)[-1] = tmp;
                            }
                            else
                            {
                                if (i != 0)
                                    General.Copy(po, pd, i);
                                if (tmp != lc)
                                    General.Copy(po + tmp, pd + (i + lFinal), lc - tmp);
                                ((int*)pd)[-1] = lc - lch;
                            }
                            str = cop;
                        }
                        else
                            str=Others.Empty;
                    }
                    return lch;
                }
            }
        }


        #endregion

        #region Replace con Chars


        /// <summary>
        /// Reemplaza las ocurrencias del caracter especificado usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="refMutable">
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(ref string str, char f,char r, Ordinal comp, bool refMutable)
        {
            return Replace(ref str,0,-1,f,r,comp,refMutable);
        }

        /// <summary>
        /// Reemplaza las ocurrencias del caracter especificado usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="f">Caracter a buscar.</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="refMutable">Si true, modificara el mismo objeto de cadena de entrada. En este caso la cadena debe ser mutable (no verifica)!</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(ref string str, int i, int l, char f, char r, Ordinal comp, bool refMutable)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return 0;
                

            string cop;
            if (refMutable)
                cop = str;
            else
                cop = Mutables.Create(lc);
            unsafe
            {
                int tot;
                char* pc;
                Mem.Heap heap;
                fixed (char* po = str, pd = cop)
                {
                    char* poi = po + i;
                    if (comp == Ordinal.Binary)
                    {
                        heap = null;
                        pc = poi;
                    }
                    else
                    {
                        if (l < General.MaxStackChars && &l>General.MinimumStack)
                        {
                            char* ptmp = stackalloc char[l];
                            pc = ptmp;
                            heap = null;
                        }
                        else
                        {
                            heap = new EnPalermo.Mem.Heap(l << 1, false);
                            pc = (char*)heap.Pointer;
                        }
                        TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
                        Texts.ToUpper(poi, pc, l, t);
                        f = Texts.ToUpper(f, t);
                    }
                    if (refMutable)
                        tot = Comp.ReplaceEqual(poi, pc, l, &f, &r, 1, true);
                    else
                        tot = Comp.ReplaceEqual(false, poi, pc, pd + i, l, &f, &r, 1, true);
                    if (tot != 0 && !refMutable)
                    {
                        int tmp = i + l;
                        if (i != 0)
                            General.Copy(po, pd, i);
                        if (tmp != lc)
                        {
                            General.Copy(po + tmp, pd + tmp, lc - tmp);
                        }
                        str = cop;
                    }
                    if (heap != null)
                        heap.Dispose();
                    return tot;
                }
            }


        }


        /// <summary>
        /// Reemplaza las ocurrencias de cada caracter en la cadena de bsqueda usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="refMutable">Si true, modificara el mismo objeto de cadena de entrada. En este caso la cadena debe ser mutable (no verifica)!</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(ref string str, char r, Ordinal comp, bool refMutable, params char[] chars)
        {
            return ReplaceAny(ref str, 0,-1,r,comp,refMutable,chars);
        }

        /// <summary>
        /// Reemplaza las ocurrencias de cada caracter en la cadena de bsqueda usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="refMutable">Si true, modificara el mismo objeto de cadena de entrada. En este caso la cadena debe ser mutable (no verifica)!</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(ref string str, int i, int l, char r, Ordinal comp, bool refMutable, params char[] chars)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);

            int lf = chars.Length;
            if (lf < 2)
            {
                if (lf == 1)
                    return Replace(ref str, i, l, chars[0], r, comp, refMutable);
                return 0;
            }

            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return 0;

            string cop;
            if (refMutable)
                cop = str;
            else
                cop = Mutables.Create(lc);

            unsafe
            {
                int tot;
                char* pc, pf, poi;
                Mem.Heap heap, heap2;
                fixed (char* po = str, pd = cop, pfx = chars)
                {
                    poi = po + i;
                    if (comp == Ordinal.Binary)
                    {
                        heap = null;
                        heap2 = null;
                        pf = pfx;
                        pc = poi;
                    }
                    else
                    {
                        if (l < General.MaxStackChars && &l>General.MinimumStack)
                        {
                            char* ptmp = stackalloc char[l];
                            pc = ptmp;
                            heap = null;
                            heap2 = null;
                            char* ptmp2 = stackalloc char[lf];
                            pf = ptmp2;
                        }
                        else
                        {
                            heap = new EnPalermo.Mem.Heap(l << 1, false);
                            pc = (char*)heap.Pointer;
                            if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                            {
                                char* ptmp2 = stackalloc char[lf];
                                pf = ptmp2;
                                heap2 = null;
                            }
                            else
                            {
                                heap2 = new EnPalermo.Mem.Heap(lf << 1, false);
                                pf = (char*)heap2.Pointer;
                            }

                        }
                        TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
                        Texts.ToUpper(poi, pc, l, t);
                        Texts.ToUpper(pfx, pf, lf, t);
                    }
                    tot = Comp.ReplaceAny(false, poi, pc, pd + i, l, pf, lf, r,true);

                    if (tot != 0 && !refMutable)
                    {
                        int tmp = i + l;
                        if (i != 0)
                            General.Copy(po, pd, i);
                        if (tmp != lc)
                            General.Copy(po + tmp, pd + tmp, lc - tmp);
                        str = cop;
                    }

                    if (heap != null)
                    {
                        heap.Dispose();
                        if (heap2 != null)
                            heap2.Dispose();
                    }
                    return tot;
                }
            }
        }


        /// <summary>
        /// Copia memoria reemplazando elementos.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="lf">
        /// Cantidad de elementos a buscar.
        /// </param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public unsafe static int ReplaceAny(bool alwaysCopy, char* po, char* pd, int l, char* pf, int lf, char r, Ordinal comp)
        {
            
            if (comp==Ordinal.Binary)
                return Comp.ReplaceAny(alwaysCopy,po,po,pd,l,pf,lf,r,false);

            if (lf<1 || l<1)
            {
                if (alwaysCopy && po!=pd && l>0)
                    General.Move(po,pd,l);
                return 0;
            }
                
            int tot;
            char* pc,pfz;
            Heap heap, heap2;
            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                pc = ptmp;
                heap = null;
                heap2 = null;
                char* ptmp2 = stackalloc char[lf];
                pfz = ptmp2;
            }
            else
            {
                heap = new Heap(l << 1, false);
                pc = (char*)heap.Pointer;
                if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                {
                    char* ptmp2 = stackalloc char[lf];
                    pfz = ptmp2;
                    heap2 = null;
                }
                else
                {
                    heap2 = new Heap(lf << 1, false);
                    pfz = (char*)heap2.Pointer;
                }

            }
            TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
            Texts.ToUpper(po, pc, l, t);
            Texts.ToUpper(pf,pfz,lf, t);
            tot = Comp.ReplaceAny(alwaysCopy, po, pc, pd, l, pfz, lf, r, true);
            if (heap != null)
            {
                heap.Dispose();
                if (heap2!=null)
                    heap2.Dispose();
            }
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos por otra cadena de menor tamao.
        /// Es una funcin directa, por lo que lr debera estar verificado correctamente 
        /// (siempre debe ser menor a lf).
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lf">
        /// Largo de la cadena de elementos a buscar.
        /// Si es menor o igual a lr se genera una excepcin.
        /// </param>
        /// <param name="lr">
        /// Largo de la cadena de elementos de reemplazo.
        /// Si es mayor o igual a lf se genera una excepcin.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan todas estas condiciones: lr es 0, alwaysCopy es false, lFinal es 0 y po-pd son diferentes,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static unsafe int ReplaceMinor(bool alwaysCopy, char* po, char* pd, int l, out int lFinal, char* pf, int lf, char* pr, int lr, bool clean, Ordinal comp)
        {
            
            if (comp==Ordinal.Binary)
                return Comp.ReplaceMinor(alwaysCopy,po,po,pd,l,out lFinal,pf,lf,pr,lr,clean,false);

            if (lf>l || lf<1 || l<1)
            {
                if (po==pd)
                    lFinal=l;
                else if (!alwaysCopy || l<1)
                    lFinal=0;
                else
                {
                    lFinal=l;
                    General.Move(po,pd,l);
                }
                return 0;
            }
                
            int tot;
            char* pc,pfz;
            Heap heap, heap2;
            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                pc = ptmp;
                heap = null;
                heap2 = null;
                char* ptmp2 = stackalloc char[lf];
                pfz = ptmp2;
            }
            else
            {
                heap = new Heap(l << 1, false);
                pc = (char*)heap.Pointer;
                if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                {
                    char* ptmp2 = stackalloc char[lf];
                    pfz = ptmp2;
                    heap2 = null;
                }
                else
                {
                    heap2 = new Heap(lf << 1, false);
                    pfz = (char*)heap2.Pointer;
                }

            }
            TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
            Texts.ToUpper(po, pc, l, t);
            Texts.ToUpper(pf,pfz,lf, t);
            tot = Comp.ReplaceMinor(alwaysCopy, po, pc, pd, l, out lFinal, pfz, lf,pr,lr, clean, true);
            if (heap != null)
            {
                heap.Dispose();
                if (heap2!=null)
                    heap2.Dispose();
            }
            return tot;
        }

        /// <summary>
        /// Reemplaza elementos encuadrados en un grupo.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="group">Grupo.</param>
        /// <param name="custom">Array de carcteres en categora custom.</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="refMutable">Si true, modificara el mismo objeto de cadena de entrada. En este caso la cadena debe ser mutable (no verifica)!</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceGroup(ref string str, CharGroup group, char r, bool refMutable, params char[] custom)
        {
            return ReplaceGroup(ref str, 0,-1,group,r,refMutable,custom);
        }

        /// <summary>
        /// Reemplaza elementos encuadrados en un grupo.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="group">Grupo.</param>
        /// <param name="custom">Array de carcteres en categora custom.</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="refMutable">Si true, modificara el mismo objeto de cadena de entrada. En este caso la cadena debe ser mutable (no verifica)!</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceGroup(ref string str, int i, int l, CharGroup group, char r, bool refMutable, params char[] custom)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);

            int lc = str.Length;
            if (l == -1 || i + l > lc)
                l = lc - i;
            if (l<1)
                return 0;
            unsafe
            {
                fixed (char* po = str)
                {
                    int lch = custom==null?0:custom.Length;
                    if (refMutable)
                    {
                        if (lch!=0)
                            fixed (char* pch = custom)
                                return ReplaceGroup(po+i,l,group,pch,lch,r);
                        return ReplaceGroup(po+i,l,group,null,0,r);
                    }
                    string cop = Mutables.Create(lc);
                    fixed (char*pd = cop)
                    {
                        if (lch!=0)
                            fixed (char* pch = custom)
                                lch=ReplaceGroup(false,po+i,pd + i,l,group,pch,lch,r);
                        else
                            lch=ReplaceGroup(false,po+i,pd + i,l,group,null,0,r);

                        if (lch != 0 && !refMutable)
                        {
                            int tmp = i + l;
                            if (i != 0)
                                General.Copy(po, pd, i);
                            if (tmp != lc)
                                General.Copy(po + tmp, pd + tmp, lc - tmp);
                            str = cop;
                        }
                        return lch;
                    }
                }
            }
            
        }

        /// <summary>
        /// Reemplaza elementos encuadrados en un grupo.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public unsafe static int ReplaceGroup(char* p, int l, CharGroup group, char* pch, int lch, char r)
        {
            
            if (l < 2)
            {
                if (l == 1)
                {
                    if (Texts.InGroup(*p,group,pch,lch))
                    {
                        *p = r;
                        return 1;
                    }
                }
                return 0;
            }

            if (pch > General.MaximumStack && lch>0 && l > 127 && (group&(CharGroup.Custom|CharGroup.Others))!=0 && lch < General.MaxStackChars && &lch > General.MinimumStack)
            {
                char* pcops = stackalloc char[lch];
                General.Copy(pch, pcops, lch);
                pch = pcops;
            }
            int tot = 0, a = 0;
            do
            {
                if (Texts.InGroup(p[a],group,pch,lch))
                {
                    tot++;
                    p[a] = r;
                }
            } while (++a != l);
            return tot;

        }

        /// <summary>
        /// Copia memoria reemplazando elementos encuadrados en un grupo.
        /// </summary>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <param name="r">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public unsafe static int ReplaceGroup(bool alwaysCopy, char* po, char* pd, int l, CharGroup group, char* pch, int lch, char r)
        {
            
            if (l < 2)
            {
                if (l == 1)
                {
                    if (Texts.InGroup(*po,group,pch,lch))
                    {
                        *pd = r;
                        return 1;
                    }
                    else if (alwaysCopy)
                        *pd = *po;
                }
                return 0;
            }

            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return ReplaceGroup(pd, l,group,pch,lch, r);
            }
            if (pch > General.MaximumStack && lch>0 && l > 127 && (group&(CharGroup.Custom|CharGroup.Others))!=0 && lch < General.MaxStackChars && &lch > General.MinimumStack)
            {
                char* pcops = stackalloc char[lch];
                General.Copy(pch, pcops, lch);
                pch = pcops;
            }

            int a = IndexOfGroup(po,l,group,pch,lch);
            if (a == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[a] = r;
            int tot=1;
            while (++a != l)
            {
                if (Texts.InGroup(pd[a],group,pch,lch))
                {
                    tot++;
                    pd[a] = r;
                }
            }
            return tot;
        }
 
        #endregion

        #region Replace con Cadenas

        /// <summary>
        /// Copia memoria reemplazando cadenas de elementos.
        /// Tanto la cadena de bsqueda como la de reemplazo tienen el mismo largo.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="pf">Puntero de elementos a reemplazar.</param>
        /// <param name="pr">Puntero de elementos de reemplazo.</param>
        /// <param name="lfr">
        /// Largo de la cadena de elementos a buscar o reemplazar.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public unsafe static int ReplaceEqual(bool alwaysCopy, char* po, char* pd, int l, char* pf, char* pr, int lfr, Ordinal comp)
        {

            if (comp==Ordinal.Binary)
                return Comp.ReplaceEqual(alwaysCopy,po,po,pd,l,pf,pr,lfr,false);

            if (lfr>l || lfr<1 || l<1)
            {
                if (alwaysCopy && po!=pd && l>0)
                    General.Move(po,pd,l);
                return 0;
            }
                
            int tot;
            char* pc,pfz;
            Heap heap, heap2;
            if (l < General.MaxStackChars && &l>General.MinimumStack)
            {
                char* ptmp = stackalloc char[l];
                pc = ptmp;
                heap = null;
                heap2 = null;
                char* ptmp2 = stackalloc char[lfr];
                pfz = ptmp2;
            }
            else
            {
                heap = new Heap(l << 1, false);
                pc = (char*)heap.Pointer;
                if (lfr < General.MaxStackChars && &lfr>General.MinimumStack)
                {
                    char* ptmp2 = stackalloc char[lfr];
                    pfz = ptmp2;
                    heap2 = null;
                }
                else
                {
                    heap2 = new Heap(lfr << 1, false);
                    pfz = (char*)heap2.Pointer;
                }

            }
            TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
            Texts.ToUpper(po, pc, l, t);
            Texts.ToUpper(pf,pfz,lfr, t);
            tot = Comp.ReplaceEqual(alwaysCopy, po, pc, pd, l, pfz, pr, lfr, true);
            if (heap != null)
            {
                heap.Dispose();
                if (heap2!=null)
                    heap2.Dispose();
            }
            return tot;

        }


  
        /// <summary>
        /// Reemplaza las ocurrencias de la subcadena especificada usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="r">Cadena de reemplazo. Si es null o vacia, elimina las ocurrencias.</param>
        /// <param name="refMutable">
        /// Ignorado si la cadena de reemplazo es mayor a la de bsqueda.
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(ref string str, string f, string r, Ordinal comp, bool refMutable)
        {
            return Replace(ref str,0,-1,f,r,comp,refMutable);
        }

        /// <summary>
        /// Reemplaza las ocurrencias de la subcadena especificada usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="f">Subcadena a buscar.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="r">Cadena de reemplazo. Si es null o vacia, elimina las ocurrencias.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="refMutable">
        /// Ignorado si la cadena de reemplazo es mayor a la de bsqueda.
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int Replace(ref string str, int i, int l, string f, string r, Ordinal comp, bool refMutable)
        {
            int lr = r.Length;
            if (lr == 0)
            {
                return Remove(ref str, i, l, f, comp, refMutable);
            }
            int lf = f.Length;
            int lc, tot, lFinal;
            string cop;
            if (lf < lr)
            {
                if (lf == 0)
                    return 0;
                Ints ubics = Locations(null,str, i, l, f, comp);
                if (ubics == null || (tot = ubics.Length)==0)
                    return 0;
                lc = str.Length;

                lFinal = tot * (lr - lf) + lc;
                if (refMutable && Mutables.Capacity(str)>=lFinal)
                    cop=str;
                else
                {
                    cop = Mutables.Create(lFinal);
                    refMutable=false;
                }
                unsafe
                {
                    fixed (int* pCont = ubics.Array)
                    {
                        fixed (char* po = str, pd = cop, pr = r)
                        {
                            Comp.ReplaceLocations(pCont, tot, po, lc, pd, lf, pr, lr, false, false);
                            if (refMutable)
                            {
                                pd[lFinal]='\0';
                                ((int*)pd)[-1] = lFinal;
                            }
                        }
                    }
                }
                str = cop;
                return tot;
            }
            lc = str.Length;
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            if (l == -1 || i + l > lc)
            {
                l = lc - i;
            }
            if (l < lf)
                return 0;

            if (refMutable)
                cop = str;
            else
                cop = Mutables.Create(lc);
            unsafe
            {
                char* pc, pf, poi;
                Mem.Heap heap, heap2;
                fixed (char* po = str, pd = cop, pfx = f, pr=r)
                {
                    poi = po + i;
                    if (comp == Ordinal.Binary)
                    {
                        heap = null;
                        heap2 = null;
                        pf = pfx;
                        pc = poi;
                    }
                    else
                    {
                        if (l < General.MaxStackChars && &l>General.MinimumStack)
                        {
                            char* ptmp = stackalloc char[l];
                            pc = ptmp;
                            heap = null;
                            heap2 = null;
                            char* ptmp2 = stackalloc char[lf];
                            pf = ptmp2;
                        }
                        else
                        {
                            heap = new EnPalermo.Mem.Heap(l << 1, false);
                            pc = (char*)heap.Pointer;
                            if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                            {
                                char* ptmp2 = stackalloc char[lf];
                                pf = ptmp2;
                                heap2 = null;
                            }
                            else
                            {
                                heap2 = new EnPalermo.Mem.Heap(lf << 1, false);
                                pf = (char*)heap2.Pointer;
                            }

                        }
                        TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
                        Texts.ToUpper(poi, pc, l, t);
                        Texts.ToUpper(pfx, pf, lf, t);
                    }
                    if (lr != lf)
                    {
                        tot = Comp.ReplaceMinor(false, poi, pc, pd + i, l, out lFinal, pf, lf,pr, lr, false,true);
                        if (tot != 0)
                        {
                            int tmp = i + l;
                            int tot2 = tot * (lf - lr);
                            if (refMutable)
                            {
                                if (tmp != lc)
                                    General.Move(pd + tmp, pd + (i + lFinal), lc - tmp);
                                tmp = lc - tot2;
                                pd[tmp]='\0';
                                ((int*)pd)[-1] = tmp;
                            }
                            else
                            {
                                if (i != 0)
                                    General.Copy(po, pd, i);
                                if (tmp != lc)
                                    General.Copy(po + tmp, pd + (i + lFinal), lc - tmp);
                                ((int*)pd)[-1] = lc - tot2;
                            }
                            str = cop;
                        }
                    }
                    else
                    {
                        tot = Comp.ReplaceEqual(false, poi, pc, pd + i, l, pf, pr, lr,false);
                            
                        if (tot != 0 && !refMutable)
                        {
                            int tmp = i + l;
                            if (i != 0)
                                General.Copy(po, pd, i);
                            if (tmp != lc)
                            {
                                General.Copy(po + tmp, pd + tmp, lc - tmp);
                            }
                            str = cop;
                        }
                    }
                    if (heap != null)
                    {
                        heap.Dispose();
                        if (heap2 != null)
                            heap2.Dispose();
                    }
                    return tot;
                }
            }
        }

        /// <summary>
        /// Reemplaza las ocurrencias de cada caracter en la cadena de bsqueda usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="r">Cadena de reemplazo. Si es null o vacia, elimina las ocurrencias.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="refMutable">
        /// Ignorado si la cadena de reemplazo es mayor a la de bsqueda.
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(ref string str, string r, Ordinal comp, bool refMutable, params char[] chars)
        {
            return ReplaceAny(ref str,0,-1,r,comp,refMutable,chars);
        }

        /// <summary>
        /// Reemplaza las ocurrencias de cada caracter en la cadena de bsqueda usando bsqueda ordinal.
        /// </summary>
        /// <param name="str">Cadena (Referencia que ser sustituida por la nueva cadena).</param>
        /// <param name="chars">Carcteres a buscar.</param>
        /// <param name="r">Cadena de reemplazo. Si es null o vacia, elimina las ocurrencias.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="i">Inicio de la busqueda. Si el valor es negativo genera excepcin. Si est fuera de rango, retorna -1.</param>
        /// <param name="l">
        /// Largo de la zona a buscar. Si es -1 o mayor al rango, buscar el resto de la cadena. Si es 0, retorna -1.
        /// </param>
        /// <param name="refMutable">
        /// Ignorado si la cadena de reemplazo es mayor a la de bsqueda.
        /// Si true, modificara el mismo objeto de cadena de entrada.
        /// En este caso la cadena debe ser mutable (no verifica)!
        /// Si el retorno es una cadena vacia, esta siempre ser inmutable.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public static int ReplaceAny(ref string str, int i, int l, string r, Ordinal comp, bool refMutable, params char[] chars)
        {
            int lr = r.Length;
            if (lr < 2)
            {
                if (lr == 1)
                    return ReplaceAny(ref str, i, l, r[0], comp, refMutable, chars);
                return RemoveAny(ref str, i, l, comp, refMutable, chars);
            }

            Ints ubics = LocationsAny(null,str, i, l, comp, chars);
            int tot;
            if (ubics == null || (tot = ubics.Length)==0)
                return 0;
            int lc = str.Length;
            int lFinal = tot * (lr - 1) + lc;
            string cop;
            if (refMutable && Mutables.Capacity(str)>=lFinal)
                cop=str;
            else
            {
                cop = Mutables.Create(lFinal);
                refMutable=false;
            }
            unsafe
            {
                fixed (int* pCont = ubics.Array)
                {
                    fixed (char* po = str, pd = cop, pr = r)
                    {
                        
                        Comp.ReplaceLocations(pCont, tot, po, lc, pd, 1, pr, lr, false,false);
                        if (refMutable)
                        {
                            pd[lFinal]='\0';
                            ((int*)pd)[-1] = lFinal;
                        }

                    }
                }
            }
            str = cop;
            return tot;
        }


        #endregion

        #region IndexOf y Exist en Array

        /// <summary>
        /// Busqueda ordinal de una cadena en array.
        /// </summary>
        /// <param name="arr">Array</param>
        /// <param name="f">Cadena a buscar. Si es null, buscar instancias nulas.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(string[] arr, string f, Ordinal comp, bool last)
        {
            return IndexOf(arr, f, comp, last) != -1;
        }
        /// <summary>
        /// Busqueda ordinal de una cadena en array.
        /// </summary>
        /// <param name="arr">Array</param>
        /// <param name="f">Cadena a buscar. Si es null, buscar instancias nulas.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(string[] arr, string f, Ordinal comp, bool last)
        {
            return IndexOf(arr, 0, -1, f, comp, last);
        }

        /// <summary>
        /// Busqueda ordinal de una cadena en array.
        /// </summary>
        /// <param name="arr">Array</param>
        /// <param name="i">Inicio de la zona de bsqueda. Si implica un valor fuera de rango, retorna -1.</param>
        /// <param name="l">Largo de la zona de bsqueda. Si implica un valor fuera de rango, se ajusta al largo de array.</param>
        /// <param name="f">Cadena a buscar. Si es null, buscar instancias nulas.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>True si existe.</returns>
        public static bool Exist(string[] arr, int i, int l, string f, Ordinal comp, bool last)
        {
            return IndexOf(arr, i, l, f, comp, last) != -1;
        }

        /// <summary>
        /// Busqueda ordinal de una cadena en array.
        /// </summary>
        /// <param name="arr">Array</param>
        /// <param name="i">Inicio de la zona de bsqueda. Si implica un valor fuera de rango, retorna -1.</param>
        /// <param name="l">Largo de la zona de bsqueda. Si es -1 o si implica un valor fuera de rango, se ajusta al largo de array.</param>
        /// <param name="f">Cadena a buscar. Si es null, buscar instancias nulas.</param>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="last">Si true, buscar la ltima ubicacin de la cadena.</param>
        /// <returns>Elemento encontrado o -1.</returns>
        public static int IndexOf(string[] arr, int i, int l, string f, Ordinal comp, bool last)
        {
            if (l<-1)
                throw new ArgumentOutOfRangeException("l",l, Errors.MINOR_ONE_NEGATIVE);
            if (i < 0)
                throw new ArgumentOutOfRangeException("i", i, Errors.NEGATIVE);
            int la = arr.Length;
            if (l==-1 || l+i>la)
                l=la-i;
            if (l<1)
                return -1;
            l += i;
            if (f == null)
            {
                if (!last)
                {
                    do
                    {
                        if (Object.ReferenceEquals(arr[i], null))
                            return i;
                    } while (++i != l);
                    return -1;
                }
                
                do
                {
                    if (Object.ReferenceEquals(arr[--l], null))
                        return l;
                } while (l != i);
                return -1;

            }
            int lf = f.Length;
            if (lf == 0)
            {
                if (!last)
                {
                    do
                    {
                        if (!Object.ReferenceEquals(arr[i], null) && arr[i].Length == 0)
                            return i;
                    } while (++i != l);
                    return -1;
                }

                do
                {
                    if (!Object.ReferenceEquals(arr[--l], null) && arr[l].Length == 0)
                        return l;
                } while (l != i);
                return -1;

            }
            unsafe
            {

                fixed (char* pf = f)
                {
                    string act;
                    if (comp == Ordinal.Binary)
                    {
                        if (!last)
                        {
                            do
                            {
                                act = arr[i];
                                if (Object.ReferenceEquals(act, f))
                                    return i;
                                if (!Object.ReferenceEquals(act, null))
                                {
                                    la = act.Length;
                                    if (lf == la)
                                    {
                                        fixed (char* pact = act)
                                        {
                                            if (Comp.Equal(pact, pf, lf))
                                                return i;
                                        }
                                    }
                                }
                            } while (++i != l);
                            return -1;
                        }
                        
                        do
                        {
                            act = arr[--l];
                            if (Object.ReferenceEquals(act, f))
                                return l;
                            if (!Object.ReferenceEquals(act, null))
                            {
                                la = act.Length;
                                if (lf == la)
                                {
                                    fixed (char* pact = act)
                                    {
                                        if (Comp.Equal(pact, pf, lf))
                                            return l;
                                    }
                                }
                            }
                        } while (l != i);
                        return -1;

                    }
                    char* pfmay;
                    Mem.Heap heap;
                    TableCase t = comp == Ordinal.Invariant ? TableCase.Invariant : Texts.TableCurrent;
                    if (lf < General.MaxStackChars && &lf>General.MinimumStack)
                    {
                        char* ptmp = stackalloc char[lf];
                        pfmay = ptmp;
                        heap = null;
                    }
                    else
                    {
                        heap = new EnPalermo.Mem.Heap(lf << 1, false);
                        pfmay = (char*)heap.Pointer;
                    }
                    Texts.ToUpper(pf, pfmay, lf, t);
                    if (!last)
                    {
                        do
                        {
                            act = arr[i];
                            if (Object.ReferenceEquals(act, f))
                            {
                                if (heap != null)
                                    heap.Dispose();
                                return i;
                            }
                            if (!Object.ReferenceEquals(act, null))
                            {
                                la = act.Length;
                                if (lf == la)
                                {
                                    fixed (char* pact = act)
                                    {
                                        la = 0;
                                        do
                                        {
                                            if (Texts.ToUpper(pact[la], t) != pfmay[la])
                                                break;
                                        } while (++la != lf);
                                        if (la == lf)
                                        {
                                            if (heap != null)
                                                heap.Dispose();
                                            return i;
                                        }
                                    }
                                }
                            }
                        } while (++i != l);
                        if (heap != null)
                            heap.Dispose();
                        return -1;
                    }
                    do
                    {
                        act = arr[--l];
                        if (Object.ReferenceEquals(act, f))
                        {
                            if (heap != null)
                                heap.Dispose();
                            return l;
                        }
                        if (!Object.ReferenceEquals(act, null))
                        {
                            la = act.Length;
                            if (lf == la)
                            {
                                fixed (char* pact = act)
                                {
                                    la = 0;
                                    do
                                    {
                                        if (Texts.ToUpper(pact[la], t) != pfmay[la])
                                            break;
                                    } while (++la != lf);
                                    if (la == lf)
                                    {
                                        if (heap != null)
                                            heap.Dispose();
                                        return l;
                                    }
                                }
                            }
                        }

                    } while (l != i);
                    if (heap != null)
                        heap.Dispose();
                    return -1;
                }
            }
        }



        #endregion

        #region FilterNoDigits

        /// <summary>
        /// Copia memoria filtrando todos los caracteres que no sean dgitos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="hexMode">Modo Hex.</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public unsafe static int RemoveNoDigits(bool alwaysCopy, char* po, char* pd, int l, out int lFinal, bool clean, bool hexMode)
        {
            if (l < 1)
            {
                lFinal = 0;
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            lFinal = IndexOfNoDigit(po, l, hexMode);
            if (lFinal == -1)
            {
                if (alwaysCopy)
                {
                    General.Copy(po, pd, l);
                    lFinal=l;
                }
                else
                    lFinal=po==pd?l:0;
                return 0;
            }
            General.Copy(po, pd, lFinal);
            int a=lFinal;
            while(++a!=l)
            {
                switch(po[a])
                {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        pd[lFinal++]=po[a];
                        continue;
                    case 'A':
                    case 'B':
                    case 'C':
                    case 'D':
                    case 'E':
                    case 'F':
                    case 'a':
                    case 'b':
                    case 'c':
                    case 'd':
                    case 'e':
                    case 'f':
                        if (hexMode)
                            pd[lFinal++]=po[a];
                        continue;
                }
                
            }
            l-=lFinal;
            if (l!=0 && clean && lFinal != 0)
                General.Clean(pd + lFinal, l);
            return l;
        }



        #endregion

        #region Carcteres blancos.

        /// <summary>
        /// Copia memoria filtrando carcteres blancos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public unsafe static int RemoveBlanks(bool alwaysCopy, char* po, char* pd, int l, out int lFinal, bool clean)
        {
            if (l < 1)
            {
                lFinal = 0;
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            lFinal = IndexOfBlank(po, l);
            if (lFinal == -1)
            {
                if (alwaysCopy)
                {
                    General.Copy(po, pd, l);
                    lFinal=l;
                }
                else
                    lFinal=po==pd?l:0;
                return 0;
            }
            General.Copy(po, pd, lFinal);
            int a=lFinal;
            while(++a!=l)
            {
                switch(po[a])
                {
                    case ' ':
                    case '\t':
                    case '\n':
                    case '\r':
                    case '\xa0':
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        continue;
                }
                pd[lFinal++]=po[a];
            }
            l-=lFinal;
            if (l!=0 && clean && lFinal != 0)
                General.Clean(pd + lFinal, l);
            return l;
        }

        /// <summary>
        /// Copia memoria filtrando carcteres blancos.
        /// </summary>
        /// <param name="po">Puntero Origen.</param>
        /// <param name="pd">Puntero Destino (puede ser el mismo de origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="lFinal">
        /// Variable de salida con el largo del puntero destino.
        /// En caso de que no exista copia de po a pd (y estos sean diferentes), siempre ser 0.
        /// </param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen eliminaciones.
        /// </param>
        /// <param name="clean">
        /// Si true, rellena con ceros el area en pd equivalente al
        /// numero de elementos eliminados a partir de lFinal.
        /// Si se dan estas condiciones: alwaysCopy es false y lFinal es 0,
        /// no se efectuar limpieza aunque existan eliminaciones.
        /// </param>
        /// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro el taburador (\t).</param>
        /// <returns>Cantidad de eliminaciones.</returns>
        public unsafe static int RemoveBlanks(bool alwaysCopy, char* po, char* pd, int l, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control, out int lFinal, bool clean)
        {
            if (l < 1)
            {
                lFinal = 0;
                return 0;
            }
            if (pd!=po && pd > po && pd - po < l)
            {
                General.Move(po, pd, l);
                po = pd;
            }
            lFinal = IndexOfBlank(po, l,sp,lat1,uni,tab,crlf,control);
            if (lFinal == -1)
            {
                if (alwaysCopy)
                {
                    General.Copy(po, pd, l);
                    lFinal=l;
                }
                else
                    lFinal=po==pd?l:0;
                return 0;
            }
            General.Copy(po, pd, lFinal);
            int a=lFinal;
            while(++a!=l)
            {
                switch(po[a])
                {
                    case ' ':
                        if (sp)
                            continue;
                        break;
                    case '\t':
                        if (tab)
                            continue;
                        break;
                    case '\n':
                    case '\r':
                        if (crlf)
                            continue;
                        break;
                    case '\xa0':
                    case '\x85':
                        if (lat1)
                            continue;
                        break;
                    case '\0':
                    case '\v':
                    case '\f':
                        if (control)
                            continue;
                        break;
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if (uni)
                            continue;
                        break;
                }
                pd[lFinal++]=po[a];
            }
            l-=lFinal;
            if (l!=0 && clean && lFinal != 0)
                General.Clean(pd + lFinal, l);
            return l;
        }

        /// <summary>
        /// Copia memoria reemplazando carcteres blancos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public unsafe static int ReplaceBlanks(char* p, int l, char r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                switch(p[a])
                {
                    case ' ':
                    case '\t':
                    case '\n':
                    case '\r':
                    case '\xa0':
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        tot++;
                        p[a] = r;
                        continue;
                }
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando carcteres blancos.
        /// </summary>
        /// <param name="p">Puntero Origen/Destino.</param>
        /// <param name="l">Largo.</param>
        /// <param name="r">Caracter de reemplazo.</param>
        /// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro el taburador (\t).</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public unsafe static int ReplaceBlanks(char* p, int l, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control, char r)
        {
            if (l < 1)
                return 0;
            int tot = 0, a = 0;
            do
            {
                switch(p[a])
                {
                    case ' ':
                        if (sp)
                        {
                            tot++;
                            p[a] = r;
                        }
                        continue;
                    case '\t':
                        if (tab)
                        {
                            tot++;
                            p[a] = r;
                        }
                        continue;
                    case '\n':
                    case '\r':
                        if (crlf)
                        {
                            tot++;
                            p[a] = r;
                        }
                        continue;
                    case '\xa0':
                    case '\x85':
                        if (lat1)
                        {
                            tot++;
                            p[a] = r;
                        }
                        continue;
                    case '\0':
                    case '\v':
                    case '\f':
                        if (control)
                        {
                            tot++;
                            p[a] = r;
                        }
                        continue;
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if (uni)
                        {
                            tot++;
                            p[a] = r;
                        }
                        continue;
                }
            } while (++a != l);
            return tot;
        }

        /// <summary>
        /// Copia memoria reemplazando carcteres blancos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <returns>Cantidad de reemplazos.</returns>
        public unsafe static int ReplaceBlanks(bool alwaysCopy, char* po, char* pd, int l, char r)
        {

            if (pd == po)
                return ReplaceBlanks(pd, l, r);
            if (l < 1)
                return 0;

            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return ReplaceBlanks(pd, l, r);
            }

            int ubi = TxtComp.IndexOfBlank(po, l);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            return ReplaceBlanks(pd + ubi, l - ubi, r) + 1;
        }

        /// <summary>
        /// Copia memoria reemplazando carcteres blancos.
        /// </summary>
        /// <param name="po">Puntero origen.</param>
        /// <param name="pd">Puntero destino (puede ser el mismo que origen).</param>
        /// <param name="l">Largo.</param>
        /// <param name="r">Reemplazo.</param>
        /// <param name="alwaysCopy">
        /// Si true siempre copiara po a pd.
        /// Si false, la copia de po a pd solo se realizar si hay que corregir solapado o
        /// se producen reemplazos.
        /// </param>
        /// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro el taburador (\t).</param>
        /// <returns>Cantidad de reemplazos.</returns>
        public unsafe static int ReplaceBlanks(bool alwaysCopy, char* po, char* pd, int l, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control, char r)
        {

            if (pd == po)
                return ReplaceBlanks(pd, l,sp,lat1,uni,tab,crlf,control, r);
            if (l < 1)
                return 0;

            if (alwaysCopy || (pd > po && pd - po < l))
            {
                General.Move(po, pd, l);
                return ReplaceBlanks(pd, l,sp,lat1,uni,tab,crlf,control, r);
            }

            int ubi = TxtComp.IndexOfBlank(po, l,sp,lat1,uni,tab,crlf,control);
            if (ubi == -1)
                return 0;
            General.Copy(po, pd, l);
            pd[ubi] = r;
            if (++ubi == l)
                return 1;
            return ReplaceBlanks(pd + ubi, l - ubi,sp,lat1,uni,tab,crlf,control, r) + 1;
        }

        #endregion

        
    }
}
