using System;
using EnPalermo.Lit;
using EnPalermo.Mem;

namespace EnPalermo.Txt
{
 
 
    /// <summary>
    /// Opciones de reemplazo de la instancia de cadena para funciones de validacin.
    /// </summary>
    public enum ValidateReplace
    {
        /// <summary>
        /// No efecta ningn reemplazo (siempre se conserva el original).
        /// </summary>
        None=0,
        /// <summary>
        /// Si la entrada es nula, la reemplaza con una cadena vacia o un valor por defecto.
        /// </summary>
        ReplaceNull=1,
        /// <summary>
        /// Si la entrada es nula, la reemplaza con una cadena vacia o un valor por defecto.
        /// Si la cadena de entrada es modificada, solo es reemplazada al salir si se superan las validaciones.
        /// </summary>
        ReplaceValidated=2,
        /// <summary>
        /// Si la entrada es nula, la reemplaza con una cadena vacia o un valor por defecto.
        /// Siempre reemplaza cadenas si ocurren modificaciones o existe un valor por defecto.
        /// </summary>
        ReplaceAlways=3
        

    }


    /// <summary>
    /// Resultado para funciones de validacin.
    /// </summary>
    [Flags()]
    public enum ValidateResult
    {
        /// <summary>
        /// La cadena de entrada es una cadena vacia o nula.
        /// La entrada no fue modificada.
        /// </summary>
        Empty=0,
        /// <summary>
        /// La cadena de entrada es una cadena vacia o nula.
        /// La cadena era nula y fue remplazada por otra vacia o el valor por defecto.
        /// </summary>
        NormalizedEmpty=1,
        /// <summary>
        /// La cadena no super las validaciones estndar.
        /// La entrada no fue modificada.
        /// </summary>
        NoValidated=2,
        /// <summary>
        /// La cadena no super las validaciones estndar.
        /// La cadena fue normalizada y la salida reemplazada por la cadena depurada o el valor por defecto.
        /// </summary>
        NoValidatedNormalized=NoValidated+NormalizedEmpty,
        /// <summary>
        /// El largo y los dgitos de la cadena (luego de las normalizaciones) son correctos, pero no se ha superado la validacin extra especfica de la funcin.
        /// La entrada no fue modificada.
        /// </summary>
        NoExtra=4,
        /// <summary>
        /// El largo y los dgitos de la cadena (luego de las normalizaciones) son correctos, pero no se ha superado la validacin extra especfica de la funcin.
        /// La cadena fue normalizada y la salida reemplazada por la cadena depurada o el valor por defecto.
        /// </summary>
        NoExtraNormalized=NoExtra+NormalizedEmpty,
        /// <summary>
        /// La cadena super las validaciones.
        /// La entrada no fue modificada.
        /// </summary>
        Validated=8,
        /// <summary>
        /// La cadena super las validaciones.
        /// La cadena fue normalizada y la salida reemplazada por la cadena depurada.
        /// </summary>
        ValidatedNormalized=Validated+NormalizedEmpty
    }

 
	/// <summary>
	/// Funciones estticas para la validacin de campos numericos y de cadena.
	/// </summary>
    public static class Validate
    {

 
        #region Solo Digitos

        /// <summary>
        /// Determina si un caracter es dgito del '0' a '9'.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <returns>True si es dgito.</returns>
        public static bool OnlyDigits(char c)
        {
            return c <= '9' && c >= '0';
        }

        /// <summary>
        /// Determina si la cadena esta compuesta solo por dgitos del '0' a '9'.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>True si solamente contiene dgitos del '0' a '9'.</returns>
        public static bool OnlyDigits(string str)
        {

            int l=str==null?0:str.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (char* p = str)
                {
                    int a = 0;
                    while (p[a] <= '9' && p[a] >= '0' && ++a != l)
                        ;
                    return a == l;
                }
            }
        }


        /// <summary>
        /// Determina si la cadena esta compuesta solo por dgitos del '0' a '9'.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si solamente contiene dgitos del '0' a '9'.</returns>
        public unsafe static bool OnlyDigits(char*p, int l)
        {
            if (l>0)
            {
                int a=0;
                do
                {
                    if (p[a]<'0' || p[a]>'9')
                        return false;
                }while(++a!=l);
                return true;
            }
            return false;
        }

        /// <summary>
        /// Determina si un caracter es dgito del '0' a '9', de 'A' a 'F' o de 'a' a 'f'.
        /// </summary>
        /// <param name="c">Caracter.</param>
        /// <returns>True si es dgito del '0' a '9', de 'A' a 'F' o de 'a' a 'f'.</returns>
        public static bool OnlyHex(char c)
        {
            switch(c)
            {
                case '0':
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                case 'A':
                case 'B':
                case 'C':
                case 'D':
                case 'E':
                case 'F':
                case 'a':
                case 'b':
                case 'c':
                case 'd':
                case 'e':
                case 'f':
                    return true;
            }
            return false;
        }

        /// <summary>
        /// Determina si la cadena esta compuesta solo por dgitos del '0' a '9', de 'A' a 'F' o de 'a' a 'f'.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <returns>True si solamente contiene dgitos del '0' a '9', de 'A' a 'F' o de 'a' a 'f'.</returns>
        public static bool OnlyHex(string str)
        {

            int l=str==null?0:str.Length;
            if (l == 0)
                return false;
            unsafe
            {
                fixed (char* p = str)
                {
                    return OnlyHex(p,l);
                }
            }
        }


        /// <summary>
        /// Determina si la cadena esta compuesta solo por dgitos del '0' a '9', de 'A' a 'F' o de 'a' a 'f'.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <returns>True si solamente contiene dgitos del '0' a '9', de 'A' a 'F' o de 'a' a 'f'.</returns>
        public unsafe static bool OnlyHex(char*p, int l)
        {
            if (l>0)
            {
                int a=0;
                do
                {
                    switch(p[a])
                    {
                        case '0':
                        case '1':
                        case '2':
                        case '3':
                        case '4':
                        case '5':
                        case '6':
                        case '7':
                        case '8':
                        case '9':
                        case 'A':
                        case 'B':
                        case 'C':
                        case 'D':
                        case 'E':
                        case 'F':
                        case 'a':
                        case 'b':
                        case 'c':
                        case 'd':
                        case 'e':
                        case 'f':
                            continue;
                    }
                    return false;
                }while(++a!=l);
                return true;
            }
            return false;
        }

        #endregion

        #region Es Numero

        /// <summary>
        /// Verifica si la cadena especificada es numrica.
        /// La verificacin, mas estricta que en StrToNum, se realiza del siguiente modo:
        /// 1- Hace trim.
        /// 2- El numero debe contener por lo menos 1 cifra.
        /// 3- Antes de entrar en zona numerica, puede estar el signo '-'.
        /// 4- Solo se acepta un separador decimal (no se aceptan separadores de miles) y este debe
        ///    encontrarse dentro de la zona numerica y no en sus limites.
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="decimalDot">Si true, el '.' ser considerado separador decimal. Si false, usar ','.</param>
        /// <param name="allowBool">Considera a las cadenas "TRUE", "FALSE", "True", "False", "true", "false", "T", "F", "t" y "f" como boleanas.</param>
        /// <returns>True si es un nmero.</returns>
        public static bool IsNumber(string str,bool decimalDot, bool allowBool)
        {
            int l=str==null?0:str.Length;
            if (l==0) return false;
            unsafe {fixed (char*p=str) return IsNumber(true, p,l,decimalDot,allowBool);}
        }

        /// <summary>
        /// Verifica si la cadena especificada es numrica.
        /// La verificacin, mas estricta que en StrToNum, se realiza del siguiente modo:
        /// 1- Hace trim.
        /// 2- El numero debe contener por lo menos 1 cifra.
        /// 3- Antes de entrar en zona numerica, puede estar el signo '-'.
        /// 4- Solo se acepta un separador decimal (no se aceptan separadores de miles) y este debe
        ///    encontrarse dentro de la zona numerica y no en sus limites.
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="trim">Efecta trim antes de comenzar.</param>
        /// <param name="decimalDot">Si true, el '.' ser considerado separador decimal. Si false, usar ','.</param>
        /// <param name="allowBool">
        /// Si true, efecta un trim inicial y considera a las cadenas "t", "true", "f" y "false" como boleanas (ignorando May/Min).
        /// </param>
        /// <returns>True si es un nmero.</returns>
        public unsafe static bool IsNumber(bool trim,char * p , int l,bool decimalDot, bool allowBool)
        {
            if (trim)
            {
                PChar pc =new PChar(p,l,true,true);
                if (pc.Length<1)
                    return false;
                p=pc.Pointer;
                l=pc.Length;
            }
            else if (l<1)
                return false;
            if (allowBool)
            {
                
                switch(l)
                {
                    case 1:
                        return (*p<='9' && *p>='0') || *p=='T' || *p=='t' || *p=='F' || *p=='f';
                    case 4:
                        if (*p=='T' || *p=='t')
                            return (p[1]=='R' || p[1]=='r') && (p[2]=='U' || p[2]=='u') && (p[3]=='E' || p[3]=='e');
                        break;
                    case 5:
                        if (*p=='F' || *p=='f')
                            return (p[1]=='A' || p[1]=='a') && (p[2]=='L' || p[2]=='l') && (p[3]=='S' || p[3]=='s') && (p[4]=='E' || p[4]=='e');
                        break;
                }
            }
            bool neg=false;
            bool zona=false;
            bool pasoDots=false;
            int a=0;
            do
            {
                switch(p[a])
                {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        zona=true;
                        continue;
                    case '-':
                        if (zona || neg) return false;
                        neg=true;
                        continue;
                    case ',':
                        if (decimalDot || !zona || pasoDots) return false;
                        goto testProx;
                    case '.':
                        if (!decimalDot || !zona || pasoDots) return false;
                        testProx:
                            int prox=a+1;
                        if (prox==l) return false;
                        prox=p[prox];
                        if ( prox>57 || prox<48) return false;
                        pasoDots=true;
                        a=prox;
                        continue;
                    default:
                        return false;
                }
            }while(++a!=l);
            return zona;
        }

        /// <summary>
        /// Verifica si la cadena especificada esta compuesta por dgitos.
        /// La verificacin, mas estricta que en StrToNum, se realiza del siguiente modo:
        /// 1- Hace trim.
        /// 2- El numero debe contener por lo menos una cifra.
        /// 3- Antes de entrar en zona numerica, puede estar el signo 'x' o "0x" (solo en modo Hex).
        /// </summary>
        /// <param name="str">Cadena.</param>
        /// <param name="allowSeps">Acepta los espacios, guiones, puntos, comas y ':' como separadores (solo se acepta 1 por separacin).</param>
        /// <param name="allowBlanks">Acepta carcteres blancos en cualquier ubicacin.</param>
        /// <param name="hexMode">Si true, incluye dgitos Hex.</param>
        /// <returns>True si la cadena esta compuesta por dgitos.</returns>
        public static bool IsPositiveInt(string str, bool allowBlanks, char allowSeps, bool hexMode)
        {
            int l=str==null?0:str.Length;
            if (l==0) return false;
            unsafe {fixed (char*p=str) return IsPositiveInt(p,l,allowBlanks,allowSeps,hexMode)!=0;}
        }

        /// <summary>
        /// Verifica si la cadena representa a un entero positivo convertible.
        /// La verificacin se realiza del siguiente modo:
        /// - No se acepta signo negativo.
        /// - Puede aceptar caracteres blancos o separadores internos si se lo especifica.
        /// - Antes de entrar en zona numerica, puede estar el smbolo 'x' o "0x" (solo en modo Hex).
        /// </summary>
        /// <param name="p">Puntero.</param>
        /// <param name="l">Largo.</param>
        /// <param name="allowBlanks">Acepta carcteres blancos en cualquier ubicacin.</param>
        /// <param name="allowSeps">
        /// Acepta los guiones internos, '\\', '/', '|', ':' como separadores (solo se acepta uno por separacin).
        /// </param>
        /// <param name="hexMode">Si true, acta en modo Hex.</param>
        /// <returns>Cantidad de dgitos o 0 si no est validado.</returns>
        public unsafe static int IsPositiveInt(char * p , int l, bool allowBlanks, char allowSeps, bool hexMode)
        {

            if (l<1)
                return 0;
            bool zona=false;
            int a=0;
            int tot=0;
            bool signHex=!hexMode;
            do
            {
            retest:
                switch(p[a])
                {
                    case '0':
                    case '1':
                    case '2':
                    case '3':
                    case '4':
                    case '5':
                    case '6':
                    case '7':
                    case '8':
                    case '9':
                        if(!zona)
                            zona=true;
                        tot++;
                        continue;
                    case 'A':
                    case 'B':
                    case 'C':
                    case 'D':
                    case 'E':
                    case 'F':
                    case 'a':
                    case 'b':
                    case 'c':
                    case 'd':
                    case 'e':
                    case 'f':
                        if (hexMode)
                            goto case '9';
                        return 0;
                    case 'x':
                    case 'X':
                        if (!signHex)
                        {
                            if (zona)
                            {
                                if (tot==1 && p[a-1]=='0')
                                {
                                    tot=0;
                                    zona=false;
                                }
                                else
                                    return 0;
                            }
                            signHex=true;
                            continue;
                        }
                        return 0;
                    case ' ':
                    case '\t':
                    case '\n':
                    case '\r':
                    case '\xa0':
                    case '\x85':
                    case '\0':
                    case '\v':
                    case '\f':
                    case '\u1680':
                    case '\u2000':
                    case '\u2001':
                    case '\u2002':
                    case '\u2003':
                    case '\u2004':
                    case '\u2005':
                    case '\u2006':
                    case '\u2007':
                    case '\u2008':
                    case '\u2009':
                    case '\u200a':
                    case '\u200b':
                    case '\u2028':
                    case '\u2029':
                    case '\u202f':
                    case '\u3000':
                    case '\ufeff':
                        if(allowBlanks)
                            continue;
                        return 0;
                    case '-':
                    case '|':
                    case ':':
                    case '/':
                    case '\\':
                        if (!zona || ++a==l || p[a]=='-' || p[a]=='|' || p[a]==':' || p[a]=='/' || p[a]=='\\')
                            return 0;
                        goto retest;
                    default:
                        return 0;
                }
            }while(++a!=l);
            return tot;
        }

        #endregion

        #region ValidateDigits y ValidateNumber

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="extraMin">
        /// Mnimo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="extraMax">
        /// Mximo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref decimal num, decimal min, decimal max, decimal extraMin, decimal extraMax, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, string format, System.IFormatProvider provider)
        {
            if (extraMin>extraMax)
                throw new ArgumentOutOfRangeException("extraMin,extraMax",Errors.MIN_MAJOR_MAX);
            ValidateResult ret=ValidateNumber(ref str,ref num,min,max,rep,mode,floatPoint,accept,format,provider);
            if ((ret&ValidateResult.Validated)!=0 && (num<extraMin || num>extraMax))
                return ValidateResult.NoExtra | (ret&ValidateResult.NormalizedEmpty);
            return ret;
        }

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult. Los modos extra no son utilizados.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref decimal num, decimal min, decimal max, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, string format, System.IFormatProvider provider)
        {
            ValidateResult ret;
            int l=str==null?0:str.Length;
            
            if (l==0)
                ret=ValidateResult.Empty;
            else
            {
                StrToNum n=new StrToNum(str,mode,floatPoint,accept);
                if (n.ReplaceIfRange(ref num,min,max))
                    ret=ValidateResult.Validated;
                else
                    ret=ValidateResult.NoValidated;
            }
            if (rep)
            {
                string nue=num.ToString(format,provider);
                if (!TxtComp.Equal(str,nue,false))
                {
                    str=nue;
                    return (ret|ValidateResult.NormalizedEmpty);
                }
            }
            return ret;
        }

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="extraMin">
        /// Mnimo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="extraMax">
        /// Mximo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref double num, double min, double max, double extraMin, double extraMax, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, string format, System.IFormatProvider provider)
        {
            if (extraMin>extraMax)
                throw new ArgumentOutOfRangeException("extraMin,extraMax",Errors.MIN_MAJOR_MAX);
            ValidateResult ret=ValidateNumber(ref str,ref num,min,max,rep,mode,floatPoint,accept,format,provider);
            if ((ret&ValidateResult.Validated)!=0 && (num<extraMin || num>extraMax))
                return ValidateResult.NoExtra | (ret&ValidateResult.NormalizedEmpty);
            return ret;
        }

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult. Los modos extra no son utilizados.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref double num, double min, double max, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, string format, System.IFormatProvider provider)
        {
            ValidateResult ret;
            int l=str==null?0:str.Length;
            
            if (l==0)
                ret=ValidateResult.Empty;
            else
            {
                StrToNum n=new StrToNum(str,mode,floatPoint,accept);
                if (n.ReplaceIfRange(ref num,min,max))
                    ret=ValidateResult.Validated;
                else
                    ret=ValidateResult.NoValidated;
            }
            if (rep)
            {
                string nue=num.ToString(format,provider);
                if (!TxtComp.Equal(str,nue,false))
                {
                    str=nue;
                    return (ret|ValidateResult.NormalizedEmpty);
                }
            }
            return ret;
        }


        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="extraMin">
        /// Mnimo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="extraMax">
        /// Mximo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="round">Redondeo para nmeros con decimales.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref ulong num, ulong min, ulong max, ulong extraMin, ulong extraMax, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, bool round, string format, System.IFormatProvider provider)
        {
            if (extraMin>extraMax)
                throw new ArgumentOutOfRangeException("extraMin,extraMax",Errors.MIN_MAJOR_MAX);
            ValidateResult ret=ValidateNumber(ref str,ref num,min,max,rep,mode,floatPoint,accept,round,format,provider);
            if ((ret&ValidateResult.Validated)!=0 && (num<extraMin || num>extraMax))
                return ValidateResult.NoExtra | (ret&ValidateResult.NormalizedEmpty);
            return ret;
            
            
        }

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="round">Redondeo para nmeros con decimales.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult. Los modos extra no son utilizados.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref ulong num, ulong min, ulong max, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, bool round, string format, System.IFormatProvider provider)
        {
            ValidateResult ret;
            int l=str==null?0:str.Length;
            
            if (l==0)
                ret=ValidateResult.Empty;
            else
            {
                StrToNum n=new StrToNum(str,mode,floatPoint,accept);
                if (((round)?(n.ReplaceIfRangeRound(ref num,min,max)):(n.ReplaceIfRange(ref num,min,max))))
                    ret=ValidateResult.Validated;
                else
                    ret=ValidateResult.NoValidated;
            }
            if (rep)
            {
                string nue=num.ToString(format,provider);
                if (!TxtComp.Equal(str,nue,false))
                {
                    str=nue;
                    return (ret|ValidateResult.NormalizedEmpty);
                }
            }
            return ret;
        }



        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="extraMin">
        /// Mnimo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="extraMax">
        /// Mximo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="round">Redondeo para nmeros con decimales.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref long num, long min, long max, long extraMin, long extraMax, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, bool round, string format, System.IFormatProvider provider)
        {
            if (extraMin>extraMax)
                throw new ArgumentOutOfRangeException("extraMin,extraMax",Errors.MIN_MAJOR_MAX);
            ValidateResult ret=ValidateNumber(ref str,ref num,min,max,rep,mode,floatPoint,accept,round,format,provider);
            if ((ret&ValidateResult.Validated)!=0 && (num<extraMin || num>extraMax))
                return ValidateResult.NoExtra | (ret&ValidateResult.NormalizedEmpty);
            return ret;
            
            
        }

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="round">Redondeo para nmeros con decimales.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult. Los modos extra no son utilizados.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref long num, long min, long max, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, bool round, string format, System.IFormatProvider provider)
        {
            ValidateResult ret;
            int l=str==null?0:str.Length;
            
            if (l==0)
                ret=ValidateResult.Empty;
            else
            {
                StrToNum n=new StrToNum(str,mode,floatPoint,accept);
                if (((round)?(n.ReplaceIfRangeRound(ref num,min,max)):(n.ReplaceIfRange(ref num,min,max))))
                    ret=ValidateResult.Validated;
                else
                    ret=ValidateResult.NoValidated;
            }
            if (rep)
            {
                string nue=num.ToString(format,provider);
                if (!TxtComp.Equal(str,nue,false))
                {
                    str=nue;
                    return (ret|ValidateResult.NormalizedEmpty);
                }
            }
            return ret;
        }


        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="extraMin">
        /// Mnimo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="extraMax">
        /// Mximo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="round">Redondeo para nmeros con decimales.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref uint num, uint min, uint max, uint extraMin, uint extraMax, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, bool round, string format, System.IFormatProvider provider)
        {
            if (extraMin>extraMax)
                throw new ArgumentOutOfRangeException("extraMin,extraMax",Errors.MIN_MAJOR_MAX);
            ValidateResult ret=ValidateNumber(ref str,ref num,min,max,rep,mode,floatPoint,accept,round,format,provider);
            if ((ret&ValidateResult.Validated)!=0 && (num<extraMin || num>extraMax))
                return ValidateResult.NoExtra | (ret&ValidateResult.NormalizedEmpty);
            return ret;
            
            
        }

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="round">Redondeo para nmeros con decimales.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult. Los modos extra no son utilizados.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref uint num, uint min, uint max, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, bool round, string format, System.IFormatProvider provider)
        {
            ValidateResult ret;
            int l=str==null?0:str.Length;
            
            if (l==0)
                ret=ValidateResult.Empty;
            else
            {
                StrToNum n=new StrToNum(str,mode,floatPoint,accept);
                if (((round)?(n.ReplaceIfRangeRound(ref num,min,max)):(n.ReplaceIfRange(ref num,min,max))))
                    ret=ValidateResult.Validated;
                else
                    ret=ValidateResult.NoValidated;
            }
            if (rep)
            {
                string nue=num.ToString(format,provider);
                if (!TxtComp.Equal(str,nue,false))
                {
                    str=nue;
                    return (ret|ValidateResult.NormalizedEmpty);
                }
            }
            return ret;
        }

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="extraMin">
        /// Mnimo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="extraMax">
        /// Mximo aceptable extra (inclusive). La validacin extra solo se aplica en nmeros validados.
        /// Este rango est diseado para la muestra de advertencias y otras situaciones no criticas.
        /// Un valor fuera de rango solo influye en el retorno ValidateResult.
        /// El nmero permanece validado de acuerdo a la validacin principal y ValidateResult pasa de Validated a NoExtra.
        /// </param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="round">Redondeo para nmeros con decimales.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref int num, int min, int max, int extraMin, int extraMax, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, bool round, string format, System.IFormatProvider provider)
        {
            if (extraMin>extraMax)
                throw new ArgumentOutOfRangeException("extraMin,extraMax",Errors.MIN_MAJOR_MAX);
            ValidateResult ret=ValidateNumber(ref str,ref num,min,max,rep,mode,floatPoint,accept,round,format,provider);
            if ((ret&ValidateResult.Validated)!=0 && (num<extraMin || num>extraMax))
                return ValidateResult.NoExtra | (ret&ValidateResult.NormalizedEmpty);
            return ret;
            
            
        }

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="num">Referencia a nmero, este nmero es reemplazado solamente si la validacin es exitosa.</param>
        /// <param name="min">Mnimo aceptable (inclusive).</param>
        /// <param name="max">Mximo aceptable (inclusive).</param>
        /// <param name="rep">
        /// Si true, siempre reemplazar la cadena por el valor convertido o por defecto (si este es diferente).
        /// Si false, solo reemplazar en caso de validar un nuevo valor (si este es diferente).
        /// </param>
        /// <param name="mode">Modo de reconocimiento.</param>
        /// <param name="round">Redondeo para nmeros con decimales.</param>
        /// <param name="floatPoint">Opciones de coma flotante. Ignoradas en los modos Integer y Hex.</param>
        /// <param name="accept">Opciones de tolerancia en el reconocimiento.</param>
        /// <param name="format">Cadena de formato para la representacin de cadena.</param>
        /// <param name="provider">Proveedor de formato para la representacin de cadena.</param>
        /// <returns>ValidateResult. Los modos extra no son utilizados.</returns>
        public static ValidateResult ValidateNumber(ref string str, ref int num, int min, int max, bool rep, NumericMode mode, NumericDecimal floatPoint, NumericAccept accept, bool round, string format, System.IFormatProvider provider)
        {
            ValidateResult ret;
            int l=str==null?0:str.Length;
            
            if (l==0)
                ret=ValidateResult.Empty;
            else
            {
                StrToNum n=new StrToNum(str,mode,floatPoint,accept);
                if (((round)?(n.ReplaceIfRangeRound(ref num,min,max)):(n.ReplaceIfRange(ref num,min,max))))
                    ret=ValidateResult.Validated;
                else
                    ret=ValidateResult.NoValidated;
            }
            if (rep)
            {
                string nue=num.ToString(format,provider);
                if (!TxtComp.Equal(str,nue,false))
                {
                    str=nue;
                    return (ret|ValidateResult.NormalizedEmpty);
                }
            }
            return ret;
        }

        /// <summary>
        /// Efecta una validacin de dgitos.
        /// Solo para nmeros enteros positivos y Hexs.
        /// </summary>
        /// <param name="str">Referencia de cadena.</param>
        /// <param name="filter">Opciones de preprocesamiento antes de ejecutar la validacin.</param>
        /// <param name="rep">Modo de reemplazo de la instancia de cadena.</param>
        /// <param name="lReq">Cantidad de dgitos requerida para validar la cadena. 0 para no validar por la cantidad de dgitos.</param>
        /// <param name="hexMode">Modo Hex.</param>
        /// <returns>ValidateResult. Los modos extra no son utilizados.</returns>
        public static ValidateResult ValidateDigits(ref string str, NormalizeFilter filter, ValidateReplace rep, int lReq, bool hexMode)
        {
            if (lReq<0)
                throw new ArgumentOutOfRangeException("lReq",lReq,Errors.NEGATIVE);
            bool trans;
            if (str==null)
            {
                trans=true;
                goto empty;
            }
            int l=str.Length;
            
            if (l==0)
            {
                trans=false;
                goto empty;
            }
            string nue;
            if (filter!=NormalizeFilter.None)
            {
                nue=Texts.NormalizeString(false,str,filter,0);
                if (nue as object != str as object)
                {
                    trans=true;
                    l=nue.Length;
                    if (l==0)
                        goto empty;
                }
                else
                    trans=false;
            }
            else
            {
                nue=str;
                trans=false;
            }
            if ((lReq!=0 && l!=lReq) || ((hexMode)?(!OnlyHex(nue)):(!OnlyDigits(nue))))
            {
                if (trans && rep==ValidateReplace.ReplaceAlways)
                {
                    str=nue;
                    return ValidateResult.NoValidatedNormalized;
                }
                return ValidateResult.NoValidated;
            }
            if (trans && rep>ValidateReplace.ReplaceNull)
            {
                str=nue;
                return ValidateResult.ValidatedNormalized;
            }
            return ValidateResult.Validated;
            empty:
                if (trans && rep!=ValidateReplace.None)
                {
                    str=Others.Empty;
                    return ValidateResult.NormalizedEmpty;
                }
            return ValidateResult.Empty;
        }



        #endregion

	}
}
