using System;
using System.IO;
using EnPalermo.Mem;
using EnPalermo.Lit;


namespace EnPalermo.Uti
{
    /// <summary>
    /// Funciones estticas de utilidad para serializar, deserializar e interactuar con streams.
    /// </summary>
    public unsafe static class Serializer
    {

        #region Clculos

        /// <summary>
        /// Efectua checkeo de parmetros.
        /// </summary>
        /// <param name="bytes">Bytes.</param>
        /// <param name="oneNeg">Acepta -1 como parmetro.</param>
        /// <param name="dLimit">Genera excepcin si el valor es igual o mayor a 1 GB.</param>
        /// <returns>True si bytes es diferente a 0.</returns>
        internal static bool CheckParams(int bytes, bool oneNeg, bool dLimit)
        {
            if (oneNeg)
            {
                if (bytes<-1)
                    throw new ArgumentOutOfRangeException("bytes",bytes,Errors.MINOR_ONE_NEGATIVE);
            }
            else if (bytes<0)
                throw new ArgumentOutOfRangeException("bytes",bytes,Errors.NEGATIVE);
            if (dLimit && bytes>=1<<30)
                throw new ArgumentOutOfRangeException("bytes",bytes,Errors.BIG_VALUE);
            return bytes!=0;
        }

        /// <summary>
        /// Efectua checkeo de parmetros.
        /// </summary>
        /// <param name="bytes">Bytes.</param>
        /// <param name="sof">SizeOf.</param>
        /// <param name="oneNeg">Acepta -1 como parmetro.</param>
        /// <param name="dLimit">Genera excepcin si el valor es igual o mayor a 1 GB.</param>
        /// <returns>True si bytes es igual o mayor a sof, o -1 si oneNeg es true.</returns>
        internal static bool CheckParams(int bytes, bool oneNeg, bool dLimit, int sof)
        {
            if (sof<1)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
            if (oneNeg)
            {
                if (bytes<sof)
                {
                    if (bytes<-1)
                        throw new ArgumentOutOfRangeException("bytes",bytes,Errors.MINOR_ONE_NEGATIVE);
                    return bytes!=0;
                }
            }
            else if (bytes<sof)
            {
                if (bytes<0)
                    throw new ArgumentOutOfRangeException("bytes",bytes,Errors.NEGATIVE);
                return false;
            }
            if (dLimit && bytes>=1<<30)
                throw new ArgumentOutOfRangeException("bytes",bytes,Errors.BIG_VALUE);
            return true;
        }

        /// <summary>
        /// Efectua checkeo de parmetros.
        /// </summary>
        /// <param name="sof">SizeOf.</param>
        /// <param name="items">Items.</param>
        /// <param name="oneNeg">Acepta -1 como parmetro.</param>
        /// <param name="dLimit">Genera excepcin si items
        /// es mayor a (2GB-1)/(sof*2). Si es false, genera excepcin si items es mayor a (2GB-1)/(sof).</param>
        /// <returns>
        /// Si oneNeg es true:
        /// Devuelve true items es mayor a cero o -1.
        /// Si oneNeg es false:
        /// Devuelve true items es mayor a cero.</returns>
        internal static bool CheckParams(bool oneNeg, bool dLimit, int items, int sof)
        {
            if (sof<1)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
            
            if (oneNeg)
            {
                if (items<1)
                {
                    if (items<-1)
                        throw new ArgumentOutOfRangeException("items",items,Errors.MINOR_ONE_NEGATIVE);
                    return items!=0;
                }
            }
            else if (items<1)
            {
                if (items!=0)
                    throw new ArgumentOutOfRangeException("items",items,Errors.NEGATIVE);
                return false;
            }
           
            if (dLimit)
            {
                if (items>((1<<30)-1)/sof)
                    throw new ArgumentOutOfRangeException("items",items,Errors.BIG_VALUE);
            }
            else if (items>Int32.MaxValue/sof)
                throw new ArgumentOutOfRangeException("items",items,Errors.BIG_VALUE);
            return true;
        }


        /// <summary>
        /// Efectua checkeo de parmetros.
        /// </summary>
        /// <param name="bytes">Bytes.</param>
        /// <param name="sof">SizeOf.</param>
        /// <param name="items">Items.</param>
        /// <param name="oneNeg">Acepta -1 como parmetro.</param>
        /// <param name="dLimit">Genera excepcin si el valor es igual o mayor a 1 GB en bytes o si items
        /// es mayor a (2GB-1)/(sof*2). Si es false, solo genera excepcin si items es mayor a (2GB-1)/(sof).</param>
        /// <returns>
        /// Si oneNeg es true:
        /// Devuelve true si bytes es -1 o mayor o igual a sof e items es diferente a cero.
        /// Si oneNeg es false:
        /// Devuelve true si bytes es mayor o igual a sof e items es mayor a cero.</returns>
        internal static bool CheckParams(int bytes, bool oneNeg, bool dLimit, int items, int sof)
        {
            if (sof<1)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
            
            if (oneNeg)
            {
                if (bytes<sof)
                {
                    if (bytes<-1)
                        throw new ArgumentOutOfRangeException("bytes",bytes,Errors.MINOR_ONE_NEGATIVE);
                    if (bytes!=-1)
                        return false;
                }
                else if (dLimit && bytes>=1<<30)
                    throw new ArgumentOutOfRangeException("bytes",bytes,Errors.BIG_VALUE);
                
                if (items<1)
                {
                    if (items<-1)
                        throw new ArgumentOutOfRangeException("items",items,Errors.MINOR_ONE_NEGATIVE);
                    return items!=0;
                }
            }
            else if (bytes<sof)
            {
                if (bytes<0)
                    throw new ArgumentOutOfRangeException("bytes",bytes,Errors.NEGATIVE);
                return false;
            }
            else if (dLimit && bytes>=1<<30)
                throw new ArgumentOutOfRangeException("bytes",bytes,Errors.BIG_VALUE);
            else if (items<1)
            {
                if (items!=0)
                    throw new ArgumentOutOfRangeException("items",items,Errors.NEGATIVE);
                return false;
            }
           
            if (dLimit)
            {
                if (items>((1<<30)-1)/sof)
                    throw new ArgumentOutOfRangeException("items",items,Errors.BIG_VALUE);
            }
            else if (items>Int32.MaxValue/sof)
                throw new ArgumentOutOfRangeException("items",items,Errors.BIG_VALUE);
            return true;
        }


        /// <summary>
        /// Verifica parmetros y calcula la cantidad necesaria de bytes para efectuar una serializacin.
        /// </summary>
        /// <param name="items">Elementos a codificar.</param>
        /// <param name="sof">Ancho en bytes de cada elemento. Debe ser mayor a cero.</param>
        /// <returns>Cantidad de bytes necesaria para serializar los items.</returns>
        public static int CalcBytes(int items, int sof)
        {
            if (!CheckParams(false,false,items,sof))
                return 0;
            return items*sof;
        }

        /// <summary>
        /// Verifica parmetros y calcula la cantidad de items a deserializar en una cadena de bytes.
        /// </summary>
        /// <param name="bytes">Largo de la cadena de bytes, esta se corrige al mltiplo de sof.</param>
        /// <param name="sof">Ancho en bytes de cada elemento. Debe ser mayor a cero.</param>
        /// <returns>Cantidad de items a deserializar segn bytes y sof.</returns>
        public static int CalcItems(ref int bytes, int sof)
        {
            if (!CheckParams(bytes,false,false,sof))
                return 0;
            int ret=bytes/sof;
            bytes=sof*ret;
            return ret;
        }

        /// <summary>
        /// Verifica parmetros y determina el mximo en Bytes requerido para la lectura de un stream.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// Si maxBytes es -1 y el largo recuperable del stream supera 2GB-1 bytes,
        /// genera excepcin.
        /// </param>
        /// <param name="maxBytes">
        /// Establece un limite en bytes para la lectura.
        /// Si es -1, se establece inicialmente en 2GB-1 bytes. 
        /// </param>
        /// <returns>
        /// Valor maxBytes sincronizado a menor cantidad con la capacidad real del stream.
        /// </returns>
        public static int CalcStreamRead(Stream stream, int maxBytes)
        {
            if (!CheckParams(maxBytes,true,false))
                return 0;
            long ls=stream.Length-stream.Position;
            if (maxBytes==-1)
            {
                if (ls>=(1L<<32))
                    throw new NotSupportedException(Errors.BIG_STREAM);
                if(ls<1L)
                    return 0;
                return (int)ls;
            }
            if (ls<(long)maxBytes)
            {
                if(ls<1L)
                    return 0;
                return (int)ls;
            }
            return maxBytes;
        }

        /// <summary>
        /// Verifica parmetros y determina el mximo en Items y Bytes requerido para la lectura de un stream.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// Si maxItems es -1 y el largo recuperable del stream supera 2GB-1 bytes,
        /// genera excepcin.
        /// </param>
        /// <param name="maxItems">
        /// En la entrada, establece un limite en items decodificados para la lectura.
        /// En la salida actualiza la variable de acuerdo a la capacidad del stream.
        /// Si es -1, se establece inicialmente en (2GB - 1)/sof. En cambio, si es mayor a (2GB - 1)/sof, genera excepcin.
        /// </param>
        /// <param name="sof">Ancho en bytes de cada elemento. Debe ser mayor a cero.</param>
        /// <returns>
        /// Valor de bytes necesarios (maxItems*sof).
        /// </returns>
        public static int CalcStreamReadItems(Stream stream, ref int maxItems, int sof)
        {
            if (!CheckParams(true,false,maxItems,sof))
                return 0;
            long ls=stream.Length-stream.Position;
            
            if (maxItems==-1)
            {
                if (ls>=(1L<<32))
                    throw new NotSupportedException(Errors.BIG_STREAM);
                if ((int)ls<sof)
                {
                    maxItems=0;
                    return 0;
                }
                maxItems=((int)ls)/sof;
                return maxItems*sof;
            }
            
            if (ls>=(1L<<32))
                return maxItems*sof;

            int maxBytes=maxItems*sof;
            if (maxBytes>(int)ls)
            {
                if ((int)ls<sof)
                {
                    maxItems=0;
                    return 0;
                }
                maxItems=((int)ls)/sof;
                return maxItems*sof;
            }
            return maxBytes;
        }

        /// <summary>
        /// Verifica parmetros y determina el mximo en Items y Bytes requerido para la lectura de un stream.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// Si tanto maxBytes y maxItems son -1 y el largo recuperable del stream supera 2GB-1 bytes,
        /// genera excepcin.
        /// </param>
        /// <param name="maxBytes">
        /// Establece un limite en bytes para la lectura.
        /// Si es -1, se establece inicialmente en 2GB-1 bytes. 
        /// </param>
        /// <param name="maxItems">
        /// En la entrada, establece un limite en items decodificados para la lectura.
        /// En la salida actualiza la variable y la sincroniza con maxBytes de acuerdo a la capacidad del stream.
        /// Si es -1, se establece inicialmente en (2GB - 1)/sof. En cambio, si es mayor a (2GB - 1)/sof, genera excepcin.
        /// </param>
        /// <param name="sof">Ancho en bytes de cada elemento. Debe ser mayor a cero.</param>
        /// <returns>
        /// Valor maxBytes sincronizado a menor cantidad con la capacidad real del stream y maxItems.
        /// </returns>
        public static int CalcStreamReadItems(Stream stream, int maxBytes, ref int maxItems, int sof)
        {
            if (!CheckParams(maxBytes,true,false,maxItems,sof))
                return 0;
            long ls=stream.Length-stream.Position;

            if (maxBytes==-1)
            {
                if (maxItems==-1)
                {
                    if (ls>=(1L<<32))
                        throw new NotSupportedException(Errors.BIG_STREAM);
                    if ((int)ls<sof)
                    {
                        maxItems=0;
                        return 0;
                    }
                    maxItems=((int)ls)/sof;
                    return maxItems*sof;
                }
                maxBytes=maxItems*sof;
            }
            else if (maxItems==-1)
                maxItems=maxBytes/sof;
            
            if (ls<(long)maxBytes)
                maxBytes=(int)ls;
            
            if (maxBytes<sof)
            {
                maxItems=0;
                return 0;
            }
            int mch=maxBytes/sof;
            if (mch<maxItems)
                maxItems=mch;
            else if (mch!=maxItems)
                return maxItems*sof;
            return mch*sof;
        }

        #endregion

        #region StreamRead

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrDest">
        /// Array destino.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes recuperados desde el Stream.
        /// </returns>
        public static int StreamRead(Stream stream, bool close, byte[] arrDest)
        {
            return StreamRead(stream,close,arrDest,0,arrDest.Length);
        }


        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrDest">
        /// Array destino. Debe tener una capacidad coherente con maxBytes.
        /// </param>
        /// <param name="indexDest">Index en el array destino donde comenzar la copia.</param>
        /// <param name="maxBytes">
        /// Cantidad mxima en bytes a recuperar.
        /// La capacidad de arrDest debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes recuperados desde el Stream.
        /// </returns>
        public static int StreamRead(Stream stream, bool close, byte[] arrDest, int indexDest, int maxBytes)
        {
            if (maxBytes<1)
            {
                if (maxBytes!=0)
                    throw new ArgumentOutOfRangeException("maxBytes",maxBytes,Errors.NEGATIVE);
                goto end;
            }
            if (indexDest<0 || indexDest+maxBytes>arrDest.Length)
                throw new ArgumentOutOfRangeException("indexDest,maxBytes",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int ret=stream.Read(arrDest,indexDest,maxBytes);
            end:
                if (close)
                    stream.Close();
            return maxBytes;
        }

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe tener una capacidad coherente con maxBytes.
        /// </param>
        /// <param name="maxBytes">
        /// Cantidad mxima en bytes a recuperar.
        /// La capacidad de pd debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes recuperados desde el Stream.
        /// </returns>
        public static int StreamRead(Stream stream, bool close, byte* pd, int maxBytes)
        {
            return StreamRead(stream,close,pd,maxBytes,null,0);
        }

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe tener una capacidad coherente con maxBytes.
        /// </param>
        /// <param name="arrTemp">
        /// Array temporal. Si es null o su capacidad til no es practica para esta funcionalidad, se crear uno.
        /// </param>
        /// <param name="tempIndex">
        /// Index en el array donde comienza el espacio utilizable para copia temporal.
        /// </param>
        /// <param name="maxBytes">
        /// Cantidad mxima en bytes a recuperar.
        /// La capacidad de pd debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes recuperados desde el Stream.
        /// </returns>
        public static int StreamRead(Stream stream, bool close, byte* pd, int maxBytes, byte[] arrTemp, int tempIndex)
        {
            if (pd==null)
                throw new ArgumentNullException("pd",Errors.NULL_POINTER);

            if (tempIndex<0)
                throw new ArgumentOutOfRangeException("tempIndex",tempIndex,Errors.NEGATIVE);

            if (maxBytes<1)
            {
                if (maxBytes!=0)
                    throw new ArgumentOutOfRangeException("maxBytes",maxBytes,Errors.NEGATIVE);
                goto end;
            }

            int maxBuf=(arrTemp!=null?arrTemp.Length:0)-tempIndex;
            
            if (maxBuf>=maxBytes)
                goto onePass;

            if (maxBuf<1 || (maxBuf<256 && maxBytes/maxBuf>255))
            {
                if (maxBuf<0)
                    throw new ArgumentOutOfRangeException("tempIndex",tempIndex,Errors.SUPERIOR_ARRAY);
                tempIndex=0;
                if (maxBytes>=1<<20)
                    maxBuf=8192;
                else if (maxBytes>=1<<18)
                    maxBuf=4096;
                else if (maxBytes>=1<<16)
                    maxBuf=2048;
                else if (maxBytes<=4096)
                {
                    arrTemp=new byte[maxBytes];
                    goto onePass;
                }
                else
                    maxBuf=1024;
                arrTemp=new byte[maxBuf];
            }

            fixed(byte*p=arrTemp)
            {
                byte*po=p+tempIndex;
                if (po==pd)
                    throw new ArgumentException(Errors.SER_STREAM_P_EQUAL_ARRAY,"arrTemp");

                int reads,ret=0;
                while((reads=stream.Read(arrTemp,tempIndex,maxBuf))!=0)
                {
                    General.Copy(po,pd+ret,reads);
                    ret+=reads;
                    if (maxBytes-ret<maxBuf)
                    {
                        if (maxBytes==ret)
                            break;
                        maxBuf=maxBytes-ret;
                    }
                }
                maxBytes=ret;
                goto end;
            }

            onePass:

                fixed(byte*p=arrTemp)
                {
                    maxBytes=stream.Read(arrTemp,tempIndex,maxBytes);
                    if (maxBytes!=0)
                        General.Copy(p+tempIndex,pd,maxBytes);
                }

            end:
                if (close)
                    stream.Close();
            return maxBytes;

        }



        #endregion

        #region SteamReadItems

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual, deserializando elementos de tamao fijo. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrDest">
        /// Array destino.
        /// </param>
        /// <param name="sof">
        /// Ancho en bytes de cada elemento. La lectura en bytes se calcula en base a maxItems*sof.
        /// Debe ser mayor a cero.
        ///  </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Items (bytes/sof) recuperados desde el Stream.
        /// </returns>
        public static int StreamReadItems(Stream stream, bool close, byte[] arrDest, int sof)
        {
            if (sof<1)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
            return StreamReadItems(stream,close,arrDest,0,arrDest.Length/sof,sof);
        }

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual, deserializando elementos de tamao fijo. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrDest">
        /// Array destino. Debe tener una capacidad coherente con maxItems.
        /// </param>
        /// <param name="destIndex">Index en el array destino donde comenzar la copia (en bytes).</param>
        /// <param name="maxItems">
        /// Cantidad mxima en items a recuperar.
        /// La capacidad de arrDest debe ser coherente con este valor.
        /// </param>
        /// <param name="sof">
        /// Ancho en bytes de cada elemento. La lectura en bytes se calcula en base a maxItems*sof.
        /// Debe ser mayor a cero.
        ///  </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Items (bytes/sof) recuperados desde el Stream.
        /// </returns>
        public static int StreamReadItems(Stream stream, bool close, byte[] arrDest, int destIndex, int maxItems, int sof)
        {
            if (maxItems<1)
            {
                if (maxItems!=0)
                    throw new ArgumentOutOfRangeException("maxItems",maxItems,Errors.NEGATIVE);
                goto end;
            }

            int maxBytes=CalcBytes(maxItems,sof);

            if (destIndex<0 || destIndex+maxBytes>arrDest.Length)
                throw new ArgumentOutOfRangeException("destIndex,maxItems",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int ret=stream.Read(arrDest,destIndex,maxBytes);
            if (ret!=maxBytes)
            {
                maxItems=ret/sof;
                maxBytes=maxItems*sof;
                if (ret!=maxBytes)
                    fixed(byte*pd=arrDest)
                        General.Clean(pd+(destIndex+maxBytes),ret-maxBytes);
            }

            end:
                if (close)
                    stream.Close();
            return maxItems;
        }

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual, deserializando elementos de tamao fijo. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe tener una capacidad coherente con maxItems.
        /// </param>
        /// <param name="maxItems">
        /// Cantidad mxima en items a recuperar.
        /// La capacidad de pd debe ser coherente con este valor.
        /// </param>
        /// <param name="sof">
        /// Ancho en bytes de cada elemento. La lectura en bytes se calcula en base a maxItems*sof.
        /// Debe ser mayor a cero.
        ///  </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Items (bytes/sof) recuperados desde el Stream.
        /// </returns>
        public static int StreamReadItems(Stream stream, bool close, void* pd, int maxItems, int sof)
        {
            return StreamReadItems(stream,close,pd,maxItems,sof,null,0);
        }

        /// <summary>
        /// Efecta lectura de un Stream, desde su posicin actual, deserializando elementos de tamao fijo. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="pd">
        /// Puntero destino. Debe tener una capacidad coherente con maxItems.
        /// </param>
        /// <param name="arrTemp">
        /// Array temporal. Si es null o su capacidad til no es practica para esta funcionalidad, se crear uno.
        /// </param>
        /// <param name="tempIndex">
        /// Index en el array donde comienza el espacio utilizable para copia temporal (en bytes).
        /// </param>
        /// <param name="maxItems">
        /// Cantidad mxima en items a recuperar.
        /// La capacidad de pd debe ser coherente con este valor.
        /// </param>
        /// <param name="sof">
        /// Ancho en bytes de cada elemento. La lectura en bytes se calcula en base a maxItems*sof.
        /// Debe ser mayor a cero.
        ///  </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la lectura.
        /// </param>
        /// <returns>
        /// Cantidad de Items (bytes/sof) recuperados desde el Stream.
        /// </returns>
        public static int StreamReadItems(Stream stream, bool close, void* pd, int maxItems, int sof, byte[] arrTemp, int tempIndex)
        {
            if (sof<2)
            {
                if (sof==1)
                    return StreamRead(stream,close,(byte*)pd,maxItems,arrTemp,tempIndex);
                throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
            }
            if (maxItems<1)
            {
                if (maxItems!=0)
                    throw new ArgumentOutOfRangeException("maxItems",maxItems,Errors.NEGATIVE);
                if (close)
                    stream.Close();
                return 0;
            }
            if (maxItems>Int32.MaxValue/sof)
                throw new ArgumentOutOfRangeException("maxItems",maxItems,Errors.BIG_VALUE);
            int maxBytes=maxItems*sof;
            int ret=StreamRead(stream,close,(byte*)pd,maxBytes,arrTemp,tempIndex);
            if (ret!=maxBytes)
            {
                maxItems=ret/sof;
                maxBytes=maxItems*sof;
                if (ret!=maxBytes)
                    General.Clean((byte*)pd+maxBytes,ret-maxBytes);
            }
            return maxItems;
        }
  
        #endregion
 
        #region StreamWrite

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream (siempre equivale a bytesCount).
        /// </returns>
        public static int StreamWrite(Stream stream, bool close, byte[] arrOrig)
        {
            return StreamWrite(stream,close,arrOrig,0,arrOrig.Length);
        }

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen. Debe tener una capacidad coherente con bytesCount.
        /// </param>
        /// <param name="indexOrig">
        /// Index en el array origen donde comenzar la copia.
        /// </param>
        /// <param name="bytesCount">
        /// Cantidad de bytes a escribr.
        /// La capacidad de arrOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream (siempre equivale a bytesCount).
        /// </returns>
        public static int StreamWrite(Stream stream, bool close, byte[] arrOrig, int indexOrig, int bytesCount)
        {
            if (bytesCount<1)
            {
                if (bytesCount!=0)
                    throw new ArgumentOutOfRangeException("bytesCount",bytesCount,Errors.NEGATIVE);
                goto end;
            }
            if (indexOrig<0 || indexOrig+bytesCount>arrOrig.Length)
                throw new ArgumentOutOfRangeException("indexOrig,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            stream.Write(arrOrig,indexOrig,bytesCount);
            end:
                if (close)
                    stream.Close();
            return bytesCount;
        }

        
        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="po">
        /// Puntero origen. Debe tener una capacidad coherente con bytesCount.
        /// </param>
        /// <param name="bytesCount">
        /// Cantidad de bytes a escribr.
        /// La capacidad de po debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream (siempre equivale a bytesCount).
        /// </returns>
        public static int StreamWrite(Stream stream, bool close, byte* po, int bytesCount)
        {
            return StreamWrite(stream,close,po,bytesCount,null,0);
        }

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="po">
        /// Puntero origen. Debe tener una capacidad coherente con bytesCount.
        /// </param>
        /// <param name="arrTemp">
        /// Array temporal. Si es null o su capacidad til no es practica para esta funcionalidad, se crear uno.
        /// </param>
        /// <param name="tempIndex">
        /// Index en el array donde comienza el espacio utilizable para copia temporal.
        /// </param>
        /// <param name="bytesCount">
        /// Cantidad de bytes a escribr.
        /// La capacidad de po debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream (siempre equivale a bytesCount).
        /// </returns>
        public static int StreamWrite(Stream stream, bool close, byte* po, int bytesCount, byte[] arrTemp, int tempIndex)
        {
            if (po==null)
                throw new ArgumentNullException("po",Errors.NULL_POINTER);

            if (tempIndex<0)
                throw new ArgumentOutOfRangeException("tempIndex",tempIndex,Errors.NEGATIVE);

            if (bytesCount<1)
            {
                if (bytesCount!=0)
                    throw new ArgumentOutOfRangeException("bytesCount",bytesCount,Errors.NEGATIVE);
                goto end;
            }

            int maxBuf=(arrTemp!=null?arrTemp.Length:0)-tempIndex;
            
            if (maxBuf>=bytesCount)
                goto onePass;

            if (maxBuf<1 || (maxBuf<256 && bytesCount/maxBuf>255))
            {
                if (maxBuf<0)
                    throw new ArgumentOutOfRangeException("tempIndex",tempIndex,Errors.SUPERIOR_ARRAY);
                tempIndex=0;
                if (bytesCount>=1<<20)
                    maxBuf=8192;
                else if (bytesCount>=1<<18)
                    maxBuf=4096;
                else if (bytesCount>=1<<16)
                    maxBuf=2048;
                else if (bytesCount<=4096)
                {
                    arrTemp=new byte[bytesCount];
                    goto onePass;
                }
                else
                    maxBuf=1024;
                arrTemp=new byte[maxBuf];
            }

            fixed(byte*p=arrTemp)
            {
                byte*pd=p+tempIndex;
                if (pd==po)
                    throw new ArgumentException(Errors.SER_STREAM_P_EQUAL_ARRAY,"arrTemp");

                int ret=0;
                do
                {
                    General.Copy(po+ret,pd,maxBuf);
                    stream.Write(arrTemp,tempIndex,maxBuf);
                    ret+=maxBuf;
                    if (bytesCount-ret<maxBuf)
                    {
                        if (bytesCount==ret)
                            break;
                        maxBuf=bytesCount-ret;
                    }

                }while(true);
                goto end;
            }

            onePass:

                fixed(byte*p=arrTemp)
                {
                    General.Copy(po,p+tempIndex,bytesCount);
                    stream.Write(arrTemp,tempIndex,bytesCount);
                }

            end:
                if (close)
                    stream.Close();
            return bytesCount;

        }


        #endregion

        #region StreamWriteItems

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, serializando elementos de tamao fijo. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="sof">
        /// Ancho en bytes de cada elemento. La escritura en bytes se calcula en base a itemsCount*sof.
        /// Debe ser mayor a cero.
        ///  </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream (siempre equivale a itemsCount*sof).
        /// </returns>
        public static int StreamWriteItems(Stream stream, bool close, byte[] arrOrig, int sof)
        {
            if (sof<1)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
            return StreamWrite(stream,close,arrOrig,0,(arrOrig.Length/sof)*sof);
        }

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, serializando elementos de tamao fijo. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="arrOrig">
        /// Array origen. Debe tener una capacidad coherente con itemsCount.
        /// </param>
        /// <param name="origIndex">
        /// Index en el array origen donde comenzar la copia (en bytes).
        /// </param>
        /// <param name="itemsCount">
        /// Cantidad de elementos a escribr.
        /// La capacidad de arrOrig debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="sof">
        /// Ancho en bytes de cada elemento. La escritura en bytes se calcula en base a itemsCount*sof.
        /// Debe ser mayor a cero.
        ///  </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream (siempre equivale a itemsCount*sof).
        /// </returns>
        public static int StreamWriteItems(Stream stream, bool close, byte[] arrOrig, int origIndex, int itemsCount, int sof)
        {
            if (sof<1)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
            return StreamWrite(stream,close,arrOrig,origIndex,itemsCount*sof);
        }

        
        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, serializando elementos de tamao fijo. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="po">
        /// Puntero origen. Debe tener una capacidad coherente con itemsCount.
        /// </param>
        /// <param name="itemsCount">
        /// Cantidad de elementos a escribr.
        /// La capacidad de po debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="sof">
        /// Ancho en bytes de cada elemento. La escritura en bytes se calcula en base a itemsCount*sof.
        /// Debe ser mayor a cero.
        ///  </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream (siempre equivale a itemsCount*sof).
        /// </returns>
        public static int StreamWriteItems(Stream stream, bool close, void* po, int itemsCount, int sof)
        {
            if (sof<1)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
            return StreamWrite(stream,close,(byte*)po,itemsCount*sof,null,0);
        }

        /// <summary>
        /// Efecta escritura en un Stream, desde su posicin actual, serializando elementos de tamao fijo. 
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="po">
        /// Puntero origen. Debe tener una capacidad coherente con itemsCount.
        /// </param>
        /// <param name="arrTemp">
        /// Array temporal. Si es null o su capacidad til no es practica para esta funcionalidad, se crear uno.
        /// </param>
        /// <param name="tempIndex">
        /// Index en el array donde comienza el espacio utilizable para copia temporal (en bytes).
        /// </param>
        /// <param name="itemsCount">
        /// Cantidad de elementos a escribr.
        /// La capacidad de po debe ser coherente con este valor.
        /// </param>
        /// <param name="close">
        /// Si true, cierra el stream luego de la escritura.
        /// </param>
        /// <param name="sof">
        /// Ancho en bytes de cada elemento. La escritura en bytes se calcula en base a itemsCount*sof.
        /// Debe ser mayor a cero.
        ///  </param>
        /// <returns>
        /// Cantidad de Bytes escritos en el Stream (siempre equivale a itemsCount*sof).
        /// </returns>
        public static int StreamWriteItems(Stream stream, bool close, void* po, int itemsCount, int sof, byte[] arrTemp, int tempIndex)
        {
            if (sof<1)
                throw new ArgumentOutOfRangeException("sof",sof,Errors.MINOR_ONE);
            return StreamWrite(stream,close,(byte*)po,itemsCount*sof,arrTemp,tempIndex);
        }


        #endregion

    }
}
