using System;
using EnPalermo.Mem;
using EnPalermo.Txt;
using EnPalermo.Lit;

namespace EnPalermo.Uti
{
	/// <summary>
	/// Elementos en que se puede componer un perodo.
	/// </summary>
	public enum PeriodItem
	{
		/// <summary>
		/// Tick (1/10000 Ms.)
		/// </summary>
		Tick,
		/// <summary>
		/// Milisegundo (1/1000 Seg.)
		/// </summary>
		Milisecond,
		/// <summary>
		/// Segundo.
		/// </summary>
		Second,
		/// <summary>
		/// Minuto.
		/// </summary>
		Minute,
		/// <summary>
		/// Hora.
		/// </summary>
		Hour,
		/// <summary>
		/// Da.
		/// </summary>
		Day,
		/// <summary>
		/// Semana.
		/// </summary>
		Week,
	}

	/// <summary>
	/// Funciones relativas a Tiempo.
	/// </summary>
    public static class Time
	{

		/// <summary>
		/// Segundos por Semana.
		/// </summary>
		public const int WeekSeconds=3600*24*7;

		/// <summary>
		/// Segundos por Da.
		/// </summary>
		public const int DaySeconds=3600*24;
		
		/// <summary>
		/// Ticks por Milisegundo.
		/// </summary>
		public const long MilisecondTicks=10000;

		/// <summary>
		/// Ticks por Segundo.
		/// </summary>
		public const long SecondTicks=MilisecondTicks*1000;

		/// <summary>
		/// Ticks por Minuto.
		/// </summary>
		public const long MinuteTicks=SecondTicks*60;

		/// <summary>
		/// Ticks por Hora.
		/// </summary>
		public const long HourTicks=MinuteTicks*60L;

		/// <summary>
		/// Ticks por Da.
		/// </summary>
		public const long DayTicks=HourTicks*24;

		/// <summary>
		/// Ticks por Semana.
		/// </summary>
		public const long WeekTicks=DayTicks*7;

        /// <summary>
        /// 1/1/1601 00:00:00 UTC.
        /// </summary>
        public const long Ticks1601UTC = 0x701CE1722770000;


		/// <summary>
		/// 1/1/1970 00:00:00 UTC.
		/// </summary>
		public const long Ticks1970UTC = 0x89F7FF5F7B58000;

        
        /// <summary>
        /// 1/1/1980 00:00:00 UTC.
        /// </summary>
        public const long Ticks1980UTC = 0x8AAB5B7044C8000;


		/// <summary>
		/// Seteando esta variable en false, se interrumpirn las esperas en la funcin Sleep.
		/// </summary>
		public static bool EnableSleep=true;

		/// <summary>
		/// Retorna true si el momento actual se encuentra dentro del periodo especificado.
		/// </summary>
		/// <param name="fromSec">Hora inicio (en segundos relativos desde las 0hs).</param>
		/// <param name="toSec">Hora limite (en segundos relativos desde las 0hs).</param>
		public static bool IsTime(int fromSec, int toSec)
		{return IsTime(fromSec,toSec,0);}


		/// <summary>
		/// Retorna true si el momento actual se encuentra dentro del periodo especificado.
		/// </summary>
		/// <param name="fromSec">Hora inicio (en segundos relativos desde las 0hs).</param>
		/// <param name="toSec">Hora limite (en segundos relativos desde las 0hs).</param>
		/// <param name="duration">
		/// Duracion estimada de la tarea en segundos. 
		/// El rango de duracin debe estar entre 1 y 
		/// 3/4 del periodo entre desde y hasta.
		/// Si no se encuentra en este rango, es ignorado.
		/// </param>
		/// <returns>
		/// Si fromSec y toSec son iguales, siempre retorna true.
		/// Si duracin es menor a 1 o mayor a 3/4 del periodo entre desde y hasta:
		/// Retorna true si el momento actual se encuentra dentro del periodo especificado.
		/// Si duracin es mayor a 0 y menor o igual a 3/4 del periodo entre desde y hasta:
		/// Retorna true si el momento actual se encuentra dentro del periodo especificado - duration.
		/// </returns>
		public static bool IsTime(int fromSec, int toSec,int duration)
		{
			if (fromSec<0 || fromSec>86399)
			{
				throw new ArgumentOutOfRangeException("fromSec", fromSec,Errors.INVALID_TIME);
			}
			if (toSec<0 || fromSec>86399)
			{
                throw new ArgumentOutOfRangeException("toSec", toSec,Errors.INVALID_TIME);
            }
			if (fromSec==toSec) return true;
			int periodo=(fromSec>toSec?86400:0)+toSec-fromSec;
			if(duration>0 && duration<=(periodo*3)>>2) periodo-=duration;
			int ahora=Convert.ToInt32(DateTime.Now.TimeOfDay.TotalSeconds);
			if (fromSec>ahora) return ahora<fromSec-86400+periodo;
			return ahora<fromSec+periodo;
		}

		/// <summary>
		/// Intenta reconocer una cadena con formato de hora y devuelve el numero de segundos relativos a las 00:00 o -1 en caso de error.
		/// </summary>
		/// <param name="str">
		/// Cadena.
		/// </param>
		/// <returns>Numero de segundos relativos a las 00:00 o -1 en caso de error.</returns>
		public static int HmsToSec(string str)
		{
			int l=str.Length;
			if (l==0) 
                return -1;
            unsafe
            {
                fixed(char* p =str)
                    return HmsToSec(p,l);
            }
            
        }

        /// <summary>
        /// Intenta reconocer una cadena con formato de hora y devuelve el numero de segundos relativos a las 00:00 o -1 en caso de error.
        /// </summary>
        /// <param name="p">
        /// Puntero.
        /// </param>
        /// <param name="l">Largo.</param>
        /// <returns>Numero de segundos relativos a las 00:00 o -1 en caso de error.</returns>
        public unsafe static int HmsToSec(char* p, int l)
        {
            if (l<1)
                return -1;
            int[] res=new int[3]{-1,0,0};
            bool pm=false;
            bool am=false;
            int a=0;
            int rel=-1;
            int ubi=-1;
            char c;
            char ant;
            char sig;
            do
            {
                c=p[a];
                if (c<='9' && c>='0')
                {
                    if (++rel==2) {rel=0;}
                    if (rel==0)
                    {
                        if (++ubi>2) return -1;
                        res[ubi]=c-48;
                    }
                    else
                    {
                        res[ubi]=(res[ubi]*10)+c-48;
                    }
                }
                else
                {
                    rel=-1;
                    if (c=='p' || c=='P' || c=='a' || c=='A')
                    {
                        if(!pm && !am)
                        {
                            ant=a!=0?p[a-1]:'\0';
                            if (ant<'A' || rel!=-1 || ant>122 || (ant>90 && ant<97))
                            {
                                sig=a!=l-1?p[a+1]:'\0';
                                if (sig<'A' || sig=='m' || sig=='M' || sig>122 || (sig>90 && sig<97))
                                {
                                    if (c=='p' || c=='P') pm=true;
                                    else am=true;
                                }
                            }
                        }
                    }
                }
            }while(++a!=l);

            if (res[1]>59 || res[2]>59) return -1;
            switch(res[0])
            {
                case -1:
                    return -1;
                case 0:
                    break;
                case 24:
                    res[0]=0;
                    break;
                case 12:
                    if (am) res[0]=0;
                    break;
                default:
                    if (res[0] >23) return -1;
                    if (res[0] <12 && pm)
                    {
                        res[0]+=12;
                    }
                    break;
            }
            return res[0]*3600+res[1]*60+res[2];

        }

		/// <summary>
		/// Devuelve el lapso absoluto entre el momento actual y el valor de entrada.
		/// </summary>
		/// <param name="t">DateTime.</param>
		/// <returns>TimeSpan (el valor es siempre positivo).</returns>
		public static TimeSpan PeriodTS(DateTime t) 
		{
			long ta=t.Ticks;
			long tn=DateTime.Now.Ticks;
			if (tn>=ta) return new TimeSpan(tn-ta);
			return new TimeSpan(ta-tn);
		}


		/// <summary>
		/// Devuelve el lapso absoluto entre el momento actual y el valor de entrada.
		/// </summary>
		/// <param name="t">DateTime.</param>
		/// <returns>Ticks (el valor es siempre positivo).</returns>
		public static long PeriodTicks(DateTime t) 
		{
			long ta=t.Ticks;
			long tn=DateTime.Now.Ticks;
			if (tn>=ta) return tn-ta;
			return ta-tn;
		}

		/// <summary>
		/// Devuelve el lapso absoluto entre el momento actual y el valor de entrada.
		/// </summary>
		/// <param name="t">DateTime.</param>
		/// <returns>Milisegundos (el valor es siempre positivo).</returns>
		public static long PeriodMS(DateTime t) 
		{
			long ta=t.Ticks;
			long tn=DateTime.Now.Ticks;
			if (tn>=ta) return (tn-ta)/MilisecondTicks;
			return (ta-tn)/MilisecondTicks;
		}

		/// <summary>
		/// Devuelve el lapso absoluto entre el momento actual y el valor de entrada.
		/// </summary>
		/// <param name="t">DateTime.</param>
		/// <returns>Segundos (el valor es siempre positivo).</returns>
		public static long PeriodSec(DateTime t) 
		{
			long ta=t.Ticks;
			long tn=DateTime.Now.Ticks;
			if (tn>=ta) return (tn-ta)/SecondTicks;
			return (ta-tn)/SecondTicks;
		}

		/// <summary>
		/// Devuelve el lapso entre el momento actual y el valor de entrada.
		/// </summary>
		/// <param name="t">DateTime.</param>
		/// <returns>TimeSpan.</returns>
		public static TimeSpan Lapse(DateTime t) 
		{
			return new TimeSpan(DateTime.Now.Ticks-t.Ticks);
		}


		/// <summary>
		/// Devuelve el lapso entre el momento actual y el valor de entrada.
		/// </summary>
		/// <param name="t">DateTime.</param>
		/// <returns>Ticks.</returns>
		public static long LapseTicks(DateTime t) 
		{
			return DateTime.Now.Ticks-t.Ticks;
		}

		/// <summary>
		/// Devuelve el lapso entre el momento actual y el valor de entrada.
		/// </summary>
		/// <param name="t">DateTime.</param>
		/// <returns>Milisegundos.</returns>
		public static long LapseMS(DateTime t) 
		{
			return (DateTime.Now.Ticks-t.Ticks)/MilisecondTicks;
		}

		/// <summary>
		/// Devuelve el lapso entre el momento actual y el valor de entrada.
		/// </summary>
		/// <param name="t">DateTime.</param>
		/// <returns>Segundos.</returns>
		public static long LapseSec(DateTime t) 
		{
			return (DateTime.Now.Ticks-t.Ticks)/SecondTicks;
		}

        /// <summary>
        /// Convierte un periodo en una cadena descriptiva en ingls.
        /// </summary>
        /// <param name="ts">TimeSpan.</param>
        /// <returns>Cadena que describe el periodo especificado.</returns>
        public static string PeriodToStringEN(TimeSpan ts)
        {
            return TicksToStringEN(ts.Ticks,PeriodItem.Tick,PeriodItem.Week,true,true);
        }

        /// <summary>
        /// Convierte un periodo en una cadena descriptiva en ingls de acuerdo al formato especificado.
        /// </summary>
        /// <param name="ts">TimeSpan.</param>
        /// <param name="maxPI">Mximo item de tiempo que ser representado.</param>
        /// <param name="minPI">Mnimo item de tiempo que ser representado.</param>
        /// <param name="startUpper">Comienza con mayuscula.</param>
        /// <param name="endDot">Punto al finalizar.</param>
        /// <returns>Cadena que describe el periodo especificado.</returns>
        public static string PeriodToStringEN(TimeSpan ts,PeriodItem minPI,PeriodItem maxPI,bool startUpper,bool endDot)
        {
            return TicksToStringEN(ts.Ticks,minPI,maxPI,startUpper,endDot);
        }



        /// <summary>
        /// Convierte un periodo en una cadena descriptiva en ingls.
        /// </summary>
        /// <param name="t">DateTime sobre el que se diferenciar con el momento actual.</param>
        /// <returns>Cadena que describe el periodo especificado.</returns>
        public static string PeriodToStringEN(DateTime t)
        {
            return TicksToStringEN(DateTime.Now.Ticks-t.Ticks,PeriodItem.Tick,PeriodItem.Week,true,true);
        }

        /// <summary>
        /// Convierte un periodo en una cadena descriptiva en ingls de acuerdo al formato especificado.
        /// </summary>
        /// <param name="t">DateTime sobre el que se diferenciar con el momento actual.</param>
        /// <param name="maxPI">Mximo item de tiempo que ser representado.</param>
        /// <param name="minPI">Mnimo item de tiempo que ser representado.</param>
        /// <param name="startUpper">Comienza con mayuscula.</param>
        /// <param name="endDot">Punto al finalizar.</param>
        /// <returns>Cadena que describe el periodo especificado.</returns>
        public static string PeriodToStringEN(DateTime t,PeriodItem minPI,PeriodItem maxPI,bool startUpper,bool endDot)
        {
            return TicksToStringEN(DateTime.Now.Ticks-t.Ticks,minPI,maxPI,startUpper,endDot);
        }


        /// <summary>
        /// Convierte un periodo de Ticks en una cadena descriptiva en ingls.
        /// </summary>
        /// <param name="ticks">Ticks.</param>
        /// <returns>Cadena que describe el periodo especificado.</returns>
        public static string TicksToStringEN(long ticks)
        {
            return TicksToStringEN(ticks,PeriodItem.Tick,PeriodItem.Week,true,true);
        }


		/// <summary>
		/// Convierte un periodo en una cadena descriptiva en espaol.
		/// </summary>
		/// <param name="ts">TimeSpan.</param>
		/// <returns>Cadena que describe el periodo especificado.</returns>
		public static string PeriodToStringSP(TimeSpan ts)
		{
			return TicksToStringSP(ts.Ticks,PeriodItem.Tick,PeriodItem.Week,true,true);
		}

		/// <summary>
		/// Convierte un periodo en una cadena descriptiva en espaol de acuerdo al formato especificado.
		/// </summary>
		/// <param name="ts">TimeSpan.</param>
		/// <param name="maxPI">Mximo item de tiempo que ser representado.</param>
		/// <param name="minPI">Mnimo item de tiempo que ser representado.</param>
		/// <param name="startUpper">Comienza con mayuscula.</param>
		/// <param name="endDot">Punto al finalizar.</param>
		/// <returns>Cadena que describe el periodo especificado.</returns>
		public static string PeriodToStringSP(TimeSpan ts,PeriodItem minPI,PeriodItem maxPI,bool startUpper,bool endDot)
		{
			return TicksToStringSP(ts.Ticks,minPI,maxPI,startUpper,endDot);
		}



		/// <summary>
		/// Convierte un periodo en una cadena descriptiva en espaol.
		/// </summary>
		/// <param name="t">DateTime sobre el que se diferenciar con el momento actual.</param>
		/// <returns>Cadena que describe el periodo especificado.</returns>
		public static string PeriodToStringSP(DateTime t)
		{
			return TicksToStringSP(DateTime.Now.Ticks-t.Ticks,PeriodItem.Tick,PeriodItem.Week,true,true);
		}

		/// <summary>
		/// Convierte un periodo en una cadena descriptiva en espaol de acuerdo al formato especificado.
		/// </summary>
		/// <param name="t">DateTime sobre el que se diferenciar con el momento actual.</param>
		/// <param name="maxPI">Mximo item de tiempo que ser representado.</param>
		/// <param name="minPI">Mnimo item de tiempo que ser representado.</param>
		/// <param name="startUpper">Comienza con mayuscula.</param>
		/// <param name="endDot">Punto al finalizar.</param>
		/// <returns>Cadena que describe el periodo especificado.</returns>
		public static string PeriodToStringSP(DateTime t,PeriodItem minPI,PeriodItem maxPI,bool startUpper,bool endDot)
		{
			return TicksToStringSP(DateTime.Now.Ticks-t.Ticks,minPI,maxPI,startUpper,endDot);
		}


		/// <summary>
		/// Convierte un periodo de Ticks en una cadena descriptiva en espaol.
		/// </summary>
		/// <param name="ticks">Ticks.</param>
		/// <returns>Cadena que describe el periodo especificado.</returns>
		public static string TicksToStringSP(long ticks)
		{
			return TicksToStringSP(ticks,PeriodItem.Tick,PeriodItem.Week,true,true);
		}

		/// <summary>
		/// Convierte un periodo de Ticks en una cadena descriptiva en espaol de acuerdo al formato especificado.
		/// </summary>
		/// <param name="ticks">Ticks.</param>
		/// <param name="maxPI">Mximo item de tiempo que ser representado.</param>
		/// <param name="minPI">Mnimo item de tiempo que ser representado.</param>
		/// <param name="startUpper">Comienza con mayuscula.</param>
		/// <param name="endDot">Punto al finalizar.</param>
		/// <returns>Cadena que describe el periodo especificado.</returns>
		public static string TicksToStringSP(long ticks,PeriodItem minPI,PeriodItem maxPI,bool startUpper,bool endDot)
		{
			if (minPI>maxPI) 
                throw new
                    ArgumentOutOfRangeException("minPI,maxPI", Errors.MIN_MAJOR_MAX);
			if (ticks<0)
			{
				if(ticks==Int64.MinValue) ticks=Int64.MaxValue;
				else ticks=-ticks;
			}
			unsafe
			{
				int inicial=-1;
				int datos=0;
				PeriodItem einicial=PeriodItem.Week;
				long* items=stackalloc long[ticks!=0?7:0];
				switch(maxPI)
				{
					case PeriodItem.Week:
						if(ticks>=WeekTicks)
						{
							*items=ticks/WeekTicks;
							ticks%=WeekTicks;
							inicial=0;
							datos=1;
						}
						if (ticks==0 || minPI==PeriodItem.Week) break;
						goto case PeriodItem.Day;
					case PeriodItem.Day:
						if(ticks>=DayTicks)
						{
							items[1]=ticks/DayTicks;
							ticks%=DayTicks;
							if (inicial==-1) {inicial=1;einicial=PeriodItem.Day;}
							datos++;
						}
						if (ticks==0 || minPI==PeriodItem.Day) break;
						goto case PeriodItem.Hour;
						
					case PeriodItem.Hour:
						if(ticks>=HourTicks)
						{
							items[2]=ticks/HourTicks;
							ticks%=HourTicks;
							if (inicial==-1) {inicial=2;einicial=PeriodItem.Hour;}
							datos++;
						}
						if (ticks==0 || minPI==PeriodItem.Hour) break;
						goto case PeriodItem.Minute;

					case PeriodItem.Minute:
						if(ticks>=MinuteTicks)
						{
							items[3]=ticks/MinuteTicks;
							ticks%=MinuteTicks;
							if (inicial==-1) {inicial=3;einicial=PeriodItem.Minute;}
							datos++;
						}
						if (ticks==0 || minPI==PeriodItem.Minute) break;
						goto case PeriodItem.Second;

					case PeriodItem.Second:
						if(ticks>=SecondTicks)
						{
							items[4]=ticks/SecondTicks;
							ticks%=SecondTicks;
							if (inicial==-1) {inicial=4;einicial=PeriodItem.Second;}
							datos++;
						}
						if (ticks==0 || minPI==PeriodItem.Second) break;
						goto case PeriodItem.Milisecond;

					case PeriodItem.Milisecond:
						if(ticks>=MilisecondTicks)
						{
							items[5]=ticks/MilisecondTicks;
							ticks%=MilisecondTicks;
							if (inicial==-1) {inicial=5;einicial=PeriodItem.Milisecond;}
							datos++;
						}
						if (ticks==0 || minPI==PeriodItem.Milisecond) break;
						goto case PeriodItem.Tick;

					case PeriodItem.Tick:
						if(ticks!=0)
						{
							items[6]=ticks;
							if (inicial==-1) {inicial=6;einicial=PeriodItem.Tick;}
							datos++;
						}
						break;
				}
				string ret;
				switch(datos)
				{
					case 0:
						ret=Mutables.Create(0,15);
						fixed(char*pret=ret)
						{
                            fillSP(pret,ref datos,0,minPI,'\0',endDot,startUpper);
							((int*)pret)[-1]=datos;
						}
						return ret;
					case 1:
						ret=Mutables.Create(0,29);
						datos=0;
						fixed(char*pret=ret)
						{
							fillSP(pret,ref datos,items[inicial],einicial,'\0',endDot,startUpper);
							((int*)pret)[-1]=datos;
						}
						return ret;
					case 2:
						ret=Mutables.Create(0,57);
						datos=0;
						fixed(char*pret=ret)
						{
							fillSP(pret,ref datos,items[inicial],einicial,'y',false,startUpper);
							fillSP(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
							((int*)pret)[-1]=datos;
						}
						return ret;
					case 3:
						ret=Mutables.Create(0,81);
						datos=0;
						fixed(char*pret=ret)
						{
							fillSP(pret,ref datos,items[inicial],einicial,',',false,startUpper);
							fillSP(pret,ref datos,items[++inicial],--einicial,'y',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
							((int*)pret)[-1]=datos;
						}
						return ret;
					case 4:
						ret=Mutables.Create(0,101);
						datos=0;
						fixed(char*pret=ret)
						{
							fillSP(pret,ref datos,items[inicial],einicial,',',false,startUpper);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,'y',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
							((int*)pret)[-1]=datos;
						}
						return ret;
					case 5:
						ret=Mutables.Create(0,119);
						datos=0;
						fixed(char*pret=ret)
						{
							fillSP(pret,ref datos,items[inicial],einicial,',',false,startUpper);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,'y',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
							((int*)pret)[-1]=datos;
						}
						return ret;
					case 6:
						ret=Mutables.Create(0,133);
						datos=0;
						fixed(char*pret=ret)
						{
							fillSP(pret,ref datos,items[inicial],einicial,',',false,startUpper);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,'y',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
							((int*)pret)[-1]=datos;
						}
						return ret;
					default:
						ret=Mutables.Create(0,150);
						datos=0;
						fixed(char*pret=ret)
						{
							fillSP(pret,ref datos,items[inicial],einicial,',',false,startUpper);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,',',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,'y',false,false);
							fillSP(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
							((int*)pret)[-1]=datos;
						}
						return ret;
				}
				
			}
		}

        /// <summary>
        /// Convierte un periodo de Ticks en una cadena descriptiva en ingls de acuerdo al formato especificado.
        /// </summary>
        /// <param name="ticks">Ticks.</param>
        /// <param name="maxPI">Mximo item de tiempo que ser representado.</param>
        /// <param name="minPI">Mnimo item de tiempo que ser representado.</param>
        /// <param name="startUpper">Comienza con mayuscula.</param>
        /// <param name="endDot">Punto al finalizar.</param>
        /// <returns>Cadena que describe el periodo especificado.</returns>
        public static string TicksToStringEN(long ticks,PeriodItem minPI,PeriodItem maxPI,bool startUpper,bool endDot)
        {
            if (minPI>maxPI) 
                throw new
                    ArgumentOutOfRangeException("minPI,maxPI", Errors.MIN_MAJOR_MAX);
            if (ticks<0)
            {
                if(ticks==Int64.MinValue) ticks=Int64.MaxValue;
                else ticks=-ticks;
            }
            unsafe
            {
                int inicial=-1;
                int datos=0;
                PeriodItem einicial=PeriodItem.Week;
                long* items=stackalloc long[ticks!=0?7:0];
                switch(maxPI)
                {
                    case PeriodItem.Week:
                        if(ticks>=WeekTicks)
                        {
                            *items=ticks/WeekTicks;
                            ticks%=WeekTicks;
                            inicial=0;
                            datos=1;
                        }
                        if (ticks==0 || minPI==PeriodItem.Week) break;
                        goto case PeriodItem.Day;
                    case PeriodItem.Day:
                        if(ticks>=DayTicks)
                        {
                            items[1]=ticks/DayTicks;
                            ticks%=DayTicks;
                            if (inicial==-1) {inicial=1;einicial=PeriodItem.Day;}
                            datos++;
                        }
                        if (ticks==0 || minPI==PeriodItem.Day) break;
                        goto case PeriodItem.Hour;
						
                    case PeriodItem.Hour:
                        if(ticks>=HourTicks)
                        {
                            items[2]=ticks/HourTicks;
                            ticks%=HourTicks;
                            if (inicial==-1) {inicial=2;einicial=PeriodItem.Hour;}
                            datos++;
                        }
                        if (ticks==0 || minPI==PeriodItem.Hour) break;
                        goto case PeriodItem.Minute;

                    case PeriodItem.Minute:
                        if(ticks>=MinuteTicks)
                        {
                            items[3]=ticks/MinuteTicks;
                            ticks%=MinuteTicks;
                            if (inicial==-1) {inicial=3;einicial=PeriodItem.Minute;}
                            datos++;
                        }
                        if (ticks==0 || minPI==PeriodItem.Minute) break;
                        goto case PeriodItem.Second;

                    case PeriodItem.Second:
                        if(ticks>=SecondTicks)
                        {
                            items[4]=ticks/SecondTicks;
                            ticks%=SecondTicks;
                            if (inicial==-1) {inicial=4;einicial=PeriodItem.Second;}
                            datos++;
                        }
                        if (ticks==0 || minPI==PeriodItem.Second) break;
                        goto case PeriodItem.Milisecond;

                    case PeriodItem.Milisecond:
                        if(ticks>=MilisecondTicks)
                        {
                            items[5]=ticks/MilisecondTicks;
                            ticks%=MilisecondTicks;
                            if (inicial==-1) {inicial=5;einicial=PeriodItem.Milisecond;}
                            datos++;
                        }
                        if (ticks==0 || minPI==PeriodItem.Milisecond) break;
                        goto case PeriodItem.Tick;

                    case PeriodItem.Tick:
                        if(ticks!=0)
                        {
                            items[6]=ticks;
                            if (inicial==-1) {inicial=6;einicial=PeriodItem.Tick;}
                            datos++;
                        }
                        break;
                }
                string ret;
                switch(datos)
                {
                    case 0:
                        ret=Mutables.Create(0,15);
                        fixed(char*pret=ret)
                        {
                            fillEN(pret,ref datos,0,minPI,'\0',endDot,startUpper);
                            ((int*)pret)[-1]=datos;
                        }
                        return ret;
                    case 1:
                        ret=Mutables.Create(0,29);
                        datos=0;
                        fixed(char*pret=ret)
                        {
                            fillEN(pret,ref datos,items[inicial],einicial,'\0',endDot,startUpper);
                            ((int*)pret)[-1]=datos;
                        }
                        return ret;
                    case 2:
                        ret=Mutables.Create(0,59);
                        datos=0;
                        fixed(char*pret=ret)
                        {
                            fillEN(pret,ref datos,items[inicial],einicial,'y',false,startUpper);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
                            ((int*)pret)[-1]=datos;
                        }
                        return ret;
                    case 3:
                        ret=Mutables.Create(0,83);
                        datos=0;
                        fixed(char*pret=ret)
                        {
                            fillEN(pret,ref datos,items[inicial],einicial,',',false,startUpper);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'y',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
                            ((int*)pret)[-1]=datos;
                        }
                        return ret;
                    case 4:
                        ret=Mutables.Create(0,103);
                        datos=0;
                        fixed(char*pret=ret)
                        {
                            fillEN(pret,ref datos,items[inicial],einicial,',',false,startUpper);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'y',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
                            ((int*)pret)[-1]=datos;
                        }
                        return ret;
                    case 5:
                        ret=Mutables.Create(0,121);
                        datos=0;
                        fixed(char*pret=ret)
                        {
                            fillEN(pret,ref datos,items[inicial],einicial,',',false,startUpper);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'y',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
                            ((int*)pret)[-1]=datos;
                        }
                        return ret;
                    case 6:
                        ret=Mutables.Create(0,135);
                        datos=0;
                        fixed(char*pret=ret)
                        {
                            fillEN(pret,ref datos,items[inicial],einicial,',',false,startUpper);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'y',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
                            ((int*)pret)[-1]=datos;
                        }
                        return ret;
                    default:
                        ret=Mutables.Create(0,152);
                        datos=0;
                        fixed(char*pret=ret)
                        {
                            fillEN(pret,ref datos,items[inicial],einicial,',',false,startUpper);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,',',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'y',false,false);
                            fillEN(pret,ref datos,items[++inicial],--einicial,'\0',endDot,false);
                            ((int*)pret)[-1]=datos;
                        }
                        return ret;
                }
				
            }
        }

		private unsafe static void fillSP(char*p, ref int i,long data, PeriodItem item, char sep,bool endDot,bool upper)
		{

			if(data<10)
			{
				p[i++]=(char)(data+48);
			}
			else 
			{
                string dat = data.ToString(Texts.CulInfoInvariant);
				int l=dat.Length;
				fixed (char*pdat=dat) General.Copy(pdat,p+i,l);
                i+=l;
			}
			p[i++]=' ';

			switch(item)
			{
				case PeriodItem.Tick:
					if (upper) p[i++]='T';
					else p[i++]='t';
					p[i++]='i';p[i++]='c';p[i++]='k';
					break;
				case PeriodItem.Milisecond:
					if (upper) p[i++]='M';
					else p[i++]='m';
					p[i++]='i';p[i++]='l';p[i++]='i';p[i++]='s';p[i++]='e';p[i++]='g';p[i++]='u';p[i++]='n';p[i++]='d';p[i++]='o';
					break;
				case PeriodItem.Second:
					if (upper) p[i++]='S';
					else p[i++]='s';
					p[i++]='e';p[i++]='g';p[i++]='u';p[i++]='n';p[i++]='d';p[i++]='o';
					break;
				case PeriodItem.Minute:
					if (upper) p[i++]='M';
					else p[i++]='m';
					p[i++]='i';p[i++]='n';p[i++]='u';p[i++]='t';p[i++]='o';
					break;
				case PeriodItem.Hour:
					if (upper) p[i++]='H';
					else p[i++]='h';
					p[i++]='o';p[i++]='r';p[i++]='a';
					break;
				case PeriodItem.Day:
					if (upper) p[i++]='D';
					else p[i++]='d';
					p[i++]='';p[i++]='a';
					break;
				case PeriodItem.Week:
					if (upper) p[i++]='S';
					else p[i++]='s';
					p[i++]='e';p[i++]='m';p[i++]='a';p[i++]='n';p[i++]='a';
					break;
			}

			if (data!=1) p[i++]='s';
			if (endDot) p[i++]='.';
			else
			{
				switch(sep)
				{
					case ',':
						p[i++]=',';p[i++]=' ';
						break;
					case 'y':
						p[i++]=' ';p[i++]='y';p[i++]=' ';
						break;
				}
			}
		}


        private unsafe static void fillEN(char*p, ref int i,long data, PeriodItem item, char sep,bool endDot,bool upper)
        {

            if(data<10)
            {
                p[i++]=(char)(data+48);
            }
            else 
            {
                string dat = data.ToString(Texts.CulInfoInvariant);
                int l=dat.Length;
                fixed (char*pdat=dat) General.Copy(pdat,p+i,l);
                i+=l;
            }
            p[i++]=' ';

            switch(item)
            {
                case PeriodItem.Tick:
                    if (upper) p[i++]='T';
                    else p[i++]='t';
                    p[i++]='i';p[i++]='c';p[i++]='k';
                    break;
                case PeriodItem.Milisecond:
                    if (upper) p[i++]='M';
                    else p[i++]='m';
                    p[i++]='i';p[i++]='l';p[i++]='i';p[i++]='s';p[i++]='e';p[i++]='c';p[i++]='o';p[i++]='n';p[i++]='d';
                    break;
                case PeriodItem.Second:
                    if (upper) p[i++]='S';
                    else p[i++]='s';
                    p[i++]='e';p[i++]='c';p[i++]='o';p[i++]='n';p[i++]='d';
                    break;
                case PeriodItem.Minute:
                    if (upper) p[i++]='M';
                    else p[i++]='m';
                    p[i++]='i';p[i++]='n';p[i++]='u';p[i++]='t';p[i++]='e';
                    break;
                case PeriodItem.Hour:
                    if (upper) p[i++]='H';
                    else p[i++]='h';
                    p[i++]='o';p[i++]='u';p[i++]='r';
                    break;
                case PeriodItem.Day:
                    if (upper) p[i++]='D';
                    else p[i++]='d';
                    p[i++]='a';p[i++]='y';
                    break;
                case PeriodItem.Week:
                    if (upper) p[i++]='W';
                    else p[i++]='w';
                    p[i++]='e';p[i++]='e';p[i++]='k';
                    break;
            }

            if (data!=1) p[i++]='s';
            if (endDot) p[i++]='.';
            else
            {
                switch(sep)
                {
                    case ',':
                        p[i++]=',';p[i++]=' ';
                        break;
                    case 'y':
                        p[i++]=' ';p[i++]='a';p[i++]='n';p[i++]='d';p[i++]=' ';
                        break;
                }
            }
        }



        /// <summary>
        /// Obtiene los Ticks desde el inicio del sistema.
        /// </summary>
        public static uint TickCount
        {
            get
            {
                return (uint)Environment.TickCount;
            }
        }

        /// <summary>
        /// Obtiene los Ticks desde el inicio del sistema.
        /// Entero con signo filtrado.
        /// </summary>
        public static int TickCountInt
        {
            get
            {
                return Environment.TickCount & Int32.MaxValue;
            }
        }

        /// <summary>
        /// Detiene la actual tarea hasta el momento final a no ser que EnableSleep sea false (verifica cada 20 ms).
        /// </summary>
        /// <param name="tickEndUtc">Tick Final (basado en UTC).</param>
        /// <returns>Estado de la variable EnableSleep.</returns>
        public static bool Sleep(long tickEndUtc)
        {
            long dif;
            while (EnableSleep)
            {
                System.Threading.Thread.Sleep(0);
                dif = tickEndUtc - DateTime.UtcNow.Ticks;
                if (dif < 200000L)
                {
                    if (dif < MilisecondTicks) return EnableSleep;
                    System.Threading.Thread.Sleep((int)(dif / MilisecondTicks));
                    return EnableSleep;
                }
                System.Threading.Thread.Sleep(20);
            }
            return false;
        }

        /// <summary>
        /// Detiene la actual tarea hasta el momento final a no ser que EnableSleep sea false.
        /// </summary>
        /// <param name="tickEndUtc">Tick Final (basado en UTC).</param>
        /// <param name="testMS">Milisegundos de espera entre muestras.</param>
        /// <returns>Estado de la variable EnableSleep.</returns>
        public static bool Sleep(long tickEndUtc,int testMS)
        {
            long dif;
            long testTicks = testMS * MilisecondTicks;
            while (EnableSleep)
            {
                System.Threading.Thread.Sleep(0);
                dif = tickEndUtc - DateTime.UtcNow.Ticks;
                if (dif < testTicks)
                {
                    if (dif < MilisecondTicks) return EnableSleep;
                    System.Threading.Thread.Sleep((int)(dif / MilisecondTicks));
                    return EnableSleep;
                }
                System.Threading.Thread.Sleep(testMS);
            }
            return false;
        }

		/// <summary>
        /// Detiene la actual tarea hasta el momento final a no ser que EnableSleep sea false (verifica cada 20 ms).
		/// </summary>
		/// <param name="dt">DateTime Final.</param>
        /// <returns>Estado de la variable EnableSleep.</returns>
		public static bool Sleep(DateTime dt)
		{
            return Sleep(dt.ToUniversalTime().Ticks);
		}

		/// <summary>
        /// Detiene la actual tarea hasta el momento final a no ser que EnableSleep sea false.
		/// </summary>
		/// <param name="dt">DateTime Final.</param>
        /// <param name="testMS">Milisegundos de espera entre muestras.</param>
        /// <returns>Estado de la variable EnableSleep.</returns>
		public static bool Sleep(DateTime dt, int testMS)
		{
            return Sleep(dt.ToUniversalTime().Ticks,testMS);
		}

		/// <summary>
        /// Detiene la actual tarea hasta el momento final a no ser que EnableSleep sea false.
		/// </summary>
		/// <param name="waitMS">MiliSegundos de Espera.</param>
        /// <param name="testMS">Milisegundos de espera entre muestras.</param>
        /// <returns>Estado de la variable EnableSleep.</returns>
		public static bool Sleep(int waitMS, int testMS)
		{
            return Sleep(DateTime.UtcNow.Ticks+(waitMS*MilisecondTicks), testMS);
		}

		/// <summary>
        /// Detiene la actual tarea hasta el momento final a no ser que EnableSleep sea false (verifica cada 20 ms).
		/// </summary>
		/// <param name="waitMS">MiliSegundos de Espera.</param>
        /// <returns>Estado de la variable EnableSleep.</returns>
		public static bool Sleep(int waitMS)
		{
            return Sleep(DateTime.UtcNow.Ticks + (waitMS * MilisecondTicks));
		}

		/// <summary>
        /// Detiene la actual tarea hasta el momento final a no ser que EnableSleep sea false.
		/// </summary>
		/// <param name="ts">TimeSpan.</param>
        /// <param name="testMS">Milisegundos de espera entre muestras.</param>
        /// <returns>Estado de la variable EnableSleep.</returns>
		public static bool Sleep(TimeSpan ts, int testMS)
		{
            return Sleep(DateTime.UtcNow.Ticks + ts.Ticks, testMS);
		}

		/// <summary>
		/// Detiene la actual tarea hasta el momento final a no ser que EnableSleep sea false (verifica cada 20 ms).
		/// </summary>
		/// <param name="ts">TimeSpan.</param>
        /// <returns>Estado de la variable EnableSleep.</returns>
		public static bool Sleep(TimeSpan ts)
		{
            return Sleep(DateTime.UtcNow.Ticks + ts.Ticks);
		}

	}
}
