using System;
using EnPalermo.Mem;
using EnPalermo.Uti;
using EnPalermo.Lit;
using EnPalermo.Arr;
using EnPalermo.Txt;

namespace EnPalermo.Col
{
    /// <summary>
    /// Coleccin de tipos de valor basada en array.
    /// A diferencia de las colecciones estndar, permite el acceso directo al array subyacente y es de mejor
    /// rendimiento dado que no usa boxing/unboxing ni es afectado por las limitaciones relativas al uso de
    /// genricos, como la imposibilidad de usar cdigo inseguro.
    /// Nota: Esta coleccin no es segura en tamaos equivalentes o mayores a (Int32.MaxValue/(2*sizeof(tipo)):
    /// Esto es: 1073741823 bytes, 536870911 shorts o chars, 268435455 ints o 134217727 longs.
    /// </summary>
    public unsafe sealed class Bytes
    {

        #region Campos

#pragma warning disable 429
        /// <summary>
        /// Capacidad por defecto para esta coleccin.
        /// </summary>
        public const int DefaultCapacity=((16 / sizeof(byte)) !=8 && (16 / sizeof(byte)) !=16) ? (4) : (16 / sizeof(byte));

        /// <summary>
        /// Sumatoria por defecto para algorritmo de aumento de capacidad.
        /// </summary>
        private const int sumCap=DefaultCapacity>=8?4:2;
#pragma warning restore 429

        /// <summary>
        /// Largo actual del array.
        /// </summary>
        private int _size;

        /// <summary>
        /// Capacidad Actual. Si es cero el array subyacente podr ser null.
        /// </summary>
        private int _cap;


        /// <summary>
        /// Flags para almacenar informacin variada acerca del objeto.
        /// Definidas: 
        /// - Bytes  0 al 7: Tolerancia para Compact (SB). De 0 a 255.
        /// - Byte   8: AppendExact.
        /// - Byte   9: Unix definido (Bytes, Bytes y SB).
        /// - Byte  10: Unix (Bytes, Bytes y SB).
        /// - Bytes 11 y 12: Tabla MayMin (Bytes y SB).
        /// - Bytes 12, 13, 14 y 15: Cantidad mxima de bytes por byte segn cdigo de pgina (Bytes).
        /// - Bytes 16 al 31: Cdigo de pgina (Bytes).
        /// </summary>
        private uint _flags;
       
        /// <summary>
        /// Array subyacente.
        /// </summary>
        private byte[] _items;

        #endregion

        #region Constructores

        /// <summary>
        /// Inicializa la coleccin con un array nulo.
        /// </summary>
        public Bytes()
        {}

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada y largo 0.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        public Bytes(int cap)
        {
            if (cap!=0)
            {
                this._items=new byte[cap];
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin con un array nulo, permitiendo especificar la propiedad AppendExact.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        public Bytes(bool appendExact)
        {
            if (appendExact)
                this._flags=1U<<8;
        }

        /// <summary>
        /// Inicializa la coleccin permitiendo especificar la propiedad AppendExact.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="cap">Capacidad inicial.</param>
        public Bytes(bool appendExact, int cap):this(cap)
        {
            if (appendExact)
                this._flags=1U<<8;
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="count">Cantidad inicial de elementos.</param>
        public Bytes(int cap, int count)
        {
            if (count<0 || count>cap)
                throw new ArgumentOutOfRangeException("cap,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (cap!=0)
            {
                this._items=new byte[cap];
                this._size=count;
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="countCap">La cantidad inicial de elementos ser igual a la capacidad.</param>
        public Bytes(int cap, bool countCap)
        {
            if (cap!=0)
            {
                this._items=new byte[cap];
                this._cap=cap;
                if (countCap)
                    this._size=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece a la capacidad del array.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        public Bytes(byte[] arr, bool asRef):this(arr,asRef,arr==null?0:arr.Length){}

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece de acuerdo a iLength.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        /// <param name="iLength">
        /// Largo inicial (no puede ser negativo ni superar el largo del array).
        /// </param>
        public Bytes(byte[] arr, bool asRef, int iLength)
        {
            if (iLength<1)
            {
                if (iLength==0)
                {
                    if (asRef && arr!=null)
                    {
                        this._items=arr;
                        this._cap=arr.Length;
                    }
                    return;
                }
            }
            else if (arr!=null)
            {
                if (asRef)
                {
                    this._cap=arr.Length;
                    if (iLength<=this._cap)
                    {
                        this._items=arr;
                        this._size=iLength;
                        return;
                    }
                }
                else
                {
                    this._cap=Calc.MultMajor(iLength+sumCap,DefaultCapacity,false);
                    this._size=iLength;
                    this._items=ArrGeneral.Clone(arr,0,iLength,0,this._cap);
                    return;
                }
                
            }
            throw new ArgumentOutOfRangeException("iLength", iLength, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array (puede ser null).</param>
        public Bytes(byte[] arr):this(arr,0,arr==null?0:arr.Length,sumCap){}


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, count deber ser 0.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public Bytes(byte[] arr, int count):this(arr,0,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public Bytes(byte[] arr, int arrIndex, int count):this(arr,arrIndex,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="extraCap">Capacidad extra para agregar al array subyacente.</param>
        public Bytes(byte[] arr, int arrIndex, int count, int extraCap)
        {
            if (count<1)
            {
                if (count==0 && arrIndex==0)
                {
                    if (extraCap>0)
                    {
                        this._cap=Calc.AddIfOdd(extraCap);
                        this._items=new byte[this._cap];
                    }
                    return;
                }
            }
            else if (arrIndex>-1 && arrIndex+count<=arr.Length)
            {
                this._items=new byte[this._cap=Calc.AddIfOdd(extraCap+count)];
                fixed(byte*po=arr,pd=this._items)
                    General.Copy(po+arrIndex,pd,count);
                this._size=count;
                return;
            }
            throw new ArgumentOutOfRangeException("arrIndex,count,extraCap", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }

        #endregion

        #region Asistentes

        /// <summary>
        /// Aplica suma sobre la mitad de DefaultCapacity y luego otra suma de redondeo a DefaultCapacity.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int round(int minItems)
        {
            return Calc.MultMajor(minItems+sumCap,DefaultCapacity,true);
        }

        /// <summary>
        /// Genera una nueva capacidad mnima aplicando algorritmo y redondeando.
        /// </summary>
        /// <param name="sum">Elementos a agregar.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int newMinCap(int sum)
        {
            if (this.AppendExact)
                return round(this._size+sum);

            if (sum==1)
            {
                if (this._size==0)
                    return sumCap;
                if (this._size<=(DefaultCapacity*(1<<10)))
                    return round(this._size<<1);
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>1));
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>2));
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>3));
                return round(this._size+(this._size>>4));
            }
            if (this._size==0)
            {
                if (sum<=DefaultCapacity)
                    return round(sum+DefaultCapacity);
                if (sum<=(DefaultCapacity*(1<<5)))
                    return round(sum<<1);
                if (sum<=(DefaultCapacity*(1<<10)))
                    return round(sum+(sum>>1));
                if (sum<=(DefaultCapacity*(1<<15)))
                    return round(sum+(sum>>2));
                if (sum<=(DefaultCapacity*(1<<18)))
                    return round(sum+(sum>>3));
                return round(sum+(sum>>4));
            }
            if (sum<=DefaultCapacity || this._size>sum)
            {
                if (this._size<=(DefaultCapacity*(1<<10)))
                {
                    if (this._size>sum)
                        return round(this._size<<1);
                    return round(this._size+(this._size>>1)+sum);
                }
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>2)+sum);
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>3)+sum);
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>4)+sum);
                return round(this._size+(this._size>>5)+sum);
            }
            if (this._size<=(DefaultCapacity*(1<<10)))
                return round(this._size+(this._size>>3)+sum);
            if (this._size<=(DefaultCapacity*(1<<15)))
                return round(this._size+(this._size>>4)+sum);
            if (this._size<=(DefaultCapacity*(1<<18)))
                return round(this._size+(this._size>>5)+sum);
            return round(this._size+sum);
        }

        /// <summary>
        /// Genera un nuevo array en base al tamao mnimo especificado,
        /// sumando la mitad de DefaultCapacity y redondeando.
        /// Tambin actualiza capacidad.
        /// A diferencia de newArraySumItems no compara en algorritmo 
        /// el crecimiento sobre el array actual.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Nuevo array.</returns>
        private byte[] newArrayMin(int minItems)
        {
            return new byte[this._cap=round(minItems)];
        }


        /// <summary>
        /// Genera un nuevo array en base al crecimiento requerido y aplicando algorritmo.
        /// Tambin actualiza capacidad.
        /// </summary>
        /// <param name="sumItems">Crecimiento requerido.</param>
        private byte[] newArraySum(int sumItems)
        {
            return new byte[this._cap=newMinCap(sumItems)];
        }

        /// <summary>
        /// Actualiza la capacidad para aumento.
        /// </summary>
        /// <param name="items">Nueva cantidad.</param>
        private void setCapacity(int items)
        {
            byte[] array2=new byte[items];
            if (this._size !=0)
            {
                fixed (byte* po=this._items, pd=array2)
                    General.Copy(po, pd, this._size);
            }
            this._items=array2;
            this._cap=items;
        }

        #endregion

        #region Propiedades

        /// <summary>
        /// Si true, no usar algoritmo de aumento para la coleccin, aunque igual efectuar redondeo.
        /// Por lo tanto, si este parmetro esta activo se redimensionar
        /// la coleccion cada vez que se agreguen nuevos elementos.
        /// </summary>
        public bool AppendExact
        {
            get
            {
                return (this._flags&(1U<<8))!=0;
            }
            set
            {
                if (value)
                    this._flags|=1U<<8;
                else
                    this._flags&=0xFFFFFFFF^(1U<<8);
            }
        }

        /// <summary>
        /// Obtiene el array que actualmente esta siendo usado por la coleccin (puede ser null) y,
        /// por lo tanto, suceptible de ser modificado o reemplazado por la coleccin.
        /// Al establecer, cambia la instancia del array por el especificado en parametro y 
        /// setea la cantidad de elementos en el largo del nuevo array (si es nulo ser 0).
        /// </summary>
        public byte[] Array
        {
            get
            {
                return this._items;
            }
            set
            {
                this._cap=value==null?0:value.Length;
                this._size=this._cap;
                this._items=value;
            }
        }

        /// <summary>
        /// Obtiene un array con los elementos de la coleccin.
        /// Esta copia puede ser el mismo objeto subyacente si la cantidad de elementos coincide con el largo de array
        /// o un array vacio si la capacidad es cero.
        /// </summary>
        public byte[] ArrayCompact
        {
            get
            {
                if (this._cap==0)
                    return ArrGeneral.ArrayEmptyByte;
                if (this._size==this._cap)
                    return this._items;
                return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
            }
        }

        /// <summary>
        /// Obtiene una copia compacta al largo actual de la coleccin.
        /// Al establecer, copia los elementos de la instancia actual al array subyacente o, en caso que el
        /// array de entrada sea de capacidad mayor, genera una nueva instancia con la copia de los elementos.
        /// Setea la cantidad de elementos en el largo del nuevo array.
        /// </summary>
        public byte[] ArrayCopy
        {
            get
            {
                if (this._size!=0)
                    return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
                return ArrGeneral.ArrayEmptyByte;
            }
            set
            {
                this._size=value==null?0:value.Length;
                if (this._size !=0)
                {
                    if (this._cap < this._size)
                        this._items=newArrayMin(this._size+sumCap);
                    fixed (byte* po=value, pd=this._items)
                        General.Copy(po, pd, this._size);
                }
            }
        }



        /// <summary>
        /// Obtiene o establece, solo para aumentar, la capacidad del Array.
        /// En caso de establecer una capacidad menor, se ignora. Si se establece capacidad mayor, el valor ser
        /// redondeado.
        /// </summary>
        public int Capacity
        {
            get
            {
                return this._cap;
            }
            set
            {
                if (value > this._cap)
                    setCapacity(round(value));
            }
        }


        /// <summary>
        /// Obtiene o establece la cantidad de elementos en la coleccin.
        /// Se permite el establecimiento de un largo mayor al actual siempre y cuanto este no exceda la capacidad
        /// del array subyacente (el contenido de los elementos excedentes no ser modificado).
        /// Por lo tanto, si se especifica una cantidad negativa o mayor a la capacidad actual,
        /// se genera una excepcin.
        /// En todo caso, para modificar el largo con mayor flexibilidad, usar el mtodo SetLength.
        /// </summary>
        public int Length
        {
            get
            {
                return this._size;
            }
            set
            {
                if (value > this._cap || value < 0)
                    throw new ArgumentOutOfRangeException("value", value, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                this._size=value;
            }
        }


        /// <summary>
        /// Obtiene o establece el elemento especificado en index.
        /// Para acceder a los elementos frecuentemente, es conveniente acceder al array directamente.
        /// Si el index se encuentra fuera de rango, genera excepcin.
        /// </summary>
        public byte this[int index]
        {
            get
            {
                if (this._size <=index)
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
                return this._items[index];
            }
            set
            {
                if (index < this._size)
                    this._items[index]=value;
                else 
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
            }
        }


        #endregion

        #region Metodos Administrativos

        /// <summary>
        /// Agrega un nuevo elemento al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        public void Append(byte item)
        {
            this.EnsureAppend();
            this._items[this._size++]=item;
        }

        /// <summary>
        /// Agrega elementos al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        /// <param name="count">Cantidad de repeticiones del elemento.</param>
        public void Append(byte item, int count)
        {
            if (item==0)
            {
                this.AppendBlank(count);
                return;
            }

            if (count < 2)
            {
                if (count==1)
                    this.Append(item);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }
            this.EnsureAppend(count);
            fixed (byte* pd=this._items)
                General.Fill(pd + this._size, count, item);
            this._size +=count;
        }

        /// <summary>
        /// Agrega un cero al final de la coleccin.
        /// </summary>
        public void AppendBlank()
        {
            this.EnsureAppend();
            this._items[this._size++]=0;
        }

        /// <summary>
        /// Agrega ceros al final de la coleccin.
        /// </summary>
        /// <param name="count">Cantidad de ceros.</param>
        public void AppendBlank(int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.AppendBlank();
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }
            this.EnsureAppend(count);
            fixed (byte* pd=this._items)
                General.Clean(pd + this._size, count);
            this._size +=count;
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        public void Append(byte[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void Append(byte[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Append(byte[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.Append(po + arrIndex, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void Append(byte* arr, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.Append(*arr);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }

            this.EnsureAppend(count);
            fixed (byte* pd=this._items)
                General.Copy(arr, pd + this._size, count);
            this._size +=count;
        }


        /// <summary>
        /// Elimina todos los elementos de la coleccin.
        /// </summary>
        public void Clear()
        {
            this._size=0;
        }

        /// <summary>
        /// Elimina todos los elementos de la coleccin y establece el array en null.
        /// </summary>
        public void ClearCompact()
        {
            this._size=0;
            this._cap=0;
            this._items=null;
        }

        /// <summary>
        /// Compacta la coleccin a la cantidad de elementos actual.
        /// Si la cantidad actual de elementos es cero, el array subyacente ser null.
        /// </summary>
        public void Compact()
        {
            if (this._size !=this._cap)
            {
                if (this._size!=0)
                {
                    byte[] array2=new byte[this._size];
                    fixed (byte* po=this._items, pd=array2)
                        General.Copy(po, pd, this._size);
                    this._items=array2;
                }
                else
                    this._items=null;
                this._cap=this._size;
            }
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        public void CopyTo(byte* arr)
        {
            if (this._size !=0)
                fixed (byte* po=this._items)
                    General.Move(po, arr, this._size);
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        public void CopyTo(byte* arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (this._size !=startIndex)
                fixed (byte* po=this._items)
                    General.Move(po + startIndex, arr, this._size - startIndex);
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        /// <param name="count">Elementos a copiar.</param>
        public void CopyTo(byte* arr, int startIndex, int count)
        {

            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (this._size !=startIndex)
                fixed (byte* po=this._items)
                    General.Move(po + startIndex, arr, count);
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        public void CopyTo(byte[] arr)
        {
            if (this._size !=0)
            {
                if (arr.Length < this._size)
                    throw new ArgumentOutOfRangeException("arr", arr.Length, Errors.ARRAY_SMALL_DEST);
                fixed (byte* po=this._items, pd=arr)
                    General.Copy(po, pd, this._size);
            }
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        public void CopyTo(byte[] arr, int startIndex)
        {
            int count=this._size - startIndex;
            if (startIndex > this._size || startIndex < 0 || count > arr.Length)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                fixed (byte* po=this._items, pd=arr)
                    General.Copy(po + startIndex, pd, count);
        }


        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        /// <param name="count">Elementos a copiar.</param>
        public void CopyTo(byte[] arr, int startIndex, int count)
        {
            if (startIndex > this._size || startIndex < 0 || count < 0 || count > arr.Length)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                fixed (byte* po=this._items, pd=arr)
                    General.Copy(po + startIndex, pd, count);
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de 1 elemento.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        public void EnsureAppend()
        {
            if (this._size == this._cap)
                setCapacity(newMinCap(1));
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de n elementos.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        /// <param name="items">Elementos a agregar.</param>
        public void EnsureAppend(int items)
        {
            if ((items + this._size) > this._cap)
                setCapacity(newMinCap(items));
        }

        /// <summary>
        /// Recupera una copia de los elementos.
        /// </summary>
        /// <returns>Array con los elementos copiados.</returns>
        public byte[] GetCopy()
        {
            return this.ArrayCopy;
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public byte[] GetCopy(int count)
        {
            if (this._size < count || count < 0)
                throw new System.ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            byte[] ret=new byte[count];
            if (count !=0)
            {
                fixed (byte* po=this._items, pd=ret)
                    General.Copy(po, pd, count);
            }
            return ret;
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="startIndex">Index donde se inicia la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public byte[] GetCopy(int startIndex, int count)
        {
            if (this._size - startIndex < count || count < 0 || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            byte[] ret=new byte[count];
            if (count !=0)
            {
                fixed (byte* po=this._items, pd=ret)
                    General.Copy(po + startIndex, pd, count);
            }
            return ret;
        }

        /// <summary>
        /// Quita todos los elementos del rango especificado.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        /// <param name="count">Cantidad de elementos a eliminar.</param>
        public void RemoveRange(int startIndex, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.RemoveAt(startIndex);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                return;
            }
            int max=startIndex + count;

            if (startIndex < 0 || max > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (max==this._size)
            {
                this._size=startIndex;
                return;
            }
            fixed (byte* p=this._items)
                General.Cut(p, this._size, startIndex, count, false);
            this._size -=count;
        }

        /// <summary>
        /// Inserta el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="index">Index en la coleccin donde se insertar el elemento.</param>
        public void Insert(int index, byte item)
        {
            this.Reserve(index);
            this._items[index]=item;
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void Insert(int startIndex, byte item, int count)
        {
            this.Reserve(startIndex,count);
            this.Set(startIndex,item,count);
        }

        /// <summary>
        /// Inserta un cero en la ubicacin especificada.
        /// </summary>
        /// <param name="index">Index en la coleccin donde insertar.</param>
        public void InsertBlank(int index)
        {
            this.Reserve(index);
            this._items[index]=0;
        }


        /// <summary>
        /// Inserta ceros en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void InsertBlank(int startIndex, int count)
        {
            this.Reserve(startIndex,count);
            this.SetBlank(startIndex,count);
        }


        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Insert(int startIndex, byte[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Insert(int startIndex, byte[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, byte[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.Insert(startIndex,po + arrIndex, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, byte* arr, int count)
        {
            this.Reserve(startIndex,count);
            this.Set(startIndex,arr,count);
        }

        /// <summary>
        /// Quita del array el elemento especificado.
        /// </summary>
        /// <param name="index">Index del elemento.</param>
        public void RemoveAt(int index)
        {
            if (index >=this._size || index < 0)
                throw new System.ArgumentOutOfRangeException("index", index, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (--this._size !=index)
            {
                fixed (byte* p=this._items)
                    General.Copy(p + (index + 1), p + index, this._size - index);
            }

        }

        /// <summary>
        /// Quita todos los elementos desde la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        public void RemoveRange(int startIndex)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
        }

        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, byte[] r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex);
            else
                fixed(byte*pr=r)
                    ReplaceRange(startIndex,pr,lr);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, byte[] r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex,count);
            else
                fixed(byte*pr=r)
                    ReplaceRange(startIndex,count,pr,lr);
        }

        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, byte* pr, int lr)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
            this.Append(pr,lr);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, byte* pr, int lr)
        {
            if (count>=lr)
            {
                Set(startIndex,pr,lr);
                if (count>lr)
                    RemoveRange(startIndex+lr,count-lr);
                return;
            }
            if (count<1)
            {
                if (count==0)
                {
                    Insert(startIndex,pr,lr);
                    return;
                }
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            Reserve(startIndex+count,lr-count);
            Set(startIndex,pr,lr);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        public void Reverse()
        {
            if (this._size > 1)
                fixed (byte* p=this._items)
                    Sort.Reverse(p, this._size);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void Reverse(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (byte* p=this._items)
                    Sort.Reverse(p + startIndex, count);
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de un elemento. A diferencia de la funcin Insert,
        /// no efecta ninguna copia o limpieza.
        /// </summary>
        /// <param name="index">Index de la insercin.</param>
        public void Reserve(int index)
        {
            if (index<0 || index>this._size)
                throw new ArgumentOutOfRangeException("index",index,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (index==this._size)
                this.EnsureAppend();
            else
            {
                if (this._size==this._cap)
                {
                    byte[] array2=newArraySum(1);
                    fixed (byte* po=this._items, pd=array2)
                    {
                        if (index!=0)
                            General.Copy(po, pd, index);
                        General.Copy(po + index, pd + (index + 1), this._size - index);
                    }
                    this._items=array2;
                }
                else
                {
                    fixed (byte* po=this._items)
                        General.Move(po + index, po + (index + 1), this._size - index);
                }
            }
            this._size++;
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de la cantidad de elementos especificada mediante las funciones Set
        /// u otras. A diferencia de la funcin Insert, no efecta ninguna copia o limpieza
        /// sobre el area de insercin, siendo el contenido de esta indeterminado.
        /// </summary>
        /// <param name="startIndex">Index de inicio de insercin.</param>
        /// <param name="count">Cantidad de elementos a reservar.</param>
        public void Reserve(int startIndex, int count)
        {
            if (count!=0)
            {
                if (count==1)
                    this.Reserve(startIndex);
                else if (startIndex<0 || count<0)
                    throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE);
                else if (startIndex==this._size)
                {
                    this.EnsureAppend(count);
                    this._size+=count;
                }
                else if (startIndex>this._size)
                    throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.SUPERIOR_ARRAY);
                else
                {
                    int nue=this._size + count;
                    if (nue > this._cap)
                    {
                        byte[] array2=newArraySum(count);
                        fixed (byte* po=this._items, pd=array2)
                        {
                            if (startIndex!=0)
                                General.Copy(po, pd, startIndex);
                            General.Copy(po + startIndex, pd + (startIndex + count), this._size - startIndex);
                        }
                        this._items=array2;
                    }
                    else
                    {
                        fixed (byte* po=this._items)
                            General.Move(po + startIndex, po + (startIndex + count), this._size - startIndex);
                    }
                    this._size=nue;
                }
            }
        }

        /// <summary>
        /// Setea el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="index">Index en la coleccin donde se setear el elemento.</param>
        public void Set(int index, byte item)
        {
            this[index]=item;
        }


        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a setear.</param>
        public void Set(int startIndex, byte item, int count)
        {
            if (count==1)
                this[startIndex]=item;
            else if (item==0)
                SetBlank(startIndex,count);
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (byte* po=this._items)
                    General.Fill(po + startIndex, count, item);
        }

        /// <summary>
        /// Setea un cero en la ubicacin especificada.
        /// </summary>
        /// <param name="index">Index en la coleccin donde setear.</param>
        public void SetBlank(int index)
        {
            this[index]=0;
        }


        /// <summary>
        /// Setea ceros en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a setear.</param>
        public void SetBlank(int startIndex, int count)
        {
            if (count==1)
                this[startIndex]=0;
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (byte* po=this._items)
                    General.Clean(po + startIndex, count);
        }

        /// <summary>
        /// Modifica la cantidad de elementos en la coleccin.
        /// </summary>
        /// <param name="count">Nueva cantidad de elementos.</param>
        /// <param name="exact">En caso de que sea necesario aumentar la capacidad, no aplica algoritmo al aumento.</param>
        /// <param name="clean">En caso de aumentar la cantidad, rellena con ceros el nuevo espacio.</param>
        public void SetLength(int count, bool exact, bool clean)
        {
            if (count <=this._cap)
            {
                if (count <=this._size)
                {
                    if (count < 0)
                        throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                }
                else if (clean)
                    fixed (byte* p=this._items)
                        General.Clean(p + this._size, count - this._size);
            }
            else
            {
                byte[] array2;
                if (exact)
                {
                    array2=new byte[count];
                    this._cap=count;
                }
                else
                    array2=newArraySum(count-this._size);

                if (this._size !=0)
                {
                    fixed (byte* po=this._items, pd=array2)
                        General.Copy(po, pd, this._size);
                }
                this._items=array2;
            }
            this._size=count;
        }

        /// <summary>
        /// Modifica la cantidad de elementos para que coincida con la capacidad actual.
        /// </summary>
        /// <param name="clean">Rellena con ceros el rea excedente.</param>
        public void SetLengthToCapacity(bool clean)
        {
            if (this._size!=this._cap)
            {
                if (clean)
                    fixed(byte*p=this._items)
                        General.Clean(p+this._size,this._cap-this._size);
                this._size=this._cap;
            }
        }


        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Set(int startIndex, byte[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Set(int startIndex, byte[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, byte[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.Set(startIndex, po + arrIndex, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, byte* arr, int count)
        {
            if (count==1)
                this[startIndex]=*arr;
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (byte* p=this._items)
                    General.Copy(arr,p + startIndex, count);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortAscending()
        {
            if (this._size > 1)
                fixed (byte* p=this._items)
                    Sort.Ascending(p, this._size);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortAscending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (byte* p=this._items)
                    Sort.Ascending(p + startIndex, count);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortDescending()
        {
            if (this._size > 1)
                fixed (byte* p=this._items)
                    Sort.Descending(p, this._size);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortDescending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (byte* p=this._items)
                    Sort.Descending(p + startIndex, count);
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        public void SortRandom()
        {
            if (this._size > 1)
                fixed (byte* p=this._items)
                    Sort.Randomize(p, this._size);
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortRandom(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (byte* p=this._items)
                    Sort.Randomize(p + startIndex, count);
        }


        #endregion

        #region Set, Insert y Append para cantidades limitadas de elementos

        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, byte item0, byte item1)
        {
            if (startIndex<0 || startIndex+1>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            fixed(byte*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
            }
        }

        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, byte item0, byte item1, byte item2)
        {
            if (startIndex<0 || startIndex+2>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            fixed(byte*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
            }
        }


        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, byte item0, byte item1, byte item2, byte item3)
        {
            if (startIndex<0 || startIndex+3>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            fixed(byte*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
                p[startIndex+3]=item3;
            }
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, byte item0, byte item1)
        {
            this.Reserve(startIndex,2);
            fixed(byte*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
            }
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, byte item0, byte item1, byte item2)
        {
            this.Reserve(startIndex,3);
            fixed(byte*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
            }
        }


        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, byte item0, byte item1, byte item2, byte item3)
        {
            this.Reserve(startIndex,4);
            fixed(byte*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
                p[startIndex+3]=item3;
            }
        }

        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        public void AppendItems(byte item0, byte item1)
        {
            this.EnsureAppend(2);
            fixed(byte*p=this._items)
            {
                p[this._size]=item0;
                p[this._size+1]=item1;
                this._size+=2;
            }
        }

        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        public void AppendItems(byte item0, byte item1, byte item2)
        {
            this.EnsureAppend(3);
            fixed(byte*p=this._items)
            {
                p[this._size]=item0;
                p[this._size+1]=item1;
                p[this._size+2]=item2;
                this._size+=3;
            }
        }


        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        public void AppendItems(byte item0, byte item1, byte item2, byte item3)
        {
            this.EnsureAppend(4);
            fixed(byte*p=this._items)
            {
                p[this._size]=item0;
                p[this._size+1]=item1;
                p[this._size+2]=item2;
                p[this._size+3]=item3;
                this._size+=4;

            }
        }

        #endregion

        #region Metodos de Bsqueda, reemplazo y comparacin.
  
        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="first">Se asegura de que la ubicacin de retorno corresponda a la primer aparicin del elemento especificado.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// La coleccin debe estar ordenada en forma ascendente.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public int BinarySearch(byte item, bool first)
        {
            if (this._size==0)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.BinarySearch(p, this._size, item);
                if (first && ret > 0)
                    while (p[ret - 1]==item && --ret !=0);
                return ret;
            }
        }

        /// <summary>
        /// Efecta una comparacin con la coleccin especificada.
        /// </summary>
        /// <param name="col2">Coleccin de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en esta coleccin que en col2.
        /// - Si todos los elementos comparados son iguales y col2 contiene ms elementos que esta coleccin.
        /// Retorna 0:
        /// - Si esta coleccin y col2 representan al mismo objeto.
        /// - Si todos los elementos comparados son iguales siempre y cuando esta coleccin y col2 tengan la misma cantidad de elementos.
        /// Retorna 1:
        /// - Si col2 es null.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en esta coleccin que en col2.
        /// - Si todos los elementos comparados son iguales y col2 contiene menos elementos que esta coleccin.
        /// </returns>
        public int CompareTo(Bytes col2)
        {
            if (Object.ReferenceEquals(this,col2))
                return 0;
            if (col2==null)
                return 1;
            if (col2._size==0)
                return this._size!=0?1:0;
            if (this._size==0)
                return -1;
            int eqComp,count;
            if (col2._size>this._size)
            {
                count=this._size;
                eqComp=-1;
            }
            else if (col2._size!=this._size)
            {
                count=col2._size;
                eqComp=1;
            }
            else
            {
                count=this._size;
                eqComp=0;
            }
            fixed(byte*p1=this._items, p2=col2._items)
            {
                count=Comp.Compare(p1,p2,count);
                if (count==-1)
                    return eqComp;
                return (p1[count]<p2[count])?-1:1;
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y array2 contiene ms elementos que la coleccin.
        /// Retorna 0: 
        /// - Si todos los elementos comparados son iguales siempre y cuando la coleccin y array2 tengan la misma cantidad de elementos.
        /// Retorna 1:
        /// - Si array2 es null.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y array2 contiene menos elementos que la coleccin.
        /// </returns>
        public int CompareTo(byte[] array2)
        {
            if (array2==null)
                return 1;
            int count=array2.Length;
            if (count==0)
                return this._size!=0?1:0;
            if (this._size==0)
                return -1;
            int eqComp;
            if (count>this._size)
            {
                count=this._size;
                eqComp=-1;
            }
            else if (count!=this._size)
                eqComp=1;
            else
                eqComp=0;
            fixed(byte*p1=this._items, p2=array2)
            {
                count=Comp.Compare(p1,p2,count);
                if (count==-1)
                    return eqComp;
                return (p1[count]<p2[count])?-1:1;
            }
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="arrIndex">Inicio de la comparacin en array2.</param>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si arrIndex esta dentro del rango y startIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y el rango comparable es mayor en array2 que en la coleccin.
        /// Retorna 0: 
        /// - Si tanto startIndex como arrIndex estn fuera de rango.
        /// - Si todos los elementos comparados son iguales siempre y cuando la coleccin y array2 tengan el mismo rango comparable.
        /// Retorna 1:
        /// - Si startIndex esta dentro del rango y arrIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y el rango comparable es menor en array2 que en la coleccin.
        /// </returns>
        public int CompareTo(byte[] array2, int startIndex, int arrIndex)
        {
            if (startIndex<0 || arrIndex<0)
                throw new ArgumentOutOfRangeException("startIndex,arrIndex",Errors.NEGATIVE);
            int eqComp=array2.Length;
            if (arrIndex>=eqComp)
            {
                if (startIndex>=this._size)
                    return 0;
                return 1;
            }
            if (startIndex>=this._size)
                return -1;
            
            int count=this._size-startIndex;
            if (count<(eqComp-=arrIndex))
                eqComp=-1;
            else if(count!=eqComp)
            {
                count=eqComp;
                eqComp=1;
            }
            else
                eqComp=0;
            fixed(byte*p1=this._items, p2=array2)
            {
                count=Comp.Compare(p1+startIndex,p2+arrIndex,count);
                if (count==-1)
                    return eqComp;
                return (p1[startIndex+count]<p2[arrIndex+count])?-1:1;
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="arrIndex">Inicio de la comparacin en array2.</param>
        /// <param name="count">Mximo de elementos a comparar.</param>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si count es mayor a cero, arrIndex esta dentro del rango y startIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales siempre y cuando count haya sido reducido total o en mayor medida para ajustarse a la cantidad de elementos de la coleccin.
        /// Retorna 0: 
        /// - Si count es cero.
        /// - Si tanto startIndex como arrIndex estn fuera de rango.
        /// - Si todos los elementos comparados son iguales siempre y cuando count no haya sido reducido o que la reduccin se deba a un mismo excedente en la coleccin y array2.
        /// Retorna 1:
        /// - Si count es mayor a cero, startIndex esta dentro del rango y arrIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales siempre y cuando count haya sido reducido total o en mayor medida para ajustarse al largo de array2.
        /// </returns>
        public int CompareTo(byte[] array2, int startIndex, int arrIndex, int count)
        {
            if (startIndex<0 || arrIndex<0)
                throw new ArgumentOutOfRangeException("startIndex,arrIndex",Errors.NEGATIVE);
            if (count<1)
            {
                if (count==0)
                    return 0;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            int eqComp=array2.Length;
            if (arrIndex>=eqComp)
            {
                if (startIndex>=this._size)
                    return 0;
                return 1;
            }
            if (startIndex>=this._size)
                return -1;
            eqComp-=arrIndex;
            int rest1=this._size-startIndex;
            if (rest1<count)
            {
                if (eqComp<count)
                {
                    if (rest1<eqComp)
                    {
                        count=rest1;
                        eqComp=-1;
                    }
                    else if (rest1!=eqComp)
                    {
                        count=eqComp;
                        eqComp=1;
                    }
                    else
                    {
                        count=rest1;
                        eqComp=0;
                    }
                }
                else
                {
                    count=rest1;
                    eqComp=-1;
                }
            }
            else if (eqComp<count)
            {
                count=eqComp;
                eqComp=1;
            }
            else
                eqComp=0;
            fixed(byte*p1=this._items, p2=array2)
            {
                count=Comp.Compare(p1+startIndex,p2+arrIndex,count);
                if (count==-1)
                    return eqComp;
                return (p1[startIndex+count]<p2[arrIndex+count])?-1:1;
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="count">Mximo de elementos a comparar.</param>
        /// <param name="p2">Puntero de comparacin. Debe contener a count.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si count es mayor a cero y startIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en p2.
        /// - Si todos los elementos comparados son iguales siempre y cuando count haya sido reducido para ajustarse a la cantidad de elementos de la coleccin.
        /// Retorna 0: 
        /// - Si count es cero.
        /// - Si todos los elementos comparados son iguales siempre y cuando count no haya sido reducido.
        /// Retorna 1:
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en p2.
        /// </returns>
        public int CompareTo(byte* p2, int startIndex, int count)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            if (count<1)
            {
                if (count==0)
                    return 0;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (startIndex>=this._size)
                return -1;
            int eqComp=this._size-startIndex;
            if (eqComp<count)
            {
                count=eqComp;
                eqComp=-1;
            }
            else
                eqComp=0;
            fixed(byte*p1=this._items)
            {
                count=Comp.Compare(p1+startIndex,p2,count);
                if (count==-1)
                    return eqComp;
                return (p1[startIndex+count]<p2[count])?-1:1;
            }
        }

        /// <summary>
        /// Retorna true si contiene el elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(byte item,bool last)
        {
            if (this._size==0)
                return false;
            fixed (byte* p=this._items)
            {
                if (!last)
                    return Comp.Exist(p, this._size, item);
                return Comp.LastIndexOf(p,this._size,item)!=-1;
            }
        }



        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(byte item)
        {
            if (this._size !=0)
            {
                fixed (byte* p=this._items)
                    return Comp.Count(p, this._size, item);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(byte item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (byte* p=this._items)
                return Comp.Count(p + startIndex, this._size - startIndex, item);
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(byte item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (byte* p=this._items)
                return Comp.Count(p + startIndex, count, item);
        }


        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(byte[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (byte* p=this._items, pf=items)
                    return Comp.CountAny(p, this._size, pf, lf, false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, byte[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (byte* p=this._items, pf=items)
                return Comp.CountAny(p + startIndex, this._size - startIndex, pf, lf, false);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, int count, byte[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (byte* p=this._items, pf=items)
                return Comp.CountAny(p + startIndex, count, pf, lf, false);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(byte* pf, int lf)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (byte* p=this._items)
                    return Comp.CountAny(p, this._size, pf, lf, false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, byte* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (byte* p=this._items)
                return Comp.CountAny(p + startIndex, this._size - startIndex, pf, lf, false);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, int count, byte* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (byte* p=this._items)
                return Comp.CountAny(p + startIndex, count, pf, lf, false);
        }


        /// <summary>
        /// Efecta una comparacin con la coleccin especificada. 
        /// </summary>
        /// <param name="col2">Coleccin a comparar.</param>
        /// <returns>Retorna true si ambas coleccin contienen la misma cantidad de elementos y su contenido es igual.</returns>
        public bool Equals(Bytes col2)
        {
            if (Object.ReferenceEquals(this,col2))
                return true;
            if (col2==null || this._size!=col2._size)
                return false;
            if (this._size<2)
            {
                if (this._size==0 || this._items[0]==col2._items[0])
                    return true;
                return false;
            }
            fixed(byte*p=this._items, p2=col2._items)
                return Comp.Equal(p,p2,this._size);
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="array2">Array a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de array2 son iguales a los la coleccin (comparando desde el inicio) o array2 tiene largo 0.</returns>
        public bool Equals(byte[] array2)
        {
            int count=array2.Length;
            if (count<2)
            {
                if (count==0 || (this._size!=0 && array2[0]==this._items[0]))
                    return true;
                return false;
            }
            if (count>this._size)
                return false;
            fixed(byte*p=this._items, p2=array2)
                return Comp.Equal(p,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="array2">Array a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <returns>Solo retorna true si todos los elementos de array2 son iguales a los la coleccin (comparando desde startIndex) o array2 tiene largo 0.</returns>
        public bool Equals(byte[] array2, int startIndex)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            int count=array2.Length;
            if (count<2)
            {
                if (count==0 || (this._size>startIndex && array2[0]==this._items[startIndex]))
                    return true;
                return false;
            }
            if (startIndex+count>this._size)
                return false;
            fixed(byte*p=this._items, p2=array2)
                return Comp.Equal(p+startIndex,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="p2">Array a comparar.</param>
        /// <param name="count">Cantidad de elementos a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de p2 son iguales a los la coleccin (comparando desde el inicio) o count es 0.</returns>
        public bool Equals(byte* p2, int count)
        {
            if (count<2)
            {
                if (count==1)
                {
                    if (this._size!=0 && this._items[0]==*p2)
                        return true;
                    return false;
                }
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (count>this._size)
                return false;
            fixed(byte*p=this._items)
                return Comp.Equal(p,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="p2">Array a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="count">Cantidad de elementos a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de p2 son iguales a los la coleccin (comparando desde el startIndex) o count es 0.</returns>
        public bool Equals(byte* p2, int startIndex, int count)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            if (count<2)
            {
                if (count==1)
                {
                    if (this._size>startIndex && this._items[startIndex]==*p2)
                        return true;
                    return false;
                }
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (startIndex+count>this._size)
                return false;
            fixed(byte*p=this._items)
                return Comp.Equal(p+startIndex,p2,count);
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(byte item)
        {
            if (this._size !=0)
            {
                fixed (byte* p=this._items)
                    return Comp.IndexOf(p, this._size, item);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(byte item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.IndexOf(p + startIndex, this._size - startIndex, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(byte item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.IndexOf(p + startIndex, count, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }


        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOfAny(byte[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (byte* p=this._items, pf=items)
                    return Comp.IndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, byte[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            int ret;
            fixed (byte* p=this._items, pf=items)
                ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, int count, byte[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            int ret;
            fixed (byte* p=this._items, pf=items)
                ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOfAny(byte* pf, int lf)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (byte* p=this._items)
                    return Comp.IndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, byte* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, int count, byte* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(byte item)
        {
            if (this._size !=0)
            {
                fixed (byte* p=this._items)
                    return Comp.LastIndexOf(p, this._size, item);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(byte item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, this._size - startIndex, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(byte item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, count, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOfAny(byte[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (byte* p=this._items, pf=items)
                    return Comp.LastIndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, byte[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            int ret;
            fixed (byte* p=this._items, pf=items)
                ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, int count, byte[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            int ret;
            fixed (byte* p=this._items, pf=items)
                ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOfAny(byte* pf, int lf)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (byte* p=this._items)
                    return Comp.LastIndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, byte* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, int count, byte* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(byte item)
        {
            if (this._size==0)
                return 0;

            fixed (byte* p=this._items)
            {
                return Comp.Remove(false, p, p, this._size, out this._size, item, false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(byte item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (byte* p=this._items)
            {
                int lFinal;
                byte* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, this._size - startIndex, out lFinal, item, false);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(byte item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (byte* p=this._items)
            {
                int lFinal;
                byte* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, count, out lFinal, item, false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(byte[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return 0;
            fixed (byte* p=this._items, pf=items)
            {
                return Comp.RemoveAny(false, p, p, this._size, out this._size, pf, lf, false, false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, byte[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (byte* p=this._items, pf=items)
            {
                int lFinal;
                byte* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, this._size - startIndex, out lFinal, pf, lf, false, false);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, int count, byte[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (byte* p=this._items, pf=items)
            {
                int lFinal;
                byte* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, count, out lFinal, pf, lf, false, false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(byte* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return 0;

            fixed (byte* p=this._items)
            {
                return Comp.RemoveAny(false, p, p, this._size, out this._size, pf, lf, false, false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, byte* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (byte* p=this._items)
            {
                int lFinal;
                byte* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, this._size - startIndex, out lFinal, pf, lf, false, false);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, int count, byte* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (byte* p=this._items)
            {
                int lFinal;
                byte* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, count, out lFinal, pf, lf, false, false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }


        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(byte item)
        {
            if (this._size==0)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, item);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(byte item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(byte item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(byte[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;

            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, byte[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, int count, byte[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(byte* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, byte* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, int count, byte* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(byte item)
        {
            if (this._size==0)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, item);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(byte item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(byte item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOfAny(byte[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;

            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, byte[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, int count, byte[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOfAny(byte* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, byte* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, int count, byte* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(byte item, byte rep)
        {
            if (this._size==0)
                return 0;

            fixed (byte* p=this._items)
            {
                return Comp.Replace(p, this._size, item, rep);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(byte item, byte rep, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (byte* p=this._items)
            {
                return Comp.Replace(p + startIndex, this._size - startIndex, item, rep);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(byte item, byte rep, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (byte* p=this._items)
            {
                return Comp.Replace(p + startIndex, count, item, rep);
            }
        }


        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(byte rep, byte[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return 0;
            fixed (byte* p=this._items, pf=items)
            {
                return Comp.ReplaceAny(p, this._size, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, byte rep, byte[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (byte* p=this._items, pf=items)
            {
                return Comp.ReplaceAny(p + startIndex, this._size - startIndex, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, int count, byte rep, byte[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (byte* p=this._items, pf=items)
            {

                return Comp.ReplaceAny(p + startIndex, count, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(byte rep, byte* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return 0;

            fixed (byte* p=this._items)
            {
                return Comp.ReplaceAny(p, this._size, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, byte rep, byte* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (byte* p=this._items)
            {
                return Comp.ReplaceAny(p + startIndex, this._size - startIndex, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, int count, byte rep, byte* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (byte* p=this._items)
            {
                return Comp.ReplaceAny(p + startIndex, count, pf, lf, rep, false);
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(byte item, byte rep)
        {
            if (this._size==0)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, item);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(byte item, byte rep, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(byte item, byte rep, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(byte rep, byte[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;

            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, byte rep, byte[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, int count, byte rep, byte[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(byte rep, byte* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, byte rep, byte* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, int count, byte rep, byte* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOf(byte item, byte rep)
        {
            if (this._size==0)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, item);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(byte item, byte rep, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(byte item, byte rep, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(byte rep, byte[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;

            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, byte rep, byte[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, int count, byte rep, byte[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (byte* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(byte rep, byte* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, byte rep, byte* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, int count, byte rep, byte* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }


        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los del array de comparacin.
        /// </summary>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los del array de comparacin.</returns>
        public bool EndsWith(byte[] array2)
        {

            int count=array2.Length;
            if (count<2)
            {
                if (count==0 || (this._size!=0 && array2[0]==this._items[this._size-1]))
                    return true;
                return false;
            }
            if (count>this._size)
                return false;
            fixed(byte*p=this._items, p2=array2)
                return Comp.Equal(p+(this._size-count),p2,count);
        }

        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los del puntero de comparacin.
        /// </summary>
        /// <param name="p2">Puntero de comparacin.</param>
        /// <param name="count">Largo del puntero de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los del puntero de comparacin.</returns>
        public bool EndsWith(byte* p2, int count)
        {
            if (count<2)
            {
                if (count==1)
                {
                    if (this._size!=0 && this._items[this._size-1]==*p2)
                        return true;
                    return false;
                }
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (count>this._size)
                return false;
            fixed(byte*p=this._items)
                return Comp.Equal(p+(this._size-count),p2,count);
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los del array de comparacin.
        /// </summary>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los del array de comparacin.</returns>
        public bool StartsWith(byte[] array2)
        {
            return Equals(array2);
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los del puntero de comparacin.
        /// </summary>
        /// <param name="p2">Puntero de comparacin.</param>
        /// <param name="count">Largo del puntero de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los del puntero de comparacin.</returns>
        public bool StartsWith(byte* p2, int count)
        {
            return Equals(p2,count);
        }

        #endregion

        #region Propiedades y Constructores Bytes y Chars

        /// <summary>
        /// Inicializa la coleccin permitiendo especificar propiedades.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="unix">
        /// Estado inicial de la propiedad Unix.
        /// </param>
        /// <param name="cap">Capacidad inicial.</param>
        public Bytes(int cap, bool appendExact, bool unix):this(cap)
        {
            this._flags=1U<<9;
            if (appendExact)
                this._flags+=1U<<8;
            if (unix)
                this._flags+=1U<<10;
        }

        /// <summary>
        /// Inicializa la coleccin con un array nulo, permitiendo especificar propiedades.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="unix">
        /// Estado inicial de la propiedad Unix.
        /// </param>
        public Bytes(bool appendExact, bool unix)
        {
            this._flags=1U<<9;
            if (appendExact)
                this._flags+=1U<<8;
            if (unix)
                this._flags+=1U<<10;
        }

        /// <summary>
        /// Obtiene o establece si la coleccin utilizar saltos de lnea Unix en las funciones
        /// AppendLine, InsertLine y Reformat.
        /// Al obtener, si no se ha establecido anteriormente, el valor ser establecido de acuerdo
        /// al siguiente procedimiento:
        /// - Si la cadena esta vacia o no se encuentran saltos de lnea, el valor ser false y por
        ///   lo tanto usar saltos de lnea CRLF.
        /// - Si se encuentra el caracter CR, el valor tambin ser false independientemente a que LF
        ///   se encuentre o no.
        /// - Si no se encuentra el caracter CR pero s LF, el valor ser true
        ///   y por lo tanto usar saltos de lnea LF. 
        /// </summary>
        public bool Unix
        {
            get
            {
                switch(this._flags&((1U<<9)+(1U<<10)))
                {
                    case 1U<<9:
                        return false;
                    case (1U<<9)+(1U<<10):
                        return true;
                }
                if (this.LastIndexOf(13)!=-1)
                {
                    this._flags|=1U<<9;
                    return false;
                }
                if (this.LastIndexOf(10)!=-1)
                {
                    this._flags|=(1U<<9)+(1U<<10);
                    return true;
                }
                this._flags|=1U<<9;
                return false;
            }
            set
            {
                if (value)
                    this._flags|=(1U<<9)+(1U<<10);
                else if ((this._flags&(1U<<10))!=0)
                    this._flags&=0xFFFFFFFF^(1U<<10);
                else
                    this._flags|=1U<<9;
            }
        }

        #endregion
     
        #region Metodos de Bsqueda y reemplazo Bytes y Chars.

        /// <summary>
        /// Retorna true si contiene la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(byte[] arr, bool last)
        {
            int lf;
            if (this._size==0 || this._size < (lf=arr.Length))
                return false;
            fixed (byte* p=this._items, pf=arr)
            {
                if (!last)
                    return Comp.IndexOf(p,this._size,pf,lf,false)!=-1;
                return Comp.LastIndexOf(p,this._size,pf,lf,false)!=-1;
            }
        }

        /// <summary>
        /// Retorna true si contiene la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(byte* pf, int lf, bool last)
        {
            if (this._size==0 || this._size<lf)
                return false;
            fixed (byte* p=this._items)
            {
                if (!last)
                    return Comp.IndexOf(p,this._size,pf,lf,false)!=-1;
                return Comp.LastIndexOf(p,this._size,pf,lf,false)!=-1;
            }
        }
        
        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(byte[] arr)
        {
            int lf;
            if (this._size !=0 && (lf=arr.Length)<=this._size)
            {
                fixed (byte* p=this._items, pf=arr)
                    return Comp.Count(p,this._size,pf,lf,false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(byte[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (byte* p=this._items, pf=arr)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de a buscar en la coleccin.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(byte[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (byte* p=this._items, pf=arr)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(byte* pf, int lf)
        {
            if (this._size !=0 && lf<=this._size)
            {
                fixed (byte* p=this._items)
                    return Comp.Count(p,this._size,pf,lf,false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(byte*pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return 0;
            fixed (byte* p=this._items)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de a buscar en la coleccin.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(byte* pf,int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return 0;
            fixed (byte* p=this._items)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(byte[] arr)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=arr.Length))
            {
                fixed (byte* p=this._items, pf=arr)
                    return Comp.IndexOf(p, this._size,pf,lf,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(byte[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (byte* p=this._items, pf=arr)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(byte[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (byte* p=this._items, pf=arr)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(byte* pf, int lf)
        {
            
            if (this._size !=0 && this._size>=lf)
            {
                fixed (byte* p=this._items)
                    return Comp.IndexOf(p, this._size,pf,lf,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(byte* pf,int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(byte* pf,int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(byte[] arr)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=arr.Length))
            {
                fixed (byte* p=this._items, pf=arr)
                    return Comp.LastIndexOf(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(byte[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (byte* p=this._items, pf=arr)
                ret=Comp.LastIndexOf(p + startIndex, count,pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(byte[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (byte* p=this._items, pf=arr)
                ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }


        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(byte* pf, int lf)
        {
            if (this._size !=0 && this._size>=lf)
            {
                fixed (byte* p=this._items)
                    return Comp.LastIndexOf(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(byte* pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, count,pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(byte* pf, int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (byte* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(byte[] arr)
        {
            int lf;
            if (this._size==0 || this._size<(lf=arr.Length))
                return 0;

            fixed (byte* p=this._items, pf=arr)
            {
                return Comp.Remove(false, p, p, p, this._size, out this._size, pf , lf, false , false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(byte[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (byte* p=this._items, pf=arr)
            {
                byte* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, po, count, out count,pf ,lf, false,false);
                if (ret!=0)
                    this._size -=ret*lf;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(byte[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (byte* p=this._items, pf=arr)
            {
                int lFinal;
                byte* po=p + startIndex;
                int ret=Comp.Remove(false, po,po, po, count, out lFinal, pf, lf, false,false);
                if (ret!=0)
                {

                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret*lf;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(byte* pf, int lf)
        {
            if (this._size==0 || this._size<lf)
                return 0;

            fixed (byte* p=this._items)
            {
                return Comp.Remove(false, p, p, p, this._size, out this._size, pf , lf, false , false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(byte* pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return 0;
            fixed (byte* p=this._items)
            {
                byte* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, po, count, out count,pf ,lf, false,false);
                this._size -=ret*lf;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(byte* pf, int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return 0;
            fixed (byte* p=this._items)
            {
                int lFinal;
                byte* po=p + startIndex;
                int ret=Comp.Remove(false, po,po, po, count, out lFinal, pf, lf, false,false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret*lf;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(byte[] arr)
        {
            int lf;
            if (this._size==0 || this._size<(lf=arr.Length))
                return -1;

            fixed (byte* p=this._items, pf=arr)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(byte[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (byte* p=this._items, pf=arr)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(byte[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (byte* p=this._items, pf=arr)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(byte* pf, int lf)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(byte* pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(byte* pf, int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(byte[] arr)
        {
            int lf;
            if (this._size==0 || this._size<(lf=arr.Length))
                return -1;

            fixed (byte* p=this._items, pf=arr)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(byte[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (byte* p=this._items, pf=arr)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(byte[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (byte* p=this._items, pf=arr)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }

        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(byte* pf, int lf)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(byte* pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(byte* pf, int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(byte[] f, byte[] r)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f);
            int lf=f.Length;
            if (lf==0 || this._size<lf)
                return 0;
            fixed(byte*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(byte[] f, byte[] r, int startIndex)
        {
            return ReplaceAll(f,r,startIndex,this._size-startIndex);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(byte[] f, byte[] r, int startIndex, int count)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f,startIndex,count);
            int lf=f.Length;
            if (lf==0 || count<lf)
                return 0;
            fixed(byte*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr,startIndex,count);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="pf">Cadena de bsqueda.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(byte* pf, int lf, byte* pr, int lr)
        {
            if (this._size==0 || lf<1 || this._size<lf)
                return 0;
            fixed (byte* p=this._items)
            {
                if (lf>=lr)
                {
                    if (lf==lr)
                        return Comp.ReplaceEqual(p,p,this._size,pf,pr,lf,false);
                    return Comp.ReplaceMinor(false,p,p,p,this._size,out this._size,pf,lf,pr,lr,false,false);
                }
                Ints ubics=Comp.Locations(null,0,p,this._size,pf,lf,false);
                int ret;
                if (ubics==null || (ret=ubics.Length)==0)
                    return 0;
                
                int lFinal=((lr-lf)*ret)+this._size;
                if (this._cap<lFinal)
                {
                    byte[] array2=newArraySum(lFinal-this._size);
                    fixed(byte*pd=array2)
                        fixed(int*pCont=ubics.Array)
                            Comp.ReplaceLocations(pCont,ret,p,this._size,pd,lf,pr,lr,false,false);
                    this._items=array2;
                }
                else
                {
                    fixed(int*pCont=ubics.Array)
                        Comp.ReplaceLocations(pCont,ret,p,this._size,p,lf,pr,lr,false,false);
                }
                this._size=lFinal;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="pf">Cadena de bsqueda.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(byte* pf, int lf, byte* pr, int lr, int startIndex)
        {
            return ReplaceAll(pf,lf,pr,lr,startIndex,this._size-startIndex);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="pf">Cadena de bsqueda.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(byte* pf, int lf, byte* pr, int lr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || lf<1 || count<lf)
                return 0;
            fixed (byte* p=this._items)
            {
                byte*po=p+startIndex;
                int ret, lFinal;
                if (lf>=lr)
                {
                    if (lf==lr)
                        return Comp.ReplaceEqual(po,po,count,pf,pr,lf,false);
                    if ((ret=Comp.ReplaceMinor(false,po,po,po,count,out lFinal,pf,lf,pr,lr,false,false))!=0)
                    {
                        General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                        this._size -=count-lFinal;
                    }
                    return ret;
                }
                Ints ubics=Comp.Locations(null,0, po,count,pf,lf,false);
                if (ubics==null || (ret=ubics.Length)==0)
                    return 0;
                lFinal=((lr-lf)*ret)+this._size;
                if (this._cap<lFinal)
                {
                    byte[] array2=newArraySum(lFinal-this._size);
                    fixed(byte*pd=array2)
                    {
                        General.Copy(p,pd,startIndex);
                        fixed(int*pCont=ubics.Array)
                            Comp.ReplaceLocations(pCont,ret,po,this._size-startIndex,pd+startIndex,lf,pr,lr,false,false);
                    }
                    this._items=array2;
                }
                else
                {
                    fixed(int*pCont=ubics.Array)
                        Comp.ReplaceLocations(pCont,ret,po,this._size-startIndex,po,lf,pr,lr,false,false);
                }
                this._size=lFinal;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(byte[] f, byte[] r)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (byte* p=this._items, pf=f)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(byte[] f, byte[] r, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (byte* p=this._items, pf=f)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(byte[] f, byte[] r, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (byte* p=this._items, pf=f)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(byte* pf, int lf, byte* pr, int lr)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(byte* pf, int lf, byte* pr, int lr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(byte* pf, int lf, byte* pr, int lr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(byte[] f, byte[] r)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (byte* p=this._items, pf=f)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(byte[] f, byte[] r, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (byte* p=this._items, pf=f)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(byte[] f, byte[] r, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (byte* p=this._items, pf=f)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(byte* pf, int lf, byte* pr, int lr)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(byte* pf, int lf, byte* pr, int lr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(byte* pf, int lf, byte* pr, int lr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (byte* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }



        #endregion

        #region NormalizeLength Bytes y Chars

        /// <summary>
        /// Rellena o contrae la coleccin de acuerdo al largo.
        /// </summary>
        /// <param name="length">Largo.</param>
        /// <param name="fill">Caracter Relleno.</param>
        /// <param name="fillLeft">Fill a la izquierda. La contraccin siempre es a la derecha.</param>
        public void NormalizeLength(int length, byte fill, bool fillLeft)
        {
            if (this._size!=length)
            {
                if (this._size>length)
                    this._size=length;
                else if (length<0)
                    throw new ArgumentOutOfRangeException("length",length,Errors.NEGATIVE);
                else if (this._size==0)
                    this.Append(fill,length);
                else if (!fillLeft)
                    this.Append(fill,length-this._size);
                else
                    this.Insert(0,fill,length-this._size);
            }
        }

        #endregion

        #region Propiedades y Constructores Bytes

        /// <summary>
        /// Inicializa la coleccin permitiendo especificar propiedades.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="unix">
        /// Estado inicial de la propiedad Unix.
        /// </param>
        /// <param name="codePage">
        /// Estado inicial de las Propiedades CodePage y, por lo tanto, IsLatin1 y SizeCP.
        /// </param>
        /// <param name="cap">Capacidad inicial.</param>
        public Bytes(int cap, bool appendExact, bool unix, CodePage codePage):this(cap)
        {
            codePage.ThrowIfUnSupported();
            this._flags=(1U<<9)+(uint)((codePage.Number<<16)+(codePage.MaxSize<<12));
            if (appendExact)
                this._flags+=1U<<8;
            if (unix)
                this._flags+=1U<<10;
        }

        /// <summary>
        /// Inicializa la coleccin con un array nulo, permitiendo especificar propiedades.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="unix">
        /// Estado inicial de la propiedad Unix.
        /// </param>
        /// <param name="codePage">
        /// Estado inicial de las Propiedades CodePage y, por lo tanto, IsLatin1 y SizeCP.
        /// </param>
        public Bytes(bool appendExact, bool unix, CodePage codePage)
        {
            codePage.ThrowIfUnSupported();
            this._flags=(1U<<9)+(uint)((codePage.Number<<16)+(codePage.MaxSize<<12));
            if (appendExact)
                this._flags+=1U<<8;
            if (unix)
                this._flags+=1U<<10;
        }

        /// <summary>
        /// Obtiene o establece la codificacin de la coleccin de Bytes.
        /// La codificacin afecta el comportamiento de las funciones Trim, RemoveQuotes, ReFormat, ToString,
        /// StrLen y ToChars.
        /// Las codificaciones multibyte solo son soportadas por las funciones ToString y ToChars. En todo caso,
        /// si se ejecuta una funcin de cadena sobre una codificacin multibyte, se genera excepcin.
        /// Desde Chars y SB, cuando se quiere exportar hacia una coleccin de Bytes, usar esta propiedad
        /// para la conversin. Las funciones de bsqueda, reemplazo y comparacin de cadenas
        /// estan excentas de esta limitacin dado que todas estas operaciones son ordinales.
        /// Al obtener, si no se ha establecido anteriormente, el valor ser establecido de acuerdo
        /// a la propiedad CodePage.AnsiCurrent.
        /// </summary>
        public CodePage CodePage
        {
            get
            {
                ushort cp=(ushort)(this._flags>>16);
                if (cp==0)
                {
                    CodePage codePage=Txt.CodePage.AnsiCurrent;
                    this._flags+=(uint)((codePage.Number<<16)+(codePage.MaxSize<<12));
                    return codePage;
                }
                return new CodePage(cp,(ushort)((this._flags&((1U<<12)+(1U<<13)+(1U<<14)+(1U<<15)))>>12));
            }
            set
            {
                value.ThrowIfUnSupported();
                this._flags=(this._flags&((1U<<12)-1))+(uint)((value.Number<<16)+(value.MaxSize<<12));
            }
        }

        /// <summary>
        /// Obtiene si la codificacin de la coleccin de Bytes es Latin1.
        /// Se puede establecer esta propiedad en true estableciendo la propiedad CodePage en CodePage.Latin1Ansi.
        /// La codificacin afecta el comportamiento de las funciones Trim, RemoveQuotes, ReFormat, ToString,
        /// StrLen y ToChars.
        /// Las codificaciones multibyte solo son soportadas por las funciones ToString y ToChars. En todo caso,
        /// si se ejecuta una funcin de cadena sobre una codificacin multibyte, se genera excepcin.
        /// Desde Chars y SB, cuando se quiere exportar hacia una coleccin de Bytes, usar esta propiedad
        /// para la conversin. Las funciones de bsqueda, reemplazo y comparacin de cadenas
        /// estan excentas de esta limitacin dado que todas estas operaciones son ordinales.
        /// </summary>
        public bool IsLatin1
        {
            get
            {
                return (ushort)(this._flags>>16)==Txt.CodePage.CP_Latin1Ansi;
            }
        }

        /// <summary>
        /// Obtiene la cantidad mxima de bytes por chars correspondiente a la codificacin actual.
        /// La codificacin afecta el comportamiento de las funciones Trim, RemoveQuotes, ReFormat, ToString,
        /// StrLen y ToChars.
        /// Las codificaciones multibyte solo son soportadas por las funciones ToString y ToChars. En todo caso,
        /// si se ejecuta una funcin de cadena sobre una codificacin multibyte, se genera excepcin.
        /// Desde Chars y SB, cuando se quiere exportar hacia una coleccin de Bytes, usar esta propiedad
        /// para la conversin. Las funciones de bsqueda, reemplazo y comparacin de cadenas
        /// estan excentas de esta limitacin dado que todas estas operaciones son ordinales.
        /// Al obtener, si no se ha establecido la codificacin anteriormente, el valor 
        /// de la codificacin ser establecido de acuerdo a la propiedad CodePage.AnsiCurrent.
        /// </summary>
        public int SizeCP
        {
            get
            {
                int cs=(int)((this._flags&((1U<<12)+(1U<<13)+(1U<<14)+(1U<<15)))>>12);
                if (cs==0)
                {
                    CodePage codePage=Txt.CodePage.AnsiCurrent;
                    int ret=codePage.MaxSize;
                    this._flags+=(uint)((codePage.Number<<16)+(ret<<12));
                    return ret;
                }
                return cs;
            }
        }

        /// <summary>
        /// Retorna true si el cdigo de pgina ya fue definido previamente.
        /// </summary>
        public bool IsDefinedCP
        {
            get
            {
                return (this._flags&((1U<<12)+(1U<<13)+(1U<<14)+(1U<<15)))!=0U;
            }
        }

        #endregion

        #region ToString, ToChars, ToSB y StrLen Bytes

        /// <summary>
        /// Obtiene largo de cadena en bytes segn la codificacin, el primer cero encontrado o la cantidad de elementos actual.
        /// </summary>
        /// <param name="setLength">Si true, actualizar el tamao actual de la coleccin con el resultado.</param>
        /// <returns>Primer cero encontrado de acuerdo a codificacin o cantidad de elementos actual.</returns>
        public int StrLen(bool setLength)
        {
            if (this._size==0)
                return 0;
            fixed(byte*p=this._items)
            {
                if (setLength)
                {
                    this._size=this.CodePage.StrLen(p,this._size);
                    return this._size;
                }
                return this.CodePage.StrLen(p,this._size);
            }
        }

        /// <summary>
        /// Genera cadena mutable con los elementos de esta coleccin usando la codificacin actual.
        /// </summary>
        /// <returns>Cadena mutable o vacia si la coleccin no contiene elementos.</returns>
        public override string ToString()
        {
            if (this._size==0)
                return Others.Empty;
            fixed(byte*p=this._items)
                return Mutables.Create(this.CodePage,p,this._size);
        }

        /// <summary>
        /// Genera cadena mutable con los elementos de esta coleccin usando la codificacin actual.
        /// </summary>
        /// <param name="count">Largo de la cadena.</param>
        /// <returns>Cadena mutable o vacia si count es cero.</returns>
        public string ToString(int count)
        {
            if (count<0 || count>this._size)
                throw new ArgumentOutOfRangeException("count", count,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            
            if (count==0)
                return Others.Empty;
            
            fixed(byte*p=this._items)
                return Mutables.Create(this.CodePage,p,count);
        }


        /// <summary>
        /// Genera cadena mutable con los elementos de esta coleccin usando la codificacin actual.
        /// </summary>
        /// <param name="startIndex">Inicio.</param>
        /// <param name="count">Largo de la cadena.</param>
        /// <returns>Cadena mutable o vacia si count es cero.</returns>
        public string ToString(int startIndex, int count)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            
            if (count==0)
                return Others.Empty;
            
            fixed(byte*p=this._items)
                return Mutables.Create(this.CodePage,p+startIndex,count);
        }

        /// <summary>
        /// Genera Chars con los elementos de esta coleccin usando la codificacin actual.
        /// </summary>
        /// <param name="extraCap">Capacidad extra a reservar en la coleccin.</param>
        /// <param name="appendExact">Estado inicial de la propiedad AppendExact del nuevo objeto.</param>
        /// <returns>Coleccin Chars.</returns>
        public Chars ToChars(int extraCap, bool appendExact)
        {
            return ToChars(0,this._size,extraCap,appendExact);
        }


        /// <summary>
        /// Genera Chars con los elementos de esta coleccin usando la codificacin actual.
        /// </summary>
        /// <param name="extraCap">Capacidad extra a reservar en la coleccin.</param>
        /// <param name="startIndex">Inicio.</param>
        /// <param name="count">Largo de la cadena.</param>
        /// <param name="appendExact">Estado inicial de la propiedad AppendExact del nuevo objeto.</param>
        /// <returns>Coleccin Chars.</returns>
        public Chars ToChars(int startIndex, int count, int extraCap, bool appendExact)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (extraCap<0)
                throw new ArgumentOutOfRangeException("extraCap",extraCap,Errors.NEGATIVE);
            CodePage codePage = this.CodePage;
            int maxChars=codePage.CalcMaxChars(count);
            if (maxChars==0)
                return new Chars(extraCap,appendExact,this.Unix);
            Chars ret=new Chars(Calc.AddIfOdd(extraCap+maxChars),appendExact,this.Unix);
            fixed(char*pd=ret.Array)
                fixed(byte*po=this._items)
                    ret.Length=codePage.BytesToChars(po+startIndex,count,pd,maxChars,false);
            return ret;
        }

        /// <summary>
        /// Genera SB con los elementos de esta coleccin usando la codificacin actual.
        /// </summary>
        /// <param name="extraCap">Capacidad extra a reservar en la coleccin.</param>
        /// <param name="appendExact">Estado inicial de la propiedad AppendExact del nuevo objeto.</param>
        /// <returns>Coleccin SB.</returns>
        public SB ToSB(int extraCap, bool appendExact)
        {
            return ToSB(0,this._size,extraCap,appendExact);
        }


        /// <summary>
        /// Genera SB con los elementos de esta coleccin usando la codificacin actual.
        /// </summary>
        /// <param name="extraCap">Capacidad extra a reservar en la coleccin.</param>
        /// <param name="startIndex">Inicio.</param>
        /// <param name="count">Largo de la cadena.</param>
        /// <param name="appendExact">Estado inicial de la propiedad AppendExact del nuevo objeto.</param>
        /// <returns>Coleccin SB.</returns>
        public SB ToSB(int startIndex, int count, int extraCap, bool appendExact)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (extraCap<0)
                throw new ArgumentOutOfRangeException("extraCap",extraCap,Errors.NEGATIVE);
            CodePage codePage = this.CodePage;
            int maxChars=codePage.CalcMaxChars(count);
            if (maxChars==0)
                return new SB(extraCap,appendExact,this.Unix);
            SB ret=new SB(Calc.AddIfOdd(extraCap+maxChars),appendExact,this.Unix);
            fixed(char*pd=ret.StringUnsafe)
                fixed(byte*po=this._items)
                    ret.Length=codePage.BytesToChars(po+startIndex,count,pd,maxChars,false);
            return ret;
        }


        #endregion

        #region Trim y RemoveQuotes Bytes

        /// <summary>
        /// Efectua trim de carcteres en blanco a derecha e izquierda.
        /// </summary>
        public void Trim()
        {
            Trim(true,true);
        }

        /// <summary>
        /// Efectua Trim con el filtro de todos los carcteres en blanco.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        public void Trim(bool left, bool right)
        {
            if (this._size!=0)
            {
                if (this.SizeCP!=1)
                    throw new NotSupportedException(Errors.COL_CP_MB);
                fixed(byte*p=this._items)
                {
                    PByte pb=new PByte(p,this._size,left,right,this.IsLatin1);
                    if (pb.Length!=this._size)
                    {
                        if (pb.Length!=0 && pb.Pointer!=p)
                            General.Copy(pb.Pointer,p,pb.Length);
                        this._size=pb.Length;
                    }
                }
            }
        }

        /// <summary>
        /// Efectua Trim.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        /// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">
        /// Incluye en filtro los carcteres en blanco Latin1 (160).
        /// Implica setear la propiedad IsLatin1 en true.
        /// </param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
        /// <param name="tab">Incluye en filtro el taburador (\t).</param>
        public void Trim(bool left, bool right, bool sp,bool lat1, bool tab, bool crlf, bool control)
        {
            if (this._size!=0)
            {
                if (this.SizeCP!=1)
                    throw new NotSupportedException(Errors.COL_CP_MB);
                if (lat1 && !this.IsLatin1)
                    this.CodePage=Txt.CodePage.Latin1Ansi;
                fixed(byte*p=this._items)
                {
                    PByte pb=new PByte(p,this._size,left,right,sp,lat1,tab,crlf,control);
                    if (pb.Length!=this._size)
                    {
                        if (pb.Length!=0 && pb.Pointer!=p)
                            General.Copy(pb.Pointer,p,pb.Length);
                        this._size=pb.Length;
                    }
                }
            }
        }

        /// <summary>
        /// Efectua Trim.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="pTrim">Puntero de carcteres de filtro.</param>
        /// <param name="lTrim">Largo del puntero de carcteres de filtro.</param>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        public void Trim(byte * pTrim, int lTrim, bool left, bool right)
        {
            if (this._size!=0)
            {
                if (this.SizeCP!=1)
                    throw new NotSupportedException(Errors.COL_CP_MB);
                fixed(byte*p=this._items)
                {
                    PByte pb=new PByte(p,this._size,pTrim,lTrim,left,right);
                    if (pb.Length!=this._size)
                    {
                        if (pb.Length!=0 && pb.Pointer!=p)
                            General.Copy(pb.Pointer,p,pb.Length);
                        this._size=pb.Length;
                    }
                }
            }
        }

        /// <summary>
        /// Efectua Trim.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        /// <param name="cTrim">Carcteres para Trim.</param>
        public void Trim(bool left, bool right, byte[] cTrim)
        {
            if (this._size!=0)
            {
                int lTrim=cTrim.Length;
                if (lTrim!=0)
                {
                    if (this.SizeCP!=1)
                        throw new NotSupportedException(Errors.COL_CP_MB);
                    fixed(byte*p=this._items, pTrim=cTrim)
                    {
                        PByte pb=new PByte(p,this._size,pTrim,lTrim,left,right);
                        if (pb.Length!=this._size)
                        {
                            if (pb.Length!=0 && pb.Pointer!=p)
                                General.Copy(pb.Pointer,p,pb.Length);
                            this._size=pb.Length;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Efectua un reconocimiento simple de texto encomillado (solo para codificaciones de un solo byte):
        /// - Efectua Trim al bloque.
        /// - Si encuentra comillas coincidentes al comienzo y al final del bloque resultante, las descarta.
        /// </summary>
        /// <param name="afterTrimR">Luego del procesamiento de comillas, efecta trim derecho.</param>
        /// <param name="afterTrimL">Luego del procesamiento de comillas, efecta trim izquierdo.</param>
        /// <param name="mode">Opciones de eliminacin de comillas.</param>
        public void RemoveQuotes(RemoveQuotesMode mode, bool afterTrimL, bool afterTrimR)
        {
            if (this._size!=0)
            {
                if (this.SizeCP!=1)
                    throw new NotSupportedException(Errors.COL_CP_MB);
                fixed(byte*p=this._items)
                {
                    PByte pc=new PByte(p,this._size,mode,afterTrimL,afterTrimR,this.IsLatin1);
                    if (pc.Length!=this._size)
                    {
                        if (pc.Length!=0 && pc.Pointer!=p)
                            General.Copy(pc.Pointer,p,pc.Length);
                        this._size=pc.Length;
                    }
                }
            }
        }


        #endregion

        #region AppendLine e InsertLine Bytes

        /// <summary>
        /// Inserta un salto de lnea al final de la coleccin.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        public void AppendLine()
        {
            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);
            if (!this.Unix)
                this.AppendItems(13,10);
            else
                this.Append(10);
        }

        /// <summary>
        /// Inserta un elemento seguido de un salto de lnea al final de la coleccin.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        public void AppendLine(byte item)
        {
            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);
            if (!this.Unix)
                this.AppendItems(item,13,10);
            else
                this.AppendItems(item,10);                
        }


        /// <summary>
        /// Inserta elementos seguido de un salto de lnea al final de la coleccin.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void AppendLine(byte item, int count)
        {
            if (count < 2)
            {
                if (count==1)
                {
                    this.AppendLine(item);
                    return;
                }
                if (count==0)
                {
                    this.AppendLine();
                    return;
                }
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);
            bool unix=this.Unix;
            this.EnsureAppend(count+(unix?1:2));
            fixed(byte*p=this._items)
            {
                General.Fill(p + this._size,count,item);
                this._size+=count;
                if (!unix)
                {
                    *(short*)(p+this._size)=Texts.BytesCrLf;
                    this._size+=2;
                }
                else
                    p[this._size++]=10;
            }
        }


        /// <summary>
        /// Inserta el contenido del array al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        public void AppendLine(byte[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.AppendLine(po, count);
            }
            else 
                this.AppendLine();
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void AppendLine(byte[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.AppendLine(po, count);
            }
            else
                this.AppendLine();
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void AppendLine(byte[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.AppendLine(po + arrIndex, count);
            }
            else
                this.AppendLine();
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin y agregando salto de lnea final.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void AppendLine(byte* arr, int count)
        {
            if (count < 2)
            {
                if (count==1)
                {
                    this.AppendLine(*arr);
                    return;
                }
                if (count==0)
                {
                    this.AppendLine();
                    return;
                }
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);
            bool unix=this.Unix;
            this.EnsureAppend(count+(unix?1:2));
            fixed(byte*p=this._items)
            {
                General.Copy(arr, p + this._size, count);
                this._size+=count;
                if (!unix)
                {
                    *(short*)(p+this._size)=Texts.BytesCrLf;
                    this._size+=2;
                }
                else
                    p[this._size++]=10;
            }
        }

        /// <summary>
        /// Inserta un salto de lnea en la ubicacin especificada.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex)
        {
            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);
            if (!this.Unix)
                this.InsertItems(startIndex,13,10);
            else
                this.Insert(startIndex, 10);
        }

        /// <summary>
        /// Inserta un elemento seguido de un salto de lnea en la ubicacin especificada.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex,byte item)
        {
            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);
            if (!this.Unix)
                this.InsertItems(startIndex,item,13,10);
            else
                this.InsertItems(startIndex,item,10);                
        }


        /// <summary>
        /// Inserta elementos seguido de un salto de lnea en la ubicacin especificada.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex,byte item, int count)
        {
            if (count < 2)
            {
                if (count==1)
                {
                    this.InsertLine(startIndex,item);
                    return;
                }
                if (count==0)
                {
                    this.InsertLine(startIndex);
                    return;
                }
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);
            if (!this.Unix)
            {
                this.Reserve(startIndex,count+2);
                this.Set(startIndex,item,count);
                this.SetItems(startIndex+count,13,10);

            }
            else
            {
                this.Reserve(startIndex,count+1);
                this.Set(startIndex,item,count);
                this.Set(startIndex+count,10);
            }
        }


        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, byte[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.InsertLine(startIndex,po, count);
            }
            else
                this.InsertLine(startIndex);
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, byte[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.InsertLine(startIndex,po, count);
            }
            else 
                this.InsertLine(startIndex);
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, byte[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (byte* po=arr)
                    this.InsertLine(startIndex,po + arrIndex, count);
            }
            else
                this.InsertLine(startIndex);
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada y agregando salto de lnea final.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, byte* arr, int count)
        {
            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);
            if (!this.Unix)
            {
                this.Reserve(startIndex,count+2);
                this.Set(startIndex,arr,count);
                this.SetItems(startIndex+count,13,10);
            }
            else
            {
                this.Reserve(startIndex,count+1);
                this.Set(startIndex,arr,count);
                this.Set(startIndex+count,10);
            }
        }

        #endregion

        #region ReFormat Bytes


        /// <summary>
        /// Reformatea en documento (versin mnima).
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="trimLeft">
        /// Efectua Trim a la izquierda de cada lnea.
        /// </param>
        /// <param name="trimRight">
        /// Efectua Trim a la derecha de cada lnea.
        /// </param>
        /// <param name="noBlankLines">
        /// Ignora lneas vacias.
        /// </param>
        public void ReFormat(bool trimLeft, bool trimRight, bool noBlankLines)
        {
            
            if (this._size==0)
                return;

            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);


            bool trim, parraf;
            if (noBlankLines && trimRight)
            {
                parraf=true;
                trimRight=false;
                trim=trimLeft;
            }
            else
            {
                parraf=false;
                trim=trimLeft || trimRight;
            }

            
            // genera copia de lectura en stack o heap.
            Heap heap;
            byte*p;
            bool lat1=this.IsLatin1;
            int l=this._size,sumline=(this.Unix?1:2),a;

            // Determina el largo e inserta las lneas en cache.
            fixed(byte*po=this._items)
            {
                if (&l>General.MinimumStack && l<General.MaxStackBytes<<3)
                {
                    byte*pstack=stackalloc byte[l];
                    p=pstack;
                    heap=null;
                }
                else
                {
                    heap=new Heap(l,false);
                    p=(byte*)heap.Pointer;
                }
                General.Copy(po,p,l);
            }
            
            this._size=0;
            PBytes Pclineas=new PBytes(l<16?4:(l>>4)+4);
            PByte pc=new PByte(p,l),pc2;
            l=0;

       otra:
            
            if (parraf)
               pc2=pc.FowardCrLf(noBlankLines);
            else
                pc2=pc.FowardParagraph(false,lat1);
            
            if (pc2.Length==0)
            {
                if (noBlankLines || pc.Pointer==null)
                    goto fin;
            }
            else if (trim)
            {
                pc2=pc2.Trim(trimLeft,trimRight,lat1);
                if (noBlankLines && pc2.Length==0)
                    goto otra;
            }

            Pclineas.Append(pc2);
            l+=pc2.Length+sumline;
            goto otra;
            fin:
            
                if (l!=0)
                {
                    this.Capacity=l;
                    l=Pclineas.Length;
                    a=0;
                    fixed(PByte* plin=Pclineas.Array)
                    {
                        do
                        {
                            pc2=plin[a];
                            if (pc2.Length==0)
                                this.AppendLine();
                            else
                                this.AppendLine(pc2.Pointer,pc2.Length);
                        }while(++a!=l);
                    }
                }
                if (heap!=null)
                    heap.Dispose();
        }



        /// <summary>
        /// Reformatea en documento (versin sin eliminacin de espacios duplicados ni carcteres basura).
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="trimLeft">
        /// Efectua Trim a la izquierda de cada lnea.
        /// Si RemoveQuotes es true, este trim se realizar luego de remover las comillas.
        /// </param>
        /// <param name="trimRight">
        /// Efectua Trim a la derecha de cada lnea.
        /// Si RemoveQuotes es true, este trim se realizar luego de remover las comillas.
        /// </param>
        /// <param name="rMode">
        /// Modo de eliminacin de comillas. Si la remocin implica trim, se efectuar este trim inicial a todas las lneas,
        /// siendo trimLeft y trimRight posteriores a una remocin exitosa.
        /// </param>
        /// <param name="noBlankLines">
        /// Ignora lneas vacias.
        /// </param>
        /// <param name="indent">Sangra aplicable a todas las lneas.
        ///  Si es cero no se agregar sangria. Si wrap es diferente a cero,
        ///  el largo mximo de cada lnea ser de indent+wrap.
        ///  </param>
        /// <param name="pIndent">
        /// Puntero para la sangra, ignorado si indent es cero.
        /// Se copiar al stack si es necesario.
        /// Si es nulo e indent es diferente a cero, se crear un puntero con espacios.
        /// </param>
        /// <param name="normLength">
        /// Normaliza el largo de cada lnea (sin incluir la sangra) rellenandla o contrayendla si su largo
        /// es menor o mayor al especificado.
        /// 0 para deshabilitar. Si es negativo, el relleno se efectuar a la izquierda, si es positivo el relleno se
        /// realizar a la derecha.
        /// </param>
        /// <param name="normFill">
        /// Caracter de relleno para normLength.
        /// </param>
        /// <param name="tabToSp">
        /// Convierte Tabs en espacios.
        /// </param>
        /// <param name="sBlToSp">
        /// Convierte carcteres en blanco superiores (Latin1) en el espacio estndar (32).
        /// Si este parametro es true, la propiedad IsLatin1 tambin se setea en true.
        /// </param>
        /// <param name="fillBlank">Aplica sangra y normalizacin de largo a las lneas en blanco.</param>
        public void ReFormat(int indent, byte* pIndent, int normLength, byte normFill, bool fillBlank,
            bool trimLeft, bool trimRight, bool noBlankLines, RemoveQuotesMode rMode,
            bool sBlToSp, bool tabToSp)
        {
            
            if (this._size==0)
                return;

            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);

            // Parmetros...
            if (indent!=0)
            {
                if (indent<0)
                    throw new ArgumentOutOfRangeException("indent",indent,Errors.NEGATIVE);
                if (pIndent==null || pIndent>General.MaximumStack)
                {
                    byte* pstack=stackalloc byte[indent];
                    if (pIndent==null)
                        General.Fill(pstack,indent,32);
                    else
                        General.Copy(pIndent,pstack,indent);
                    pIndent=pstack;
                }
            }

            bool normLeft;
            if (normLength<0)
            {
                normLength=-normLength;
                normLeft=true;
            }
            else
                normLeft=false;
            bool lat1;
            if (sBlToSp)
            {
                if (!this.IsLatin1)
                    this.CodePage=Txt.CodePage.Latin1Ansi;
                lat1=true;
            }
            else
                lat1=this.IsLatin1;

            if (fillBlank && normLength==0 && indent==0)
                fillBlank=false;

            bool trim=trimLeft || trimRight || rMode!=RemoveQuotesMode.NoRemove;
            
            // genera copia de lectura en stack o heap.
            Heap heap;
            byte*p;
            int l=this._size,sumline=(this.Unix?1:2)+indent+normLength,a;

            // Determina el largo e inserta las lneas en cache.
            fixed(byte*po=this._items)
            {
                if (&l>General.MinimumStack && l<General.MaxStackBytes<<3)
                {
                    byte*pstack=stackalloc byte[l];
                    p=pstack;
                    heap=null;
                }
                else
                {
                    heap=new Heap(l,false);
                    p=(byte*)heap.Pointer;
                }
                General.Copy(po,p,l);
            }
            
            this._size=0;
            PBytes Pclineas=new PBytes(l<16?4:(l>>4)+4);
            PByte pc=new PByte(p,l),pc2;
            l=0;

            otra:
                //
                pc2=pc.FowardCrLf(noBlankLines);
            if (pc2.Length==0)
            {
                if (noBlankLines || pc.Pointer==null)
                    goto fin;
            }
            else if (trim)
            {
                pc2=new PByte(pc2.Pointer,pc2.Length,rMode,trimLeft,trimRight,lat1);
                if (noBlankLines && pc2.Length==0)
                    goto otra;
            }

            if (pc2.Length!=0)
            {
                if (pc2.Length>normLength)
                    pc2.Length=normLength;
                a=0;
                do
                {
                    switch(pc.Pointer[a])
                    {
                        case 0:
                        case 11:
                        case 12:
                            pc.Pointer[a]=32;
                            continue;
                        case 9:
                            if (tabToSp)
                                pc.Pointer[a]=32;
                            continue;
                        case 160:
                            if (sBlToSp)
                                pc.Pointer[a]=32;
                            continue;
                    }
                }while(++a!=pc2.Length);
            }
            Pclineas.Append(pc2);
            l+=sumline;
            if (normLength==0)
                l+=pc2.Length;
            goto otra;
            fin:
            
                if (l!=0)
                {
                    this.Capacity=l;
                    l=Pclineas.Length;
                    a=0;
                    fixed(PByte* plin=Pclineas.Array)
                    {
                        do
                        {
                            pc2=plin[a];
                            if (pc2.Length==0)
                            {
                                if (fillBlank)
                                {
                                    if (indent!=0)
                                        this.Append(pIndent,indent);
                                    if (normLength!=0)
                                        this.Append(normFill,normLength);
                                }
                                this.AppendLine();
                                continue;
                            }
                            if (indent!=0)
                            {
                                this.Append(pIndent,indent);
                            }
                            if (normLength!=0 && pc2.Length!=normLength)
                            {
                                if (normLeft)
                                {
                                    this.Append(normFill,normLength-pc2.Length);
                                    this.Append(pc2.Pointer,pc2.Length);
                                }
                                else
                                {
                                    this.Append(pc2.Pointer,pc2.Length);
                                    this.Append(normFill,normLength-pc2.Length);
                                }
                                this.AppendLine();
                            }
                            else
                                this.AppendLine(pc2.Pointer,pc2.Length);

                        }while(++a!=l);
                    
                    }
                }
            if (heap!=null)
                heap.Dispose();
        }


        /// <summary>
        /// Reformatea en documento (orientado a archivos de datos).
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="trimLeft">
        /// Efectua Trim a la izquierda de cada lnea.
        /// Si RemoveQuotes es true, este trim se realizar luego de remover las comillas.
        /// </param>
        /// <param name="trimRight">
        /// Efectua Trim a la derecha de cada lnea.
        /// Si RemoveQuotes es true, este trim se realizar luego de remover las comillas.
        /// </param>
        /// <param name="rMode">
        /// Modo de eliminacin de comillas. Si la remocin implica trim, se efectuar este trim inicial a todas las lneas,
        /// siendo trimLeft y trimRight posteriores a una remocin exitosa.
        /// </param>
        /// <param name="noBlankLines">
        /// Ignora lneas vacias.
        /// </param>
        /// <param name="indent">Sangra aplicable a todas las lneas.
        ///  Si es cero no se agregar sangria. Si wrap es diferente a cero,
        ///  el largo mximo de cada lnea ser de indent+wrap.
        ///  </param>
        /// <param name="pIndent">
        /// Puntero para la sangra, ignorado si indent es cero.
        /// Se copiar al stack si es necesario.
        /// Si es nulo e indent es diferente a cero, se crear un puntero con espacios.
        /// </param>
        /// <param name="normLength">
        /// Normaliza el largo de cada lnea (sin incluir la sangra) rellenandla o contrayendla si su largo
        /// es menor o mayor al especificado.
        /// 0 para deshabilitar. Si es negativo, el relleno se efectuar a la izquierda, si es positivo el relleno se
        /// realizar a la derecha.
        /// </param>
        /// <param name="normFill">
        /// Caracter de relleno para normLength.
        /// </param>
        /// <param name="noDupSp">
        /// Elimina espacios duplicados y la aparicin de espacios junto a Tabs.
        /// </param>
        /// <param name="tabToSp">
        /// Convierte Tabs en espacios.
        /// </param>
        /// <param name="sBlToSp">
        /// Convierte carcteres en blanco superiores (Latin1) en el espacio estndar (32).
        /// Si este parametro es true, la propiedad IsLatin1 tambin se setea en true.
        /// </param>
        /// <param name="fillBlank">Aplica sangra y normalizacin de largo a las lneas en blanco.</param>
        public void ReFormat(int indent, byte* pIndent, int normLength, byte normFill, bool fillBlank,
            bool trimLeft, bool trimRight, bool noBlankLines, RemoveQuotesMode rMode,
            bool sBlToSp, bool tabToSp, bool noDupSp)
        {
            
            if (this._size==0)
                return;

            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);


            // Parmetros...
            if (indent!=0)
            {
                if (indent<0)
                    throw new ArgumentOutOfRangeException("indent",indent,Errors.NEGATIVE);
                if (pIndent==null || pIndent>General.MaximumStack)
                {
                    byte* pstack=stackalloc byte[indent];
                    if (pIndent==null)
                        General.Fill(pstack,indent,32);
                    else
                        General.Copy(pIndent,pstack,indent);
                    pIndent=pstack;
                }
            }

            bool normLeft;
            if (normLength<0)
            {
                normLength=-normLength;
                normLeft=true;
            }
            else
                normLeft=false;

            bool lat1;
            if (sBlToSp)
            {
                if (!this.IsLatin1)
                    this.CodePage=Txt.CodePage.Latin1Ansi;
                lat1=true;
            }
            else
                lat1=this.IsLatin1;

            if (fillBlank && normLength==0 && indent==0)
                fillBlank=false;

            bool trim=trimLeft || trimRight || rMode!=RemoveQuotesMode.NoRemove,antBl;
            
            // genera copia de lectura en stack o heap.
            Heap heap;
            byte*p;
            int l=this._size,sumline=(this.Unix?1:2)+indent+normLength,a,b;

            // Determina el largo e inserta las lneas en cache.
            fixed(byte*po=this._items)
            {
                if (&l>General.MinimumStack && l<General.MaxStackBytes<<3)
                {
                    byte*pstack=stackalloc byte[l];
                    p=pstack;
                    heap=null;
                }
                else
                {
                    heap=new Heap(l,false);
                    p=(byte*)heap.Pointer;
                }
                General.Copy(po,p,l);
            }
            
            this._size=0;
            PBytes Pclineas=new PBytes(l<16?4:(l>>4)+4);
            PByte pc=new PByte(p,l),pc2;
            l=0;

       otra:
            //
            pc2=pc.FowardCrLf(noBlankLines);
            if (pc2.Length==0)
            {
                if (noBlankLines || pc.Pointer==null)
                    goto fin;
            }
            else if (trim)
            {
                pc2=new PByte(pc2.Pointer,pc2.Length,rMode,trimLeft,trimRight,lat1);
                if (noBlankLines && pc2.Length==0)
                    goto otra;
            }
            b=0;
            if (pc2.Length!=0)
            {
                a=0;
                antBl=false;
                do
                {
                    switch(pc.Pointer[a])
                    {
                        case 32:
                            if (!noDupSp || !antBl)
                            {
                                pc.Pointer[b++]=32;
                                antBl=true;
                            }
                            continue;
                        case 9:
                            if (tabToSp)
                                goto case 32;
                            if (!noDupSp || !antBl)
                            {
                                pc.Pointer[b++]=9;
                                antBl=true;
                            }
                            continue;
                        case 160:
                            if (sBlToSp)
                                goto case 32;
                            if (!lat1)
                                goto default;
                            if (!noDupSp || !antBl)
                            {
                                pc.Pointer[b++]=pc.Pointer[a];
                                antBl=true;
                            }
                            continue;
                        case 0:
                        case 11:
                        case 12:
                            continue;
                        default:
                            if (antBl)
                                antBl=false;
                            pc.Pointer[b++]=pc.Pointer[a];
                            continue;
                    }
                }while(++a!=pc2.Length);
                if (b==0 && noBlankLines)
                {
                    goto otra;
                }
                if (b>normLength)
                    b=normLength;
                pc2.Length=b;
            }
            Pclineas.Append(pc2);
            l+=sumline;
            if (normLength==0)
                l+=b;
            goto otra;
        fin:
            
            if (l!=0)
            {
                this.Capacity=l;
                l=Pclineas.Length;
                a=0;
                fixed(PByte* plin=Pclineas.Array)
                {
                    do
                    {
                        pc2=plin[a];
                        if (pc2.Length==0)
                        {
                            if (fillBlank)
                            {
                                if (indent!=0)
                                    this.Append(pIndent,indent);
                                if (normLength!=0)
                                    this.Append(normFill,normLength);
                            }
                            this.AppendLine();
                            continue;
                        }
                        if (indent!=0)
                        {
                            this.Append(pIndent,indent);
                        }
                        if (normLength!=0 && pc2.Length!=normLength)
                        {
                            if (normLeft)
                            {
                                this.Append(normFill,normLength-pc2.Length);
                                this.Append(pc2.Pointer,pc2.Length);
                            }
                            else
                            {
                                this.Append(pc2.Pointer,pc2.Length);
                                this.Append(normFill,normLength-pc2.Length);
                            }
                            this.AppendLine();
                        }
                        else
                            this.AppendLine(pc2.Pointer,pc2.Length);

                    }while(++a!=l);
                    
                }
            }
            if (heap!=null)
                heap.Dispose();
        }


        /// <summary>
        /// Reformatea documento de texto.
        /// Solo para codificaciones de 1 byte.
        /// </summary>
        /// <param name="par">Opciones de formato de parrafo.</param>
        /// <param name="wrap">
        /// Mximo de carcteres por lnea: Mnimo 8, 0 para deshabilitar.
        /// La divisin de lneas se efectua de acuerdo a estas reglas:
        /// - No hace nada si el largo de lnea es menor o igual a wrap, o wrap es 0.
        /// - El proceso de wrap se realiza al finalizar todas las otras depuraciones.
        /// - Buscar el ltimo caracter en blanco dentro del rango.
        ///   El rango es variable y estar entre el 50% de wrap y wrap.
        /// - Si esto no resulta, la lnea se dividir arbitrariamente (usndo guionado).
        /// </param>
        /// <param name="trimLeft">
        /// Efectua Trim a la izquierda de cada lnea.
        /// Independientemente a este parametro,
        /// siempre har trim a la derecha.
        /// Por otra parte, aunque el parametro sea false har trim
        /// de todos modos si la cantidad de carcteres en blanco a
        /// la izquierda supera o iguala al rango de wrap
        /// (en caso de que wrap sea diferente a 0).
        /// </param>
        /// <param name="noDupSp">
        /// Elimina espacios duplicados y la aparicin de espacios junto a Tabs.
        /// </param>
        /// <param name="tabToSp">
        /// Convierte Tabs en espacios.
        /// </param>
        /// <param name="indent">Sangra aplicable a todas las lneas.
        ///  Si es cero no se agregar sangria. Si wrap es diferente a cero,
        ///  el largo mximo de cada lnea ser de indent+wrap.
        ///  </param>
        /// <param name="pIndent">
        /// Puntero para la sangra, ignorado si indent es cero.
        /// Se copiar al stack si es necesario.
        /// Si es nulo e indent es diferente a cero, se crear un puntero con espacios.
        /// </param>
        /// <param name="sBlToSp">
        /// Convierte carcteres en blanco superiores (Latin1) en el espacio estndar (32).
        /// Si este parametro es true, los trims a la izquierda y derecha incluirn en filtro a los carcteres Latin1
        /// y la propiedad IsLatin1 se setea en true.
        /// </param>
        public void ReFormat(ParagraphFmt par, int wrap, int indent, byte* pIndent, bool trimLeft, bool noDupSp, bool tabToSp, bool sBlToSp)
        {

            if (this._size==0)
                return;
            
            if (this.SizeCP!=1)
                throw new NotSupportedException(Errors.COL_CP_MB);


            // Parmetros...
            if (indent!=0)
            {
                if (indent<0)
                    throw new ArgumentOutOfRangeException("indent",indent,Errors.NEGATIVE);
                if (pIndent==null || pIndent>General.MaximumStack)
                {
                    byte* pstack=stackalloc byte[indent];
                    if (pIndent==null)
                        General.Fill(pstack,indent,32);
                    else
                        General.Copy(pIndent,pstack,indent);
                    pIndent=pstack;
                }
            }
                       
            int medioWrap;
            if(wrap==0)
                medioWrap=0;
            else if (wrap<8) 
                throw new ArgumentOutOfRangeException("wrap",wrap,Errors.COL_WRAP);
            else if (wrap>39) 
                medioWrap=wrap-12;
            else 
                medioWrap=wrap-(wrap>>2)-2;

            if (sBlToSp && !this.IsLatin1)
                this.CodePage=Txt.CodePage.Latin1Ansi;

            // genera copia de lectura en stack o heap.
            Heap heap;
            byte*p;
            int l=this._size,sumline=(this.Unix?1:2)+indent;

            // Determina el largo e inserta las lneas en cache.
            fixed(byte*po=this._items)
            {
                if (&l>General.MinimumStack && l<General.MaxStackBytes<<3)
                {
                    byte*pstack=stackalloc byte[l];
                    p=pstack;
                    heap=null;
                }
                else
                {
                    heap=new Heap(l,false);
                    p=(byte*)heap.Pointer;
                }
                General.Copy(po,p,l);
            }
            
            this._size=0;
            PBytes Pclineas=new PBytes(l<16?4:(l>>4)+4);
            PByte pc=new PByte(p,l),pc2,pc3;
            l=0;
            
            switch(par)
            {
                case ParagraphFmt.NoEmptysSingle:
                    while((pc2=pc.FowardParagraph(false,sBlToSp)).Length!=0)
                    {
                        Pclineas.Append(pc2);
                        l+=pc2.Length+sumline;
                        if (wrap!=0 && pc2.Length>wrap)
                            l+=(pc2.Length/medioWrap)*(2+sumline);
                    }
                    break;
                case ParagraphFmt.NoEmptysDouble:
                    while((pc2=pc.FowardParagraph(false,sBlToSp)).Length!=0)
                    {
                        Pclineas.Append(pc2);
                        Pclineas.AppendBlank();
                        l+=pc2.Length+sumline+2;
                        if (wrap!=0 && pc2.Length>wrap)
                            l+=(pc2.Length/medioWrap)*(2+sumline);

                    }
                    break;
                case ParagraphFmt.ParagraphSingle:
                    while((pc2=pc.FowardParagraph(true,sBlToSp)).Length!=0)
                    {
                        Pclineas.Append(pc2);
                        l+=pc2.Length+sumline;
                        if (wrap!=0 && pc2.Length>wrap)
                            l+=(pc2.Length/medioWrap)*(2+sumline);

                    }
                    break;
                case ParagraphFmt.ParagraphDouble:
                    while((pc2=pc.FowardParagraph(true,sBlToSp)).Length!=0)
                    {
                        Pclineas.Append(pc2);
                        Pclineas.AppendBlank();
                        l+=pc2.Length+sumline+2;
                        if (wrap!=0 && pc2.Length>wrap)
                            l+=(pc2.Length/medioWrap)*(2+sumline);

                    }
                    break;

                default:
                    while((pc2=pc.FowardParagraph(true,sBlToSp)).Length!=0)
                    {
                        while((pc3=pc2.FowardParagraph(false,sBlToSp)).Length!=0)
                        {
                            Pclineas.Append(pc3);
                            l+=pc3.Length+sumline;
                            if (wrap!=0 && pc3.Length>wrap)
                                l+=(pc3.Length/medioWrap)*(2+sumline);
                        }
                        Pclineas.AppendBlank();
                        l+=2;
                    }
                    break;
            }
            int total=Pclineas.Length;
            if (total==0)
            {
                if (heap!=null)
                    heap.Dispose();
                return;
            }
            this.Capacity=l+32;
            
            if (par!=ParagraphFmt.NoEmptysSingle && par!=ParagraphFmt.ParagraphSingle)
                --total;
            l=0;
            int a,b,comTabs,pText;
            bool antBl;
            
            fixed(PByte* pl=Pclineas.Array)
            {
                do
                {
                    pc=pl[l];
                    if(pc.Length==0)
                    {
                        this.AppendLine();
                        continue;
                    }
                    a=0;
                    b=0;
                    pText=-1;
                    comTabs=0;
                    antBl=true;
                    
                    do
                    {

                        switch(pc.Pointer[a])
                        {
                            case 32:
                                if (pText==-1)
                                {
                                    if (!trimLeft)
                                        pc.Pointer[b++]=32;
                                }
                                else if (!noDupSp || !antBl)
                                {
                                    pc.Pointer[b++]=32;
                                    antBl=true;
                                }
                                continue;
                            case 9:
                                if (tabToSp)
                                    goto case 32;
                                if (pText==-1)
                                {
                                    if (!trimLeft && comTabs!=4)
                                    {
                                        pc.Pointer[b++]=9;
                                        comTabs++;
                                    }
                                }
                                else if (!noDupSp || !antBl)
                                {
                                    pc.Pointer[b++]=9;
                                    antBl=true;                                      
                                }
                                continue;
                            case 13:
                            case 10:
                                if (!antBl)
                                {
                                    pc.Pointer[b++]=32;
                                    antBl=true;
                                }
                                pc3=new PByte(pc.Pointer+ ++a,pc.Length-a,true,false,sBlToSp);
                                a=(int)(pc3.Pointer-pc.Pointer);
                                continue;
                            case 160:
                                if (sBlToSp)
                                    goto case 32;
                                goto default;
                            case 0:
                            case 11:
                            case 12:
                                continue;
                            default:
                                if (pText==-1) 
                                    pText=b;
                                if (antBl)
                                    antBl=false;
                                pc.Pointer[b++]=pc.Pointer[a];
                                continue;
                        }
                    }while(++a!=pc.Length);

                    if (wrap!=0)
                    {
                        if (pText>=medioWrap) 
                        {
                            b-=pText;
                            pc.Pointer+=pText;
                            pText=0;
                        }
                        if (b>wrap)
                        {
                            this.wrap(pc.Pointer,b,medioWrap,wrap,pText,pIndent,indent);
                            continue;
                        }
                    }
                    if (indent!=0)
                        this.Append(pIndent,indent);
                    this.AppendLine(pc.Pointer,b);
                }while(++l!=total);
            }
            if (heap!=null)
                heap.Dispose();
        }


        private void wrap(byte* pc, int l, int medioWrap,int wrap,int pText,byte*pi,int li)
        {
            PByte ptrim,ptrim2;
            byte* p;
            if (pText!=0)
            {
                p=pc+pText;
                l-=pText;
                wrap-=pText;
                medioWrap-=pText;
            }
            else
                p=pc;
            int puntoWrap, ubiGuion=wrap-2;
            do
            {
            
                puntoWrap=wrap;
            while(--puntoWrap>=medioWrap && p[puntoWrap]!=32 && p[puntoWrap]!=9);
                if(puntoWrap<medioWrap)
                {
                    if (li!=0)
                        this.Append(pi,li);
                    if (pText!=0)
                        this.Append(pc,pText);
                    this.Append(p,ubiGuion);
                    if (!this.Unix)
                        this.AppendItems(32,0x2D,13,10);
                    else
                        this.AppendItems(32,0x2D,10);
                    l-=ubiGuion;
                    p+=ubiGuion;
                }
                else
                {
                    ptrim=new PByte(p,puntoWrap,false,true,false);
                    if (ptrim.Length<medioWrap)
                    {
                        p[ptrim.Length++]=32;
                        ptrim2=new PByte(p+(++puntoWrap),l-puntoWrap,puntoWrap==wrap-1,false,false);
                        General.Copy(ptrim2.Pointer,p+ptrim.Length,ptrim2.Length);
                        l=ptrim2.Length+ptrim.Length;
                    }
                    else
                    {
                        if (li!=0)
                            this.Append(pi,li);
                        if (pText!=0)
                            this.Append(pc,pText);
                        this.AppendLine(ptrim.Pointer,ptrim.Length);
                        p+=++puntoWrap;
                        l-=puntoWrap;
                    }
                }
                if (l<=wrap)
                {
                    if (l!=0)
                    {
                        if (li!=0)
                            this.Append(pi,li);
                        if (pText!=0)
                            this.Append(pc,pText);
                        this.AppendLine(p,l);
                    }
                    break;
                }

            }while(true);
        }

        #endregion

        #region Funciones con Streams exclusivas de Bytes

        private void detectCP(System.IO.Stream stream)
        {
            int sign;
            CodePage cp;
            if (!this.IsDefinedCP)
                this.CodePage=new CodePage(stream,false,Txt.CodePage.AnsiCurrent,out sign);
            else if (this._size==0)
            {
                cp=new CodePage(stream,false,this.CodePage,out sign);
                if (sign!=0)
                    this.CodePage=cp;
            }
            else
            {
                CodePage act=this.CodePage;
                cp=new CodePage(stream,false,act,out sign);
                if (cp!=act)
                    throw new InvalidOperationException(Errors.COL_CP_INCOMP);
            }
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream en forma directa y sin decodificar.
        /// Los elementos son insertados al final de la coleccin.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="maxCount">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <param name="detectSign">
        /// Si el stream est en posicin cero intenta detectar el cdigo de pgina a partir de la firma.
        /// Esto es de utilidad para no cargar la firma desde el stream,
        /// actualizar el cdigo de pgina del objeto (si el tamao actual es cero) y, en caso de que el
        /// CP del archivo sea diferente al definido actualmente, lanzar una excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int Append(System.IO.Stream stream, int maxCount, bool close, bool detectSign)
        {
            if (detectSign)
                this.detectCP(stream);
            maxCount=Serializer.CalcStreamRead(stream,maxCount);
            if (maxCount!=0)
            {
                this.EnsureAppend(maxCount);
                maxCount=Serializer.StreamRead(stream,close,this._items,this._size,maxCount);
                this._size+=maxCount;
            }
            else if (close)
                stream.Close();
            return maxCount;
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="maxCount">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <param name="detectSign">
        /// Si el stream est en posicin cero intenta detectar el cdigo de pgina a partir de la firma.
        /// Esto es de utilidad para no cargar la firma desde el stream,
        /// actualizar el cdigo de pgina del objeto (si el tamao actual es cero) y, en caso de que el
        /// CP del archivo sea diferente al definido actualmente, lanzar una excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int Insert(int startIndex, System.IO.Stream stream, int maxCount, bool close, bool detectSign)
        {
            if (startIndex==this._size)
                return Append(stream,maxCount,close,detectSign);
            if (startIndex<0 || startIndex>this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (detectSign)
                this.detectCP(stream);
            
            maxCount=Serializer.CalcStreamRead(stream,maxCount);
            if (maxCount!=0)
            {
                this.Reserve(startIndex,maxCount);
                int ret=Serializer.StreamRead(stream,close,this._items,startIndex,maxCount);
                if (ret!=maxCount)
                    this.RemoveRange(startIndex+ret,maxCount-ret);
                return ret;
            }
            else if (close)
                stream.Close();
            return 0;
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream en forma directa y sin decodificar.
        /// Los elementos son insertados en la ubicacin especificada eliminando todos los elementos
        /// existentes a partir de esta ubicacin.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="maxLR">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <param name="detectSign">
        /// Si el stream est en posicin cero intenta detectar el cdigo de pgina a partir de la firma.
        /// Esto es de utilidad para no cargar la firma desde el stream,
        /// actualizar el cdigo de pgina del objeto (si el tamao actual es cero) y, en caso de que el
        /// CP del archivo sea diferente al definido actualmente, lanzar una excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int ReplaceRange( int startIndex, System.IO.Stream stream, int maxLR, bool close, bool detectSign)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
            return this.Append(stream,maxLR,close,detectSign);
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream en forma directa y sin decodificar.
        /// Los elementos copiados reemplazarn al rea comprendida entre startIndex y count.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="count">Area a reemplazar con el contenido del Stream.</param>
        /// <param name="maxLR">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <param name="detectSign">
        /// Si el stream est en posicin cero intenta detectar el cdigo de pgina a partir de la firma.
        /// Esto es de utilidad para no cargar la firma desde el stream,
        /// actualizar el cdigo de pgina del objeto (si el tamao actual es cero) y, en caso de que el
        /// CP del archivo sea diferente al definido actualmente, lanzar una excepcin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int ReplaceRange( int startIndex, int count, System.IO.Stream stream, int maxLR, bool close, bool detectSign)
        {
            if (count<1)
            {
                if (count==0)
                    return Insert(startIndex,stream,maxLR,close,detectSign);
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            if (startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE);
            
            int l=startIndex+count;
            if (l>=this._size)
            {
                if (l==this._size)
                    return ReplaceRange(startIndex,stream,maxLR,close,detectSign);
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.SUPERIOR_ARRAY);
            }

            if (detectSign)
                this.detectCP(stream);

            maxLR=Serializer.CalcStreamRead(stream,maxLR);
            
            if (maxLR<=count)
            {
                if (maxLR!=0)
                {
                    maxLR=Serializer.StreamRead(stream,close,this._items,startIndex,maxLR);
                }
                else if (close)
                    stream.Close();
                if (maxLR<count)
                    this.RemoveRange(startIndex+maxLR,count-maxLR);
                return maxLR;
            }
            this.Reserve(startIndex+count,maxLR-count);
            l=Serializer.StreamRead(stream,close,this._items,startIndex,maxLR);
            if (l!=maxLR)
                this.RemoveRange(startIndex+l,maxLR-l);
            return l;
        }

        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de un Stream en forma directa y sin codificar.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="close">Cierra el Stream luego de la escritura.</param>
        /// <param name="signCP">
        /// Si true y el stream se encuentra en posicin cero, la cantidad de bytes
        /// a escribr es mayor a cero y el cdigo de pgina actual est definido
        /// y lo requiere, firmar el stream.
        /// </param>
        /// <returns>Bytes escritos, incluyendo firma.</returns>
        public int StreamWrite(System.IO.Stream stream, bool close, bool signCP)
        {
            return StreamWrite(stream,0,this._size,close,signCP);
        }


        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de un Stream en forma directa y sin codificar.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">Indice de comienzo de la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="close">Cierra el Stream luego de la escritura.</param>
        /// <param name="signCP">
        /// Si true y el stream se encuentra en posicin cero, la cantidad de bytes
        /// a escribr es mayor a cero y el cdigo de pgina actual est definido
        /// y lo requiere, firmar el stream.
        /// </param>
        /// <returns>Bytes escritos, incluyendo firma.</returns>
        public int StreamWrite(System.IO.Stream stream, int startIndex, int count, bool close, bool signCP)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                if (signCP && this.IsDefinedCP)
                {
                    int l=this.CodePage.SignCP(stream,false);
                    stream.Write(this._items,startIndex,count);
                    count+=l;
                }
                else
                    stream.Write(this._items,startIndex,count);

            }
            if (close)
                stream.Close();
            return count;
        }


        #endregion

    }
}
