using System;
using EnPalermo.Mem;
using EnPalermo.Uti;
using EnPalermo.Lit;
using EnPalermo.Arr;
using EnPalermo.Txt;

namespace EnPalermo.Col
{
    /// <summary>
    /// Coleccin de tipos de valor basada en array.
    /// A diferencia de las colecciones estndar, permite el acceso directo al array subyacente y es de mejor
    /// rendimiento dado que no usa boxing/unboxing ni es afectado por las limitaciones relativas al uso de
    /// genricos, como la imposibilidad de usar cdigo inseguro.
    /// Nota: Esta coleccin no es segura en tamaos equivalentes o mayores a (Int32.MaxValue/(2*sizeof(tipo)):
    /// Esto es: 1073741823 bytes, 536870911 shorts o chars, 268435455 ints o 134217727 longs.
    /// </summary>
    public unsafe sealed class Chars
    {
        #region Campos

#pragma warning disable 429
        /// <summary>
        /// Capacidad por defecto para esta coleccin.
        /// </summary>
        public const int DefaultCapacity=((16 / sizeof(char)) !=8 && (16 / sizeof(char)) !=16) ? (4) : (16 / sizeof(char));

        /// <summary>
        /// Sumatoria por defecto para algorritmo de aumento de capacidad.
        /// </summary>
        private const int sumCap=DefaultCapacity>=8?4:2;
#pragma warning restore 429
        /// <summary>
        /// Largo actual del array.
        /// </summary>
        private int _size;

        /// <summary>
        /// Capacidad Actual. Si es cero el array subyacente podr ser null.
        /// </summary>
        private int _cap;


        /// <summary>
        /// Flags para almacenar informacin variada acerca del objeto.
        /// Definidas: 
        /// - Bytes  0 al 7: Tolerancia para Compact (SB). De 0 a 255.
        /// - Byte   8: AppendExact.
        /// - Byte   9: Unix definido (Bytes, Chars y SB).
        /// - Byte  10: Unix (Bytes, Chars y SB).
        /// - Bytes 11 y 12: Tabla MayMin (Chars y SB).
        /// - Bytes 12, 13, 14 y 15: Cantidad mxima de bytes por char segn cdigo de pgina (Bytes).
        /// - Bytes 16 al 31: Cdigo de pgina (Bytes).
        /// </summary>
        private uint _flags;

        /// <summary>
        /// Array subyacente.
        /// </summary>
        private char[] _items;

        #endregion

        #region Constructores

        /// <summary>
        /// Inicializa la coleccin con un array nulo.
        /// </summary>
        public Chars()
        {}

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada y largo 0.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        public Chars(int cap)
        {
            if (cap!=0)
            {
                this._items=new char[cap];
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin con un array nulo, permitiendo especificar la propiedad AppendExact.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        public Chars(bool appendExact)
        {
            if (appendExact)
                this._flags=1U<<8;
        }

        /// <summary>
        /// Inicializa la coleccin permitiendo especificar la propiedad AppendExact.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="cap">Capacidad inicial.</param>
        public Chars(bool appendExact, int cap):this(cap)
        {
            if (appendExact)
                this._flags=1U<<8;
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="count">Cantidad inicial de elementos.</param>
        public Chars(int cap, int count)
        {
            if (count<0 || count>cap)
                throw new ArgumentOutOfRangeException("cap,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (cap!=0)
            {
                this._items=new char[cap];
                this._size=count;
                this._cap=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin generando un array con la capacidad inicial especificada.
        /// </summary>
        /// <param name="cap">Capacidad inicial.</param>
        /// <param name="countCap">La cantidad inicial de elementos ser igual a la capacidad.</param>
        public Chars(int cap, bool countCap)
        {
            if (cap!=0)
            {
                this._items=new char[cap];
                this._cap=cap;
                if (countCap)
                    this._size=cap;
            }
        }

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece a la capacidad del array.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        public Chars(char[] arr, bool asRef):this(arr,asRef,arr==null?0:arr.Length){}

        /// <summary>
        /// Inicializa la coleccin usando directamente el objeto array de entrada (sin clonar) 
        /// o copindolo. En ambos casos, el largo inicial se establece de acuerdo a iLength.
        /// </summary>
        /// <param name="arr">Array inicial (puede ser nulo).</param>
        /// <param name="asRef">
        /// Si true, intentar establecer el array de entrada como subyacente, sin clonar.
        /// Si false, copia el contenido del array de entrada.
        /// </param>
        /// <param name="iLength">
        /// Largo inicial (no puede ser negativo ni superar el largo del array).
        /// </param>
        public Chars(char[] arr, bool asRef, int iLength)
        {
            if (iLength<1)
            {
                if (iLength==0)
                {
                    if (asRef && arr!=null)
                    {
                        this._items=arr;
                        this._cap=arr.Length;
                    }
                    return;
                }
            }
            else if (arr!=null)
            {
                if (asRef)
                {
                    this._cap=arr.Length;
                    if (iLength<=this._cap)
                    {
                        this._items=arr;
                        this._size=iLength;
                        return;
                    }
                }
                else
                {
                    this._cap=Calc.MultMajor(iLength+sumCap,DefaultCapacity,false);
                    this._size=iLength;
                    this._items=ArrGeneral.Clone(arr,0,iLength,0,this._cap);
                    return;
                }
                
            }
            throw new ArgumentOutOfRangeException("iLength", iLength, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array (puede ser null).</param>
        public Chars(char[] arr):this(arr,0,arr==null?0:arr.Length,sumCap){}


        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, count deber ser 0.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public Chars(char[] arr, int count):this(arr,0,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public Chars(char[] arr, int arrIndex, int count):this(arr,arrIndex,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando el array de entrada.
        /// </summary>
        /// <param name="arr">Array. Si es null, arrIndex y count debern ser 0.</param>
        /// <param name="arrIndex">Index de inicio de la copia en el array origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="extraCap">Capacidad extra para agregar al array subyacente.</param>
        public Chars(char[] arr, int arrIndex, int count, int extraCap)
        {
            if (count<1)
            {
                if (count==0 && arrIndex==0)
                {
                    if (extraCap>0)
                    {
                        this._cap=Calc.AddIfOdd(extraCap);
                        this._items=new char[this._cap];
                    }
                    return;
                }
            }
            else if (arrIndex>-1 && arrIndex+count<=arr.Length)
            {
                this._items=new char[this._cap=Calc.AddIfOdd(extraCap+count)];
                fixed(char*po=arr,pd=this._items)
                    General.Copy(po+arrIndex,pd,count);
                this._size=count;
                return;
            }
            throw new ArgumentOutOfRangeException("arrIndex,count,extraCap", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }

        #endregion

        #region Asistentes

        /// <summary>
        /// Aplica suma sobre la mitad de DefaultCapacity y luego otra suma de redondeo a DefaultCapacity.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int round(int minItems)
        {
            return Calc.MultMajor(minItems+sumCap,DefaultCapacity,true);
        }

        /// <summary>
        /// Genera una nueva capacidad mnima aplicando algorritmo y redondeando.
        /// </summary>
        /// <param name="sum">Elementos a agregar.</param>
        /// <returns>Largo Necesario para generar array.</returns>
        private int newMinCap(int sum)
        {
            if (this.AppendExact)
                return round(this._size+sum);

            if (sum==1)
            {
                if (this._size==0)
                    return sumCap;
                if (this._size<=(DefaultCapacity*(1<<10)))
                    return round(this._size<<1);
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>1));
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>2));
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>3));
                return round(this._size+(this._size>>4));
            }
            if (this._size==0)
            {
                if (sum<=DefaultCapacity)
                    return round(sum+DefaultCapacity);
                if (sum<=(DefaultCapacity*(1<<5)))
                    return round(sum<<1);
                if (sum<=(DefaultCapacity*(1<<10)))
                    return round(sum+(sum>>1));
                if (sum<=(DefaultCapacity*(1<<15)))
                    return round(sum+(sum>>2));
                if (sum<=(DefaultCapacity*(1<<18)))
                    return round(sum+(sum>>3));
                return round(sum+(sum>>4));
            }
            if (sum<=DefaultCapacity || this._size>sum)
            {
                if (this._size<=(DefaultCapacity*(1<<10)))
                {
                    if (this._size>sum)
                        return round(this._size<<1);
                    return round(this._size+(this._size>>1)+sum);
                }
                if (this._size<=(DefaultCapacity*(1<<15)))
                    return round(this._size+(this._size>>2)+sum);
                if (this._size<=(DefaultCapacity*(1<<18)))
                    return round(this._size+(this._size>>3)+sum);
                if (this._size<=(DefaultCapacity*(1<<21)))
                    return round(this._size+(this._size>>4)+sum);
                return round(this._size+(this._size>>5)+sum);
            }
            if (this._size<=(DefaultCapacity*(1<<10)))
                return round(this._size+(this._size>>3)+sum);
            if (this._size<=(DefaultCapacity*(1<<15)))
                return round(this._size+(this._size>>4)+sum);
            if (this._size<=(DefaultCapacity*(1<<18)))
                return round(this._size+(this._size>>5)+sum);
            return round(this._size+sum);
        }

        /// <summary>
        /// Genera un nuevo array en base al tamao mnimo especificado,
        /// sumando la mitad de DefaultCapacity y redondeando.
        /// Tambin actualiza capacidad.
        /// A diferencia de newArraySumItems no compara en algorritmo 
        /// el crecimiento sobre el array actual.
        /// </summary>
        /// <param name="minItems">Largo requerido.</param>
        /// <returns>Nuevo array.</returns>
        private char[] newArrayMin(int minItems)
        {
            return new char[this._cap=round(minItems)];
        }


        /// <summary>
        /// Genera un nuevo array en base al crecimiento requerido y aplicando algorritmo.
        /// Tambin actualiza capacidad.
        /// </summary>
        /// <param name="sumItems">Crecimiento requerido.</param>
        private char[] newArraySum(int sumItems)
        {
            return new char[this._cap=newMinCap(sumItems)];
        }

        /// <summary>
        /// Actualiza la capacidad para aumento.
        /// </summary>
        /// <param name="items">Nueva cantidad.</param>
        private void setCapacity(int items)
        {
            char[] array2=new char[items];
            if (this._size !=0)
            {
                fixed (char* po=this._items, pd=array2)
                    General.Copy(po, pd, this._size);
            }
            this._items=array2;
            this._cap=items;
        }

        #endregion

        #region Propiedades

        /// <summary>
        /// Si true, no usar algoritmo de aumento para la coleccin, aunque igual efectuar redondeo.
        /// Por lo tanto, si este parmetro esta activo se redimensionar
        /// la coleccion cada vez que se agreguen nuevos elementos.
        /// </summary>
        public bool AppendExact
        {
            get
            {
                return (this._flags&(1U<<8))!=0;
            }
            set
            {
                if (value)
                    this._flags|=1U<<8;
                else
                    this._flags&=0xFFFFFFFF^(1U<<8);
            }
        }

        /// <summary>
        /// Obtiene el array que actualmente esta siendo usado por la coleccin (puede ser null) y,
        /// por lo tanto, suceptible de ser modificado o reemplazado por la coleccin.
        /// Al establecer, cambia la instancia del array por el especificado en parametro y 
        /// setea la cantidad de elementos en el largo del nuevo array (si es nulo ser 0).
        /// </summary>
        public char[] Array
        {
            get
            {
                return this._items;
            }
            set
            {
                this._cap=value==null?0:value.Length;
                this._size=this._cap;
                this._items=value;
            }
        }

        /// <summary>
        /// Obtiene un array con los elementos de la coleccin.
        /// Esta copia puede ser el mismo objeto subyacente si la cantidad de elementos coincide con el largo de array
        /// o un array vacio si la capacidad es cero.
        /// </summary>
        public char[] ArrayCompact
        {
            get
            {
                if (this._cap==0)
                    return ArrGeneral.ArrayEmptyChar;
                if (this._size==this._cap)
                    return this._items;
                return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
            }
        }

        /// <summary>
        /// Obtiene una copia compacta al largo actual de la coleccin.
        /// Al establecer, copia los elementos de la instancia actual al array subyacente o, en caso que el
        /// array de entrada sea de capacidad mayor, genera una nueva instancia con la copia de los elementos.
        /// Setea la cantidad de elementos en el largo del nuevo array.
        /// </summary>
        public char[] ArrayCopy
        {
            get
            {
                if (this._size!=0)
                    return ArrGeneral.Clone(this._items, 0, this._size, 0, this._size);
                return ArrGeneral.ArrayEmptyChar;
            }
            set
            {
                this._size=value==null?0:value.Length;
                if (this._size !=0)
                {
                    if (this._cap < this._size)
                        this._items=newArrayMin(this._size+sumCap);
                    fixed (char* po=value, pd=this._items)
                        General.Copy(po, pd, this._size);
                }
            }
        }



        /// <summary>
        /// Obtiene o establece, solo para aumentar, la capacidad del Array.
        /// En caso de establecer una capacidad menor, se ignora. Si se establece capacidad mayor, el valor ser
        /// redondeado.
        /// </summary>
        public int Capacity
        {
            get
            {
                return this._cap;
            }
            set
            {
                if (value > this._cap)
                    setCapacity(round(value));
            }
        }


        /// <summary>
        /// Obtiene o establece la cantidad de elementos en la coleccin.
        /// Se permite el establecimiento de un largo mayor al actual siempre y cuanto este no exceda la capacidad
        /// del array subyacente (el contenido de los elementos excedentes no ser modificado).
        /// Por lo tanto, si se especifica una cantidad negativa o mayor a la capacidad actual,
        /// se genera una excepcin.
        /// En todo caso, para modificar el largo con mayor flexibilidad, usar el mtodo SetLength.
        /// </summary>
        public int Length
        {
            get
            {
                return this._size;
            }
            set
            {
                if (value > this._cap || value < 0)
                    throw new ArgumentOutOfRangeException("value", value, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                this._size=value;
            }
        }


        /// <summary>
        /// Obtiene o establece el elemento especificado en index.
        /// Para acceder a los elementos frecuentemente, es conveniente acceder al array directamente.
        /// Si el index se encuentra fuera de rango, genera excepcin.
        /// </summary>
        public char this[int index]
        {
            get
            {
                if (this._size <=index)
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
                return this._items[index];
            }
            set
            {
                if (index < this._size)
                    this._items[index]=value;
                else 
                    throw new ArgumentOutOfRangeException("index", index, Errors.SUPERIOR_ARRAY);
            }
        }


        #endregion

        #region Metodos Administrativos

        /// <summary>
        /// Agrega un nuevo elemento al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        public void Append(char item)
        {
            this.EnsureAppend();
            this._items[this._size++]=item;
        }

        /// <summary>
        /// Agrega elementos al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento.</param>
        /// <param name="count">Cantidad de repeticiones del elemento.</param>
        public void Append(char item, int count)
        {
            if (item==0)
            {
                this.AppendBlank(count);
                return;
            }

            if (count < 2)
            {
                if (count==1)
                    this.Append(item);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }
            this.EnsureAppend(count);
            fixed (char* pd=this._items)
                General.Fill(pd + this._size, count, item);
            this._size +=count;
        }

        /// <summary>
        /// Agrega un cero al final de la coleccin.
        /// </summary>
        public void AppendBlank()
        {
            this.EnsureAppend();
            this._items[this._size++]='\0';
        }

        /// <summary>
        /// Agrega ceros al final de la coleccin.
        /// </summary>
        /// <param name="count">Cantidad de ceros.</param>
        public void AppendBlank(int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.AppendBlank();
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }
            this.EnsureAppend(count);
            fixed (char* pd=this._items)
                General.Clean(pd + this._size, count);
            this._size +=count;
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        public void Append(char[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void Append(char[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Append(char[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.Append(po + arrIndex, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void Append(char* arr, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.Append(*arr);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                return;
            }

            this.EnsureAppend(count);
            fixed (char* pd=this._items)
                General.Copy(arr, pd + this._size, count);
            this._size +=count;
        }


        /// <summary>
        /// Elimina todos los elementos de la coleccin.
        /// </summary>
        public void Clear()
        {
            this._size=0;
        }

        /// <summary>
        /// Elimina todos los elementos de la coleccin y establece el array en null.
        /// </summary>
        public void ClearCompact()
        {
            this._size=0;
            this._cap=0;
            this._items=null;
        }

        /// <summary>
        /// Compacta la coleccin a la cantidad de elementos actual.
        /// Si la cantidad actual de elementos es cero, el array subyacente ser null.
        /// </summary>
        public void Compact()
        {
            if (this._size !=this._cap)
            {
                if (this._size!=0)
                {
                    char[] array2=new char[this._size];
                    fixed (char* po=this._items, pd=array2)
                        General.Copy(po, pd, this._size);
                    this._items=array2;
                }
                else
                    this._items=null;
                this._cap=this._size;
            }
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        public void CopyTo(char* arr)
        {
            if (this._size !=0)
                fixed (char* po=this._items)
                    General.Move(po, arr, this._size);
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        public void CopyTo(char* arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (this._size !=startIndex)
                fixed (char* po=this._items)
                    General.Move(po + startIndex, arr, this._size - startIndex);
        }

        /// <summary>
        /// Copia al puntero destino (mediante mover) el contenido actual de la coleccin.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        /// <param name="count">Elementos a copiar.</param>
        public void CopyTo(char* arr, int startIndex, int count)
        {

            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (this._size !=startIndex)
                fixed (char* po=this._items)
                    General.Move(po + startIndex, arr, count);
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        public void CopyTo(char[] arr)
        {
            if (this._size !=0)
            {
                if (arr.Length < this._size)
                    throw new ArgumentOutOfRangeException("arr", arr.Length, Errors.ARRAY_SMALL_DEST);
                fixed (char* po=this._items, pd=arr)
                    General.Copy(po, pd, this._size);
            }
        }

        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        public void CopyTo(char[] arr, int startIndex)
        {
            int count=this._size - startIndex;
            if (startIndex > this._size || startIndex < 0 || count > arr.Length)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                fixed (char* po=this._items, pd=arr)
                    General.Copy(po + startIndex, pd, count);
        }


        /// <summary>
        /// Copia al array destino el contenido actual de la coleccin.
        /// El largo del array destino debe contener la cantidad de elementos.
        /// </summary>
        /// <param name="arr">Array destino.</param>
        /// <param name="startIndex">Indice en coleccin donde comienza la copia.</param>
        /// <param name="count">Elementos a copiar.</param>
        public void CopyTo(char[] arr, int startIndex, int count)
        {
            if (startIndex > this._size || startIndex < 0 || count < 0 || count > arr.Length)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
                fixed (char* po=this._items, pd=arr)
                    General.Copy(po + startIndex, pd, count);
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de 1 elemento.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        public void EnsureAppend()
        {
            if (this._size == this._cap)
                setCapacity(newMinCap(1));
        }

        /// <summary>
        /// Verifica si la capacidad soporta el agregado de n elementos.
        /// Si no es asi, ajusta la capacidad para que pueda agregarlo (el valor de ajuste es de acuerdo a algoritmo).
        /// </summary>
        /// <param name="items">Elementos a agregar.</param>
        public void EnsureAppend(int items)
        {
            if ((items + this._size) > this._cap)
                setCapacity(newMinCap(items));
        }

        /// <summary>
        /// Recupera una copia de los elementos.
        /// </summary>
        /// <returns>Array con los elementos copiados.</returns>
        public char[] GetCopy()
        {
            return this.ArrayCopy;
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public char[] GetCopy(int count)
        {
            if (this._size < count || count < 0)
                throw new System.ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            char[] ret=new char[count];
            if (count !=0)
            {
                fixed (char* po=this._items, pd=ret)
                    General.Copy(po, pd, count);
            }
            return ret;
        }

        /// <summary>
        /// Recupera una copia de los elementos en el rango especificado.
        /// </summary>
        /// <param name="startIndex">Index donde se inicia la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <returns>Array con los elementos copiados.</returns>
        public char[] GetCopy(int startIndex, int count)
        {
            if (this._size - startIndex < count || count < 0 || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            char[] ret=new char[count];
            if (count !=0)
            {
                fixed (char* po=this._items, pd=ret)
                    General.Copy(po + startIndex, pd, count);
            }
            return ret;
        }

        /// <summary>
        /// Quita todos los elementos del rango especificado.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        /// <param name="count">Cantidad de elementos a eliminar.</param>
        public void RemoveRange(int startIndex, int count)
        {
            if (count < 2)
            {
                if (count==1)
                    this.RemoveAt(startIndex);
                else if (count < 0)
                    throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
                return;
            }
            int max=startIndex + count;

            if (startIndex < 0 || max > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (max==this._size)
            {
                this._size=startIndex;
                return;
            }
            fixed (char* p=this._items)
                General.Cut(p, this._size, startIndex, count, false);
            this._size -=count;
        }

        /// <summary>
        /// Inserta el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="index">Index en la coleccin donde se insertar el elemento.</param>
        public void Insert(int index, char item)
        {
            this.Reserve(index);
            this._items[index]=item;
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void Insert(int startIndex, char item, int count)
        {
            this.Reserve(startIndex,count);
            this.Set(startIndex,item,count);
        }

        /// <summary>
        /// Inserta un cero en la ubicacin especificada.
        /// </summary>
        /// <param name="index">Index en la coleccin donde insertar.</param>
        public void InsertBlank(int index)
        {
            this.Reserve(index);
            this._items[index]='\0';
        }


        /// <summary>
        /// Inserta ceros en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void InsertBlank(int startIndex, int count)
        {
            this.Reserve(startIndex,count);
            this.SetBlank(startIndex,count);
        }


        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Insert(int startIndex, char[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Insert(int startIndex, char[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (char* po=arr)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, char[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.Insert(startIndex,po + arrIndex, count);
            }
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, char* arr, int count)
        {
            this.Reserve(startIndex,count);
            this.Set(startIndex,arr,count);
        }

        /// <summary>
        /// Quita del array el elemento especificado.
        /// </summary>
        /// <param name="index">Index del elemento.</param>
        public void RemoveAt(int index)
        {
            if (index >=this._size || index < 0)
                throw new System.ArgumentOutOfRangeException("index", index, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (--this._size !=index)
            {
                fixed (char* p=this._items)
                    General.Copy(p + (index + 1), p + index, this._size - index);
            }

        }

        /// <summary>
        /// Quita todos los elementos desde la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index.</param>
        public void RemoveRange(int startIndex)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
        }

        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, char[] r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex);
            else
                fixed(char*pr=r)
                    ReplaceRange(startIndex,pr,lr);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, char[] r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex,count);
            else
                fixed(char*pr=r)
                    ReplaceRange(startIndex,count,pr,lr);
        }

        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, char* pr, int lr)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
            this.Append(pr,lr);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, char* pr, int lr)
        {
            if (count>=lr)
            {
                Set(startIndex,pr,lr);
                if (count>lr)
                    RemoveRange(startIndex+lr,count-lr);
                return;
            }
            if (count<1)
            {
                if (count==0)
                {
                    Insert(startIndex,pr,lr);
                    return;
                }
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            Reserve(startIndex+count,lr-count);
            Set(startIndex,pr,lr);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        public void Reverse()
        {
            if (this._size > 1)
                fixed (char* p=this._items)
                    Sort.Reverse(p, this._size);
        }

        /// <summary>
        /// Invierte el orden de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void Reverse(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (char* p=this._items)
                    Sort.Reverse(p + startIndex, count);
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de un elemento. A diferencia de la funcin Insert,
        /// no efecta ninguna copia o limpieza.
        /// </summary>
        /// <param name="index">Index de la insercin.</param>
        public void Reserve(int index)
        {
            if (index<0 || index>this._size)
                throw new ArgumentOutOfRangeException("index",index,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (index==this._size)
                this.EnsureAppend();
            else
            {
                if (this._size==this._cap)
                {
                    char[] array2=newArraySum(1);
                    fixed (char* po=this._items, pd=array2)
                    {
                        if (index!=0)
                            General.Copy(po, pd, index);
                        General.Copy(po + index, pd + (index + 1), this._size - index);
                    }
                    this._items=array2;
                }
                else
                {
                    fixed (char* po=this._items)
                        General.Move(po + index, po + (index + 1), this._size - index);
                }
            }
            this._size++;
        }

        /// <summary>
        /// Agranda la coleccin reservando espacio para la futura 
        /// insercin de la cantidad de elementos especificada mediante las funciones Set
        /// u otras. A diferencia de la funcin Insert, no efecta ninguna copia o limpieza
        /// sobre el area de insercin, siendo el contenido de esta indeterminado.
        /// </summary>
        /// <param name="startIndex">Index de inicio de insercin.</param>
        /// <param name="count">Cantidad de elementos a reservar.</param>
        public void Reserve(int startIndex, int count)
        {
            if (count!=0)
            {
                if (count==1)
                    this.Reserve(startIndex);
                else if (startIndex<0 || count<0)
                    throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE);
                else if (startIndex==this._size)
                {
                    this.EnsureAppend(count);
                    this._size+=count;
                }
                else if (startIndex>this._size)
                    throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.SUPERIOR_ARRAY);
                else
                {
                    int nue=this._size + count;
                    if (nue > this._cap)
                    {
                        char[] array2=newArraySum(count);
                        fixed (char* po=this._items, pd=array2)
                        {
                            if (startIndex!=0)
                                General.Copy(po, pd, startIndex);
                            General.Copy(po + startIndex, pd + (startIndex + count), this._size - startIndex);
                        }
                        this._items=array2;
                    }
                    else
                    {
                        fixed (char* po=this._items)
                            General.Move(po + startIndex, po + (startIndex + count), this._size - startIndex);
                    }
                    this._size=nue;
                }
            }
        }

        /// <summary>
        /// Setea el elemento en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="index">Index en la coleccin donde se setear el elemento.</param>
        public void Set(int index, char item)
        {
            this[index]=item;
        }


        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a setear.</param>
        public void Set(int startIndex, char item, int count)
        {
            if (count==1)
                this[startIndex]=item;
            else if (item==0)
                SetBlank(startIndex,count);
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (char* po=this._items)
                    General.Fill(po + startIndex, count, item);
        }

        /// <summary>
        /// Setea un cero en la ubicacin especificada.
        /// </summary>
        /// <param name="index">Index en la coleccin donde setear.</param>
        public void SetBlank(int index)
        {
            this[index]='\0';
        }


        /// <summary>
        /// Setea ceros en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la insercin.</param>
        /// <param name="count">Cantidad de elementos a setear.</param>
        public void SetBlank(int startIndex, int count)
        {
            if (count==1)
                this[startIndex]='\0';
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (char* po=this._items)
                    General.Clean(po + startIndex, count);
        }

        /// <summary>
        /// Modifica la cantidad de elementos en la coleccin.
        /// </summary>
        /// <param name="count">Nueva cantidad de elementos.</param>
        /// <param name="exact">En caso de que sea necesario aumentar la capacidad, no aplica algoritmo al aumento.</param>
        /// <param name="clean">En caso de aumentar la cantidad, rellena con ceros el nuevo espacio.</param>
        public void SetLength(int count, bool exact, bool clean)
        {
            if (count <=this._cap)
            {
                if (count <=this._size)
                {
                    if (count < 0)
                        throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
                }
                else if (clean)
                    fixed (char* p=this._items)
                        General.Clean(p + this._size, count - this._size);
            }
            else
            {
                char[] array2;
                if (exact)
                {
                    array2=new char[count];
                    this._cap=count;
                }
                else
                    array2=newArraySum(count-this._size);

                if (this._size !=0)
                {
                    fixed (char* po=this._items, pd=array2)
                        General.Copy(po, pd, this._size);
                }
                this._items=array2;
            }
            this._size=count;
        }

        /// <summary>
        /// Modifica la cantidad de elementos para que coincida con la capacidad actual.
        /// </summary>
        /// <param name="clean">Rellena con ceros el rea excedente.</param>
        public void SetLengthToCapacity(bool clean)
        {
            if (this._size!=this._cap)
            {
                if (clean)
                    fixed(char*p=this._items)
                        General.Clean(p+this._size,this._cap-this._size);
                this._size=this._cap;
            }
        }


        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Set(int startIndex, char[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio del array.</param>
        public void Set(int startIndex, char[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (char* po=arr)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, char[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.Set(startIndex, po + arrIndex, count);
            }
        }

        /// <summary>
        /// Setea el contenido del array en la ubicacin especificada.
        /// </summary>
        /// <param name="arr">Array a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, char* arr, int count)
        {
            if (count==1)
                this[startIndex]=*arr;
            else if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            else if (count!=0)
                fixed (char* p=this._items)
                    General.Copy(arr,p + startIndex, count);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortAscending()
        {
            if (this._size > 1)
                fixed (char* p=this._items)
                    Sort.Ascending(p, this._size);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortAscending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (char* p=this._items)
                    Sort.Ascending(p + startIndex, count);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        public void SortDescending()
        {
            if (this._size > 1)
                fixed (char* p=this._items)
                    Sort.Descending(p, this._size);
        }

        /// <summary>
        /// Orden ascendente de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortDescending(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (char* p=this._items)
                    Sort.Descending(p + startIndex, count);
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        public void SortRandom()
        {
            if (this._size > 1)
                fixed (char* p=this._items)
                    Sort.Randomize(p, this._size);
        }

        /// <summary>
        /// Orden aleatoreo de los elementos.
        /// </summary>
        /// <param name="startIndex">Inicio de la zona de orden.</param>
        /// <param name="count">Cantidad de elementos a ordenar.</param>
        public void SortRandom(int startIndex, int count)
        {
            if (startIndex < 0 || count < 0 || startIndex + count > this._size)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count > 1)
                fixed (char* p=this._items)
                    Sort.Randomize(p + startIndex, count);
        }


        #endregion

        #region Set, Insert y Append para cantidades limitadas de elementos

        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, char item0, char item1)
        {
            if (startIndex<0 || startIndex+1>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            fixed(char*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
            }
        }

        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, char item0, char item1, char item2)
        {
            if (startIndex<0 || startIndex+2>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            fixed(char*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
            }
        }


        /// <summary>
        /// Setea elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se setearn los elementos.</param>
        public void SetItems(int startIndex, char item0, char item1, char item2, char item3)
        {
            if (startIndex<0 || startIndex+3>=this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            fixed(char*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
                p[startIndex+3]=item3;
            }
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, char item0, char item1)
        {
            this.Reserve(startIndex,2);
            fixed(char*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
            }
        }

        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, char item0, char item1, char item2)
        {
            this.Reserve(startIndex,3);
            fixed(char*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
            }
        }


        /// <summary>
        /// Inserta elementos en la ubicacin especificada.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertarn los elementos.</param>
        public void InsertItems(int startIndex, char item0, char item1, char item2, char item3)
        {
            this.Reserve(startIndex,4);
            fixed(char*p=this._items)
            {
                p[startIndex]=item0;
                p[startIndex+1]=item1;
                p[startIndex+2]=item2;
                p[startIndex+3]=item3;
            }
        }

        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        public void AppendItems(char item0, char item1)
        {
            this.EnsureAppend(2);
            fixed(char*p=this._items)
            {
                p[this._size]=item0;
                p[this._size+1]=item1;
                this._size+=2;
            }
        }

        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        public void AppendItems(char item0, char item1, char item2)
        {
            this.EnsureAppend(3);
            fixed(char*p=this._items)
            {
                p[this._size]=item0;
                p[this._size+1]=item1;
                p[this._size+2]=item2;
                this._size+=3;
            }
        }


        /// <summary>
        /// Inserta elementos al final de la coleccin.
        /// </summary>
        /// <param name="item0">Primer elemento.</param>
        /// <param name="item1">Segundo elemento.</param>
        /// <param name="item2">Tercer elemento.</param>
        /// <param name="item3">Cuarto elemento.</param>
        public void AppendItems(char item0, char item1, char item2, char item3)
        {
            this.EnsureAppend(4);
            fixed(char*p=this._items)
            {
                p[this._size]=item0;
                p[this._size+1]=item1;
                p[this._size+2]=item2;
                p[this._size+3]=item3;
                this._size+=4;

            }
        }

        #endregion

        #region Metodos de Bsqueda, reemplazo y comparacin.
  
        /// <summary>
        /// Busca un elemento en memoria (previamente ordenada) mediante algoritmo BinarySearch.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="first">Se asegura de que la ubicacin de retorno corresponda a la primer aparicin del elemento especificado.</param>
        /// <returns>
        /// Si se encuentra:
        /// Ubicacin del elemento especificado.
        /// Si no se encuentra y existen valores mayores al especificado:
        /// Complemento bit a bit del indice del primer elemento mayor.
        /// Si no se encuentra y no existen valores mayores al especificado:
        /// Complemento bit a bit del largo. Por lo tanto, si l es 0 el retorno siempre ser -1.
        /// </returns>
        /// <remarks>
        /// La coleccin debe estar ordenada en forma ascendente.
        /// Si no se encuentra el elemento, el retorno siempre ser negativo.
        /// </remarks>
        public int BinarySearch(char item, bool first)
        {
            if (this._size==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.BinarySearch(p, this._size, item);
                if (first && ret > 0)
                    while (p[ret - 1]==item && --ret !=0);
                return ret;
            }
        }

        /// <summary>
        /// Efecta una comparacin con la coleccin especificada.
        /// </summary>
        /// <param name="col2">Coleccin de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en esta coleccin que en col2.
        /// - Si todos los elementos comparados son iguales y col2 contiene ms elementos que esta coleccin.
        /// Retorna 0:
        /// - Si esta coleccin y col2 representan al mismo objeto.
        /// - Si todos los elementos comparados son iguales siempre y cuando esta coleccin y col2 tengan la misma cantidad de elementos.
        /// Retorna 1:
        /// - Si col2 es null.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en esta coleccin que en col2.
        /// - Si todos los elementos comparados son iguales y col2 contiene menos elementos que esta coleccin.
        /// </returns>
        public int CompareTo(Chars col2)
        {
            if (Object.ReferenceEquals(this,col2))
                return 0;
            if (col2==null)
                return 1;
            if (col2._size==0)
                return this._size!=0?1:0;
            if (this._size==0)
                return -1;
            int eqComp,count;
            if (col2._size>this._size)
            {
                count=this._size;
                eqComp=-1;
            }
            else if (col2._size!=this._size)
            {
                count=col2._size;
                eqComp=1;
            }
            else
            {
                count=this._size;
                eqComp=0;
            }
            fixed(char*p1=this._items, p2=col2._items)
            {
                count=Comp.Compare(p1,p2,count);
                if (count==-1)
                    return eqComp;
                return (p1[count]<p2[count])?-1:1;
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y array2 contiene ms elementos que la coleccin.
        /// Retorna 0: 
        /// - Si todos los elementos comparados son iguales siempre y cuando la coleccin y array2 tengan la misma cantidad de elementos.
        /// Retorna 1:
        /// - Si array2 es null.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y array2 contiene menos elementos que la coleccin.
        /// </returns>
        public int CompareTo(char[] array2)
        {
            if (array2==null)
                return 1;
            int count=array2.Length;
            if (count==0)
                return this._size!=0?1:0;
            if (this._size==0)
                return -1;
            int eqComp;
            if (count>this._size)
            {
                count=this._size;
                eqComp=-1;
            }
            else if (count!=this._size)
                eqComp=1;
            else
                eqComp=0;
            fixed(char*p1=this._items, p2=array2)
            {
                count=Comp.Compare(p1,p2,count);
                if (count==-1)
                    return eqComp;
                return (p1[count]<p2[count])?-1:1;
            }
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="arrIndex">Inicio de la comparacin en array2.</param>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si arrIndex esta dentro del rango y startIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y el rango comparable es mayor en array2 que en la coleccin.
        /// Retorna 0: 
        /// - Si tanto startIndex como arrIndex estn fuera de rango.
        /// - Si todos los elementos comparados son iguales siempre y cuando la coleccin y array2 tengan el mismo rango comparable.
        /// Retorna 1:
        /// - Si startIndex esta dentro del rango y arrIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales y el rango comparable es menor en array2 que en la coleccin.
        /// </returns>
        public int CompareTo(char[] array2, int startIndex, int arrIndex)
        {
            if (startIndex<0 || arrIndex<0)
                throw new ArgumentOutOfRangeException("startIndex,arrIndex",Errors.NEGATIVE);
            int eqComp=array2.Length;
            if (arrIndex>=eqComp)
            {
                if (startIndex>=this._size)
                    return 0;
                return 1;
            }
            if (startIndex>=this._size)
                return -1;
            
            int count=this._size-startIndex;
            if (count<(eqComp-=arrIndex))
                eqComp=-1;
            else if(count!=eqComp)
            {
                count=eqComp;
                eqComp=1;
            }
            else
                eqComp=0;
            fixed(char*p1=this._items, p2=array2)
            {
                count=Comp.Compare(p1+startIndex,p2+arrIndex,count);
                if (count==-1)
                    return eqComp;
                return (p1[startIndex+count]<p2[arrIndex+count])?-1:1;
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="arrIndex">Inicio de la comparacin en array2.</param>
        /// <param name="count">Mximo de elementos a comparar.</param>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si count es mayor a cero, arrIndex esta dentro del rango y startIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales siempre y cuando count haya sido reducido total o en mayor medida para ajustarse a la cantidad de elementos de la coleccin.
        /// Retorna 0: 
        /// - Si count es cero.
        /// - Si tanto startIndex como arrIndex estn fuera de rango.
        /// - Si todos los elementos comparados son iguales siempre y cuando count no haya sido reducido o que la reduccin se deba a un mismo excedente en la coleccin y array2.
        /// Retorna 1:
        /// - Si count es mayor a cero, startIndex esta dentro del rango y arrIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en array2.
        /// - Si todos los elementos comparados son iguales siempre y cuando count haya sido reducido total o en mayor medida para ajustarse al largo de array2.
        /// </returns>
        public int CompareTo(char[] array2, int startIndex, int arrIndex, int count)
        {
            if (startIndex<0 || arrIndex<0)
                throw new ArgumentOutOfRangeException("startIndex,arrIndex",Errors.NEGATIVE);
            if (count<1)
            {
                if (count==0)
                    return 0;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            int eqComp=array2.Length;
            if (arrIndex>=eqComp)
            {
                if (startIndex>=this._size)
                    return 0;
                return 1;
            }
            if (startIndex>=this._size)
                return -1;
            eqComp-=arrIndex;
            int rest1=this._size-startIndex;
            if (rest1<count)
            {
                if (eqComp<count)
                {
                    if (rest1<eqComp)
                    {
                        count=rest1;
                        eqComp=-1;
                    }
                    else if (rest1!=eqComp)
                    {
                        count=eqComp;
                        eqComp=1;
                    }
                    else
                    {
                        count=rest1;
                        eqComp=0;
                    }
                }
                else
                {
                    count=rest1;
                    eqComp=-1;
                }
            }
            else if (eqComp<count)
            {
                count=eqComp;
                eqComp=1;
            }
            else
                eqComp=0;
            fixed(char*p1=this._items, p2=array2)
            {
                count=Comp.Compare(p1+startIndex,p2+arrIndex,count);
                if (count==-1)
                    return eqComp;
                return (p1[startIndex+count]<p2[arrIndex+count])?-1:1;
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="count">Mximo de elementos a comparar.</param>
        /// <param name="p2">Puntero de comparacin. Debe contener a count.</param>
        /// <returns>
        /// Retorna -1:
        /// - Si count es mayor a cero y startIndex esta fuera de rango.
        /// - Si se encuentra un elemento diferente en la comparacin y este es menor en la coleccin que en p2.
        /// - Si todos los elementos comparados son iguales siempre y cuando count haya sido reducido para ajustarse a la cantidad de elementos de la coleccin.
        /// Retorna 0: 
        /// - Si count es cero.
        /// - Si todos los elementos comparados son iguales siempre y cuando count no haya sido reducido.
        /// Retorna 1:
        /// - Si se encuentra un elemento diferente en la comparacin y este es mayor en la coleccin que en p2.
        /// </returns>
        public int CompareTo(char* p2, int startIndex, int count)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            if (count<1)
            {
                if (count==0)
                    return 0;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (startIndex>=this._size)
                return -1;
            int eqComp=this._size-startIndex;
            if (eqComp<count)
            {
                count=eqComp;
                eqComp=-1;
            }
            else
                eqComp=0;
            fixed(char*p1=this._items)
            {
                count=Comp.Compare(p1+startIndex,p2,count);
                if (count==-1)
                    return eqComp;
                return (p1[startIndex+count]<p2[count])?-1:1;
            }
        }

        /// <summary>
        /// Retorna true si contiene el elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(char item,bool last)
        {
            if (this._size==0)
                return false;
            fixed (char* p=this._items)
            {
                if (!last)
                    return Comp.Exist(p, this._size, item);
                return Comp.LastIndexOf(p,this._size,item)!=-1;
            }
        }



        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(char item)
        {
            if (this._size !=0)
            {
                fixed (char* p=this._items)
                    return Comp.Count(p, this._size, item);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(char item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
                return Comp.Count(p + startIndex, this._size - startIndex, item);
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(char item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (char* p=this._items)
                return Comp.Count(p + startIndex, count, item);
        }


        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(char[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (char* p=this._items, pf=items)
                    return Comp.CountAny(p, this._size, pf, lf, false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items, pf=items)
                return Comp.CountAny(p + startIndex, this._size - startIndex, pf, lf, false);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, int count, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (char* p=this._items, pf=items)
                return Comp.CountAny(p + startIndex, count, pf, lf, false);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(char* pf, int lf)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (char* p=this._items)
                    return Comp.CountAny(p, this._size, pf, lf, false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, char* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
                return Comp.CountAny(p + startIndex, this._size - startIndex, pf, lf, false);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, int count, char* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (char* p=this._items)
                return Comp.CountAny(p + startIndex, count, pf, lf, false);
        }


        /// <summary>
        /// Efecta una comparacin con la coleccin especificada. 
        /// </summary>
        /// <param name="col2">Coleccin a comparar.</param>
        /// <returns>Retorna true si ambas coleccin contienen la misma cantidad de elementos y su contenido es igual.</returns>
        public bool Equals(Chars col2)
        {
            if (Object.ReferenceEquals(this,col2))
                return true;
            if (col2==null || this._size!=col2._size)
                return false;
            if (this._size<2)
            {
                if (this._size==0 || this._items[0]==col2._items[0])
                    return true;
                return false;
            }
            fixed(char*p=this._items, p2=col2._items)
                return Comp.Equal(p,p2,this._size);
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="array2">Array a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de array2 son iguales a los la coleccin (comparando desde el inicio) o array2 tiene largo 0.</returns>
        public bool Equals(char[] array2)
        {
            int count=array2.Length;
            if (count<2)
            {
                if (count==0 || (this._size!=0 && array2[0]==this._items[0]))
                    return true;
                return false;
            }
            if (count>this._size)
                return false;
            fixed(char*p=this._items, p2=array2)
                return Comp.Equal(p,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="array2">Array a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <returns>Solo retorna true si todos los elementos de array2 son iguales a los la coleccin (comparando desde startIndex) o array2 tiene largo 0.</returns>
        public bool Equals(char[] array2, int startIndex)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            int count=array2.Length;
            if (count<2)
            {
                if (count==0 || (this._size>startIndex && array2[0]==this._items[startIndex]))
                    return true;
                return false;
            }
            if (startIndex+count>this._size)
                return false;
            fixed(char*p=this._items, p2=array2)
                return Comp.Equal(p+startIndex,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="p2">Array a comparar.</param>
        /// <param name="count">Cantidad de elementos a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de p2 son iguales a los la coleccin (comparando desde el inicio) o count es 0.</returns>
        public bool Equals(char* p2, int count)
        {
            if (count<2)
            {
                if (count==1)
                {
                    if (this._size!=0 && this._items[0]==*p2)
                        return true;
                    return false;
                }
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (count>this._size)
                return false;
            fixed(char*p=this._items)
                return Comp.Equal(p,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado. 
        /// </summary>
        /// <param name="p2">Array a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="count">Cantidad de elementos a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de p2 son iguales a los la coleccin (comparando desde el startIndex) o count es 0.</returns>
        public bool Equals(char* p2, int startIndex, int count)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            if (count<2)
            {
                if (count==1)
                {
                    if (this._size>startIndex && this._items[startIndex]==*p2)
                        return true;
                    return false;
                }
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (startIndex+count>this._size)
                return false;
            fixed(char*p=this._items)
                return Comp.Equal(p+startIndex,p2,count);
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(char item)
        {
            if (this._size !=0)
            {
                fixed (char* p=this._items)
                    return Comp.IndexOf(p, this._size, item);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.IndexOf(p + startIndex, this._size - startIndex, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.IndexOf(p + startIndex, count, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }


        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOfAny(char[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (char* p=this._items, pf=items)
                    return Comp.IndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items, pf=items)
                ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, int count, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            int ret;
            fixed (char* p=this._items, pf=items)
                ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOfAny(char* pf, int lf)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (char* p=this._items)
                    return Comp.IndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, char* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, int count, char* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(char item)
        {
            if (this._size !=0)
            {
                fixed (char* p=this._items)
                    return Comp.LastIndexOf(p, this._size, item);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, this._size - startIndex, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, count, item);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOfAny(char[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (char* p=this._items, pf=items)
                    return Comp.LastIndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items, pf=items)
                ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, int count, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            int ret;
            fixed (char* p=this._items, pf=items)
                ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOfAny(char* pf, int lf)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (char* p=this._items)
                    return Comp.LastIndexOfAny(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, char* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, int count, char* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char item)
        {
            if (this._size==0)
                return 0;

            fixed (char* p=this._items)
            {
                return Comp.Remove(false, p, p, this._size, out this._size, item, false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, this._size - startIndex, out lFinal, item, false);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, count, out lFinal, item, false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                return Comp.RemoveAny(false, p, p, this._size, out this._size, pf, lf, false, false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, this._size - startIndex, out lFinal, pf, lf, false, false);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, int count, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, count, out lFinal, pf, lf, false, false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(char* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return 0;

            fixed (char* p=this._items)
            {
                return Comp.RemoveAny(false, p, p, this._size, out this._size, pf, lf, false, false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, char* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, this._size - startIndex, out lFinal, pf, lf, false, false);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, int count, char* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=Comp.RemoveAny(false, po, po, count, out lFinal, pf, lf, false, false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }


        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char item)
        {
            if (this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, item);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;

            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, int count, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(char* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, char* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, int count, char* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(char item)
        {
            if (this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, item);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char item, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char item, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOfAny(char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;

            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, int count, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOfAny(char* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, char* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, int count, char* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char item, char rep)
        {
            if (this._size==0)
                return 0;

            fixed (char* p=this._items)
            {
                return Comp.Replace(p, this._size, item, rep);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char item, char rep, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
            {
                return Comp.Replace(p + startIndex, this._size - startIndex, item, rep);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char item, char rep, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (char* p=this._items)
            {
                return Comp.Replace(p + startIndex, count, item, rep);
            }
        }


        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(char rep, char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                return Comp.ReplaceAny(p, this._size, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, char rep, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                return Comp.ReplaceAny(p + startIndex, this._size - startIndex, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, int count, char rep, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (char* p=this._items, pf=items)
            {

                return Comp.ReplaceAny(p + startIndex, count, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(char rep, char* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return 0;

            fixed (char* p=this._items)
            {
                return Comp.ReplaceAny(p, this._size, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, char rep, char* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
            {
                return Comp.ReplaceAny(p + startIndex, this._size - startIndex, pf, lf, rep, false);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, int count, char rep, char* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (char* p=this._items)
            {
                return Comp.ReplaceAny(p + startIndex, count, pf, lf, rep, false);
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(char item, char rep)
        {
            if (this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, item);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(char item, char rep, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(char item, char rep, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(char rep, char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;

            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, char rep, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, int count, char rep, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(char rep, char* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, char rep, char* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, int count, char rep, char* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOf(char item, char rep)
        {
            if (this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, item);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(char item, char rep, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, this._size - startIndex, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(char item, char rep, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, item);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(char rep, char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;

            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, char rep, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, int count, char rep, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(char rep, char* pf, int lf)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOfAny(p, this._size, pf, lf, false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, char rep, char* pf, int lf)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, int count, char rep, char* pf, int lf)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOfAny(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }


        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los del array de comparacin.
        /// </summary>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los del array de comparacin.</returns>
        public bool EndsWith(char[] array2)
        {

            int count=array2.Length;
            if (count<2)
            {
                if (count==0 || (this._size!=0 && array2[0]==this._items[this._size-1]))
                    return true;
                return false;
            }
            if (count>this._size)
                return false;
            fixed(char*p=this._items, p2=array2)
                return Comp.Equal(p+(this._size-count),p2,count);
        }

        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los del puntero de comparacin.
        /// </summary>
        /// <param name="p2">Puntero de comparacin.</param>
        /// <param name="count">Largo del puntero de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los del puntero de comparacin.</returns>
        public bool EndsWith(char* p2, int count)
        {
            if (count<2)
            {
                if (count==1)
                {
                    if (this._size!=0 && this._items[this._size-1]==*p2)
                        return true;
                    return false;
                }
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (count>this._size)
                return false;
            fixed(char*p=this._items)
                return Comp.Equal(p+(this._size-count),p2,count);
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los del array de comparacin.
        /// </summary>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los del array de comparacin.</returns>
        public bool StartsWith(char[] array2)
        {
            return Equals(array2);
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los del puntero de comparacin.
        /// </summary>
        /// <param name="p2">Puntero de comparacin.</param>
        /// <param name="count">Largo del puntero de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los del puntero de comparacin.</returns>
        public bool StartsWith(char* p2, int count)
        {
            return Equals(p2,count);
        }

        #endregion

        #region Propiedades y Constructores Bytes y Chars

        /// <summary>
        /// Inicializa la coleccin permitiendo especificar propiedades.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="unix">
        /// Estado inicial de la propiedad Unix.
        /// </param>
        /// <param name="cap">Capacidad inicial.</param>
        public Chars(int cap, bool appendExact, bool unix):this(cap)
        {
            this._flags=1U<<9;
            if (appendExact)
                this._flags+=1U<<8;
            if (unix)
                this._flags+=1U<<10;
        }

        /// <summary>
        /// Inicializa la coleccin con un array nulo, permitiendo especificar propiedades.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="unix">
        /// Estado inicial de la propiedad Unix.
        /// </param>
        public Chars(bool appendExact, bool unix)
        {
            this._flags=1U<<9;
            if (appendExact)
                this._flags+=1U<<8;
            if (unix)
                this._flags+=1U<<10;
        }

        /// <summary>
        /// Obtiene o establece si la coleccin utilizar saltos de lnea Unix en las funciones
        /// AppendLine, InsertLine y Reformat.
        /// Al obtener, si no se ha establecido anteriormente, el valor ser establecido de acuerdo
        /// al siguiente procedimiento:
        /// - Si la cadena esta vacia o no se encuentran saltos de lnea, el valor ser false y por
        ///   lo tanto usar saltos de lnea CRLF.
        /// - Si se encuentra el caracter CR, el valor tambin ser false independientemente a que LF
        ///   se encuentre o no.
        /// - Si no se encuentra el caracter CR pero s LF, el valor ser true
        ///   y por lo tanto usar saltos de lnea LF. 
        /// </summary>
        public bool Unix
        {
            get
            {
                switch(this._flags&((1U<<9)+(1U<<10)))
                {
                    case 1U<<9:
                        return false;
                    case (1U<<9)+(1U<<10):
                        return true;
                }
                if (this.LastIndexOf('\r')!=-1)
                {
                    this._flags|=1U<<9;
                    return false;
                }
                if (this.LastIndexOf('\n')!=-1)
                {
                    this._flags|=(1U<<9)+(1U<<10);
                    return true;
                }
                this._flags|=1U<<9;
                return false;
            }
            set
            {
                if (value)
                    this._flags|=(1U<<9)+(1U<<10);
                else if ((this._flags&(1U<<10))!=0)
                    this._flags&=0xFFFFFFFF^(1U<<10);
                else
                    this._flags|=1U<<9;
            }
        }

        #endregion
     
        #region Metodos de Bsqueda y reemplazo Bytes y Chars.

        /// <summary>
        /// Retorna true si contiene la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(char[] arr, bool last)
        {
            int lf;
            if (this._size==0 || this._size < (lf=arr.Length))
                return false;
            fixed (char* p=this._items, pf=arr)
            {
                if (!last)
                    return Comp.IndexOf(p,this._size,pf,lf,false)!=-1;
                return Comp.LastIndexOf(p,this._size,pf,lf,false)!=-1;
            }
        }

        /// <summary>
        /// Retorna true si contiene la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(char* pf, int lf, bool last)
        {
            if (this._size==0 || this._size<lf)
                return false;
            fixed (char* p=this._items)
            {
                if (!last)
                    return Comp.IndexOf(p,this._size,pf,lf,false)!=-1;
                return Comp.LastIndexOf(p,this._size,pf,lf,false)!=-1;
            }
        }
        
        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char[] arr)
        {
            int lf;
            if (this._size !=0 && (lf=arr.Length)<=this._size)
            {
                fixed (char* p=this._items, pf=arr)
                    return Comp.Count(p,this._size,pf,lf,false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (char* p=this._items, pf=arr)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de a buscar en la coleccin.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (char* p=this._items, pf=arr)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char* pf, int lf)
        {
            if (this._size !=0 && lf<=this._size)
            {
                fixed (char* p=this._items)
                    return Comp.Count(p,this._size,pf,lf,false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char*pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return 0;
            fixed (char* p=this._items)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de a buscar en la coleccin.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char* pf,int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return 0;
            fixed (char* p=this._items)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(char[] arr)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=arr.Length))
            {
                fixed (char* p=this._items, pf=arr)
                    return Comp.IndexOf(p, this._size,pf,lf,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=arr)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=arr)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(char* pf, int lf)
        {
            
            if (this._size !=0 && this._size>=lf)
            {
                fixed (char* p=this._items)
                    return Comp.IndexOf(p, this._size,pf,lf,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char* pf,int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char* pf,int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(char[] arr)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=arr.Length))
            {
                fixed (char* p=this._items, pf=arr)
                    return Comp.LastIndexOf(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=arr)
                ret=Comp.LastIndexOf(p + startIndex, count,pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=arr)
                ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }


        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(char* pf, int lf)
        {
            if (this._size !=0 && this._size>=lf)
            {
                fixed (char* p=this._items)
                    return Comp.LastIndexOf(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char* pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, count,pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char* pf, int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char[] arr)
        {
            int lf;
            if (this._size==0 || this._size<(lf=arr.Length))
                return 0;

            fixed (char* p=this._items, pf=arr)
            {
                return Comp.Remove(false, p, p, p, this._size, out this._size, pf , lf, false , false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (char* p=this._items, pf=arr)
            {
                char* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, po, count, out count,pf ,lf, false,false);
                if (ret!=0)
                    this._size -=ret*lf;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (char* p=this._items, pf=arr)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=Comp.Remove(false, po,po, po, count, out lFinal, pf, lf, false,false);
                if (ret!=0)
                {

                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret*lf;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char* pf, int lf)
        {
            if (this._size==0 || this._size<lf)
                return 0;

            fixed (char* p=this._items)
            {
                return Comp.Remove(false, p, p, p, this._size, out this._size, pf , lf, false , false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char* pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return 0;
            fixed (char* p=this._items)
            {
                char* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, po, count, out count,pf ,lf, false,false);
                this._size -=ret*lf;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char* pf, int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=Comp.Remove(false, po,po, po, count, out lFinal, pf, lf, false,false);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret*lf;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char[] arr)
        {
            int lf;
            if (this._size==0 || this._size<(lf=arr.Length))
                return -1;

            fixed (char* p=this._items, pf=arr)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (char* p=this._items, pf=arr)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (char* p=this._items, pf=arr)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char* pf, int lf)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char* pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char* pf, int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(char[] arr)
        {
            int lf;
            if (this._size==0 || this._size<(lf=arr.Length))
                return -1;

            fixed (char* p=this._items, pf=arr)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char[] arr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (char* p=this._items, pf=arr)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char[] arr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (char* p=this._items, pf=arr)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }

        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(char* pf, int lf)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char* pf, int lf, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char* pf, int lf, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char[] f, char[] r)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f);
            int lf=f.Length;
            if (lf==0 || this._size<lf)
                return 0;
            fixed(char*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char[] f, char[] r, int startIndex)
        {
            return ReplaceAll(f,r,startIndex,this._size-startIndex);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char[] f, char[] r, int startIndex, int count)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f,startIndex,count);
            int lf=f.Length;
            if (lf==0 || count<lf)
                return 0;
            fixed(char*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr,startIndex,count);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="pf">Cadena de bsqueda.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char* pf, int lf, char* pr, int lr)
        {
            if (this._size==0 || lf<1 || this._size<lf)
                return 0;
            fixed (char* p=this._items)
            {
                if (lf>=lr)
                {
                    if (lf==lr)
                        return Comp.ReplaceEqual(p,p,this._size,pf,pr,lf,false);
                    return Comp.ReplaceMinor(false,p,p,p,this._size,out this._size,pf,lf,pr,lr,false,false);
                }
                Ints ubics=Comp.Locations(null,0,p,this._size,pf,lf,false);
                int ret;
                if (ubics==null || (ret=ubics.Length)==0)
                    return 0;
                
                int lFinal=((lr-lf)*ret)+this._size;
                if (this._cap<lFinal)
                {
                    char[] array2=newArraySum(lFinal-this._size);
                    fixed(char*pd=array2)
                        fixed(int*pCont=ubics.Array)
                            Comp.ReplaceLocations(pCont,ret,p,this._size,pd,lf,pr,lr,false,false);
                    this._items=array2;
                }
                else
                {
                    fixed(int*pCont=ubics.Array)
                        Comp.ReplaceLocations(pCont,ret,p,this._size,p,lf,pr,lr,false,false);
                }
                this._size=lFinal;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="pf">Cadena de bsqueda.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char* pf, int lf, char* pr, int lr, int startIndex)
        {
            return ReplaceAll(pf,lf,pr,lr,startIndex,this._size-startIndex);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="pf">Cadena de bsqueda.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char* pf, int lf, char* pr, int lr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || lf<1 || count<lf)
                return 0;
            fixed (char* p=this._items)
            {
                char*po=p+startIndex;
                int ret, lFinal;
                if (lf>=lr)
                {
                    if (lf==lr)
                        return Comp.ReplaceEqual(po,po,count,pf,pr,lf,false);
                    if ((ret=Comp.ReplaceMinor(false,po,po,po,count,out lFinal,pf,lf,pr,lr,false,false))!=0)
                    {
                        General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                        this._size -=count-lFinal;
                    }
                    return ret;
                }
                Ints ubics=Comp.Locations(null,0, po,count,pf,lf,false);
                if (ubics==null || (ret=ubics.Length)==0)
                    return 0;
                lFinal=((lr-lf)*ret)+this._size;
                if (this._cap<lFinal)
                {
                    char[] array2=newArraySum(lFinal-this._size);
                    fixed(char*pd=array2)
                    {
                        General.Copy(p,pd,startIndex);
                        fixed(int*pCont=ubics.Array)
                            Comp.ReplaceLocations(pCont,ret,po,this._size-startIndex,pd+startIndex,lf,pr,lr,false,false);
                    }
                    this._items=array2;
                }
                else
                {
                    fixed(int*pCont=ubics.Array)
                        Comp.ReplaceLocations(pCont,ret,po,this._size-startIndex,po,lf,pr,lr,false,false);
                }
                this._size=lFinal;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char[] f, char[] r)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char[] f, char[] r, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char[] f, char[] r, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char* pf, int lf, char* pr, int lr)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char* pf, int lf, char* pr, int lr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char* pf, int lf, char* pr, int lr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char[] f, char[] r)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char[] f, char[] r, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char[] f, char[] r, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char* pf, int lf, char* pr, int lr)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char* pf, int lf, char* pr, int lr, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char* pf, int lf, char* pr, int lr, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }



        #endregion

        #region NormalizeLength Bytes y Chars

        /// <summary>
        /// Rellena o contrae la coleccin de acuerdo al largo.
        /// </summary>
        /// <param name="length">Largo.</param>
        /// <param name="fill">Caracter Relleno.</param>
        /// <param name="fillLeft">Fill a la izquierda. La contraccin siempre es a la derecha.</param>
        public void NormalizeLength(int length, char fill, bool fillLeft)
        {
            if (this._size!=length)
            {
                if (this._size>length)
                    this._size=length;
                else if (length<0)
                    throw new ArgumentOutOfRangeException("length",length,Errors.NEGATIVE);
                else if (this._size==0)
                    this.Append(fill,length);
                else if (!fillLeft)
                    this.Append(fill,length-this._size);
                else
                    this.Insert(0,fill,length-this._size);
            }
        }

        #endregion

        #region Propiedades y Constructores Chars

        /// <summary>
        /// Inicializa la coleccin con un array nulo, permitiendo especificar propiedades.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="unix">
        /// Estado inicial de la propiedad Unix.
        /// </param>
        /// <param name="tableCase">
        /// Opcin de tabla May/Min para efectuar conversiones.
        /// </param>
        /// <param name="cap">Capacidad inicial.</param>
        public Chars(int cap, bool appendExact, bool unix, TableCase tableCase):this(cap)
        {
            switch (tableCase)
            {
                case TableCase.Normal:
                    this._flags=(1U<<9)+(1U<<11);
                    break;
                case TableCase.Turkish:
                    this._flags=(1U<<9)+(1U<<12);
                    break;
                case TableCase.Invariant:
                    this._flags=(1U<<9)+(1U<<11)+(1U<<12);
                    break;
                default:
                    this._flags=1U<<9;
                    break;
            }
            
            if (appendExact)
                this._flags+=1U<<8;
            if (unix)
                this._flags+=1U<<10;
        }

        /// <summary>
        /// Inicializa la coleccin con un array nulo, permitiendo especificar propiedades.
        /// </summary>
        /// <param name="appendExact">
        /// Estado inicial de la propiedad AppendExact.
        /// </param>
        /// <param name="unix">
        /// Estado inicial de la propiedad Unix.
        /// </param>
        /// <param name="tableCase">
        /// Opcin de tabla May/Min para efectuar conversiones.
        /// </param>
        public Chars(bool appendExact, bool unix, TableCase tableCase)
        {
            switch (tableCase)
            {
                case TableCase.Normal:
                    this._flags=(1U<<9)+(1U<<11);
                    break;
                case TableCase.Turkish:
                    this._flags=(1U<<9)+(1U<<12);
                    break;
                case TableCase.Invariant:
                    this._flags=(1U<<9)+(1U<<11)+(1U<<12);
                    break;
                default:
                    this._flags=1U<<9;
                    break;
            }
            
            if (appendExact)
                this._flags+=1U<<8;
            if (unix)
                this._flags+=1U<<10;
        }


        /// <summary>
        /// Obtiene o establece la tabla para conversiones May/Min.
        /// Esto afecta el comportamiento de las funciones ToUpper, ToLower y ToProper
        /// Si al obtener no se habia establecido anteriormente, 
        /// se establecer segn la propiedad Texts.TableCurrent. 
        /// </summary>
        public TableCase TableCase
        {
            get
            {
                switch(this._flags&((1U<<11)+(1U<<12)))
                {
                    case 1U<<11:
                        return TableCase.Normal;
                    case 1U<<12:
                        return TableCase.Turkish;
                    case (1U<<11)+(1U<<12):
                        return TableCase.Invariant;
                }
                switch(Texts.TableCurrent)
                {
                    case TableCase.Normal:
                        this._flags+=1U<<11;
                        return TableCase.Normal;
                    case TableCase.Turkish:
                        this._flags+=1U<<12;
                        return TableCase.Turkish;
                }
                this._flags+=(1U<<11)+(1U<<12);
                return TableCase.Invariant;
            }
            set
            {
                this._flags&=0xFFFFFFFF^((1U<<11)+(1U<<12));
                switch (value)
                {
                    case TableCase.Normal:
                        this._flags+=1U<<11;
                        return;
                    case TableCase.Turkish:
                        this._flags+=1U<<12;
                        return;
                    case TableCase.Invariant:
                        this._flags+=(1U<<11)+(1U<<12);
                        return;
                }
            }
        }

        #endregion

        #region ToString y StrLen Chars

        /// <summary>
        /// Obtiene largo de cadena segn el primer cero encontrado o la cantidad de elementos actual.
        /// </summary>
        /// <param name="setLength">Si true, actualizar el tamao actual de la coleccin con el resultado.</param>
        /// <returns>Primer cero encontrado o cantidad de elementos actual.</returns>
        public int StrLen(bool setLength)
        {
            int l=this._size;
            int a=0;
            if (l!=0)
                fixed(char*p=this._items)
                    while(p[a]!=0 && ++a!=l);
            if (setLength && a!=l)
                this._size=a;
            return a;
        }


        /// <summary>
        /// Genera cadena mutable con los elementos de esta coleccin.
        /// </summary>
        /// <returns>Cadena mutable o vacia si la coleccin no contiene elementos.</returns>
        public override string ToString()
        {
            if (this._size==0)
                return Others.Empty;
            fixed(char*p=this._items)
                return Mutables.Create(p,this._size);
        }

        /// <summary>
        /// Genera cadena mutable con los elementos de esta coleccin.
        /// </summary>
        /// <param name="count">Largo de la cadena.</param>
        /// <returns>Cadena mutable o vacia si count es cero.</returns>
        public string ToString(int count)
        {
            if (count<0 || count>this._size)
                throw new ArgumentOutOfRangeException("count", count,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            
            if (count==0)
                return Others.Empty;
            
            fixed(char*p=this._items)
                return Mutables.Create(p,count);
        }

        /// <summary>
        /// Genera cadena mutable con los elementos de esta coleccin.
        /// </summary>
        /// <param name="startIndex">Inicio.</param>
        /// <param name="count">Largo de la cadena.</param>
        /// <returns>Cadena mutable o vacia si count es cero.</returns>
        public string ToString(int startIndex, int count)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            
            if (count==0)
                return Others.Empty;
            
            fixed(char*p=this._items)
                return Mutables.Create(p+startIndex,count);
        }


        #endregion

        #region Trim y RemoveQuotes Chars
       
        /// <summary>
        /// Efectua trim de carcteres en blanco a derecha e izquierda.
        /// </summary>
        public void Trim()
        {
            Trim(true,true);
        }

        /// <summary>
        /// Efectua Trim con el filtro de todos los carcteres en blanco. 
        /// </summary>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        public void Trim(bool left, bool right)
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                {
                    PChar pc=new PChar(p,this._size,left,right);
                    if (pc.Length!=this._size)
                    {
                        if (pc.Length!=0 && pc.Pointer!=p)
                            General.Copy(pc.Pointer,p,pc.Length);
                        this._size=pc.Length;
                    }
                }
            }
        }

        /// <summary>
        /// Efectua Trim.
        /// </summary>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        /// <param name="control">Incluye en filtro carcteres de control \0, \v y \f.</param>
        /// <param name="crlf">Incluye en filtro \r y \n.</param>
        /// <param name="lat1">Incluye en filtro los carcteres en blanco Latin1 (160 y 133).</param>
        /// <param name="sp">Incluye en filtro el espacio estndar (32).</param>
        /// <param name="uni">Incluye en filtro espacios unicode (\u1680 y superiores).</param>
        /// <param name="tab">Incluye en filtro el taburador (\t).</param>
        public void Trim(bool left, bool right, bool sp, bool lat1, bool uni, bool tab, bool crlf, bool control)
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                {
                    PChar pc=new PChar(p,this._size,left,right,sp,lat1,uni,tab,crlf,control);
                    if (pc.Length!=this._size)
                    {
                        if (pc.Length!=0 && pc.Pointer!=p)
                            General.Copy(pc.Pointer,p,pc.Length);
                        this._size=pc.Length;
                    }
                }
            }
        }

        /// <summary>
        /// Efectua Trim.
        /// </summary>
        /// <param name="pTrim">Puntero de carcteres de filtro.</param>
        /// <param name="lTrim">Largo del puntero de carcteres de filtro.</param>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        public void Trim(char * pTrim, int lTrim, bool left, bool right)
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                {
                    PChar pc=new PChar(p,this._size,pTrim,lTrim,left,right);
                    if (pc.Length!=this._size)
                    {
                        if (pc.Length!=0 && pc.Pointer!=p)
                            General.Copy(pc.Pointer,p,pc.Length);
                        this._size=pc.Length;
                    }
                }
            }
        }

        /// <summary>
        /// Efectua Trim.
        /// </summary>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        /// <param name="cTrim">Carcteres para Trim.</param>
        public void Trim(bool left, bool right, char[] cTrim)
        {
            if (this._size!=0)
            {
                int lTrim=cTrim.Length;
                if (lTrim!=0)
                {
                    fixed(char*p=this._items, pTrim=cTrim)
                    {
                        PChar pc=new PChar(p,this._size,pTrim,lTrim,left,right);
                        if (pc.Length!=this._size)
                        {
                            if (pc.Length!=0 && pc.Pointer!=p)
                                General.Copy(pc.Pointer,p,pc.Length);
                            this._size=pc.Length;
                        }
                    }
                }
            }
        }

        /// <summary>
        /// Efectua Trim.
        /// </summary>
        /// <param name="group">Grupo.</param>
        /// <param name="pch">Puntero a carcteres en categora custom.</param>
        /// <param name="lch">Largo del puntero de carcteres en categora custom.</param>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        public void Trim(CharGroup group, char * pch, int lch, bool left, bool right)
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                {
                    PChar pc=new PChar(p,this._size,group,pch,lch,left,right);
                    if (pc.Length!=this._size)
                    {
                        if (pc.Length!=0 && pc.Pointer!=p)
                            General.Copy(pc.Pointer,p,pc.Length);
                        this._size=pc.Length;
                    }
                }
            }
        }

        /// <summary>
        /// Efectua Trim.
        /// </summary>
        /// <param name="right">Trim derecho.</param>
        /// <param name="left">Trim izquierdo.</param>
        /// <param name="group">Grupo.</param>
        /// <param name="custom">Array de carcteres en categora custom.</param>
        public void Trim(CharGroup group, bool left, bool right, char[] custom)
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                {
                    PChar pc=new PChar(p,this._size,left,right,group,custom);
                    if (pc.Length!=this._size)
                    {
                        if (pc.Length!=0 && pc.Pointer!=p)
                            General.Copy(pc.Pointer,p,pc.Length);
                        this._size=pc.Length;
                    }
                }
            }
        }


        /// <summary>
        /// Efectua un reconocimiento simple de texto encomillado:
        /// - Efectua Trim al bloque.
        /// - Si encuentra comillas coincidentes al comienzo y al final del bloque resultante, las descarta.
        /// </summary>
        /// <param name="afterTrimR">Luego del procesamiento de comillas, efecta trim derecho.</param>
        /// <param name="afterTrimL">Luego del procesamiento de comillas, efecta trim izquierdo.</param>
        /// <param name="mode">Opciones de eliminacin de comillas.</param>
        public void RemoveQuotes(RemoveQuotesMode mode, bool afterTrimL, bool afterTrimR)
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                {
                    PChar pc=new PChar(p,this._size,mode,afterTrimL,afterTrimR);
                    if (pc.Length!=this._size)
                    {
                        if (pc.Length!=0 && pc.Pointer!=p)
                            General.Copy(pc.Pointer,p,pc.Length);
                        this._size=pc.Length;
                    }
                }
            }
        }


        #endregion

        #region AppendLine e InsertLine Chars

        /// <summary>
        /// Inserta un salto de lnea al final de la coleccin.
        /// </summary>
       public void AppendLine()
        {
            if (!this.Unix)
                this.AppendItems('\r','\n');
            else
                this.Append('\n');
        }

        /// <summary>
        /// Inserta un elemento seguido de un salto de lnea al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        public void AppendLine(char item)
        {
            if (!this.Unix)
                this.AppendItems(item,'\r','\n');
            else
                this.AppendItems(item,'\n');                
        }


        /// <summary>
        /// Inserta elementos seguido de un salto de lnea al final de la coleccin.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        public void AppendLine(char item, int count)
        {
            if (count < 2)
            {
                if (count==1)
                {
                    this.AppendLine(item);
                    return;
                }
                if (count==0)
                {
                    this.AppendLine();
                    return;
                }
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            bool unix=this.Unix;
            this.EnsureAppend(count+(unix?1:2));
            fixed(char*p=this._items)
            {
                General.Fill(p + this._size,count,item);
                this._size+=count;
                if (!unix)
                {
                    *(int*)(p+this._size)=Texts.CharsCrLf;
                    this._size+=2;
                }
                else
                    p[this._size++]='\n';
            }
        }


        /// <summary>
        /// Inserta el contenido del array al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        public void AppendLine(char[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.AppendLine(po, count);
            }
            else
                this.AppendLine();
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void AppendLine(char[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.AppendLine(po, count);
            }
            else
                this.AppendLine();
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void AppendLine(char[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.AppendLine(po + arrIndex, count);
            }
            else
                this.AppendLine();
        }

        /// <summary>
        /// Inserta el contenido del array al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        public void AppendLine(char* arr, int count)
        {
            if (count < 2)
            {
                if (count==1)
                {
                    this.AppendLine(*arr);
                    return;
                }
                if (count==0)
                {
                    this.AppendLine();
                    return;
                }
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            bool unix=this.Unix;
            this.EnsureAppend(count+(unix?1:2));
            fixed(char*p=this._items)
            {
                General.Copy(arr, p + this._size, count);
                this._size+=count;
                if (!unix)
                {
                    *(int*)(p+this._size)=Texts.CharsCrLf;
                    this._size+=2;
                }
                else
                    p[this._size++]='\n';
            }
        }

        /// <summary>
        /// Inserta un salto de lnea en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex)
        {
            if (!this.Unix)
                this.InsertItems(startIndex,'\r','\n');
            else
                this.Insert(startIndex, '\n');
        }

        /// <summary>
        /// Inserta un elemento seguido de un salto de lnea en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex,char item)
        {
            if (!this.Unix)
                this.InsertItems(startIndex,item,'\r','\n');
            else
                this.InsertItems(startIndex,item,'\n');                
        }


        /// <summary>
        /// Inserta elementos seguido de un salto de lnea en la ubicacin especificada.
        /// </summary>
        /// <param name="item">Elemento a insertar.</param>
        /// <param name="count">Cantidad de elementos a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex,char item, int count)
        {
            if (count < 2)
            {
                if (count==1)
                {
                    this.InsertLine(startIndex,item);
                    return;
                }
                if (count==0)
                {
                    this.InsertLine(startIndex);
                    return;
                }
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            if (!this.Unix)
            {
                this.Reserve(startIndex,count+2);
                this.Set(startIndex,item,count);
                this.SetItems(startIndex+count,'\r','\n');

            }
            else
            {
                this.Reserve(startIndex,count+1);
                this.Set(startIndex,item,count);
                this.Set(startIndex+count,'\n');
            }
        }


        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, char[] arr)
        {
            int count=arr.Length;
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.InsertLine(startIndex,po, count);
            }
            else
                this.InsertLine(startIndex);
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, char[] arr, int count)
        {
            if (arr.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.InsertLine(startIndex,po, count);
            }
            else
                this.InsertLine(startIndex);
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="arrIndex">Index en el array donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, char[] arr, int arrIndex, int count)
        {
            if (arrIndex < 0 || arr.Length < arrIndex + count)
                throw new ArgumentOutOfRangeException("arrIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=arr)
                    this.InsertLine(startIndex,po + arrIndex, count);
            }
            else
                this.InsertLine(startIndex);
        }

        /// <summary>
        /// Inserta el contenido del array en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="arr">Array a insertar.</param>
        /// <param name="count">Cantidad de elementos del array a copiar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, char* arr, int count)
        {
            if (!this.Unix)
            {
                this.Reserve(startIndex,count+2);
                this.Set(startIndex,arr,count);
                this.SetItems(startIndex+count,'\r','\n');
            }
            else
            {
                this.Reserve(startIndex,count+1);
                this.Set(startIndex,arr,count);
                this.Set(startIndex+count,'\n');
            }
        }

        #endregion

        #region ToUpper Chars

        /// <summary>
        /// Convierte los carcteres de esta coleccin a maysculas.
        /// </summary>
        public void ToUpper()
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToUpper(p,this._size,this.TableCase);
            }
        }

        /// <summary>
        /// Convierte carcteres de esta coleccin a maysculas.
        /// </summary>
        /// <param name="startIndex">Indice de comienzo de la conversin.</param>
        /// <param name="count">Cantidad de carcteres a convertir.</param>
        public void ToUpper(int startIndex, int count)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToUpper(p+startIndex,count,this.TableCase);
            }
        }

        /// <summary>
        /// Convierte los carcteres de esta coleccin a maysculas usando la cultura invariable.
        /// </summary>
        public void ToUpperInvariant()
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToUpperInvariant(p,this._size);
            }
        }

        /// <summary>
        /// Convierte carcteres de esta coleccin a maysculas usando la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Indice de comienzo de la conversin.</param>
        /// <param name="count">Cantidad de carcteres a convertir.</param>
        public void ToUpperInvariant(int startIndex, int count)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToUpperInvariant(p+startIndex,count);
            }
        }

        #endregion

        #region ToLower Chars

        /// <summary>
        /// Convierte los carcteres de esta coleccin a minsculas.
        /// </summary>
        public void ToLower()
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToLower(p,this._size,this.TableCase);
            }
        }

        /// <summary>
        /// Convierte carcteres de esta coleccin a minsculas.
        /// </summary>
        /// <param name="startIndex">Indice de comienzo de la conversin.</param>
        /// <param name="count">Cantidad de carcteres a convertir.</param>
        public void ToLower(int startIndex, int count)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToLower(p+startIndex,count,this.TableCase);
            }
        }

        /// <summary>
        /// Convierte los carcteres de esta coleccin a minsculas usando la cultura invariable.
        /// </summary>
        public void ToLowerInvariant()
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToLowerInvariant(p,this._size);
            }
        }

        /// <summary>
        /// Convierte carcteres de esta coleccin a minsculas usando la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Indice de comienzo de la conversin.</param>
        /// <param name="count">Cantidad de carcteres a convertir.</param>
        public void ToLowerInvariant(int startIndex, int count)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToLowerInvariant(p+startIndex,count);
            }
        }

        #endregion

        #region ToProper Chars

        /// <summary>
        /// Convierte los carcteres de esta coleccin a texto titulado.
        /// </summary>
        public void ToProper()
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToProper(p,this._size,this.TableCase);
            }
        }

        /// <summary>
        /// Convierte carcteres de esta coleccin a texto titulado.
        /// </summary>
        /// <param name="startIndex">Indice de comienzo de la conversin.</param>
        /// <param name="count">Cantidad de carcteres a convertir.</param>
        public void ToProper(int startIndex, int count)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToProper(p+startIndex,count,this.TableCase);
            }
        }

        /// <summary>
        /// Convierte los carcteres de esta coleccin a texto titulado.
        /// </summary>
        /// <param name="mPar">
        /// Opciones para la deteccin de prrafo y oraciones.
        /// - Nothing: Efectuar la conversion en todas las palabras y stopUpLow se aplica una sola vez
        /// a toda la cadena.
        /// - Sentence: Solo convierte la primer palabra de cada oracin. Toda la cadena es considerada
        /// un parrafo nico por lo que no se toman en cuenta los saltos de lnea como cambio de oracin.
        /// SingleLine y Doble: Solo convierte la primer palabra de cada oracin. Los saltos de lnea (simples o dobles)
        /// son considerados cambios de oracin.
        /// </param>
        /// <param name="stopUpLow">
        /// Si la primer letra de una oracin es mayscula y la segunda es minscula,
        /// no efectuar modificacin alguna a esa oracin (o a la cadena completa si mPar es Nothing).
        /// </param>
        public void ToProper(ParagraphMode mPar, bool stopUpLow)
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToProper(p,this._size,mPar,stopUpLow,this.TableCase);
            }
        }

        /// <summary>
        /// Convierte carcteres de esta coleccin a texto titulado.
        /// </summary>
        /// <param name="mPar">
        /// Opciones para la deteccin de prrafo y oraciones.
        /// - Nothing: Efectuar la conversion en todas las palabras y stopUpLow se aplica una sola vez
        /// a toda la cadena.
        /// - Sentence: Solo convierte la primer palabra de cada oracin. Toda la cadena es considerada
        /// un parrafo nico por lo que no se toman en cuenta los saltos de lnea como cambio de oracin.
        /// SingleLine y Doble: Solo convierte la primer palabra de cada oracin. Los saltos de lnea (simples o dobles)
        /// son considerados cambios de oracin.
        /// </param>
        /// <param name="stopUpLow">
        /// Si la primer letra de una oracin es mayscula y la segunda es minscula,
        /// no efectuar modificacin alguna a esa oracin (o a la cadena completa si mPar es Nothing).
        /// </param>
        /// <param name="startIndex">Indice de comienzo de la conversin.</param>
        /// <param name="count">Cantidad de carcteres a convertir.</param>
        public void ToProper(int startIndex, int count, ParagraphMode mPar, bool stopUpLow)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
            {
                fixed(char*p=this._items)
                    Texts.ToProper(p+startIndex,count,mPar,stopUpLow,this.TableCase);
            }
        }

        #endregion

        #region BlankToSpace Chars


        /// <summary>
        /// Convierte todos los carcteres en blanco a espacio estndar.
        /// </summary>
        public void BlankToSpace()
        {
            if (this._size!=0)
            {
                fixed(char*p=this._items)
                    TxtComp.ReplaceBlanks(p,this._size,' ');
            }
        }

        #endregion
 
        #region ReFormat Chars
        
        /// <summary>
        /// Reformatea en documento (versin mnima).
        /// </summary>
        /// <param name="trimLeft">
        /// Efectua Trim a la izquierda de cada lnea.
        /// </param>
        /// <param name="trimRight">
        /// Efectua Trim a la derecha de cada lnea.
        /// </param>
        /// <param name="noBlankLines">
        /// Ignora lneas vacias.
        /// </param>
        public void ReFormat(bool trimLeft, bool trimRight, bool noBlankLines)
        {

            
            if (this._size==0)
                return;

            bool trim, parraf;
            if (noBlankLines && trimRight)
            {
                parraf=true;
                trimRight=false;
                trim=trimLeft;
            }
            else
            {
                parraf=false;
                trim=trimLeft || trimRight;
            }

            
            // genera copia de lectura en stack o heap.
            Heap heap;
            char*p;
            int l=this._size,sumline=(this.Unix?1:2),a;

            // Determina el largo e inserta las lneas en cache.
            fixed(char*po=this._items)
            {
                if (&l>General.MinimumStack && l<General.MaxStackChars<<3)
                {
                    char*pstack=stackalloc char[l];
                    p=pstack;
                    heap=null;
                }
                else
                {
                    heap=new Heap(l<<1,false);
                    p=(char*)heap.Pointer;
                }
                General.Copy(po,p,l);
            }
            
            this._size=0;
            PChars Pclineas=new PChars(l<16?4:(l>>4)+4);
            PChar pc=new PChar(p,l),pc2;
            l=0;

            otra:
            
                if (parraf)
                    pc2=pc.FowardCrLf(noBlankLines);
                else
                    pc2=pc.FowardParagraph(false);
            
            if (pc2.Length==0)
            {
                if (noBlankLines || pc.Pointer==null)
                    goto fin;
            }
            else if (trim)
            {
                pc2=pc2.Trim(trimLeft,trimRight);
                if (noBlankLines && pc2.Length==0)
                    goto otra;
            }

            Pclineas.Append(pc2);
            l+=pc2.Length+sumline;
            goto otra;
            fin:
            
                if (l!=0)
                {
                    this.Capacity=l;
                    l=Pclineas.Length;
                    a=0;
                    fixed(PChar* plin=Pclineas.Array)
                    {
                        do
                        {
                            pc2=plin[a];
                            if (pc2.Length==0)
                                this.AppendLine();
                            else
                                this.AppendLine(pc2.Pointer,pc2.Length);
                        }while(++a!=l);
                    }
                }
            if (heap!=null)
                heap.Dispose();
        }

        /// <summary>
        /// Reformatea en documento (versin sin eliminacin de espacios duplicados ni carcteres basura).
        /// </summary>
        /// <param name="trimLeft">
        /// Efectua Trim a la izquierda de cada lnea.
        /// Si RemoveQuotes es true, este trim se realizar luego de remover las comillas.
        /// </param>
        /// <param name="trimRight">
        /// Efectua Trim a la derecha de cada lnea.
        /// Si RemoveQuotes es true, este trim se realizar luego de remover las comillas.
        /// </param>
        /// <param name="rMode">
        /// Modo de eliminacin de comillas. Si la remocin implica trim, se efectuar este trim inicial a todas las lneas,
        /// siendo trimLeft y trimRight posteriores a una remocin exitosa.
        /// </param>
        /// <param name="noBlankLines">
        /// Ignora lneas vacias.
        /// </param>
        /// <param name="indent">Sangra aplicable a todas las lneas.
        ///  Si es cero no se agregar sangria. Si wrap es diferente a cero,
        ///  el largo mximo de cada lnea ser de indent+wrap.
        ///  </param>
        /// <param name="pIndent">
        /// Puntero para la sangra, ignorado si indent es cero.
        /// Se copiar al stack si es necesario.
        /// Si es nulo e indent es diferente a cero, se crear un puntero con espacios.
        /// </param>
        /// <param name="normLength">
        /// Normaliza el largo de cada lnea (sin incluir la sangra) rellenandla o contrayendla si su largo
        /// es menor o mayor al especificado.
        /// 0 para deshabilitar. Si es negativo, el relleno se efectuar a la izquierda, si es positivo el relleno se
        /// realizar a la derecha.
        /// </param>
        /// <param name="normFill">
        /// Caracter de relleno para normLength.
        /// </param>
        /// <param name="tabToSp">
        /// Convierte Tabs en espacios.
        /// </param>
        /// <param name="sBlToSp">
        /// Convierte carcteres en blanco superiores en el espacio estndar (32).
        /// </param>
        /// <param name="fillBlank">Aplica sangra y normalizacin de largo a las lneas en blanco.</param>
        public void ReFormat(int indent, char* pIndent, int normLength, char normFill, bool fillBlank,
            bool trimLeft, bool trimRight, bool noBlankLines, RemoveQuotesMode rMode,
            bool sBlToSp, bool tabToSp)
        {
            
            if (this._size==0)
                return;

            // Parmetros...
            if (indent!=0)
            {
                if (indent<0)
                    throw new ArgumentOutOfRangeException("indent",indent,Errors.NEGATIVE);
                if (pIndent==null || pIndent>General.MaximumStack)
                {
                    char* pstack=stackalloc char[indent];
                    if (pIndent==null)
                        General.Fill(pstack,indent,' ');
                    else
                        General.Copy(pIndent,pstack,indent);
                    pIndent=pstack;
                }
            }

            bool normLeft;
            if (normLength<0)
            {
                normLength=-normLength;
                normLeft=true;
            }
            else
                normLeft=false;

            if (fillBlank && normLength==0 && indent==0)
                fillBlank=false;

            bool trim=trimLeft || trimRight || rMode!=RemoveQuotesMode.NoRemove;
            
            // genera copia de lectura en stack o heap.
            Heap heap;
            char*p;
            int l=this._size,sumline=(this.Unix?1:2)+indent+normLength,a;

            // Determina el largo e inserta las lneas en cache.
            fixed(char*po=this._items)
            {
                if (&l>General.MinimumStack && l<General.MaxStackChars<<3)
                {
                    char*pstack=stackalloc char[l];
                    p=pstack;
                    heap=null;
                }
                else
                {
                    heap=new Heap(l<<1,false);
                    p=(char*)heap.Pointer;
                }
                General.Copy(po,p,l);
            }
            
            this._size=0;
            PChars Pclineas=new PChars(l<16?4:(l>>4)+4);
            PChar pc=new PChar(p,l),pc2;
            l=0;

            otra:
                pc2=pc.FowardCrLf(noBlankLines);
            if (pc2.Length==0)
            {
                if (noBlankLines || pc.Pointer==null)
                    goto fin;
            }
            else if (trim)
            {
                pc2=new PChar(pc2.Pointer,pc2.Length,rMode,trimLeft,trimRight);
                if (noBlankLines && pc2.Length==0)
                    goto otra;
            }

            if (pc2.Length!=0)
            {
                if (pc2.Length>normLength)
                    pc2.Length=normLength;
                a=0;
                do
                {
                    switch(pc.Pointer[a])
                    {
                        case '\0':
                        case '\v':
                        case '\f':
                            pc.Pointer[a]=' ';
                            continue;
                        case '\t':
                            if (tabToSp)
                                pc.Pointer[a]=' ';
                            continue;
                        case '\xa0':
                        case '\x85':
                        case '\u1680':
                        case '\u2000':
                        case '\u2001':
                        case '\u2002':
                        case '\u2003':
                        case '\u2004':
                        case '\u2005':
                        case '\u2006':
                        case '\u2007':
                        case '\u2008':
                        case '\u2009':
                        case '\u200a':
                        case '\u200b':
                        case '\u2028':
                        case '\u2029':
                        case '\u202f':
                        case '\u3000':
                        case '\ufeff':
                            if (sBlToSp)
                                pc.Pointer[a]=' ';
                            continue;
                    }
                }while(++a!=pc2.Length);
            }
            Pclineas.Append(pc2);
            l+=sumline;
            if (normLength==0)
                l+=pc2.Length;
            goto otra;
            fin:
            
                if (l!=0)
                {
                    this.Capacity=l;
                    l=Pclineas.Length;
                    a=0;
                    fixed(PChar* plin=Pclineas.Array)
                    {
                        do
                        {
                            pc2=plin[a];
                            if (pc2.Length==0)
                            {
                                if (fillBlank)
                                {
                                    if (indent!=0)
                                        this.Append(pIndent,indent);
                                    if (normLength!=0)
                                        this.Append(normFill,normLength);
                                }
                                this.AppendLine();
                                continue;
                            }
                            if (indent!=0)
                            {
                                this.Append(pIndent,indent);
                            }
                            if (normLength!=0 && pc2.Length!=normLength)
                            {
                                if (normLeft)
                                {
                                    this.Append(normFill,normLength-pc2.Length);
                                    this.Append(pc2.Pointer,pc2.Length);
                                }
                                else
                                {
                                    this.Append(pc2.Pointer,pc2.Length);
                                    this.Append(normFill,normLength-pc2.Length);
                                }
                                this.AppendLine();
                            }
                            else
                                this.AppendLine(pc2.Pointer,pc2.Length);

                        }while(++a!=l);
                    
                    }
                }
            if (heap!=null)
                heap.Dispose();
        }


        /// <summary>
        /// Reformatea en documento (orientado a archivos de datos).
        /// </summary>
        /// <param name="trimLeft">
        /// Efectua Trim a la izquierda de cada lnea.
        /// Si RemoveQuotes es true, este trim se realizar luego de remover las comillas.
        /// </param>
        /// <param name="trimRight">
        /// Efectua Trim a la derecha de cada lnea.
        /// Si RemoveQuotes es true, este trim se realizar luego de remover las comillas.
        /// </param>
        /// <param name="rMode">
        /// Modo de eliminacin de comillas. Si la remocin implica trim, se efectuar este trim inicial a todas las lneas,
        /// siendo trimLeft y trimRight posteriores a una remocin exitosa.
        /// </param>
        /// <param name="noBlankLines">
        /// Ignora lneas vacias.
        /// </param>
        /// <param name="indent">Sangra aplicable a todas las lneas.
        ///  Si es cero no se agregar sangria. Si wrap es diferente a cero,
        ///  el largo mximo de cada lnea ser de indent+wrap.
        ///  </param>
        /// <param name="pIndent">
        /// Puntero para la sangra, ignorado si indent es cero.
        /// Se copiar al stack si es necesario.
        /// Si es nulo e indent es diferente a cero, se crear un puntero con espacios.
        /// </param>
        /// <param name="normLength">
        /// Normaliza el largo de cada lnea (sin incluir la sangra) rellenandla o contrayendla si su largo
        /// es menor o mayor al especificado.
        /// 0 para deshabilitar. Si es negativo, el relleno se efectuar a la izquierda, si es positivo el relleno se
        /// realizar a la derecha.
        /// </param>
        /// <param name="normFill">
        /// Caracter de relleno para normLength.
        /// </param>
        /// <param name="noDupSp">
        /// Elimina espacios duplicados y la aparicin de espacios junto a Tabs.
        /// </param>
        /// <param name="tabToSp">
        /// Convierte Tabs en espacios.
        /// </param>
        /// <param name="sBlToSp">
        /// Convierte carcteres en blanco superiores en el espacio estndar (32).
        /// </param>
        /// <param name="fillBlank">Aplica sangra y normalizacin de largo a las lneas en blanco.</param>
        public void ReFormat(int indent, char* pIndent, int normLength, char normFill, bool fillBlank,
            bool trimLeft, bool trimRight, bool noBlankLines, RemoveQuotesMode rMode,
            bool sBlToSp, bool tabToSp, bool noDupSp)
        {
            
            if (this._size==0)
                return;

            // Parmetros...
            if (indent!=0)
            {
                if (indent<0)
                    throw new ArgumentOutOfRangeException("indent",indent,Errors.NEGATIVE);
                if (pIndent==null || pIndent>General.MaximumStack)
                {
                    char* pstack=stackalloc char[indent];
                    if (pIndent==null)
                        General.Fill(pstack,indent,' ');
                    else
                        General.Copy(pIndent,pstack,indent);
                    pIndent=pstack;
                }
            }

            bool normLeft;
            if (normLength<0)
            {
                normLength=-normLength;
                normLeft=true;
            }
            else
                normLeft=false;

            if (fillBlank && normLength==0 && indent==0)
                fillBlank=false;

            bool trim=trimLeft || trimRight || rMode!=RemoveQuotesMode.NoRemove,antBl;
            
            // genera copia de lectura en stack o heap.
            Heap heap;
            char*p;
            int l=this._size,sumline=(this.Unix?1:2)+indent+normLength,a,b;

            // Determina el largo e inserta las lneas en cache.
            fixed(char*po=this._items)
            {
                if (&l>General.MinimumStack && l<General.MaxStackChars<<3)
                {
                    char*pstack=stackalloc char[l];
                    p=pstack;
                    heap=null;
                }
                else
                {
                    heap=new Heap(l<<1,false);
                    p=(char*)heap.Pointer;
                }
                General.Copy(po,p,l);
            }
            
            this._size=0;
            PChars Pclineas=new PChars(l<16?4:(l>>4)+4);
            PChar pc=new PChar(p,l),pc2;
            l=0;

            otra:
                pc2=pc.FowardCrLf(noBlankLines);
            if (pc2.Length==0)
            {
                if (noBlankLines || pc.Pointer==null)
                    goto fin;
            }
            else if (trim)
            {
                pc2=new PChar(pc2.Pointer,pc2.Length,rMode,trimLeft,trimRight);
                if (noBlankLines && pc2.Length==0)
                    goto otra;
            }

            b=0;
            if (pc2.Length!=0)
            {
                a=0;
                antBl=false;
                do
                {
                    switch(pc.Pointer[a])
                    {
                        case ' ':
                            if (!noDupSp || !antBl)
                            {
                                pc.Pointer[b++]=' ';
                                antBl=true;
                            }
                            continue;
                        case '\t':
                            if (tabToSp)
                                goto case ' ';
                            if (!noDupSp || !antBl)
                            {
                                pc.Pointer[b++]='\t';
                                antBl=true;
                            }
                            continue;
                        case '\xa0':
                        case '\x85':
                        case '\u1680':
                        case '\u2000':
                        case '\u2001':
                        case '\u2002':
                        case '\u2003':
                        case '\u2004':
                        case '\u2005':
                        case '\u2006':
                        case '\u2007':
                        case '\u2008':
                        case '\u2009':
                        case '\u200a':
                        case '\u200b':
                        case '\u2028':
                        case '\u2029':
                        case '\u202f':
                        case '\u3000':
                        case '\ufeff':
                            if (sBlToSp)
                                goto case ' ';
                            if (!noDupSp || !antBl)
                            {
                                pc.Pointer[b++]=pc.Pointer[a];
                                antBl=true;
                            }
                            continue;
                        case '\0':
                        case '\v':
                        case '\f':
                            continue;
                        default:
                            if (antBl)
                                antBl=false;
                            pc.Pointer[b++]=pc.Pointer[a];
                            continue;
                    }
                }while(++a!=pc2.Length);
                if (b==0 && noBlankLines)
                {
                    goto otra;
                }
                if (b>normLength)
                    b=normLength;
                pc2.Length=b;
            }
            Pclineas.Append(pc2);
            l+=sumline;
            if (normLength==0)
                l+=b;
            goto otra;
            fin:
            
                if (l!=0)
                {
                    this.Capacity=l;
                    l=Pclineas.Length;
                    a=0;
                    fixed(PChar* plin=Pclineas.Array)
                    {
                        do
                        {
                            pc2=plin[a];
                            if (pc2.Length==0)
                            {
                                if (fillBlank)
                                {
                                    if (indent!=0)
                                        this.Append(pIndent,indent);
                                    if (normLength!=0)
                                        this.Append(normFill,normLength);
                                }
                                this.AppendLine();
                                continue;
                            }
                            if (indent!=0)
                            {
                                this.Append(pIndent,indent);
                            }
                            if (normLength!=0 && pc2.Length!=normLength)
                            {
                                if (normLeft)
                                {
                                    this.Append(normFill,normLength-pc2.Length);
                                    this.Append(pc2.Pointer,pc2.Length);
                                }
                                else
                                {
                                    this.Append(pc2.Pointer,pc2.Length);
                                    this.Append(normFill,normLength-pc2.Length);
                                }
                                this.AppendLine();
                            }
                            else
                                this.AppendLine(pc2.Pointer,pc2.Length);

                        }while(++a!=l);
                    
                    }
                }
            if (heap!=null)
                heap.Dispose();
        }

        /// <summary>
        /// Reformatea documento de texto.
        /// </summary>
        /// <param name="par">Opciones de formato de parrafo.</param>
        /// <param name="wrap">
        /// Mximo de carcteres por lnea: Mnimo 8, 0 para deshabilitar.
        /// La divisin de lneas se efectua de acuerdo a estas reglas:
        /// - No hace nada si el largo de lnea es menor o igual a wrap, o wrap es 0.
        /// - El proceso de wrap se realiza al finalizar todas las otras depuraciones.
        /// - Buscar el ltimo caracter en blanco dentro del rango.
        ///   El rango es variable y estar entre el 50% de wrap y wrap.
        /// - Si esto no resulta, la lnea se dividir arbitrariamente (usndo guionado).
        /// </param>
        /// <param name="trimLeft">
        /// Efectua Trim a la izquierda de cada lnea.
        /// Independientemente a este parametro,
        /// siempre har trim a la derecha.
        /// Por otra parte, aunque el parametro sea false har trim
        /// de todos modos si la cantidad de carcteres en blanco a
        /// la izquierda supera o iguala al rango de wrap
        /// (en caso de que wrap sea diferente a 0).
        /// </param>
        /// <param name="noDupSp">
        /// Elimina espacios duplicados y la aparicin de espacios junto a Tabs.
        /// </param>
        /// <param name="tabToSp">
        /// Convierte Tabs en espacios.
        /// </param>
        /// <param name="indent">Sangra aplicable a todas las lneas.
        ///  Si es cero no se agregar sangria. Si wrap es diferente a cero,
        ///  el largo mximo de cada lnea ser de indent+wrap.
        ///  </param>
        /// <param name="pIndent">
        /// Puntero para la sangra, ignorado si indent es cero.
        /// Se copiar al stack si es necesario.
        /// Si es nulo e indent es diferente a cero, se crear un puntero con espacios.
        /// </param>
        /// <param name="sBlToSp">
        /// Convierte carcteres en blanco superiores (Latin1 y Unicode) en el espacio estndar (32).
        /// </param>
        public void ReFormat(ParagraphFmt par, int wrap, int indent, char* pIndent, bool trimLeft, bool noDupSp, bool tabToSp, bool sBlToSp)
        {

            if (this._size==0)
                return;

            // Parmetros...
            if (indent!=0)
            {
                if (indent<0)
                    throw new ArgumentOutOfRangeException("indent",indent,Errors.NEGATIVE);
                if (pIndent==null || pIndent>General.MaximumStack)
                {
                    char* pstack=stackalloc char[indent];
                    if (pIndent==null)
                        General.Fill(pstack,indent,' ');
                    else
                        General.Copy(pIndent,pstack,indent);
                    pIndent=pstack;
                }
            }
                       
            int medioWrap;
            if(wrap==0)
                medioWrap=0;
            else if (wrap<8) 
                throw new ArgumentOutOfRangeException("wrap",wrap,Errors.COL_WRAP);
            else if (wrap>39) 
                medioWrap=wrap-12;
            else 
                medioWrap=wrap-(wrap>>2)-2;

            

            // genera copia de lectura en stack o heap.
            Heap heap;
            char*p;
            int l=this._size,sumline=(this.Unix?1:2)+indent;

            // Determina el largo e inserta las lneas en cache.
            fixed(char*po=this._items)
            {
                if (&l>General.MinimumStack && l<General.MaxStackChars<<3)
                {
                    char*pstack=stackalloc char[l];
                    p=pstack;
                    heap=null;
                }
                else
                {
                    heap=new Heap(l<<1,false);
                    p=(char*)heap.Pointer;
                }
                General.Copy(po,p,l);
            }
            
            this._size=0;
            PChars Pclineas=new PChars(l<16?4:(l>>4)+4);
            PChar pc=new PChar(p,l),pc2,pc3;
            l=0;
            
            switch(par)
            {
                case ParagraphFmt.NoEmptysSingle:
                    while((pc2=pc.FowardParagraph(false)).Length!=0)
                    {
                        Pclineas.Append(pc2);
                        l+=pc2.Length+sumline;
                        if (wrap!=0 && pc2.Length>wrap)
                            l+=(pc2.Length/medioWrap)*(2+sumline);
                    }
                    break;
                case ParagraphFmt.NoEmptysDouble:
                    while((pc2=pc.FowardParagraph(false)).Length!=0)
                    {
                        Pclineas.Append(pc2);
                        Pclineas.AppendBlank();
                        l+=pc2.Length+sumline+2;
                        if (wrap!=0 && pc2.Length>wrap)
                            l+=(pc2.Length/medioWrap)*(2+sumline);

                    }
                    break;
                case ParagraphFmt.ParagraphSingle:
                    while((pc2=pc.FowardParagraph(true)).Length!=0)
                    {
                        Pclineas.Append(pc2);
                        l+=pc2.Length+sumline;
                        if (wrap!=0 && pc2.Length>wrap)
                            l+=(pc2.Length/medioWrap)*(2+sumline);

                    }
                    break;
                case ParagraphFmt.ParagraphDouble:
                    while((pc2=pc.FowardParagraph(true)).Length!=0)
                    {
                        Pclineas.Append(pc2);
                        Pclineas.AppendBlank();
                        l+=pc2.Length+sumline+2;
                        if (wrap!=0 && pc2.Length>wrap)
                            l+=(pc2.Length/medioWrap)*(2+sumline);

                    }
                    break;

                default:
                    while((pc2=pc.FowardParagraph(true)).Length!=0)
                    {
                        while((pc3=pc2.FowardParagraph(false)).Length!=0)
                        {
                            Pclineas.Append(pc3);
                            l+=pc3.Length+sumline;
                            if (wrap!=0 && pc3.Length>wrap)
                                l+=(pc3.Length/medioWrap)*(2+sumline);
                        }
                        Pclineas.AppendBlank();
                        l+=2;
                    }
                    break;
            }
            int total=Pclineas.Length;
            if (total==0)
            {
                if (heap!=null)
                    heap.Dispose();
                return;
            }
            this.Capacity=l+32;
            
            if (par!=ParagraphFmt.NoEmptysSingle && par!=ParagraphFmt.ParagraphSingle)
                --total;
            l=0;
            int a,b,comTabs,pText;
            bool antBl;
            
            fixed(PChar* pl=Pclineas.Array)
            {
                do
                {
                    pc=pl[l];
                    if(pc.Length==0)
                    {
                        this.AppendLine();
                        continue;
                    }
                    a=0;
                    b=0;
                    pText=-1;
                    comTabs=0;
                    antBl=true;
                    
                    do
                    {

                        switch(pc.Pointer[a])
                        {
                            case ' ':
                                if (pText==-1)
                                {
                                    if (!trimLeft)
                                        pc.Pointer[b++]=' ';
                                }
                                else if (!noDupSp || !antBl)
                                {
                                    pc.Pointer[b++]=' ';
                                    antBl=true;
                                }
                                continue;
                            case '\t':
                                if (tabToSp)
                                    goto case ' ';
                                if (pText==-1)
                                {
                                    if (!trimLeft && comTabs!=4)
                                    {
                                        pc.Pointer[b++]='\t';
                                        comTabs++;
                                    }
                                }
                                else if (!noDupSp || !antBl)
                                {
                                    pc.Pointer[b++]='\t';
                                    antBl=true;                               
                                }
                                continue;
                            case '\r':
                            case '\n':
                                if (!antBl)
                                {
                                    pc.Pointer[b++]=' ';
                                    antBl=true;
                                }
                                pc3=new PChar(pc.Pointer+ ++a,pc.Length-a,true,false);
                                a=(int)(pc3.Pointer-pc.Pointer);
                                continue;
                            case '\xa0':
                            case '\x85':
                            case '\u1680':
                            case '\u2000':
                            case '\u2001':
                            case '\u2002':
                            case '\u2003':
                            case '\u2004':
                            case '\u2005':
                            case '\u2006':
                            case '\u2007':
                            case '\u2008':
                            case '\u2009':
                            case '\u200a':
                            case '\u200b':
                            case '\u2028':
                            case '\u2029':
                            case '\u202f':
                            case '\u3000':
                            case '\ufeff':
                                if (sBlToSp)
                                    goto case ' ';
                                if (pText==-1)
                                {
                                    if (!trimLeft)
                                        pc.Pointer[b++]=pc.Pointer[a];
                                }
                                else if (!noDupSp || !antBl)
                                {
                                    pc.Pointer[b++]=pc.Pointer[a];
                                    antBl=true;
                                }
                                continue;
                            case '\0':
                            case '\v':
                            case '\f':
                                continue;
                            default:
                                if (pText==-1) 
                                    pText=b;
                                if (antBl)
                                    antBl=false;
                                pc.Pointer[b++]=pc.Pointer[a];
                                continue;
                        }
                    }while(++a!=pc.Length);

                    if (wrap!=0)
                    {
                        if (pText>=medioWrap) 
                        {
                            b-=pText;
                            pc.Pointer+=pText;
                            pText=0;
                        }
                        if (b>wrap)
                        {
                            this.wrap(pc.Pointer,b,medioWrap,wrap,pText,pIndent,indent);
                            continue;
                        }
                    }
                    if (indent!=0)
                        this.Append(pIndent,indent);
                    this.AppendLine(pc.Pointer,b);
                }while(++l!=total);
            }
            if (heap!=null)
                heap.Dispose();
        }


        private void wrap(char* pc, int l, int medioWrap,int wrap,int pText,char*pi,int li)
        {
            PChar ptrim,ptrim2;
            char* p;
            if (pText!=0)
            {
                p=pc+pText;
                l-=pText;
                wrap-=pText;
                medioWrap-=pText;
            }
            else
                p=pc;
            int puntoWrap, ubiGuion=wrap-2;
            do
            {
            
                puntoWrap=wrap;
                while(--puntoWrap>=medioWrap)
                {
                    switch(p[puntoWrap])
                    {
                        case ' ':
                        case '\t':
                        case '\xa0':
                        case '\x85':
                        case '\u1680':
                        case '\u2000':
                        case '\u2001':
                        case '\u2002':
                        case '\u2003':
                        case '\u2004':
                        case '\u2005':
                        case '\u2006':
                        case '\u2007':
                        case '\u2008':
                        case '\u2009':
                        case '\u200a':
                        case '\u200b':
                        case '\u2028':
                        case '\u2029':
                        case '\u202f':
                        case '\u3000':
                        case '\ufeff':
                            break;
                    }
                
                }

                if(puntoWrap<medioWrap)
                {
                    if (li!=0)
                        this.Append(pi,li);
                    if (pText!=0)
                        this.Append(pc,pText);
                    this.Append(p,ubiGuion);
                    if (!this.Unix)
                        this.AppendItems(' ','-','\r','\n');
                    else
                        this.AppendItems(' ','-','\n');
                    l-=ubiGuion;
                    p+=ubiGuion;
                }
                else
                {
                    ptrim=new PChar(p,puntoWrap,false,true);
                    if (ptrim.Length<medioWrap)
                    {
                        p[ptrim.Length++]=' ';
                        ptrim2=new PChar(p+(++puntoWrap),l-puntoWrap,puntoWrap==wrap-1,false);
                        General.Copy(ptrim2.Pointer,p+ptrim.Length,ptrim2.Length);
                        l=ptrim2.Length+ptrim.Length;
                    }
                    else
                    {
                        if (li!=0)
                            this.Append(pi,li);
                        if (pText!=0)
                            this.Append(pc,pText);
                        this.AppendLine(ptrim.Pointer,ptrim.Length);
                        p+=++puntoWrap;
                        l-=puntoWrap;
                    }
                }
                if (l<=wrap)
                {
                    if (l!=0)
                    {
                        if (li!=0)
                            this.Append(pi,li);
                        if (pText!=0)
                            this.Append(pc,pText);
                        this.AppendLine(p,l);
                    }
                    break;
                }

            }while(true);
        }

        #endregion

        #region Funciones NC Chars
        
        /// <summary>
        /// Retorna true si contiene el elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(char item, Ordinal comp, bool last)
        {
            if (this._size==0)
                return false;
            fixed (char* p=this._items)
                return TxtComp.Exist(p, this._size, item, comp, last);
        }

        /// <summary>
        /// Retorna true si contiene la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(char[] arr, Ordinal comp, bool last)
        {
            int lf;
            if (this._size==0 || this._size < (lf=arr.Length))
                return false;
            fixed (char* p=this._items, pf=arr)
                return TxtComp.Exist(p,this._size,pf,lf,false,comp,last);
        }

        /// <summary>
        /// Retorna true si contiene la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(char* pf, int lf, Ordinal comp, bool last)
        {
            if (this._size==0 || this._size<lf)
                return false;
            fixed (char* p=this._items)
                return TxtComp.Exist(p,this._size,pf,lf,false,comp,last);
        }


        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(char item, Ordinal comp)
        {
            if (this._size !=0)
            {
                fixed (char* p=this._items)
                    return TxtComp.Count(p, this._size, item,comp);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(char item, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
                return TxtComp.Count(p + startIndex, this._size - startIndex, item,comp);
        }

        /// <summary>
        /// Cuenta las apariciones del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOf(char item, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (char* p=this._items)
                return TxtComp.Count(p + startIndex, count, item, comp);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char[] arr, Ordinal comp)
        {
            int lf;
            if (this._size !=0 && (lf=arr.Length)<=this._size)
            {
                fixed (char* p=this._items, pf=arr)
                    return TxtComp.Count(p,this._size,pf,lf,false,comp);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char[] arr, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (char* p=this._items, pf=arr)
                return TxtComp.Count(p + startIndex, count,pf,lf,false,comp);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de a buscar en la coleccin.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char[] arr, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (char* p=this._items, pf=arr)
                return TxtComp.Count(p + startIndex, count,pf,lf,false,comp);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char* pf, int lf, Ordinal comp)
        {
            if (this._size !=0 && lf<=this._size)
            {
                fixed (char* p=this._items)
                    return TxtComp.Count(p,this._size,pf,lf,false,comp);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char*pf, int lf, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return 0;
            fixed (char* p=this._items)
                return TxtComp.Count(p + startIndex, count,pf,lf,false,comp);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de a buscar en la coleccin.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(char* pf,int lf, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return 0;
            fixed (char* p=this._items)
                return TxtComp.Count(p + startIndex, count,pf,lf,false,comp);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(Ordinal comp, char[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (char* p=this._items, pf=items)
                    return TxtComp.CountAny(p, this._size, pf, lf, false,comp);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, Ordinal comp, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items, pf=items)
                return TxtComp.CountAny(p + startIndex, this._size - startIndex, pf, lf, false,comp);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, int count, Ordinal comp, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (char* p=this._items, pf=items)
                return TxtComp.CountAny(p + startIndex, count, pf, lf, false,comp);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(char* pf, int lf, Ordinal comp)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (char* p=this._items)
                    return TxtComp.CountAny(p, this._size, pf, lf, false,comp);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, char* pf, int lf, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
                return TxtComp.CountAny(p + startIndex, this._size - startIndex, pf, lf, false,comp);
        }

        /// <summary>
        /// Cuenta las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de elementos encontrados.</returns>
        public int CountOfAny(int startIndex, int count, char* pf, int lf, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (char* p=this._items)
                return TxtComp.CountAny(p + startIndex, count, pf, lf, false,comp);
        }



        /// <summary>
        /// Efecta una comparacin con la coleccin especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="col2">Coleccin a comparar.</param>
        /// <returns>Retorna true si ambas coleccin contienen la misma cantidad de elementos y su contenido es igual.</returns>
        public bool Equals(Chars col2, Ordinal comp)
        {
            if (Object.ReferenceEquals(this,col2))
                return true;
            if (col2==null || this._size!=col2._size)
                return false;
            if (this._size<1)
                return this._size==0;
            fixed(char*p=this._items, p2=col2._items)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p,p2,false,this._size,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p,p2,false,this._size,TableCase.Invariant);
                    default:
                        return Comp.Equal(p,p2,this._size);
                }
            }
        }

        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="array2">Array a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de array2 son iguales a los la coleccin (comparando desde el inicio) o array2 tiene largo 0.</returns>
        public bool Equals(char[] array2, Ordinal comp)
        {
            int count=array2.Length;
            if (count<1)
                return count==0;
            if (count>this._size)
                return false;
            fixed(char*p=this._items, p2=array2)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p,p2,false,count,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p,p2,false,count,TableCase.Invariant);
                    default:
                        return Comp.Equal(p,p2,count);
                }
            }
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="array2">Array a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <returns>Solo retorna true si todos los elementos de array2 son iguales a los la coleccin (comparando desde startIndex) o array2 tiene largo 0.</returns>
        public bool Equals(char[] array2, int startIndex, Ordinal comp)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            int count=array2.Length;
            if (count<1)
                return count==0;
            if (startIndex+count>this._size)
                return false;
            fixed(char*p=this._items, p2=array2)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p+startIndex,p2,false,count,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p+startIndex,p2,false,count,TableCase.Invariant);
                    default:
                        return Comp.Equal(p+startIndex,p2,count);
                }
            }
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="p2">Array a comparar.</param>
        /// <param name="count">Cantidad de elementos a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de p2 son iguales a los la coleccin (comparando desde el inicio) o count es 0.</returns>
        public bool Equals(char* p2, int count, Ordinal comp)
        {
            if (count<1)
            {
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (count>this._size)
                return false;
            fixed(char*p=this._items)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p,p2,false,count,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p,p2,false,count,TableCase.Invariant);
                    default:
                        return Comp.Equal(p,p2,count);
                }
            }
        }


        /// <summary>
        /// Efecta una comparacin con el array especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="p2">Array a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <param name="count">Cantidad de elementos a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de p2 son iguales a los la coleccin (comparando desde el startIndex) o count es 0.</returns>
        public bool Equals(char* p2, int startIndex, int count, Ordinal comp)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            if (count<1)
            {
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (startIndex+count>this._size)
                return false;
            fixed(char*p=this._items)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p+startIndex,p2,false,count,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p+startIndex,p2,false,count,TableCase.Invariant);
                    default:
                        return Comp.Equal(p+startIndex,p2,count);
                }
            }
        }




        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(char item, Ordinal comp)
        {
            if (this._size !=0)
            {
                fixed (char* p=this._items)
                    return TxtComp.IndexOf(p, this._size, item,comp,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char item, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOf(p + startIndex, this._size - startIndex, item,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char item, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOf(p + startIndex, count, item,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(char[] arr, Ordinal comp)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=arr.Length))
            {
                fixed (char* p=this._items, pf=arr)
                    return TxtComp.IndexOf(p, this._size,pf,lf,false,comp,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char[] arr, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=arr)
                ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char[] arr, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=arr)
                ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(char* pf, int lf, Ordinal comp)
        {

            if (this._size !=0 && this._size>=lf)
            {
                fixed (char* p=this._items)
                    return TxtComp.IndexOf(p, this._size,pf,lf,false,comp,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char* pf,int lf, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(char* pf,int lf, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOfAny(Ordinal comp, char[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (char* p=this._items, pf=items)
                    return TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, Ordinal comp, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items, pf=items)
                ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf,false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, int count, Ordinal comp, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            int ret;
            fixed (char* p=this._items, pf=items)
                ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOfAny(char* pf, int lf, Ordinal comp)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (char* p=this._items)
                    return TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, char* pf, int lf, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOfAny(int startIndex, int count, char* pf, int lf, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(char item, Ordinal comp)
        {
            if (this._size !=0)
            {
                fixed (char* p=this._items)
                    return TxtComp.IndexOf(p, this._size, item,comp,true);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char item, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOf(p + startIndex, this._size - startIndex, item,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char item, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOf(p + startIndex, count, item,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(char[] arr, Ordinal comp)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=arr.Length))
            {
                fixed (char* p=this._items, pf=arr)
                    return TxtComp.IndexOf(p, this._size, pf, lf, false,comp,true);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char[] arr, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=arr)
                ret=TxtComp.IndexOf(p + startIndex, count,pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char[] arr, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=arr)
                ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }


        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(char* pf, int lf, Ordinal comp)
        {
            if (this._size !=0 && this._size>=lf)
            {
                fixed (char* p=this._items)
                    return TxtComp.IndexOf(p, this._size, pf, lf, false,comp,true);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char* pf, int lf, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOf(p + startIndex, count,pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(char* pf, int lf, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOfAny(Ordinal comp, char[] items)
        {
            int lf=items.Length;
            if (lf !=0 && this._size !=0)
            {
                fixed (char* p=this._items, pf=items)
                    return TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,true);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, Ordinal comp, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items, pf=items)
                ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, int count, Ordinal comp, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            int ret;
            fixed (char* p=this._items, pf=items)
                ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOfAny(char* pf, int lf, Ordinal comp)
        {
            if (lf > 0 && this._size !=0)
            {
                fixed (char* p=this._items)
                    return TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,true);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, char* pf, int lf, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a buscar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOfAny(int startIndex, int count, char* pf, int lf, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            int ret;
            fixed (char* p=this._items)
                ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }



        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char item, Ordinal comp)
        {
            if (this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p, this._size, item,comp,true);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char item, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, this._size - startIndex, item,comp,true);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char item, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, item,comp,true);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char[] arr, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=arr.Length))
                return -1;

            fixed (char* p=this._items, pf=arr)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char[] arr, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (char* p=this._items, pf=arr)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char[] arr, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (char* p=this._items, pf=arr)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char* pf, int lf, Ordinal comp)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char* pf, int lf, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(char* pf, int lf, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }


        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(Ordinal comp, char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;

            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, Ordinal comp, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, int count, Ordinal comp, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(char* pf, int lf, Ordinal comp)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, char* pf, int lf, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOfAny(int startIndex, int count, char* pf, int lf, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }


        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(char item, Ordinal comp)
        {
            if (this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p, this._size, item,comp,false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char item, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, this._size - startIndex, item,comp,false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char item, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, item,comp,false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(char[] arr, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=arr.Length))
                return -1;

            fixed (char* p=this._items, pf=arr)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char[] arr, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (char* p=this._items, pf=arr)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char[] arr, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return -1;
            fixed (char* p=this._items, pf=arr)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }

        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(char* pf, int lf, Ordinal comp)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char* pf, int lf, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(char* pf, int lf, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOfAny(Ordinal comp, char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;

            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, Ordinal comp, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, int count, Ordinal comp, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOfAny(char* pf, int lf, Ordinal comp)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveAt(ret);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, char* pf, int lf, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOfAny(int startIndex, int count, char* pf, int lf, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveAt(ret +=startIndex);
                return ret;
            }
        }



        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(char item, char rep, Ordinal comp)
        {
            if (this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p, this._size, item,comp,true);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(char item, char rep, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, this._size - startIndex, item,comp,true);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOf(char item, char rep, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, item,comp,true);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char[] f, char[] r, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char[] f, char[] r, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,true);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char[] f, char[] r, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char* pf, int lf, char* pr, int lr, Ordinal comp)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.ReplaceRange(ret, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char* pf, int lf, char* pr, int lr, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,true);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(char* pf, int lf, char* pr, int lr, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(char rep, Ordinal comp, char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;

            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, char rep, Ordinal comp, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,true);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, int count, char rep, Ordinal comp, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(char rep, char* pf, int lf, Ordinal comp)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, char rep, char* pf, int lf, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,true);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceLastOfAny(int startIndex, int count, char rep, char* pf, int lf, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOf(char item, char rep, Ordinal comp)
        {
            if (this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p, this._size, item,comp,false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(char item, char rep, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, this._size - startIndex, item,comp,false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOf(char item, char rep, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, item,comp,false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char[] f, char[] r, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char[] f, char[] r, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char[] f, char[] r, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char* pf, int lf, char* pr, int lr, Ordinal comp)
        {
            if (this._size==0 || this._size<lf)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.ReplaceRange(ret, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char* pf, int lf, char* pr, int lr, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(char* pf, int lf, char* pr, int lr, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.ReplaceRange(ret +=startIndex, lf, pr, lr);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(char rep, Ordinal comp, char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;

            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, char rep, Ordinal comp, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, int count, char rep, Ordinal comp, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return -1;
            fixed (char* p=this._items, pf=items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Ubicacin del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(char rep, char* pf, int lf, Ordinal comp)
        {
            if (lf < 1 || this._size==0)
                return -1;

            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    p[ret]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, char rep, char* pf, int lf, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, this._size - startIndex, pf, lf, false,comp,false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de alguno de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento reemplazado o -1.</returns>
        public int ReplaceOfAny(int startIndex, int count, char rep, char* pf, int lf, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return -1;
            fixed (char* p=this._items)
            {
                int ret=TxtComp.IndexOfAny(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    p[ret +=startIndex]=rep;
                return ret;
            }
        }


        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los del array de comparacin.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los del array de comparacin.</returns>
        public bool EndsWith(char[] array2, Ordinal comp)
        {

            int count=array2.Length;
            if (count<1)
                return count==0;
            if (count>this._size)
                return false;
            fixed(char*p=this._items, p2=array2)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p+(this._size-count),p2,false,count,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p+(this._size-count),p2,false,count,TableCase.Invariant);
                    default:
                        return Comp.Equal(p+(this._size-count),p2,count);
                }
            }
        }

        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los del puntero de comparacin.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="p2">Puntero de comparacin.</param>
        /// <param name="count">Largo del puntero de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los del puntero de comparacin.</returns>
        public bool EndsWith(char* p2, int count, Ordinal comp)
        {
            if (count<1)
            {
                if (count==0)
                    return true;
                throw new ArgumentOutOfRangeException("count",count,Errors.NEGATIVE);
            }
            if (count>this._size)
                return false;
            fixed(char*p=this._items)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p+(this._size-count),p2,false,count,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p+(this._size-count),p2,false,count,TableCase.Invariant);
                    default:
                        return Comp.Equal(p+(this._size-count),p2,count);
                }
            }
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los del array de comparacin.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="array2">Array de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los del array de comparacin.</returns>
        public bool StartsWith(char[] array2, Ordinal comp)
        {
            return Equals(array2,comp);
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los del puntero de comparacin.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="p2">Puntero de comparacin.</param>
        /// <param name="count">Largo del puntero de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los del puntero de comparacin.</returns>
        public bool StartsWith(char* p2, int count, Ordinal comp)
        {
            return Equals(p2,count,comp);
        }



        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char item, Ordinal comp)
        {
            if (this._size==0)
                return 0;

            fixed (char* p=this._items)
            {
                return TxtComp.Remove(false, p, p, this._size, out this._size, &item, 1, false,comp);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char item, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=TxtComp.Remove(false, po, po, this._size - startIndex, out lFinal, &item,1, false,comp);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char item, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=TxtComp.Remove(false, po, po, count, out lFinal, &item, 1, false, comp);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char[] arr, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=arr.Length))
                return 0;

            fixed (char* p=this._items, pf=arr)
            {
                return TxtComp.Remove(false, p, p, this._size, out this._size, pf , lf, false , comp);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char[] arr, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (char* p=this._items, pf=arr)
            {
                char* po=p + startIndex;
                int ret=TxtComp.Remove(false, po, po, count, out count,pf ,lf, false,comp);
                if (ret!=0)
                    this._size -=ret*lf;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="arr">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char[] arr, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=arr.Length))
                return 0;
            fixed (char* p=this._items, pf=arr)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=TxtComp.Remove(false, po, po, count, out lFinal, pf, lf, false,comp);
                if (ret!=0)
                {

                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret*lf;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char* pf, int lf, Ordinal comp)
        {
            if (this._size==0 || this._size<lf)
                return 0;

            fixed (char* p=this._items)
            {
                return TxtComp.Remove(false, p, p, this._size, out this._size, pf , lf, false , comp);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char* pf, int lf, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex;
            if (count==0 || count<lf)
                return 0;
            fixed (char* p=this._items)
            {
                char* po=p + startIndex;
                int ret=TxtComp.Remove(false, po, po, count, out count,pf ,lf, false,comp);
                this._size -=ret*lf;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de elementos a buscar.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(char* pf, int lf, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || count<lf)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=TxtComp.Remove(false, po, po, count, out lFinal, pf, lf, false,comp);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret*lf;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(Ordinal comp, char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                return TxtComp.RemoveAny(false, p, p, this._size, out this._size, pf, lf, false, comp);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, Ordinal comp, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=TxtComp.RemoveAny(false, po, po, this._size - startIndex, out lFinal, pf, lf, false, comp);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, int count, Ordinal comp, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=TxtComp.RemoveAny(false, po, po, count, out lFinal, pf, lf, false, comp);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(char* pf, int lf, Ordinal comp)
        {
            if (lf < 1 || this._size==0)
                return 0;

            fixed (char* p=this._items)
            {
                return TxtComp.RemoveAny(false, p, p, this._size, out this._size, pf, lf, false, comp);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, char* pf, int lf, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=TxtComp.RemoveAny(false, po, po, this._size - startIndex, out lFinal, pf, lf, false, comp);
                this._size -=ret;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a eliminar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAllAny(int startIndex, int count, char* pf, int lf, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (char* p=this._items)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=TxtComp.RemoveAny(false, po, po, count, out lFinal, pf, lf, false, comp);
                if (ret!=0)
                {
                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret;
                }
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char item, char rep, Ordinal comp)
        {
            if (this._size==0)
                return 0;

            fixed (char* p=this._items)
            {
                return TxtComp.ReplaceEqual(false,p,p, this._size, &item, &rep,1,comp);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char item, char rep, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
            {
                char* po=p + startIndex;
                return TxtComp.ReplaceEqual(false,po,po, this._size - startIndex, &item, &rep, 1, comp);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones del elemento especificado.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="item">Elemento a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char item, char rep, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0)
                return 0;
            fixed (char* p=this._items)
            {
                char* po=p + startIndex;
                return TxtComp.ReplaceEqual(false,po,po, count, &item, &rep, 1, comp);
            }
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char[] f, char[] r, Ordinal comp)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f,comp);
            int lf=f.Length;
            if (lf==0 || this._size<lf)
                return 0;
            fixed(char*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr,comp);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char[] f, char[] r, int startIndex, Ordinal comp)
        {
            return ReplaceAll(f,r,startIndex,this._size-startIndex,comp);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char[] f, char[] r, int startIndex, int count, Ordinal comp)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f,startIndex,count,comp);
            int lf=f.Length;
            if (lf==0 || count<lf)
                return 0;
            fixed(char*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr,startIndex,count,comp);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de bsqueda.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char* pf, int lf, char* pr, int lr, Ordinal comp)
        {
            if (this._size==0 || lf<1 || this._size<lf)
                return 0;
            fixed (char* p=this._items)
            {
                if (lf>=lr)
                {
                    if (lf==lr)
                        return TxtComp.ReplaceEqual(false,p,p,this._size,pf,pr,lf,comp);
                    return TxtComp.ReplaceMinor(false,p,p,this._size,out this._size,pf,lf,pr,lr,false,comp);
                }
                int ret;
                Ints ubics=TxtComp.Locations(null,0,p,this._size,pf,lf,false,comp);
                if (ubics==null || (ret=ubics.Length)==0)
                    return 0;
                
                int lFinal=((lr-lf)*ret)+this._size;
                if (this._cap<lFinal)
                {
                    char[] array2=newArraySum(lFinal-this._size);
                    fixed(char*pd=array2)
                        fixed(int*pCont=ubics.Array)
                            Comp.ReplaceLocations(pCont,ret,p,this._size,pd,lf,pr,lr,false,false);
                    this._items=array2;
                }
                else
                {
                    fixed(int*pCont=ubics.Array)
                        Comp.ReplaceLocations(pCont,ret,p,this._size,p,lf,pr,lr,false,false);
                }
                this._size=lFinal;
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de bsqueda.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char* pf, int lf, char* pr, int lr, int startIndex, Ordinal comp)
        {
            return ReplaceAll(pf,lf,pr,lr,startIndex,this._size-startIndex,comp);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Cadena de bsqueda.</param>
        /// <param name="lf">Largo del puntero de bsqueda.</param>
        /// <param name="pr">Cadena de reemplazo.</param>
        /// <param name="lr">Largo del puntero de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(char* pf, int lf, char* pr, int lr, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count==0 || lf<1 || count<lf)
                return 0;
            fixed (char* p=this._items)
            {
                char*po=p+startIndex;
                int ret, lFinal;
                if (lf>=lr)
                {
                    if (lf==lr)
                        return TxtComp.ReplaceEqual(false,po,po,count,pf,pr,lf,comp);
                    if ((ret=TxtComp.ReplaceMinor(false,po,po,count,out lFinal,pf,lf,pr,lr,false,comp))!=0)
                    {
                        General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                        this._size -=count-lFinal;
                    }
                    return ret;
                }
                Ints ubics=TxtComp.Locations(null,0,po,count,pf,lf,false,comp);
                if (ubics==null || (ret=ubics.Length)==0)
                    return 0;

                lFinal=((lr-lf)*ret)+this._size;
                if (this._cap<lFinal)
                {
                    char[] array2=newArraySum(lFinal-this._size);
                    fixed(char*pd=array2)
                    {
                        General.Copy(p,pd,startIndex);
                        fixed(int*pCont=ubics.Array)
                            Comp.ReplaceLocations(pCont,ret,po,this._size-startIndex,pd+startIndex,lf,pr,lr,false,false);
                    }
                    this._items=array2;
                }
                else
                {
                    fixed(int*pCont=ubics.Array)
                        Comp.ReplaceLocations(pCont,ret,po,this._size-startIndex,po,lf,pr,lr,false,false);
                }
                this._size=lFinal;
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(char rep, Ordinal comp, char[] items)
        {
            int lf=items.Length;
            if (lf==0 || this._size==0)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                return TxtComp.ReplaceAny(false,p,p, this._size, pf, lf, rep, comp);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, char rep, Ordinal comp, char[] items)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items, pf=items)
            {
                char*po=p + startIndex;
                return TxtComp.ReplaceAny(false,po,po, this._size - startIndex, pf, lf, rep, comp);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="items">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, int count, char rep, Ordinal comp, char[] items)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf=items.Length;
            if (lf==0 || count==0)
                return 0;
            fixed (char* p=this._items, pf=items)
            {

                char*po=p + startIndex;
                return TxtComp.ReplaceAny(false,po,po, count, pf, lf, rep, comp);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(char rep, char* pf, int lf, Ordinal comp)
        {
            if (lf < 1 || this._size==0)
                return 0;

            fixed (char* p=this._items)
            {
                return TxtComp.ReplaceAny(false,p,p, this._size, pf, lf, rep, comp);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, char rep, char* pf, int lf, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || startIndex==this._size)
                return 0;
            fixed (char* p=this._items)
            {
                char*po=p + startIndex;
                return TxtComp.ReplaceAny(false,po,po, this._size - startIndex, pf, lf, rep, comp);
            }
        }

        /// <summary>
        /// Reemplaza todas las apariciones de los elementos especificados.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="pf">Elementos a reemplazar.</param>
        /// <param name="lf">Cantidad de elementos de bsqueda.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="rep">Elemento de reemplazo.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAllAny(int startIndex, int count, char rep, char* pf, int lf, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (lf < 1 || count==0)
                return 0;
            fixed (char* p=this._items)
            {
                char*po=p + startIndex;
                return TxtComp.ReplaceAny(false,po,po, count, pf, lf, rep, comp);
            }
        }


        #endregion

        #region Constructores Strings

        /// <summary>
        /// Inicializa el objeto copiando la cadena de entrada.
        /// </summary>
        /// <param name="str">Cadena (puede ser null).</param>
        public Chars(string str):this(str,0,str==null?0:str.Length,sumCap){}


        /// <summary>
        /// Inicializa el objeto copiando la cadena de entrada.
        /// </summary>
        /// <param name="str">Cadena. Si es null, count deber ser 0.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public Chars(string str, int count):this(str,0,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando la cadena de entrada.
        /// </summary>
        /// <param name="str">Cadena. Si es null, strIndex y count debern ser 0.</param>
        /// <param name="strIndex">Index de inicio de la copia en la cadena origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public Chars(string str, int strIndex, int count):this(str,strIndex,count,sumCap){}

        /// <summary>
        /// Inicializa el objeto copiando la cadena de entrada.
        /// </summary>
        /// <param name="str">Cadena. Si es null, strIndex y count debern ser 0.</param>
        /// <param name="strIndex">Index de inicio de la copia en la cadena origen.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="extraCap">Capacidad extra para agregar al array subyacente.</param>
        public Chars(string str, int strIndex, int count, int extraCap)
        {
            if (count<1)
            {
                if (count==0 && strIndex==0)
                {
                    if (extraCap>0)
                    {
                        this._cap=Calc.AddIfOdd(extraCap);
                        this._items=new char[this._cap];
                    }
                    return;
                }
            }
            else if (strIndex>-1 && strIndex+count<=str.Length)
            {
                this._items=new char[this._cap=Calc.AddIfOdd(extraCap+count)];
                fixed(char*po=str,pd=this._items)
                    General.Copy(po+strIndex,pd,count);
                this._size=count;
            }
            throw new ArgumentOutOfRangeException("strIndex,count,extraCap", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
        }

        #endregion 

        #region Propiedades Strings

        /// <summary>
        /// Obtiene una copia compacta al largo actual de la coleccin.
        /// Al establecer, copia los elementos de la instancia actual al array subyacente o, en caso que la
        /// cadena de entrada sea de capacidad mayor, genera una nueva instancia con la copia de los elementos.
        /// Setea la cantidad de elementos en el largo del nuevo array.
        /// </summary>
        public string StringCopy
        {
            get
            {
                return this.ToString();
            }
            set
            {
                this._size=value==null?0:value.Length;
                if (this._size !=0)
                {
                    if (this._cap < this._size)
                        this._items=newArrayMin(this._size);
                    fixed (char* po=value, pd=this._items)
                        General.Copy(po, pd, this._size);
                }
            }
        }

        #endregion

        #region Metodos Administrativos Strings

        /// <summary>
        /// Inserta el contenido de la cadena al final de la coleccin.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        public void Append(string str)
        {

            int count=str==null?0:str.Length;
            if (count !=0)
            {
                fixed (char* po=str)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido de la cadena al final de la coleccin.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="count">Cantidad de elementos de la cadena a copiar.</param>
        public void Append(string str, int count)
        {
            if (str.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=str)
                    this.Append(po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido de la cadena al final de la coleccin.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="strIndex">Index en la cadena donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Append(string str, int strIndex, int count)
        {
            if (strIndex < 0 || str.Length < strIndex + count)
                throw new ArgumentOutOfRangeException("strIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=str)
                    this.Append(po + strIndex, count);
            }
        }


        /// <summary>
        /// Inserta el contenido de la cadena en la ubicacin especificada.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Insert(int startIndex, string str)
        {
            int count=str==null?0:str.Length;
            if (count !=0)
            {
                fixed (char* po=str)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido de la cadena en la ubicacin especificada.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio de la cadena.</param>
        public void Insert(int startIndex, string str, int count)
        {
            if (str.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (char* po=str)
                    this.Insert(startIndex,po, count);
            }
        }

        /// <summary>
        /// Inserta el contenido de la cadena en la ubicacin especificada.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="strIndex">Index en la cadena donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Insert(int startIndex, string str, int strIndex, int count)
        {
            if (strIndex < 0 || str.Length < strIndex + count)
                throw new ArgumentOutOfRangeException("strIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=str)
                    this.Insert(startIndex,po + strIndex, count);
            }
        }


        /// <summary>
        /// Remplaza todos los elementos de la coleccin desde la ubicacin especificada.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        public void ReplaceRange(int startIndex, string r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex);
            else
                fixed(char*pr=r)
                    ReplaceRange(startIndex,pr,lr);
        }

        /// <summary>
        /// Reemplaza los elementos del rango especificado.
        /// </summary>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Index donde comienza el reemplazo.</param>
        /// <param name="count">Cantidad de elementos a reemplazar.</param>
        public void ReplaceRange(int startIndex, int count, string r)
        {
            int lr=r.Length;
            if (lr==0)
                RemoveRange(startIndex,count);
            else
                fixed(char*pr=r)
                    ReplaceRange(startIndex,count,pr,lr);
        }


        /// <summary>
        /// Setea el contenido de la cadena en la ubicacin especificada.
        /// </summary>
        /// <param name="str">Cadena a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        public void Set(int startIndex, string str)
        {
            int count=str.Length;
            if (count !=0)
            {
                fixed (char* po=str)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido de la cadena en la ubicacin especificada.
        /// </summary>
        /// <param name="str">Cadena a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar desde el inicio de la cadena.</param>
        public void Set(int startIndex, string str, int count)
        {
            if (str.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);

            if (count !=0)
            {
                fixed (char* po=str)
                    this.Set(startIndex, po, count);
            }
        }

        /// <summary>
        /// Setea el contenido de la cadena en la ubicacin especificada.
        /// </summary>
        /// <param name="str">Cadena a setear.</param>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="strIndex">Index en la cadena donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void Set(int startIndex, string str, int strIndex, int count)
        {
            if (strIndex < 0 || str.Length < strIndex + count)
                throw new ArgumentOutOfRangeException("strIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=str)
                    this.Set(startIndex, po + strIndex, count);
            }
        }


        #endregion

        #region Metodos de Bsqueda y reemplazo Texts.

        /// <summary>
        /// Efecta una comparacin con la cadena especificada. 
        /// </summary>
        /// <param name="str2">Cadena a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de str2 son iguales a los la coleccin (comparando desde el inicio) o str2 tiene largo 0.</returns>
        public bool Equals(string str2)
        {
            int count=str2.Length;
            if (count<2)
            {
                if (count==0 || (this._size!=0 && str2[0]==this._items[0]))
                    return true;
                return false;
            }
            if (count>this._size)
                return false;
            fixed(char*p=this._items, p2=str2)
                return Comp.Equal(p,p2,count);
        }


        /// <summary>
        /// Efecta una comparacin con la cadena especificada. 
        /// </summary>
        /// <param name="str2">Cadena a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <returns>Solo retorna true si todos los elementos de str2 son iguales a los la coleccin (comparando desde startIndex) o str2 tiene largo 0.</returns>
        public bool Equals(string str2, int startIndex)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            int count=str2.Length;
            if (count<2)
            {
                if (count==0 || (this._size>startIndex && str2[0]==this._items[startIndex]))
                    return true;
                return false;
            }
            if (startIndex+count>this._size)
                return false;
            fixed(char*p=this._items, p2=str2)
                return Comp.Equal(p+startIndex,p2,count);
        }

        
        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los de la cadena de comparacin.
        /// </summary>
        /// <param name="str2">Cadena de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los de la cadena de comparacin.</returns>
        public bool EndsWith(string str2)
        {

            int count=str2.Length;
            if (count<2)
            {
                if (count==0 || (this._size!=0 && str2[0]==this._items[this._size-1]))
                    return true;
                return false;
            }
            if (count>this._size)
                return false;
            fixed(char*p=this._items, p2=str2)
                return Comp.Equal(p+(this._size-count),p2,count);
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los de la cadena de comparacin.
        /// </summary>
        /// <param name="str2">Cadena de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los de la cadena de comparacin.</returns>
        public bool StartsWith(string str2)
        {
            return Equals(str2);
        }


        /// <summary>
        /// Retorna true si contiene la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(string str, bool last)
        {
            int lf;
            if (this._size==0 || this._size < (lf=str.Length))
                return false;
            fixed (char* p=this._items, pf=str)
            {
                if (!last)
                    return Comp.IndexOf(p,this._size,pf,lf,false)!=-1;
                return Comp.LastIndexOf(p,this._size,pf,lf,false)!=-1;
            }
        }

       
        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(string str)
        {
            int lf;
            if (this._size !=0 && (lf=str.Length)<=this._size)
            {
                fixed (char* p=this._items, pf=str)
                    return Comp.Count(p,this._size,pf,lf,false);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(string str, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=str.Length))
                return 0;
            fixed (char* p=this._items, pf=str)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de a buscar en la coleccin.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(string str, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return 0;
            fixed (char* p=this._items, pf=str)
                return Comp.Count(p + startIndex, count,pf,lf,false);
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(string str)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=str.Length))
            {
                fixed (char* p=this._items, pf=str)
                    return Comp.IndexOf(p, this._size,pf,lf,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(string str, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=str)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(string str, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            int lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=str)
                ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(string str)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=str.Length))
            {
                fixed (char* p=this._items, pf=str)
                    return Comp.LastIndexOf(p, this._size, pf, lf, false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(string str, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=str)
                ret=Comp.LastIndexOf(p + startIndex, count,pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(string str, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=str)
                ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }


        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(string str)
        {
            int lf;
            if (this._size==0 || this._size<(lf=str.Length))
                return 0;

            fixed (char* p=this._items, pf=str)
            {
                return Comp.Remove(false, p, p, p, this._size, out this._size, pf , lf, false , false);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(string str, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=str.Length))
                return 0;
            fixed (char* p=this._items, pf=str)
            {
                char* po=p + startIndex;
                int ret=Comp.Remove(false, po, po, po, count, out count,pf ,lf, false,false);
                if (ret!=0)
                    this._size -=ret*lf;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(string str, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return 0;
            fixed (char* p=this._items, pf=str)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=Comp.Remove(false, po,po, po, count, out lFinal, pf, lf, false,false);
                if (ret!=0)
                {

                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret*lf;
                }
                return ret;
            }
        }


        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(string str)
        {
            int lf;
            if (this._size==0 || this._size<(lf=str.Length))
                return -1;

            fixed (char* p=this._items, pf=str)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(string str, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            fixed (char* p=this._items, pf=str)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(string str, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            fixed (char* p=this._items, pf=str)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }


        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(string str)
        {
            int lf;
            if (this._size==0 || this._size<(lf=str.Length))
                return -1;

            fixed (char* p=this._items, pf=str)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(string str, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            fixed (char* p=this._items, pf=str)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(string str, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            fixed (char* p=this._items, pf=str)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }

        }


        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(string f, string r)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f);
            int lf=f.Length;
            if (lf==0 || this._size<lf)
                return 0;
            fixed(char*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(string f, string r, int startIndex)
        {
            return ReplaceAll(f,r,startIndex,this._size-startIndex);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(string f, string r, int startIndex, int count)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f,startIndex,count);
            int lf=f.Length;
            if (lf==0 || count<lf)
                return 0;
            fixed(char*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr,startIndex,count);
        }


        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(string f, string r)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.IndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(string f, string r, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.IndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(string f, string r, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.IndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(string f, string r)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.LastIndexOf(p, this._size, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(string f, string r, int startIndex)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count,  pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(string f, string r, int startIndex, int count)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=Comp.LastIndexOf(p + startIndex, count, pf, lf, false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        #endregion

        #region AppendLine e InsertLine Strings


        /// <summary>
        /// Inserta el contenido de la cadena al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        public void AppendLine(string str)
        {
            int count=str==null?0:str.Length;
            if (count !=0)
            {
                fixed (char* po=str)
                    this.AppendLine(po, count);
            }
            else
                this.AppendLine();
        }

        /// <summary>
        /// Inserta el contenido de la cadena al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="count">Cantidad de elementos de la cadena a copiar.</param>
        public void AppendLine(string str, int count)
        {
            if (str.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=str)
                    this.AppendLine(po, count);
            }
            else
                this.AppendLine();
        }

        /// <summary>
        /// Inserta el contenido de la cadena al final de la coleccin y agregando salto de lnea final.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="strIndex">Index en la cadena donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        public void AppendLine(string str, int strIndex, int count)
        {
            if (strIndex < 0 || str.Length < strIndex + count)
                throw new ArgumentOutOfRangeException("strIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=str)
                    this.AppendLine(po + strIndex, count);
            }
            else
                this.AppendLine();
        }

        /// <summary>
        /// Inserta el contenido de la cadena en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, string str)
        {
            int count=str==null?0:str.Length;
            if (count !=0)
            {
                fixed (char* po=str)
                    this.InsertLine(startIndex,po, count);
            }
            else
                this.InsertLine(startIndex);
        }

        /// <summary>
        /// Inserta el contenido de la cadena en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="count">Cantidad de elementos de la cadena a copiar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, string str, int count)
        {
            if (str.Length < count)
                throw new ArgumentOutOfRangeException("count", count, Errors.SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=str)
                    this.InsertLine(startIndex,po, count);
            }
            else
                this.InsertLine(startIndex);
        }

        /// <summary>
        /// Inserta el contenido de la cadena en la ubicacin especificada y agregando salto de lnea final.
        /// </summary>
        /// <param name="str">Cadena a insertar.</param>
        /// <param name="strIndex">Index en la cadena donde comenzar la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="startIndex">Index en la coleccin donde se insertar la lnea.</param>
        public void InsertLine(int startIndex, string str, int strIndex, int count)
        {
            if (strIndex < 0 || str.Length < strIndex + count)
                throw new ArgumentOutOfRangeException("strIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count !=0)
            {
                fixed (char* po=str)
                    this.InsertLine(startIndex,po + strIndex, count);
            }
            else
                this.InsertLine(startIndex);
        }

        #endregion

        #region Funciones NC Strings
        
        /// <summary>
        /// Retorna true si contiene la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="last">Busca hacia atras.</param>
        /// <returns>True si existe.</returns>
        public bool Contains(string str, Ordinal comp, bool last)
        {
            int lf;
            if (this._size==0 || this._size < (lf=str.Length))
                return false;
            fixed (char* p=this._items, pf=str)
                return TxtComp.Exist(p,this._size,pf,lf,false,comp,last);
        }


        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(string str, Ordinal comp)
        {
            int lf;
            if (this._size !=0 && (lf=str.Length)<=this._size)
            {
                fixed (char* p=this._items, pf=str)
                    return TxtComp.Count(p,this._size,pf,lf,false,comp);
            }
            return 0;
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(string str, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=str.Length))
                return 0;
            fixed (char* p=this._items, pf=str)
                return TxtComp.Count(p + startIndex, count,pf,lf,false,comp);
        }

        /// <summary>
        /// Cuenta las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de a buscar en la coleccin.</param>
        /// <returns>Cantidad de cadenas encontradas.</returns>
        public int CountOf(string str, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return 0;
            fixed (char* p=this._items, pf=str)
                return TxtComp.Count(p + startIndex, count,pf,lf,false,comp);
        }

        /// <summary>
        /// Efecta una comparacin con la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str2">Cadena a comparar.</param>
        /// <returns>Solo retorna true si todos los elementos de str2 son iguales a los la coleccin (comparando desde el inicio) o str2 tiene largo 0.</returns>
        public bool Equals(string str2, Ordinal comp)
        {
            int count=str2.Length;
            if (count<1)
                return count==0;
            if (count>this._size)
                return false;
            fixed(char*p=this._items, p2=str2)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p,p2,false,count,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p,p2,false,count,TableCase.Invariant);
                    default:
                        return Comp.Equal(p,p2,count);
                }
            }
        }


        /// <summary>
        /// Efecta una comparacin con la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str2">Cadena a comparar.</param>
        /// <param name="startIndex">Inicio de la comparacin en la coleccin.</param>
        /// <returns>Solo retorna true si todos los elementos de str2 son iguales a los la coleccin (comparando desde startIndex) o str2 tiene largo 0.</returns>
        public bool Equals(string str2, int startIndex, Ordinal comp)
        {
            if (startIndex<0)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE);
            int count=str2.Length;
            if (count<1)
                return count==0;
            if (startIndex+count>this._size)
                return false;
            fixed(char*p=this._items, p2=str2)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p+startIndex,p2,false,count,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p+startIndex,p2,false,count,TableCase.Invariant);
                    default:
                        return Comp.Equal(p+startIndex,p2,count);
                }
            }
        }


        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int IndexOf(string str, Ordinal comp)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=str.Length))
            {
                fixed (char* p=this._items, pf=str)
                    return TxtComp.IndexOf(p, this._size,pf,lf,false,comp,false);
            }
            return -1;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(string str, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=str)
                ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int IndexOf(string str, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            int lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=str)
                ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin o -1.</returns>
        public int LastIndexOf(string str, Ordinal comp)
        {
            int lf;
            if (this._size !=0 && this._size>=(lf=str.Length))
            {
                fixed (char* p=this._items, pf=str)
                    return TxtComp.IndexOf(p, this._size, pf, lf, false,comp,true);
            }
            return -1;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(string str, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=str)
                ret=TxtComp.IndexOf(p + startIndex, count,pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }

        /// <summary>
        /// Busca la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta o -1.</returns>
        public int LastIndexOf(string str, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            int ret;
            fixed (char* p=this._items, pf=str)
                ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,true);
            if (ret==-1)
                return -1;
            return ret + startIndex;
        }



        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveLastOf(string str, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=str.Length))
                return -1;

            fixed (char* p=this._items, pf=str)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(string str, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            fixed (char* p=this._items, pf=str)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveLastOf(string str, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            fixed (char* p=this._items, pf=str)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }


        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <returns>Ubicacin del elemento eliminado o -1.</returns>
        public int RemoveOf(string str, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=str.Length))
                return -1;

            fixed (char* p=this._items, pf=str)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveRange(ret, lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(string str, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            fixed (char* p=this._items, pf=str)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }
        }

        /// <summary>
        /// Elimina la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena de elementos a buscar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta del elemento eliminado o -1.</returns>
        public int RemoveOf(string str, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return -1;
            fixed (char* p=this._items, pf=str)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.RemoveRange(ret +=startIndex , lf);
                return ret;
            }

        }


        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(string f, string r, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(string f, string r, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,true);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la ltima aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceLastOf(string f, string r, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,true);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Ubicacin de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(string f, string r, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=f.Length))
                return -1;

            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p, this._size, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.ReplaceRange(ret,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(string f, string r, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex,lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count,  pf, lf, false,comp,false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }

        /// <summary>
        /// Reemplaza la primer aparicin de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de elementos a buscar.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Ubicacin absoluta de la cadena reemplazada o -1.</returns>
        public int ReplaceOf(string f, string r, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=f.Length))
                return -1;
            fixed (char* p=this._items, pf=f)
            {
                int ret=TxtComp.IndexOf(p + startIndex, count, pf, lf, false,comp,false);
                if (ret !=-1)
                    this.ReplaceRange(ret+=startIndex,lf,r);
                return ret;
            }
        }


        /// <summary>
        /// Retorna true si los ltimos elementos de la coleccin coinciden con los de la cadena de comparacin.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str2">Cadena de comparacin.</param>
        /// <returns>True si los ltimos elementos de la coleccin coinciden con los de la cadena de comparacin.</returns>
        public bool EndsWith(string str2, Ordinal comp)
        {

            int count=str2.Length;
            if (count<1)
                return count==0;
            if (count>this._size)
                return false;
            fixed(char*p=this._items, p2=str2)
            {
                switch(comp)
                {
                    case Ordinal.Current:
                        return TxtComp.EqualNC(p+(this._size-count),p2,false,count,Texts.TableCurrent);
                    case Ordinal.Invariant:
                        return TxtComp.EqualNC(p+(this._size-count),p2,false,count,TableCase.Invariant);
                    default:
                        return Comp.Equal(p+(this._size-count),p2,count);
                }
            }
        }

        /// <summary>
        /// Retorna true si los primeros elementos de la coleccin coinciden con los de la cadena de comparacin.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str2">Cadena de comparacin.</param>
        /// <returns>True si los primeros elementos de la coleccin coinciden con los de la cadena de comparacin.</returns>
        public bool StartsWith(string str2, Ordinal comp)
        {
            return Equals(str2,comp);
        }


        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena a eliminar.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(string str, Ordinal comp)
        {
            int lf;
            if (this._size==0 || this._size<(lf=str.Length))
                return 0;

            fixed (char* p=this._items, pf=str)
            {
                return TxtComp.Remove(false, p, p, this._size, out this._size, pf , lf, false , comp);
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(string str, int startIndex, Ordinal comp)
        {
            if (startIndex > this._size || startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int count=this._size-startIndex, lf;
            if (count==0 || count<(lf=str.Length))
                return 0;
            fixed (char* p=this._items, pf=str)
            {
                char* po=p + startIndex;
                int ret=TxtComp.Remove(false, po, po, count, out count,pf ,lf, false,comp);
                if (ret!=0)
                    this._size -=ret*lf;
                return ret;
            }
        }

        /// <summary>
        /// Elimina todas las apariciones de la cadena especificada.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="str">Cadena a eliminar.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de eliminaciones o 0.</returns>
        public int RemoveAll(string str, int startIndex, int count, Ordinal comp)
        {
            if (startIndex + count > this._size || startIndex < 0 || count < 0)
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            int lf;
            if (count==0 || count<(lf=str.Length))
                return 0;
            fixed (char* p=this._items, pf=str)
            {
                int lFinal;
                char* po=p + startIndex;
                int ret=TxtComp.Remove(false, po, po, count, out lFinal, pf, lf, false,comp);
                if (ret!=0)
                {

                    General.Copy(po+count,po+lFinal,this._size-(startIndex+count));
                    this._size -=ret*lf;
                }
                return ret;
            }
        }


        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(string f, string r, Ordinal comp)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f,comp);
            int lf=f.Length;
            if (lf==0 || this._size<lf)
                return 0;
            fixed(char*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr,comp);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(string f, string r, int startIndex, Ordinal comp)
        {
            return ReplaceAll(f,r,startIndex,this._size-startIndex,comp);
        }

        /// <summary>
        /// Reemplaza cadenas.
        /// </summary>
        /// <param name="comp">Modo de comparacin.</param>
        /// <param name="f">Cadena de bsqueda.</param>
        /// <param name="r">Cadena de reemplazo.</param>
        /// <param name="startIndex">Inicio de la bsqueda.</param>
        /// <param name="count">Cantidad de elementos a buscar en la coleccin.</param>
        /// <returns>Cantidad de reemplazos o 0.</returns>
        public int ReplaceAll(string f, string r, int startIndex, int count, Ordinal comp)
        {
            int lr=r.Length;
            if (lr==0)
                return RemoveAll(f,startIndex,count,comp);
            int lf=f.Length;
            if (lf==0 || count<lf)
                return 0;
            fixed(char*pf=f,pr=r)
                return ReplaceAll(pf,lf,pr,lr,startIndex,count,comp);
        }


        #endregion

        #region Mtodos estilo StringBuilder

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(bool obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(byte obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(sbyte obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(short obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(ushort obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(int obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(uint obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(long obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(ulong obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(float obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(double obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(decimal obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(DateTime obj)
        {
            this.Append(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena al final de la coleccin.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Append(object obj)
        {
            if (obj!=null)
                this.Append(obj.ToString());
        }


        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, bool obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, byte obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, sbyte obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, short obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, ushort obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, int obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, uint obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, long obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, ulong obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, float obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, double obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, decimal obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, DateTime obj)
        {
            this.Insert(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta el contenido de la representacin de cadena en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void Insert(int startIndex, object obj)
        {
            if (obj!=null)
                this.Insert(startIndex, obj.ToString());
        }


        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(bool obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(byte obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(sbyte obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(short obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(ushort obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(int obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(uint obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(long obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(ulong obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(float obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(double obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(decimal obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(DateTime obj)
        {
            this.AppendLine(obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena al final de la coleccin.
        /// </summary>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void AppendLine(object obj)
        {
            if (obj!=null)
                this.AppendLine(obj.ToString());
        }


        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, bool obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, byte obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, sbyte obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, short obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, ushort obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, int obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, uint obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, long obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, ulong obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, float obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, double obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, decimal obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// El formato a usar se basa en la cultura invariable.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, DateTime obj)
        {
            this.InsertLine(startIndex, obj.ToString(Texts.CulInfoInvariant));
        }

        /// <summary>
        /// Inserta una lnea con el contenido de la representacin de cadena en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">Index en la coleccin donde comenzar la copia.</param>
        /// <param name="obj">Objeto cuya representacin de cadena se va a insertar.</param>
        public void InsertLine(int startIndex, object obj)
        {
            if (obj!=null)
                this.InsertLine(startIndex, obj.ToString());
        }

        /// <summary>
        /// Inserta la cadena con formato al final de la coleccin.
        /// </summary>
        /// <param name="format">Cadena de formato.</param>
        /// <param name="args">Argumentos para la cadena de formato.</param>
        public void AppendFormat(string format, params object[] args)
        {
            AppendFormat(null,format,args);
        }

        /// <summary>
        /// Inserta la cadena con formato al final de la coleccin.
        /// </summary>
        /// <param name="provider">Proveedor de formato.</param>
        /// <param name="format">Cadena de formato.</param>
        /// <param name="args">Argumentos para la cadena de formato.</param>
        public void AppendFormat(IFormatProvider provider, string format, params object[] args)
        {
            int l = format == null ? 0 : format.Length;
            if (l == 0)
                return;
            int lobj = args == null ? 0 : args.Length;
            this.EnsureAppend(l);

            char* pf;
            Mem.Heap heap;
            if (&l>General.MinimumStack && l < General.MaxStackChars)
            {
                char* ptmp = stackalloc char[l];
                pf = ptmp;
                heap = null;
            }
            else
            {
                heap = new Mem.Heap(l<<1,false);
                pf = (char*)heap.Pointer;
            }
            fixed (char* ptmp = format)
                General.Copy(ptmp, pf, l);
                
            int num1 = 0;
            char ch1;
            ICustomFormatter formatter1 = null;
            if (provider != null)
            {
                formatter1 = (ICustomFormatter)provider.GetFormat(typeof(ICustomFormatter));
            }
            while (true)
            {
                int num3 = num1;
                int num4 = num1;
                while (num1 < l)
                {
                    ch1 = pf[num1];
                    num1++;
                    if (ch1 == '}')
                    {
                        if ((num1 < l) && (pf[num1] == '}'))
                        {
                            num1++;
                        }
                        else
                        {
                            if (heap != null)
                                heap.Dispose();
                            throw new FormatException(Errors.COL_FMT_INV);
                        }
                    }
                    if (ch1 == '{')
                    {
                        if ((num1 < l) && (pf[num1] == '{'))
                        {
                            num1++;
                        }
                        else
                        {
                            num1--;
                            break;
                        }
                    }
                    pf[num4++] = ch1;
                }
                if (num4 > num3)
                {
                    this.Append(pf+num3, num4 - num3);
                }
                if (num1 == l)
                {
                    if (heap != null)
                        heap.Dispose();
                    return;
                }
                num1++;
                if (((num1 == l) || ((ch1 = pf[num1]) < '0')) || (ch1 > '9'))
                {
                    if (heap != null)
                        heap.Dispose();
                    throw new FormatException(Errors.COL_FMT_INV);
                }
                int num5 = 0;
                do
                {
                    num5 = ((num5 * 10) + ch1) - 0x30;
                    num1++;
                    if (num1 == l)
                    {
                        if (heap != null)
                            heap.Dispose();
                        throw new FormatException(Errors.COL_FMT_INV);
                    }
                    ch1 = pf[num1];
                }
                while (((ch1 >= '0') && (ch1 <= '9')) && (num5 < 0xf4240));
                if (num5 >= lobj)
                {
                    if (heap != null)
                        heap.Dispose();
                    throw new FormatException(Errors.COL_FMT_RANGE);
                }
                while ((num1 < l) && ((ch1 = pf[num1]) == ' '))
                {
                    num1++;
                }
                bool flag1 = false;
                int num6 = 0;
                if (ch1 == ',')
                {
                    num1++;
                    while ((num1 < l) && (pf[num1] == ' '))
                    {
                        num1++;
                    }
                    if (num1 == l)
                    {
                        if (heap != null)
                            heap.Dispose();
                        throw new FormatException(Errors.COL_FMT_INV);
                    }
                    ch1 = pf[num1];
                    if (ch1 == '-')
                    {
                        flag1 = true;
                        num1++;
                        if (num1 == l)
                        {
                            if (heap != null)
                                heap.Dispose();
                            throw new FormatException(Errors.COL_FMT_INV);
                        }
                        ch1 = pf[num1];
                    }
                    if ((ch1 < '0') || (ch1 > '9'))
                    {
                        if (heap != null)
                            heap.Dispose();
                        throw new FormatException(Errors.COL_FMT_INV);
                    }
                    do
                    {
                        num6 = ((num6 * 10) + ch1) - 0x30;
                        num1++;
                        if (num1 == l)
                        {
                            if (heap != null)
                                heap.Dispose();
                            throw new FormatException(Errors.COL_FMT_INV);
                        }
                        ch1 = pf[num1];
                    }
                    while (((ch1 >= '0') && (ch1 <= '9')) && (num6 < 0xf4240));
                }
                while ((num1 < l) && ((ch1 = pf[num1]) == ' '))
                {
                    num1++;
                }
                object obj1 = args[num5];
                string text1 = null;
                if (ch1 == ':')
                {
                    num1++;
                    num3 = num1;
                    num4 = num1;
                    while (true)
                    {
                        if (num1 == l)
                        {
                            if (heap != null)
                                heap.Dispose();
                            throw new FormatException(Errors.COL_FMT_INV);
                        }
                        ch1 = pf[num1];
                        num1++;
                        if (ch1 == '{')
                        {
                            if ((num1 < l) && (pf[num1] == '{'))
                            {
                                num1++;
                            }
                            else
                            {
                                if (heap != null)
                                    heap.Dispose();
                                throw new FormatException(Errors.COL_FMT_INV);
                            }
                        }
                        else if (ch1 == '}')
                        {
                            if ((num1 < l) && (pf[num1] == '}'))
                            {
                                num1++;
                            }
                            else
                            {
                                num1--;
                                break;
                            }
                        }
                        pf[num4++] = ch1;
                    }
                    if (num4 > num3)
                    {
                        text1 = Mutables.Create(pf+num3, num4 - num3);
                    }
                }
                if (ch1 != '}')
                {
                    if (heap != null)
                        heap.Dispose();
                    throw new FormatException(Errors.COL_FMT_INV);
                }
                num1++;
                string text2 = null;
                if (formatter1 != null)
                {
                    text2 = formatter1.Format(text1, obj1, provider);
                }
                if (text2 == null)
                {
                    if (obj1 is IFormattable)
                    {
                        text2 = ((IFormattable)obj1).ToString(text1, provider);
                    }
                    else if (obj1 != null)
                    {
                        text2 = obj1.ToString();
                    }
                }
                if (text2 == null)
                {
                    text2 = Others.Empty;
                }
                int num7 = num6 - text2.Length;
                if (!flag1 && (num7 > 0))
                {
                    this.Append(' ', num7);
                }
                this.Append(text2);
                if (flag1 && (num7 > 0))
                {
                    this.Append(' ', num7);
                }
            }

        }

 


        #endregion
      
        #region Funciones con Streams codificados Chars

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream codificado.
        /// Los elementos son insertados al final de la coleccin.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="maxCount">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Si el stream est en posicin cero intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int Append(System.IO.Stream stream, int maxCount, bool close, CodePage codePage, bool detectSign)
        {
            if (detectSign)
                codePage=new CodePage(stream,false,codePage);
            codePage.CalcStreamRead(stream,ref maxCount);
            if (maxCount!=0)
            {
                this.EnsureAppend(maxCount);
                fixed(char*p=this._items)
                    maxCount=codePage.StreamRead(stream,close,p+this._size,maxCount,null,0);
                this._size+=maxCount;
            }
            else if (close)
                stream.Close();
            return maxCount;            
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream codificado.
        /// Los elementos son insertados en la ubicacin especificada.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="maxCount">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Si el stream est en posicin cero intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int Insert(int startIndex, System.IO.Stream stream, int maxCount, bool close, CodePage codePage, bool detectSign)
        {
            if (startIndex==this._size)
                return Append(stream,maxCount,close,codePage,detectSign);
            if (startIndex<0 || startIndex>this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);

            if (detectSign)
                codePage=new CodePage(stream,false,codePage);
            codePage.CalcStreamRead(stream,ref maxCount);

            if (maxCount!=0)
            {
                this.Reserve(startIndex,maxCount);
                int ret;
                fixed(char*p=this._items)
                    ret=codePage.StreamRead(stream,close,p+startIndex,maxCount,null,0);
                if (ret!=maxCount)
                {
                    this.RemoveRange(startIndex+ret,maxCount-ret);
                    return ret;
                }
            }
            else if (close)
                stream.Close();
            return maxCount;
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream codificado.
        /// Los elementos son insertados en la ubicacin especificada eliminando todos los elementos
        /// existentes a partir de esta ubicacin.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="maxLR">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Si el stream est en posicin cero intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int ReplaceRange( int startIndex, System.IO.Stream stream, int maxLR, bool close, CodePage codePage, bool detectSign)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
            return this.Append(stream,maxLR,close,codePage,detectSign);
        }

        /// <summary>
        /// Copia elementos desde la posicin actual de un Stream codificado.
        /// Los elementos copiados reemplazarn al rea comprendida entre startIndex y count.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="count">Area a reemplazar con el contenido del Stream.</param>
        /// <param name="maxLR">
        /// Cantidad mxima de elementos a copiar desde el Stream.
        /// Si es -1, se copiarn todos los elementos disponibles.
        /// </param>
        /// <param name="close">
        /// Cierra el Stream al finalizar.
        /// </param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Si el stream est en posicin cero intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el Stream.
        /// </returns>
        public int ReplaceRange( int startIndex, int count, System.IO.Stream stream, int maxLR, bool close, CodePage codePage, bool detectSign)
        {
            if (count<1)
            {
                if (count==0)
                    return Insert(startIndex,stream,maxLR,close,codePage,detectSign);
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            if (startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE);

            int l=startIndex+count;
            if (l>=this._size)
            {
                if (l==this._size)
                    return ReplaceRange(startIndex,stream,maxLR,close,codePage,detectSign);
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.SUPERIOR_ARRAY);
            }

            if (detectSign)
                codePage=new CodePage(stream,false,codePage);
            codePage.CalcStreamRead(stream,ref maxLR);
            
            if (maxLR<=count)
            {
                if (maxLR!=0)
                {
                    fixed(char*p=this._items)
                        maxLR=codePage.StreamRead(stream,close,p+startIndex,maxLR,null,0);
                }
                else if (close)
                    stream.Close();
                if (maxLR<count)
                    this.RemoveRange(startIndex+maxLR,count-maxLR);
                return maxLR;
            }
            this.Reserve(startIndex+count,maxLR-count);
            fixed(char*p=this._items)
                l=codePage.StreamRead(stream,close,p+startIndex,maxLR,null,0);
            if (l!=maxLR)
                this.RemoveRange(startIndex+l,maxLR-l);
            return l;
        }

 
        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de un Stream codificado.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="codePage">
        /// Cdigo de pgina para la salida.
        /// </param>
        /// <param name="close">Cierra el Stream luego de la escritura.</param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Si true y el stream se encuentra en posicin cero, la cantidad de bytes
        /// a escribr es mayor a cero y el cdigo de pgina lo requiere, firmar el stream.
        /// </param>
        /// <returns>Bytes escritos.</returns>
        public int StreamWrite(System.IO.Stream stream,bool close, CodePage codePage, bool noBestFitChar, bool signCP)
        {
            int count=this._size;
            if (count!=0)
            {
                fixed(char*p=this._items)
                    count=codePage.StreamWrite(stream,signCP,false,close,p,count,noBestFitChar,null,null,null,0);
            }
            else if (close)
                stream.Close();
            return count;
        }

        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de un Stream codificado.
        /// </summary>
        /// <param name="stream">
        /// Stream.
        /// </param>
        /// <param name="startIndex">Indice de comienzo de la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="close">Cierra el Stream luego de la escritura.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la salida.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="signCP">
        /// Si true y el stream se encuentra en posicin cero, la cantidad de bytes
        /// a escribr es mayor a cero y el cdigo de pgina lo requiere, firmar el stream.
        /// </param>
        /// <returns>Bytes escritos.</returns>
        public int StreamWrite(System.IO.Stream stream, int startIndex, int count,bool close, CodePage codePage, bool noBestFitChar, bool signCP)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
                fixed(char*p=this._items)
                    count=codePage.StreamWrite(stream,signCP,false,close,p+startIndex,count,noBestFitChar,null,null,null,0);
            if (close)
                stream.Close();
            return count;
        }

        #endregion

        #region Funciones con Arrays de Bytes codificados Chars


        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos son insertados al final de la coleccin.
        /// </summary>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Append(byte[] bytes, CodePage codePage, bool detectSign)
        {
            int bytesCount=bytes.Length;
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return Append(p,bytesCount,codePage,detectSign);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos son insertados en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Insert(int startIndex, byte[] bytes, CodePage codePage, bool detectSign)
        {
            int bytesCount=bytes.Length;
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return Insert(startIndex,p,bytesCount,codePage,detectSign);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos son insertados en la ubicacin especificada eliminando todos los elementos
        /// existentes a partir de esta ubicacin.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange( int startIndex, byte[] bytes, CodePage codePage,bool detectSign)
        {
            int bytesCount=bytes.Length;
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return ReplaceRange(startIndex,p,bytesCount,codePage,detectSign);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos copiados reemplazarn al rea comprendida entre startIndex y count.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="count">Area a reemplazar con el contenido del Array.</param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange( int startIndex, int count, byte[] bytes, CodePage codePage, bool detectSign)
        {
            int bytesCount=bytes.Length;
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return ReplaceRange(startIndex,count,p,bytesCount,codePage,detectSign);
            return 0;
        }



        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos son insertados al final de la coleccin.
        /// </summary>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesIndex">Index en el array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Append(byte[] bytes, int bytesIndex, int bytesCount, CodePage codePage, bool detectSign)
        {
            if (bytesIndex<0 || bytesCount<0 || bytesIndex+bytesCount>bytes.Length)
                throw new ArgumentOutOfRangeException("bytesIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return Append(p+bytesIndex,bytesCount,codePage,detectSign);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos son insertados en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesIndex">Index en el array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Insert(int startIndex, byte[] bytes, int bytesIndex, int bytesCount, CodePage codePage, bool detectSign)
        {
            if (bytesIndex<0 || bytesCount<0 || bytesIndex+bytesCount>bytes.Length)
                throw new ArgumentOutOfRangeException("bytesIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return Insert(startIndex,p+bytesIndex,bytesCount,codePage,detectSign);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos son insertados en la ubicacin especificada eliminando todos los elementos
        /// existentes a partir de esta ubicacin.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesIndex">Index en el array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange( int startIndex, byte[] bytes, int bytesIndex, int bytesCount, CodePage codePage, bool detectSign)
        {
            if (bytesIndex<0 || bytesCount<0 || bytesIndex+bytesCount>bytes.Length)
                throw new ArgumentOutOfRangeException("bytesIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return ReplaceRange(startIndex,p+bytesIndex,bytesCount,codePage,detectSign);
            return 0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos copiados reemplazarn al rea comprendida entre startIndex y count.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="count">Area a reemplazar con el contenido del Array.</param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesIndex">Index en el array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange( int startIndex, int count, byte[] bytes, int bytesIndex, int bytesCount, CodePage codePage, bool detectSign)
        {
            if (bytesIndex<0 || bytesCount<0 || bytesIndex+bytesCount>bytes.Length)
                throw new ArgumentOutOfRangeException("bytesIndex,bytesCount",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (bytesCount!=0)
                fixed(byte*p=bytes)
                    return ReplaceRange(startIndex,count,p+bytesIndex,bytesCount,codePage,detectSign);
            return 0;
        }

        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de una coleccin de bytes codificando.
        /// </summary>
        /// <param name="codePage">
        /// Cdigo de pgina para la salida.
        /// </param>
        /// <param name="col">Coleccin destino.</param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <returns>Bytes escritos.</returns>
        public int BytesWrite(Bytes col,  CodePage codePage, bool noBestFitChar)
        {
            if (this._size!=0)
                fixed(char*p=this._items)
                    return codePage.BytesWrite(col,p,this._size,noBestFitChar,null,null);
            return 0;
        }

        /// <summary>
        /// Copia el contenido de la coleccin hacia la posicin actual de una coleccin de bytes codificando.
        /// </summary>
        /// <param name="startIndex">Indice de comienzo de la copia.</param>
        /// <param name="count">Cantidad de elementos a copiar.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la salida.
        /// </param>
        /// <param name="noBestFitChar">
        /// Cualquier caracter que no se traduzca directamente a multibyte ser reemplazado por el caracter por defecto sin bscar un caracter alternativo.
        /// Esta opcin no es compatible con estas pginas de cdigos:
        /// 1200 (Unicode LE), 1201 (Unicode BE), 12000 (Unicode 32 LE), 12001 (Unicode 32 BE),
        /// 50220, 50221, 50222, 50225, 50227, 50229, 52936, 54936, 57002 a 57011, 
        /// 65000 (UTF7), 65001 (UTF8) y 42 (Symbol).
        /// Si se usa alguna de estas pginas este parmetro se autocorregir
        /// a false sin generar excepcin.
        /// </param>
        /// <param name="col">Coleccin destino.</param>
        /// <returns>Bytes escritos.</returns>
        public int BytesWrite(Bytes col, int startIndex, int count,  CodePage codePage, bool noBestFitChar)
        {
            if (startIndex<0 || count<0 || startIndex+count>this._size)
                throw new ArgumentOutOfRangeException("startIndex,count",Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            if (count!=0)
                fixed(char*p=this._items)
                    return codePage.BytesWrite(col,p+startIndex,count,noBestFitChar,null,null);
            return 0;
        }

        #endregion

        #region Funciones con Punteros de Bytes codificados Chars

        private static bool getchcount(out int charsCount, ref byte*bytes, ref int bytesCount, ref CodePage codePage, bool detectSign)
        {
            if (detectSign)
            {
                codePage=new CodePage(bytes,bytesCount,codePage,out charsCount);
                if (charsCount!=0)
                {
                    if (charsCount==bytesCount)
                        return false;
                    bytes+=charsCount;
                    bytesCount-=charsCount;
                }
            }
            charsCount=codePage.CalcMaxChars(bytesCount);
            return charsCount!=0;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos son insertados al final de la coleccin.
        /// </summary>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Append(byte*bytes, int bytesCount, CodePage codePage, bool detectSign)
        {
            int charsCount;
            if (!getchcount(out charsCount,ref bytes,ref bytesCount,ref codePage,detectSign))
                return 0;
            this.EnsureAppend(charsCount);
            fixed(char*p=this._items)
                charsCount=codePage.BytesToChars(bytes,bytesCount,p+this._size,charsCount,false);
            this._size+=charsCount;
            return charsCount;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos son insertados en la ubicacin especificada.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int Insert(int startIndex, byte*bytes, int bytesCount, CodePage codePage, bool detectSign)
        {
            if (startIndex==this._size)
                return Append(bytes,bytesCount,codePage, detectSign);
            if (startIndex<0 || startIndex>this._size)
                throw new ArgumentOutOfRangeException("startIndex",startIndex,Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            
            int charsCount;
            if (!getchcount(out charsCount,ref bytes,ref bytesCount,ref codePage,detectSign))
                return 0;
            this.Reserve(startIndex,charsCount);
            fixed(char*p=this._items)
                bytesCount=codePage.BytesToChars(bytes,bytesCount,p+startIndex,charsCount,false);
            if (bytesCount!=charsCount)
                this.RemoveRange(startIndex+bytesCount,charsCount-bytesCount);
            return bytesCount;
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos son insertados en la ubicacin especificada eliminando todos los elementos
        /// existentes a partir de esta ubicacin.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange( int startIndex, byte*bytes, int bytesCount, CodePage codePage,bool detectSign)
        {
            if (startIndex < 0 || startIndex > this._size)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE_OR_SUPERIOR_ARRAY);
            this._size=startIndex;
            return this.Append(bytes,bytesCount,codePage,detectSign);
        }

        /// <summary>
        /// Copia elementos desde un array de bytes codificado.
        /// Los elementos copiados reemplazarn al rea comprendida entre startIndex y count.
        /// </summary>
        /// <param name="startIndex">
        /// Ubicacin en la coleccin donde se insertarn los elementos.
        /// </param>
        /// <param name="count">Area a reemplazar con el contenido del Array.</param>
        /// <param name="bytes">Array de bytes.</param>
        /// <param name="bytesCount">Largo del array de bytes.</param>
        /// <param name="codePage">
        /// Cdigo de pgina para la entrada.
        /// </param>
        /// <param name="detectSign">
        /// Intenta detectar el cdigo de pgina a partir de la firma.
        /// En este caso codePage es un valor por defecto para cuando falla la deteccin.
        /// </param>
        /// <returns>
        /// Cantidad de elementos agregados desde el array de bytes.
        /// </returns>
        public int ReplaceRange( int startIndex, int count, byte*bytes, int bytesCount, CodePage codePage, bool detectSign)
        {
            if (count<1)
            {
                if (count==0)
                    return Insert(startIndex,bytes,bytesCount,codePage,detectSign);
                throw new ArgumentOutOfRangeException("count", count, Errors.NEGATIVE);
            }
            if (startIndex < 0)
                throw new ArgumentOutOfRangeException("startIndex", startIndex, Errors.NEGATIVE);
            
            int l=startIndex+count;
            if (l>=this._size)
            {
                if (l==this._size)
                    return ReplaceRange(startIndex,bytes,bytesCount,codePage,detectSign);
                throw new ArgumentOutOfRangeException("startIndex,count", Errors.SUPERIOR_ARRAY);
            }

            int charsCount;
            if (!getchcount(out charsCount,ref bytes,ref bytesCount,ref codePage,detectSign))
            {
                this.RemoveRange(startIndex,count);
                return 0;
            }
            if (charsCount<=count)
            {
                fixed(char*p=this._items)
                    charsCount=codePage.BytesToChars(bytes,bytesCount,p+startIndex,charsCount,false);
                if (charsCount<count)
                    this.RemoveRange(startIndex+charsCount,count-charsCount);
                return charsCount;
            }
            this.Reserve(startIndex+count,charsCount-count);
            fixed(char*p=this._items)
                l=codePage.BytesToChars(bytes,bytesCount,p+startIndex,charsCount,false);
            if (l!=charsCount)
                this.RemoveRange(startIndex+l,charsCount-l);
            return l;
        }


        #endregion
    
    }
}
